Rem * Filename: dnddoor.bas Version: v5.0a r3.0a for QB64.
Rem * This subprogram contains the main input and communication routines.

Rem * Version v5.0a r3.0a mods include:
Rem    Fixes recursive clock$ function.

Rem **** BEGIN INCLUDE FILE ****

Rem * Filename: dnddoor.inc  Version: v5.0a r2.0a
Rem * Variables here are included common to all subprograms.

' define all default variables as integers
DefInt A-Z

' allocate common variables and arrays at runtime
Rem $DYNAMIC

' declare titlebar text
_Title "DNDDOOR"

'  define structure for containers
Type ContainerType
   Closed As Integer ' closed flag 1=closed, 0=n/a, -1=open
   ContainerName As String * 30 ' container name from treasure.dat file
   Locked As Integer ' locked flag 1=locked, 0=n/a, -1=unlocked
   Keyed As Integer ' container key number 1-32767
   ShortName As String * 30 ' mnemonic of container name from treasure.dat file
   Charges As String * 10 ' treasure charges of container contents
   Inventory As String * 10 ' index to treasure.dat of container contents
   Invisible As String * 10 ' invisible flag -1/0 of container contents
   Unused As String * 150 ' treasure name of container contents (unused)
   Permanent As Integer ' flag for permanent container
End Type

'  action structure
Type ActionType
   Attribute1 As Integer ' room attribute one, 0=lit/-1=unlit
   Attribute2 As Integer ' room attribute two, 0=land/1=air/2=underwater
   EncounterRate As Integer ' rate of action prompts to encounter monster
   Fumble As Integer ' fumble weapons/shield result
   HealthRate As Integer ' rate of action prompts to increase user health
   HitPoints As Integer ' negative/positive hits for vit/fat result
   Inventory As Integer ' weapons=1/shields=2/armor=3/magic items=4 breakage
   Level As Integer ' negative/positive value=lower/highest user level to enter
   MonsterTalk As Integer ' talk to monster number trigger
   MonsterTrigger As Integer ' number of room monster trigger
   Restrictions As Integer ' level direction is restricted to, 12 bit flags
   RustRate As Integer ' rate of action prompts weapons/shields rust
   SpellTrigger As Integer ' number of room spell trigger
   StealRate As Integer ' rate of action prompts monsters steal
   Teleport As Single ' teleport trap room number result
End Type

'  define room structure. this main data file contains room descriptions,
'  directions, one container, action number, object and treasure indexes.
Type RoomType
   Action As Integer ' index to action file
   Direct As String * 48 ' direction link values to next room number
   Flags As String * 40 ' treasure is: hidden=1/trap=2/magically hidden=3
   LongDesc As String * 400 ' five line long description of room
   MonsterClass As Integer ' number of room monster class in monclass.dat file
   Object As String * 40 ' number of object from objects.dat file
   HiddenObj As String * 40 ' object flags, -1=hidden
   ShortDesc As String * 80 ' one line short description of room for brief mode
   Treasure As String * 40 ' number of treasure from treasure.dat file
   TreCharges As String * 40 ' treasure charges
   Container As ContainerType ' single container per room
End Type

'  define user structure. this main data file contains user statistics,
'  values, flags, object and treasure indexes.
Type UserType
   CodeName As String * 30 ' encrypted user codename
   PassWord As String * 20 ' encrypted user password
   ClassName As String * 20 ' encrypted user classname
   ClassType As Integer ' classname number 1-10
   ' user's maximum of 3 containers
   Container1 As ContainerType: Container2 As ContainerType: Container3 As ContainerType
   Inv As String * 40 ' user's inventory index to treasure.dat file
   Charges As String * 40 ' treasure's charges/plus
   Object As String * 40 ' user's object index to objects.dat file
   TrainRoom As String * 40 ' training room array
   Stats As String * 14 ' player statistics str/int/wis/dex/con/pie/cha
   Weapons As String * 8 ' weapons proficiency percent blu/pol/sha/thr
   Room As Single ' room number
   Level As Integer ' player's level
   Experience As Double ' player's experience
   Gold As Double ' collected gold
   NumCalls As Integer ' number of calls during one day
   Fatigue As Integer ' fatigue points during fighting
   FatigueMax As Integer ' maximum possible fatigue points
   Vitality As Integer ' vitality points during fighting
   VitalityMax As Integer ' maximum possible vitality points
   Magic As Integer ' magic points during fighting/using spells
   MagicMax As Integer ' maximum possible magic points
   Poison As Integer ' user is poisoned true/false, vitality decreasing
   Borrow As Double ' gold borrowed from bank
   Invisible As Integer ' user is invisible true/false
   Race As Integer ' player's race 1-8, hum/elf/gno/dwa/hal/hal-elf/hal-orc/ogr
   Proficiency As Integer ' weapon user is proficient in 1-4
   Bank As Double ' amount gold stored in bank
   Sort As Integer ' sort inventory (-1/0/1)
   MonstersKilled As Single ' total of all monsters killed during play
   Brief As Integer ' brief mode true/false
   DateOn As String * 10 ' last date user logged on, form: MM-DD-YYYY
   Beauty As Integer ' lady class beauty points
   Glamour As Integer ' lady class glamour points
   Psionic As Integer ' psionic points during fighting/casting psi spells
   PsionicMax As Integer ' maximum possible psionic points
   Align1 As Integer ' alignment type 1, good/neutral/evil
   Align2 As Integer ' alignment type 2, lawful/neutral/chaotic
   Flags As Integer ' 1=rerolled/2-128=stat wish/256=wish/512=realigned/chars
   MaxCalls As Integer ' maximum calls user is allowed per day
   FromHour As Integer ' time restriction logon from hour,
   FromMin As Integer ' time restriction logon from minute,
   ToHour As Integer ' time restriction logon to hour,
   ToMin As Integer ' time restriction logon to minute.
   Echo As Integer ' echo mode 0=on,-1=off
   LastMessage As Integer ' last message read
   Linefeeds As Integer ' linefeed mode 0=on,-1=off
   Linelength As Integer ' linelength for display
   Pagelength As Integer ' pagelength for display
   Wordwrap As Integer ' word wrap 0=on,-1=off
End Type

'  define spell structure. this data file contains spell statistics,
'  values, and flags. this struture is used by treasure and spells.
Type SpellType
   SpellName As String * 30 ' full name of spell
   Chant As String * 80 ' chant to be used for spell
   Level As Integer ' level of spell/level user must be at least to cast spell
   SpellType As Integer ' spell type 1-28
   Teleport As Single ' spell teleports when cast
   Psionic As Integer ' spell is psionic, true/false
   PsionicMode As Integer ' spell is psionic, 1=attack/2=defense
   ClassType As Integer ' user class allowed to cast spell, flags 1-8
   Desc As String * 80 ' description displayed when spell is cast
   Ingred As String * 10 ' index to treasure.dat of required ingredients
   SpellFlag As Integer ' ingredients type required flag, 1=use/2=read/4=cast
End Type

'  define monster class structure. this data file contains lists of monsters.
Type MonclassType
   Monsters As String * 20 ' indexed list of ten monsters from monster.dat
End Type

'  define object structure. this data file contains object statistics,
'  values, and flags. this struture is used by users and rooms.
Type ObjectType
   ObjectName As String * 30 ' full name of object
   ShortName As String * 30 ' mnemonic of object name
   RoomLink As Single ' link to room number for entering object, or for jail
   Invisible As Integer ' is invisible (-1/0)
   JailTrap As Integer ' jails user if attacked, roomlink is jail room number
   DoorLock As Integer ' locked (0=n/a,1=unlocked/2=locked)
   Destination As Integer ' n/a (used to be room object was created)
   Permanent As Integer ' cannot be picked up. default.
   Hidden As Integer ' hidden (cannot not be seen or searched for)
   Closed As Integer ' closed (-1=closed/0=open)
   Keyed As Integer ' key number 1-32767
   Relocks As Integer ' relocks after entering
   LongDesc As String * 80 ' description displayed when object looked at
   FromHour As Integer ' lights room from hour,
   FromMin As Integer ' lights room from minute,
   ToHour As Integer ' lights room to hour,
   ToMin As Integer ' lights room to minute.
   LightRoom As Integer ' lights room (-1=true/0=false)
   LightTime As Integer ' light time (-1=from-to/0=anytime)
   ShortDesc As String * 80 ' description displayed when object entered
   Trap As Integer ' trapped portal (1=poison/2=teleport/3=hits for fat/vit)
   Teleport As Single ' (room number, trap=2), (hits -fat/hits +vit, trap=3)
End Type

'  define monster structure. this data file contains monster statistics,
'  values, and flags.
Type MonsterType
   MonsterName As String * 30 ' full name of monster
   PluralName As String * 30 ' mnemonic plural of monster name
   Treasure As String * 10 ' five indexes to treasure.dat
   Experience As Double ' experience gained by killing monster
   Gold As Double ' money gained by killing monster
   NumberAppearing As Integer ' maximum monsters encontered at one time
   Level As Integer ' monster level/level of user required to kill monster
   Hits As Integer ' hit points monster has for user to kill
   Poison As Integer ' monster can poison=-1/monster is poisoned>0
   LevelDrain As Integer ' can drain user levels (-1/0)
   Spell As Integer ' can cast spells (equal to spell record number)
   Block As Integer ' can block user from exiting room
   Prevent As Integer ' can prevent user from taking treasure
   Follow As Integer ' can follow user, otherwise contains jail room number
   Magic As Integer ' casts magic spell (equals index to spell record number)
   Jail As Integer ' can jail attacker (-1/0), room number is in follow
   Teleport As Single ' can teleport (equals room number)
   FollowPercent As Integer ' percentage to follow
   BlockPercent As Integer ' percentage to block
   PreventPercent As Integer ' percentage to prevent
   SpellPercent As Integer ' percentage to cast spell
   PoisonPercent As Integer ' percentage to poison
   DrainPercent As Integer ' percentage to drain levels
   Rate As Integer ' encounter rate (action prompts)
   RatePercent As Integer ' percentage encountered at rate
   Permanent As Integer ' monster is permanent to room
   Talk As String * 10 ' five indexes to montalk.dat
   Psionic As Integer ' monster casts psionic spells
   PsionicSpell As Integer ' psionic spell casted
End Type

'  define treasure structure. this data file contains treasure statistics,
'  values, and flags. this struture is used by users and rooms.
Type TreasureType
   TreasureName As String * 30 ' full treasure name
   ShortName As String * 30 ' mnemonic treasure name
   Weight As Integer ' weight in gold of treasure
   Gold As Integer ' value in gold of treasure
   Spell As Integer ' treasure is magical (index of spell record)
   Plus As Integer ' treasure plus (0=normal treasure/greater>0=weapon plus)
   Coin As Integer ' coins (-1/0)
   Keyed As Integer ' key number (1-32767)
   Scroll As Integer ' scroll (-1/0)
   Charges As Integer ' charges for weapon/armor/shield/spell
   Potion As Integer ' potion (-1/0)
   Armor As Integer ' treasure is armor
   Shield As Integer ' treasure is shield
   Weapon As Integer ' treasure is weapon
   Container As Integer ' container (-1/0)
   Type As Integer ' treasure type (-1=shield/0=normal/1=armor/0,<plus>=weapon)
   Locked As Integer ' container (1=locked,0=n/a,-1=unlocked)
   Closed As Integer ' container (1=closed,0=n/a,-1=open)
   Loadable As Integer ' can be loaded with ammunition (-1/0)
   Ammunition As Integer ' is ammunition (-1/0)
   AmmoLoads As Integer ' if ammunition, then index to treasure.dat it loads
   Invisible As Integer ' invisible (-1/0)
   Permanent As Integer ' treasure remains in room (not default)
   Proficiency As Integer ' weapon proficiency related (1-4)
   RingType As Integer ' is a ring (1=antipoison/2=antileveldrain/3=antispell)
   RingSpell As Integer ' ring antispell (-1=generic/greater>0=spell number)
   Edible As Integer ' edible (-1/0)
   Rustable As Integer ' rusts (-1/0)
   RustPercent As Integer ' percentage rusts during room rust rate
   Stealable As Integer ' stealable (-1/0)
   StealPercent As Integer ' percentage monster steals during room steal rate
   Launchable As Integer ' can be launched with launch ammunition (-1/0)
   LaunchLoads As Integer ' is launch ammunition (-1/0)
   LaunchAmmo As Integer ' if launch ammo, then index to treasure.dat it loads
   Movable As Integer ' launchable device can be moved (-1/0)
   Vehicle As Integer ' vehicle (-1/0)
   VehicleHits As Integer ' vehicle hit points for combat
   VehicleType As Integer ' vehicle type (0=land,1=air,2=underwater,3=any)
   LightType As Integer ' is a light (-1/0)
   LightCharges As Integer ' light charges
   FuelType As Integer ' is light fuel (-1/0)
   FuelCharges As Integer ' fuel charges
End Type

'  define monster talk structure. this data file contains lists of strings
'  monsters can respond with when talked to.
Type MonsterTalkType
   TalkMessage As String * 80 ' an 80 character string response
End Type

'  help file record structure. used for help command during play,
'  uses files help.dat, edithelp.dat, dmhelp.dat, and logon.dat.
Type HelpType
   Text As String * 79 ' text to display (updated 11/16/2001).
   CName As String * 15 ' command name of command to display
End Type

'  structure for messages text file. indexed by the table record structure.
Type MessageType
   Message As String * 80 ' length of a message line
End Type

'  table for messages. stores subject, from/to, date entered, private message
'  and if the message is deleted. fields for start and length define the
'  position of the message in the messages text file.
Type TableType
   ClassType As Integer ' class type of the player who entered the message
   Clock As String * 21 ' stores the time the message was entered from date/time
   Date As String * 10 ' system date message was stored
   Flags As Integer ' user record flags for special characters entered message
   From As String * 30 ' field storing user who entered the message
   Killed As Integer ' true if the message is deleted
   Length As Integer ' length of lines in messages text file
   Private As Integer ' true if the message is private
   Received As Integer ' flag indicating message was read
   Reply As Integer ' flag for message reply
   Start As Single ' beginning of the line of the message in the text file
   Subject As String * 20 ' stores the subject of the message
   Thread As Integer ' number of message replied to
   Time As String * 10 ' stores system time message was entered
   Timer As Single ' stores system time in seconds message was entered
   TimesRead As Integer ' stores number of times message was read
   ToUser As String * 30 ' stores name of the user the message is to, or empty for all
End Type

'  define variables, fixed length/variable strings, and
'  dynamic arrays common to all subprograms.
Dim Shared ActionRecord As ActionType ' defines action structure
Dim Shared Action.Prompt As String ' stores action prompt Next: or Command>
Dim Shared Alignment.Name1(1) As String * 7 ' stores three type 1 alignment names
Dim Shared Alignment.Name2(1) As String * 7 ' stores three type 2 alignment names
Dim Shared Allow.Break As Integer ' allows control-k checking during file output
Dim Shared Array(1) As String ' string array for mail
Dim Shared Array.List(1) As String * 16 ' string array for command help
Dim Shared Array.Number As Integer ' contains number of array element searched for
Dim Shared Baud.Override As Integer ' command line baud override
Dim Shared BBS.Name As String ' name of calling bbs name from rbbs .def file
Dim Shared Break As Integer ' returns -1 is control-k pressed
Dim Shared Buffer As String ' stores modem i/o
Dim Shared Carriage.Return As Integer ' supresses linefeed during output
Dim Shared Charges.Number As Integer ' contains charges of item searched for
Dim Shared Chained.Rooms As Integer ' used in launch routine
Dim Shared Chat As Integer ' chat mode flag
Dim Shared Choice As Integer ' monster number selected
Dim Shared Class.Name(1) As String * 15 ' stores ten class names
Dim Shared Color.Code As Integer ' current ansi color
Dim Shared Color.Graphics As Integer ' ansi color activated remote
Dim Shared Command.Name As String ' storage of user command
Dim Shared Continue As Integer ' flag for continuous output
Dim Shared ContainerRec As ContainerType ' container structure
Dim Shared Direction As Integer ' direction 1-11
Dim Shared Direction(1) As String * 9 ' stores eleven direction names
Dim Shared Door.Name As String ' name of calling bbs user from rbbs .def file
Dim Shared Door.Time As Single ' time left from rbbs .def file
Dim Shared Dungeon.Master As Integer ' user is dungeon master (-1/0)
Dim Shared Dungeon.Master.Assistant As Integer ' user is asst. DM (-1/0)
Dim Shared Entry.Command As Integer ' last command number used for room entry
Dim Shared FileName As String ' file name storage
Dim Shared FunctionBuffer As String ' buffer of interactive pressed function key
Dim Shared Governor As Integer ' user is governor (-1/0)
Dim Shared Graphics.Off As Integer ' set ansi graphics off (to white) (-1/0)
Dim Shared Guild.Master As Integer ' user is guild master (-1/0)
Dim Shared HelpRecord As HelpType ' help record structure
Dim Shared Hidden.Player As Integer ' player is hidden (-1/0)
Dim Shared Hidden As Integer ' mask input for security i/o with Mask$
Dim Shared High.Class.Name(1) As String * 15 ' ten high class names
Dim Shared Inpt As String ' user i/o input
Dim Shared Index.Number As Integer ' contains index number of an item searched for
Dim Shared Intoxicated As Integer ' user is intoxicated (equal to action prompts)
Dim Shared Invisible As Integer ' user is invisible (-1/0)
Dim Shared Jail.Room.Number As Single ' room to send user when jailed
Dim Shared Last.Command As String ' last command entered
Dim Shared Last.Command.Number As Integer ' number of last command entered
Dim Shared Last.Monster As String ' name of last monster attacked
Dim Shared Last.Room As Single ' stores temporary room number
Dim Shared Learned.Spells As String ' string of 1's and 0's of spells learned
Dim Shared Line.Length As Integer ' maximum length of input allowed
Dim Shared Line.Limit As Integer ' used with word wrap
Dim Shared Local.Mode As Integer ' user is logged in at console
Dim Shared Locks.Smashed As Integer ' used in launch routine
Dim Shared Logged.In As Integer ' flag indicating user is logged in
Dim Shared Magic.Spell As Integer ' magic spell number of spells.dat being cast
Dim Shared Max.Row As Integer ' maximum rows used for screen scrolling/output
Dim Shared Message1 As String ' stores message when user killed for death routine
Dim Shared Message.Length As Integer ' stores length of message in mail
Dim Shared Message.Reply As Integer ' flag for message reply
Dim Shared Message.Thread As Integer ' message number replied to
Dim Shared Message.To As String ' stores message to in mail
Dim Shared MessageRecord As MessageType ' message record structure for mail
Dim Shared Modem.Access As Integer ' modem in use flag
Dim Shared Modem.Error As Integer ' modem error flag
Dim Shared Modem.Baud As Integer ' connect speed, 00 truncated
Dim Shared Monclass As Integer ' current room monster class
Dim Shared Monclass.Max As Integer ' maximum monster classes in monclass array
Dim Shared MonclassRecord As MonclassType ' monster class record structure
Dim Shared Monster.Class(1, 1) As Integer ' array of monster classes
Dim Shared Monster.Cycle As Integer ' incrementing counter for monster attacks
Dim Shared Monster.Number As Integer ' stores currently accessed monster number
Dim Shared Monster.Percent(1, 1) As Integer ' array of monster percents
Dim Shared Monster.Rate(1, 1) As Integer ' array of monster encounter rates
Dim Shared Monster.Rate1 As Integer ' counts file monster rates
Dim Shared Monster.Rate2 As Integer ' counts room monster rates
Dim Shared MonsterArray(1) As MonsterType ' array of 20 monster record structures
Dim Shared MonsterIndex(1) As Integer ' array of 20 indexes to monster.dat file
Dim Shared MonsterRecord As MonsterType ' defines monster structure
Dim Shared Monsters.Killed As Integer ' total monsters killed in session
Dim Shared MonsterTalkRecord As MonsterTalkType ' monster class structure
Dim Shared Multiplier As Integer ' multiplier used in combat
Dim Shared New.Room As Integer ' added new room flag
Dim Shared Next.Room As Single ' next room storage
Dim Shared No As Integer ' entered N during input (-1/0)
Dim Shared No.Echo As Integer ' do not echo user input
Dim Shared No.Input As Integer ' null was entered (-1/0)
Dim Shared No.Input.Out As String ' string to return if null entered during input
Dim Shared Node As String ' node in string form
Dim Shared Node1 As Integer ' node number (0-9,A-Z)
Dim Shared Node2 As Integer ' node number (0-99)
Dim Shared Normal.User As Integer ' user has special attributes (-1/0)
Dim Shared Number.Appearing As Integer ' maximum monsters of one type encountered
Dim Shared Number.Monsters As Integer ' current number of active monsters in room
Dim Shared ObjectRecord As ObjectType ' object structure
Dim Shared Outpt As String ' user i/o output
Dim Shared Outpts As String ' temporary user i/o output variable
Dim Shared Parse.Count As Integer ' parsed count to decrement
Dim Shared Parse.Number As Integer ' stores command numbers (pound sign) quantity
Dim Shared Parsed.Command1 As String ' first or second command parsed from input
Dim Shared Parsed.Command2 As String ' first or second command parsed from input
Dim Shared Parser As Integer ' position of command second word parsed
Dim Shared Pass.Door As Integer ' passdoor spell cast
Dim Shared Player.CodeName As String ' storage for user codename
Dim Shared Player.PassWord As String ' storage for user password
Dim Shared Port As Integer ' modem port being used (0-7)
Dim Shared Port.Override As Integer ' command line override
Dim Shared Prefix1 As String ' the/or/an
Dim Shared Previous.Command As Integer ' -1=use,0=n/a,equals last command number
Dim Shared Previous.Command1 As String ' storage of previous command
Dim Shared Previous.Command2 As String ' storage of previous parsed command
Dim Shared Previous.Command3 As String ' storage of previous second parsed command
Dim Shared Private.Message As Integer ' flag for mail
Dim Shared Psi.Attack.Mode As Integer ' last offense psi spell cast
Dim Shared Psi.Defense.Mode As Integer ' last defense psi spell cast
Dim Shared Quit As Integer ' true if Q entered
Dim Shared Race(1) As String * 8 ' stores eight race names
Dim Shared Resurrection.Room As Integer ' specifies room number to be resurrected to
Dim Shared Room As Single ' current room number
Dim Shared Room.Health.Rate As Integer ' number of action prompts for health update
Dim Shared Room.Monster.Rate As Integer ' number of action promptsfor monster update
Dim Shared Room.Rate As Integer ' counter of action prompts for room rate
Dim Shared Room.Rust.Rate As Integer ' counter of room rust rate
Dim Shared Room.Steal.Rate As Integer ' counter of room steal rate
Dim Shared RoomRecord As RoomType ' room structure
Dim Shared Rust.Rate As Integer ' rust rate in action prompts
Dim Shared Share.Count As Integer ' count violations
Dim Shared Sorting As Integer ' inventory sorting (-1/0/1)
Dim Shared SpellRecord As SpellType ' spell structure
Dim Shared Spell.Number As Integer ' spell number being used
Dim Shared Spell.Cast.Type As Integer ' type of spell being cast
Dim Shared Stat(1) As String * 12 ' storage of eight statistics names
Dim Shared Steal.Rate As Integer ' steal rate in action prompts
Dim Shared Stored.Parsed.Command1 As String ' first parsed command from user input
Dim Shared Stored.Parsed.Command2 As String ' second parsed command from user input
Dim Shared Subject As String ' store mail subject
Dim Shared Sysop As Integer ' contains -1 if player is sysop
Dim Shared TableRecord As TableType ' message table for mail
Dim Shared Teleported As Integer ' user entered room by teleporting (-1/0)
Dim Shared Temp.Array1(1) As Integer ' global temporary variables
Dim Shared Temp.ArrayZ(1) As Double ' global temporary variables
Dim Shared Time.Left As Single ' amount of time user is allowed on in seconds
Dim Shared Time.On As String ' time user logged on in Date$ format
Dim Shared Timeon As Single ' time user logged on in Timer format
Dim Shared Timelogged.On As String * 8 ' time user logged on
Dim Shared Town.Mayor As Integer ' user is town mayor (-1/0)
Dim Shared Training.Stats(1, 1) As Integer ' storage for training statistics
Dim Shared TreasureRecord As TreasureType ' treasure structure
Dim Shared Two.Minutes.Left As Integer ' flag indicating two minutes warning shown
Dim Shared TypeNumber As Integer ' flag for type of object in room or inventory
Dim Shared Upper.Case As Integer ' flag to force first character uppercase in io.i
Dim Shared User.Echo As Integer ' stores online user echo preference
Dim Shared User.LineFeeds As Integer ' stores online user linefeed preference
Dim Shared User.LineLength As Integer ' stores online user linelength preference
Dim Shared User.PageLength As Integer ' stores online user pagelength preference
Dim Shared User.Wordwrap As Integer ' stores online user wordwrap preference
Dim Shared User.Command As String ' stores original user input
Dim Shared User.Index As Integer ' index of user number to users.dat file
Dim Shared User.Number As Integer ' alternate user index variable
Dim Shared UserRecord As UserType ' user structure
Dim Shared UserRecord2 As UserType ' alternate user structure
Dim Shared Vehicle1 As Integer ' vehicle index to room treasure array
Dim Shared Vehicle2 As Integer ' charges in vehicle
Dim Shared Vehicle3 As Integer ' index to treasure.dat file
Dim Shared Vehicle4 As Integer ' vehicle type (0-3)
Dim Shared WeaponTypeName(1) As String * 9 ' storage of four weapon names
Dim Shared Weapon1 As Integer ' plus of armor worn
Dim Shared Weapon2 As Integer ' plus of weapon held
Dim Shared Weapon3 As Integer ' plus in shield held
Dim Shared Weapon4 As Integer ' index to user inventory array of armor worn
Dim Shared Weapon5 As Integer ' index to user inventory array of shield held
Dim Shared Weapon6 As Integer ' index to user inventory array of weapon held
Dim Shared Weapon7 As Integer ' index to user inventory array of ring worn
Dim Shared Weapon8 As Integer ' ring type (1=antipoison/2=antileveldrain/3=antispell)
Dim Shared Weapon9 As Integer ' ring antispell (-1=generic/greater>0=spell number)
Dim Shared Weapon10 As Integer ' proficiency number of weapon held
Dim Shared Weapon11 As Integer ' item plus of ring being worn
Dim Shared Weight As Integer ' total amount in gold of user's inventory
Dim Shared Word.Wrap As Integer ' used for word wrap during input
Dim Shared Yes As Integer ' user entered Y during input (-1/0)

'  constants used in place of variables inline programs
Const Author$ = "Erik Jon Oredson" ' name of author
Const Dead$ = "-dead-" ' used internal to user file
Const Deleted$ = "<deleted>" ' used internal to data files
Const Email$ = "eoredson@gmail.com" ' author's email address
Const Enter$ = "<enter>" ' generic response substring
Const False = 0 ' false
Const Mask$ = "#" ' define mask character
Const MaxCommands = 85 ' max user commands
Const MaxSysopCommands = 9 ' max sysop commands
Const MaxInt = 32767 ' maximum integer
Const MaxStat = 250 ' maximum player statistic
Const MaxViolations = 1024 ' maximum share violations
Const MinInt = -32768 ' minimum integer
Const More$ = "More(y/n/c/q)?" ' generic response substring
Const None$ = "<null>" ' used internally
Const Nul = "" ' empty string
Const Program$ = "DNDDOOR" ' program name
Const Range$ = "<range error>" ' generic response substring
Const Release$ = "2.0a" ' program minor version
Const True = Not False ' true
Const UnTrue = False + 1 ' untrue
Const Version$ = "5.0a" ' program major version

'  constant variables corresponding to last command entered
Const Picklock.Command = 26 ' compare to picklock
Const Hide.Command = 49 ' compare to hide
Const Identify.Command = 52 ' compare to identify

'  constant variables for driving vehicles
Const RideVehicle = 48 ' ride vehicle command number
Const DriveVehicle = 73 ' drive vehicle command number

'  constant variables of weapon proficiency types
Const Blunt = 1 ' blunt weapon number
Const Pole = 2 ' pole weapon number
Const Sharp = 3 ' sharp weapon number
Const Thrusting = 4 ' thrusting weapon number

'  bit constants of room record flags array attributes of room inventory
Const Hidden.Object = 1 ' object was hidden with hide command
Const Magic.Trap = 2 ' object was made into a trap with spell
Const Magically.Hidden = 4 ' object was magically hidden with spell

'  constant variable used to indicate spell method being used
Const Use.Spell.Type = 1 ' spell cast is by the use command
Const Scroll.Spell.Type = 2 ' spell cast by using a scroll
Const Cast.Spell.Type = 4 ' spell cast with the cast command

'  bit constants of player class, character, and command flags,
'  bits 2 through 128 correspond to the seven statistics players may wish for.
Const Rerolled = 1 ' bit flag of user record flags field for rerolled player
Const Wished = 256 ' bit flag of user record flags field of player wished
Const Alignmented = 512 ' bit flag of user record flags field of player align
Const Locked.User = 1024 ' bit for locked mailbox
Const Special.Char1 = 2048 ' bit flag of user record flags field town mayor
Const Special.Char2 = 4096 ' bit flag of user record flags field governor
Const Special.Char3 = 8192 ' bit flag of user record flags field guild master
Const Special.Char4 = 16384 ' bit flag of user record flags field sysop

' constants of class type numbers
Const Fighter = 1 ' fighter class
Const MagicUser = 2 ' MU class
Const Thief = 3 ' thief class
Const Cleric = 4 ' cleric class
Const Paladin = 5 ' paladin class
Const Ranger = 6 ' ranger class
Const Druid = 7 ' druid class
Const Lady = 8 ' lady class
Const AsstDM = 9 ' asst. DM class
Const DM = 10 ' DM class

' constants of attack command numbers
Const Charm = 31 ' attack command number constant
Const Beguile = 32 ' attack command number constant
Const Shield = 34 ' attack command number constant
Const Guard = 35 ' attack command number constant
Const Lunge = 39 ' attack command number constant
Const Dodge = 40 ' attack command number constant
Const Killed = 55 ' attack command number constant
Const Backstab = 59 ' attack command number constant
Const Circled = 60 ' attack command number constant
Const Feint = 61 ' attack command number constant
Const Parry = 62 ' attack command number constant
Const Thrust = 63 ' attack command number constant
Const Charge = 64 ' attack command number constant
Const Turn = 65 ' attack command number constant
Const Counter = 66 ' attack command number constant
Const Bemuse = 67 ' attack command number constant
Const Befuddle = 68 ' attack command number constant
Const Pummel = 69 ' attack command number constant
Const Resist = 70 ' attack command number constant
Const PsiMode = 71 ' attack command number constant
Const Beat = 82 ' attack command number constant
Const Punch = 83 ' attack command number constant
Const Bewitch = 84 ' attack command number constant
Const Bewilder = 85 ' attack command number constant

' constants of spell type numbers
Const Enchant = 1 ' spell number constant
Const Offense = 2 ' spell number constant
Const Bless = 3 ' spell number constant
Const Wish = 4 ' spell number constant
Const Poison = 5 ' spell number constant
Const Vigor = 6 ' spell number constant
Const Heal = 7 ' spell number constant
Const CurePoison = 8 ' spell number constant
Const LevelDrain = 9 ' spell number constant
Const Teleport = 10 ' spell number constant
Const Befuddled = 11 ' spell number constant
Const TurnUndead = 12 ' spell number constant
Const PassDoor = 13 ' spell number constant
Const Conjure = 14 ' spell number constant
Const Psionic = 15 ' spell number constant
Const DetectLock = 16 ' spell number constant
Const DetectEvil = 17 ' spell number constant
Const DetectTrap = 18 ' spell number constant
Const Intoxicate = 19 ' spell number constant
Const SetTrap = 20 ' spell number constant
Const Hiding = 21 ' spell number constant
Const Search = 22 ' spell number constant
Const Invisibility = 23 ' spell number constant
Const Identify = 24 ' spell number constant
Const Enlighten = 25 ' spell number constant
Const Illuminate = 26 ' spell number constant
Const Psyche = 27 ' spell number constant
Const Telepathy = 28 ' spell number constant (not used)

' constants for psionic spells
Const PsiAttack = 1 ' psionic attack mode
Const PsiDefense = 2 ' psionic attack mode

' constants for room attribute 1
Const LitRoom = 0 ' flag constant for dark room
Const UnlitRoom = -1 ' bit flag constant for lit room

' constants for room attribute 2 and vehicle type
Const Land = 0 ' flag value for ground type room
Const Air = 1 ' flag value for above ground room
Const Water = 2 ' flag value constant for underwater room

' constants of room entry command
Const Fly = 74 ' entry command number
Const Swim = 79 ' entry command number
 
' constants for vehicle type
Const AllTerrain = 4 ' vehicle terrain type

' constants of file numbers
Const UserFile = 1 ' file number
Const NonPlayerFile = 2 ' file number
Const RoomFile = 3 ' file number
Const ObjectFile = 4 ' file number
Const MonsterFile = 5 ' file number
Const TreasureFile = 6 ' file number
Const MonTalkFile = 7 ' file number
Const MonClassFile = 8 ' file number
Const SpellFile = 9 ' file number
Const TableFile = 10 ' file number
Const MessageFile = 11 ' file number
Const ActionFile = 12 ' file number
Const TempFile = 13 ' file number
Const TempFile2 = 14 ' file number
Const HelpFile = 15 ' file number
Const UserFile2 = 16 ' file number
Const ModemFile = 17 ' modem file number

SysopCommands1:
Data "ABORT","HINT","CALL","EDIT","HELP","INVISIBLE","LINK","<>","STATUS"

SysopCommands2:
Data "<>","CALL","DISCARD","EDIT","GET","HELP","KILL","REDUCE","TELEPORT"

CommandSet1:
Data "KILL","ATTACK","HIT","STRIKE","READ","LOOK","RUN","PANIC","SEARCH","SAVE"
Data "INFORMATION","INVENTORY","SUICIDE","TRAIN","HEALTH","APPEAL","BYE","STOP","QUIT","END"
Data "EXPERIENCE","HELP","HIDE","CAST","LIST","BRIEF","BANK","TIME","DATE","CLOCK"
Data "CHARM","BEGUILE","ANSI","SHIELD","GUARD","<>","<>","<>","LUNGE","DODGE"
Data "TOP","BLESS","CURSE","PASSWORD","MAIL","USERS","CHAT","EXIT","LEAVE","CLIMB"
Data "STORE","LOGIN","RELOGIN","STATUS","LINELENGTH","CATALOG","LINEFEEDS","PAGELENGTH","ECHO","WORDWRAP"
Data "ALIGNMENT","<>","<>","NORTH","EAST","SOUTH","WEST","NORTHEAST","SOUTHEAST","SOUTHWEST"
Data "NORTHWEST","UP","DOWN","IN","OUT","HINT","SORT","COMMANDS","<>","<>"
Data "<>","<>","REROLL","<>","WISH"

CommandSet2:
Data "GO","ENTER","READ","EXAMINE","LOOK","USE","GET","TAKE","DROP","PUT"
Data "SELL","PAWN","BREAK","SMASH","CLOSE","OPEN","DRAW","WIELD","HOLD","WEAR"
Data "RETURN","CAST","BUY","LOCK","UNLOCK","PICKLOCK","FIX","REPAIR","RECHARGE","DRINK"
Data "<>","THROW","STEAL","<>","BLESS","CURSE","FUEL","<>","<>","TALK"
Data "PARLEY","BRIBE","OFFER","LOAD","FIRE","MOUNT","DISMOUNT","RIDE","HIDE","SEARCH"
Data "HELP","IDENTIFY","<>","<>","KILL","ATTACK","HIT","STRIKE","BACKSTAB","CIRCLE"
Data "FEINT","PARRY","THRUST","CHARGE","TURN","COUNTER","BEMUSE","BEFUDDLE","PUMMEL","RESIST"
Data "PSIONIC","EAT","DRIVE","FLY","MOVE","LAUNCH","LEARN","LIGHT","SWIM","EXIT"
Data "LEAVE","BEAT","PUNCH","BEWITCH","BEWILDER"

ConfigDATA:
Data 12,4,6,3,9,6,10,5,10,3,5,4,9,6,5,4,11,5,6,3,11,4,6,3,10,3,7,3,9,6,8,3,125,125,125,125,250,250,250,250
Data "Lord","Wizard","Assassin","Bard","Monk","Knight","Priest","Empress","Assistant DM","Dungeon Master"
Data "Human","Elf","Gnome","Dwarf","Halfling","Half-elf","Half-orc","Ogre"
Data "Fighter","Magic User","Thief","Cleric","Paladin","Ranger","Druid","Lady","Assistant DM","Dungeon Master"
Data "Strength","Intelligence","Wisdom","Dexterity","Constitution","Piety","Charisma"
Data "north","east","south","west","northeast","southeast","southwest","northwest","up","down","in","out"
Data "blunt","pole","sharp","thrusting","good","neutral","evil","lawful","neutral","chaotic"

Rem **** END INCLUDE FILE ****

'  main program. all processing starts here.
On Error GoTo Error.Routine ' error trapping
Randomize Timer ' reseed random generator
Call Read.Config ' loads all necessary variables, and configuration
Call Login ' logs in local or remote player
Do ' main input loop
   Call Encounter.Monster ' search for random monster encounter
   Call Health.Update ' increment/decrement player statistics
   Call Monster.Attack ' search for active monsters that attack
   Call New.Stats ' recalculate player statistics
   Call Restricted.Login ' compute restrictive time
   Call Rust.Weapon ' get weapon rusting rate
   Call Steal.Treasure ' get treasure stealing rate
   Call Sorter ' sorts player inventory
   If Room.Rate = False Then ' compare room rate reset
      Call Status.Line(False) ' redisplay status line with player statistics
   End If
   Call Main.Input ' get and process player input
Loop ' end main input loop
End ' prevent drop through of loop into routines

' global error routine
Error.Routine:
If Err = 70 Then
   Share.Count = Share.Count + 1
   If Share.Count < MaxViolations Then
      Resume
   End If
   Print "Share violation!"
   Print "IDE Line: "; _ErrorLine
   Resume Next
End If
If Modem.Access Then
   Modem.Error = Err
   Resume Next
End If
Color 7, 0 ' reset color
Print "Error:" + Str$(Err) ' print message
Print "IDE Line: "; _ErrorLine
Resume Next

Rem * Configure routine:
Rem * routine to read command line, dimension and load dynamic arrays,
Rem * open files, read door file, and read monster class arrays.

Sub Read.Config
   Call Get.Command ' read command line
   Call Read.Arrays ' load runtime arrays
   Call Open.Files ' open work files
   Call Read.Door ' read door file
   Call Read.Monclass ' load monster class arrays
End Sub ' end routine to read configure

Rem * routine to process action prompt input. all commands processed
Rem * in this routine call most major routines.

Sub Main.Input
   Graphics.Off = False ' ansi/color on
   Outpt = Action.Prompt ' set prompt (normally Do? is brief, Action? is verbose)
   Upper.Case = True ' set uppercase flag
   Word.Wrap = True ' set word wrap
   Call IO.I ' get player command input
   Upper.Case = False ' reset uppercase flag
   Word.Wrap = False ' reset word wrap
   If Len(FunctionBuffer) Then ' process funtion key during remote player on
      FunctionKeyNumber = Asc(FunctionBuffer) ' convert function key to variable
      FunctionBuffer = Nul ' reset function key buffer to null
      Call Function.Key(FunctionKeyNumber) ' process function key
      Exit Sub ' exit routine
   End If ' end process function key
   Inpt = LTrim$(Inpt) ' strip leading blanks from command
   If Inpt = "?" Then ' command is a question mark
      Graphics.Off = True ' ansi/color to white
      Outpt = "Type Hint for for short help, Help for detailed help," ' message
      Call IO.O ' send help output
      Outpt = "Catalog for command list, or Commands for detailed command list."
      Call IO.O ' send help output
      Graphics.Off = False ' set ansi/color on
      Exit Sub ' return to main
   End If ' end question mark command
   If Inpt = "??" Then ' check for brief help
      MaxArrayList = MaxCommands * 2
      ReDim Array.List(1 To MaxArrayList) As String * 16 ' dimension string array
      For Last.Command.Number = 1 To MaxArrayList
         Array.List(Last.Command.Number) = Nul
      Next
      Restore CommandSet1 ' reset data pointer
      DataElement = False ' reset data read counter
      For DataNumber = 1 To MaxArrayList ' loop to read all user commands
         Read Outpt ' read command
         Outpt = LCase$(Outpt) ' lowercase command
         Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase start
         If RTrim$(Outpt) <> "<>" Then ' check command name
            DataElement = DataElement + 1 ' increment data read counter
            Array.List(DataElement) = Outpt ' store command
         End If ' end check data read counter
      Next ' end command read loop
      For Sort1 = 1 To MaxArrayList ' bubble sort loop
         For Sort2 = Sort1 + 1 To MaxArrayList ' bubble sort inner loop
            If Array.List(Sort1) > Array.List(Sort2) Then ' compare values
               Swap Array.List(Sort1), Array.List(Sort2) ' swap values
            End If ' end compare values
         Next ' end inner sort
      Next ' end sort loop
      Item.Counter = False ' reset line count
      Line.Counter = 2 ' reset page count
      Outpt = Nul ' reset output line
      Previous.Item$ = Nul ' reset display item
      For DataNumber = 1 To MaxArrayList ' loop to read all user commands
         DataItem$ = RTrim$(Array.List(DataNumber)) ' store command
         If Previous.Item$ <> DataItem$ Then ' compare duplicates
            Previous.Item$ = DataItem$ ' store previous item
            Outpt = Outpt + DataItem$ + Space$(15 - Len(DataItem$)) ' make output line
            Item.Counter = Item.Counter + 1
            If Item.Counter = 5 Then ' count line length
               Item.Counter = False ' reset line count
               Outpt = RTrim$(Outpt) ' strip blanks
               Graphics.Off = True ' ansi color off
               Call IO.O ' send output
               Line.Counter = Line.Counter + 1 ' increment page count
               If Line.Counter >= User.PageLength Then ' compare to page length
                  Line.Counter = 2 ' reset lines counted
                  Call More.Prompt ' routine for more
                  If No Then ' check no reply
                     Exit Sub ' exit routine
                  End If ' end check reply
               End If ' end check page count
            End If ' end counter line length
         End If ' end compare duplicates
      Next ' end command read loop
      Erase Array.List ' clear temporary array
      Exit Sub ' exit routine
   End If ' end check help
   Sysop.Command = False ' set sysop entered command flag to false
   If Left$(Inpt, 1) = "!" Then ' sysop entered a command (sysop prefix of !)
      Sysop.Command = True ' set sysop entered command flag to true
      If Normal.User Then ' verify player has no special character
         Sysop.Command = False ' reset sysop command flag
         Outpt = "That command is reserved for DMs only!" ' format message
         Call IO.O ' send output
         Exit Sub ' return to main
      End If ' end verify normal player
      If Len(Inpt) <= 1 Then ' length of command is too brief
         Sysop.Command = False ' reset sysop command flag
         Outpt = "That command is reserved for DMs only!" ' message
         Call IO.O ' send output
         Exit Sub ' return to main
      End If ' end compare command length
      If Inpt = "!?" Then ' check for short dm help
         Outpt = "Brief list of DM commands:" ' make display message
         Call IO.O ' send output
         Command.Count = False ' reset counter
         MaxArrayList = MaxSysopCommands * 2
         ReDim Array.List(1 To MaxArrayList) As String * 16 ' dimension string array
         For Last.Command.Number = 1 To MaxArrayList
            Array.List(Last.Command.Number) = Nul
         Next
         Graphics.Off = True ' ansi color off
         Restore SysopCommands1 ' reset data pointer
         For Last.Command.Number = 1 To MaxArrayList ' loop to read all sysop commands
            Read Outpt ' read command
            If Outpt <> "<>" Then ' check valid command
               Select Case Last.Command.Number ' compare unique command
                  Case 1, 2, 3, 4, 5, 6, 7, 9, 12, 14, 16, 17, 18 ' commands
                     Command.Count = Command.Count + 1 ' increment counter
                     Outpt = LCase$(Outpt) ' lowercase command
                     Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase start
                     ' store prepended DM prefixed command
                     Array.List(Command.Count) = "!" + Outpt
               End Select
            End If ' end check command
         Next ' end command read loop
         For Sort1 = 1 To MaxArrayList ' bubble sort loop
            For Sort2 = Sort1 + 1 To MaxArrayList ' bubble sort inner loop
               If Array.List(Sort1) > Array.List(Sort2) Then ' compare values
                  Swap Array.List(Sort1), Array.List(Sort2) ' swap values
               End If ' end compare values
            Next ' end inner sort
         Next ' end sort loop
         For Last.Command.Number = 1 To MaxArrayList ' loop to display all sysop commands
            Outpt = RTrim$(Array.List(Last.Command.Number)) ' read DM command
            If Len(Outpt) Then
               Call IO.O ' send output
            End If
         Next ' end command read loop
         Erase Array.List ' clear temporary array
         Exit Sub ' exit command routine
      End If ' end check short dm help
   End If ' end compute sysop entered a command
   Stored.Parsed.Command1 = Inpt ' store the original command entered
   Inpt = UCase$(Inpt) ' convert original command to upper case
   Stored.Parsed.Command2 = Inpt ' store the upper case command
   If Left$(Inpt, 1) = "\" Then ' verify command is the repeat command character
      User.Command = Previous.Command1 ' restore previous command
      Stored.Parsed.Command1 = Previous.Command2 ' restore previous parsed
      Stored.Parsed.Command2 = Previous.Command3 ' restore previous parsed
   Else ' not the repeat command
      User.Command = Inpt ' store original command
      Previous.Command1 = User.Command ' store original command
      Previous.Command2 = Stored.Parsed.Command1 ' store original command
      Previous.Command3 = Stored.Parsed.Command2 ' store upper case command
   End If ' end compare repeat command
   If User.Command = Nul Then ' command entered is null
      Exit Sub ' return to main
   End If ' end command is null
   Magic.Spell = False ' reset magic spell is being cast flag
   Command.Name = User.Command ' store command entered
   If InStr(Command.Name, " ") Then ' check temporary command variable for space
      Command.Name = Left$(Command.Name, InStr(Command.Name, " ") - 1) ' parse left of
   End If ' space and store in temporary variable
   Parser = InStr(User.Command, " ") ' set parsed flag if command has space
   If Parser = False Then ' check command has no space/parameters
      If Sysop.Command Then ' command entered is sysop command
         Restore SysopCommands1 ' reset data pointer
         Command.Number = False ' set variable containing command number to false
         Command.Name = Mid$(Command.Name, 2) ' strip leading ! from command
         For Last.Command.Number = 1 To MaxSysopCommands ' loop to read all nine sysop commands
            Read Outpt ' read command
            If Left$(Outpt, Len(Command.Name)) = Command.Name Then ' compare to
               Command.Number = Last.Command.Number ' and store command number
               OutX$ = Outpt ' store command from data
               Exit For ' exit the loop, command found
            End If ' end compare
         Next ' end data read loop
         If Command.Number = False Then ' no command found
            Outpt = "Unknown command. Type !? for help." ' message
            Call IO.O ' send output
            Exit Sub ' return to main
         End If ' end check command variable
         Last.Command = OutX$ ' store data command
         Last.Command.Number = Command.Number ' store command number
         Select Case Command.Number ' select routine to call
            Case 1 ' abort function
               Call Abort
            Case 2 ' hint display
               Call Out.File("dmhint.doc")
            Case 3 ' summon random monster
               Call Call.Monster
            Case 4 ' editor routine
               Call Editor ' editor routine
            Case 5 ' read dm help
               Stored.Parsed.Command1 = "HELP" ' help text to search for
               Call Read.Help(True) ' true equals read dmhelp
            Case 6 ' toggle sysop invisible status
               Call Toggle.Invisible
            Case 7 ' enter room link edit routine
               Call Link.Room
            Case 9 ' display memory, free data space
               Call Display.Memory
         End Select ' end select routine
      Else ' command entered is not sysop command
         If User.Command = "OU" Then ' compare player command to parse of OUT
            User.Command = "O" ' reset player command to abbreviation of out
         End If ' end compare player command
         If User.Command = "OUT" Then ' compare player command to parse of OUT
            User.Command = "O" ' reset player command to abbreviation of out
         End If ' end compare player command
         ' routine to verify command entered is direction
         Call Get.Direction(Direction.Number)
         If Direction.Number Then ' returned variable true if direction
            Call Go.Direction(Direction.Number) ' routine goes certain direction
            Exit Sub ' return to main
         End If ' end verify direction entered
         Command.Number = False ' set variable containing command number to false
         Restore CommandSet1 ' reset data pointer
         For Last.Command.Number = 1 To MaxCommands ' loop to read all user commands
            Read Last.Command ' read command
            If Command.Name = Left$(Last.Command, Len(Command.Name)) Then ' compare
               Command.Number = Last.Command.Number ' command from data, left part
               Exit For ' of player command, set variable to command number and
            End If ' exit data read loop
         Next ' next data item
         If Command.Number = False Then ' loop ended without any matching commands
            Graphics.Off = True ' ansi/color off
            Outpt = "Unknown command! Type ? for help or ?? for a brief list of all commands."
            Call IO.O ' send output
            Graphics.Off = False ' ansi/color on
            Exit Sub ' return to main
         End If ' end command number
         Last.Command.Number = Command.Number ' set command variable to match
         Select Case Command.Number ' select command number
            Case 1 To 4 ' attack commands
               Parsed.Command1 = Last.Monster ' restore last monster number attacked
               Last.Command.Number = 55 ' attack command number
               Call Attack.Monster ' go attack the monster
            Case 5, 6 ' display the current room
               Call Display.Room
            Case 7, 8 ' panic the character
               Call Panic
            Case 9 ' search the current room
               Call Search.Room
            Case 10, 51 ' update the player file
               Call Share.Record(UserFile, User.Index) ' store player record
               Outpt = "Player file updated." ' message
               Call IO.O ' send output
            Case 11 ' display player information
               Call Display.Stats
            Case 12 ' display player inventory
               Call Display.Inventory
            Case 13 ' kill of the character
               Call Suicide
            Case 14 ' train character (must have 0 experience and enough gold)
               Call Train ' train routine
            Case 15 ' health command
               Call Display.Health
            Case 16 ' appeal for low stats/room w/o exits
               Call Appeal
            Case 17 To 20 ' end session
               Call Quit.Game
            Case 21 ' experience
               Call Display.Experience
            Case 22 ' enter help
               Stored.Parsed.Command1 = "HELP" ' get help text
               Call Read.Help(False) ' false equals help.dat file
            Case 23 ' hide
               Call Hide.User
            Case 24 ' cast command
               Call Cast.Spell(True) ' true equals enter chant
            Case 25 ' list command
               Call Weapon.List
            Case 26 ' brief mode toggle/must be level 2 at least
               Call Brief.Mode
            Case 27 ' bank command
               Call Bank
            Case 28 To 30 ' display time on/time left
               Call Time.Online
            Case 33 ' ansi on/off
               Call Toggle.ANSI
            Case 39 ' lunge attack command
               Parsed.Command1 = Last.Monster ' restore last monster attacked
               Last.Command.Number = 39 ' store lunge command number
               Call Attack.Monster ' go attack the monster
            Case 40 ' dodge attack command
               Parsed.Command1 = Last.Monster ' restore last monster attacked
               Last.Command.Number = 40 ' store dodge command number
               Call Attack.Monster ' go attack the monster
            Case 41 ' display top ten players/create ranklist.dat
               Call Share.Record(UserFile, User.Index) ' store player record
               Call Top.Ten ' generate top ten list
               Call Read.Record(UserFile, User.Index) ' read player record
            Case 42 ' bless w/o parameter
               Call Bless.Self ' therefore bless self
            Case 43 ' curse w/o parameters
               Call Curse.Self ' therefore curse self
            Case 44 ' change password
               Call Change.PassWord ' call routine
               Call Share.Record(UserFile, User.Index) ' store player record
            Case 45 ' mail routine
               Call Mail
            Case 46 ' display list of all players
               Call Share.Record(UserFile, User.Index) ' store player record
               Call User.List ' generate player list
               Call Read.Record(UserFile, User.Index) ' store player record
            Case 47 ' page sysop
               Call Page.Sysop
            Case 48, 49 ' exit/leave room
               Call Exit.Room
            Case 50 ' climb/go up/up
               Call Climb
            Case 52, 53 ' relogin if player level >1
               Call Relogin
            Case 54 ' display preferences
               Call Display.Prefs ' routine to display preferences
            Case 55 ' change line length
               Call Change.Linelength ' routine to change line length
            Case 56 ' display catalog file
               Call Out.File("catalog.doc") ' display text file
            Case 57 ' toggle linefeed mode
               Call Toggle.Linefeeds ' routine to toggle linefeed mode
            Case 58 ' change pagelength
               Call Change.Pagelength ' routine to change pagelength
            Case 59 ' change echo mode
               Call Toggle.Echo ' routine to toggle echo mode
            Case 60 ' toggle word wrap
               Call Toggle.Wordwrap ' routine to toggle word wrap
            Case 31 To 35 ' attack commands
               Call Attack.Monster ' go attack monster
            Case 61 ' realign once per character
               Call Align
            Case 64 To 75 ' direction entered
               ' set direction of data set number to 1-12
               Direction.Number = Command.Number - 63
               ' get the direction and move player
               Call Go.Direction(Direction.Number)
            Case 76 ' display hint file
               Call Out.File("hint.doc") ' display text file
            Case 77 ' toggle inventory sorting preference
               Call Sort.Inventory
            Case 78 ' display summary file
               Call Out.File("summary.doc") ' display text file
            Case 83 ' reroll all character statistics once per character
               Call Reroll.Character
            Case 85 ' wish for an item
               Call Wish.Item
         End Select
      End If ' end compare player command
   Else ' command has parameter(s)
      Parsed.Command1 = Mid$(User.Command, Parser + 1) ' get first parsed part of
      Parsed.Command2 = Parsed.Command1 ' command and store remaining parsed
      ' truncate stored parsed command
      Stored.Parsed.Command1 = Mid$(Stored.Parsed.Command1, InStr(Stored.Parsed.Command1, " ") + 1)
      Stored.Parsed.Command2 = Mid$(Stored.Parsed.Command2, InStr(Stored.Parsed.Command2, " ") + 1)
      Call Numeric ' store parse number
      If Sysop.Command Then ' command entered is sysop command
         Restore SysopCommands2 ' reset data pointer
         Command.Number = False ' set variable containing command number to false
         Command.Name = Mid$(Command.Name, 2) ' strip leading ! from command
         For Last.Command.Number = 1 To MaxSysopCommands ' loop to read all nine commands
            Read Outpt ' read command
            If Left$(Outpt, Len(Command.Name)) = Command.Name Then ' compare to,
               Command.Number = Last.Command.Number ' set command number variable
               OutX$ = Outpt ' store command from data
               Exit For ' exit loop, command found
            End If ' end compare sysop command
         Next ' end loop through data
         If Command.Number = False Then ' if no matching command in variable
            Outpt = "Unknown command. Type !? for help." ' message
            Call IO.O ' send output
            Exit Sub ' return to main
         End If ' end matching compare
         Last.Command = OutX$ ' store data command
         Last.Command.Number = Command.Number ' store command number
         Select Case Command.Number ' select sysop command number
            Case 2 ' get monster number and call
               Call Summon.Monster
            Case 3 ' get object/treasure name and discard
               Call Discard.Object
            Case 4 ' edit room and room monclass
               Change.Number = Val(Parsed.Command1) ' room number parameter to edit
               If Change.Number > False Then ' check file bounds
                  If Change.Number <= LOF(RoomFile) / Len(RoomRecord) Then ' file bounds
                     Call Change.Room(Change.Number) ' edit room number
                  End If ' end check room number range
               End If ' end check room number range
            Case 5 ' get object/treasure name
               Call Drop.Object ' get from files, drop in room
            Case 6 ' call DM help about command in parameter
               Call DM.Help
            Case 7 ' kill monster name in parameter
               Call Kill.Monster
            Case 8 ' reduce monsters by the value in number parameter
               Call Reduce.Monsters
            Case 9 ' teleport self/online player to room number in parameter
               Call Teleport.User
         End Select ' end select sysop command
      Else ' not sysop command
         Command.Number = False ' set variable containing command number to false
         Restore CommandSet2 ' reset data pointer
         For Last.Command.Number = 1 To MaxCommands ' loop to read all user commands
            Read Last.Command ' read command
            If Command.Name = Left$(Last.Command, Len(Command.Name)) Then ' compare
               Command.Number = Last.Command.Number ' to and store command number
               Exit For ' exit read data loop
            End If ' end compare
         Next ' end data loop
         If Command.Number = False Then ' if command number not matched
            Graphics.Off = True ' ansi/color off
            Outpt = "Unknown command! Type ? for help or ?? for a brief list of all commands."
            Call IO.O ' send output
            Graphics.Off = False ' ansi/color on
            Exit Sub ' return to main
         End If ' end command number match
         Last.Command.Number = Command.Number ' store command number
         Select Case Command.Number ' select command number
            Case 1, 2, 74, 79 ' enter/go object command
               Call Enter.Object
            Case 3 To 5 ' display player information commands
               Call Display.Information
            Case 6 ' cast spell on target
               Call Cast.Spell(False) ' false equals get chant
            Case 7, 8 ' take command
               If InStr(Parsed.Command2, " ") Then ' second parameter
                  Call Take.From.Container ' must be from container
               Else ' no extra parameters
                  Call Take.Object ' must be from room
               End If ' end compare second parameter
            Case 9, 10 ' drop command
               If InStr(Parsed.Command2, " ") Then ' second parameter
                  Call Drop.Into.Container ' must be into container
               Else ' no extra parameters
                  Call Drop.Item ' must be into room
               End If ' end compare second parameter
            Case 11, 12 ' sell an item
               Call Pawn.Shop
            Case 13, 14 ' smash commands
               Call Smash.Object
            Case 15 ' close command
               Call Close.Object
            Case 16 ' open command
               Call Open.Object
            Case 17 To 19 ' hold/wield command
               Call Hold.Object
            Case 20 ' wear command
               Call Wear.Object
            Case 21 ' return a weapon command
               Call Return.Object
            Case 22 ' cast spell on target
               Call Cast.Spell(True) ' true equals no chant
            Case 23 ' repair item
               Call Weapons.Shoppe
            Case 24 ' lock command
               Call Lock.Object
            Case 25, 26 ' unlock/picklock commands
               Call Unlock.Object
            Case 27 ' fix command
               Call Fix.Object
            Case 28 ' buy command
               Call Weapons.Shop
            Case 29 ' charge to repair magic item
               Call Alchemist
            Case 30 ' drink command
               Call Drink.Potion
            Case 32 ' throw command
               Call Throw.Object
            Case 33 ' steal command
               Call Steal.Object
            Case 35 ' bless a target
               Call Bless.Object
            Case 36 ' curse a target
               Call Curse.Object
            Case 37 ' fuel command
               Call Fuel.Object
            Case 40, 41 ' talk/parley commands
               Call Talk.To.Monster
            Case 42, 43 ' offer commands
               Call Offer
            Case 44 ' load command
               Call Load.Object
            Case 45 ' fire command
               Call Fire.Object
            Case 46 ' enter vehicle command
               Call Enter.Vehicle
            Case 48, 73 ' ride/go vehicle commands
               Call Ride.Vehicle
            Case 49 ' hide command
               If InStr(Parsed.Command2, " ") Then ' second parameter
                  Call Drop.Into.Container ' must be hide in container
               Else ' no extra parameters
                  Call Drop.Item ' must be hide in room
               End If ' end check hide parameters
            Case 50 ' search command
               Call Search.Object
            Case 51 ' help w/ parameter of help topic
               Call Read.Help(False) ' read help file
            Case 52 ' identify/extended information
               Call Identify.Object
            Case 55 To 58 ' attack routine
               Last.Command.Number = 55
               Call Attack.Monster
            Case 59 To 70, 82 To 85 ' attack routine
               Call Attack.Monster
            Case 71 ' psi mode, attack/defense
               Call Psi.Mode
            Case 72 ' eat command
               Call Eat.Object
            Case 75 ' move command
               Call Move.Object
            Case 76 ' launch command
               Call Launch.Object
            Case 77 ' learn spell, parameter equals spell name to learn
               Call Learn.Spell ' prompts for spell chant
            Case 78 ' light command
               Call Light.Object
            Case 47, 80, 81 ' exit vehicle commands
               Call Exit.Vehicle
         End Select ' end select commmand number
      End If ' end command type
   End If ' end parameter type
End Sub ' end command processing

Rem * routine to logoff player and terminate program.
Rem * any call to this routine results in program termination.

Sub Terminate.Program
   Logged.In = False ' reset player logged in flag
   Call Status.Line(-2) ' erase status lines
   Call Put.User.Record ' record player record structure
   Call Clean.Room ' remove extra treasure from room
   Graphics.Off = False ' reset color
   Outpt = "Your online time was:" ' make message
   Call IO.O ' display message
   Graphics.Off = True ' reset color
   Call Time.Online ' display time used and time left
   Call More.Prompt ' pause for more
   Call Top.Ten ' top player list
   Call Out.File("logoff.dat") ' display logoff text file
   Outpt = "Press " + Enter$ + " to exit the adventure:" ' exit program prompt
   No.Echo = True ' no echo of player input
   Line.Length = 1 ' get one character
   Call IO.I ' get player input
   No.Echo = False ' reset no echo player input
   Call IO.O ' empty C/R
   Outpt = "Returning you to " + BBS.Name + ".." ' display return message
   Call IO.O ' send output
   Call Reset.ANSI ' restore local color and remote ansi color
   Sleep 5 ' pause for 5 seconds
   System ' terminate program
End Sub ' end terminate program routine

Rem * routine to display current time, user's time on, and user's time left.
Rem * input variables:
Rem *   Timelogged.On - containing the user time on in system time format hh:mm:ss.
Rem *   Time.Left - containing the user's time limit in seconds from login.
Rem * processing variables:
Rem *   OnTime1 - contains the time calculation.
Rem *   Hours1 - contains hours since login.
Rem *   Minutes1 - contains minutes since login.
Rem *   Seconds1 - contains seconds since login.

Sub Time.Online
   Graphics.Off = True ' reset color
   Outpt = "It is now " + clock$ + "." ' make display message
   Call IO.O ' send output message
   Var1! = Val(Mid$(Time$, 1, 2)) * 3600! + Val(Mid$(Time$, 4, 2)) * 60! + Val(Mid$(Time$, 7, 2))
   Var2! = Val(Mid$(Timelogged.On$, 1, 2)) * 3600! + Val(Mid$(Timelogged.On$, 4, 2)) * 60! + Val(Mid$(Timelogged.On$, 7, 2))
   OnTime! = Var1! - Var2!: If OnTime! < 0! Then OnTime! = OnTime! + 86400!
   OnTime2! = OnTime!
   Outpt = "You have been on for" ' format time display message
   Time.Display$ = Nul ' time display message
   GoSub Time.Display ' subroutine to display message
   OnTime! = Time.Left! - OnTime2!
   Outpt = "You have" ' format time display message
   Time.Display$ = " remaining" ' time display message
   GoSub Time.Display ' subroutine to display message
   Exit Sub ' exit routine

   ' subroutine to display time message
   Time.Display:
   Hour1 = Int(OnTime! / 3600!): OnTime! = OnTime! - Hour1 * 3600!
   Minute1 = Int(OnTime! / 60!): OnTime! = OnTime! - Minute1 * 60!: Second1 = OnTime!
   If Hour1 > 0 Then ' compare hours of time variable
      If Hour1 = 1 Then
         Outpt = Outpt + " 1 hour,"
      Else
         Outpt = Outpt + Str$(Hour1) + " hours," ' append hours to string
      End If
   End If ' end compare hours
   If Minute1 > 0 Then ' compare minutes of time variable
      If Minute1 = 1 Then
         Outpt = Outpt + " 1 minute,"
      Else
         Outpt = Outpt + Str$(Minute1) + " minutes," ' append minutes to string
      End If
   End If ' end compare minutes
   If Second1 > 0 Then ' compare seconds of time variable
      If Second1 = 1 Then
         Outpt = Outpt + " 1 second,"
      Else
         Outpt = Outpt + Str$(Second1) + " seconds," ' append seconds to string
      End If
   End If ' end compare seconds
   If Hour1 <= 0 And Minute1 <= 0 And Second1 <= 0 Then
      Outpt = Outpt + " 0 seconds."
   End If
   Outpt = Left$(Outpt, Len(Outpt) - 1) ' trim trailing comma
   Outpt = Outpt + Time.Display$ + "." ' combine message
   Call IO.O ' send output message
   Return ' exit time display subroutine
End Sub ' end routine to display time on

Rem * routine to allow player login with another codename.

Sub Relogin
   If Normal.User Then ' check non DM
      If UserRecord.Level <= 1 Then ' minimum player level requirement to relogin
         Outpt = "You can't relogin until level two!" ' display restriction
         Call IO.O ' send output
         Exit Sub ' return from routine
      End If ' end check minimum level
   End If ' end check normal player
   Outpt = "Are you sure you want to relogin(y/n)? " ' verify player relogin
   No.Input.Out = "N" ' default input to no
   Call IO.I ' get player input
   If Yes Then ' verified to suicide
      Logged.In = False ' reset player logged in flag
      Call Put.User.Record ' store player record structure
      Call Clean.Room ' remove extra treasure from room
      Call Login ' get player codename
   End If ' end check response
End Sub ' end relogin routine

Rem * routine allows player to erase character.
Rem * sometimes player characters are no longer useful
Rem * when their statistics become too low.

Sub Suicide
   Outpt = "Are you sure you want to commit suicide(y/n)? " ' verify player suicide
   No.Input.Out = "N" ' default input to no
   Call IO.I ' get player input
   If Yes Then ' verified to suicide
      Logged.In = False ' reset player logged in flag
      Graphics.Off = True ' ansi/color to white
      Outpt = "Your character falls into deep sleep.." ' display suicide message
      Call IO.O ' send output
      Outpt = "   The Ghods take your player to another world.." ' display
      Call IO.O ' send output
      Graphics.Off = False ' ansi/color to normal
      Call Delete.User ' remove player record from file
      Call Share.Record(UserFile, User.Index) ' put deleted record in player file
      Call Terminate.Program ' end program
   End If ' end check response
End Sub ' end suicide routine

Rem * routine for normal logoff.

Sub Quit.Game
   Graphics.Off = True ' ansi/color to white
   Outpt = "Are you sure you want to quit(y/n)? " ' verify player to quit
   No.Input.Out = "N" ' default to no
   Call IO.I ' get input
   If Yes Then ' verified to quit
      Logged.In = False ' reset player logged in flag
      Call Terminate.Program ' end program
   End If ' end check response
End Sub ' end quit routine

Rem * routine to quit and terminate quick.

Sub Abort
   Outpt = "Are you sure you want to abort(y/n)? " ' verify abort
   No.Input.Out = "N" ' default to no
   Call IO.I ' get input
   If Yes Then ' verified yes
      Logged.In = False ' reset player logged in flag
      Outpt = "Program task abort!" ' display abort message
      Call IO.O ' output
      Call Put.User.Record ' store player record structure
      Call Reset.ANSI ' restore ansi/color
      Call Status.Line(-2) ' remove status lines
      Sleep 5 ' pause for 5 seconds
      System ' end program
   End If ' end check response
End Sub ' end abort routine

Rem * routine reads command line node number 0-9, A-Z, 10-99, and switches.

Sub Get.Command
   Locate , , 1 ' turn on cursor
   Max.Row = 24 ' set maximum rows on screen before scroll
   Local.Mode = False ' default not local mode
   Node1 = False ' reset node number
   Node2 = False ' reset node number
   User.Echo = False ' reset preference
   User.LineFeeds = False ' reset preference
   User.LineLength = 80 ' reset preference
   User.PageLength = 24 ' reset preference
   User.Wordwrap = False ' reset preference
   Do
      Command.Line$ = UCase$(Command$) ' store command line
      Do ' loop to remove spaces form command
         Imbedded = InStr(Command.Line$, " ") ' check for space
         If Imbedded Then ' verify space
            Command.Line$ = Left$(Command.Line$, Imbedded - 1) + Mid$(Command.Line$, Imbedded + 1) ' remove space
         Else ' verify space
            Exit Do ' exit no space
         End If ' end check space
      Loop ' end loop to remove spaces
      If Command.Line$ = "" Or Command.Line$ = "/?" Then ' check command
         Exit Do ' jump to command display
      End If ' end check command
      Imbedded = InStr(Command.Line$, "/P:") ' check port over-ride switch
      If Imbedded Then ' check switch
         Store.Command$ = Left$(Command.Line$, Imbedded - 1) ' store command line
         Command.Line$ = Mid$(Command.Line$, Imbedded + 3) ' get remaining command line
         GoSub Get.Numeric ' convert command to value
         Command.Line$ = Store.Command$ + Command.Line$ ' restore command line
         Port.Value = Int(Val(Value$)) ' store value
         If Port.Value >= 1 And Port.Value <= 16 Then ' compare value range
            Port.Override = Port.Value ' store value
         End If ' end compare
         If Port.Override = False Then ' check port value
            Exit Do ' jump to usage
         End If ' end check port
      End If ' end check switch
      Imbedded = InStr(Command.Line$, "/B:") ' check baud over-ride switch
      If Imbedded Then ' check switch
         Store.Command$ = Left$(Command.Line$, Imbedded - 1) ' store command line
         Command.Line$ = Mid$(Command.Line$, Imbedded + 3) ' get remaining command line
         GoSub Get.Numeric ' convert command to value
         Command.Line$ = Store.Command$ + Command.Line$ ' restore command line
         Baud.Value! = Int(Val(Value$)) ' store value
         If Baud.Value! > 0! Then ' compare value range
            Baud.Override = CInt(Baud.Value! / 100!) ' store value
         End If ' end compare
         If Baud.Override = False Then ' check baud value
            Exit Do ' jump to usage
         End If ' end check baud
      End If ' end check switch
      Imbedded = InStr(Command.Line$, "/L") ' check command
      If Imbedded Then ' verify command is local mode
         ' take switch from command
         Command.Line$ = Left$(Command.Line$, Imbedded - 1) + Mid$(Command.Line$, Imbedded + 2)
         Command.Line$ = RTrim$(Command.Line$) ' trim command
         Command.Line$ = LTrim$(Command.Line$) ' trim command
         Local.Mode = True ' reset local mode
      End If ' end verify command switch
      If Len(Command.Line$) = False Then ' check command
         If Local.Mode Or Port.Override Then ' check switches
            Node = "<n/a>" ' assign default node type
            Exit Sub ' exit from command line routine
         End If ' end check switches
      End If ' end check command
      If Left$(Command.Line$, 1) = "/" Then ' check for command switch
         Command.Line$ = Mid$(Command.Line$, 2) ' store command line
         Select Case Len(Command.Line$) ' get length of command line node
            Case 1 ' command line length
               Select Case Command.Line$ ' get command line node number
                  Case "0" To "9", "A" To "Z" ' command line node
                     Node1 = Asc(Command.Line$) ' assign to node type 1 variable
               End Select ' end command line node number
            Case 2 ' command line length
               Select Case Int(Val(Command.Line$)) ' select extended node number
                  Case 10 To 99 ' command line node
                     Node2 = Int(Val(Command.Line$)) ' assign to node type 2 variable
               End Select ' end select node number
         End Select ' end select node number
      End If ' end check command
      If Node1 > False Or Node2 > False Then ' verify valid node number
         Exit Sub ' exit routine
      Else
         Exit Do
      End If ' end check node number
   Loop
   Color 15, 0 ' color white on black
   Print "Dnddoor usage:"
   Color 14, 0 ' color yellow on black
   Print "   Dnddoor [/#] [/L] [/P:x] [/B:<rate>] [/A:<base>]"
   Color 15, 0 ' color white on black
   Print "Where:"
   Color 14, 0 ' color yellow on black
   Print "   /# is door node number 0 - 9, A - Z, or 10 - 99."
   Print "   /L to logon without door support."
   Print "   /P:x  over-ride port number (1 to 16)."
   Print "   /B:<rate>  over-ride connect baud rate."
   Color 7, 0 ' color white on black
   End ' halt program

   ' routine to convert string to value
   Get.Numeric:
   Value$ = Nul ' reset value
   Do ' start convert loop
      Line.Value$ = Left$(Command.Line$, 1) ' get command line character
      If Line.Value$ >= "0" And Line.Value$ <= "9" Then ' check ascii value
         Value$ = Value$ + Line.Value$ ' store value
         Command.Line$ = Mid$(Command.Line$, 2) ' get next command character
      Else ' check ascii value
         Exit Do ' exit routine loop
      End If ' end check ascii value
   Loop ' end convert loop
   Return ' exit form routine
End Sub ' end routine to read command line

Rem * routine reads runtime arrays.

Sub Read.Arrays
   ' allocate runtime arrays
   ReDim Alignment.Name1(1 To 3) As String * 7
   ReDim Alignment.Name2(1 To 3) As String * 7
   ReDim Class.Name(1 To 10) As String * 15
   ReDim Direction(1 To 12) As String * 9
   ReDim High.Class.Name(1 To 10) As String * 15
   ReDim Race(1 To 8) As String * 8
   ReDim Stat(1 To 7) As String * 12
   ReDim Training.Stats(1 To 10, 1 To 4) As Integer
   ReDim WeaponTypeName(1 To 4) As String * 9
   Restore ConfigDATA
   For Array.Index1 = 1 To 10 ' loop through ten class statistic increment array
      For Array.Index2 = 1 To 4 ' loop through each four class statistic array
         Read Training.Stats(Array.Index1, Array.Index2) ' read statistics
      Next ' loop through
   Next ' loop through
   For Array.Index2 = 1 To 10 ' loop through high class names
      Read High.Class.Name(Array.Index2) ' read high class names
   Next ' loop through
   For Array.Index2 = 1 To 8 ' loop through race names
      Read Race(Array.Index2) ' read race names
   Next ' loop through
   For Array.Index2 = 1 To 10 ' loop through class names
      Read Class.Name(Array.Index2) ' read class names
   Next ' loop through
   For Array.Index2 = 1 To 7 ' loop through statistic names
      Read Stat(Array.Index2) ' read statistic names
   Next ' loop through
   For Array.Index2 = 1 To 12 ' loop through direction names
      Read Direction(Array.Index2) ' read direction names
   Next ' loop through
   For Array.Index2 = 1 To 4 ' loop through weapon proficiency names
      Read WeaponTypeName(Array.Index2) ' read weapons proficiency names
   Next ' loop through
   For Array.Index2 = 1 To 3 ' loop through first set of alignment names
      Read Alignment.Name1(Array.Index2) ' read alignment names
   Next ' loop through
   For Array.Index2 = 1 To 3 ' loop through second set of alignment names
      Read Alignment.Name2(Array.Index2) ' read alignment names
   Next ' loop through
End Sub ' end routine to read arrays

Rem * routine opens runtime file handles.

Sub Open.Files
   Close ' close all files
   ' files opened in random mode, opened in share mode for multi access, and each
   ' file record length specified as the length of the random record which will
   ' be used to get/put data to the file. 10 files, 2 mail files. Remaining unopened
   ' files include 2 temp files, 1 help file.
   Open "actions.dat" For Random Shared As #ActionFile Len = Len(ActionRecord)
   Open "messages.dat" For Random Shared As #MessageFile Len = Len(MessageRecord)
   Open "montalk.dat" For Random Shared As #MonTalkFile Len = Len(MonsterTalkRecord)
   Open "monclass.dat" For Random Shared As #MonClassFile Len = Len(MonclassRecord)
   Open "monsters.dat" For Random Shared As #MonsterFile Len = Len(MonsterRecord)
   Open "msgtable.dat" For Random Shared As #TableFile Len = Len(TableRecord)
   Open "nonplyrs.dat" For Random Shared As #NonPlayerFile Len = Len(MonsterRecord)
   Open "objects.dat" For Random Shared As #ObjectFile Len = Len(ObjectRecord)
   Open "rooms.dat" For Random Shared As #RoomFile Len = Len(RoomRecord)
   Open "spells.dat" For Random Shared As #SpellFile Len = Len(SpellRecord)
   Open "treasure.dat" For Random Shared As #TreasureFile Len = Len(TreasureRecord)
   Open "users.dat" For Random Shared As #UserFile Len = Len(UserRecord)
End Sub ' end routine to open files

Rem * routine makes door filename.

Sub Read.Door
   BBS.Name = Nul ' reset name of bbs
   Color.Graphics = True ' default remote ansi color on
   Door.Name = Nul ' reset name of user
   Door.Time = 1800! ' reset time remaining
   If Node1 Then ' check node type 1
      Node = Chr$(Node1) ' store node in string form
   Else ' check door node
      If Node2 Then ' check node type 2
         Node = Mid$(Str$(Node2), 2) ' store node in string form
      End If ' end check node
   End If ' end check node type
   If Local.Mode Then ' verify local command line switch
      BBS.Name = "SYSTEM" ' store name of bbs
      Port = False ' reset modem port
      Modem.Baud = 96 ' reset modem baud
      Door.Name = "LOCAL SYSOP" ' store name of user
      Color.Type = 2 ' store color value
      Exit Sub ' exit door file routine
   End If ' end verify local switch
   If Node1 = False And Node2 = False Then ' check node type
      BBS.Name = "SYSTEM" ' store name of bbs
      Port = False ' reset modem port
      Modem.Baud = 96 ' reset modem baud
      Door.Name = "REMOTE USER" ' store name of user
      Color.Type = 2 ' store color value
      If Port.Override Then ' verify port number for no node
         Port = Port.Override - 1 ' store port value
      End If ' end check port/node
      If Baud.Override Then ' check baud value for no node
         Modem.Baud = Baud.Override ' store baud value
      End If ' end check baud/node
      Call Open.Modem ' init baud rate
      Exit Sub ' exit routine
   End If ' end check node type
   If Len(Node) = 1 Then ' check node length
      FileName = "DORINFO" + Node + ".DEF" ' make door info filename
      If _FileExists(FileName) Then ' check file exists
         Call Read.Door.File ' routine to read door file
         Exit Sub ' exit door file routine
      End If ' end check file
   End If ' end check node length
   FileName = "DINF" + Node + ".DEF" ' make door info filename
   If _FileExists(FileName) Then ' check file exists
      Call Read.Door.File ' routine to read door file
      Exit Sub ' exit door file routine
   End If ' end check file
End Sub ' end routine to read door file

Rem * routine reads door file.

Sub Read.Door.File
   Close #TempFile ' close work file
   Open FileName For Input Shared As #TempFile ' open the door file for input
   For Door.File.Line = 1 To 13 ' loop through all lines in door file
      Line Input #TempFile, Inpt ' read next text line from door file
      Inpt = UCase$(Inpt) ' convert text string to upper case
      Select Case Door.File.Line ' select which line to process
         Case 1 ' line one
            BBS.Name = Inpt ' store name of bbs
         Case 4 ' line four
            Port = Val(Mid$(Inpt, 4, 1)) ' set port to value of rightmost character
            If Port = False Then ' port zero speficies local console mode
               Local.Mode = True ' set local mode flag
            Else ' other port number
               Port = Port - 1 ' convert modem comm (1-8) to modem port (0-7)
               If Port.Override Then ' check over-ride switch
                  Port = Port.Override - 1 ' store value
               End If ' end check switch
            End If ' end compare port number
         Case 5 ' line five
            Input.Length = InStr(Inpt, " ") - 3 ' get baud rate
            Inpt = Left$(Inpt, Input.Length) ' store rate
            Modem.Baud = Int(Val(Inpt)) ' store modem baud rate
            If Baud.Override Then ' check over-ride switch
               Modem.Baud = Baud.Override ' store value
            End If ' end check switch
         Case 7 ' line seven
            Door.Name = Inpt ' store name of user
         Case 8 ' line eight
            If Len(Inpt) Then ' compare length of user last name
               Door.Name = Door.Name + " " + Inpt ' append user last name
            End If ' end compare user name length
         Case 10 ' line ten
            Color.Type = Int(Val(Inpt)) ' get value of text line
            If Color.Type < 2 Then ' compare graphics preference
               Color.Graphics = False ' value other than two sets graphics off
            End If ' end compare graphcis
         Case 12 ' line twelve
            Door.Time = Val(Inpt) * 60! ' store value of text line of remaining time
      End Select ' end process line
   Next ' end loop through file
   Call Open.Modem
End Sub ' end routine to read door file

Rem * routine loads runtime file info.

Sub Read.Monclass
   Locate , , 1 ' turn on cursor
   Monclass.Max = LOF(MonClassFile) / Len(MonclassRecord) ' compute number of records
   If Monclass.Max = False Then ' monster class file, compare to minimum
      Monclass.Max = 1 ' and increment if none found
      For Monster.Classes = 1 To 10 ' loop through all monsters in class
         Call PutMonclassRecordMonsters(Monster.Classes, False) ' reset to empty
      Next ' end loop
      Call Share.Record(MonClassFile, 1) ' write first monster class record if empty
   End If ' end compare length of file
   ' check upper bounds on monclass file
   If Monclass.Max > 1024 Then ' check bounds
      Monclass.Max = 1024 ' reset maximum
   End If ' end check bounds
   ' allocate monster class, rate, and percent arrays
   ReDim Monster.Class(1 To Monclass.Max, 1 To 10) As Integer
   ReDim Monster.Percent(1 To Monclass.Max, 1 To 10) As Integer
   ReDim Monster.Rate(1 To Monclass.Max, 1 To 10) As Integer
   ' loop through all monster classes in file
   For Monster.Classes = 1 To Monclass.Max
      Call Read.Record(MonClassFile, Monster.Classes) ' read monster class record
      For Monster.Class.Number = 1 To 10 ' loop through all monsters in record
         ' get monster number in class
         Monster.Array.Number = GetMonclassRecordMonsters(Monster.Class.Number)
         ' store in monster class array
         Monster.Class(Monster.Classes, Monster.Class.Number) = Monster.Array.Number
         If Monster.Array.Number > False And Monster.Array.Number <= LOF(MonsterFile) / Len(MonsterRecord) Then
            ' read monster number from monster file
            Call Read.Record(MonsterFile, Monster.Array.Number)
            ' store encounter rate
            Monster.Rate(Monster.Classes, Monster.Class.Number) = MonsterRecord.Rate
            ' store encounter rate percent
            Monster.Percent(Monster.Classes, Monster.Class.Number) = MonsterRecord.RatePercent
         End If ' end compare file bounds
      Next ' end loop through ten monsters in class
   Next ' end loop through all monster classes
   Max.Spells = LOF(SpellFile) / Len(SpellRecord) ' get number of spell records
   ' check bounds of spell file
   If Max.Spells > 1024 Then ' check bounds
      Max.Spells = 1024 ' reste maximum
   End If ' end check bounds
   Spell.Number = Len(Learned.Spells) ' get length of learned spells array
   If Max.Spells > Spell.Number Then ' compare spells length
      ' extended array
      Learned.Spells = Learned.Spells + String$(Max.Spells - Spell.Number, "0")
   End If ' end compare length
End Sub ' end configure routine

Rem * routine matches online baud rate
Rem * and opens modem as file.

Sub Open.Modem
   If Local.Mode Then ' check online mode
      _Title "DNDDOOR - LOCAL"
      Exit Sub ' return from routine
   End If ' end check online mode
   Modem.Speed$ = "96" ' default baud
   Select Case Modem.Baud ' select baud rate to match
      Case 3 ' 300 baud
         Modem.Speed$ = "3" ' calculate baud
      Case 6 ' 600 baud
         Modem.Speed$ = "6" ' calculate baud
      Case 12 ' 1200 baud
         Modem.Speed$ = "12" ' calculate baud
      Case 24, 48 ' 2400 baud
         Modem.Speed$ = "24" ' calculate baud
      Case 96, 144 ' multiple bauds
         Modem.Speed$ = "96" ' calculate baud
      Case 192, 288 ' multiple bauds
         Modem.Speed$ = "192" ' calculate baud
      Case 384, 576, 786 ' multiple bauds
         Modem.Speed$ = "192" ' calculate baud
      Case 1152, 2304 ' multiple bauds
         Modem.Speed$ = "1152" ' calculate baud
   End Select ' end calculate baud

   ' open modem as file
   Modem.Access = True
   Modem.Error = False
   P$ = "COM" + Mid$(Str$(Port + 1), 2) + ":"
   FileName$ = P$ + Modem.Speed$ + "00,N,8,1,CD1"
   Close ModemFile
   Open FileName$ For Random As #ModemFile
   If Modem.Error Then
      Modem.Access = False
      Call Lost.Carrier
      End
   End If
   Put #ModemFile, 0 ' test modem
   Modem.Access = False
   If Modem.Error Then
      Call Lost.Carrier
      End
   End If
   _Title "DNDDOOR - " + P$
End Sub

Rem * standard output routine to send data to the screen and modem.
Rem * input variables:
Rem *   Allow.Break - set to true to allow control-k checking.
Rem *   Carriage.Return - set to true to suppress cr/lf after output.
Rem *   Outpt - contains text to output to screen/modem.
Rem * output variables:
Rem *   Break - set to true if user entered control-k.
Rem *   Carriage.Return, Outpt - reset to zero/null.
Rem * processing variables:
Rem *   Char.Output$ - character to send to modem/screen.
Rem *   Count - loop variable for character of output sent.

Sub IO.O
   If Len(Outpt) Then ' check length of output string
      Call Out.ANSI ' routine to output ansi sequence to modem/color code locally
   End If ' end check output string
   Count = False ' reset output character counter
   Do ' output characters loop
      Call Keyboard(Keyboard.Break) ' routine reads keyboard
      If Break Then ' check break flag
         Exit Do ' exit output loop
      End If ' end check break flag
      Count = Count + 1 ' increment output counter
      GoSub IO.Wrap ' subroutine to word wrap
      If Count > Len(Outpt) Then ' check length of output
         Exit Do ' exit output loop
      End If ' end check length of output
      Char.Output$ = Mid$(Outpt, Count, 1) ' get single character to output
      Call Put.Modem(Char.Output$) ' send character to modem
      Call Scrn(Char.Output$) ' send character to screen
   Loop ' output character loop
   If Carriage.Return = False Then ' check output cr/lf suppressed
      If Break = False Then ' check control-k flag
         Call Line.Return ' routine to send cr/lf to modem/screen
      End If ' end check control-k flag
   End If ' end check i/o cr/lf
   Carriage.Return = False ' reset supressed cr/lf
   Outpt = Nul ' reset output
   Exit Sub ' exit routine

   ' subroutine to word wrap
   IO.Wrap:
   If User.LineLength = False Then ' check linelength
      Return ' exit subroutine
   End If ' end check of linelength
   Select Case User.Wordwrap ' check word wrap preference
      Case True ' word wrap toggled off
         If Pos(0) >= User.LineLength Then ' check line length
            If Mid$(Outpt, Count, 1) = " " Then ' check wrapped space
               Count = Count + 1 ' increment space from loop
            End If ' end check wrapped space
            If Count <= Len(Outpt) Then ' check end of line
               Call Line.Return ' send cr/lf
            End If ' check eol
         End If ' end check line length
      Case False ' word wrap toggled on
         Space.Wrap = InStr(Count + 1, Outpt, " ") ' store next word
         If Space.Wrap Then ' find first imbedded space
            Word$ = Mid$(Outpt, Count) ' truncate word
            Space.Wrap = InStr(2, Word$, " ") ' store next word
            Word$ = Left$(Word$, Space.Wrap - 1) ' store remaining output of line
         Else ' store empty remaining word to wrap
            If Mid$(Outpt, Count, 1) = " " Then ' verify last possible word
               Word$ = Mid$(Outpt, Count) ' store last remaining word
            Else ' calculate last word
               Word$ = Nul ' store empty word
            End If ' end calculate last word
         End If ' end storing next word to send
         ' compute line position plus next word
         If Pos(0) + Len(Word$) > User.LineLength Then
            If Mid$(Outpt, Count, 1) = " " Then ' check next word is space
               Count = Count + 1 ' increment space from loop
            End If ' end check next word is space
            Call Line.Return ' next word wraps, send cr/lf
         End If ' end compute wrap length
   End Select ' end select word wrap preference
   Return ' exit subroutine
End Sub ' end routine to send a string to the modem and screen

Rem * standard input routine to get data from the keyboard and modem.
Rem * input variables:
Rem *   Buffer - any previous user input stored during output.
Rem *   Carriage.Return - set to true to suppress cr/lf after output.
Rem *   Hide - flag to echo mask characters in place of input.
Rem *   Line.Length - maximum characters allowed to input.
Rem *   No.Echo - do not echo user input.
Rem *   No.Input.Out - string to use if no input entered.
Rem *   Outpt - contains input prompt.
Rem *   Word.Wrap - allow words entered to wrap.
Rem * output variables:
Rem *   Inpt - contains user input string.
Rem *   No.Input - flag set if return entered.
Rem *   No - set if N was entered.
Rem *   Yes - set if Y was entered.
Rem *   Quit - set if Q was entered.
Rem * processing variables:
Rem *   Buffered.Input$ 0 stores buffered input to add to next input prompt.
Rem *   Char$ - current character received from modem/keyboard.
Rem *   Char - ascii code of character input.
Rem *   Line.Limit - stores length of input line during input.
Rem *   Time.Out! - current time character input loop entered.
Rem *   Word - set if a word wrapped.
Rem *   OutY$, Valid.Char$, Last.Wrap, Last.Word - local work variables.

Sub IO.I
   Static Buffered.Input$ ' stores data input between calls to input routine
   If Pos(0) > 1 Then ' check io supressed cr/lf
      Call Line.Return ' make cr/lf before input prompt
   End If ' end check previous supressed cr/lf
   Carriage.Return = True ' set cr/lf flag
   Line.Limit = Len(Outpt) ' reset length of line for io.o
   Outpt = Outpt + Buffered.Input$ ' add stored data input to input prompt
   Call IO.O ' output the input prompt
   Inpt = Buffered.Input$ ' reset input string+data input between calls
   Buffered.Input$ = Nul ' reset data input between calls
   Do ' loop through input characters
      Char$ = Nul ' reset input character
      Time.Out! = Timer ' set time loop entered
      Do While Char$ = Nul ' loop until received a character
         GoSub IO.Timing ' routine to calculate timing values
         Call Keyboard(Keyboard.Break) ' routine to get any keyboard input
         ' keyboard input received is a function key/extended key
         If Keyboard.Break Then
            Buffer = Nul ' reset type ahead buffer
            Carriage.Return = False ' reset cr/lf flag
            No.Input.Out = Nul ' reset null input echo string
            Exit Sub ' exit routine to process extended keyboard input
         End If ' end check keyboard routine flag
         Call Get.Modem(Input.Char) ' routine to get character from modem
         If Input.Char Then ' modem input in return variable
            ' append modem input to buffer variable
            Buffer = Buffer + Chr$(Input.Char)
         End If ' end check modem for character
         If Len(Buffer) Then ' check buffer contents
            Char$ = Left$(Buffer, 1) ' get first buffer character
            Buffer = Mid$(Buffer, 2) ' store buffer minus first character
         End If ' end check buffer
         'Call Release.Time(1) ' give up time slice
      Loop ' end loop until character received
      Char = Asc(Char$) ' convert character string to ascii integer
      Select Case Char ' process character ascii integer
         Case 8 ' backspace
            If Len(Inpt) Then ' check length of input
               Inpt = Left$(Inpt, Len(Inpt) - 1) ' strip off last character from input
               Call Back.Space ' routine to send backspace codes
            End If ' end check string length
         Case 13 ' return key
            If Inpt = Nul Then ' no input in string
               If Len(No.Input.Out) Then ' check null input echo string
                  Inpt = No.Input.Out ' set input string to default input string
                  OutY$ = LCase$(No.Input.Out) ' convert null input string
                  Call Scrn(OutY$) ' send string to screen
                  If User.Echo = False Then ' check echo mode
                     Call Put.Modem(OutY$) ' send string to modem
                  End If ' end check echo mode
               End If ' end check null input flag
            End If ' end check null input
            Call Scrn(Chr$(13)) ' send carriage return to screen
            If User.Echo = False Then ' check echo mode
               Call Put.Modem(Chr$(13)) ' send carriage return to modem
            End If ' end check echo mode
            Exit Do ' end routine after return key prssed/return input string
         Case 32 To 127 ' remaining valid input ascii codes
            If Inpt = Nul Then ' check length of stored input
               If Upper.Case Then ' verify force uppercase
                  Char$ = UCase$(Char$) ' change first character to uppercase
               End If ' end verify force
            End If ' end check length
            Valid.Char$ = Char$ ' store character
            If No.Echo = False Then ' check echo flag
               If Hidden Then ' check hide flag
                  Valid.Char$ = Mask$ ' echo mask character
               End If ' end check hide flag
               If User.Echo = False Then ' check echo mode
                  Call Put.Modem(Valid.Char$) ' echo character entered to modem
               End If ' end check echo mode
               If Word.Wrap = False Then ' check word wrap flag
                  Call Scrn(Valid.Char$) ' echo character entered to screen
               End If ' end check word wrap flag
            End If ' end check echo flag
            Inpt = Inpt + Char$ ' append character input to input string
            ' check echo and word wrap
            If No.Echo = False And Word.Wrap Then
               Select Case Len(Inpt) + Line.Limit ' check length of input+line length
                  Case 0 To User.LineLength - 2 ' position to left, no word wrap
                     Call Scrn(Valid.Char$) ' send echo character
                  Case Else ' position to left, word wrap
                     Last.Word = False ' counter of characters at last word to wrap
                     Word = False ' set flag for word wrapped
                     ' loop through last word reverse
                     For Last.Wrap = Len(Inpt) To 1 Step -1
                        If Mid$(Inpt, Last.Wrap, 1) = " " Then ' locate space of word
                           For Word.Break = 1 To Last.Word ' loop through length of word
                              Call Back.Space ' routine to backspace
                           Next ' end loop through word
                           ' store wrapped word for next i/o
                           Buffered.Input$ = Mid$(Inpt, Last.Wrap + 1)
                           ' truncate word from input string
                           Inpt = Left$(Inpt, Last.Wrap)
                           Call Scrn(Chr$(13)) ' send return
                           Word = True ' word has wrapped flag
                           Exit For ' end loop for a space
                        End If ' end check space location
                        Last.Word = Last.Word + 1 ' increment last word wrapped counter
                     Next
                     If Word = False Then ' check flag for word wrapped
                        Call Scrn(Chr$(13)) ' send return to screen
                     End If ' end check word wrapped flag
                     Call Put.Modem(Chr$(13)) ' send return to modem
                     Exit Do ' end input loop
               End Select ' end check length of input+line length
            Else ' word wrap flag not set
               If Line.Length > False Then ' check line length variable
                  If Len(Inpt) >= Line.Length Then ' check input string length
                     Call Scrn(Chr$(13)) ' send return to screen
                     Call Put.Modem(Chr$(13)) ' send return to modem
                     Exit Do ' exit input loop
                  End If ' end check input string length
               Else ' check line length
                  ' check right column
                  If Len(Inpt) + Line.Limit > User.LineLength - 2 Then
                     Call Scrn(Chr$(13)) ' send return to screen
                     Call Put.Modem(Chr$(13)) ' send return to modem
                     Exit Do ' exit input loop
                  End If ' end check line limit
               End If ' end check line length
            End If ' end check word wrap
      End Select ' end select character ascii code
   Loop ' end input loop
   Carriage.Return = False ' reset output cr/lf flag
   Outpt = Nul ' reset input prompt
   No.Echo = False ' reset echoing mask character flag
   No.Input = False ' reset null input flag
   Line.Length = False ' reset length of input line
   If Inpt = Nul Then ' check input string to null
      No.Input = True ' set null input flag
   End If ' end check input string length
   No.Input.Out = Nul ' reset echo string for null input flag
   If User.LineFeeds = False Then ' check linefeed mode
      If User.Echo = False Then ' check echo mode
         Call Put.Modem(Chr$(10)) ' send linefeed
      End If ' end check echo mode
   End If ' end check linefeed mode
   Word.Char$ = UCase$(Left$(Inpt, 1)) ' get uppercase of first character of input
   No = (Word.Char$ = "N") ' set no flag (user entered N as first character)
   Yes = (Word.Char$ = "Y") ' set yes flag (user entered Y as first character)
   Quit = (Word.Char$ = "Q") ' set quit flag (user entered Q as first character)
   Continue = (Word.Char$ = "C") ' set continuous flag
   Exit Sub ' exit routine to get input string in Inpt

   Rem * subroutine to verify time limits.

   IO.Timing:
   ' routine to calculate input timing
   Call Second.Timer(Time.Expired, Time.Out!, 180!)
   If Time.Expired Then ' timing has expired
      Call Hang.Up(1) ' routine to hang up modem with message
   End If ' end check timing
   ' routine to calculate time on
   Call Second.Timer(Time.Expired, Timeon, Time.Left)
   If Time.Expired Then ' time limit has expired
      Call Hang.Up(2) ' routine to hang up modem with message
   End If ' end check time limit
   If Two.Minutes.Left = False Then ' check two minutes message flag
      ' calculate two minutes time
      Call Second.Timer(Time.Expired, Timeon, Time.Left - 120!)
      If Time.Expired Then ' time has expired, user has two minutes left
         Two.Minutes.Left = True ' set two minutes message flag
         Outpt = Nul ' reset output string
         Call IO.O ' send null return
         Call Put.Modem(Chr$(7)) ' send beep to modem
         Outpt = "Two Minutes Left!" ' send two minutes remaining message
         Call IO.O ' to modem
      End If ' end check two minutes
   End If ' end check two minutes flag
   Return ' end timing subroutine
End Sub ' end routine to get input string in Inpt

Rem * Routine to send a string to the modem.
Rem * input variables:
Rem *   Allow.Break - set if control-k checking allowed.
Rem *   Local.Mode - flag if the console is operating program.
Rem *   Output$ - string to send.
Rem * output variables:
Rem *   Break - flag set if control-k entered.
Rem *   Buffer - any input from modem during output.
Rem * processing variables:
Rem *   Char - ascii code of character to send.
Rem *   Count - loop variable of output string sent.
Rem *   Input.Char - local work variable.

Sub Put.Modem (Output$)
   If Local.Mode = False Then ' check if local mode is activated
      For Count = 1 To Len(Output$) ' loop through output string
         Call Get.Modem(Input.Char) ' routine to get data from modem
         If Input.Char Then ' character received from modem
            If Input.Char = 11 Then ' check if character is control-k
               If Allow.Break Then ' check if break allowed
                  Break = True ' set break flag
                  Exit Sub ' end routine
               End If ' end check break allowed
            Else ' any other character
               Buffer = Buffer + Chr$(Input.Char) ' append to input buffer
            End If ' end check character is control-k
         End If ' end check character received
         ' convert output string character to ascii
         Char = Asc(Mid$(Output$, Count, 1))
         Modem.Access = True
         Modem.Error = False
         Put #ModemFile, Char
         Modem.Access = False
         If Modem.Error Then
            Call Lost.Carrier
         End If
      Next ' end loop through output string
   End If ' end check local mode
End Sub ' end routine to send output to modem

Rem * Routine to get a character from the modem.
Rem * input variables:
Rem *   Local.Mode - flag if console is operating system.
Rem * output variables:
Rem *   Input.Char - character received from modem.

Sub Get.Modem (Input.Char)
   If Local.Mode Then
      Exit Sub
   End If
   Input.Char = 0
   Modem.Access = True
   Modem.Error = False
   If Loc(ModemFile) Then
      Get #ModemFile, Input.Char
   End If
   Modem.Access = False
   If Modem.Error Then
      Call Lost.Carrier
   End If
End Sub

Rem * subroutine to display lost carrier or modem not found message
Rem * and terminate program.

Sub Lost.Carrier
   Call Status.Line(-2) ' routine to clear lower two console status areas
   Color 15, 0 ' reset color
   If Modem.Error = 68 Then ' device unavailable
      Outpt = Chr$(13) + "Dnddoor terminated (no modem found)." + Chr$(13)
   Else
      Outpt = Chr$(13) + "Dnddoor terminated (lost carrier)." + Chr$(13)
   End If
   Call Scrn(Outpt) ' send message
   Outpt = "Logoff: port" + Str$(Port) + " at " + clock$ + ", node " + Node + "." + Chr$(13)
   Call Scrn(Outpt) ' send message
   Color 7, 0 ' reset color
   Sleep 5 ' pause for 5 seconds
   System ' terminate program
End Sub

Rem * routine to calculate elapsed time
Rem * input variables:
Rem *   Start.Time! - equals stored start time from Timer function.
Rem *   Wait.Time! - equals elapsed time to calculate in seconds.
Rem * output variables:
Rem *   Time.Expired - set to true if Wait.Time has elapsed.
Rem * working variables:
Rem *   Time.Elapsed! - contains time difference.

Sub Second.Timer (Time.Expired, Start.Time!, Wait.Time!)
   Time.Expired = False ' set return flag time has not expired
   Time.Elapsed! = Timer - Start.Time! ' calculate time elapsed from start time
   If Time.Elapsed! < 0! Then ' check if midnight has passed
      Time.Elapsed! = Time.Elapsed! + 86400! ' increment calculated time elapsed
   End If ' end check midnight passed
   ' compare calculated elapsed time to wait time
   If Time.Elapsed! >= Wait.Time! Then
      Time.Expired = True ' set return flag time has expired
   End If ' end compare times
End Sub ' end routine to calculate time

Rem * Routine to get and process keyboard entry.
Rem * output variables:
Rem *   Keyboard.Break - true to exit from IO.I when function key pressed.
Rem * work variables:
Rem *   Keyboard.Char$ - contains key pressed.

Sub Keyboard (Keyboard.Break)
   Keyboard.Break = False ' reset extended/function key input flag
   Keyboard.Char$ = InKey$ ' reset next keyboard character
   Select Case Len(Keyboard.Char$) ' process keyboard character type
      Case 0 ' nothing entered
         Exit Sub ' end routine
      Case 1 ' single keystroke
         Select Case Asc(Keyboard.Char$) ' process single key
            Case 8, 13, 32 To 127 ' valid keys entered during i/o
               Buffer = Buffer + Keyboard.Char$ ' append keystroke to buffer
            Case 11 ' control-k
               If Allow.Break Then ' check break allowed
                  Break = True ' set break flag
               End If ' end check break allowed
            Case 27 ' escape key
               If Chat Then ' check in chat flag
                  Chat = False ' set chat exit flag
                  Keyboard.Break = True ' set function/extended key flag
               Else ' escape key at console
                  If Logged.In Then ' check player in game
                     Call Status.Line(True) ' toggles status line area
                  End If ' end check player logged in game
               End If ' end check flag
         End Select ' end sigle keystroke
      Case 2 ' function/extended key
         Select Case Asc(Right$(Keyboard.Char$, 1)) ' process extended key code
            Case 79 ' end key/terminate key
               Call Hang.Up(4) ' routine to hang up modem with message
               Keyboard.Break = True ' set function/extended key flag
            Case 71 ' home key/chat key
               If Chat = False Then ' check if already in chat
                  If Local.Mode = False Then ' check local mode not activated
                     If Logged.In Then ' check user logged in flag
                        Chat = True ' set in chat flag
                        Call Enter.Chat ' routine to chat with user
                        Chat = False ' reset in chat flag
                        Keyboard.Break = True ' set function/extended key flag
                     End If ' end check user logged in
                  End If ' end check local mode
               End If ' end check chat flag
            Case 59 To 68 ' function key entered
               If Logged.In Then ' check user logged in flag
                  FunctionBuffer = Right$(Keyboard.Char$, 1) ' store function key in buffer
                  Keyboard.Break = True ' set function/extended key flag
               End If ' end check user logged in
         End Select ' end process extended key code
   End Select ' end process keyboard character type
End Sub ' end routine to get/process keyboard entry

Rem * Routine to send carriage return/linefeed (cr/lf) to screen/modem.

Sub Line.Return
   Call Scrn(Chr$(13)) ' send return character to screen
   Call Put.Modem(Chr$(13)) ' send cr to modem
   If User.LineFeeds = False Then ' check linefeed mode
      Call Put.Modem(Chr$(10)) ' send lf to modem
   End If ' end check linefeed mode
End Sub ' end routine to send cr/lf

Rem * Routine to backspace over one previous character.

Sub Back.Space
   ' Store current screen location of cursor column minus one
   Cursor.Column = Pos(0) - 1
   Locate CsrLin, Cursor.Column, 0 ' locate back one space, cursor off
   Print " "; ' erase character with space
   Locate CsrLin, Cursor.Column, 1 ' locate back one space, cursor on
   If User.Echo = False Then ' check echo mode
      Call Put.Modem(Chr$(8) + " " + Chr$(8)) ' send backspace to modem
   End If ' end check echo mode
End Sub ' end routine to backspace

Rem * Routine to change color. ANSI to modem, COLOR statement to screen.

Sub Out.ANSI
   Ansi.ColorCode = 37 ' default to ANSI white
   If Graphics.Off = False Then ' check graphics color change flag
      Color.Code = Color.Code + 1 ' increment current ANSI color variable
      If Color.Code < 31 Or Color.Code > 36 Then ' check ANSI color bounds
         Color.Code = 31 ' reset to base ANSI color
      End If ' end check color bounds
      Ansi.ColorCode = Color.Code ' store new color code
   End If ' end check graphics change flag
   Call Modem.ANSI(Ansi.ColorCode) ' send ANSI color code number
End Sub ' end routine to change color

Rem * Routine to send ANSI color change code to modem/screen.
Rem * input variables:
Rem *   Ansi.ColorCode - contains the Ansi color code number.
Rem * work variables:
Rem *   Ansi.Output$ - stores the Ansi string to send.

Sub Modem.ANSI (Ansi.ColorCode)
   Call Convert.Color(Ansi.ColorCode) ' change screen color
   If Color.Graphics Then ' check remote ANSI graphics flag
      ' construct ANSI code
      Ansi.Output$ = Chr$(27) + "[0;1;" + Mid$(Str$(Ansi.ColorCode), 2) + "m"
      Call Put.Modem(Ansi.Output$) ' send code to modem
   End If ' end check remote ANSI flag
End Sub ' end routine to send ANSI code

Rem * Routine to reset ANSI color code to default.
Rem * work variables:
Rem *   Ansi.Output$ - stores the default Ansi color string.

Sub Reset.ANSI
   If Color.Graphics Then ' check remote ANSI graphics flag
      Ansi.Output$ = Chr$(27) + "[0;37m" ' construct ANSI black on white code
      Call Put.Modem(Ansi.Output$) ' send code to modem
   End If ' end check remote ANSI flag
   Color 7, 0 ' reset local screen to black on white
   Print Nul; ' display a zero length string to change color
End Sub ' end routine to reset ANSI color code

Rem * Routine to change local screen color to ANSI color code number.
Rem * input variables:
Rem *   Ansi.ColorCode - the Ansi color code number.
Rem * work variables:
Rem *   Scrn.ColorCode - stores the CGA screen color number.

Sub Convert.Color (Ansi.ColorCode)
   Select Case Ansi.ColorCode ' choose ANSI color code number
      Case 31 ' ANSI red
         Scrn.ColorCode = 12 ' COLOR statement red
      Case 32 ' ANSI green
         Scrn.ColorCode = 10 ' COLOR statement green
      Case 33 ' ANSI yellow
         Scrn.ColorCode = 14 ' COLOR statement yellow
      Case 34 ' ANSI blue
         Scrn.ColorCode = 9 ' COLOR statement blue
      Case 35 ' ANSI magenta
         Scrn.ColorCode = 13 ' COLOR statement magenta
      Case 36 ' ANSI cyan
         Scrn.ColorCode = 11 ' COLOR statement cyan
      Case 37 ' ANSI white
         Scrn.ColorCode = 15 ' COLOR statement white
      Case Else ' default ANSI white
         Scrn.ColorCode = 15 ' COLOR statement white
   End Select ' end choose ANSI color code
   Color Scrn.ColorCode, 0 ' change screen color
End Sub ' end routine to change local screen color

Rem * Routine to process stored function key pressed during remote play.
Rem * input variables:
Rem *   FunctionKey.Number - the ascii value of the extended function key.

Sub Function.Key (FunctionKey.Number)
   If Logged.In = False Then ' check user is logged in
      Exit Sub ' exit routine
   End If ' end check user logged in
   If Local.Mode Then ' check console is logged in
      Select Case FunctionKey.Number ' choose function key number
         Case 59 ' F1 - !Edit
            Call IO.O ' empty output
            Call Editor ' editor routine
         Case 60 ' F2 - !Status
            Call IO.O ' empty output
            Call Display.Memory ' routine to display memory stats
         Case 61 ' F3 - !Discard
            Outpt = Action.Prompt + "!DISCARD " ' display action prompt plus command
            Call IO.I ' get item to discard
            Parsed.Command1 = UCase$(Inpt) ' store item name
            Call Discard.Object ' routine to discard item from player inventory
         Case 62 ' F4 - !Reduce
            Outpt = Action.Prompt + "!REDUCE " ' display action prompt plus command
            Call IO.I ' get number
            Parsed.Command1 = Inpt ' store number
            Call Reduce.Monsters ' routine to reduce monsters in room
         Case 63 ' F5 - !Call
            Outpt = Action.Prompt + "!CALL " ' display action prompt plus command
            Call IO.I ' get monster number
            Parsed.Command1 = Inpt ' store monster number
            Call Summon.Monster ' routine to get a monster into room
         Case 64 ' F6 - !Kill
            Outpt = Action.Prompt + "!KILL " ' display action prompt plus command
            Call IO.I ' get monster name
            Parsed.Command1 = UCase$(Inpt) ' store monster name
            Parsed.Command2 = Parsed.Command1 ' store monster name
            Call Kill.Monster ' routine to kill monster name
         Case 65 ' F7 - !Teleport
            Outpt = Action.Prompt + "!TELEPORT " ' display action prompt plus command
            Call IO.I ' get room number
            Parsed.Command1 = Inpt ' store room number
            Parsed.Command2 = Parsed.Command1 ' store room number
            Call Teleport.User ' routine to teleport user to a room number
         Case 66 ' F8 - !Invisibility
            Call IO.O ' empty output
            Call Toggle.Invisible ' routine to toggle invisibility mode
         Case 67 ' F9 - !Get
            Outpt = Action.Prompt + "!GET " ' display action prompt plus command
            Call IO.I ' get item name
            Inpt = UCase$(Inpt) ' store item name
            Stored.Parsed.Command2 = Inpt ' store item name
            Call Drop.Object ' routine to get an item
         Case 68 ' F9 - !Link
            Call IO.O ' empty output
            Call Link.Room ' routine to add/remove room links
      End Select ' end select function key number
      Exit Sub ' exit routine
   End If ' end check console logged in
   Inpt = Nul ' reset input string
   Outpt = Nul ' reset output string
   Call IO.O ' send empty cr/lf
   Select Case FunctionKey.Number ' choose function key number
      Case 59 ' F1 - !Edit
         Outpt = "System Operator is using the editor. Please wait.." ' send message
         Call IO.O ' sysop will edit during game
         Local.Mode = True ' set local mode
         Outpt = "Sysop: Enter room number to edit, or press enter to start !Edit:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!EDIT " ' display action prompt plus command
         Call IO.I ' get room number to edit(if any)
         If Inpt <> Nul Then ' determine edit room number
            Room.Number = Int(Val(Inpt)) ' room number parameter to edit
            If Room.Number > False And Room.Number <= LOF(RoomFile) / Len(RoomRecord) Then ' range
               Call Change.Room(Room.Number) ' edit room from number parameter
            End If ' end compare room number range
         Else ' edit users
            Call Editor ' editor routine
         End If ' end determine edit prompt
         Local.Mode = False ' reset local mode
      Case 60 ' F2 - !Status
         Local.Mode = True ' reset local mode
         Call Display.Memory ' routine to display memory stats
         Local.Mode = False ' restore local mode
      Case 61 ' F3 - !Discard
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter player inventory treasure name to discard:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!DISCARD " ' display action prompt plus command
         Call IO.I ' get item to discard
         Parsed.Command1 = UCase$(Inpt) ' store item name
         Call Discard.Object ' routine to discard item from player inventory
         Local.Mode = False ' restore local mode
      Case 62 ' F4 - !Reduce
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter number of monsters in room to reduce to:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!REDUCE " ' display action prompt plus command
         Call IO.I ' get number
         Parsed.Command1 = Inpt ' store number
         Call Reduce.Monsters ' routine to reduce monsters in room
         Local.Mode = False ' restore local mode
      Case 63 ' F5 - !Call
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter name or number of monster to call:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!CALL " ' display action prompt plus command
         Call IO.I ' get monster number
         Local.Mode = False ' restore local mode
         Parsed.Command1 = Inpt ' store monster number
         Call Summon.Monster ' routine to get a monster into room
      Case 64 ' F6 - !Kill
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter name of monster in room to kill off:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!KILL " ' display action prompt plus command
         Call IO.I ' get monster name
         Local.Mode = False ' restore local mode
         Parsed.Command1 = UCase$(Inpt) ' store monster name
         Parsed.Command2 = Parsed.Command1 ' store monster name
         Call Kill.Monster ' routine to kill monster name
      Case 65 ' F7 - !Teleport
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter number of room to teleport player to:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!TELEPORT " ' display action prompt plus command
         Call IO.I ' get room number
         Local.Mode = False ' restore local mode
         Parsed.Command1 = Inpt ' store room number
         Parsed.Command2 = Parsed.Command1 ' store room number
         Call Teleport.User ' routine to teleport user to a room number
      Case 66 ' F8 - !Invisibility
         Call Toggle.Invisible ' routine to toggle invisibility mode
      Case 67 ' F9 - !Get
         Local.Mode = True ' reset local mode
         Outpt = "Sysop: Enter name of treasure item to get from treasure file:"
         Call IO.O ' write local sysop info message
         Outpt = Action.Prompt + "!GET " ' display action prompt plus command
         Call IO.I ' get item name
         Local.Mode = False ' restore local mode
         Inpt = UCase$(Inpt) ' store item name
         Stored.Parsed.Command2 = Inpt ' store item name
         Call Drop.Object ' routine to get an item
      Case 68 ' F10 - !Link
         Outpt = "System Operator is using the editor. Please wait.." ' send message
         Call IO.O ' sysop will edit during game
         Local.Mode = True ' reset local mode
         Call Link.Room ' routine to add/remove room links
         Local.Mode = False ' restore local mode
   End Select ' end choose function key number
   Outpt = Nul ' reset output string
   Inpt = Nul ' reset input string
End Sub ' end routine to process game function keys

Rem * Routine to hang up modem with a message.
Rem * input variables:
Rem *   HangUp.Type - the hang up message to display.

Sub Hang.Up (HangUp.Type)
   Allow.Break = False ' disable break
   Break = False ' reset control-k flag
   Buffer = Nul ' reset buffer
   FunctionBuffer = Nul ' reset function key buffer
   Graphics.Off = True ' turn off graphics color changing
   Inpt = Nul ' set input string to null
   Outpt = Nul ' set output string to null
   Timeon = Timer ' store current time in time on variable
   Time.Left = 180 ' store 3 minutes remaining in variable
   Call IO.O ' empty output
   Outpt = "Dnddoor terminating at " + clock$ + ", " + Node + "." ' make message
   Call IO.O ' send message
   Select Case HangUp.Type ' choose hang up message
      Case 1 ' type 1
         Outpt = "Connect timeout"
      Case 2 ' type 2
         Outpt = "Time limit exceeded"
      Case 4 ' type 4
         Outpt = "Forced logoff"
      Case 5 ' type 5
         Outpt = "Call limit exceeded"
      Case 6 ' type 6
         Call Restriction.Notice ' routine to display restricted time
      Case 7 ' type 7
         Outpt = "Illegal login attempt"
      Case 8 ' type 8
         Outpt = "Password verification!"
   End Select ' end choose hang up message
   Outpt = "Reason: " + Outpt + "."
   Call IO.O ' display message
   Call Status.Line(-2) ' clear status line areas
   Color 7, 0 ' reset screen color
   Sleep 5 ' pause for 5 seconds
   System ' terminate program
End Sub ' end routine to hang up modem

Rem * Routine to put a character on the screen at current row/column.
Rem * input variables:
Rem *   Scrn.Char$ - the character to display.

Sub Scrn (Scrn.Char$)
   Row = CsrLin - 1 ' calculate current row
   Column = Pos(0) - 1 ' calculate current column
   If Row = Max.Row Then ' compare current row
      ' scroll at max.row (status area)
      If Column = User.LineLength Then ' verify column to scroll
         Print ; ' force screen scroll
         If Scrn.Char$ = Chr$(13) Then ' check cr/lf at corner
            Exit Sub ' return from routine
         End If ' end check corner cr/lf
      End If ' end verify column
   End If ' end compare row
   Print Scrn.Char$; ' display character at row, column
End Sub ' end routine to display a character on screen

Rem * Routine to make users restricted time online output message.

Sub Restriction.Notice
   Outpt = "Your calls are restricted from" ' start of message
   Restrict.Time = UserRecord.FromHour ' store users hour of restricted time
   If Restrict.Time > 12 Then ' compare 24 hour format time to standard time
      Outpt = Outpt + Str$(UserRecord.FromHour - 12) ' decrement to 24 hour format
      Inpt = "pm" ' 24 hour indicator
   Else ' compare to standard time
      Outpt = Outpt + Str$(UserRecord.FromHour) ' store hour format
      Inpt = "am" ' 12 hour indicator
   End If ' end compare time type
   ' format string of restricted time from, plus minutes
   Outpt = Outpt + ":" + Right$(Str$(UserRecord.FromMin + 100), 2) + Inpt + " to"
   Restrict.Time = UserRecord.ToHour ' store users hour of restricted time
   If Restrict.Time > 12 Then ' compare 24 hour format time to standard time
      Outpt = Outpt + Str$(UserRecord.ToHour - 12) ' decrement to 24 huur format
      Inpt = "pm" ' 24 hour indicator
   Else ' compare to standard time
      Outpt = Outpt + Str$(UserRecord.ToHour) ' store hour format
      Inpt = "am" ' 12 hour indicator
   End If ' end compare time type
   ' format string of restricted time to, plus minutes
   Outpt = Outpt + ":" + Right$(Str$(UserRecord.ToMin + 100), 2) + Inpt + "."
End Sub ' end routine to make restricted time message

Rem * Routine to display a text file.
Rem * input variables:
Rem *   File.Output$ - filename of file to display.
Rem * processing variables:
Rem *   Allow.Break - flag to allow control-k.
Rem *   Break - set to true if control-k pressed.
Rem *   Continue - set to true for continuous output
Rem *   Page.Length, Page.Break - paginating variables.

Sub Out.File (File.Output$)
   Graphics.Off = True ' turn off graphics color changing
   File.Page = User.PageLength ' store pagelength
   If File.Page = False Then ' check pagelength
      File.Page = 24 ' reset pagelength
   End If ' end check pagelength
   Allow.Break = True ' turn on allow break flag
   Break = False ' reset control-k flag
   Continue = False ' reset continuous flag
   Page.Length = False ' reset page counter
   Close #TempFile ' close temporary file
   If _FileExists(File.Output$) = 0 Then ' check file exists
      Outpt = "File " + File.Output$ + " not found." ' make output
      Call IO.O ' send output
      Allow.Break = False ' reset allow break flag
      Break = False ' reset control-k flag
      Graphics.Off = False ' reset color change flag
      Exit Sub ' exit routine
   End If ' end check file exists
   Open File.Output$ For Random Shared As #TempFile Len = 1 ' open the file
   If LOF(TempFile) = False Then ' check file length
      Outpt = "File " + File.Output$ + " length zero." ' make output
      Call IO.O ' send output
      Allow.Break = False ' reset allow break flag
      Break = False ' reset control-k flag
      Graphics.Off = False ' reset color change flag
      Exit Sub ' exit routine
   End If ' end check file length
   Close #TempFile ' close temporary file
   Open File.Output$ For Input Shared As #TempFile ' open the file
   Do While Not EOF(TempFile) ' loop through all the text in file
      Line Input #TempFile, Outpt ' get the next text line
      Call IO.O ' display the text line
      If Break Then ' check control-k pressed
         Exit Do ' exit text input loop
      End If ' end check control-k
      Page.Length = Page.Length + 1 ' increment line displayed counter
      If Page.Length = File.Page Then ' compare page counter
         Page.Length = False ' reset page counter
         If Continue = False Then ' check continuous flag
            Call More.Prompt ' routine to prompt for more
            If No Then ' check more prompt returned no flag
               Exit Do ' exit text input loop
            End If ' end check more prompt no flag
         End If ' end check page counter
      End If ' end check continuous flag
   Loop ' end text file input loop
   Allow.Break = False ' reset allow break flag
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
   Graphics.Off = False ' reset graphics color changer
   If Page.Length Then ' check page counter
      Call More.Prompt ' routine to prompt for more
   End If ' end check page counter
   Allow.Break = False ' reset allow break flag
   Break = False ' reset control-k flag
   Graphics.Off = False ' reset color change flag
End Sub ' end routine to display a text file

Rem * Routine to display the more prompt.
Rem * work variables:
Rem *   Allow.Type - store break flag type
Rem *   Graphics.Type - stores graphics cycling variable.

Sub More.Prompt
   Allow.Type = Allow.Break ' store break type
   Allow.Break = False ' disable break during prompt
   Graphics.Type = Graphics.Off ' store graphics changing flag
   Graphics.Off = False ' turn off graphics changing
   Outpt = More$ ' get the more prompt
   Line.Length = 1 ' allow one key input
   No.Echo = True ' do not echo any key
   Call IO.I ' input routine
   Allow.Break = Allow.Type ' restore break type
   Graphics.Off = Graphics.Type ' restore graphics changing flag
End Sub

Rem * Routine to restrict a room direction to a players level.
Rem * input variable:
Rem *   Room.Direction - direction to restrict.
Rem * output variable:
Rem *   Direction.Restricted - set true to restrict player.
Rem * work variables:
Rem *   Action.Number - contains the room action number.

Sub Restrict (Room.Direction, Direction.Restricted)
   Direction.Restricted = False ' reset return variable
   Action.Number = RoomRecord.Action ' store room action number
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then ' check file bounds
      Call Read.Record(ActionFile, Action.Number) ' read action record
      Select Case ActionRecord.Level ' determine player level
         Case Is > False ' player at least room level
            If UserRecord.Level < ActionRecord.Level Then ' compare levels
               ' compare room direction
               If ActionRecord.Restrictions And 2 ^ Room.Direction Then
                  Direction.Restricted = True ' direction restricted flag
               End If ' end compare room direction
            End If ' end compare levels
         Case Is < False ' player at most room level
            If UserRecord.Level > Abs(ActionRecord.Level) Then ' compare levels
               ' compare room direction
               If ActionRecord.Restrictions And 2 ^ Room.Direction Then
                  Direction.Restricted = True ' direction restricted flag
               End If ' end compare room direction
            End If ' end compare levels
      End Select ' end choose room/player level
   End If ' end check file bounds
End Sub ' end routine to restrict room entrance

Rem * Routine to restrict a room direction to entry Type
Rem * output variable:
Rem *   Direction.Restricted - set true to restrict player.
Rem * work variables:
Rem *   Action.Number - contains the room action number.

Sub Restrict.Room.Type (Direction.Restricted)
   Call Read.Room.Record(Next.Room) ' read next room number record
   Direction.Restricted = False ' reset return variable
   Action.Number = RoomRecord.Action ' store room action number
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then ' check file bounds
      Call Read.Record(ActionFile, Action.Number) ' read action record
      If ActionRecord.Attribute2 = Air Then ' verify room is air
         If Entry.Command <> Fly Then ' verify player fly
            Direction.Restricted = True ' set restricted flag
         End If ' end verify player fly command
      End If ' end verify room air attribute
      If ActionRecord.Attribute2 <> Air Then ' verify room is air
         If Entry.Command = Fly Then ' verify player fly
            Direction.Restricted = True ' set restricted flag
         End If ' end verify player fly command
      End If ' end verify room air attribute
      If ActionRecord.Attribute2 = Water Then ' verify room water
         If Entry.Command <> Swim Then ' verify player swim
            Direction.Restricted = True ' set restricted flag
         End If ' end verify player swim command
      End If ' end verify room water attribute
      If ActionRecord.Attribute2 <> Water Then ' verify room water
         If Entry.Command = Swim Then ' verify player swim
            Direction.Restricted = True ' set restricted flag
         End If ' end verify player swim command
      End If ' end verify room water attribute
   Else ' check file bounds
      If Entry.Command = Fly Or Entry.Command = Swim Then ' check room entry
         Direction.Restricted = True ' set restricted flag
      End If ' end check room entry
   End If ' end check file bounds
   Call Read.Room.Record(Room) ' read current room record
End Sub ' end routine to restrict room entrance

Rem * Routine to determine validity of a string.
Rem * input variables:
Rem *   Validate$ - string to verify.
Rem *   Length - stores length string must be (an even number).
Rem * output variables:
Rem *   Validate$ - set to null if invalid.
Rem * work variables:
Rem *   String.Length - loop counter of string to validate.
Rem *   Char - character in string to check.

Sub Valid (Validate$, Length)
   Validate$ = Left$(Validate$, Length) ' truncate string to length
   Validate$ = Validate$ + Space$(Length - Len(Validate$)) ' extend string length
   For String.Length = 1 To Len(Validate$) ' loop through all characters in string
      ' get ascii value of string character
      Char = Asc(Mid$(Validate$, String.Length, 1))
      If Char < 32 Or Char > 127 Then ' verify bounds of character
         Validate$ = Nul ' return null if not valid
         Exit Sub ' end routine
      End If ' end verify ascii code
   Next ' end loop
End Sub ' end routine to determine string validity

Rem * Routine to encrypt a string.
Rem * input variables:
Rem *   Encrypted$ - the string to encrypt.
Rem *   Add.Checksum - set to false to add byte checksum.
Rem * output variables:
Rem *   Encrypted$ - the encrypted string.
Rem * work variables:
Rem *   Encrypt.Work$ - variable containing encrypted string.
Rem *   Encrypt.Count - loop variable.
Rem *   Encrypted.Sum - calculated byte encryption variable.
Rem *   Encrypt.Work1, Encrypt.Work2, Encrypt.Work3.

Sub Encrypt (Encrypted$, Add.Checksum)
   Encrypt.Work$ = Nul ' reset encrypted work variable
   ' loop through string to encrypt in byte pairs
   For Encrypt.Count = 1 To Len(Encrypted$) Step 2
      ' get first byte substring of pair
      Encrypt.Work1 = Asc(Mid$(Encrypted$, Encrypt.Count, 1))
      ' get second byte substring of pair
      Encrypt.Work2 = Asc(Mid$(Encrypted$, Encrypt.Count + 1, 1))
      If Add.Checksum Then ' check checksum ordering
         Encrypted.Sum = 20000 ' checksum bit off
      Else ' checksum ordering on
         Encrypt.Work3 = Encrypt.Work1 + Encrypt.Work2 ' compute sum
         ' compute even sum of bytes
         If Int(Encrypt.Work3 / 2) = Encrypt.Work3 / 2 Then
            Encrypted.Sum = 10000 ' checksum bit for even parity
         Else ' checksum ordering
            Encrypted.Sum = False ' checksum bit for odd parity
         End If ' end check checksum ordering
      End If ' end verify checksum used
      ' convert checksum and bytes to integer
      Encrypted.Sum = Encrypted.Sum + (Encrypt.Work1 - 32) * 100 + (Encrypt.Work2 - 32)
      ' convert integer to string
      Encrypt.Work$ = Encrypt.Work$ + MKI$(Encrypted.Sum)
   Next ' end loop through string to encrypt
   Encrypted$ = Encrypt.Work$ ' set return variable to encrypted string
End Sub ' end routine to encrypt a string

Rem * Routine to decrypt a string.
Rem * input variables:
Rem *   Decrypted$ - the string to decrypt.
Rem * output variables:
Rem *   Decrypted$ - the decrypted string, or null for checksum error.
Rem * work variables:
Rem *   Decrypt.Work$ - variable containing encrypted string.
Rem *   Decrypt.Count - loop variable.
Rem *   Decrypted.Sum - calculated byte encryption variable.
Rem *   Decrypt.Work1, Decrypt.Work2, Decrypt.Work3.
Rem *   Decrypted.Checksum - contains the bit parity.

Sub Decrypt (Decrypted$)
   Decrypt.Work$ = Nul ' reset decrypted work variable
   ' loop through string to decrypt in byte pairs
   For Decrypt.Count = 1 To Len(Decrypted$) Step 2
      ' convert the two byte string to integer
      Decrypted.Sum = CVI(Mid$(Decrypted$, Decrypt.Count, 2))
      ' divide two byte integer to one byte integer
      Decrypted.Checksum = Decrypted.Sum \ 100
      ' get low byte from pair
      Decrypt.Work1 = Decrypted.Sum - Decrypted.Checksum * 100
      ' adjust byte ascii value
      Decrypt.Work1 = Decrypt.Work1 + 32
      ' reset integer of string to high byte
      Decrypted.Sum = Decrypted.Checksum
      ' divide byte integer out leaving checksum bit
      Decrypted.Checksum = Decrypted.Sum \ 100
      ' get high byte from pair
      Decrypt.Work2 = Decrypted.Sum - Decrypted.Checksum * 100
      Decrypt.Work2 = Decrypt.Work2 + 32 ' adjust byte ascii value
      Decrypt.Work3 = Decrypt.Work1 + Decrypt.Work2 ' compute sum
      Select Case Decrypted.Checksum ' select checksum parity
         Case 0 ' verify the checksum bit for odd parity
            If Int(Decrypt.Work3 / 2) = Decrypt.Work3 / 2 Then ' check not odd parity
               Decrypted$ = Nul ' return null for checksum error
               Exit Sub ' end routine
            End If ' end check not odd parity
         Case 1 ' verify the checksum bit for even parity
            If Int(Decrypt.Work3 / 2) <> Decrypt.Work3 / 2 Then ' check not even parity
               Decrypted$ = Nul ' return null for checksum error
               Exit Sub ' end routine
            End If ' end check not even parity
      End Select ' end verify checksum bit
      ' convert integer bytes to string
      Decrypt.Work$ = Decrypt.Work$ + Chr$(Decrypt.Work2) + Chr$(Decrypt.Work1)
   Next ' end loop through string to decrypt
   Decrypted$ = Decrypt.Work$ ' set return variable to decryptes string
End Sub ' end routine to decrypt a string

Rem * Routine to store a file record
Rem * input variables:
Rem *   File.Number - number of the file to write record.
Rem *   Record.Number - the number of the record to write.

Sub Share.Record (File.Number, Record.Number)
   Share.Count = 0
   Lock File.Number, Record.Number ' lock record number
   Select Case File.Number ' select file number to write a record to
      Case UserFile ' type 1
         Put UserFile, Record.Number, UserRecord ' write user record
      Case NonPlayerFile ' type 2
         Put NonPlayerFile, Record.Number, MonsterRecord ' write nonplayers record
      Case ObjectFile ' type 4
         Put ObjectFile, Record.Number, ObjectRecord ' write object record
      Case MonsterFile ' type 5
         Put MonsterFile, Record.Number, MonsterRecord ' write monsters record
      Case TreasureFile ' type 6
         Put TreasureFile, Record.Number, TreasureRecord ' write treasure record
      Case MonTalkFile ' type 7
         Put MonTalkFile, Record.Number, MonsterTalkRecord ' write monster talk record
      Case MonClassFile ' type 8
         Put MonClassFile, Record.Number, MonclassRecord ' write monster class record
      Case SpellFile ' type 9
         Put SpellFile, Record.Number, SpellRecord ' write spell record
      Case TableFile ' type 10
         Put TableFile, Record.Number, TableRecord ' write message table record
      Case ActionFile ' type 12
         Put ActionFile, Record.Number, ActionRecord ' write action record
      Case TempFile ' type 13
         Put TempFile, Record.Number, TableRecord ' write temp table record
      Case HelpFile ' type 15
         Put HelpFile, Record.Number, HelpRecord ' write helpfile record
      Case UserFile2 ' type 16
         Put UserFile, Record.Number, UserRecord2 ' write user record
   End Select ' end select file number
   Share.Count = 0
   Unlock File.Number, Record.Number ' lock record number
End Sub ' end routine to write file record using share

Rem * Routine to read a file record
Rem * input variables:
Rem *   File.Number - number of the file to read record.
Rem *   Record.Number - the number of the record to read.

Sub Read.Record (File.Number, Record.Number)
   Select Case File.Number ' select file number to read a record to
      Case UserFile ' type 1
         Get UserFile, Record.Number, UserRecord ' read user record
      Case NonPlayerFile ' type 2
         Get NonPlayerFile, Record.Number, MonsterRecord ' read nonplayers record
      Case ObjectFile ' type 4
         Get ObjectFile, Record.Number, ObjectRecord ' read object record
      Case MonsterFile ' type 5
         Get MonsterFile, Record.Number, MonsterRecord ' read monsters record
      Case TreasureFile ' type 6
         Get TreasureFile, Record.Number, TreasureRecord ' read treasure record
      Case MonTalkFile ' type 7
         Get MonTalkFile, Record.Number, MonsterTalkRecord ' read monster talk record
      Case MonClassFile ' type 8
         Get MonClassFile, Record.Number, MonclassRecord ' read monster class record
      Case SpellFile ' type 9
         Get SpellFile, Record.Number, SpellRecord ' read spell record
      Case TableFile ' type 10
         Get TableFile, Record.Number, TableRecord ' read message table record
      Case ActionFile ' type 12
         Get ActionFile, Record.Number, ActionRecord ' read action record
      Case TempFile ' type 13
         Get TempFile, Record.Number, TableRecord ' read temp table record
      Case HelpFile ' type 15
         Get HelpFile, Record.Number, HelpRecord ' read help record
      Case UserFile2 ' type 16
         Get UserFile, Record.Number, UserRecord2 ' read user record
   End Select ' end select file number
End Sub ' end routine to read file record using share

Rem * Routine to store a message file record
Rem * input variables:
Rem *   File.Number - number of the file to write record.
Rem *   Record.Number! - the number of the record to write.

Sub Share.Message (File.Number, Record.Number!)
   Share.Count = 0
   Lock File.Number, Record.Number! ' lock record number
   Put File.Number, Record.Number!, MessageRecord ' write message record
   Share.Count = 0
   Unlock File.Number, Record.Number! ' lock record number
End Sub ' end routine to write file record using share

Rem * Routine to read a message file record
Rem * input variables:
Rem *   File.Number - number of the file to read record.
Rem *   Record.Number! - the number of the record to read.

Sub Read.Message.Record (File.Number, Record.Number!)
   Get File.Number, Record.Number!, MessageRecord ' write message record
End Sub ' end routine to write file record using share

Rem * Routine to store a room file record
Rem * input variables:
Rem *   Record.Number! - the number of the record to write.

Sub Share.Room.Record (Record.Number!)
   Share.Count = 0
   Lock RoomFile, Record.Number! ' lock record number
   Put RoomFile, Record.Number!, RoomRecord ' write room record
   Share.Count = 0
   Unlock RoomFile, Record.Number! ' lock record number
End Sub ' end routine to write file record using share

Rem * Routine to read a room file record
Rem * input variables:
Rem *   Record.Number! - the number of the record to read.

Sub Read.Room.Record (Record.Number!)
   Get RoomFile, Record.Number!, RoomRecord ' write message record
End Sub ' end routine to write file record using share

Rem * Filename: dnds1.bas Version: v5.0a r2.0a
Rem * This subprogram contains most display and find routines.

Rem * This routines displays information of the action file.
Rem * input variables:
Rem *   Number - the room file record.

Sub Display.Action (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Action number" + Str$(Number) + ":" ' action number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   If ActionRecord.MonsterTrigger Then ' check action trigger
      ' get monster trigger record
      Call Read.Record(MonsterFile, ActionRecord.MonsterTrigger)
      Outpt = MonsterRecord.MonsterName ' store monster name
      Outpt = RTrim$(Outpt) ' trim name
      Inpt = Str$(ActionRecord.MonsterTrigger) + ", " + Outpt + "." ' make message
   Else ' check trigger
      Inpt = " <none>" ' make default message
   End If ' end check trigger
   Outpt = "[A]Monster trigger in effect for monster:" + Inpt ' make message
   Call IO.O ' send output message
   If ActionRecord.SpellTrigger Then ' check action trigger
      Call Read.Record(SpellFile, ActionRecord.SpellTrigger) ' get spell record
      Outpt = SpellRecord.SpellName ' store spell name
      Outpt = RTrim$(Outpt) ' trim name
      Inpt = Str$(ActionRecord.SpellTrigger) + ", " + Outpt + "." ' make message
   Else ' check trigger
      Inpt = " <none>" ' make default message
   End If ' end check trigger
   Outpt = "[B]Spell trigger in effect for spell:" + Inpt
   Call IO.O ' send output message
   If ActionRecord.MonsterTalk Then ' check action trigger
      Call Read.Record(MonsterFile, ActionRecord.MonsterTalk) ' get monster record
      Outpt = MonsterRecord.MonsterName ' store monster name
      Outpt = RTrim$(Outpt) ' trim name
      Inpt = Str$(ActionRecord.MonsterTalk) + ", " + Outpt + "." ' make message
   Else ' check trigger
      Inpt = " <none>" ' make default message
   End If ' end check trigger
   Outpt = "[C]Monster talk trigger in effect for monster:" + Inpt ' make message
   Call IO.O ' send output message
   If ActionRecord.Level > False Then ' check action trigger
      Inpt = Str$(ActionRecord.Level) ' make message
   Else ' check trigger
      Inpt = " <none>" ' make default message
   End If ' end check trigger
   Outpt = "[D]High level room entry:" + Inpt ' message
   If ActionRecord.Level < False Then ' check action trigger
      Inpt = Str$(Abs(ActionRecord.Level)) ' make message
   Else ' check trigger
      Inpt = " <none>" ' make default message
   End If ' end check trigger
   Outpt = Outpt + ", Low level room entry:" + Inpt ' message
   Call IO.O ' send output message
   Outpt = "Restricted room directions:" ' make message
   Call IO.O ' send output message
   If ActionRecord.Restrictions > False Then ' check action trigger
      Outpt = Nul ' reset message
      For Direction.Number = 1 To 12 ' loop through directions
         ' check restriction trigger
         If ActionRecord.Restrictions And 2 ^ Direction.Number Then
            Outpts = Direction(Direction.Number) ' make direction name
            Outpt = Outpt + Outpts + ", " ' append direction
         End If ' end check trigger
      Next ' end loop
      If ActionRecord.Restrictions And 2 ^ 13 Then ' check restriction trigger
         Outpt = Outpt + "enter portal, " ' append message
      End If ' end check trigger
      If Len(Outpt) Then ' check string length
         Outpt = Left$(Outpt, Len(Outpt) - 2) ' truncate comma
      End If ' end check string length
   Else ' check trigger
      Outpt = "<none>" ' default message
   End If ' end check trigger
   Call IO.O ' send output message
   Outpt = "[E]Action hits for:"
   Select Case ActionRecord.HitPoints ' check action trigger
      Case Is > False
         Inpt = Str$(ActionRecord.HitPoints) + " fatigue" ' make message
      Case Is < False
         Inpt = Str$(Abs(ActionRecord.HitPoints)) + " vitality" ' make message
      Case Else ' check trigger
         Inpt = " <none>" ' default message
   End Select ' end check trigger
   Outpt = Outpt + Inpt
   Call IO.O ' send output message
   If ActionRecord.EncounterRate Then ' check action trigger
      Inpt = Str$(ActionRecord.EncounterRate) ' make message
   Else ' check trigger
      Inpt = " <none>" ' default message
   End If ' end check trigger
   Outpt = "[F]Encounter rate:" + Inpt
   If ActionRecord.HealthRate Then ' check action trigger
      Inpt = Str$(ActionRecord.HealthRate) ' make message
   Else ' check trigger
      Inpt = " <none>" ' default message
   End If ' end check trigger
   Outpt = Outpt + ", Health rate:" + Inpt
   Call IO.O ' send output message
   Outpt = "[G]Inventory: "
   Select Case ActionRecord.Inventory ' make selection of inventory actions
      Case 1 ' weapon action
         Inpt = "breaks weapons" ' make message
      Case 2 ' shield action
         Inpt = "smashes shields" ' make message
      Case 3 ' armor action
         Inpt = "wrecks armor" ' make message
      Case 4 ' magic item action
         Inpt = "drains magic items" ' make message
      Case Else ' check trigger
         Inpt = "<none>"
   End Select ' end selection of inventory actions
   Outpt = Outpt + Inpt
   Call IO.O
   If ActionRecord.Fumble Then ' check action trigger
      Outpt = "[H]Action fumbles." ' make message
   Else ' check trigger
      Outpt = "[H]Action does not fumble." ' make message
   End If ' end check trigger
   Call IO.O ' send output message
   If ActionRecord.Teleport Then ' check action trigger
      Outpt = "[I]Action teleports to room" + Str$(ActionRecord.Teleport) + "."
   Else ' check trigger
      Outpt = "[I]Action does not teleport." ' make message
   End If ' end check trigger
   Call IO.O ' send output message
   If ActionRecord.RustRate Then ' check action trigger
      Inpt = Str$(ActionRecord.RustRate) ' make message
   Else ' check trigger
      Inpt = " <none>" ' default message
   End If ' end check trigger
   Outpt = "[J]Action has weapon rusting rate of" + Inpt + " rounds." ' message
   Call IO.O ' send output message
   If ActionRecord.StealRate Then ' check action trigger
      Inpt = Str$(ActionRecord.StealRate) ' make message
   Else ' check trigger
      Inpt = " <none>" ' default message
   End If ' end check trigger
   Outpt = "[K]Action has monster stealing rate of" + Inpt + " rounds." ' message
   Call IO.O ' send output message
   Outpt = "[L]Action attributes: " ' make edit display message
   If ActionRecord.Attribute1 = LitRoom Then ' check attribute
      Outpt = Outpt + "Lit" ' append attribute message
   Else ' check attribute
      If ActionRecord.Attribute1 = UnlitRoom Then ' check attribute
         Outpt = Outpt + "Unlit" ' append attribute message
      End If ' end check attribute
   End If ' end check attribute
   Outpt = Outpt + ", " ' append comma
   Select Case ActionRecord.Attribute2 ' select attribute
      Case Land ' check attribute
         Outpt = Outpt + "Land" ' append attribute message
      Case Air ' check attribute
         Outpt = Outpt + "Air" ' append attribute message
      Case Water ' check attribute
         Outpt = Outpt + "Water" ' append attribute message
   End Select ' end select attribute
   Call IO.O ' send attribute message
   Outpt = "[X]Clear action" ' make alternate display message
   Call IO.O ' send alternate display message
End Sub

Rem * This routines displays information on the monster file.
Rem * input variables:
Rem *   Number - the monster file record.

Sub Display.Monster (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Monster number" + Str$(Number) + ":" ' monster number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "[A]Monster name: " + RTrim$(MonsterRecord.MonsterName) ' message
   Call IO.O ' send output message
   Outpt = "[B]Plural of monster name: " + MonsterRecord.PluralName ' message
   Call IO.O ' send output message
   Outpt = "[C]Monster level:" + Str$(MonsterRecord.Level) ' message
   Call IO.O ' send output message
   Outpt = "[D]Magical monster: " ' message
   If MonsterRecord.Magic Then ' verify monster
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[E]Hit points:" + Str$(MonsterRecord.Hits) ' message
   Call IO.O ' send output message
   Outpt = "[F]Experience points:" + Str$(MonsterRecord.Experience) ' message
   Call IO.O ' send output message
   Outpt = "[G]Gold points:" + Str$(MonsterRecord.Gold) ' message
   Call IO.O ' send output message
   Outpt = "[H]Number appearing:" + Str$(MonsterRecord.NumberAppearing) ' message
   Call IO.O ' send output message
   Outpt = "[I]Poisonous monster: " ' message
   If MonsterRecord.Poison Then ' verify monster
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.PoisonPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[J]Level draining monster: " ' message
   If MonsterRecord.LevelDrain Then ' verify monster
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.DrainPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[K]Monster blocks exits: " ' message
   If MonsterRecord.Block Then ' verify monster
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.BlockPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[L]Monster prevents treasure take: " ' message
   If MonsterRecord.Prevent Then ' verify monster
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.PreventPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[M]Monster follows player: " ' message
   If MonsterRecord.Follow Then ' verify monster
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.FollowPercent), 2) + " percent)(" + Mid$(Str$(MonsterRecord.Teleport), 2) + "% teleport)"
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[N]Monster casts spells: " ' message
   Spell.Number = MonsterRecord.Spell ' store spell number
   If Spell.Number = False Then ' verify monster
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      ' file bounds
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Outpt = Outpt + "Yes, " + RTrim$(SpellRecord.SpellName) + "(" + Mid$(Str$(MonsterRecord.SpellPercent), 2) + " percent)" ' message
      End If ' end check file bounds
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[O]Monster jails attacker: " ' message
   If MonsterRecord.Jail Then ' verify monster
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[P]Encounter rate:"
   If MonsterRecord.Rate = False Then ' verify rate
      Outpt = Outpt + " No"
   Else ' verify rate
      Outpt = Outpt + Str$(MonsterRecord.Rate) + " (" + Mid$(Str$(MonsterRecord.RatePercent), 2) + " percent)" ' message
   End If ' end verify rate
   Call IO.O ' send output message
   Outpt = "[R]Permanent monster: " ' message
   If MonsterRecord.Permanent Then ' verify monster
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[S]Monster uses psionics: " ' message
   If MonsterRecord.Psionic = False Then ' verify monster
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Spell.Number = MonsterRecord.PsionicSpell ' store psionic spell number
      ' file bounds
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Outpt = Outpt + "Yes, " + SpellRecord.SpellName ' message
      End If ' end check file bounds
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[T]Inventory:" ' message
   Number.Items = False ' reset counter
   For Array.Number = 1 To 5 ' loop through monster treasure
      ' get monster treasure number
      Treasure.Number = GetMonsterRecordTreasure(Array.Number)
      ' check file bounds
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
         Number.Items = Number.Items + 1 ' increment counter
      End If ' end check file bounds
   Next ' end loop through monster treasure
   If Number.Items = 1 Then ' check value
      Outpt = Outpt + Str$(Number.Items) + " item." ' make display message
   Else ' check value
      Outpt = Outpt + Str$(Number.Items) + " items." ' make display message
   End If ' end check value
   Call IO.O ' send output message
End Sub ' end routine

Rem * This routines displays information on the monster class file.
Rem * input variables:
Rem *   Number - the monster class file record.

Sub Display.Monster.Class (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Monster class number" + Str$(Number) + ":"
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   For Array.Number = 1 To 10 ' loop through the monster classes
      ' get monster number in class
      Monster.Index = GetMonclassRecordMonsters(Array.Number)
      ' file bounds
      If Monster.Index > False And Monster.Index <= LOF(MonsterFile) / Len(MonsterRecord) Then
         Call Read.Record(MonsterFile, Monster.Index) ' get monster record
         Outpt = "Monster number" + Str$(Array.Number) + ":" + RTrim$(MonsterRecord.MonsterName) ' make display message
         Call IO.O ' send output message
      End If ' end check file bounds
   Next ' end loop through monster class
End Sub ' end routine to display monster class

Rem * This routines displays information on the nonplayer file.
Rem * input variables:
Rem *   Number - the nonplayer file record.

Sub Display.Nonplayer (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Nonplayer number" + Str$(Number) + ":" ' nonplayer number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "[A]Nonplayer name: " + RTrim$(MonsterRecord.MonsterName) ' message
   Call IO.O ' send output message
   Outpt = "[B]Nonplayer rooms: " + MonsterRecord.PluralName ' message
   Call IO.O ' send output message
   Outpt = "[C]Nonplayer level:" + Str$(MonsterRecord.Level) ' message
   Call IO.O ' send output message
   Outpt = "[D]Hit points:" + Str$(MonsterRecord.Hits) ' message
   Call IO.O ' send output message
   Outpt = "[E]Experience points:" + Str$(MonsterRecord.Experience) ' message
   Call IO.O ' send output message
   Outpt = "[F]Gold points:" + Str$(MonsterRecord.Gold) ' message
   Call IO.O ' send output message
   Outpt = "[G]Poisonous nonplayer: " ' message
   If MonsterRecord.Poison Then ' verify nonplayer
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.PoisonPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[H]Level draining nonplayer: " ' message
   If MonsterRecord.LevelDrain Then ' verify nonplayer
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.DrainPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[I]Nonplayer blocks exits: " ' message
   If MonsterRecord.Block Then ' verify nonplayer
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.BlockPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[J]Nonplayer prevents treasure take: " ' message
   If MonsterRecord.Prevent Then ' verify nonplayer
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.PreventPercent), 2) + " percent)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[K]Nonplayer follows player: " ' message
   If MonsterRecord.Follow Then ' verify nonplayer
      Outpt = Outpt + "Yes(" + Mid$(Str$(MonsterRecord.FollowPercent), 2) + " percent)(" + Mid$(Str$(MonsterRecord.Teleport), 2) + "% teleport)" ' message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[L]Nonplayer casts spells: " ' message
   Spell.Number = MonsterRecord.Spell ' store spell number
   If Spell.Number = False Then ' verify nonplayer
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      ' file bounds
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Outpt = Outpt + "Yes, " + RTrim$(SpellRecord.SpellName) + "(" + Mid$(Str$(MonsterRecord.SpellPercent), 2) + " percent)" ' message
      End If ' end check file bounds
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[M]Nonplayer jails attacker: " ' message
   If MonsterRecord.Jail Then ' verify nonplayer
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[N]Encounter rate:"
   If MonsterRecord.Rate = False Then ' verify rate
      Outpt = Outpt + " No"
   Else ' verify rate
      Outpt = Outpt + Str$(MonsterRecord.Rate) + "(" + Mid$(Str$(MonsterRecord.RatePercent), 2) + " percent)" ' message
   End If ' end verify rate
   Call IO.O ' send output message
   Outpt = "[O]Nonplayer uses psionics: " ' message
   If MonsterRecord.Psionic = False Then ' verify nonplayer
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Spell.Number = MonsterRecord.PsionicSpell ' store psionic spell number
      ' file bounds
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Outpt = Outpt + "Yes, " + SpellRecord.SpellName ' message
      End If ' end check file bounds
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[P]Carries treasure:" ' message
   Call IO.O ' send output message
   For Array.Number = 1 To 5 ' loop through nonplayer treasure
      ' get nonplayer treasure number
      Treasure.Number = GetMonsterRecordTreasure(Array.Number)
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Outpt = TreasureRecord.TreasureName ' store treasure name
         Call IO.O ' send output message
      End If ' end check file bounds
   Next ' end loop through nonplayer treasure
End Sub ' end routine

Rem * This routines displays information on the object file.
Rem * input variables:
Rem *   Number - the object file record.

Sub Display.Object (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Object number" + Str$(Number) + ":" ' object number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "[A]Object name: " + RTrim$(ObjectRecord.ObjectName) ' message
   Call IO.O ' send output message
   Outpt = "[B]Object identifier: " + LCase$(RTrim$(ObjectRecord.ShortName))
   Call IO.O ' send output message
   Outpt = "[C]Room number link:" + Str$(ObjectRecord.RoomLink) ' message
   Call IO.O ' send output message
   Outpt = "[D]Trapped portal: " ' message
   Select Case ObjectRecord.Trap ' selection of object trap type
      Case False ' verify object
         Outpt = Outpt + "No" ' add to message
      Case 1 ' verify object
         Outpt = Outpt + "poison needles" ' message
      Case 2 ' berify object
         Outpt = Outpt + "teleport to" + Str$(ObjectRecord.Teleport) ' message
      Case 3 ' verify object
         Outpt = Outpt + "hits for" + Str$(Abs(ObjectRecord.Teleport)) + " " ' message
         Select Case Object.Teleport ' selection of object trap hit type
            Case Is < False ' verify object
               Outpt = Outpt + "vitality" ' message
            Case Else ' verify object
               Outpt = Outpt + "fatigue" ' message
         End Select ' end selection of object hit type
   End Select ' end selection of object type
   Call IO.O ' send output message
   Outpt = "[E]Long description:" ' message
   Call IO.O ' send output message
   Outpt = RTrim$(ObjectRecord.LongDesc) ' message
   If Outpt = Nul Then ' check message
      Outpt = "<none>" ' reset
   End If ' end check message
   Call IO.O ' send output message
   Outpt = "[F]Entry description:" ' message
   Call IO.O ' send output message
   Outpt = RTrim$(ObjectRecord.ShortDesc) ' message
   If Outpt = Nul Then ' check message
      Outpt = "<none>" ' reset
   End If ' end check message
   Call IO.O ' send output message
   Outpt = "[G]Hidden object: " ' message
   If ObjectRecord.Hidden Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[H]Invisible object: " ' message
   If ObjectRecord.Invisible Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[I]Jail attacker trap: " ' message
   If ObjectRecord.JailTrap Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[J]Locked portal: " ' message
   If ObjectRecord.DoorLock = 2 Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[K]Relocking portal: " ' message
   If ObjectRecord.Relocks Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[L]Key number:" + Str$(ObjectRecord.Keyed) ' message
   Call IO.O ' send output message
   Outpt = "[M]Permanent object: " ' message
   If ObjectRecord.Permanent Then ' verify object
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[N]Object is a light: " ' message
   If ObjectRecord.LightRoom = False Then ' verify object
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Outpt = Outpt + "Yes, Light Time: " ' message
      If ObjectRecord.LightTime = False Then ' verify object
         Outpt = Outpt + "<any>" ' message
      Else ' verify
         Outpt = Outpt + "From:" + Right$(Str$(ObjectRecord.FromHour + 100), 2) + ":" + Right$(Str$(ObjectRecord.FromMin + 100), 2) + " to " + Right$(Str$(ObjectRecord.ToHour + 100), 2) + ":" + Right$(Str$(ObjectRecord.ToMin + 100), 2) ' message
      End If ' end verify
   End If ' end verify
   Call IO.O ' send output message
End Sub ' end routine

Rem * This routines displays information on the room file.
Rem * input variables:
Rem *   Number! - the room file record.

Sub Display.Room.Desc (Number!)
   If Number! <= 0! Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Room number" + Str$(Number!) + ": Action" + Str$(RoomRecord.Action) + ": Monster class:" + Str$(RoomRecord.MonsterClass) + "." ' make room message
   Call IO.O ' send output message
   Outpt = "Short description:" ' message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = RTrim$(RoomRecord.ShortDesc) ' message
   If InStr(Outpt, Chr$(0)) Then ' find any null characters from old structures
      Outpt = Left$(Outpt, InStr(Outpt, Chr$(0)) - 1) ' truncate off nulls
   End If ' end find old nulls
   Call IO.O ' send output message
   Graphics.Off = False ' reset color
   Outpt = "Long description:" ' message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   For Array.Number = 1 To 4 ' loop through all four long room description lines
      Outpt = GetRoomRecordLongDesc$(Array.Number) ' get next room description line
      Outpt = RTrim$(Outpt) ' trim room description line
      If InStr(Outpt, Chr$(0)) Then ' find any nul characters from old files
         Outpt = Left$(Outpt, InStr(Outpt, Chr$(0)) - 1) ' truncate off nulls
      End If ' end find old nulls
      If Outpt = Nul Then ' check if long description ends
         Exit For ' exit long description display loop
      End If ' end check description end
      Call IO.O ' send output message
   Next ' end long room description loop
End Sub ' end routine

Rem * This routines displays information on the spell file.
Rem * input variables:
Rem *   Number - the spell file record.

Sub Display.Spell (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Spell number" + Str$(Number) + ":" ' spell number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "[A]Spell name: " + RTrim$(SpellRecord.SpellName) ' message
   Call IO.O ' send output message
   Outpt = "[B]Spell chant:" ' message
   Call IO.O ' send output message
   Outpt = LCase$(RTrim$(SpellRecord.Chant)) ' message
   Call IO.O ' send output message
   Outpt = "[C]Spell cast description:" ' message
   Call IO.O ' send output message
   Outpt = RTrim$(SpellRecord.Desc)
   Call IO.O ' send output message
   Outpt = "[D]Spell level:" + Str$(SpellRecord.Level) ' message
   Call IO.O ' send output message
   Outpt = "[E]" ' set to option
   Select Case SpellRecord.SpellType ' selection of spell type
      Case Enchant
         Inpt = "Enchant" ' message
      Case Offense
         Inpt = "Offense" ' message
      Case Bless
         Inpt = "Bless" ' message
      Case Wish
         Inpt = "Wish" ' message
      Case Poison
         Inpt = "Poison" ' message
      Case Vigor
         Inpt = "Vigor" ' message
      Case Heal
         Inpt = "Heal" ' message
      Case CurePoison
         Inpt = "Curepoison" ' message
      Case LevelDrain
         Inpt = "Level Drain" ' message
      Case Teleport
         Inpt = "Teleport to Room" + Str$(SpellRecord.Teleport)
      Case Befuddled
         Inpt = "Befuddle" ' message
      Case TurnUndead
         Inpt = "Turn Undead" ' message
      Case PassDoor
         Inpt = "Pass Door" ' message
      Case Conjure
         Inpt = "Conjure" ' message
      Case Psionic
         Inpt = "Psionic" ' message
      Case DetectLock
         Inpt = "Detect Lock" ' message
      Case DetectEvil
         Inpt = "Detect Evil" ' message
      Case DetectTrap
         Inpt = "Detect Trap" ' message
      Case Intoxicate
         Inpt = "Intoxicate" ' message
      Case SetTrap
         Inpt = "Set Trap" ' message
      Case Hiding
         Inpt = "Hide" ' message
      Case Search
         Inpt = "Search" ' message
      Case Invisibility
         Inpt = "Invisibility" ' message
      Case Identify
         Inpt = "Identify" ' message
      Case Enlighten
         Inpt = "Enlighten" ' message
      Case Illuminate
         Inpt = "Illuminate" ' message
      Case Psyche
         Inpt = "Psyche" ' message
      Case Telepathy
         Inpt = "Telepathy" ' message
      Case Else ' other
         Inpt = "<none>" ' message
   End Select ' end selection of spell type
   Outpt = Outpt + "Spell type: " + Inpt ' message
   Call IO.O ' send output message
   Outpt = "[F]Spell is psionic: " ' message
   If SpellRecord.Psionic = False Then ' verify spell
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Outpt = Outpt + "Yes, " ' message
      Select Case SpellRecord.PsionicMode ' selection of spell psionic type
         Case PsiAttack
            Outpt = Outpt + "Attack" ' message
         Case PsiDefense
            Outpt = Outpt + "Defense" ' message
         Case Else ' other
            Outpt = Outpt + "<none>" ' message
      End Select ' end selection of spell type
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[G]Characters which can cast spell:" ' message
   Call IO.O ' send output message
   Spell.Cast = False ' set spell cast found flag
   For Class.Number = 1 To 8 ' loop through spell cast flags
      If SpellRecord.ClassType And 2 ^ Class.Number Then ' compare spell cast flag
         Class.Type$ = Class.Name(Class.Number) ' store class name
         Class.Type$ = RTrim$(Class.Type$) ' trim name
         ' uppercase first word
         Mid$(Class.Type$, 1, 1) = UCase$(Mid$(Class.Type$, 1, 1))
         Outpt = Outpt + Class.Type$ + ", " ' append to output string
         Spell.Cast = True ' set spell cast found flag
      End If ' end compare spell cast flags
   Next ' end loop through spell cast flags
   If Spell.Cast Then ' check spell cast found flag
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' message
   Else ' check spell flag
      Outpt = "<none>" ' message
   End If ' end check spell cast found flag
   Call IO.O ' send output message
   Outpt = "[H]Spell ingredients:" ' message
   Call IO.O ' send output message
   Spell.Ingredient = False ' set spell ingredient found flag
   For Array.Number = 1 To 5 ' loop through spell ingerdients
      ' get spell ingredient number
      Treasure.Number = GetSpellRecordIngred(Array.Number)
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Ingredient.Name$ = TreasureRecord.TreasureName ' store treasure name
         Ingredient.Name$ = RTrim$(Ingredient.Name$) ' trim name
         Ingredient.Name$ = LCase$(Ingredient.Name$) ' lowercase name
         ' uppercase first word
         Mid$(Ingredient.Name$, 1, 1) = UCase$(Mid$(Ingredient.Name$, 1, 1))
         Outpt = Outpt + Ingredient.Name$ + ", " ' message
         Spell.Ingredient = True ' set spell ingredient found flag
      End If ' end check file bounds
   Next ' end loop through spell ingredients
   If Spell.Ingredient Then ' check spell ingredient found flag
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' message
   Else ' check spell ingredient
      Outpt = "<none>" ' message
   End If ' end check spell ingredient found flag
   Call IO.O ' send output message
   Outpt = "[I]Spell casting type which requires ingredients:" ' message
   Call IO.O ' send output message
   Spell.Requirement = False ' set ingredient requirement flag
   If SpellRecord.SpellFlag And Use.Spell.Type Then ' verify spell
      Outpt = "use command, " ' message
      Spell.Requirement = True ' set ingredient flag
   End If ' end verify
   If SpellRecord.SpellFlag And Scroll.Spell.Type Then ' verify spell
      Outpt = Outpt + "read scroll, " ' message
      Spell.Requirement = True ' set ingredient flag
   End If ' end verify
   If SpellRecord.SpellFlag And Cast.Spell.Type Then ' verify spell
      Outpt = Outpt + "cast spell, " ' message
      Spell.Requirement = True ' set ingredient flag
   End If ' end verify
   If Spell.Requirement Then ' check ingredient flag
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' message
   Else ' check flag
      Outpt = "<none>" ' message
   End If ' end check ingredient flag
   Call IO.O ' send output message
End Sub ' end routine

Rem * This routines displays information on the treasure file.
Rem * input variables:
Rem *   Number - the treasure file record.

Sub Display.Treasure (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Treasure number" + Str$(Number) + ":" ' treasure number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "[A]Treasure name: " + TreasureRecord.TreasureName ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[U]Treasure is coins: " ' message
   If TreasureRecord.Coin Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[B]Treasure identifier: " + LCase$(TreasureRecord.ShortName) ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[V]Treasure is a potion: " ' message
   If TreasureRecord.Potion Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[C]Weight:" + Str$(TreasureRecord.Weight) ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[W]Treasure is a scroll: " ' message
   If TreasureRecord.Scroll Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[D]Gold coin value:" + Str$(TreasureRecord.Gold) ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[X]Treasure is invisible: " ' message
   If TreasureRecord.Invisible Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[E]Treasure type: " ' message
   If TreasureRecord.Loadable Then ' check treasure type
      Outpt = Outpt + "Loadable" ' append message
   Else ' check next type
      If TreasureRecord.Launchable Then ' check treasure type
         Outpt = Outpt + "Launchable" ' append message
      Else ' check next type
         Select Case TreasureRecord.Type ' selection of treasure type
            Case False ' verify treasure type
               If TreasureRecord.Plus = False Then ' verify weapon type
                  Outpt = Outpt + "Treasure" ' message
               Else ' verify type
                  Outpt = Outpt + "Weapon" ' message
               End If ' end verify treasure type
            Case Is < False ' verify shield type
               Outpt = Outpt + "Shield" ' message
            Case Is > False ' verify armor type
               Outpt = Outpt + "Armor" ' message
         End Select ' end selection of treasure type
      End If ' end check treasure type
   End If ' end check treasure type
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[Y]Treasure is a container: " ' message
   If TreasureRecord.Container Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[F]Hit plus:" + Str$(Abs(TreasureRecord.Plus)) ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[Z]Container is locked: " ' message
   If TreasureRecord.Locked = 1 And TreasureRecord.Closed = 1 Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[G]Charges:" + Str$(TreasureRecord.Charges) ' message
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[1]Treasure key number:" + Str$(TreasureRecord.Keyed) ' message
   Call IO.O ' send output message
   Outpt = "[H]Weapon class: " ' message
   ' selection of treasure proficiency type
   Select Case TreasureRecord.Proficiency
      Case Blunt
         Outpt = Outpt + "Blunt" ' message
      Case Pole
         Outpt = Outpt + "Pole" ' message
      Case Sharp
         Outpt = Outpt + "Sharp" ' message
      Case Thrusting
         Outpt = Outpt + "Thrusting" ' message
      Case Else ' other
         Outpt = Outpt + "None" ' message
   End Select ' end selection of weapon proficiency type
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[2]Treasure is fuel: " ' message
   If TreasureRecord.FuelType = False Then ' verify treasure
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Outpt = Outpt + "Yes:" + Str$(TreasureRecord.FuelCharges) + " charges" ' message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[I]Treasure is permanent: " ' message
   If TreasureRecord.Permanent Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[3]Weapon rusts: " ' message
   If TreasureRecord.Rustable Then ' verify treasure
      Outpt = Outpt + "Yes:" + Str$(TreasureRecord.RustPercent) + "%"
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[J]Treasure is edible: " ' message
   If TreasureRecord.Edible Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Outpt = Left$(Outpt, 40) ' truncate string
   Outpt = Outpt + Space$(40 - Len(Outpt)) ' pad with blanks
   Outpt = Outpt + "[4]Weapon is stealable: " ' message
   If TreasureRecord.Stealable Then ' verify treasure
      Outpt = Outpt + "Yes:" + Str$(TreasureRecord.StealPercent) + "%"
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[K]Treasure is magical: " ' message
   Spell.Number = TreasureRecord.Spell ' store spell number
   If Spell.Number = False Then ' verify treasure
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      ' file bounds
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Outpt = Outpt + "Yes, " + RTrim$(SpellRecord.SpellName) + " spell" ' message
      End If ' end check file bounds
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[L]Treasure is a ring: " ' message
   Select Case TreasureRecord.RingType
      Case 1
         Outpt = Outpt + "protection from poison" ' message
      Case 2
         Outpt = Outpt + "protection from level drain" ' message
      Case 3
         Outpt = Outpt + "protection from spell: " ' message
         Spell.Number = TreasureRecord.RingSpell ' store ring spell number
         If Spell.Number = True Then ' verify treasure
            Outpt = Outpt + "generic spell" ' message
         Else ' verify
            ' file bounds
            If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
               Call Read.Record(SpellFile, Spell.Number) ' get spell record
               Outpt = Outpt + RTrim$(LCase$(SpellRecord.SpellName)) + " spell" ' message
            End If ' end check file bounds
         End If ' end verify
      Case Else ' verify
         Outpt = Outpt + "No" ' add to message
   End Select
   Call IO.O ' send output message
   Outpt = "[M]Treasure is a light: " ' message
   If TreasureRecord.LightType = False Then ' verify treasure
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Outpt = Outpt + "Yes, Charges:" + Str$(TreasureRecord.LightCharges) ' message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[N]Treasure is a vehicle: " ' message
   If TreasureRecord.Vehicle = False Then ' verify treasure
      Outpt = Outpt + "No" ' add to message
   Else ' verify
      Outpt = Outpt + "Yes, hits:" + Str$(TreasureRecord.VehicleHits) ' message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[O]Treasure loads from devices: " ' message
   If TreasureRecord.Ammunition Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[P]Treasure is loadable: " ' message
   If TreasureRecord.Loadable Then ' verify treasure
      Outpt = Outpt + "Yes: Loads(" ' add to message
      ' store ammunition treasure number
      Treasure.Number = TreasureRecord.AmmoLoads
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Share.Record(TreasureFile, Number) ' store current treasure record
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) + ")" ' message
         Call Read.Record(TreasureFile, Number) ' restore treasure record
      Else ' check file bounds
         Outpt = Outpt + "<none>)" ' append message
      End If ' end check file bounds
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[R]Treasure launchs from devices: " ' message
   If TreasureRecord.LaunchAmmo Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[S]Treasure is launchable: " ' message
   If TreasureRecord.Launchable Then ' verify treasure
      Outpt = Outpt + "Yes: Launchs(" ' add to message
      ' store ammunition treasure number
      Treasure.Number = TreasureRecord.LaunchLoads
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Share.Record(TreasureFile, Number) ' store current treasure record
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) + ")" ' message
         Call Read.Record(TreasureFile, Number) ' restore treasure record
      Else ' check file bounds
         Outpt = Outpt + "<none>)" ' append message
      End If ' end check file bounds
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[T]Launchable device can be moved: " ' message
   If TreasureRecord.Movable Then ' verify treasure
      Outpt = Outpt + "Yes" ' add to message
   Else ' verify
      Outpt = Outpt + "No" ' add to message
   End If ' end verify
   Call IO.O ' send output message
End Sub ' end routine

Rem * This routines displays information on the user file.
Rem * input variables:
Rem *   Number - the user file record.

Sub Display.User (Number)
   If Number <= False Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "User number" + Str$(Number) + "." ' user number message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Inpt = UserRecord.CodeName ' store player codename
   Call Decrypt(Inpt) ' decrypt codename
   Inpt = RTrim$(Inpt) ' trim codename
   Inpt = LCase$(Inpt) ' lowercase codename
   Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' uppercase first word
   Outpt = "[A]Codename: " + Inpt + " " ' message
   Inpt = String$(10, Mask$) ' mask passwword
   Outpt = Outpt + "[B]Password: " + Inpt ' message
   Call IO.O ' send output message
   Outpt = "[C]Level:" + Str$(UserRecord.Level) ' message
   Call IO.O ' send output message
   Outpt = "[D]" ' set to option
   Class.Number = UserRecord.ClassType ' store player class type
   Select Case Class.Number ' selection of class type
      Case 1 To 10 ' check class type
         Class.Type$ = Class.Name(Class.Number) ' store class name
      Case Else ' check
         Class.Type$ = Class.Name(1) ' default class name
   End Select ' end selection of class type
   Outpt = Outpt + "Class: " + RTrim$(Class.Type$) ' message
   Call IO.O ' send output message
   Outpt = "[E]" ' set to option
   Weapon.Number = UserRecord.Proficiency ' store player proficiency number
   Select Case Weapon.Number ' selection of proficiency
      Case 1 To 4 ' check type
         Weapon.Type$ = WeaponTypeName(Weapon.Number) ' store proficiency name
      Case Else ' check
         Weapon.Type$ = WeaponTypeName(1) ' default name
   End Select ' end seelction of proficiency
   Mid$(Weapon.Type$, 1, 1) = UCase$(Mid$(Weapon.Type$, 1, 1)) ' uppercase first letter
   Outpt = Outpt + "Weapon proficiency: " + RTrim$(Weapon.Type$) ' message
   Call IO.O ' send output message
   Outpt = "[F]Blunt%:" + Str$(GetUserRecordWeapons(1)) + " " ' message
   Outpt = Outpt + "[G]Pole%:" + Str$(GetUserRecordWeapons(2)) ' message
   Call IO.O ' send output message
   Outpt = "[H]Sharp%:" + Str$(GetUserRecordWeapons(3)) + " " ' message
   Outpt = Outpt + "[I]Thrusting%:" + Str$(GetUserRecordWeapons(4)) ' message
   Call IO.O ' send output message
   Outpt = "[J]Classname: " ' message
   Inpt = UserRecord.ClassName
   Call Decrypt(Inpt)
   Inpt = RTrim$(Inpt)
   Outpt = Outpt + Inpt
   Call IO.O ' send output message
   Outpt = "[K]Strength:" + Str$(GetUserRecordStats(1)) ' message
   Call IO.O ' send output message
   Outpt = "[L]Intelligence:" + Str$(GetUserRecordStats(2)) ' message
   Call IO.O ' send output message
   Outpt = "[M]Wisdom:" + Str$(GetUserRecordStats(3)) ' message
   Call IO.O ' send output message
   Outpt = "[N]Dexterity:" + Str$(GetUserRecordStats(4)) ' message
   Call IO.O ' send output message
   Outpt = "[O]Constitution:" + Str$(GetUserRecordStats(5)) ' message
   Call IO.O ' send output message
   Outpt = "[P]Piety:" + Str$(GetUserRecordStats(6)) ' message
   Call IO.O ' send output message
   Outpt = "[R]Charisma:" + Str$(GetUserRecordStats(7)) ' message
   Call IO.O ' send output message
   Outpt = "[S]Experience: " + FormatString$(UserRecord.Experience) ' message
   Call IO.O ' send output message
   Outpt = "[T]Gold: " + FormatString$(UserRecord.Gold) ' message
   Call IO.O ' send output message
   Outpt = "[U]Room number:" + Str$(UserRecord.Room) ' message
   Call IO.O ' send output message
   Outpt = "[V]Call restrictions." ' message
   Call IO.O ' send output message
   Outpt = "[W]Inventory:" ' message
   ' count user inventory items
   Number.Items = False ' reset counter
   ' loop through player objects
   For Object.Index = 1 To 20
      ' store object
      Object.Number = GetUserRecordObject(Object.Index)
      ' check object number
      If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
         Number.Items = Number.Items + 1 ' increment number of items
      End If ' end check object number
   Next ' end count objects
   ' loop through inventory
   For Treasure.Index = 1 To 20
      ' store number
      Treasure.Number = GetUserRecordInv(Treasure.Index)
      ' check number
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
         Number.Items = Number.Items + 1 ' increment number of items
      End If ' end check object number
   Next ' end count inventory
   ' loop through user containers
   For Number.Containers = 1 To 3 ' loop through user containers
      X = GetUserRecordContainer(Number.Containers) ' store user container into record
      If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
         Number.Items = Number.Items + 1 ' increment number of items
      End If ' end check object number
   Next ' end count inventory
   If Number.Items = 1 Then ' check value
      Outpt = Outpt + Str$(Number.Items) + " item." ' make display message
   Else ' check value
      Outpt = Outpt + Str$(Number.Items) + " items." ' make display message
   End If ' end check value
   Call IO.O ' send output message
   Outpt = "[X]Special characters: " ' message
   Special.Flag = False
   If UserRecord.Flags And Special.Char1 Then ' verify user
      Outpt = Outpt + "Town Mayor, " ' message
      Special.Flag = True
   End If ' end verify
   If UserRecord.Flags And Special.Char2 Then ' verify user
      Outpt = Outpt + "Governor, " ' message
      Special.Flag = True
   End If ' end verify
   If UserRecord.Flags And Special.Char3 Then ' verify user
      Outpt = Outpt + "Guild Master, " ' message
      Special.Flag = True
   End If ' end verify
   If UserRecord.Flags And Special.Char4 Then ' verify user
      Outpt = Outpt + "Sysop, " ' message
      Special.Flag = True
   End If ' end verify
   If Special.Flag Then ' verify user
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' message
   Else ' verify
      Outpt = Outpt + "None." ' message
   End If ' end verify
   Call IO.O ' send output message
   Outpt = "[Y]Training rooms" ' message
   Call IO.O ' send output message
   Graphics.Off = False ' reset color
End Sub ' end routine

Rem * routine used by the editor to find monster file number
Rem * of a monster name.
Rem * output variables:
Rem *   Number - record number of monster.
Rem * processing variables:
Rem *   Inpt - monster name or number to search for.
Rem *   Parsed.Value - monster number increment.

Sub Find.Monster (Number)
   Outpt = "Monster name, or number" ' input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(MonsterFile) / Len(MonsterRecord)), 2) + ")? "
   Call IO.I ' get input
   If Inpt = Nul Then ' compare input length
      Number = False ' reset number found
      Exit Sub ' exit routine
   End If ' end compare length
   Call Parse.Num(Inpt, Parsed.Value) ' parse number
   Number.Count = False ' reset monster found counter
   For Number = 1 To LOF(MonsterFile) / Len(MonsterRecord) ' loop through monsters
      Call Read.Record(MonsterFile, Number) ' get monster record
      ' compare monster record name to input monster name,
      ' truncated to input length
      If Left$(MonsterRecord.MonsterName, Len(Inpt)) = Inpt Then ' compare
         Number.Count = Number.Count + 1 ' increment monster found counter
         ' compare parsed number of monster to search for
         If Parsed.Value = False Or Number.Count = Parsed.Value Then ' compare
            Exit Sub ' exit routine
         End If ' end compare counters
      End If ' end compare monster names
   Next ' end loop through monster file
   Number = Int(Val(Inpt)) ' convert input to integer
   If Number <= False Or Number > LOF(MonsterFile) / Len(MonsterRecord) Then ' file bounds
      Outpt = Range$ ' make error message
      Call IO.O ' send error message
      Number = False ' reset monster found
   Else ' end check file bounds
      Call Read.Record(MonsterFile, Number) ' get monster record number
   End If ' end check file bounds
End Sub ' end routine

Rem * routine used by the editor to find nonplayer file number
Rem * of a nonplayer name.
Rem * output variables:
Rem *   Number - record number of nonplayer.
Rem * processing variables:
Rem *   Inpt - nonplayer name or number to search for.
Rem *   Parsed.Value - nonplayer number increment.

Sub Find.Nonplayer (Number)
   Outpt = "Nonplayer name, or number" ' input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(NonPlayerFile) / Len(MonsterRecord)), 2) + ")? "
   Call IO.I ' get input
   If Inpt = Nul Then ' compare input length
      Number = False ' reset number found
      Exit Sub ' exit routine
   End If ' end compare length
   Call Parse.Num(Inpt, Parsed.Value) ' parse number
   Number.Count = False ' reset nonplayer counter
   For Number = 1 To LOF(NonPlayerFile) / Len(MonsterRecord) ' loop through nonplayers
      Call Read.Record(NonPlayerFile, Number) ' get nonplayer record
      ' compare nonplayer record name to input nonplayer name,
      ' truncated to input length
      If Left$(MonsterRecord.MonsterName, Len(Inpt)) = Inpt Then ' compare
         Number.Count = Number.Count + 1 ' increment nonplayer found counter
         ' compare parsed number of object to search for
         If Parsed.Value = False Or Number.Count = Parsed.Value Then ' compare
            Exit Sub ' exit routine
         End If ' end compare counters
      End If ' end compare nonplayer names
   Next ' end loop through nonplayer file
   Number = Int(Val(Inpt)) ' convert input to integer
   If Number <= False Or Number > LOF(NonPlayerFile) / Len(MonsterRecord) Then ' file bounds
      Outpt = Range$ ' make error message
      Call IO.O ' send error message
      Number = False ' reset nonplayer found
   Else ' end check file bounds
      Call Read.Record(NonPlayerFile, Number) ' get nonplayer record number
   End If ' end check file bounds
End Sub ' end routine

Rem * routine used by the editor to find the object file number
Rem * of an object name.
Rem * output variables:
Rem *   Number - record number of object.
Rem * processing variables:
Rem *   Inpt - object name or number to search for.
Rem *   Parsed.Value - object number increment.

Sub Find.Objects (Number)
   Outpt = "Object name, or number" ' make input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(ObjectFile) / Len(ObjectRecord)), 2) + ")? "
   Call IO.I ' get input
   If Inpt = Nul Then ' compare input length
      Number = False ' reset number found
      Exit Sub ' exit routine
   End If ' end compare length
   Call Parse.Num(Inpt, Parsed.Value) ' get parsed number
   Number.Count = False ' reset spell found counter
   For Number = 1 To LOF(ObjectFile) / Len(ObjectRecord) ' loop through object file
      Call Read.Record(ObjectFile, Number) ' get object record
      ' compare object record name to input object name,
      ' truncated to input length
      If Left$(ObjectRecord.ObjectName, Len(Inpt)) = Inpt Then ' compare
         Number.Count = Number.Count + 1 ' increment object counter
         ' compare parsed number of object to search for
         If Parsed.Value = False Or Number.Count = Parsed.Value Then ' compare
            Exit Sub ' exit routine
         End If ' end compare counters
      End If ' end compare object names
   Next ' end loop through object file
   Number = Int(Val(Inpt)) ' convert input to integer
   If Number <= False Or Number > LOF(ObjectFile) / Len(ObjectRecord) Then ' file bounds
      Outpt = Range$ ' make error message
      Call IO.O ' send error message
      Number = False ' reset object number to zero
   Else ' end check file bounds
      Call Read.Record(ObjectFile, Number) ' get object record number
   End If ' end check file bounds
End Sub ' end routine

Rem * routine used by the editor to find the spell file number
Rem * of a spell name.
Rem * output variables:
Rem *   Number - record number of spell.
Rem * processing variables:
Rem *   Inpt - spell name or number to search for.
Rem *   Parsed.Value - spell number increment.

Sub Find.Spell (Number)
   Outpt = "Spell name, or number" ' input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(SpellFile) / Len(SpellRecord)), 2) + ")? "
   Call IO.I ' get input
   If Inpt = Nul Then ' check length of input
      Number = False ' reset number found
      Exit Sub ' exit routine
   End If ' end check length
   Call Parse.Num(Inpt, Parsed.Value) ' parse out number
   Number.Count = False ' reset spell found counter
   For Number = 1 To LOF(SpellFile) / Len(SpellRecord) ' loop through spell file
      Call Read.Record(SpellFile, Number) ' get spell record
      ' compare spell record name to input spell name, truncated to input length
      If Left$(SpellRecord.SpellName, Len(Inpt)) = Inpt Then ' compare
         Number.Count = Number.Count + 1 ' increment spell found
         ' compare parsed number of spell to search for
         If Parsed.Value = False Or Number.Count = Parsed.Value Then ' compare
            Exit Sub ' exit routine
         End If ' end compare counters
      End If ' end compare spell names
   Next ' end loop through spell file
   Number = Int(Val(Inpt)) ' convert input to integer
   If Number <= False Or Number > LOF(SpellFile) / Len(SpellRecord) Then ' compare file bounds
      Outpt = Range$ ' make error message
      Call IO.O ' send error message
      Number = False ' reset return spell number to zero
   Else ' end compare spell number
      Call Read.Record(SpellFile, Number) ' get spell record number
   End If ' end check file bounds
End Sub ' end routine

Rem * routine used by the editor to find the treasure file number
Rem * of a treasure name.
Rem * output variables:
Rem *   Number - record number of treasure.
Rem * processing variables:
Rem *   Inpt - treasure name or number to search for.
Rem *   Parsed.Value - treasure number increment.

Sub Find.Treasure (Number)
   Outpt = "Treasure name, or number" ' input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(TreasureFile) / Len(TreasureRecord)), 2) + ")? "
   Call IO.I ' get input
   If Inpt = Nul Then ' compare input length
      Number = False ' reset number found
      Exit Sub ' exit routine
   End If ' end compare length
   Call Parse.Num(Inpt, Parsed.Value) ' parse number
   Number.Count = False ' reset treasure found counter
   For Number = 1 To LOF(TreasureFile) / Len(TreasureRecord) ' loop through treasure
      Call Read.Record(TreasureFile, Number) ' get treasure record
      ' compare treasure record name to input treasure name,
      ' truncated to input length
      If Left$(TreasureRecord.TreasureName, Len(Inpt)) = Inpt Then ' compare
         Number.Count = Number.Count + 1 ' increment treasure found counter
         ' compare parsed number of treasure to search for
         If Parsed.Value = False Or Number.Count = Parsed.Value Then ' compare
            Exit Sub ' exit routine
         End If ' end compare counters
      End If ' end compare treasure names
   Next ' end loop through treasure file
   Number = Int(Val(Inpt)) ' convert input to integer
   If Number <= False Or Number > LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
      Outpt = Range$ ' make error message
      Call IO.O ' send error message
      Number = False ' reset treasure found
   Else ' end check file bounds
      Call Read.Record(TreasureFile, Number) ' get treasure record number
   End If ' end check file bounds
End Sub ' end routine

Rem * routine for player to hold an item of inventory.

Sub Hold.Object
   Call Check.Inventory.Treasure ' check player inventory treasure mneomnic
   If Index.Number = False Then ' found treasure file index
      Outpt = "You aren't carrying that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare player inventory
   If TreasureRecord.Type > False Then ' compare treasure type
      Outpt = "You can't hold that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure type
   If TreasureRecord.Loadable Then ' verify treasure type
      Outpt = "You can't hold that!" ' make output message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end verify treasure type
   If TreasureRecord.Launchable Then ' verify treasure type
      Outpt = "You can't hold that!" ' make output message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end verify treasure type
   ' compute treasure charges remaining
   If GetUserRecordCharges(Array.Number) = False Then
      Outpt = "You can't, it has zero charges!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure charges
   Item.Plus = Abs(TreasureRecord.Plus)
   Select Case TreasureRecord.Type ' select treasure type
      Case Is < False ' compare treasure type to shield
         Weapon3 = Item.Plus ' set variable of holding shield plus
         ' set variable of holding shield of inventory array index
         Weapon5 = Array.Number
      Case False ' compare treasure type to weapon
         If Item.Plus = False Then ' verify weapon type
            Outpt = "You can't hold that!" ' make output message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end verify weapon type
         Index.Number = TreasureRecord.Proficiency ' store treasure proficiency
         If UserRecord.ClassType = Cleric Then ' compare player classtype to cleric
            Select Case Index.Number ' select treasure proficiency type
               Case 3 To 4 ' compare to sharp or thrusting
                  Outpt = "Clerics may only use blunt or pole type weapons!" ' message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
            End Select ' end compare weapon proficiency type
         End If ' end compare to cleric
         Weapon2 = Item.Plus ' set variable of holding weapon plus
         ' set variable of holding weapon of inventory array index
         Weapon6 = Array.Number
         Weapon10 = Index.Number ' set variable of holding weapon proficiency type
   End Select ' end select treasure type
   Outpt = "You hold " + Outpts + "(+" + Mid$(Str$(Item.Plus), 2) + ")!" ' format message
   Call IO.O ' send message
End Sub ' end routine to hold an item

Rem * routine for player to wear an item of inventory.

Sub Wear.Object
   Call Check.Inventory.Treasure ' check player inventory treasure mnemonic
   If Index.Number = False Then ' found treasure index
      Outpt = "You aren't carrying that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare index
   If TreasureRecord.Type <= False Then ' commpare treasure type
      If TreasureRecord.RingType = False Then ' compare ring type
         Outpt = "You can't wear that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare ring
   End If ' end compare treasure type
   If GetUserRecordCharges(Array.Number) = False Then ' compare player charges array
      Outpt = "You can't, it has zero charges!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare charges
   Item.Plus = Abs(TreasureRecord.Plus) ' store treasure item plus hits
   Select Case TreasureRecord.Type ' selecct treasure type
      Case Is > False ' treasure type is armor
         Weapon1 = Item.Plus ' set variable of wearing item to plus
         ' set variable of wearing item to inventory array index
         Weapon4 = Array.Number
      Case Else ' treasure type is ring
         Weapon7 = Array.Number ' set wearing ring item to inventory array index
         Weapon8 = TreasureRecord.RingType ' set wearing ring type variable
         Weapon9 = TreasureRecord.RingSpell ' set wearing ring spell type variable
         Weapon11 = Item.Plus ' set wearing ring plus
   End Select ' end select treasure type
   Outpt = "You wear " + Outpts + "(+" + Mid$(Str$(Item.Plus), 2) + ")!" ' format message
   Call IO.O ' send message
End Sub ' end routine to wear an item

Rem * routine to return an item from being held or worn.

Sub Return.Object
   Call Check.Inventory.Treasure ' check player inventory mnemonic
   If Index.Number = False Then ' treasure index of item
      Outpt = "You aren't carrying that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure inventory
   Outpt = "You return " + Outpts + "!" ' format message
   Select Case Array.Number ' select treasure inventory array variable
      Case Weapon4 ' compare treasure inventory array to armor variable
         Weapon1 = False ' reset armor being worn holding variable
         Weapon4 = False ' reset armor being worn variable
         Call IO.O ' send message
      Case Weapon5 ' compare treasure inventory array to shield variable
         Weapon3 = False ' reset shield holding variable
         Weapon5 = False ' reset shield holding variable
         Call IO.O ' send message
      Case Weapon6 ' compare treasure inventory array to weapon variable
         Weapon2 = False ' reset weapon holding variable
         Weapon6 = False ' reset weapon holding variable
         Weapon10 = False ' reset weapon holding variable
         Call IO.O ' send message
      Case Weapon7 ' compare treasure inventory array to ring variable
         Weapon7 = False ' reset ring being worn variable
         Weapon8 = False ' reset ring being worn variable
         Weapon9 = False ' reset ring being worn variable
         Weapon11 = False ' reset ring worn variable
         Call IO.O ' send message
      Case Else ' no matching variables being held or worn
         Outpt = "Your aren't wearing that!" ' make message
         Call IO.O ' send message
   End Select ' end select treasure array variable
End Sub ' end routine to return item

Rem * routine to reset an item with original charges if broken.

Sub Fix.Object
   Call Examine.Treasure ' check treasure mnemonic
   If Index.Number = False Then ' treasure index
      Outpt = "You can't fix that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check treasure
   If TreasureRecord.Spell Then ' treasure has spell
      Outpt = "You can't fix that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare spell
   If TreasureRecord.Plus = False Then ' treasure is not weapon, armor, shield
      Outpt = "You can't fix that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare spell
   If GetUserRecordCharges(Array.Number) Then ' check remaining treasure charges
      Outpt = "That's not broken!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare charges
   If GetUserRecordStats(3) < 7 Then ' compute random chance
      Outpt = "You didn't fix it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end random chance
   If Abs(TreasureRecord.Plus) > 10 Then ' compare treasure plus
      Outpt = "You didn't fix it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare plus
   If Rnd < .5 Then ' random chance
      Outpt = "You didn't fix it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end chance
   Item.Charges = TreasureRecord.Charges ' store treasure maximum charges
   If Item.Charges > 250 Then ' only repair item to maximum of 250 charges
      Item.Charges = 250 ' set storage variable to 250 charges
   End If ' end compare charges
   Select Case TypeNumber ' select room or inventory
      Case 1 ' treasure fixed in room
         ' reset room treasure charges
         Call PutRoomRecordTreCharges(Array.Number, Item.Charges)
         Call Share.Room.Record(Room) ' write room record
      Case 0 ' treasure fixed in inventory
         ' reset inventory treasure charges
         Call PutUserRecordCharges(Array.Number, Item.Charges)
   End Select ' end treasure fixed charges
   Outpt = "You fix it!" ' make message
   Call IO.O ' send message
End Sub ' end fix routine

Rem * routine to drink an item.

Sub Drink.Potion
   Call Check.Inventory.Treasure ' check treasure mnemonic
   If Index.Number = False Then ' inventory treasure index
      Outpt = "You can't drink that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end treasure index
   If TreasureRecord.Potion = False Then ' compare treasure to potion
      Outpt = "That's not a potion!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare potion
   ' check inventory charges remaining
   If GetUserRecordCharges(Array.Number) = False Then
      Outpt = "You can't, it's empty!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare charges
   Spell.Number = TreasureRecord.Spell ' get treasure spell number
   ' check spell file
   If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
      Call Read.Record(SpellFile, Spell.Number) ' get spell record
      ' reduce charges
      Call PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
      Magic.Spell = SpellRecord.SpellType ' store spell type
      Multiplier = SpellRecord.Level ' store spell level
      Monster.Number = False ' monster not being attacked
      Parser = False ' no extra parameters
      Index.Number = False ' not cast on an object
      Outpt = "You drink " + Outpts + "!" ' format message
      Call IO.O ' send message
      Call Magic(True) ' cast the spell
      Exit Sub ' exit routine
   End If ' end check spell file
   Outpt = "You can't drink that!" ' make message
   Call IO.O ' send message
End Sub ' end drink potion

Rem * routine to eat an item.

Sub Eat.Object
   Call Check.Inventory.Treasure ' check treasure mnemonic
   If Index.Number = False Then ' inventory treasure index
      Outpt = "You can't eat that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end treasure index
   If TreasureRecord.Edible = False Then ' compare treasure to food
      Outpt = "That's not edible!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare food
   ' check inventory charges remaining
   If GetUserRecordCharges(Array.Number) = False Then
      Outpt = "You can't, it's been eaten!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare charges
   Spell.Number = TreasureRecord.Spell ' get treasure spell number
   ' check spell file
   If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
      Call Read.Record(SpellFile, Spell.Number) ' get spell record
      ' reduce charges
      Call PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
      Magic.Spell = SpellRecord.SpellType ' store spell type
      Multiplier = SpellRecord.Level ' store spell level
      Monster.Number = False ' monster not being attacked
      Parser = False ' no extra parameters
      Index.Number = False ' not cast on a target
      Outpt = "You eat " + Outpts + "!" ' format message
      Call IO.O ' send message
      Call Magic(True) ' cast the spell
      Exit Sub ' exit routine
   End If ' end check spell file
   Outpt = "You can't eat that!" ' make message
   Call IO.O ' send message
End Sub ' end eat routine

Rem * routine to load device with charges from an item, loads device on
Rem * ground or player inventory, removes item from from room or inventory.

Sub Load.Object
   Call Examine.Treasure ' check treasure mnemonic
   If Index.Number = False Then ' found treasure index
      Outpt = "You can't load that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end treasure index
   Load.Type = False ' set load type flag
   If TreasureRecord.Loadable = True Then ' treasure can be loaded
      ' store treasure index which loads device
      Device.Type = TreasureRecord.AmmoLoads
      Load.Type = True ' set load type flag
   End If ' end compare loaded treasure
   If TreasureRecord.Launchable = True Then ' treasure can be launched
      ' store treasure index which launchs item
      Device.Type = TreasureRecord.LaunchLoads
      Load.Type = 1 ' set load type flag
   End If ' end compare launched treasure
   If Load.Type = False Then ' compare load type flag
      Outpt = "You can't load that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare load type
   ' check index bounds
   If Device.Type < False Or Device.Type > LOF(TreasureFile) / Len(TreasureRecord) Then
      Outpt = "There's no ammunition!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check index bounds
   Charges.Number = False ' set treasure ammunition flag
   Call Read.Record(TreasureFile, Device.Type) ' get treasure index of ammunition
   If Load.Type = True Then ' check load type
      If TreasureRecord.Ammunition Then ' treasure index ammunition found
         Charges.Number = True ' set treasure ammunition flag
      End If ' end compare ammunition index
   End If ' end load type
   If Load.Type = 1 Then ' check load type
      If TreasureRecord.LaunchAmmo Then ' treasure index ammunition found
         Charges.Number = True ' set treasure ammunition flag
      End If ' end compare ammunition index
   End If ' end load type
   If Charges.Number = False Then ' check ammunition flag
      Outpt = "There's no ammunition!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check ammunition
   Ammo.Used = False ' set ammunition used flag
   For Inventory.Number = 1 To 20 ' loop through all player inventory
      ' compare inventory index to ammo index
      If Device.Type = GetUserRecordInv(Inventory.Number) Then
         ' remove ammo from inventory
         Call Discard.Inventory(Inventory.Number, True)
         Ammo.Used = True ' set ammunition used flag
         Exit For ' exit loop to find ammo index
      End If ' end compare inventory index
   Next ' end loop to find ammo index
   If Ammo.Used = False Then ' check ammunition used flag
      For Inventory.Number = 1 To 20 ' loop through all room inventory
         ' compare room inventory to
         If Device.Type = GetRoomRecordTreasure(Inventory.Number) Then
            ' ammo index, remove room ammo
            Call Discard.Room.Treasure(Inventory.Number)
            Ammo.Used = True ' set ammunition used flag
            Exit For ' exit loop to find ammo index
         End If ' end compare inventory index
      Next ' end loop to find ammo index
   End If ' end ammunition used flag
   If Ammo.Used = False Then ' check ammunition used flag
      Outpt = "There's no ammunition!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check ammunition used flag
   Device.Loaded = False ' set ammunition loaded flag
   Select Case TypeNumber ' select room or inventory
      Case 0 ' treasure loaded in player inventory
         ' store increment charges
         Device.Charges = GetUserRecordCharges(Array.Number) + 1
         If Device.Charges > 10 Then ' compare to maximum charges
            Device.Loaded = True ' set ammunition loaded flag
         Else ' compare less than maximum
            Call PutUserRecordCharges(Array.Number, Device.Charges) ' increment charges
         End If ' end compare maximum charges
      Case 1 ' treasure loaded in room inventory
         ' store increment charges
         Device.Charges = GetRoomRecordTreCharges(Array.Number) + 1
         If Device.Charges > 10 Then ' compare to maximum charges
            Device.Loaded = True ' set ammunition loaded flag
         Else ' compare less than maximum
            Call PutRoomRecordTreCharges(Array.Number, Device.Charges) ' increment charges
         End If ' end compare maximum charges
         Call Share.Room.Record(Room) ' write room record
   End Select ' end compare inventory to load
   If Device.Loaded Then ' compare ammunition loaded flag
      Outpt = "The device can't load that much!" ' make message
   Else ' ammunition loaded
      Outpt = "You load the device!" ' make message
   End If ' end compare ammunition loaded flag
   Call IO.O ' send message
End Sub ' end routine to load ammunition to an item

Rem * routine to fire a device at a monster in room, device explodes in
Rem * player by chance, device discharges harmlessly.

Sub Fire.Object
   Call Find.Inventory ' check player inventory for mnemonic of device to fire
   If Index.Number = False Then ' found treasure index to player inventory
      Outpt = "You can't fire that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check treasure index
   If TreasureRecord.Loadable = False Then ' check device can be loaded
      Outpt = "You can't fire that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare device
   If TreasureRecord.AmmoLoads = False Then ' check item which loads device
      Outpt = "It's not loaded with ammunition!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare device
   Call Read.Record(TreasureFile, TreasureRecord.AmmoLoads) ' get record ammo used
   If TreasureRecord.Ammunition = False Then ' item which loads device is nto ammo
      Outpt = "It's not loaded with ammunition!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare device
   ' read treasure record index of device to fire
   Call Read.Record(TreasureFile, Index.Number)
   If Parser = False Then ' check second parameter of monster to fire at
      Outpt = "Fire at what?" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare monster
   ' compare remaining charges in device
   If GetUserRecordCharges(Array.Number) = False Then
      Outpt = "It's not loaded!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare device
   ' decrement charges
   Call PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
   Parsed.Command1 = Parsed.Command2 ' get second parameter of monster
   Call Numeric ' parse out temp variable
   If Rnd < .2 Then ' compute random chance
      Outpt = "Bang! The ammunition explodes in your face!" ' device backfires
      Call IO.O ' send message
      Outpt = "The device hits you for" ' string preceding points hit on player
      Hits# = CDbl(Int(Rnd * Abs(TreasureRecord.Plus) + 1)) ' calculate random hits
      Call Hit.Player(Hits#) ' routine to get hits on player
      Exit Sub ' exit routine
   End If ' end random chance
   If Rnd < .2 Then ' random chance
      Outpt = "The device fires harmlessly!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end random chance
   Treasure.Name$ = TreasureRecord.ShortName ' store treasure name being fired
   Treasure.Name$ = RTrim$(Treasure.Name$) ' strip trailing blanks
   Treasure.Name$ = LCase$(Treasure.Name$) ' format to lower case
   Call Check.Monster ' find monster being fired at in parsed string
   If Monster.Number Then ' monster found from second parameter
      Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
      Outpts = RTrim$(Outpts) ' strip trailing blanks
      Outpts = LCase$(Outpts) ' format to lower case
      Outpt = "You fire the " + Treasure.Name$ + " at the " + Outpts + "!" ' format message
      Call IO.O ' send message
      Multiplier = Abs(TreasureRecord.Plus) ' calculate hits on monster multiplier
      Call Hit.Monster ' routine to attack monster in Monster.Number
      Exit Sub ' exit routine
   End If ' end compare monster to fire at
   Outpt = "The device fires into empty air!" ' make message
   Call IO.O ' send message
End Sub ' end routine to fire a device at a monster

Rem * dm/sysop routine to get an item from treasure or object files and
Rem * add to room, interactive function during remote logon also.

Sub Drop.Object
   Parsed.Command1 = Stored.Parsed.Command2 ' get entire second half of command
   Call Numeric ' parse temp variable
   Inpt = Parsed.Command1 ' put full name of item in variable
   Inpt = LCase$(Inpt) ' convert to lower case
   Call Drop(True) ' routine to put item name in room
End Sub ' end dm/sysop drop routine

Rem * routine to bless an object by increasing the magical charges.

Sub Bless.Object
   If Normal.User Then ' check player is normal player
      If UserRecord.ClassType <> Cleric Then ' compare player class type to cleric
         Outpt = "Only clerics can bless!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   Call Examine.Treasure ' check treasure mnemonic
   If Index.Number Then ' found treasure index
      Charges.Number = TreasureRecord.Spell ' store treasure spell type
      If Charges.Number = False Then ' compare treasure spell type
         Outpt = "You can't bless that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare treasure type
      ' check file bounds
      If Charges.Number > False And Charges.Number < LOF(SpellFile) / Len(SpellRecord) Then
         ' read spell record from treasure index
         Call Read.Record(SpellFile, Charges.Number)
         If SpellRecord.SpellType = 4 Then ' compare spell to wish
            Outpt = "You can't bless that!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' and compare spell
         Object.Blessed = False ' set object blessed flag
         Max.Charges = TreasureRecord.Charges ' store maximum charges
         Select Case TypeNumber ' select room or inventory
            Case 0 ' bless item in player inventory
               ' get incremented charges
               TypeCharges = GetUserRecordCharges(Array.Number) + 1
               If TypeCharges > Max.Charges Then ' compare maximum charges
                  Object.Blessed = True ' set bless flag
               Else ' compare charges
                  ' set incremented charges
                  Call PutUserRecordCharges(Array.Number, TypeCharges)
               End If ' end compare maximum charges
            Case 1 ' bless item in room inventory
               ' get incremented charges
               TypeCharges = GetRoomRecordTreCharges(Array.Number) + 1
               If TypeCharges > Max.Charges Then ' compare maximum charges
                  Object.Blessed = True ' set bless flag
               Else ' compare charges
                  ' set incremented charges
                  Call PutRoomRecordTreCharges(Array.Number, TypeCharges)
                  Call Share.Room.Record(Room) ' write room record
               End If ' end compare maximum charges
         End Select ' end inventory type
         If Object.Blessed Then ' compare bless flag
            Outpt = "Nothing happens!" ' make message
         Else ' compare bless flag
            Outpt = "You bless it!" ' make message
         End If ' end compare flag
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If
   End If
   Outpt = "You can't bless that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to bless an object

Rem * routine to curse a monster by decreasing hits.

Sub Curse.Object
   If Normal.User Then ' check player type
      If UserRecord.ClassType <> Cleric Then ' check player class type to cleric
         Outpt = "Only clerics can curse!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   Call Check.Monster ' search for monster parameter
   If Monster.Number Then ' index found to monster array
      Outpts = MonsterArray(Monster.Number).MonsterName ' store full monster name
      Outpts = RTrim$(Outpts) ' strip trailing blanks
      Outpts = LCase$(Outpts) ' convert to lower case
      Outpt = "You curse the " + Outpts + "!" ' format message
      Call IO.O ' send message
      Multiplier = Int(GetUserRecordStats(6) / 2) ' compute hits on monster multiplier
      Call Hit.Monster ' routine to attack a monster
      Exit Sub ' exit routine
   End If ' end get monster index
   Outpt = "You can't curse that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to curse a monster

Rem * routine to bless player (bless without parameter).

Sub Bless.Self
   If Normal.User Then ' check player type
      If UserRecord.ClassType <> Cleric Then ' check player class type
         Outpt = "Only clerics can bless!" ' message for clerics only
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   TypeCharges = GetUserRecordStats(6) + 1 ' store incremented piety
   If TypeCharges > MaxStat Then ' compare to maximum piety
      Outpt = "Nothing happens!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare piety
   Call PutUserRecordStats(6, TypeCharges) ' update new piety
   Outpt = "You feel a glow about you!" ' make message
   Call IO.O ' send message
   Outpt = "Your piety is now" + Str$(TypeCharges) + "!" ' format message
   Call IO.O ' send message
End Sub ' end routine to bless self

Rem * routine to curse player (curse without parameter).

Sub Curse.Self
   If Normal.User Then ' check player type
      If UserRecord.ClassType <> Cleric Then ' check player class type
         Outpt = "Only clerics can curse!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   TypeCharges = GetUserRecordStats(6) - 1 ' store decremented piety
   If TypeCharges <= False Then ' compare to minimum piety
      Outpt = "Nothing happens!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare piety
   Call PutUserRecordStats(6, TypeCharges) ' set new piety
   Outpt = "You feel a darkening about you!" ' make message
   Call IO.O ' send message
   Outpt = "Your piety is now" + Str$(TypeCharges) + "!" ' format message
   Call IO.O ' send message
End Sub ' end routine to curse self

Rem * routine to move a lanuch type object to a nearby room and move player
Rem * to room also, verify room traps.

Sub Move.Object
   Call Parse ' parse out parameters
   If Parser Then ' check for first parameter
      Call Numeric '  parse temp variable from first parameter
   End If ' end check parameter
   Call Examine.Treasure ' check treasure mnemonic in inventory or room
   If Index.Number = False Then ' locate treasure index
      Outpt = "You can't move that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If TreasureRecord.Movable = False Then ' compare treasure can be moved
      Outpt = "You can't move that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If TreasureRecord.Launchable = False Then ' compare treasure can be launched
      Outpt = "You can't move that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If Parser = False Then ' compare parameter parsed
      Outpt = "Move which direction?" ' no direction specified
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check parse
   Treasure.Name$ = Outpts ' store full treasure name to be moved
   Item.Number = Array.Number ' store treasure index to array
   Item.Charges = Charges.Number ' store treasure charges
   Item.Index = Index.Number ' store treasure index to file
   User.Command = Parsed.Command2 ' get second parameter
   Call Get.Direction(Direction.Number) ' compare moving parameter to direction
   If Direction.Number Then ' direction found flag set
      Next.Room = GetRoomRecordDirect(Direction.Number) ' get room number direction
      Outpts = RTrim$(Direction(Direction.Number)) ' store direction name
      If Next.Room = False Then ' compare room number
         Outpt = "You can't move it there!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare room number
   Else ' no direction flag set, check for moving through an object
      Parsed.Command1 = User.Command ' get second parameter
      Call Check.Room.Objects ' check with mnemonic name of object in room
      If Index.Number = False Then ' check object index
         Outpt = "You can't move it there!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare room object
      If ObjectRecord.RoomLink = False Then ' verify room object links to room
         Outpt = "You can't move it there!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare object is a portal
      If ObjectRecord.JailTrap Then ' verify room is trapped
         Outpt = "Trapped portal!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare room trap
      If ObjectRecord.Closed Then ' compare room closed
         If Pass.Door = False Then ' check is passdoor spell cast
            Outpt = "You can't, it's closed!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end cmpare passdoor
      End If ' end compare object closed
      If ObjectRecord.Relocks Then ' object portal closes after entry
         ObjectRecord.DoorLock = 2 ' set object portal lock
         ObjectRecord.Closed = True ' set object portal closed
         Call Share.Record(ObjectFile, Index.Number) ' write object record
      End If ' end compare relocking object portal
      Pass.Door = False ' reset passdoor spell
      Next.Room = ObjectRecord.RoomLink ' get room number to enter portal
      Outpts = "into a " + RTrim$(LCase$(ObjectRecord.ShortName)) ' store object name
   End If ' end compare move to a direction or to a portal
   Swap Room, Next.Room ' interchange current room with next room to move item
   Call Read.Room.Record(Room) ' read next room to move item to room file
   ' routine to add room item
   Call Add.Room.Treasure(Item.Index, Item.Charges, False, Item.Added)
   Swap Room, Next.Room ' interchange current room with next room
   Call Read.Room.Record(Room) ' read current room record
   If Item.Added Then ' add item to room flag
      Select Case TypeNumber ' select room or inventory
         Case 0 ' treasure moved from player inventory
            Call Discard.Inventory(Item.Number, True) ' remove item from inventory
         Case 1 ' treasure moved from room
            Call Discard.Room.Treasure(Item.Number) ' remove item from room
      End Select ' end compare treasure moved
      Outpt = "You move " + Treasure.Name$ + " " + Outpts + "!" ' format message
      Call IO.O ' send message
      Call Enter.Room ' move player to next room
      Exit Sub ' exit routine
   End If ' end locate empty room inventory
   Outpt = "You can't move it there!" ' make message
   Call IO.O ' send message
End Sub ' end routine to move an item

Rem * routine to launch an item in a specific direction to smash through
Rem * a random amount of object portal locks connected through successive
Rem * rooms, or to launch a weapon at a monster. weapon ammunition lands
Rem * in last room accessed.

Sub Launch.Object
   Call Parse ' get first parameter
   If Parser Then ' found parameter
      Call Numeric ' parse number after # sign
   End If ' end compare parse
   Call Examine.Treasure ' check treasure mnemonic to room or player inventory
   If Index.Number = False Then ' treasure index not found
      Outpt = "You can't launch that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare index
   If TreasureRecord.Launchable = False Then ' check if treasure can be launched
      Outpt = "That's not a launchable device!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare item to launch
   If Parser = False Then ' compare second parameter
      Outpt = "Launch where?" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare second parameter
   Item.Launched = False ' item was launched flag
   Item.Type = TypeNumber ' store inventory type
   Item.Index = Array.Number ' store launched item index to array
   Item.Charges = Charges.Number ' store launched item charges
   Launch.Item = TreasureRecord.LaunchLoads ' store item launched
   ' check item launch treasure record
   If Launch.Item > False And Launch.Item <= LOF(TreasureFile) / Len(TreasureRecord) Then
      Call Read.Record(TreasureFile, Launch.Item) ' read item launched record
      Call TreasureCharges(Launch.Charges) ' store item launched charges
   End If ' end check item launched number
   Call Read.Record(TreasureFile, Index.Number) ' reread launch item
   If Charges.Number = False Then ' compare remaining charges in device
      Outpt = "You can't, it's not loaded!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare device
   If Item.Charges > 10 Then ' compare device charges
      Item.Charges = 10 ' reset maximum charges
   End If ' end compare device
   Launch.Number = False ' variable of number of loads launched
   Treasure.Name$ = TreasureRecord.ShortName ' store name of device
   Treasure.Name$ = RTrim$(Treasure.Name$) ' strip trailing blanks
   Treasure.Name$ = LCase$(Treasure.Name$) ' convert to lower case
   User.Command = Parsed.Command2 ' get second parameter
   Parsed.Command1 = User.Command ' store parameter
   Call Get.Direction(Direction.Number) ' determine direction to launch at
   If Direction.Number Then ' returned variable contains direction number
      Store.Room! = GetRoomRecordDirect(Direction.Number) ' get next room direction
      If Store.Room! = SFalse Then ' compare room number
         Outpt = "You can't launch at that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare room number
      Item.Launched = True ' set item is launched flag
      Outpts = Direction(Direction.Number) ' store direction name
      Outpts = RTrim$(Outpts) ' strip trailing blanks
      Outpts = LCase$(Outpts) ' convert to lower case
      Outpt = "You launch the " + Treasure.Name$ + " " + Outpts + "!" ' format message
      Call IO.O ' send message
      TempZ! = Room ' store current room.
      ' loop through all charges in device to launch
      For Launchs = 1 To Item.Charges
         Graphics.Off = False ' color normal
         Outpt = "For launch" + Str$(Launchs) + ";" ' format message of launch number
         Call IO.O ' send message
         Launch.Number = Launch.Number + 1 ' increment number of launches
         Graphics.Off = True ' color white
         Outpt = "The device launchs: " ' message
         Carriage.Return = True ' suppress cr/lf
         Call IO.O ' send message
         Outpt = Outpts + ", " ' first direction device launches
         Carriage.Return = True ' suppress cr/lf
         Call IO.O ' send direction message
         Locks.Smashed = False ' counter of number of locks smashed
         Chained.Rooms = False ' number of rooms chained to
         Next.Room = Store.Room! ' store room number
         Call Chain.Rooms(Next.Room, Room) ' routine to send launched item
         Outpt = "and stopped!" ' trailing message
         Call IO.O ' send message
         ' place launched item in last room
         Room = Last.Room ' reset current room
         Call Read.Room.Record(Room) ' read room
         ' add ammunition
         Call Add.Room.Treasure(Launch.Item, Launch.Charges, False, Added)
         Room = TempZ! ' restore current room
         Call Read.Room.Record(Room) ' restore room record
      Next ' end loop through all device launch charges
      If Locks.Smashed = False Then ' compare number of locks smashed
         Outpt = "You didn't smash anything!" ' make message
      Else ' compare lock number
         Outpt = "You smashed open" ' format message
         Outpt = Outpt + Str$(Locks.Smashed) ' with number of locks smashed
         If Locks.Smashed = 1 Then ' compare number
            Outpt = Outpt + " lock.." ' append message
         Else ' compare number
            Outpt = Outpt + " locks!" ' append message
         End If ' end compare
      End If ' end compare number
      Call IO.O ' send message
      Graphics.Off = False ' color normal
      Call Read.Room.Record(Room) ' read current room record
   Else ' verify monster to launch at
      Call Check.Monster ' find monster mnemonic to launch device at
      If Monster.Number Then ' found monster variable indexed to array
         Multiplier = Abs(TreasureRecord.Plus) ' compute attack multiplier
         ' store full monster name
         Outpts = MonsterArray(Monster.Number).MonsterName
         Outpts = RTrim$(Outpts) ' strip trailing blanks
         Outpts = LCase$(Outpts) ' convert to lower case
         Outpt = "You launch " + Treasure.Name$ + " at the " + Outpts + "!" ' make message
         Call IO.O ' send message
         Item.Launched = True ' set item is launched flag
         Graphics.Off = True ' color white
         ' loop through all charges in device to launch
         For Launchs = 1 To Item.Charges
            ' format message of launch number
            Outpt = "For launch" + Str$(Launchs) + ";"
            Call IO.O ' send message
            Launch.Number = Launch.Number + 1 ' increment number of launches
            Call Attack.Monster ' routine to hit monster
            Call Check.Monster ' reverify monster being attacked
            ' place launched item in room
            Call Add.Room.Treasure(Launch.Item, Launch.Charges, False, Added)
            If Monster.Number = False Then ' monster is not found
               Exit For ' exit device launch loop
            End If ' end compare monster
         Next ' end loop through all device launch charges
         Graphics.Off = False ' color normal
      End If ' end find monster
   End If ' end compare launch direction or monster
   If Item.Launched = False Then ' device was launched flag
      Outpt = "You can't launch at that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare if launched
   If Item.Type = False Then ' device launched from player inventory
      ' compute loads launched decrement
      Total.Charges = GetUserRecordCharges(Item.Index) - Launch.Number
      If Total.Charges < False Then ' compare lower limit
         Total.Charges = False ' set lower limit
      End If ' end compare limit
      Call PutUserRecordCharges(Item.Index, Total.Charges) ' store inventory charges
   Else ' device launched from room
      ' compute loads launched decrement
      Total.Charges = GetRoomRecordTreCharges(Item.Index) - Launch.Number
      If Total.Charges < False Then ' compare lower limit
         Total.Charges = False ' set lower limit
      End If ' end compare limit
      Call PutRoomRecordTreCharges(Item.Index, Total.Charges) ' store room charges
      Call Share.Room.Record(Room) ' write room record
   End If ' end compare launched from
End Sub ' end routine to launch an item from a device

Rem * routine to recursively enter rooms by nearby room links, smashing a
Rem * random number of portals.

Sub Chain.Rooms (Chain.Room!, Previous.Room!)
   ' Chain.Room! is a passed parameter of the next room to access.
   ' Previous.Room! stores the called room number to be restored upon exit.
   Last.Room = Chain.Room! ' store last room recursed into
   Call Read.Room.Record(Chain.Room!) ' get next chained to room
   For Array.Number = 1 To 20 ' loop through all objects in room
      Object.Number = GetRoomRecordObject(Array.Number) ' get object index to file
      ' check object file bounds
      If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
         Call Read.Record(ObjectFile, Object.Number) ' get the object record
         If ObjectRecord.DoorLock = 2 Then ' check if object is locked
            If Rnd > .66 Then ' compute random chance
               Locks.Smashed = Locks.Smashed + 1 ' increment locks smashed counter
               ObjectRecord.DoorLock = 1 ' set object unlocked
               ObjectRecord.Closed = False ' set object to open
               Call Share.Record(ObjectFile, Object.Number) ' write object record
            End If ' end random chance
         End If ' end compare object lock
      End If ' end end object bounds
   Next ' end loop through all objects
   For Direction.Number = 1 To 12 ' loop through all directions in chained to room
      Next.Room = GetRoomRecordDirect(Direction.Number) ' get the room number of the
      If Next.Room Then ' direction and compare to
         If Next.Room <> Previous.Room! Then ' the current room in routine
            If Rnd > .66 Then ' compute random chance
               If Chained.Rooms < 5 Then ' compare to maximum of 5 recursive calls
                  Carriage.Return = True ' suppress cr/lf
                  Outpt = Direction(Direction.Number) ' get direction name
                  Outpt = RTrim$(Outpt) + ", " ' format message
                  Call IO.O ' send message
                  ' increment number of recursively chained rooms
                  Chained.Rooms = Chained.Rooms + 1
                  Call Chain.Rooms(Next.Room, Chain.Room!) ' routine calls self
                  Call Read.Room.Record(Chain.Room!) ' next room gets previous room
                  Exit Sub ' recursively exit previous calls
               End If ' end compare maximum
            End If ' end random chance
         End If ' end compare to current room
      End If ' end compare next direction
   Next ' end loop through all directions in called room (exit recursively)
End Sub ' end routine to recursively loop through all objects in neraby rooms

Rem * routine for player to steal an item from a monster.

Sub Steal.Object
   If Normal.User Then ' check player type
      If UserRecord.ClassType <> Thief Then ' check player class type
         Outpt = "Only thieves can steal!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   Call ParseX ' parse the second parameter
   If Parser Then ' parse the
      Call Numeric ' number after the # sign
   End If ' end parse
   If Parser = False Then ' no second parameter
      Outpt = "Steal from whom?" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare parse
   Call Check.Monster ' verify monster mnemonic
   If Monster.Number Then ' monster index
      Parsed.Command1 = LCase$(Parsed.Command2) ' get first parameter
      Call Numeric ' parse number
      For Array.Number = 1 To 5 ' loop through all monster treasure
         ' get monster treasure index
         Treasure.Number = GetMonsterRecordTreasure(Array.Number)
         ' check file bounds
         If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
            Call Read.Record(TreasureFile, Treasure.Number) 'read treasure record
            Inpt = TreasureRecord.ShortName ' store short treasure mnemonic
            Inpt = RTrim$(Inpt) ' strip trailing blanks
            Inpt = LCase$(Inpt) ' convert to lower case
            Inpt = Left$(Inpt, Len(Parsed.Command1)) ' shorten to parsed length
            If Inpt = Parsed.Command1 Then ' compare treasure to parsed treasure
               ' compare monster to magical, or permanent
               If MonsterArray(Monster.Number).Magic <= True Or MonsterArray(Monster.Number).Permanent <= True Then
                  Outpt = "You can't steal from that monster!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end compare monster
               ' compute random chance
               If Int(Rnd * 5 + 5) > GetUserRecordStats(4) / 2 Then
                  Outpt = "Didn't work!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end random chance
               Inpt = TreasureRecord.TreasureName ' get full name of treasure
               Inpt = RTrim$(Inpt) ' strip trailing blanks
               Inpt = LCase$(Inpt) ' convert to lower case
               ' get monster name
               Outpts = MonsterArray(Monster.Number).MonsterName
               Outpts = RTrim$(Outpts) ' strip trailing blanks
               Outpts = LCase$(Outpts) ' convert to lower case
               Item.Charges = TreasureRecord.Charges ' store charges
               ' add to player inventory
               Call Add.Inventory(Treasure.Number, Item.Charges, Item.Added)
               If Item.Added Then ' flag set to add inventory
                  Outpt = "You steal " + Inpt + " from the " + Outpts + "!" ' make message
                  Call IO.O ' send message
                  ' clear element in monster inventory array
                  Call PutMonsterRecordTreasure(Array.Number, False)
                  Exit Sub ' exit routine
               End If ' end compare flag
            End If ' end compare treasure
         End If ' end compare treasure
      Next ' end loop through all monster inventory
   End If ' end find monster number
   Outpt = "You can't steal that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to steal

Rem * routine to throw an item of player inventory at monster, weapon falls
Rem * into the room.

Sub Throw.Object
   If Normal.User Then ' verify player type
      If UserRecord.ClassType <> Thief Then ' verify player class type
         Outpt = "Only thieves can throw!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare class type
   End If ' end compare player type
   Call ParseX ' get second parameter
   If Parser Then ' parse parameter
      Call Numeric ' store number
   End If ' end compare parse
   If Parser = False Then ' missing second parameter
      Outpt = "Throw at whom?" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare parameter
   Call Check.Monster ' find monster in room
   If Monster.Number Then ' found monster index
      Last.Monster = Parsed.Command1 ' reset last monster attacked
      Parsed.Command1 = UCase$(Parsed.Command2) ' get first parameter
      Call Numeric ' parse number sign
      Call Check.Inventory.Treasure ' search player inventory for treasure
      If Index.Number > False Then ' found treasure index in player inventory
         ' weapon being held must match the weapon thrown
         If Weapon6 = Array.Number Then
            Multiplier = Weapon2 ' set attack multiplier
            Inpt = MonsterArray(Monster.Number).MonsterName ' get monster name
            Inpt = RTrim$(Inpt) ' strip trailing blanks
            Inpt = LCase$(Inpt) ' convert to lower case
            Outpt = "You throw " + Outpts + " at the " + Inpt + "!" ' format message
            Call IO.O ' send message
            If Int(Rnd * 5 + 2) > GetUserRecordStats(4) / 2 Then ' compute random chance
               Outpt = "You missed!" ' make message
               Call IO.O ' send message
            Else ' random chance
               Parsed.Command1 = Last.Monster ' reset parameter to last monster
               Last.Command.Number = 55 ' reset the command number to attack
               Call Attack.Monster ' routine to hit monster
            End If ' end random chance
            Call Discard.Inventory(Weapon6, False) ' remove item from inventory
            Exit Sub ' exit routine
         End If ' end compare weapon being held
         Outpt = "You aren't holding the weapon!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end search for weapon to throw
   End If ' end search for monster
   Outpt = "You can't throw that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to throw an object

Rem * routine to fuel an item from the charges of another item.

Sub Fuel.Object
   Call Find.Inventory ' parse first parameter, search player inventory
   Treasure.Name$ = TreasureRecord.ShortName ' store treasure short mnemonic name
   Treasure.Name$ = RTrim$(Treasure.Name$) ' strip trailing blanks
   Treasure.Name$ = LCase$(Treasure.Name$) ' convert to lower case
   If Index.Number = False Then ' compare treasure index
      Outpt = "You can't fuel that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare index
   If TreasureRecord.LightType = False Then ' check treasure can be fueled
      Outpt = "You can't fuel that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If TreasureRecord.LightCharges = False Then ' check treasure can be fueled
      Outpt = "It has no light charges!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If GetUserRecordCharges(Array.Number) Then ' check treasure already fueled
      Outpt = "It's already fueled!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   Fuel.Charges = TreasureRecord.LightCharges ' store treasure light charges
   Item.Number = Array.Number ' store treasure array index
   If Parser = False Then ' compare second parameter
      Outpt = "Fuel with what?" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare parse
   Parsed.Command1 = Parsed.Command2 ' get second parameter
   Call Numeric ' parse number
   Call Find.Inventory ' search player inventory
   If Index.Number = False Then ' compare treasure index
      Outpt = "You can't fuel with that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare index
   If TreasureRecord.FuelType = False Then ' check treasure is a fuel
      Outpt = "You can't fuel with that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If TreasureRecord.FuelCharges <= False Then ' check treasure fuel amount
      Outpt = "It has no fuel charges!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If GetUserRecordCharges(Array.Number) = False Then ' check fuel charges
      Outpt = "It has no fuel charges!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   ' decrement fuel
   Call PutUserRecordCharges(Array.Number, GetUserRecordCharges(Array.Number) - 1)
   Call PutUserRecordCharges(Item.Number, Fuel.Charges) ' add fuel charges to treasure
   Outpt = "You fuel the " + Treasure.Name$ + "!" ' format message
   Call IO.O ' send message
End Sub ' end routine to fuel an item

Rem * routine to light an item.

Sub Light.Object
   Call Check.Inventory.Treasure ' search for player inventory treasure
   If Index.Number = False Then ' compare treasure index
      Outpt = "You can't light that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure index
   If TreasureRecord.LightType = False Then ' check treasure is a light
      Outpt = "That's not a light!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   If GetUserRecordCharges(Array.Number) = False Then ' check treasure is fueled
      Outpt = "You can't, it's not fueled!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   ' check treasure is already lit
   If GetUserRecordCharges(Array.Number) < False Then
      Outpt = "It's already lit!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare treasure
   ' set inventory charges
   Var = GetUserRecordCharges(Array.Number): Var = -Var
   Call PutUserRecordCharges(Array.Number, Var)
   Outpts = TreasureRecord.ShortName ' to negative to indicate treasure is a light
   Outpts = RTrim$(Outpts) ' store treasure name, strip trailing blanks
   Outpts = LCase$(Outpts) ' convert to lower case
   Outpt = "You light the " + Outpts + "!" ' format message
   Call IO.O ' send message
End Sub ' end routine to light an item

Rem * dm/sysop routine to discard an item of inventory to nowhere, can be
Rem * used interactively during remote with function key.

Sub Discard.Object
   Call Check.Inventory.Treasure ' find treasure in player inventory
   If Index.Number Then ' treasure index to player array
      ' routine to remove from inventory
      Call Discard.Inventory(Array.Number, True)
      Outpt = "You discard " + Outpts + "!" ' format message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end search treasure inventory
   Call Num ' decrement parse counter
   Call Check.Inventory.Objects ' find object in player inventory
   If Index.Number Then ' object index to player array
      Call Discard.Inventory.Object(Array.Number)
      Outpt = "You discard " + Outpts + "!" ' format message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end search object inventory
   Call Num ' decrement parse temp
   Call Check.Inventory.Container ' check player inventory for container
   If Index.Number Then ' returns index of container to treasure
      Outpts = RTrim$(ContainerRec.ContainerName) ' message
      Call Clear.Container(0, True) ' clear player container structure
      Call PutUserRecordContainer(Array.Number) ' clear container record
      Outpt = "You discard " + Outpts + "!" ' format message
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check container index
   Outpt = "You can't discard that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to discard an item

Rem * Filename: dnds2.bas Version: v5.0a r2.0a
Rem * This subprogram contains most login routines, and parse routines.

Rem * routine to login user.
Rem * output variables:
Rem *   Time.On - contains time logged in.
Rem *   Timeon - contains time logged in past midnight.
Rem *   Time.Left - stores time limit in seconds past midnight.
Rem *   User.Index - number of record in user file codename will use.
Rem * processing variables:
Rem *   Time.Left - during logging in.
Rem *   Two.Minutes.Left - time remaining flag.
Rem *   Login.Try - total login attempts.
Rem *   New.User - flag indicates new user login.
Rem *   Logged.User - flag indicates user exists in files.

Sub Login
   Time.On = Time$ ' store system time logged in (string form hh:mm:ss)
   Timeon = Timer ' store system time logged in (seconds past midnight)
   Timelogged.On = Time$ ' store time logged in (double format)
   Time.Left = 600! ' set time limit during logging in
   Two.Minutes.Left = False ' flag for two minute left display message
   Logged.In = False ' set user logged in flag
   ' format initial login display line
   Outpt = "The Adventure Door v" + Version$ + ", Node: " + Node + ", " + clock$
   If Local.Mode = False Then
      Outpt = Outpt + ", baud" + Str$(Modem.Baud) + "00"
   End If
   Outpt = Outpt + "."
   Call IO.O ' display initial message
   ' format prompt for login welcome
   Outpt = "Press <enter> to display the welcome, or <space> to skip:"
   No.Echo = True ' supress prompt input echo
   Line.Length = 1 ' get only one keypress
   Call IO.I ' input routine
   No.Echo = False ' reset echo
   If Inpt = Nul Then ' check empty return
      Call Out.File("welcome.dat") ' display welcome file
   End If ' end display welcome
   Do ' main login processing loop
      Login.Try = False ' reset login attempts
      Do ' get codename processing loop
         New.User = False ' reset new user flag
         Call Get.Codename ' prompt user for codename
         Call Get.PassWord ' prompt user for password
         Call Find.PassWord(Logged.User) ' find codename in user file
         If Logged.User Then ' compare user has entered an existing codename
            Call Verify.PassWord(Logged.Pass) ' compare entered password
            If Logged.Pass Then ' check if password matches
               Exit Do ' exit codename processing/entry loop
            End If ' end check password match
            If Login.Try >= 3 Then ' compare login attempts
               Call Hang.Up(7) ' routine to terminate program w/ message
               Exit Sub ' exit login routine/return to main
            End If ' end compare login attempts
            Login.Try = Login.Try + 1 ' increment login attempt
            Outpt = "Illegal password attempt!" ' display error message
            Call IO.O ' send output/continue codename entry loop
         Else ' user has not entered an existing codename
            Call Get.Newuser.Record ' find an empty user file record
            Call Verify.Newuser(New.User) ' verify password entered
            If New.User > False Then ' user selected disconnect
               Call Hang.Up(8) ' routine to terminate program w/ message
               Exit Sub ' exit routine
            End If ' end compare disconect
            If New.User < False Then ' user password verified
               Call Init.Newuser ' initialize some new user variables
               Call Verify.Newlogin(New.User, True) ' verify user is new
               If New.User Then ' user selects to continue as a new user
                  Exit Do ' exit codename/password entry loop
               End If ' end new user continue
            End If ' end password verify
         End If ' end compare existing codename
      Loop ' codename/password entry loop
      Call Update.Login ' intialize some login variables
      If New.User = False Then ' compare user is new user
         Exit Do ' exit main login processing loop
      End If ' end compare new user
      If New.User Then ' compare new user login is verified
         Call Roll.Character ' get user selected character statistics
         Call Verify.Newlogin(New.User, False) ' verify new user to continue
         If New.User Then ' new user is logged in
            Exit Do ' exit main login processing loop
         End If ' end compare new user login
      End If ' end compare new user login
   Loop ' end main login processing loop
   Call Login.User ' routine to initialize some login variables
   Outpt = Nul ' send empty output
   Call IO.O ' send output
   Outpt = "Press <enter> to begin the adventure:" ' format message
   No.Echo = True ' set flag to supress echo input
   Line.Length = 1 ' get one keypress
   Call IO.I ' get input
   No.Echo = False ' reset echo flag
   Next.Room = Room ' store room number
   Call Enter.Room ' display room description
   Logged.In = True ' set user logged in flag
   FunctionBuffer = Nul ' reset function key flag
End Sub ' end routine to login user

Rem * routine to find codename entered in user file.
Rem * output variables:
Rem *   PassWord.Found - flag if codename is in user file.

Sub Find.PassWord (PassWord.Found)
   PassWord.Found = False ' set flag to false
   Inpt = RTrim$(Player.CodeName) ' store codename
   For User.Index = 1 To LOF(UserFile) / Len(UserRecord) ' loop through entire user file
      Call Read.Record(UserFile, User.Index) ' get next user file record
      Outpt = UserRecord.CodeName ' store user file codename
      Call Decrypt(Outpt) ' decrypt user file codename
      Outpt = RTrim$(Outpt) ' trim user file codename
      If Outpt = Inpt Then ' compare user file codename to codename entered
         PassWord.Found = True ' set return variable flag
         Exit For ' exit user file loop
      End If ' end check codenames
   Next ' loop through user file
End Sub ' end routine to find user file codename

Rem * routine to verify valid password of codename entered.
Rem * output variables:
Rem *   PassWord.Found - flag indicates password exists.

Sub Verify.PassWord (PassWord.Found)
   PassWord.Found = False ' set flag to false
   Outpt = UserRecord.PassWord ' store user file password
   Call Decrypt(Outpt) ' decrypt password
   If Len(Outpt) = False Then ' verify decrypt result
      Outpt = "Password has a checksum error!" ' format message
      Call IO.O ' send message
      PassWord.Found = False ' set flag to verify password
      Exit Sub ' exit check password routine
   End If ' end check password
   Outpt = RTrim$(Outpt) ' store trimmed user file password
   Inpt = RTrim$(Player.PassWord) ' store password entered
   If Outpt = Inpt Then ' compare user file password to entered password
      PassWord.Found = True ' set flag to verify password
   End If ' end compare passwords
End Sub ' end routine to check valid password

Rem * routine to verify new user is continuing.
Rem * input variables:
Rem *   Message.Type - message to select.
Rem * output variables:
Rem *   Response.Type - returns true to continue, false if not.

Sub Verify.Newlogin (Response.Type, Message.Type)
   Do ' process input loop
      Graphics.Off = False ' reset color
      If Message.Type Then ' compare prompt
         Outpt = "Press <enter> to roll character, or <space> to reenter:"
      Else ' select prompt
         Outpt = "Press <enter> to use character, or <space> to reroll:"
      End If ' end compare prompt
      No.Echo = True ' supress input echo
      Line.Length = 1 ' input one keypress
      Call IO.I ' get user input
      No.Echo = False ' reset input echo
      If Inpt = " " Then ' selected space
         Response.Type = False ' set return flag
         Exit Do ' exit routine
      End If ' end compare select
      If No.Input Then ' compare empty input
         Response.Type = True ' set return flag
         Exit Do ' exit routine
      End If ' end compare select
   Loop ' process input loop
End Sub ' end routine to prompt to continue

Rem * routine to get codename, check illegal character in codename, verify
Rem * user has entered correct codename.
Rem * output variables:
Rem *   Player.CodeName - contains lowercased, trimmed codename entered.

Sub Get.Codename
   Do ' main codename entry processing loop
      Do ' loop until valid codename entered
         Outpt = Nul ' empty output
         Call IO.O ' send output
         Graphics.Off = True ' reset color
         Outpt = "         +---------+---------+---------+" ' make length bar
         Call IO.O ' send output
         Outpt = "Codename? " ' codename prompt
         Line.Length = 30 ' set line length of codename
         Upper.Case = True ' reset uppercase flag
         Call IO.I ' get codename input
         Upper.Case = False ' reset uppercase flag
         Graphics.Off = False ' reset color
         Inpt = LTrim$(Inpt) ' trim blanks
         Inpt = RTrim$(Inpt) ' trim blanks
         Inpt = UCase$(Inpt) ' set input to uppercase
         Player.CodeName = Inpt ' store codename
         If Len(Player.CodeName) > False Then ' check length of codename
            Call Valid(Player.CodeName, 30) ' verify valid characters
            If Len(Player.CodeName) > False Then ' check valid codename
               Exit Do ' exit codename entry loop
            End If ' end check valid characters
            Outpt = "Illegal characters in codename!" ' format message
            Call IO.O ' send output
         End If ' end check codename length
      Loop ' continue entry loop
      Outpt = RTrim$(Player.CodeName) ' store codename
      Outpt = LCase$(Outpt) ' set to lowercase
      Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' make first character uppercase
      Outpt = "You are " + Chr$(34) + Outpt + Chr$(34) + "(y/n)? " ' format prompt
      No.Input.Out = "Y" ' set default input to yes
      Line.Length = 1 ' reset line length
      Call IO.I ' prompt for correct codename entered
      If Yes Then ' verify user has verified correct codename
         Exit Do ' exit main codename entery loop
      End If ' end verify user input
   Loop ' end codename entry loop
End Sub ' end routine to get and verify codename entry

Rem * routine to prompt user for password, check valid characters in password.
Rem * output variables:
Rem *   Player.PassWord - contains lowercased, trimmed password entered.

Sub Get.PassWord
   Do ' password entry loop
      Outpt = Nul ' empty output
      Call IO.O ' send output
      Graphics.Off = True ' reset color
      Outpt = "         +---------+---------+" ' make length bar
      Call IO.O ' send output
      Outpt = "Password? " ' password prompt
      Line.Length = 20 ' set line length of password
      Hidden = True ' set flag to echo mask characters
      Call IO.I ' get input
      Hidden = False ' reset mask flag
      Graphics.Off = False ' reset color
      Inpt = LTrim$(Inpt) ' trim blanks
      Inpt = RTrim$(Inpt) ' trim blanks
      Inpt = UCase$(Inpt) ' set input to uppercase
      Player.PassWord = Inpt ' store password
      If Len(Player.PassWord) > False Then ' check length of password
         Call Valid(Player.PassWord, 20) ' verify valid characters
         If Len(Player.PassWord) > False Then ' check valid password
            Exit Do ' exit password entry loop
         End If ' end check valid characters
         Outpt = "Illegal characters in password!" ' format message
         Call IO.O ' send output
      End If ' end check password length
   Loop ' end password entry loop
End Sub ' end routine to enter password

Rem * routine to find empty user file record.
Rem * output variables:
Rem *    User.Index - unused number of record in user file.

Sub Get.Newuser.Record
   For User.Index = 1 To LOF(UserFile) / Len(UserRecord) ' loop through users
      Call Read.Record(UserFile, User.Index) ' load the user record
      Outpt = UserRecord.CodeName ' store user file codename
      Call Decrypt(Outpt) ' routine to decrypt codename
      If Left$(Outpt, 9) = Deleted$ Then ' compare to deleted record
         Exit For ' end loop through user file
      End If ' end compare deleted record
   Next ' end loop through user file
   ' exit of loop w/o finding a deleted record will set User.Index to one
   ' record past the last in the user file, appending the next record.
   Call Read.Record(UserFile, User.Index) ' get empty user file record
   Outpt = "Codename not found in files!" ' make message
   Call IO.O ' send message
End Sub ' end routine to find empty user file record

Rem * routine to verify new user login.
Rem * output variables:
Rem *   Response.Type - flag set to 1 to disconnect,
Rem *   0 if password not verified, -1 if password is verified.

Sub Verify.Newuser (Response.Type)
   Do ' password verify loop
      Outpt = "Press <C>ontinue, <H>angup, or <R>estart:" ' make message
      No.Input.Out = "C" ' default empty input to continue
      Line.Length = 1 ' reset line length
      Call IO.I ' get input
      Response.Type = False ' set return flag to unverified
      Select Case UCase$(Inpt) ' compare input selection
         Case "C" ' continue selected
            Outpt = Nul ' set empty output
            Call IO.O ' send output
            Outpt = "Verify password: " ' prompt for password
            Hidden = True ' set echo characters masked
            Line.Length = 20 ' line length of password
            Call IO.I ' get user input
            Hidden = False ' reset echo mask flag
            Inpt = LTrim$(Inpt) ' trim input
            Inpt = RTrim$(Inpt) ' trim input
            Inpt = UCase$(Inpt) ' set input uppercase
            Outpt = RTrim$(Player.PassWord) ' store password entered
            If Outpt <> Inpt Then ' compare password entered to verify entry
               Outpt = "Passwords don't match!" ' make error message
               Call IO.O ' send output
               Response.Type = False ' set flag to unverified
               Exit Do ' exit routine
            End If ' end compare password
            Outpt = "Memorize your password!" ' make message
            Call IO.O ' send output
            Call IO.O ' send empty line
            Response.Type = True ' set flag to verified
            Exit Do ' exit routine
         Case "H" ' hangup selected
            Response.Type = 1 ' set flag to hangup user
            Exit Do ' exit routine
         Case "R" ' restart selected
            Response.Type = False ' set flag to restart/unverified
            Exit Do ' exit routine
      End Select ' end compare input selection
   Loop ' end password verify loop
End Sub ' end routine to verify new password

Rem * routine to allow user to select new character statistics.
Rem * processing variables:
Rem *   Display.Help - flag to display login help messages.

Sub Roll.Character
   Display.Help = False ' set flag to display help text
   Outpt = "List help text during character logon(y/n)? " ' prompt for help
   No.Input.Out = "N" ' default prompt
   Line.Length = 1 ' reset line length
   Call IO.I ' get user input
   If Yes Then ' check input
      Display.Help = True ' set help text flag
   End If ' end check input
   If Display.Help Then ' check help flag
      Call Logon.Help(1) ' display class help text
   End If ' end check help flag
   Call Modify.Class ' routine to select class type
   If Display.Help Then ' check help flag
      Call Logon.Help(2) ' display statistics entry help text
   End If ' end check help flag
   Call Modify.Stats ' routine to select character statistics
   If Display.Help Then ' check help flag
      Call Logon.Help(3) ' display race entry help text
   End If ' end check help flag
   Call Modify.Race ' routine to select character race
   Call Init.Race.Stats ' routine to initialize some race statistics
   If Display.Help Then ' check help flag
      Call Logon.Help(4) ' display weapon proficiency entry help text
   End If ' end check help flag
   Call Modify.Proficiency ' routine to select weapon proficiency
   Call Init.Proficiency.Stats ' routine to intialize proficiency statistics
   Call Init.Stats ' routine to intialize some character statistics
   If Display.Help Then ' check help flag
      Call Logon.Help(5) ' display alignment entry help text
   End If ' end check help flag
   Call Modify.Alignment ' routine to select character alignment
   Call Display.Init.Stats ' routine to display character statistics
End Sub ' end routine to get new character statistics

Rem * routine to display help text.
Rem * input variables:
Rem *   Help.Number - range of help text file records to display.
Rem * work variables:
Rem *   Start.Help, End.Help, Help.Count.

Sub Logon.Help (Help.Number)
   Close #HelpFile ' close work file number
   FileName = "logon.dat" ' store logon helptext filename
   Open FileName For Random Shared As #HelpFile Len = Len(HelpRecord) ' open file
   Graphics.Off = True ' set color flag
   Outpt = Nul ' send empty line
   Call IO.O ' send output
   Select Case Help.Number ' selection for logon help record ranges
      Case 1 ' help records
         Start.Help = 2
         End.Help = 6
      Case 2 ' help records
         Start.Help = 7
         End.Help = 15
      Case 3 ' help records
         Start.Help = 16
         End.Help = 25
      Case 4 ' help records
         Start.Help = 26
         End.Help = 32
      Case 5 ' help records
         Start.Help = 33
         End.Help = 36
   End Select ' end select record ranges
   For Help.Count = Start.Help To End.Help ' loop through help text file range
      Call Read.Record(HelpFile, Help.Count) ' read help record
      Outpt = RTrim$(HelpRecord.Text) ' format help text
      Call IO.O ' send help output
   Next 'end loop through help file
   Call IO.O ' send ampty output
   Call More.Prompt ' get keypress
   Graphics.Off = False ' reset color flag
   Close #HelpFile
End Sub ' end routine to display help text

Rem * routine to allow user to modify character alignment.

Sub Modify.Alignment
   Do ' process modify alignment one loop
      Graphics.Off = False ' reset color
      Outpt = "Player Alignment:" ' make message
      Call IO.O ' display message
      Outpt = "Press " + Enter$ + " for default." ' make message
      Call IO.O ' display message
      Graphics.Off = True ' reset color
      For Align.Count = 1 To 3 ' display alignment choices
         Outpt = Mid$(Str$(Align.Count), 2) + "> " + RTrim$(Alignment.Name1(Align.Count))
         Call IO.O ' send output
      Next ' loop through alignment choices
      Outpt = "?" ' prompt for alignment number
      No.Input.Out = "2" ' default to neutral
      Line.Length = 1 ' reset line length
      Call IO.I ' get input
      Player.Alignment = Int(Val(Inpt)) ' convert to number
      If Player.Alignment >= 1 And Player.Alignment <= 3 Then ' compare valid choice
         Exit Do ' exit first loop
      End If ' end compare choice
   Loop ' continue alignment loop
   UserRecord.Align1 = Player.Alignment - 2 ' store alignment as -1/0/1
   Do ' process modify alignment two loop
      Graphics.Off = False ' reset color
      Outpt = "Player Alignment:" ' make message
      Call IO.O ' display message
      Outpt = "Press " + Enter$ + " for default." ' make message
      Call IO.O ' display message
      Graphics.Off = True ' reset color
      For Align.Count = 1 To 3 ' display alignment choices
         Outpt = Mid$(Str$(Align.Count), 2) + "> " + RTrim$(Alignment.Name2(Align.Count))
         Call IO.O ' send output
      Next ' loop through alignment chocies
      Outpt = "?" ' prompt for alignment number
      No.Input.Out = "2" ' default to neutral
      Line.Length = 1 ' reset line length
      Call IO.I ' get input
      Player.Alignment = Int(Val(Inpt)) ' convert to number
      If Player.Alignment >= 1 And Player.Alignment <= 3 Then ' compare valid choice
         Exit Do ' exit second loop
      End If ' end compare choice
   Loop ' continue alignment loop
   UserRecord.Align2 = Player.Alignment - 2 ' store alignment as -1/0/1
End Sub ' end routine to modify character alignment

Rem * routine to allow user to modify character class Type

Sub Modify.Class
   Outpt = Nul ' make empty line
   Call IO.O ' send output
   Do ' class entry process loop
      Graphics.Off = False ' reset color
      Call IO.O ' send blank line
      Outpt = "Select your character class:" ' make message
      Call IO.O ' send output
      Outpt = "Press " + Enter$ + " for default." ' make message
      Call IO.O ' send output
      If Local.Mode = False Then ' console mode allows DM/Asst. DM entries
         Max.Class = 8 ' set number of class choices
      Else ' compare console mode
         Max.Class = 10 ' set number of class choices
      End If ' end compare console mode
      Graphics.Off = True ' reset color
      For List.Counter = 1 To Max.Class ' loop through class chocies
         Outpt = Mid$(Str$(List.Counter), 2) ' store class number
         If List.Counter = 10 Then ' store DM class number
            Outpt = "#" ' choice ten is pound sign
         End If ' end compare DM class number
         Outpt = Outpt + "> " + RTrim$(Class.Name(List.Counter)) ' append class name
         Call IO.O ' send output
      Next ' loop through class choices
      Outpt = "?" ' set input prompt
      No.Input.Out = "1" ' set default choice
      Line.Length = 1 ' reset line length
      Call IO.I ' get user input
      Player.Class = Int(Val(Inpt)) ' convert to number
      If Inpt = "#" Then ' compare DM selection
         Player.Class = 10 ' set to ten
      End If ' end compare DM selection
      If Player.Class >= 1 And Player.Class <= Max.Class Then ' check class range
         Exit Do ' exit class type entry loop
      End If ' end check class range
   Loop ' end class type entry loop
   UserRecord.ClassType = Player.Class ' store class number in user record
   Outpt = Class.Name(UserRecord.ClassType) ' get class name
   Call Valid(Outpt, 20) ' validate class name
   If Outpt = Nul Then ' verify class name validity
      Outpt = "<checksum>" ' set error message
      Call Valid(Outpt, 20) ' validate error
   End If ' end verify class name
   Call Encrypt(Outpt, True) ' encrypt class name
   UserRecord.ClassName = Outpt ' store class name in user record
End Sub ' end routine to modify character class type

Rem * routine to allow user to modify character statistics.

Sub Modify.Stats
   Do ' loop until statistics selected are accepted
      Do ' loop until statistics are valid
         Graphics.Off = False ' reset color
         Outpt = "Enter character statistics, range from 8 to 18." ' message
         Call IO.O ' send message
         Outpt = "Average less than or equal to 12." ' message
         Call IO.O ' send message
         Outpt = "Press " + Enter$ + " for default." ' message
         Call IO.O ' send message
         Stat.Total! = False ' reset total of selected statistics
         For Class.Number = 1 To 7 ' loop through entry of all statistics
            Do ' loop until a valid statistic entered
               Graphics.Off = True ' reset color
               Outpt = RTrim$(Stat(Class.Number)) + ">" ' make message w/ stat name
               No.Input.Out = "12" ' set default
               Line.Length = 2 ' reset line length
               Call IO.I ' get input
               Stat = Int(Val(Inpt)) ' convert to number
               If Stat < 8 Or Stat > 18 Then ' check range
                  Graphics.Off = False ' reset color
                  Outpt = "The average statistic must range from 8 to 18."
                  Call IO.O ' send output message
               Else ' check range
                  Stat.Total! = Stat.Total! + Stat ' increment stat total
                  Call PutUserRecordStats(Class.Number, Stat) ' store stat in user record
                  Exit Do ' exit validity loop
               End If ' end check range
            Loop ' continue valid statistic loop
         Next ' loop through all statistics
         Stat.Total! = Stat.Total! / 7! ' calculate average of total statistics
         Stats$ = Str$(Stat.Total!) ' convert to string
         Stat.Delimit = InStr(Stats$, ".") ' search string for decimal
         If Stat.Delimit = False Then ' compare decimal
            Inpt = Stats$ ' set output string to converted string
         Else ' check decimal, truncate to one place
            ' set string
            Inpt = Left$(Stats$, Stat.Delimit - 1) + "." + Mid$(Stats$, Stat.Delimit + 1, 1)
         End If ' end compare decimal
         Graphics.Off = False ' reset color
         If Stat.Total! <= 12 Then ' verify average
            Exit Do ' exit validity loop
         End If ' end verify average
         Outpt = "Average" + Inpt + " to high! Try again.." ' set message
         Call IO.O ' send message
      Loop ' end statistic validity loop
      Outpt = "Your average is" + Inpt + ". Change anything(y/n)? " ' make message
      No.Input.Out = "N" ' set default input
      Line.Length = 1 ' reset line length
      Call IO.I ' get user input
      If No Then ' check no entered
         Exit Sub ' exit routine
      End If ' end check entry
   Loop ' end loop to verify statistics accepted
End Sub ' end routine to modify character statistics

Rem * routine to allow user to modify character race.

Sub Modify.Race
   Do ' loop until race entry is accepted
      Graphics.Off = False ' reset color
      Outpt = "Select your character race:" ' make message
      Call IO.O ' send message
      Outpt = "Press " + Enter$ + " for default." ' make message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      For Race.Count = 1 To 8 ' loop through all race choices
         ' choice display
         Outpt = Mid$(Str$(Race.Count), 2) + ">" + RTrim$(Race(Race.Count))
         Call IO.O ' send choice
      Next ' end race display loop
      Outpt = "?" ' set input prompt
      No.Input.Out = "1" ' set default
      Line.Length = 1 ' reset line length
      Call IO.I ' get user input
      Player.Race = Int(Val(Inpt)) ' convert to number
      If Player.Race >= 1 And Player.Race <= 8 Then ' check race range
         UserRecord.Race = Player.Race ' store race in user record
         Exit Sub ' exit routine
      End If ' end compare race range
   Loop ' end loop to accept race entry
End Sub ' end routine to modify race

Rem * routine to allow user to modify character weapon proficiency.

Sub Modify.Proficiency
   Do ' loop until proficiency entry accepted
      Graphics.Off = False ' reset color
      Outpt = "Weapon Proficiency:" ' set message
      Call IO.O ' send output
      Outpt = "Clerics may only use blunt or pole type weapons." ' message
      Call IO.O ' send output
      Outpt = "Press " + Enter$ + " for default." ' message
      Call IO.O ' send output
      Graphics.Off = True ' reset color
      For Prof.Count = 1 To 4 ' loop through all weapon proficiencies
         Outpt = Mid$(Str$(Prof.Count), 2) + "> " + RTrim$(WeaponTypeName(Prof.Count))
         Call IO.O ' send choice output
      Next ' end weapon choices
      Outpt = "?" ' set user prompt
      If UserRecord.ClassType = Cleric Then ' compare class to cleric
         No.Input.Out = "1" ' set default
      Else ' compare class
         No.Input.Out = "3" ' set default
      End If ' end compare class
      Line.Length = 1 ' reset line length
      Call IO.I ' get user input
      Player.Prof = Int(Val(Inpt)) ' convert to number
      If UserRecord.ClassType = Cleric Then ' compare class to cleric
         ' compare valid choices for cleric
         If Player.Prof = 1 Or Player.Prof = 2 Then
            Exit Do ' exit weapon input loop
         End If ' end compare valid chocies
      Else ' compare to non cleric
         If Player.Prof >= 1 And Player.Prof <= 4 Then ' compare valid choices
            Exit Do ' exit weapon input loop
         End If ' end compare valid choices
      End If ' end compare class type
   Loop ' end loop to accept weapon proficiency
   UserRecord.Proficiency = Player.Prof ' store character weapon selection
   For Weapon.Number = 1 To 4 ' loop through user record weapon proficiencies
      Call PutUserRecordWeapons(Weapon.Number, False) ' reset to zero
   Next ' end loop through weapon proficiencies
   ' set user record selected weapon profciency
   Call PutUserRecordWeapons(Player.Prof, 10)
End Sub ' end routine to modify character weapon proficinecy

Rem * routine to initialize character proficiency statistics.

Sub Init.Proficiency.Stats
   Graphics.Off = False ' reset color
   Select Case UserRecord.Race ' compare player character race
      Case 3 ' gnome race
         Call PutUserRecordWeapons(GetUserRecordProficiency, 15) ' increment proficiency
         Outpt = "Gnomes weapon proficiency is raised to 15%" ' make message
         Call IO.O ' send message
      Case 6 ' half-elf race
         Call PutUserRecordWeapons(4, GetUserRecordWeapons(4) + 5) ' increment proficiency
         Outpt = "Half-elves thrusting weapon proficiency is raised by 5%" ' message
         Call IO.O ' send message
      Case 7 ' half-orc race
         Call PutUserRecordWeapons(3, GetUserRecordWeapons(3) + 5) ' increment proficiency
         Outpt = "Half-orcs sharp weapon proficiency is raised by 5%" ' message
         Call IO.O ' send message
   End Select ' end compare race
End Sub ' end routine to initialize proficiency statistics

Rem * routine to initialize character race statistics.

Sub Init.Race.Stats
   Graphics.Off = False ' reset color
   Select Case UserRecord.Race ' compare player character race
      Case 1 ' human race
         Call PutUserRecordStats(1, GetUserRecordStats(1) + 1) ' increment statistics
         Outpt = "Humans strength is raised one point!" ' make message
         Call IO.O ' send message
      Case 2 ' elf race
         Call PutUserRecordStats(4, GetUserRecordStats(4) + 1) ' increment statistics
         Outpt = "Elves dexterity is raised by one point!" ' make message
         Call IO.O ' send message
      Case 4 ' dwarf race
         Call PutUserRecordStats(2, GetUserRecordStats(2) + 1) ' increment statistics
         Outpt = "Dwarves intelligence is raised by one point!" ' make message
         Call IO.O ' send message
      Case 5 ' halfling race
         Call PutUserRecordStats(3, GetUserRecordStats(3) + 1) ' increment statistics
         Outpt = "Halflings wisdom is raised by one point!" ' make message
         Call IO.O ' send message
      Case 8 ' ogre race
         Call PutUserRecordStats(1, GetUserRecordStats(1) + 1) ' increment statistics
         Call PutUserRecordStats(4, GetUserRecordStats(4) + 1) ' increment statistics
         Outpt = "Ogres strength and dexterity are raised by one point!" ' message
         Call IO.O ' send output
   End Select ' end compare race
   UserRecord.Beauty = Int(Rnd * 15 + 5) ' reset ladies beauty
   UserRecord.Glamour = Int(Rnd * 15 + 5) ' reset ladies glamour
End Sub ' end routine to initialize character race statistics

Rem * routine to initialize some character statistics.

Sub Init.Stats
   User.Echo = False ' reset preference
   User.LineFeeds = False ' reset preference
   User.LineLength = 80 ' reset preference
   User.PageLength = 24 ' reset preference
   User.Wordwrap = False ' reset preference
   UserRecord.Room = 1 ' reset user record room number
   UserRecord.Level = 1 ' reset user record character level
   UserRecord.Experience = 64 ' reset experience
   UserRecord.Gold = 2048 ' reset user record gold
   UserRecord.Bank = False ' reset user record amount of gold in bank
   UserRecord.Borrow = False ' reset user record amount of gold borrowed from bank
   UserRecord.Brief = False ' reset user record brief mode
   UserRecord.Echo = False ' reset user echo mode
   UserRecord.Linefeeds = False ' reset user linefeed mode
   UserRecord.Linelength = 80 ' reset user linelength
   UserRecord.Pagelength = 24 ' reset pagelength
   UserRecord.Wordwrap = False ' reset user word wrap
   UserRecord.FatigueMax = Training.Stats(UserRecord.ClassType, 1) ' reset user
   UserRecord.VitalityMax = Training.Stats(UserRecord.ClassType, 2) ' record maximum
   UserRecord.MagicMax = Training.Stats(UserRecord.ClassType, 3) ' statistic
   UserRecord.PsionicMax = Training.Stats(UserRecord.ClassType, 4) ' points
   UserRecord.Fatigue = UserRecord.FatigueMax ' reset user
   UserRecord.Vitality = UserRecord.VitalityMax ' record working
   UserRecord.Magic = UserRecord.MagicMax ' statistic
   UserRecord.Psionic = UserRecord.PsionicMax ' points
   UserRecord.MaxCalls = False ' reset user record maximum calls
   UserRecord.FromHour = False ' reset user record time restrictions
   UserRecord.FromMin = False ' reset user record time restrictions
   UserRecord.ToHour = False ' reset user record time restrictions
   UserRecord.ToMin = False ' reset user record time restrictions
   UserRecord.Flags = False ' reset user record flags variable
   Call Clear.Container(0, True) ' routine to clear the container structure
   For Container.Item = 1 To 3 ' loop through user record containers
      Call PutUserRecordContainer(Container.Item) ' reset container record
   Next ' end loop through user record
   ' initialize training rooms
   Call PutUserRecordTrainRoom(1, 36)
   Call PutUserRecordTrainRoom(2, 4)
   Call PutUserRecordTrainRoom(3, 123)
   Call PutUserRecordTrainRoom(4, 163)
   Call PutUserRecordTrainRoom(5, 162)
   Call PutUserRecordTrainRoom(6, 426)
   Call PutUserRecordTrainRoom(7, 421)
   Call PutUserRecordTrainRoom(8, 417)
   Call PutUserRecordTrainRoom(9, 1)
   Call PutUserRecordTrainRoom(10, 1)
End Sub ' end routine to initialize character statistics

Rem * routine to display login character statistics.

Sub Display.Init.Stats
   Graphics.Off = False ' reset color
   Outpt = "Your character statistics are:" ' message
   Call IO.O ' send output
   Graphics.Off = True ' reset color
   Outpt = "Level:" + Str$(UserRecord.Level) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Alignment.Type$ = Alignment.Name1(UserRecord.Align1 + 2) ' message
   Alignment.Type$ = RTrim$(Alignment.Type$) ' trim blanks
   ' first character uppercase
   Mid$(Alignment.Type$, 1, 1) = UCase$(Mid$(Alignment.Type$, 1, 1))
   Outpt = Outpt + "Align1: " + Alignment.Type$ ' combine message
   Call IO.O ' send output
   Outpt = "Gold: " + Str$(UserRecord.Gold) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Alignment.Type$ = Alignment.Name2(UserRecord.Align2 + 2) ' message
   Alignment.Type$ = RTrim$(Alignment.Type$) ' trim blanks
   ' first character uppercase
   Mid$(Alignment.Type$, 1, 1) = UCase$(Mid$(Alignment.Type$, 1, 1))
   Outpt = Outpt + "Align2: " + Alignment.Type$ ' combine message
   Call IO.O ' send output
   Outpt = "Room: " + Str$(UserRecord.Room) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Player.Prof = UserRecord.Proficiency ' get statistic
   Weapon.Type$ = WeaponTypeName(Player.Prof) ' message
   Weapon.Type$ = RTrim$(Weapon.Type$) ' trim blanks
   ' first character uppercase
   Mid$(Weapon.Type$, 1, 1) = UCase$(Mid$(Weapon.Type$, 1, 1))
   Outpt = Outpt + "Prof:   " + Weapon.Type$ ' combine message
   Weapon.Proficiency$ = Str$(GetUserRecordWeapons(Player.Prof)) ' message
   Outpt = Outpt + ">" + Weapon.Proficiency$ + "%" ' combine message
   Call IO.O ' send output
   Outpt = "Exp:  " + Str$(UserRecord.Experience) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(1), 3) + ":   " + Str$(GetUserRecordStats(1)) ' combine
   Call IO.O ' send output
   Outpt = "Fat:  " + Str$(UserRecord.Fatigue) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(2), 3) + ":   " + Str$(GetUserRecordStats(2)) ' combine
   Call IO.O ' send output
   Outpt = "Vit:  " + Str$(UserRecord.Vitality) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(3), 3) + ":   " + Str$(GetUserRecordStats(3)) ' combine
   Call IO.O ' send output
   Outpt = "Mag:  " + Str$(UserRecord.Magic) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(4), 3) + ":   " + Str$(GetUserRecordStats(4)) ' combine
   Call IO.O ' send output
   Outpt = "Psi:  " + Str$(UserRecord.Psionic) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(5), 3) + ":   " + Str$(GetUserRecordStats(5)) ' combine
   Call IO.O ' send output
   Outpt = "Race:  " + Race(UserRecord.Race) ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(6), 3) + ":   " + Str$(GetUserRecordStats(6)) ' combine
   Call IO.O ' send output
   Class.Type$ = UserRecord.ClassName ' message
   Call Decrypt(Class.Type$) ' decrypt string
   ' first character uppercase
   Mid$(Class.Type$, 1, 1) = UCase$(Mid$(Class.Type$, 1, 1))
   Class.Type$ = Left$(Class.Type$, 15) ' truncate to right
   Outpt = "Class: " + Class.Type$ ' message
   Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
   Outpt = Outpt + Left$(Stat(7), 3) + ":   " + Str$(GetUserRecordStats(7)) ' combine
   Call IO.O ' send output
   If UserRecord.ClassType = Lady Then ' compare class to lady
      Outpt = "Lady stats:" ' message
      Call IO.O ' send output
      Outpt = "Beauty:" + Str$(UserRecord.Beauty) ' message
      Outpt = Outpt + Space$(25 - Len(Outpt)) ' append blanks
      Outpt = Outpt + "Glamour:" + Str$(UserRecord.Glamour) ' combine
      Call IO.O ' send output
   End If ' end compare class type
   Call More.Prompt ' pause to continue
End Sub ' end routine to display character login statistics

Rem * routine to initialize some variables after login.
Rem * output variables:
Rem *   Time.Left - player time left in seconds from login time.

Sub Update.Login
   Outpt = UserRecord.DateOn ' get user's last login date
   Call Decrypt(Outpt) ' decrypt date
   If Outpt <> Date$ Then ' compare last login date to today
      UserRecord.NumCalls = False ' reset user's number of calls logged in per day
   End If ' end compare last login date
   If UserRecord.ClassType <= Lady Then ' compare user to non DM status
      Calls.Exceeded = False ' set flag to maximum calls exceeded
      UserRecord.NumCalls = UserRecord.NumCalls + 1 ' increment user's max calls
      If UserRecord.MaxCalls > False Then ' compare maximum calls
         If UserRecord.NumCalls > UserRecord.MaxCalls Then ' compare calls
            Calls.Exceeded = True ' set maximum call flag
         End If ' end compare calls to maximum calls
      Else ' end compare maximum calls
         If UserRecord.NumCalls > 5 Then ' compare maximum calls to default
            Calls.Exceeded = True ' set maximum call flag
         End If ' end compare maximum default calls
      End If ' end compare maximum calls
      If Calls.Exceeded Then ' check user has exceeded maximum calls
         Call Share.Record(UserFile, User.Index) ' put user record
         Call Hang.Up(5) ' routine to terminate program w/ message
         Exit Sub ' exit routine
      End If ' end check maximum calls
   End If ' end non DM status
   Call Restricted.Login ' check user is restricted to a time to login
   If UserRecord.ClassType < AsstDM Then ' compare user is non DM status
      UserRecord.Invisible = False ' reset user invisibility
   End If ' end compare non DM status
   Select Case UserRecord.ClassType ' compare the user class type
      Case Is >= AsstDM ' user is DM/Asst. DM
         Time.Left = 3600! ' user gets 60 minutes
      Case Else ' user is non DM
         Select Case UserRecord.Level ' select by user level
            Case Is < 2 ' user level is less than two
               Time.Left = 900! ' user gets 15 mminutes
            Case Else ' user level is two or more
               Time.Left = 1800! ' user gets 30 minutes
         End Select ' end user level
   End Select ' end user DM status
   If Time.Left > Door.Time Then ' compare user's time left to door file time left
      If Door.Time > False Then ' compare door file time
         Time.Left = Door.Time ' reset time left to door time left
      End If ' end compare door file time
   End If ' end compare user time left
   Two.Minutes.Left = False ' reset two minutes left message flag
   User.Echo = UserRecord.Echo ' store user preference
   User.LineFeeds = UserRecord.Linefeeds ' store user preference
   User.LineLength = UserRecord.Linelength ' store user preference
   User.PageLength = UserRecord.Pagelength ' store user preference
   User.Wordwrap = UserRecord.Wordwrap ' store user preference
End Sub ' end routine to initialize some login variables

Rem * routine to check user is restricted to specific login times.
Rem * work variables:
Rem *   Restrict.Start! - time in seconds to restrict logon.
Rem *   Restrict.End! - time in seconds to restrict logon.

Sub Restricted.Login
   ' calculate time restrictions
   Restrict.Start! = CSng(UserRecord.FromHour * 3600! + UserRecord.FromMin * 60!)
   Restrict.End! = CSng(UserRecord.ToHour * 3600! + UserRecord.ToMin * 60!)
   ' compare any time restriction
   If Restrict.Start! > False Or Restrict.End! > False Then
      If Timer < Restrict.Start! Or Timer > Restrict.End! Then ' compare time to now
         Call Hang.Up(6) ' routine to terminate program w/ message
      End If ' end compare time
   End If ' end compare time restriction
End Sub ' end routine to check restricted time login

Rem * routine to intialize some new user variables in user file record.

Sub Init.Newuser
   UserRecord.NumCalls = False ' reset maximum calls made today
   UserRecord.ClassType = False ' reset class type
   Outpt = Player.CodeName ' store codename
   Call Valid(Outpt, 30) ' validate codename
   Call Encrypt(Outpt, True) ' encrypt codename
   UserRecord.CodeName = Outpt ' restore codename
   Outpt = Player.PassWord ' store password
   Call Valid(Outpt, 20) ' validate password
   Call Encrypt(Outpt, False) ' encrypt password
   UserRecord.PassWord = Outpt ' restore password
   Outpt = Deleted$ ' set deleted
   Call Valid(Outpt, 20) ' validate deleted
   Call Encrypt(Outpt, True) ' encrypt deleted
   UserRecord.ClassName = Outpt ' reset classname
   Outpt = Date$ ' store current date
   Call Valid(Outpt, 10) ' validate date
   Call Encrypt(Outpt, True) ' encrypt date
   UserRecord.DateOn = Outpt ' reset date
   UserRecord.MaxCalls = False ' reset maximum calls made
   UserRecord.FromHour = False ' reset time restrictions
   UserRecord.FromMin = False
   UserRecord.ToHour = False
   UserRecord.ToMin = False
End Sub ' end routine to intialize new user variables

Rem * routine to initialize some user variables.

Sub Login.User
   Call Read.Room.Record(1!) ' get room record #1
   Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
   Number.Monsters = False ' counter of monsters currently in the room
   Monsters.Killed = False ' counter of monster killed by player during session
   ' allocate room monster arrays
   ReDim MonsterArray(1 To 20) As MonsterType
   ReDim MonsterIndex(1 To 20) As Integer
   Max.Spells = LOF(SpellFile) / Len(SpellRecord) ' compute number of spells in file
   If Max.Spells = False Then ' compare empty file
      Max.Spells = 1 ' set number to at least one
      Call Share.Record(SpellFile, 1) ' put default spell record
   End If ' end compare file
   ' check bounds of spell file
   If Max.Spells > 1024 Then ' check bounds
      Max.Spells = 1024 ' reste maximum
   End If ' end check bounds
   ' make string of zeros length of spell file
   Learned.Spells = String$(Max.Spells, "0")
   If UserRecord.Race <= False Then ' check user race
      UserRecord.Race = 1 ' reset to one
   End If ' end check race
   Call Share.Record(UserFile, User.Index) ' put the user record
   Room = UserRecord.Room ' store the room number
   If UserRecord.Level = False Then ' check user level
      Outpt = "You are level zero. You can use the train command once free."
      Call IO.O ' send output message
   End If ' end compare level
   Call Bank.Interest ' calculate bank interest for balance and loan
   Call Check.Mail ' routine to display number of new messages to player
   Weapon1 = False ' reset working game weapon, shield, armor, and ring variables
   Weapon2 = False ' reset variable
   Weapon3 = False ' reset variable
   Weapon4 = False ' reset variable
   Weapon5 = False ' reset variable
   Weapon6 = False ' reset variable
   Weapon7 = False ' reset variable
   Weapon8 = False ' reset variable
   Weapon9 = False ' reset variable
   Weapon10 = False ' reset variable
   Weapon11 = False ' reset variable
   Call Get.User.Record ' read the user record
   Call Status.Line(1) ' initialize the console status lines
   FunctionBuffer = Nul ' reset function key buffer
End Sub ' end routine to initialize some user variables

Rem * routine to display treasure item.
Rem * input variables:
Rem *   Index.Number - treasure record number.
Rem *   TypeNumber - room/inventory flag.

Sub Show.Treasure
   If TypeNumber Then ' compare treasure in room
      Prefix1 = "It's " ' format prefix
   Else ' compare treasure in player inventory
      Prefix1 = "You are carrying " ' format prefix
   End If ' end compare treasure
   Graphics.Off = True ' reset color
   If TreasureRecord.Scroll Then ' compare treasure to scroll
      If TreasureRecord.Spell Then ' check scroll spell number
         Call Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
         Inpt = SpellRecord.Chant ' store spell chant
         Inpt = RTrim$(Inpt) ' trim chant
         Inpt = LCase$(Inpt) ' trim chant
         Outpt = "It reads: '" + Inpt + "'." ' display scroll chant
         Call IO.O ' send message
         Outpt = "It disintegrated!" ' scroll vanished message
         Call IO.O ' send output
         If TypeNumber = False Then ' compare treasure in room
            ' remove scroll from inventory
            Call Discard.Inventory(Array.Number, True)
         Else ' compare treasure
            Call Discard.Room.Treasure(Array.Number) ' remove scroll from room
         End If ' end compare treasure
      End If ' end compare scroll spell number
      Exit Sub ' exit routine
   End If ' end compare treasure is scroll
   Outpt = Prefix1 + Outpts ' format treasure name description
   If TreasureRecord.Keyed Then ' append key number to treasure name
      Outpt = Outpt + "(#" + Right$(Str$(TreasureRecord.Keyed + 100000!), 5) + ")"
   End If ' end compare treasure key number
   If TreasureRecord.Plus Then ' append plus number to treasure name
      Outpt = Outpt + "(+" + Mid$(Str$(Abs(TreasureRecord.Plus)), 2) + ")"
   End If ' end compare treasure plus
   If TreasureRecord.Spell Then ' append spell plus to treasure name
      Call Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
      Outpt = Outpt + "(+" + Mid$(Str$(SpellRecord.Level), 2) + ")"
   End If ' end compare treasure spell plus
   If TreasureRecord.LightType Then ' compare treasure to a light
      If Charges.Number < False Then ' check treasure is also lit
         Outpt = Outpt + "[lit]" ' append to treasure name
      End If ' end check lit treasure
   End If ' end compare treasure to light
   If TreasureRecord.Invisible Then ' compare treasure is invisible
      Outpt = Outpt + "[inv]" ' append to treasure name
   Else ' compare treasure
      If TypeNumber = 1 Then ' check treasure is in room
         ' verify treasure in
         If GetRoomRecordFlags(Array.Number) = Hidden.Object Then
            Outpt = Outpt + "[inv]" ' room is invisible, append to name
         End If ' end verify treasure in room was hidden
      End If ' end check treasure in room
   End If ' end compare treasure is invisible
   Outpt = Outpt + "." ' append message
   Call IO.O ' display treasure name message
   If TreasureRecord.Proficiency Then ' compare treasure proficiency
      Outpt = WeaponTypeName(TreasureRecord.Proficiency) ' get proficiency
      Outpt = RTrim$(Outpt) ' name and trim
      Outpt = "This is a " + Outpt + " weapon." ' make weapon type message
      Call IO.O ' display weapon type message
   End If ' end compare treasure proficiency
   If Last.Command.Number = Identify.Command Then ' check identify command used
      Outpt = "It's worth" + Str$(TreasureRecord.Gold) + " gold peices."
      Call IO.O ' display treasure item gold value
      Outpt = "It weighs" + Str$(TreasureRecord.Weight) + " pounds."
      Call IO.O ' display weight of item
      If TreasureRecord.RingType Then ' compare treasure ring type
         Select Case TreasureRecord.RingType ' determine ring type
            Case 1 ' ring type
               Outpt = "protection from poison." ' ring type message
            Case 2 ' ring type
               Outpt = "protection from level drain." ' ring type message
            Case 3 ' ring type
               Outpt = "protection from spells." ' ring type message
         End Select ' end dtermine ring type
         Outpt = "Its ring spell is " + Outpt ' make ring type message
         Call IO.O ' send ring type message
      End If ' end compare treasure to ring
      If TreasureRecord.Spell Then ' compare treasure spell type
         Call Read.Record(SpellFile, TreasureRecord.Spell) ' get treasure spell
         Outpt = "Its magical spell is " + RTrim$(SpellRecord.SpellName) + "."
         Call IO.O ' display name of treasure spell
      End If ' end compare treasure spell type
      ' compare treasure is loaded
      If TreasureRecord.Loadable Or TreasureRecord.Launchable Then
         If Charges.Number <= False Then ' compare treasure charges
            Outpt = "It's not loaded." ' display message
         Else ' compare treasure charges
            Outpt = "It's loaded with" + Str$(Charges.Number) + " charges." ' message
         End If ' end compare loaded treasure charges
         Call IO.O ' send message of charges in loaded treasure
      Else ' compare treasure item
         If TreasureRecord.LightType Then ' compare treasure is a light
            If Charges.Number < False Then ' compare light charges (is negative)
               Outpt = "It's fueled with" + Str$(Abs(Charges.Number)) + " charges."
               Call IO.O ' send message of charges in light
            End If ' end compare light charges
         Else ' compare other treasure plus
            If TreasureRecord.RingType Or TreasureRecord.Spell Or TreasureRecord.Plus Then ' treasure has charges
               If Charges.Number <= False Then ' compare charges
                  Outpt = "It's empty of charges." ' message
               Else ' compare charges
                  ' message of charges
                  Outpt = "It has" + Str$(Charges.Number) + " charges."
               End If ' end compare charges
               Call IO.O ' send message of charges
            End If ' end compare treasure charges
         End If ' end compare treasure plus
      End If ' end compare treasure
      ' compare treasure is ammunition
      If TreasureRecord.Ammunition Or TreasureRecord.LaunchAmmo Then
         Outpt = "It's ammunition." ' treasure nessage
         Call IO.O ' send message
      End If ' end compare treasure
      If TreasureRecord.Vehicle Then ' compare treasure to vehicle
         Outpt = "It's a vehicle." ' display message
         Call IO.O ' message
         Outpt = "It has" + Str$(Charges.Number) + " charges." ' display message
         Call IO.O ' message
      End If ' end compare treasure
      If TreasureRecord.Potion Then ' compare treasure to potion
         Outpt = "It's a potion." ' make message
         Call IO.O ' send message
      End If ' end compare to potion
      If TreasureRecord.Edible Then ' compare treasure to food
         Outpt = "It's edible." ' make message
         Call IO.O ' send message
      End If ' end compare to food
   End If ' end identify command
   Graphics.Off = False ' reset color
End Sub ' end routine to display an itemof treasure

Rem * routine to display object information.
Rem * input variables:
Rem *   Index.Number - object record number.
Rem *   TypeNumber - object is in room/inventory.

Sub Show.Object
   Hidden.Flag = False ' reset flag
   If TypeNumber Then ' determine object in room
      Prefix1 = "It's " ' make prefix
      If GetRoomRecordHiddenObj(Array.Number) Then ' check object hidden
         Hidden.Flag = True ' reset flag
      End If ' end check object
   Else ' object in inventory
      Prefix1 = "You are carrying " ' make prefix
   End If ' end determine in room
   Graphics.Off = True ' reset color
   Outpt = Prefix1 + Outpts ' make message with object name
   If ObjectRecord.DoorLock > 1 Then ' compare object is locked
      Outpt = Outpt + "[locked]" ' append to object name
   End If ' end compare locked object
   If ObjectRecord.DoorLock = 1 Then ' compare object is unlocked
      If ObjectRecord.Closed Then ' compare object is closed
         Outpt = Outpt + "[closed]" ' append to object name
      End If ' end compare closed object
   End If ' end compare object lock
   If ObjectRecord.Invisible Or Hidden.Flag Then ' compare object is invisible
      Outpt = Outpt + "[inv]" ' append to object name
   End If ' end compare object invisible
   If ObjectRecord.Keyed Then ' compare object key, append number to name
      Outpt = Outpt + "(#" + Right$(Str$(ObjectRecord.Keyed + 100000!), 5) + ")"
   End If ' end compare object key number
   Outpt = Outpt + "." ' append message
   Call IO.O ' display message with object name
   Outpt = ObjectRecord.LongDesc ' store object additional description
   Outpt = RTrim$(Outpt) ' trim description
   If Outpt <> Nul Then ' compare length of description
      Call IO.O ' display additional object description
   End If ' end compare object description length
   Graphics.Off = False ' reset color
End Sub ' end routine to display object information

Rem * routine to display information on a monster.
Rem * input variables:
Rem *   Monster.Number - number of monster array.

Sub Show.Monster
   Graphics.Off = True ' reset color
   Call The.Or.An ' routine for monster name prefix (a, an, the)
   Level = MonsterArray(Monster.Number).Level ' store monster level
   Outpt = "It's " + Prefix1 + Outpts ' make message of monster name
   ' append monster level (range of player's level capable to kill monster)
   Outpt = Outpt + "(level" + Str$((Level - 1) * 2 + 1) + " to" + Str$(Level * 2) + ")."
   Call IO.O ' send message with monster name and level range
   If Last.Command.Number = Identify.Command Then ' compare identify command
      Gold.Points# = MonsterArray(Monster.Number).Gold ' store monster gold
      If Gold.Points# <= False Then ' compare monster gold
         Gold.Points# = 10 ' set to minimum
      End If ' end compare monster gold
      Outpt = "It has" + Str$(MonsterArray(Monster.Number).Hits) + " hits," + Str$(MonsterArray(Monster.Number).Experience) + " experience, and" + Str$(Gold.Points#) + " gold."
      Call IO.O ' display message of monster gold
      Outpt = "It carries the following treasure:" ' make message
      Call IO.O ' send message of treasure carried by monster
      Inventory.Count = False ' reset number of monster inventory items displayed
      For Array.Count = 1 To 5 ' loop through all monster inventory
         ' get treasure
         Treasure.Number = GetMonsterRecordTreasure(Array.Count)
         ' number and check range in treasure file
         If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
            Carriage.Return = True ' flag to disable return/linefeed
            Call IO.O ' send output of previous item
            Call Read.Record(TreasureFile, Treasure.Number) 'get treasure record
            Outpts = TreasureRecord.TreasureName ' store treasure name
            Outpt = RTrim$(Outpts) + ", " ' trim name, append comma
            Inventory.Count = Inventory.Count + 1 ' increment items displayed flag
            If Inventory.Count = 1 Then ' compare item to first displayed
               Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first item
            End If ' end compare first item
         End If ' end check treasure file range
      Next ' end loop through monster inventory
      If Inventory.Count = False Then ' check if any items displayed
         Outpt = "Nothing at all." ' make message for none
      Else ' check items displayed
         Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, append period
         If Inventory.Count > 1 Then ' check more than one item displayed
            Outpt = "and " + Outpt ' append to last item
         End If ' end check items
      End If ' end check item
      Call IO.O ' send output for last item
      ' compare monster spell ability
      Spell.Number = MonsterArray(Monster.Number).Spell
      ' check spell
      If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
         Call Read.Record(SpellFile, Spell.Number) ' range and get spell record
         Outpt = "It can cast " + RTrim$(SpellRecord.SpellName) + " spells!"
         Call IO.O ' send message of spell name monster can cast
      End If ' end compare monster spell
      If MonsterArray(Monster.Number).Poison Then ' compare monster poisonous
         Outpt = "It can poison!" ' make message
         Call IO.O ' send output message
      End If ' end compare monster poisonous
      If MonsterArray(Monster.Number).LevelDrain Then ' compare monster undead
         Outpt = "It can drain levels!" ' make message
         Call IO.O ' send output message
      End If ' end compare monster undead
      If MonsterArray(Monster.Number).Psionic Then ' compare monster astral
         Outpt = "It can cast psi spells!" ' make message
         Call IO.O ' send output message
      End If ' end compare monster astral
   End If ' end check identify command used
   Graphics.Off = False ' reset color
End Sub ' end routine to display monster information

Rem * routine to determine validity of room number.
Rem * input variables:
Rem *   Room - contains room number to check.

Sub Check.Next.Room
   Do ' loop until room is valid, room is created, or nondescriptive hangup
      If Room > False And Room <= LOF(RoomFile) / Len(RoomRecord) Then ' check room range
         Call Read.Room.Record(Room) ' valid range, get room record
         Exit Sub ' exit routine
      End If ' end check valid range
      If Room > LOF(RoomFile) / Len(RoomRecord) Then ' compare room number range
         If Not Normal.User Then ' check non DM status
            Call Add.Room(False, Room.Created) ' routine to create new room
            If Room.Created Then ' return variable indicates new room created
               Exit Sub ' exit routine
            End If ' end create new room
         End If ' end check normal user
      End If ' end compare room number range
      ' otherwise, any room number out of range will be changed to room 1, or
      ' changed to the resurrection room number.
      If LOF(RoomFile) / Len(RoomRecord) >= 1 Then ' check for room
         Graphics.Off = False ' reset color
         Outpt = "Nondescriptive room number" + Str$(Room) + "!" ' make error message
         Call IO.O ' display room number error message
         Room = 1 ' reset room number to resurrection room, continue loop
      Else ' room file is invalid, room file length is zero
         Graphics.Off = False ' reset color
         Room = 1 ' reset room number
         Call Clear.Room(1) ' add first room
         Exit Sub ' exit routine
      End If ' end check room file length
   Loop ' end loop until valid room number
End Sub ' end routine to check room number validity

Rem * routine to initialize some room variables, check next room number, and
Rem * display next room description.
Rem * input variables:
Rem *   Room - room number to move to.
Rem * output variables:
Rem *   Room.Rust.Rate - number of prompts to check weapon rusting.
Rem *   Room.Steal.Rate - number of prompts to check monster stealing.
Rem *   Room.Monster.Rate - number of prompts to check monster encounter.
Rem *   Room.Health.Rate - number of prompts to check health increases.

Sub Display.Room
   Call Check.Next.Room ' routine to verify next room number
   Room.Rust.Rate = False ' store room rust rate
   Room.Steal.Rate = False ' store room steal rate
   Room.Monster.Rate = 6 ' store default room encounter rate
   Room.Health.Rate = 6 ' store default room health rate
   Room.Action = RoomRecord.Action
   If Room.Action > False And Room.Action <= LOF(ActionFile) / Len(ActionRecord) Then
      Call Read.Record(ActionFile, Room.Action)
      If ActionRecord.RustRate > False Then ' check room record action rust rate
         Room.Rust.Rate = ActionRecord.RustRate ' store room rust rate
      End If ' end check action
      If ActionRecord.StealRate > False Then ' check room record action steal rate
         Room.Steal.Rate = ActionRecord.StealRate ' store room steal rate
      End If ' end check action
      ' check room record action encounter rate
      If ActionRecord.EncounterRate Then
         ' store action encounter rate
         Room.Monster.Rate = ActionRecord.EncounterRate
      End If ' end check action
      If ActionRecord.HealthRate Then ' check room record action health rate
         Room.Health.Rate = ActionRecord.HealthRate ' store action health rate
      End If ' end check action
   End If
   Call Show.Room ' routine to display room
End Sub ' end routine to process next room

Rem * routine to determine if a room is unlit.
Rem * return variables:
Rem *   Lit.Room - true if room is unlit.

Sub Check.Lit.Room (Lit.Room)
   Lit.Room = False ' room is lit by default
   Call Read.Room.Record(Room) ' get room record
   Action.Number = RoomRecord.Action ' store action number
   ' check action number
   If Action.Number < 1 Or Action.Number > LOF(ActionFile) / Len(ActionRecord) Then
      Lit.Room = False ' room is lit
      Exit Sub ' exit lit check routine
   End If ' end check action number
   Call Read.Record(ActionFile, Action.Number) ' get action record number
   If ActionRecord.Attribute1 = LitRoom Then ' compare lit flag
      Lit.Room = False ' room is lit
      Exit Sub ' exit check lit routine
   End If ' end check lit flag
   Lit.Room = True ' flag for unlit room
   For Array.Index = 1 To 20 ' loop through all user inventory
      Treasure.Number = GetUserRecordInv(Array.Index) ' get inventory number
      If Treasure.Number Then ' compare user treasure number
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         If TreasureRecord.LightType Then ' check treasure item is a light
            ' check light is charged/lit
            If GetUserRecordCharges(Array.Index) < False Then
               Lit.Room = False ' set flag for lit room
               Exit Sub ' exit routine
            End If ' end check charged light
         End If ' end check treasure is a light
      End If ' end compare treasure number
   Next ' end loop through user inventory
   For Array.Index = 1 To 20 ' loop through all treasure in room
      ' get room treasure number
      Treasure.Number = GetRoomRecordTreasure(Array.Index)
      If Treasure.Number Then ' compare treasure number
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         If TreasureRecord.LightType Then ' compare treasure is a light
            ' compare light is charged
            If GetRoomRecordTreCharges(Array.Index) < False Then
               Lit.Room = False ' set flag for lit room
               Exit Sub ' exit routine
            End If ' end compare charged light
         End If ' end compare treasure is a light
      End If ' end compare treasure number
   Next ' end loop through room treasure
   For Array.Index = 1 To 20 ' loop through all room objects
      If GetRoomRecordObject(Array.Index) Then ' compare room object number
         Call Read.Record(ObjectFile, GetRoomRecordObject(Array.Index)) 'get record
         If ObjectRecord.LightRoom Then ' check object is a light
            If ObjectRecord.LightTime = False Then ' object lights at any time
               Lit.Room = False ' set flag for lit room
               Exit Sub ' exit routine
            Else ' light has light time restriction
               ' calculate seconds light from/to
               Start.Time! = CSng(ObjectRecord.FromHour * 3600! + ObjectRecord.FromMin * 60!)
               End.Time! = CSng(ObjectRecord.ToHour * 3600! + ObjectRecord.ToMin * 60!)
               ' check valid light time
               If Start.Time! > False Or End.Time! > False Then
                  ' compare times
                  If Timer >= Start.Time! And Timer <= End.Time! Then
                     Lit.Room = False ' set room lit flag
                     Exit Sub ' exit routine
                  End If ' end compare times
               End If ' end check valid light time
            End If ' end check object light type
         End If ' end check object is a light
      End If ' end compare object number
   Next ' end loop through room objects
   For Array.Index = 1 To 20 ' loop through all user object inventory
      If GetUserRecordObject(Array.Index) Then ' compare user object number
         Call Read.Record(ObjectFile, GetUserRecordObject(Array.Index)) 'get object
         If ObjectRecord.LightRoom Then ' compare object is a light
            If ObjectRecord.LightTime = False Then ' check object lights any time
               Lit.Room = False ' set flag for lit room
               Exit Sub ' exit routine
            Else ' compare object light time restriction
               ' calculate seconds light from/to
               Start.Time! = CSng(ObjectRecord.FromHour * 3600! + ObjectRecord.FromMin * 60!)
               End.Time! = CSng(ObjectRecord.ToHour * 3600! + ObjectRecord.ToMin * 60!)
               ' check valid light time
               If Start.Time! > False Or End.Time! > False Then
                  ' compare times
                  If Timer >= Start.Time! And Timer <= End.Time! Then
                     Lit.Room = False ' set lit room flag
                     Exit Sub ' exit routine
                  End If ' end compare times
               End If ' end check valid light times
            End If ' end compare object light type
         End If ' end compare object is a light
      End If ' end compare object number
   Next ' end loop through user objects
End Sub ' end routine to determine lit room

Rem * routine to display all the player character statistics.

Sub Display.Stats
   Graphics.Off = True ' reset color
   Outpt = UserRecord.CodeName ' get user codename
   Call Decrypt(Outpt) ' decrypt codename
   Outpt = RTrim$(Outpt) ' trim codename
   Outpt = LCase$(Outpt) ' set codename to lowercase
   Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
   Outpt = "Information: " + Outpt + ". " + clock$ + "." ' make information message
   Call IO.O ' send message
   Call Show.Align ' routine to display player alignment
   Call More.Prompt ' pause display
   If No Then ' check more prompt response
      Exit Sub ' exit info display
   End If ' end check more prompt
   Call Show.Health ' routine to display player statistics
   Call More.Prompt ' pause display
   If No Then ' check more prompt response
      Exit Sub ' exit info display
   End If ' end check more prompt
   Call Display.Info ' routine to display additional player information
   Call More.Prompt ' pause display
   If No Then ' check more prompt response
      Exit Sub ' exit info display
   End If ' end check more prompt
   Call Display.Inventory ' routine to display player inventory
   Call More.Prompt ' pause display
   If No Then ' check more prompt response
      Exit Sub ' exit info display
   End If ' end check more prompt
   Call Display.Experience ' routine to display player experience and gold
   Call More.Prompt ' pause display
   If No Then ' check more prompt response
      Exit Sub ' exit info display
   End If ' end check more prompt
End Sub ' end routine to display all player character statistics

Rem * routine to display player character experience, and gold.
Rem * routine notes:
Rem *    although gold and experience required to reach the next level double
Rem *    each player level, the experience and gold required for players over
Rem *    level 10 only increase by 10,000 points per level over 10.

Sub Display.Experience
   Var1$ = FormatString$(UserRecord.Gold)
   Var2$ = FormatString$(UserRecord.Experience)
   Outpt = "You have " + Var1$ + " gold and " + Var2$ + " experience." ' make gold/experience message
   Graphics.Off = True ' reset color
   Call IO.O ' send output message
   Level = UserRecord.Level ' store player level
   If Level > False And Level < MaxInt Then ' compare level maximum
      Level = Level + 1 ' increment next level needed
      ' routine to calculate gold required for next level
      Call Gold(Gold.Required#)
      ' routine to calculate experience needed
      Call Experience(Exp.Required#)
      Var1$ = FormatString$(Gold.Required#)
      Var2$ = FormatString$(Exp.Required#)
      Outpt = "You need " + Var1$ + " gold and " + Var2$ + " experience to reach level" + Str$(Level) + "."
      Call IO.O ' send output message
      Exit Sub
   End If
   Outpt = "There is no gold or experience at your level."
   Call IO.O
End Sub

Rem * routine to display player characteristics, and weapons, shields, armor,
Rem * and rings being held/worn.

Sub Display.Info
   Graphics.Off = True ' reset color
   If Sysop Then ' verify user is a sysop
      Outpt = "You are a Sysop!" ' make message
      Call IO.O ' send message
   End If ' end verify sysop
   If Dungeon.Master Then ' verify user is a DM
      Outpt = "You are a Dungeon Master!" ' make message
      Call IO.O ' send message
   End If ' end verify DM
   If Dungeon.Master.Assistant Then ' verify user is an Asst. DM
      Outpt = "You are an Assistant Dungeon Master!" ' make message
      Call IO.O ' send message
   End If ' end verify Asst. DM
   If Town.Mayor Then ' verify user is the mayor
      Outpt = "You are the Town Mayor!" ' make message
      Call IO.O ' send message
   End If ' end verify mayor
   If Governor Then ' verify user is governor
      Outpt = "You are the Governor!" ' make message
      Call IO.O ' send message
   End If ' end verify governor
   If Guild.Master Then ' verify user is guild master
      Outpt = "You are the Guild Master!" ' make message
      Call IO.O ' send message
   End If ' end verify guild master
   If UserRecord.Invisible Or Invisible Then ' check invisibility
      Outpt = "You are invisible!" ' make message
      Call IO.O ' send message
   End If ' end check invisibility
   If UserRecord.Poison Then ' check poisoned
      Outpt = "You are poisoned!" ' make message
      Call IO.O ' send message
   End If ' end check poisoned
   If Weapon4 = False Then ' check wearing armor
      If Weapon7 = False Then ' check wearing ring
         Outpt = "You are wearing nothing." ' make message
         Call IO.O ' send message
      End If ' end check ring
   End If ' end check armor
   If Weapon5 = False Then ' check holding weapon
      If Weapon6 = False Then ' check holding shield
         Outpt = "You are holding nothing." ' make message
         Call IO.O ' send message
      End If ' end check shield
   End If ' end check weapon
   Outpt = Nul ' clear display string
   If Weapon4 Or Weapon7 Then ' check either armor or ring being worn
      Outpt = "You are wearing " ' initialize display string
      If Weapon4 Then ' check armor being worn
         Call Read.Record(TreasureFile, Abs(GetUserRecordInv(Weapon4))) ' get armor
         Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) ' append name
         Outpt = Outpt + "(+" + Mid$(Str$(Weapon1), 2) + ")" ' append plus
      End If ' end check armor worn
   End If ' end check either being worn
   If Weapon7 Then ' check ring being worn
      Call Read.Record(TreasureFile, Abs(GetUserRecordInv(Weapon7))) ' ring treasure
      If Weapon4 Then ' check armor worn again
         Outpt = Outpt + " and " ' append
      Else ' armor not worn
         Outpt = "You are wearing "
      End If ' end check armor worn
      Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) ' append name
      Outpt = Outpt + "(+" + Mid$(Str$(Weapon11), 2) + ")." ' append plus
      Call IO.O ' items being worn
   Else ' end check ring worn
      If Weapon4 Then ' check armor worn, ring not
         Outpt = Outpt + "." ' append period
         Call IO.O ' display only armor worn
      End If ' end check armor, ring
   End If ' end check ring worn
   Outpt = Nul ' clear display string
   If Weapon5 Or Weapon6 Then ' check either weapon or shield being held
      Outpt = "You are holding " ' initialize display string
      If Weapon6 Then ' check weapon being held
         Call Read.Record(TreasureFile, Abs(GetUserRecordInv(Weapon6))) ' get weapon
         Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) ' append name
         Outpt = Outpt + "(+" + Mid$(Str$(Weapon2), 2) + ")" ' append plus
      End If ' end check weapon held
   End If ' end check either being held
   If Weapon5 Then ' check shield being held
      Call Read.Record(TreasureFile, Abs(GetUserRecordInv(Weapon5))) ' shield record
      If Weapon6 Then ' check weapon held again
         Outpt = Outpt + " and " ' append
      Else ' weapon not held
         Outpt = "You are holding "
      End If ' end check weapon held
      Outpt = Outpt + RTrim$(TreasureRecord.TreasureName) ' append name
      Outpt = Outpt + "(+" + Mid$(Str$(Weapon3), 2) + ")." ' append plus
      Call IO.O ' items being held
   Else ' end check shield held
      If Weapon6 Then ' check weapon held, not shield
         Outpt = Outpt + "." ' append period
         Call IO.O ' display only weapon held
      End If ' end check weapon, shield
   End If ' end check shield held
End Sub ' end routine to display player characteristics

Rem * routine to display player character alignment and health statistics.

Sub Display.Health
   Graphics.Off = True ' reset color
   Call Show.Align ' routine to display alignment
   Call Show.Health ' routine to display health statistics
End Sub ' end routine to display alignment/health

Rem * routine to display health statistics, and weapon, shield, armor plus.

Sub Show.Health
   Graphics.Off = True ' reset color
   ' display vital health statistics in percentage form
   Outpt = "Vitals: " ' make output message
   Var1# = CDbl(UserRecord.FatigueMax) ' calculate total hits
   Var1# = Var1# + CDbl(UserRecord.VitalityMax) ' calculate total hits
   Var2# = CDbl(UserRecord.Fatigue) ' calculate hits remaining
   Var2# = Var2# + CDbl(UserRecord.Vitality) ' calculate hits remaining
   Outpt = Outpt + "Body" ' append health message
   If Var1# = 0# Then ' check divide by zero
      Temp# = 0# ' reset health hits
   Else ' check divide
      Temp# = ((Var2# / Var1#) * .40#) * 100# ' compute health percentage
      Temp# = Int(Temp#) ' compute health integer
      If Temp# < 1.0# Then ' check percentage
         Temp# = 1.0# ' reset percentage
      End If ' end check percentage
   End If ' end check divide by zero
   If Temp# < 0# Then ' check overflow
      Temp# = 0# ' reset percentage
   End If ' end check overflow
   Outpt = Outpt + Str$(Temp#) + "%" ' append health percentage
   Outpt = Outpt + " Arms" ' append health message
   If Var1# = 0# Then ' check divide by zero
      Temp# = 0# ' reset health hits
   Else ' check divide
      Temp# = ((Var2# / Var1#) * .25#) * 100# ' compute health percentage
      Temp# = Int(Temp#)
      If Temp# < 1.0# Then ' check percentage
         Temp# = 1.0# ' reset percentage
      End If ' end check percentage
   End If ' end check divide by zero
   If Temp# < 0# Then ' check overflow
      Temp# = 0# ' reset percentage
   End If ' end check overflow
   Outpt = Outpt + Str$(Temp#) + "%" ' append health percentage
   Outpt = Outpt + " Legs" ' append health message
   If Var1# = 0# Then ' check divide by zero
      Temp# = 0# ' reset health hits
   Else ' check divide
      Temp# = ((Var2# / Var1#) * .25#) * 100# ' compute health percentage
      Temp# = Int(Temp#)
      If Temp# < 1.0# Then ' check percentage
         Temp# = 1.0# ' reset percentage
      End If ' end check percentage
   End If ' end check divide by zero
   If Temp# < 0# Then ' check overflow
      Temp# = 0# ' reset percentage
   End If ' end check overflow
   Outpt = Outpt + Str$(Temp#) + "%" ' append health percentage
   Outpt = Outpt + " Head" ' append health message
   If Var1# = 0# Then ' check divide by zero
      Temp# = 0# ' reset health hits
   Else ' check divide
      Temp# = ((Var2# / Var1#) * .10#) * 100# ' cmpute health percentage
      Temp# = Int(Temp#)
      If Temp# < 1.0# Then ' check percentage
         Temp# = 1.0# ' reset percentage
      End If ' end check percentage
   End If ' end check divide by zero
   If Temp# < 0# Then ' check overflow
      Temp# = 0# ' reset percentage
   End If ' end check overflow
   Outpt = Outpt + Str$(Temp#) + "%" ' append health percentage
   Call IO.O ' send output
   ' health statistics line one contains vitals in form:
   ' Vitals: Fat 10/10(+10) Vit 10/10 Mag 10/10 Psi 10/10
   If Weapon4 Or Weapon5 Then ' check armor, shield
      ' add pluses to message
      Weapon.Plus$ = "(+" + Mid$(Str$(Weapon1 + Weapon3), 2) + ")"
   Else ' neither armor, shield
      Weapon.Plus$ = Nul ' clear plusses message
   End If ' end check armor, shield
   Outpt = "Vital hits:" ' initialize health one line
   ' append current fatigue and maximum fatigue
   Outpt = Outpt + " Fat" ' append stat name
   Outpt = Outpt + Str$(UserRecord.Fatigue) + "/" + Mid$(Str$(UserRecord.FatigueMax), 2)
   Outpt = Outpt + Weapon.Plus$ ' append plusses message
   ' append current vitality and maximum vitality
   Outpt = Outpt + " Vit" ' append stat name
   Outpt = Outpt + Str$(UserRecord.Vitality) + "/" + Mid$(Str$(UserRecord.VitalityMax), 2)
   ' append current magic points and maximum magic points
   Outpt = Outpt + " Mag" ' append stat name
   Outpt = Outpt + Str$(UserRecord.Magic) + "/" + Mid$(Str$(UserRecord.MagicMax), 2)
   ' append current psionic points and maximum psionic points
   Outpt = Outpt + " Psi" ' append stat name
   Outpt = Outpt + Str$(UserRecord.Psionic) + "/" + Mid$(Str$(UserRecord.PsionicMax), 2)
   Call IO.O ' display vitals message line
   ' health statistics line two contains vitals in form:
   ' Stats: Str 10(+10) Int 10 Wis 10 Dex 10 Con 10 Pie 10 Cha 10
   Outpt = "Stats: " ' initialize vitals message
   For Array.Index = 1 To 7 ' loop through all health statistics
      ' append first three letters of statistics name and player statistic value
      Outpt = Outpt + Left$(Stat(Array.Index), 3) + Str$(GetUserRecordStats(Array.Index))
      If Array.Index = 1 Then ' check strength selected
         If Weapon6 Then ' verify weapon being held
            Outpt = Outpt + "(+" + Mid$(Str$(Weapon2), 2) + ")" ' append weapon plus
         End If ' end check weapon
      End If ' end check strength
      Outpt = Outpt + " " ' append one space
   Next ' end loop through health statistics
   Call IO.O ' display vitals message line
   ' health statistics line three contains vitals in form:
   ' Weapons: Blunt> 0% Pole> 0% Sharp> 10% Thrusting> 0%
   Outpt = "Weapons: " ' initialize vitals message
   For Weapon.Number = 1 To 4 ' loop through all weapon classes
      Weapon$ = RTrim$(WeaponTypeName(Weapon.Number)) ' make weapon name
      Mid$(Weapon$, 1, 1) = UCase$(Mid$(Weapon$, 1, 1)) ' make weapon name
      Outpt = Outpt + Weapon$ ' append weapon class name
      ' append player weapon class percentage value
      Outpt = Outpt + Str$(GetUserRecordWeapons(Weapon.Number)) + "% "
   Next ' loop through weapon classes
   Call IO.O ' display vitals message line
   ' display lady statistics
   If UserRecord.ClassType = Lady Then ' compare user class type to lady
      ' make message for lady statistics
      Outpt = "Lady stats: Beauty " + Str$(UserRecord.Beauty) ' append beauty value
      Outpt = Outpt + " Glamour " + Str$(UserRecord.Glamour) ' append glamour value
      Call IO.O ' send lady statistics message
   End If ' end compare user class type
End Sub ' end routine to display health statistics

Rem * routine to display player character alignment.

Sub Show.Align
   Graphics.Off = True ' reset color
   If UserRecord.Level <= False Then ' check user level
      Outpt = "You are dead!" ' make user level message
      Call IO.O ' send output message
      Exit Sub ' exit routine
   End If ' end check user level
   Outpt = "You are a level" + Str$(UserRecord.Level) ' make user level message
   If UserRecord.Race <= False Then ' check valid user race
      UserRecord.Race = 1 ' reset user race
   End If ' end check valid user race
   Outpt = Outpt + " " + RTrim$(Race(UserRecord.Race)) + " " ' append user race name
   Inpt = UserRecord.ClassName ' store user class name
   Call Decrypt(Inpt) ' decrypt class name
   Outpt = Outpt + RTrim$(Inpt) + "." ' append classname
   Call IO.O ' send user type message
   Outpt = "You are aligned " ' make aligned message
   ' append player alignment type name 1 through 3 (-1,0,1 plus 2) number 1
   Outpt = Outpt + RTrim$(Alignment.Name1(UserRecord.Align1 + 2)) + " "
   ' append player alignment type name 1 through 3 (-1,0,1 plus 2) number 2
   Outpt = Outpt + RTrim$(Alignment.Name2(UserRecord.Align2 + 2)) + "."
   Call IO.O ' send player alignment message
End Sub ' end routine to display player character alignment

Rem * DM routine to display status of system.

Sub Display.Memory
   Graphics.Off = False ' reset color
   Outpt = "Dnddoor Author: " + Author$ ' get author name
   Call IO.O ' display author name string
   Outpt = "Author email: " + Email$ ' get email address
   Call IO.O ' display email address
   Outpt = _OS$ ' get system type
   Graphics.Off = True ' reset color
   Outpt = "This System: " + Outpt
   Call IO.O ' display system status message
End Sub ' end DM routine to display system memory

Rem * routine to display extended information on an object, treasure,
Rem * monster, or container using the identify command.

Sub Identify.Object
   If Normal.User Then ' check non DM status
      If UserRecord.Level <= 4 Then ' check player level
         Outpt = "You are not high enough level!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check level
   End If ' end check DM status
   Call Display.Information ' routine to display information on an item
End Sub ' end identify routine

Rem * routine to display list of weapons for sale, the first 15 items in the
Rem * treasure file.

Sub Weapon.List
   Graphics.Off = False ' reset color
   Outpt = "The Blacksmith says: Here's a list of my inventory." ' message
   Call IO.O ' send output message
   Graphics.Off = True ' reset color
   Outpt = "To purchase, enter number to buy, for example: Buy 15." ' make message
   Call IO.O ' send output message
   Graphics.Off = False ' reset color
   Outpt = "Number Weapon                        Weight Plus   Gold" ' make output
   Call IO.O ' send output message
   Allow.Break = True ' allow control-k breaking
   Break = False ' reset control-k flag
   Graphics.Off = True ' reset color
   For List.Count = 1 To 20 ' loop through the first 20 treasure items
      Call Read.Record(TreasureFile, List.Count) ' get the next treasure record
      Item.Weight = TreasureRecord.Weight ' store the treasure item weight
      Gold.Value# = TreasureRecord.Gold ' store the treasure item gold value
      ' store the treasure name
      WeaponList.Output$ = TreasureRecord.TreasureName
      ' set first character uppercase
      Mid$(WeaponList.Output$, 1, 1) = UCase$(Mid$(WeaponList.Output$, 1, 1))
      Weapon.Plus = False ' reset plus
      If TreasureRecord.Spell Then ' compare item to spell
         Call Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
         Weapon.Plus = SpellRecord.Level ' store spell level of item
      Else ' end compare spell item
         If TreasureRecord.Plus Then ' compare treasure item plus
            Weapon.Plus = Abs(TreasureRecord.Plus) ' store item plus
         End If ' end compare treasure item plus
      End If ' end compare item spell plus
      ' combine the treasure weight, gold value, and name with blanks imbedded
      Outpt = Mid$(Str$(List.Count), 2) ' append item value
      Outpt = Outpt + Space$(8 - Len(Str$(List.Count))) ' pad blanks
      Outpt = Outpt + WeaponList.Output$ ' append item value
      Outpt = Outpt + Space$(21 - Len(WeaponList.Output$)) ' pad blanks
      Outpt = Outpt + Mid$(Str$(Item.Weight), 2) ' append item value
      Outpt = Outpt + Space$(8 - Len(Str$(Item.Weight))) ' pad blanks
      Outpt = Outpt + Mid$(Str$(Weapon.Plus), 2) ' append item value
      Outpt = Outpt + Space$(8 - Len(Str$(Weapon.Plus))) ' pad blanks
      Outpt = Outpt + Mid$(Str$(Gold.Value#), 2) ' append item value
      Call IO.O ' send message output
      If Break Then ' check break
         Exit For ' exit treasure file loop
      End If ' end compare break
   Next ' end treasure file item display loop
   Allow.Break = False ' reset control-k breaking
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
End Sub ' end routine to list treasure items for sale

Rem * routine to allow user to change password.

Sub Change.PassWord
   Graphics.Off = True ' reset color
   Outpt = "Change your password(y/n)? " ' input prompt
   No.Input.Out = "N" ' default input
   Call IO.I ' get user input
   If Yes Then ' compare input
      Outpt = "Type in old password for verification:" ' input prompt
      Line.Length = 20 ' line length for password
      Hidden = True ' echo mask characters
      Call IO.I ' get user input
      Hidden = False ' reset echo mask flag
      Inpt = LTrim$(Inpt) ' trim entry password
      Inpt = RTrim$(Inpt) ' trim entry password
      Inpt = UCase$(Inpt) ' set entry password to uppercase
      Outpt = UserRecord.PassWord ' get user's current password
      Call Decrypt(Outpt) ' decrypt user password
      If Outpt = Nul Then ' verify password validity
         Outpt = "This password has a checksum error!" ' make error message
         Call IO.O ' send output message
         Exit Sub ' exit routine
      End If ' end compare password validity
      Outpt = RTrim$(Outpt) ' trim password
      If Outpt <> Inpt Then ' compare entered password to user password
         Outpt = "Passwords don't match!" ' make error message
         Call IO.O ' send output message
         Exit Sub ' exit routine
      End If ' end compare passwords
      Outpt = "Type in new password(20 char. max.)" ' format input message
      Call IO.O ' send output message
      Line.Length = 20 ' set line length of new password
      Outpt = "?" ' set input prompt
      Hidden = True ' set echo mask character flag
      Call IO.I ' get user input
      Hidden = False ' reset echo mask flag
      If No.Input Then ' check length of input
         Outpt = "Password not changed." ' make error message
         Call IO.O ' send output message
         Exit Sub ' exit routine
      End If ' end compare length of input
      Inpt = LTrim$(Inpt) ' trim new password
      Inpt = RTrim$(Inpt) ' trim new password
      Inpt = UCase$(Inpt) ' convert to uppercase
      Call Valid(Inpt, 20) ' check validity of new password
      If Inpt = Nul Then ' compare validity of new password
         Outpt = "Illegal characters in password!" ' make error message
         Call IO.O ' send output message
         Exit Sub ' exit routine
      End If ' end compare password validity
      Call Encrypt(Inpt, False) ' encrypt new password
      UserRecord.PassWord = Inpt ' store new password in user record
      Outpt = "Password changed." ' make message
      Call IO.O ' send output message
      Exit Sub ' exit routine
   End If ' end compare input
   Outpt = "Password not changed." ' make message
   Call IO.O ' send output message
End Sub ' end routine to change password

Rem * routine to change alignment once per player character.

Sub Align
   If Normal.User Then ' compare to non DM
      If UserRecord.Flags And Alignmented Then ' compare user record flag
         Outpt = "You've already changed alignment once!" ' message
         Call IO.O ' send output
         Exit Sub ' exit routine
      End If ' end compare user record flag
   End If ' end compare normal user
   Outpt = "Change alignment(y/n)? " ' input prompt
   No.Input.Out = "Y" ' default input
   Call IO.I ' get user input
   If Yes Then ' compare yes entered
      UserRecord.Flags = UserRecord.Flags Or Alignmented ' set user record flag
      Call Modify.Alignment ' routine to change alignment
      Outpt = "Alignment is now " ' message with new alignment
      Outpt = Outpt + RTrim$(Alignment.Name1(UserRecord.Align1 + 2)) + " " ' message
      Outpt = Outpt + RTrim$(Alignment.Name2(UserRecord.Align2 + 2)) ' message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare input
   Outpt = "Alignment not changed!" ' make output message
   Call IO.O ' send output message
End Sub ' end routine to change alignment

Rem * routine to allow user to change all statistics once per character.

Sub Reroll.Character
   If Normal.User Then ' compare to non DM
      If UserRecord.Flags And Rerolled Then ' check user record flag
         Outpt = "You've already rerolled your character!" ' message
         Call IO.O ' send output
         Exit Sub ' exit routine
      End If ' end compare user flag
   End If ' end compare normal user
   Outpt = "Reroll character(y/n)? " ' prompt user to reroll
   No.Input.Out = "Y" ' set default input
   Call IO.I ' get user input
   If Yes Then ' compare yes entered
      UserRecord.Flags = UserRecord.Flags Or Rerolled ' set user record flag
      Do ' loop until changes completed
         Outpt = "Character reroll:" ' message
         Call IO.O ' send output
         Outpt = "Change class type/name(y/n)? " ' input prompt
         No.Input.Out = "Y" ' set default input
         Call IO.I ' get user input
         If Yes Then ' compare yes entered
            Call Modify.Class ' routine to modify class type
         End If ' end compare yes entered
         Outpt = "Character reroll:" ' message
         Call IO.O ' send output
         Outpt = "Change vital statistics(y/n)? " ' input prompt
         No.Input.Out = "Y" ' set default input
         Call IO.I ' get user input
         If Yes Then ' compare yes entered
            Call Modify.Stats ' routine to modify statistics
         End If ' end compare yes entered
         Outpt = "Character reroll:" ' message
         Call IO.O ' send output
         Outpt = "Change character race type/name(y/n)? " ' input prompt
         No.Input.Out = "Y" ' set default input
         Call IO.I ' get user input
         If Yes Then ' compare yes entered
            Call Modify.Race ' routine to modify race
         End If ' end compare yes entered
         Outpt = "Character reroll:" ' message
         Call IO.O ' send output
         Outpt = "Change weapon proficiency(y/n)? " ' input prompt
         No.Input.Out = "Y" ' set default input
         Call IO.I ' get user input
         If Yes Then ' compare yes entered
            Call Modify.Proficiency ' routine to modify weapon proficiency
         End If ' end compare yes entered
         Outpt = "Character reroll:" ' message
         Call IO.O ' send output
         Outpt = "Change character alignment(y/n)? " ' input prompt
         No.Input.Out = "Y" ' set default input
         Call IO.I ' get user input
         If Yes Then ' compare yes entered
            Call Modify.Alignment ' routine to modify alignment
         End If ' end compare yes entered
         Do ' loop until changes finished prompt
            Outpt = "All changes finished(y/n)? " ' input prompt
            No.Input.Out = "Y" ' set default input
            Call IO.I ' get user input
            If Yes Then ' compare yes entered
               Exit Sub ' exit routine
            End If ' end compare
            If No Then ' compare no entered
               Exit Do ' exit changes loop
            End If ' end compare
         Loop ' loop until yes or no entered
      Loop ' end loop until changes completed
      Exit Sub ' exit routine
   End If ' end compare yes entered
   Outpt = "Your character has not been rerolled!" ' make output message
   Call IO.O ' send output message
End Sub ' end routine to modify all statistics

Rem * routine returns a prefix for monster name.
Rem * output variables:
Rem *   Prefix1 - monster name prefix.

Sub The.Or.An
   If MonsterArray(Monster.Number).Permanent < True Then ' check for nonplayer
      Prefix1 = "the " ' make prefix
   Else ' check monster type
      Prefix$ = MonsterArray(Monster.Number).MonsterName ' get monster name
      Prefix$ = Left$(Prefix$, 1) ' get first letter of monster name
      If InStr("aeiou", Prefix$) Then ' check monster name vowel
         Prefix1 = "an " ' set prefix
      Else ' check vowel
         Prefix1 = "a " ' set prefix
      End If ' end check monster name vowel
   End If ' end check nonplayer
End Sub ' end routine to get monster name prefix

Rem * routine for parsing numeric value from parameter.
Rem * input variables:
Rem *   Parsed.Command1 - string with imbedded pound sign to check.
Rem * output variables:
Rem *   Parse.Number - value of number after pound sign.
Rem * work variables:
Rem *   Delimit - position of # sign.

Sub Numeric
   Parse.Number = False ' reset numeric value
   Parse.Delimit = InStr(Parsed.Command1, "#") ' search parameter for # sign
   If Parse.Delimit Then ' check # sign in string
      ' store numeric value after #
      Parse.Number = Int(Val(Mid$(Parsed.Command1, Parse.Delimit + 1)))
      ' trim # from string
      Parsed.Command1 = Left$(Parsed.Command1, Parse.Delimit - 1)
   End If ' end check for # sign in string
End Sub ' end routine to parse part of parameter

Rem * routine decrements parameter # value after calls to search routines.
Rem * input variables:
Rem *   Parse.Count - counter for search routines.
Rem * output variables:
Rem *   Parse.Number - decremented # sign value counter.

Sub Num
   If Parse.Number > False Then ' check counter
      ' decrement search routine value from counter
      Parse.Number = Parse.Number - Parse.Count
      If Parse.Number < False Then ' check counter
         Parse.Number = False ' reset counter
      End If ' end check counter
   End If ' end check counter
End Sub ' end routine to decrement # sign value counter

Rem * routine to separate two parameters after command input.
Rem * input variables:
Rem *   Parsed.Command2 - first/second parameters combined.
Rem * output variables:
Rem *   Parsed.Command1 - first parsed parameter.
Rem *   Parsed.Command2 - second parsed parameter.
Rem * work variables:
Rem *   Delimit - position of # sign.

Sub Parse
   ' find imbedded space in command parameter
   Parse.Delimit = InStr(Parsed.Command2, " ")
   Parser = False ' reset position of space
   If Parse.Delimit Then ' check imbedded space
      ' store first parameter
      Parsed.Command1 = Left$(Parsed.Command2, Parse.Delimit - 1)
      ' store second parameter
      Parsed.Command2 = Mid$(Parsed.Command2, Parse.Delimit + 1)
      Parser = Parse.Delimit ' store parsed space position
   End If ' end check for space
End Sub ' end routine to separate parameters

Rem * routine to separate two parameters after command input in reverse order.
Rem * input variables:
Rem *   Parsed.Command2 - first/second parameters combined.
Rem * output variables:
Rem *   Parsed.Command1 - second parsed parameter.
Rem *   Parsed.Command2 - first parsed parameter.
Rem * work variables:
Rem *   Delimit - position of # sign.

Sub ParseX
   ' find imbedded space in command parameter
   Parse.Delimit = InStr(Parsed.Command2, " ")
   Parser = False ' reset position of space
   If Parse.Delimit Then ' check imbedded space
      ' store second parameter
      Parsed.Command1 = Mid$(Parsed.Command2, Parse.Delimit + 1)
      ' store first parameter
      Parsed.Command2 = Left$(Parsed.Command2, Parse.Delimit - 1)
      Parser = Parse.Delimit ' storeparsed space position
   End If ' end check for space
End Sub ' end routine to separate parameters in reverse order

Rem * routine computes gold player needs for next training level.
Rem * output variables:
Rem *   Gold.Required# - gold points.

Sub Gold (Gold.Required#)
   If UserRecord.Level <= 10 Then ' check player level
      Gold.Required# = 2 ^ (UserRecord.Level + 5) ' calculate gold
   Else ' player level over ten
      Gold.Required# = 2 ^ 15 + (UserRecord.Level - 10) * 10000! ' calculate gold
   End If ' end check player level
End Sub ' end routine to calculate gold

Rem * routine computes experience player needs for next training level.
Rem * output variables:
Rem *   Exp.Required# - experience points.

Sub Experience (Exp.Required#)
   If UserRecord.Level <= 10 Then ' check player level
      Exp.Required# = 2 ^ (UserRecord.Level + 6) ' calculate experience
   Else ' player level over ten
      Exp.Required# = 2 ^ 16 + (UserRecord.Level - 10) * 10000! ' calculate experience
   End If ' end check player level
End Sub ' end routine to calculate experience

Rem * routine returns range of numbers.
Rem * input variables:
Rem *   Upper.Range - contains upper range.
Rem * output variables:
Rem *   Start.Range - start of range.
Rem *   End.Range - end of range.

Sub Get.Range (Upper.Range, Start.Range, End.Range)
   Range.Type$ = Mid$(Str$(Upper.Range), 2) ' convert upper range to string
   Outpt = "From(1-" + Range.Type$ + ")? " ' make input prompt
   No.Input.Out = "1" ' default input
   Call IO.I ' get input
   Start.Range = Int(Val(Inpt)) ' convert input to integer
   If Start.Range < 1 Then ' check bounds of input
      Start.Range = 1 ' reset input
   End If ' end check bounds
   If Start.Range > Upper.Range Then ' check bounds of input
      Start.Range = Upper.Range ' reset input
   End If ' end check bounds
   Outpt = "To(" + Mid$(Str$(Start.Range), 2) + "-" + Range.Type$ + ")? " ' input prompt
   No.Input.Out = Range.Type$ ' default input
   Call IO.I ' get input
   End.Range = Int(Val(Inpt)) ' convert input to integer
   If End.Range < Start.Range Then ' check bounds
      End.Range = Start.Range ' reset input
   End If ' end check bounds
   If End.Range > Upper.Range Then ' check bounds
      End.Range = Upper.Range ' reset input
   End If ' end check bounds
End Sub ' end routine to get range of numbers

Rem * routine returns range of numbers.
Rem * input variables:
Rem *   Start.Range - starting of range.
Rem *   End.Range - end of range.
Rem * output variables:
Rem *   Upper.Range - contains upper range.

Sub Get.Range2 (Start.Range, End.Range, Upper.Range)
   Start.Range$ = Mid$(Str$(Start.Range), 2) ' convert starting range to string
   End.Range$ = Mid$(Str$(End.Range), 2) ' convert upper range to string
   Outpt = Outpt + "(" + Start.Range$ + "-" + End.Range$ + ")? " ' make input prompt
   No.Input.Out = Start.Range$ ' default input
   Call IO.I ' get input
   Upper.Range = Int(Val(Inpt)) ' convert input to integer
   If Upper.Range < Start.Range Then ' check bounds of input
      Upper.Range = Start.Range ' reset input
   End If ' end check bounds
   If Upper.Range > End.Range Then ' check bounds of input
      Upper.Range = End.Range ' reset input
   End If ' end check bounds
End Sub ' end routine to get range of numbers

Rem * routine returns range of single precision numbers.
Rem * input variables:
Rem *   Upper.Range! - contains upper range.
Rem * output variables:
Rem *   Start.Range! - start of range.
Rem *   End.Range! - end of range.

Sub Get.Room.Range (Upper.Range!, Start.Range!, End.Range!)
   Range.Type$ = Mid$(Str$(Upper.Range!), 2) ' convert upper range to string
   Outpt = "From(1-" + Range.Type$ + ")? " ' make input prompt
   No.Input.Out = "1" ' default input
   Call IO.I ' get input
   Start.Range! = Int(Val(Inpt)) ' convert input to integer
   If Start.Range! < 1! Then ' check bounds of input
      Start.Range! = 1! ' reset input
   End If ' end check bounds
   If Start.Range! > Upper.Range! Then ' check bounds of input
      Start.Range! = Upper.Range! ' reset input
   End If ' end check bounds
   Outpt = "To(" + Mid$(Str$(Start.Range!), 2) + "-" + Range.Type$ + ")? " ' input prompt
   No.Input.Out = Range.Type$ ' default input
   Call IO.I ' get input
   End.Range! = Int(Val(Inpt)) ' convert input to integer
   If End.Range! < Start.Range! Then ' check bounds
      End.Range! = Start.Range! ' reset input
   End If ' end check bounds
   If End.Range! > Upper.Range! Then ' check bounds
      End.Range! = Upper.Range! ' reset input
   End If ' end check bounds
End Sub ' end routine to get range of numbers

Rem * routine returns range of single precision numbers.
Rem * input variables:
Rem *   Start.Range - starting of range.
Rem *   End.Range - end of range.
Rem * output variables:
Rem *   Upper.Range - contains upper range.

Sub Get.Room.Range2 (Start.Range!, End.Range!, Upper.Range!)
   Start.Range$ = Mid$(Str$(Start.Range!), 2) ' convert starting range to string
   End.Range$ = Mid$(Str$(End.Range!), 2) ' convert upper range to string
   Outpt = Outpt + "(" + Start.Range$ + "-" + End.Range$ + ")? " ' make input prompt
   No.Input.Out = Start.Range$ ' default input
   Call IO.I ' get input
   Upper.Range! = Int(Val(Inpt)) ' convert input to integer
   If Upper.Range! < Start.Range! Then ' check bounds of input
      Upper.Range! = Start.Range! ' reset input
   End If ' end check bounds
   If Upper.Range! > End.Range! Then ' check bounds of input
      Upper.Range! = End.Range! ' reset input
   End If ' end check bounds
End Sub ' end routine to get range of numbers

Rem * routine returns the charges of an item of treasure.
Rem * output variables:
Rem *   Charges.Amount - stores treasure type charges.

Sub TreasureCharges (Charges.Amount)
   Charges.Amount = TreasureRecord.Charges ' store treasure charges
   If TreasureRecord.FuelType Then ' compare treasure to fuel
      Charges.Amount = TreasureRecord.FuelCharges ' reset treasure charges
   End If ' end compare fuel charges
   If TreasureRecord.LightType Then ' compare treasure to light
      Charges.Amount = False ' reset treasure charges
   End If ' compare charges
   If TreasureRecord.Vehicle Then ' compare treasure to vehicle
      Charges.Amount = TreasureRecord.VehicleHits ' reset treasure charges
   End If ' compare charges
End Sub ' end routine to return charges

Rem * routine to wish for an item
Rem * input variables:
Rem *   Inpt - stores name of item

Sub Wish.Item
   Outpt = "The Ghods Thunder.." ' make output message
   Call IO.O ' send message
   Outpt = "   What Do You Wish For?" ' make input prompt
   Call IO.I ' get input
   Stored.Parsed.Command2 = Inpt ' store input
   Parsed.Command1 = Stored.Parsed.Command2 ' store input
   Call Numeric ' parse number
   Inpt = Parsed.Command1 ' restore input
   Inpt = LCase$(Inpt) ' convert to lowercase
   Call Drop(False) ' call routine to get item
End Sub

Rem * routine to wish for points or an item, or get an object or treasure.
Rem * input variables:
Rem *   Drop.Type - false to use normal drop routine, true for extended drop.

Sub Drop (Drop.Type)
   Wish.Points = 1 ' store points number to wish for
   If Right$(Inpt, 7) = " points" Then ' compare wish for two points
      Inpt = Left$(Inpt, Len(Inpt) - 7) ' truncate wish parameter
      Wish.Points = 2 ' store points number to wish for
   End If ' end compare points wish
   For Stat.Number = 1 To 7 ' loop through statistic names
      Outpts = Stat(Stat.Number) ' get statistic name
      Outpts = RTrim$(Outpts) ' trim name
      Outpts = LCase$(Outpts) ' lowercase name
      If Inpt = Outpts Then ' compare wish item to statistic name
         If Normal.User Then ' check non DM
            ' check point already wished for
            If (UserRecord.Flags And 2 ^ Stat.Number) Then
               GoTo Wish.Denied ' jump to wish denied subroutine
            End If ' end check point wished for
         End If ' end check normal player
         ' add player wish bitflag
         UserRecord.Flags = (UserRecord.Flags Or 2 ^ Stat.Number)
         Wish.Points = Wish.Points * Int(Rnd * 3 + 1) ' calculate points to add
         ' calculate new statistic
         New.Stat# = GetUserRecordStats(Stat.Number) + Wish.Points
         If New.Stat# > MaxInt Then ' compare maximum integer
            New.Stat# = MaxInt ' reset to maximum integer
         End If ' end check maximum integer
         New.Stat = CInt(New.Stat#) ' store in integer
         If Normal.User Then ' check non DM
            If New.Stat > MaxStat Then ' check maximum statistic allowed
               GoTo Wish.Denied ' jump to wish denied subroutine
            End If ' end check maximum stat
         End If ' end check normal player
         Call PutUserRecordStats(Stat.Number, New.Stat) ' increment point wished for
         Graphics.Off = True ' reset color
         Outpt = "The Ghods Thunder..." ' make ghod message
         Call IO.O ' send message
         Outpt = "   Your " + Outpts + " Has Been Raised!" ' make stat message
         Call IO.O ' send update stat message
         Graphics.Off = False ' reset color
         Exit Sub ' exit routine
      End If ' end compare point wish
   Next ' end loop through statistic names
   If Drop.Type = False Then ' check drop type
      If Normal.User Then ' check normal player
         If UserRecord.Flags And Wished Then ' check player has already wished
            GoTo Wish.Denied ' jump to wish denied subroutine
         End If ' end check already wished
      End If ' end check normal user
   End If ' end check drop type
   UserRecord.Flags = UserRecord.Flags Or Wished ' set player wish bitflag
   Parse.Value = False ' item counter
   Wish.Charges = False ' item charges
   Wish.Index = False ' item index
   ' loop through treasure file
   For Treasure.Number = 1 To LOF(TreasureFile) / Len(TreasureRecord)
      Call Read.Record(TreasureFile, Treasure.Number) ' get next record
      Outpts = TreasureRecord.TreasureName ' store treasure name
      Outpts = Left$(Outpts, Len(Inpt)) ' truncate name
      If Inpt = Outpts Then ' compare treasure name to wish item name
         Parse.Value = Parse.Value + 1 ' increment item counter
         ' compare counters
         If Parse.Number = False Or Parse.Value = Parse.Number Then
            Wish.Index = Treasure.Number ' store treasure file number
            Call TreasureCharges(Wish.Charges) ' routine to get treasure charges
            Exit For ' exit loop through treasure file
         End If ' end compare counters
      End If ' end compare names
   Next ' end loop through treasure file
   If Wish.Index = False Then ' check no treasure match found
      If Normal.User = False Or Drop.Type Then ' compare DM/Sysop or drop type
         ' loop through object file
         For Object.Number = 1 To LOF(ObjectFile) / Len(ObjectRecord)
            Call Read.Record(ObjectFile, Object.Number) ' get object record
            Outpts = ObjectRecord.ObjectName ' store object name
            Outpts = Left$(Outpts, Len(Inpt)) ' truncate object name
            If Inpt = Outpts Then ' compare object name to wish name
               Parse.Value = Parse.Value + 1 ' increment counter
               ' compare counters
               If Parse.Number = False Or Parse.Value = Parse.Number Then
                  ' store negation of object file index
                  Wish.Index = -Object.Number
                  Wish.Charges = False ' clear charges
                  Exit For ' exit loop through object file
               End If ' end compare counters
            End If ' end compare names
         Next ' end loop through object file
      End If ' end compare drop type/DM, Sysop
   End If ' end check treasure found
   If Drop.Type = False Then ' check drop type
      If Wish.Index > False Then ' check treasure found
         If Normal.User Then ' check normal player/not DM
            If TreasureRecord.Container Then ' check treasure container
               Wish.Index = False ' clear treasure found
            Else ' check treasure
               If TreasureRecord.Vehicle Then ' check treasure vehicle
                  Wish.Index = False ' clear treasure found
               Else ' check treasure
                  Spell.Number = TreasureRecord.Spell ' get treasure spell
                  ' check spell file bounds
                  If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
                     Call Read.Record(SpellFile, Spell.Number) ' get spell record
                     If SpellRecord.SpellType = 4 Then ' check spell type wish
                        Wish.Index = False ' clear treasure found
                     End If ' end check wish spell
                  End If ' end check spell file bounds
               End If ' end check treasure
            End If ' end check treasure
         End If ' end check normal player
      End If ' and check treasure found to drop
   End If ' end check drop type
   Drop.Type = False ' clear drop flag
   Select Case Wish.Index ' selection of item type to drop
      Case Is < False ' check object being dropped
         ' add object to room
         Call Add.Room.Object(Abs(Wish.Index), False, Drop.Type)
      Case Is > False ' check treasure being dropped
         Select Case TreasureRecord.Container ' selection of container dropped
            Case False ' check treasure container
               ' add treasure to room
               Call Add.Room.Treasure(Wish.Index, Wish.Charges, False, Drop.Type)
            Case True ' check container
               ' check container name
               If RTrim$(RoomRecord.Container.ShortName) = Nul Then
                  Drop.Type = True ' set drop flag
                  ' store container variables
                  ContainerRec.Closed = TreasureRecord.Closed
                  ContainerRec.ContainerName = TreasureRecord.TreasureName
                  ContainerRec.Locked = TreasureRecord.Locked
                  ContainerRec.Keyed = TreasureRecord.Keyed
                  ContainerRec.ShortName = TreasureRecord.ShortName
                  ContainerRec.Permanent = TreasureRecord.Permanent
                  For Container.Count = 1 To 5 ' loop through container contents
                     ' clear container contents
                     Call Clear.Container(Container.Count, False)
                  Next ' end loop through container
                  RoomRecord.Container = ContainerRec ' add container record to room
                  Call Share.Room.Record(Room) ' write room record
               End If ' end check container
         End Select ' end select container
   End Select ' end select treasure
   If Drop.Type = False Then ' check drop flag
      GoTo Wish.Denied ' jump to wish denied subroutine
   End If ' end check drop flag
   Graphics.Off = True ' reset color
   Outpt = "A Dark Cloud Passes Overhead..." ' make ghod message
   Call IO.O ' send message
   Outpt = "   Some Treasure Falls From The Sky..." ' make ghod message
   Call IO.O ' send message
   Outpt = "The Cloud Disappears..." ' make ghod message
   Call IO.O ' send message
   Graphics.Off = False ' reset color
   Exit Sub ' exit routine

   Wish.Denied:
   Graphics.Off = True ' reset color
   Outpt = "The Ghods Thunder..." ' make ghod message
   Call IO.O ' send message
   Outpt = "   Your Wish Is Denied!" ' make ghod message
   Call IO.O ' send message
   Graphics.Off = False ' reset color
End Sub ' end routine to drop item to ground

Rem * Filename: dnds3.bas Version: v5.0a r2.0a
Rem * This subprogram contains most of the attack and player death routines.

Rem * routine to attack a monster, or object.
Rem * input variables:
Rem *   Parsed.Command1 - contains the attack target.
Rem *   Room - contains room number user is in.
Rem * work variables:
Rem *   Prefix1 - contains prefix to target (he/the/it).
Rem *   (constants) - command numbers.

Sub Attack.Monster
   Prefix1 = Nul ' reset target prefix
   Monster.Number = False ' reset monster target number
   If Room = 1! Then ' compare room number
      Outpt = "This is a safe haven!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare room number
   Call Check.Room.Objects ' verify target is an object in room
   If Index.Number Then ' compare object index
      If ObjectRecord.JailTrap Then ' check object jails attacker
         Jail.Room.Number = ObjectRecord.RoomLink
         Call Jail ' routine to send player to jail room
      Else ' compare check jail object
         Outpt = "Nothing happens.." ' make message
         Call IO.O ' send message
      End If ' end compare jail object
      Exit Sub ' exit routine
   End If ' end compare object index
   Previous.Command = False ' reset number of previous attack command
   If Intoxicated > False Then ' compare player is drunk
      Outpt = "You are too drunk to fight!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare drunk player
   Call Check.Monster ' verify target is monster
   If Monster.Number = False Then ' check monster index
      If Parser = False Then ' verify command type
         Call Get.Last.Monster(Monster.Found) ' routine to get default monster
         If Monster.Found Then ' check monster exists
            Parsed.Command1 = Last.Monster ' store monster name
         End If ' end check monster
      End If ' end verufy command
      If Monster.Number = False Then ' verify target found
         Outpt = "You can't attack that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end verify target
   End If ' end compare monster target
   If MonsterArray(Monster.Number).Jail Then ' verify monster jails attacker
      Jail.Room.Number = MonsterRecord.Teleport ' store jail room number
      Call Jail ' routine to send player to jail room
      Exit Sub ' exit routine
   End If ' end verify jail monster
   Call The.Or.An ' routine to get prefix
   Outpts = MonsterArray(Monster.Number).MonsterName ' store monster target name
   Outpts = RTrim$(Outpts) ' trim monster name
   Outpts = LCase$(Outpts) ' lowercase monster name
   Select Case Last.Command.Number ' make selection of attack command number
      Case Charm ' charm
         If UserRecord.ClassType = Lady Then ' compare class type of player
            If MonsterArray(Monster.Number).Magic <= True Then ' check magic monster
               Outpt = "Didn't work!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end check magic monster
            If Int(Rnd * 10 + 4) > UserRecord.Beauty Then ' calculate lady hits
               Outpt = "Didn't work!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end calculate hits
            MonsterArray(Monster.Number).Magic = 5 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare class type
         Outpt = "Your class does not allow that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Beguile ' beguile
         If UserRecord.ClassType = Lady Then ' compare class type of player
            If Int(Rnd * 12 + 5) > (UserRecord.Beauty + UserRecord.Glamour) Then ' calculate
               Outpt = "Didn't work!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end calculate monster death
            Call Monster.Died ' calculated monster died
            Exit Sub ' exit routine
         End If ' end compare class type
         Outpt = "Your class does not allow that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Circled ' circle
         If MonsterArray(Monster.Number).Magic <= True Then ' compare magic monster
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare magic monster
         ' compute
         Calculate# = (Rnd * 5 + UserRecord.Level) > MonsterArray(Monster.Number).Level
         If Calculate# Then ' compare circle monster
            Previous.Command = True ' reset previous attack command number
            MonsterArray(Monster.Number).Magic = 4 ' update befuddle monster rounds
            Outpt = "It worked!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare circled monster
         Outpt = "Didn't work!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Feint ' feint
         If MonsterArray(Monster.Number).Magic <= True Then ' compare magic monster
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare magic monster
         Calculate# = GetUserRecordStats(4) > (Rnd * 5 + MonsterArray(Monster.Number).Level / 2)
         If Calculate# Then ' compare calculated befuddle
            Previous.Command = Last.Command.Number ' store last command number
            MonsterArray(Monster.Number).Magic = 3 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare calculation
         Outpt = "Didn't work!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Resist, Parry ' parry, resist
         Previous.Command = Last.Command.Number ' store last command number
         Outpt = "It worked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Turn ' turn
         Magic.Spell = 12 ' force turn spell cast
      Case Bemuse ' bemuse
         If UserRecord.ClassType = Lady Then ' compare player class type
            If MonsterArray(Monster.Number).LevelDrain Then ' check monster type
               Outpt = "Didn't work!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end compare class type
            If Int(Rnd * 8 + 5) > UserRecord.Glamour Then ' calculate lady level drain
               Outpt = "Didn't work!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end calculate level drain
            MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
            Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
            Call IO.O ' send message
            If MonsterArray(Monster.Number).Level <= False Then ' check monster level
               Outpt = "You killed the " + Outpts + "!" ' make message
               Call IO.O ' send message
               Call Monster.Died ' routine for dead monster
               Exit Sub ' exit routine
            End If ' end compare dead monster
         End If ' end compare lady class
         Outpt = "Your class does not allow that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Befuddle ' befuddle
         If MonsterArray(Monster.Number).Magic <= True Then ' compare magic monster
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare magic monster
         If (GetUserRecordStats(3) + GetUserRecordStats(4)) > (Rnd * 10 + MonsterArray(Monster.Number).Level / 2) Then ' calculate befuddle
            Previous.Command = True ' reset previous command number
            MonsterArray(Monster.Number).Magic = 6 ' update monster befuddle rounds
            Outpt = "It worked!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end calculate befuddle
         Outpt = "Didn't work!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Lunge, Dodge ' lunge, dodge
         If Weapon5 Then ' verify player holding shield
            Weapon3 = False ' reset shield
            Weapon5 = False ' reset shield
            Outpt = "You return your shield!" ' make message
            Call IO.O ' send message
         End If ' end verify shield
      Case Beat, Punch ' beat, punch
         If Weapon6 Then ' verify holding weapon
            Weapon2 = False ' reset weapon
            Weapon6 = False ' reset weapon
            Weapon10 = False ' reset weapon
            Outpt = "You return your weapon!" ' make message
            Call IO.O ' send message
         End If ' end verify weapon
      Case Shield, Guard ' shield, guard
         If Weapon3 Then ' verify holding shield
            Previous.Command = True ' reset previous command
            Outpt = "It worked!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end verify shield
         Previous.Command = False ' reset previous command
         Outpt = "You are not holding a shield!" ' make error message
         Call IO.O ' send message
         Exit Sub ' exit routine
   End Select ' end select command number
   Select Case Magic.Spell ' select attack of spell number
      Case Poison ' poison spell
         If MonsterArray(Monster.Number).Poison Then ' check monster is poisoned
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check monster
         If Rnd > .5 Then ' random chance
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random chance
         ' poison monster rounds
         MonsterArray(Monster.Number).Poison = UserRecord.Level
         Outpt = "You poison the " + Outpts + "!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case LevelDrain ' leveldrain spell
         If MonsterArray(Monster.Number).LevelDrain Then ' check monster is undead
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check monster
         If Rnd > .5 Then ' random chance
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check random chance
         MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
         Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
         Call IO.O ' send message
         If MonsterArray(Monster.Number).Level <= False Then ' check monster level
            Outpt = "You killed the " + Outpts + "!" ' make message
            Call IO.O ' send message
            Call Monster.Died ' routine for dead monster
         End If ' end check monster level
         Exit Sub ' exit routine
      Case Befuddled ' befuddle spell
         If MonsterArray(Monster.Number).Magic <= True Then ' check magic monster
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check magic monster
         If Rnd > .5 Then ' random chance
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
         End If ' end chance
         MonsterArray(Monster.Number).Magic = 4 ' update monster befuddle rounds
         Outpt = "It worked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case TurnUndead ' turn undead spell
         ' check monster undead
         If MonsterArray(Monster.Number).LevelDrain = False Then
            Outpt = "That's not undead!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check monster
         Calculate# = (Rnd * UserRecord.Level + 2) > MonsterArray(Monster.Number).Level
         If Calculate# Then ' calculate cast spell
            Outpt = "You damned the " + Outpts + "!" ' make message
            Call IO.O ' send message
            MonsterArray(Monster.Number).Level = MonsterArray(Monster.Number).Level - 1
            Outpt = "You drain a level from the " + Outpts + "!" ' decrement monster level
            Call IO.O ' send message
            If MonsterArray(Monster.Number).Level <= False Then ' check monster level
               Outpt = "You killed the " + Outpts + "!" ' make message
               Call IO.O ' send message
               Call Monster.Died ' routine for dead monster
               Exit Sub ' exit routine
            End If ' end check monster level
         End If ' end calculate undead spell
         Outpt = "Didn't work!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      Case Intoxicate ' intoxicate spell
         If MonsterArray(Monster.Number).Magic <= True Then ' check magic monster
            Intoxicated = UserRecord.Level ' store intoxicated rounds counter
            Outpt = "You just became very drunk!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check magic monster
         If Rnd > .5 Then ' random chance
            Outpt = "Didn't work!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check chance
         MonsterArray(Monster.Number).Magic = 3 ' update monster befuddle rounds
         Outpt = "It worked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
   End Select ' end selection of magic spell number
   If Magic.Spell = False Then ' compare spell being cast
      If UserRecord.Fatigue <= False Then ' check player fatigue
         UserRecord.Fatigue = False ' reset fatigue
         Outpt = "You are too exhausted to fight!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check player fatigue
      ' normal attack can't hit magical monsters, only spells can
      If MonsterArray(Monster.Number).Magic <= True Then ' check magic monster
         Outpt = "A magical force prevents your attack!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check magic monster
   End If ' end compare spell being cast
   ' nonplayers can't be attacked by normal weapons or spells.
   ' check permanent nonplayer
   If MonsterArray(Monster.Number).Permanent < True Then
      Outpt = "A mysterious force prevents your attack!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check nonplayer
   ' calculate miss on monster
   Calculate# = (Rnd * 10 + MonsterArray(Monster.Number).Level / 10) > GetUserRecordStats(4)
   If Calculate# Then ' player missed
      If Magic.Spell Then ' check spell being cast
         Outpt = "You missed!" ' spell missed message
         Call IO.O ' send output message
         Exit Sub ' exit routine
      End If ' end check spell being cast
      ' attacking with a weapon
      Calculate# = (Rnd * 10) > GetUserRecordStats(1) ' calculate fumble
      If Calculate# Then ' fumbled
         Call Fumble ' routine to drop weapon, shield
      End If ' end fumble check
      Outpt = "You missed the " + Outpts + "!" ' make miss message
      Call IO.O ' send output message
      Exit Sub ' exit routine
   End If ' end check player missed
   ' calculate attack weapon bonus multiplier
   If Magic.Spell = False Then ' verify attack by weapon, not spell
      Outpt = Nul ' reset output string
      Multiplier = 1 ' reset multiplier
      Select Case Weapon6 ' select type of weapon being held
         Case False ' no weapon being held
            Outpt = "Punch!" ' punch with fists
         Case Else ' weapon is being held
            Select Case Rnd ' select random chance
               Case Is > .96 ' highest probability
                  Outpt = "Deathly damage!" ' make damage message
                  Multiplier = 4 ' set multiplier
               Case Is > .91 ' median probability
                  Outpt = "Triple damage!" ' make damage message
                  Multiplier = 3 ' set multiplier
               Case Is > .86 ' lowest probability
                  Outpt = "Double damage!" ' make damage message
                  Multiplier = 2 ' set multiplier
            End Select ' end selection of random chance for weapon
      End Select ' end selection of weapon type
   End If ' end verify weapon attack
   Select Case Last.Command.Number ' selection of attack command number
      Case Killed ' kill
         If Outpt <> Nul Then ' check bonus hit string
            Call IO.O ' send bonus output message
         End If ' end check bonus flag
      Case Lunge, Dodge ' lunge, dodge
         Multiplier = 2 ' store lunge/dodge multiplier
         Previous.Command = Last.Command.Number ' store last command attack number
      Case Backstab ' backstab
         If Hidden.Player Then ' verify player hidden for backstab
            Multiplier = Multiplier + 1 ' increment hit multiplier
         Else ' player not hidden
            Outpt = "The " + Outpts + " sees you!" ' make message
            Call IO.O ' send message
            Multiplier = Multiplier - 1 ' decrement multiplier
         End If ' end verify hidden player
      Case Thrust ' thrust
         Multiplier = Multiplier + 1 ' increment hit multiplier for thrust
         Previous.Command = Last.Command.Number ' store last command attack number
      Case Charge ' charge
         Outpt = "Argh! You charge towards the " + Outpts + "!" ' make hit message
         Call IO.O ' send hit message
         Multiplier = Multiplier + 1 ' increment hit multiplier
         Previous.Command = Last.Command.Number ' store last command attack number
      Case Counter ' counter
         Multiplier = Multiplier + 1 ' increment multiplier for counter attack
      Case Pummel ' pummel
         Outpt = "Your fists fly at the monster!" ' make hit message
         Call IO.O ' send hit message
         Multiplier = 2 ' set hit multiplier
      Case Beat, Punch ' beat, punch
         Multiplier = 2 ' set hit multiplier
         Previous.Command = Last.Command.Number ' store last command attack number
      Case Bewitch ' bewitch
         Multiplier = Multiplier + 1 ' increment hit multiplier for bewitch attack
         Previous.Command = Last.Command.Number ' store last command attack number
      Case Bewilder ' bewilder
         Multiplier = Multiplier + 1 ' increment hit multiplier for bewilder attack
         Previous.Command = Last.Command.Number ' store last command attack number
   End Select ' end selection of attack command number
   If Magic.Spell = False Then ' verify weapon being used, not spell cast
      If Weapon6 Then ' check player holding weapon
         ' decrement weapon strikes remaining.
         ' store weapon strikes remaining
         Charges.Number = GetUserRecordCharges(Weapon6)
         If Charges.Number > False Then ' check weapon strikes remaining
            Charges.Number = Charges.Number - 1 ' decrement weapon strikes
         End If ' end check strikes remaining
         ' store weapon strikes in user record
         Call PutUserRecordCharges(Weapon6, Charges.Number)
         If Charges.Number = False Then ' verify weapon strikes
            Call Read.Record(TreasureFile, GetUserRecordInv(Weapon6)) ' get record
            Inpt = TreasureRecord.ShortName ' store weapon mnemonic
            Inpt = RTrim$(Inpt) ' trim mnemonic
            Inpt = LCase$(Inpt) ' lowercase mnemonic
            Outpt = "Your " + Inpt + " breaks in half!" ' make weapon message
            Call IO.O ' send weapon message
            Call Discard.Inventory(Weapon6, False) ' weapon falls to ground
         End If ' end verify remaining weapon strikes
      End If ' end check player holding weapon
   End If ' end verify weapon being used
   Magic.Spell = True ' reset spell cast flag
   Call Hit.Monster ' get actual hits on monster
End Sub ' end routine to attack monster

Rem * routine to send player to jail room number.

Sub Jail
   Outpt = "You were just thrown in jail!" ' make message
   Call IO.O ' send jail message
   If Jail.Room.Number = 0! Then ' check jail room number
      Next.Room = 1! ' reset player room number
   Else ' check jail room number
      Next.Room = Jail.Room.Number ' store new room number
   End If ' end check jail room number
   Call Enter.Room ' send player to jail room
End Sub ' end routine to jail player

Rem * routine to decrease the number of monsters in the room.

Sub Reduce.Monsters
   Inpt = Parsed.Command1 ' store command parameter
   Inpt = RTrim$(Inpt) ' trim command parameter
   New.Monsters = Int(Val(Inpt)) ' convert parameter to integer
   ' compare number to reduce
   If New.Monsters >= False And New.Monsters < Number.Monsters Then
      Number.Monsters = New.Monsters ' reduce monsters in room
      Outpt = "Number of Monsters reduced to:" + Str$(Number.Monsters) + ".."
      Call IO.O ' send monsters reduced message
   End If ' end compare number to reduce
End Sub ' end routine to decrease monsters in room

Rem * routine to kill off a monster.

Sub Kill.Monster
   Monster.Number = False ' reset monster number
   Call Check.Monster ' get monster number from command parameter
   If Monster.Number Then ' compare monster name found to kill
      Graphics.Off = True ' reset color
      Outpt = "Evil Laughter Sounds From Above..." ' make message
      Call IO.O ' send message
      Outpt = "   A Bolt Of Lightning Strikes..." ' make message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "The " + Outpts + " was just struck dead!" ' make monster died message
      Call IO.O ' send monster died message
      Call Monster.Died ' kill monster
      Exit Sub ' exit routine
   End If ' end compare monster name
   Outpt = "You can't kill that!" ' make error message
   Call IO.O ' send error message
End Sub ' end routine to kill monster

Rem * routine to attack monster.
Rem * input variables:
Rem *   Monster.Number - number of monster in array to attack.
Rem *   Outpts - name of monster being attacked.
Rem * processing variables:
Rem *   Calculate# - maximum possible hits on monster.

Sub Hit.Monster
   Hidden.Player = False ' reset hidden player
   If Last.Command.Number = PsiMode Then ' compare attack type
      ' calculate hits on monster for psionic attack
      Calculate# = CDbl(Int(Rnd * GetUserRecordStats(2) + GetUserRecordStats(3) * (Psi.Attack.Mode + Multiplier)))
   Else ' compare attack type
      ' calculate hits on monster for normal attack
      Calculate# = CDbl(Int((Rnd * GetUserRecordStats(1) + UserRecord.Level / 2) * (Multiplier + (GetUserRecordWeapons(UserRecord.Proficiency) / 100)) + Weapon2))
   End If ' end compare attack type
   Magic.Spell = False ' reset magic spell being used to attack
   If Calculate# <= False Then ' check hits are zero
      Outpt = "You missed!" ' make missed message
      Call IO.O ' send missed message
      Exit Sub ' exit routine
   End If ' end check hits
   Outpt = "You hit the " + Outpts + " for" + Str$(Calculate#) + " hits!" ' hits message
   Call IO.O ' send hits message
   Call Monster.Flees(Ran.Away) ' check if monster flees
   If Ran.Away Then ' compare flee
      Call Monster.Died ' routine for dead monster
      Exit Sub ' exit routine
   End If ' end compare flee
   ' magical monsters don't attack until hit.
   ' check magical monster attacked
   If MonsterArray(Monster.Number).Magic = True Then
      MonsterArray(Monster.Number).Magic = -2 ' set magical monster to attack also
   End If ' end check attacking a magical monster
   ' decrement the hits on the monster
   MonsterArray(Monster.Number).Hits = MonsterArray(Monster.Number).Hits - Calculate#
   If MonsterArray(Monster.Number).Hits <= False Then ' check monster still alive
      Outpt = "You just killed the " + Outpts + "!" ' make message
      Call IO.O ' send message
      Call Monster.Died ' routine for dead monster
   End If ' end check live monster
End Sub ' end routine to attack monster
 
Rem * routine to calculate hits causes monster to run.
Rem * output variables:
Rem *   Ran.Away - true if monster flees.
Rem * input variables:
Rem *   Outpts - name of monster.

Sub Monster.Flees (Ran.Away)
   Ran.Away = False ' reset return variable
   If MonsterArray(Monster.Number).Magic >= False Then ' nonmagical monster
      If MonsterArray(Monster.Number).Permanent = False Then ' nonpermanent monster
         ' calculate hits on monster
         If MonsterArray(Monster.Number).Hits - Calculate# > False Then
            ' calculate 25% of hits
            If Calculate# > (MonsterArray(Monster.Number).Hits * .75) Then
               Outpt = "The " + Outpts + " flees from your attack!" ' make message
               Call IO.O ' send message
               MonsterArray(Monster.Number).Experience = Int(MonsterArray(Monster.Number).Experience / 2) ' half experience
               Ran.Away = True ' reset return variable
            End If ' end calculate hits on monster is 25%
         End If ' end check if monster has hits remaining after attack
      End If ' end verify permanent monster
   End If ' end verify magical monster
End Sub ' end routine to check monster flees

Rem * routine for dead monster.
Rem * input variables:
Rem *   Monster.Number - index of monster to array.
Rem * processing variables:
Rem *   Monsters.Killed - total monsters killed by player this session.
Rem *   Choice - index of monster file.
Rem *   Weapon6, Weapon10 - proficiency calculations.

Sub Monster.Died
   ' check permanent nonplayer
   If MonsterArray(Monster.Number).Permanent < True Then
      Call Remove.Monster ' nonplayer only leaves room
      Exit Sub ' exit routine
   End If ' end check nonplayer
   Graphics.Off = True ' reset color
   Last.Command.Number = False ' reset last attack command number
   Outpt = "On it you find " ' format message prefix
   Carriage.Return = True ' disable cr/lf
   Call IO.O ' send message prefix
   Number.Items = False ' number of items
   ' loop through monster array for all treasure to leave on ground
   For Treasure.Index = 1 To 5 ' loop through monster inventory
      ' get next inventory
      Treasure.Number = GetMonsterRecordTreasure(Treasure.Index)
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Call TreasureCharges(Charges.Amount) ' routine to get treasure charges
         ' add inventory to ground
         Call Add.Room.Treasure(Treasure.Number, Charges.Amount, False, Item.Added)
         If Item.Added Then ' compare return variable to add to room
            Carriage.Return = True ' disable cr/lf
            Call IO.O ' send message of previous treasure name
            ' format treasure name
            Outpt = RTrim$(TreasureRecord.TreasureName) + ", "
            Number.Items = Number.Items + 1 ' increment number of items
         End If ' end compare reutnr variable
      End If ' end compare file bounds
   Next ' end loop through monster inventory
   If Number.Items = False Then ' compare number of items
      Outpt = "nothing.." ' format message
   Else ' compare items
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' strip comma, add period
      If Number.Items > 1 Then ' compare more than one item
         Outpt = "and " + Outpt ' append to message
      End If ' end compare number of items
   End If ' end compare number of items
   Call IO.O ' send last part of message
   Call Share.Room.Record(Room) ' write the room record
   Monsters.Killed = Monsters.Killed + 1 ' increment player monsters killed session
   ' increment player record
   UserRecord.MonstersKilled = UserRecord.MonstersKilled + 1
   Choice = MonsterIndex(Monster.Number) ' store monster record number
   If MonsterArray(Monster.Number).Permanent = True Then ' check permanent monster
      Call Read.Record(MonsterFile, Choice) ' get permanent monster record
      MonsterRecord.Permanent = False ' clear permanent monster flag
      Call Share.Record(MonsterFile, Choice) ' put permanent monster record
   End If ' end check permanent monster
   If Weapon6 And Weapon10 Then ' check player is holding a weapon
      ' calculate difference between player and monster level
      Level = MonsterArray(Monster.Number).Level - UserRecord.Level
      If Level > False Then ' check level is positive
         ' store player weapon proficiency number
         Proficient.Weapon = UserRecord.Proficiency
         ' check player is holding a weapon in his weapon proficiency
         If Weapon10 = Proficient.Weapon Then
            Level = Level * 10 ' compute proficiency for kill
         Else ' compare weapon proficiency for weapon held
            Level = Level * 5 ' compute proficiency for kill
         End If ' end compare weapon proficiency held
         If Level > 100 Then ' check calculated proficiency over 100 percent
            Level = 100 ' reset to 100 percent
         End If ' end check maximum proficiency percent
         ' compare to current
         If Level > GetUserRecordWeapons(Proficient.Weapon) Then
            ' proficiency, store new percent
            Call PutUserRecordWeapons(Proficient.Weapon, Level)
            Outpt = "Your " + RTrim$(WeaponTypeName(Proficient.Weapon)) + " weapon proficiency is now" + Str$(Level) + " percent!" ' make message
            Call IO.O ' send weapon message
         End If ' end check maximum percent
      End If ' end check attack level
   End If ' end check player holding weapon
   Exp.Points# = MonsterArray(Monster.Number).Experience ' store monster experience
   Gold.Points# = MonsterArray(Monster.Number).Gold ' store monster gold
   If Gold.Points# <= False Then ' compare monster gold
      Gold.Points# = 10 ' set to minimum
   End If ' end compare monster gold
   Outpt = "You gained" + Str$(Exp.Points#) + " experience points" ' make message
   If Gold.Points# > False Then ' compare gold
      Outpt = Outpt + " and" + Str$(Gold.Points#) + " gold" ' append gold message
   End If ' end compare gold
   Outpt = Outpt + "!" ' append message
   Call IO.O ' send experience and gold message
   UserRecord.Gold = UserRecord.Gold + Gold.Points# ' increment player gold
   ' increment player experience
   UserRecord.Experience = UserRecord.Experience + Exp.Points#
   Call Remove.Monster ' take monster out of active monster array
   If UserRecord.Level > False Then ' verify player level
      ' calculate experience needed to reach next level
      Call Experience(Exp.Required#)
      If UserRecord.Experience >= Exp.Required# Then ' compare player experience
         Call Gold(Gold.Required#) ' routine calculates gold needed for level
         If UserRecord.Gold >= Gold.Required# Then ' compare to player gold
            Call Is.Training.Room(In.Room) ' check training room
            If In.Room Then ' check flag
               Call Train.Stats ' routine to train for next level
            End If ' end check flag
         End If ' end check player gold required
      End If ' end check player experience required
   End If ' end check player level
End Sub ' end monster death routine

Rem * routine to remove a monster from the monster arrays.
Rem * input variables:
Rem *   Monster.Number - number of monster to remove.
Rem * output variables:
Rem *   Number.Monsters - number of monster currently in room.

Sub Remove.Monster
   ' loop through monster array from monster to remove
   For Array.Index = Monster.Number To 19 ' pack array
      ' move next array element
      MonsterIndex(Array.Index) = MonsterIndex(Array.Index + 1)
      ' move next array element
      MonsterArray(Array.Index) = MonsterArray(Array.Index + 1)
   Next ' end loop through all monsters in arrays
   Number.Monsters = Number.Monsters - 1 ' decrement number of monsters in room
   If Number.Monsters < False Then ' compare number of monsters in room
      Number.Monsters = False ' reset numberof monsters to zero
   End If ' end compare number of monsters
End Sub ' end routine to remove a monster

Rem * routine for monsters to attack player.

Sub Monster.Attack
   Monster.Cycle = Monster.Cycle + 1 ' increment attack counter
   If Monster.Cycle >= Room.Monster.Rate Then ' compare attack counter to
      Monster.Cycle = False ' room monster attack rate counter, reset to zero
   End If ' end compare rate counters
   If Number.Monsters = False Then ' check monsters in room
      Exit Sub ' exit routine if no monsters
   End If ' end check monsters in room
   If Room = 1! Then ' check player in safe room
      Exit Sub ' exit routine for safe room
   End If ' end check player in safe room
   For Monster.Number = 1 To Number.Monsters ' loop through all monsters in room
      ' get monster befuddle rounds
      Inactive.Rounds = MonsterArray(Monster.Number).Magic
      If Inactive.Rounds >= False Then ' compare number of befuddle rounds
         If GetUserRecordStats(6) <= 1 Then ' check player piety
            Inactive.Rounds = False ' monsters attack on sight for low piety
         End If ' end check player piety
         ' compare inactive befuddle rounds for monster
         If Inactive.Rounds > False Then
            Inactive.Rounds = Inactive.Rounds - 1 ' decrement befuddle rounds
         End If ' end compare monster befuddle rounds
         ' store monster befuddle rounds
         MonsterArray(Monster.Number).Magic = Inactive.Rounds
      End If ' end compare number of monster befuddle rounds
      If Inactive.Rounds < True Then ' check if magical monster is attacking
         ' reset befuddle rounds remaining for monster to zero
         Inactive.Rounds = False
      End If ' end check magical monster is attacking
      ' routine for monster hit regeneration
      If MonsterArray(Monster.Number).Magic <= True Then ' check magical monster
         If MonsterArray(Monster.Number).Level >= 15 Then ' check monster level
            If Monster.Cycle = False Then ' compare room monster update rate
               Calculate# = MonsterArray(Monster.Number).Hits ' store monster hits
               ' regenrate hits by ten percent
               Calculate# = CDbl(Int(Calculate# + Calculate# * .1))
               If Calculate# > MaxInt Then ' check max integer
                  ' store new monster hits
                  MonsterArray(Monster.Number).Hits = MaxInt
               End If ' end check integer
               If Calculate# <= MaxInt Then ' check max integer
                  ' store new monster hits
                  MonsterArray(Monster.Number).Hits = Calculate#
               End If ' end check integer
            End If ' end compare room monster update rate
         End If ' end check monster level
      End If ' end check magical monster
      ' continue with monster attack if player is not hidden,
      ' and monster is not befuddled.
      ' continue with monster attack
      If Hidden.Player = False And Inactive.Rounds = False Then
         Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
         Outpts = RTrim$(Outpts) ' trim monster name
         Outpts = LCase$(Outpts) ' lowercase monster name
         ' compute monster attack multiplier is monster level plus one
         Multiplier = MonsterArray(Monster.Number).Level + 1 ' compute multiplier
         If MonsterArray(Monster.Number).Poison Then ' check monster is poisonous
            ' compute random chance of monster poison percent
            If Rnd < (MonsterArray(Monster.Number).PoisonPercent / 100) Then
               Outpt = "The " + Outpts + " casts a poison spell!" ' make message
               Call IO.O ' send poison message
               ' check player is wearing antipoison ring
               If Weapon8 = 1 Then ' check for ring
                  Ring.Charges = GetUserRecordCharges(Weapon7) ' store ring strikes
                  ' decrement strikes
                  Ring.Charges = Ring.Charges - MonsterArray(Monster.Number).Level
                  If Ring.Charges < False Then ' compare strikes remaining
                     Ring.Charges = False ' store zero strikes
                  End If ' end compare strikes
                  ' store strikes in record
                  Call PutUserRecordCharges(Weapon7, Ring.Charges)
                  Outpt = "Your ring absorbs the poison spell!" ' make message
                  Call IO.O ' send ring protection message
                  If Ring.Charges = False Then ' compare ring charges remaining
                     Outpt = "Your ring disintegrated!" ' make message
                     Call IO.O ' send ring message
                     Call Discard.Inventory(Weapon7, True) ' ring falls to ground
                  End If ' end compare ring charges
               Else ' check for ring
                  UserRecord.Poison = True ' reset user poisoned flag
                  Outpt = "You've been poisoned!" ' make message
                  Call IO.O ' send poisoned message
               End If ' end compare ring
            End If ' end compute monster poisons
         End If ' end compare monster is poisonous
         ' check monster is undead
         If MonsterArray(Monster.Number).LevelDrain Then
            ' compute random chance vs. undead monster leveldrain percent
            If Rnd < (MonsterArray(Monster.Number).DrainPercent / 100) Then ' compute
               Outpt = "The " + Outpts + " casts a level drain spell!" ' message
               Call IO.O ' send level drain message
               ' check player is wearing anti level drain ring
               If Weapon8 = 2 Then ' check for ring
                  Ring.Charges = GetUserRecordCharges(Weapon7) ' store ring strikes
                  ' decrement ring charges
                  Ring.Charges = Ring.Charges - MonsterArray(Monster.Number).Level
                  If Ring.Charges < False Then ' compare strikes
                     Ring.Charges = False ' reset ring strikes
                  End If ' end compare strikes
                  ' store srikes in record
                  Call PutUserRecordCharges(Weapon7, Ring.Charges)
                  Outpt = "Your ring absorbs the level drain!" ' make message
                  Call IO.O ' send ring message
                  If Ring.Charges = False Then ' check ring charges
                     Outpt = "Your ring disintegrated!" ' make message
                     Call IO.O ' send ring charges message
                     Call Discard.Inventory(Weapon7, True) ' ring falls to ground
                  End If ' end check remaining ring charges
               Else ' check player wearing ring
                  UserRecord.Level = UserRecord.Level - 1 ' decrement player level
                  Call New.Stats ' get updated statistics
                  Outpt = "You've been drained a level!" ' make message
                  Call IO.O ' send leveldrain message
                  If UserRecord.Level <= False Then ' compare player level to dead
                     UserRecord.Level = 1 ' reset player level
                     Call The.Or.An ' get monster prefix
                     ' store message for death routine
                     Message1 = "You were just killed by " + Prefix1 + " " + Outpts + "!"
                     Call Player.Died ' routine for dead player
                     Exit For ' exit monster attack loop
                  End If ' end compare player level
               End If ' end check player has ring
            End If ' end compute random chance
         End If ' end check monster is undead
         Magic.Spell = False ' reset magic spell being cast
         ' get monster spell cast number
         Spell.Number = MonsterArray(Monster.Number).Spell
         ' file bounds
         If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
            Call Read.Record(SpellFile, Spell.Number) ' get spell record
            Magic.Spell = SpellRecord.SpellType ' store monster spell cast type
            Multiplier = Int(MonsterArray(Monster.Number).Level) ' calculate spell
            Multiplier = Multiplier + Int(SpellRecord.Level / 2 + .5) ' attack multiplier
            If Magic.Spell = Offense Then ' compare spell type to offense attack
               ' compute random chance to monster spell percent
               If Rnd < (MonsterArray(Monster.Number).SpellPercent / 100) Then
                  ' make message of spell monster casts
                  Outpt = "The " + Outpts + " casts a " + RTrim$(SpellRecord.SpellName) + " spell!" ' message
                  Call IO.O ' send cast message
                  ' check player is wearing antispell ring
                  If Weapon8 = 3 Then ' check ring
                     ' check ring is generic antispell or the spell cast
                     If Weapon9 = True Or Weapon9 = Spell.Number Then ' check ring
                        ' store ring strikes
                        Ring.Charges = GetUserRecordCharges(Weapon7)
                        ' subtract spell level
                        Ring.Charges = Ring.Charges - SpellRecord.Level
                        If Ring.Charges < False Then ' compare strikes
                           Ring.Charges = False ' reset strikes
                        End If ' end compare strikes
                        Call PutUserRecordCharges(Weapon7, Ring.Charges) ' store strikes
                        Outpt = "Your ring absorbs the spell!" ' make message
                        Call IO.O ' send ring message
                        ' check remaining ring charges
                        If Ring.Charges = False Then ' compare
                           Outpt = "Your ring disintegrated!" ' make message
                           Call IO.O ' send ring message
                           Call Discard.Inventory(Weapon7, True) ' falls to ground
                        End If ' end compare ring charges
                     End If ' end ring type
                  End If ' end check ring
               End If ' end computer random chance
            End If ' end compare spell attack type
         End If ' end check spell file bounds
         ' check if monster is poisoned
         If MonsterArray(Monster.Number).Poison > False Then ' check poisoned
            If Monster.Cycle = False Then ' compare rounds counter
               ' decrement monster poisoned rounds counter
               MonsterArray(Monster.Number).Poison = MonsterArray(Monster.Number).Poison - 1 ' decrement
               Outpt = "Poison drains the " + Outpts + "!" ' make message
               Call IO.O ' send poisoned message
               ' calculate hits on monster from poison.
               ' store monster hits
               Monster.Hits# = MonsterArray(Monster.Number).Hits
               ' store monster level
               Monster.Level# = MonsterArray(Monster.Number).Level
               ' compute poison
               Poison.Hits# = Int(UserRecord.Level * GetUserRecordStats(7))
               ' compute poison
               Monster.Hits# = Int(Monster.Hits# - Poison.Hits# / Monster.Level#)
               ' store monster hits
               MonsterArray(Monster.Number).Hits = Monster.Hits#
               ' check monster hits
               If MonsterArray(Monster.Number).Hits <= False Then
                  Outpt = "The " + Outpts + " died from poison!" ' make message
                  Call IO.O ' send died message
                  Call Monster.Died ' routine for dead monster
                  Exit For ' exit monster attack routine
               End If ' end check monster hits
            End If ' end compare rounds counter
         End If ' end check monster is poisoned
         ' calculate monster misses
         Calculate# = (GetUserRecordStats(4) / 2) > (Rnd * 10 + MonsterArray(Monster.Number).Level / 10) ' calculate miss
         If Calculate# Then ' compare miss equation
            Outpt = "The " + Outpts + " fumbled!" ' make message
            Call IO.O ' send missed message
         Else ' compare miss
            If Previous.Command = True Then ' compare previous attack command type
               Multiplier = Multiplier - 1 ' decrement monster multiplier
               Previous.Command = False ' reset last attack command type
            End If ' end compare previous attack command type
            ' compare psionic monster
            If MonsterArray(Monster.Number).Psionic = False Then ' compare
               ' calculate normal hits on player
               Calculate# = CDbl(Int(Rnd * (MonsterArray(Monster.Number).Hits / 10 - (Weapon1 + Weapon3) / 2) * Multiplier))
            Else ' calculate psionic hits on player
               ' calculate psionic hits on player
               Calculate# = CDbl(Int(Rnd * (MonsterArray(Monster.Number).Hits / 10 - Psi.Defense.Mode) * MonsterArray(Monster.Number).Psionic))
            End If ' end compare attack type
            Call The.Or.An ' get prefix of monster name
            Prefix1 = "The " ' reset prefix
            Call Get.Hits(Calculate#) ' routine to hit player
            If Calculate# = True Then ' return variable for dead player
               Exit For ' exit monster attack loop
            End If ' end check dead player
         End If ' end compare monster miss equation
      End If ' end compare monster befuddle rounds
   Next ' end loop through all monsters in room
End Sub ' end routine for monster attacks

Rem * routine to hit player.
Rem * input variables:
Rem *   Calculate# - actual hits.
Rem * output variables:
Rem *   Calculate# - true if player died.

Sub Get.Hits (Calculate#)
   ' selecting the previous attack command player used modifies the actual
   ' hits by the type of attack, and reduces or increases the actual hits
   Select Case Previous.Command ' select the previous attack command
      Case Lunge ' lunge
         Calculate# = Calculate# - GetUserRecordStats(4) / 2 - GetUserRecordStats(1) / 2
      Case Dodge ' dodge
         Calculate# = Calculate# - (GetUserRecordStats(4) + GetUserRecordStats(1)) / 2
      Case Feint ' feint
         Calculate# = Calculate# - GetUserRecordStats(4) / 2 - GetUserRecordStats(1)
      Case Parry ' parry
         Calculate# = Calculate# - GetUserRecordStats(4) - GetUserRecordStats(1)
      Case Thrust ' thrust
         Calculate# = Calculate# + GetUserRecordStats(4) / 2 + GetUserRecordStats(1)
      Case Charge ' charge
         Calculate# = Calculate# + GetUserRecordStats(4) + GetUserRecordStats(1)
      Case Resist ' resist
         Calculate# = Calculate# - GetUserRecordStats(1) / 2
      Case Beat ' beat
         Calculate# = Calculate# + GetUserRecordStats(4) / 2 + GetUserRecordStats(1) / 2
      Case Punch ' punch
         Calculate# = Calculate# + (GetUserRecordStats(4) + GetUserRecordStats(1)) / 2
      Case Bewitch ' bewitch
         Calculate# = Int(Calculate# / 2) - UserRecord.Beauty
      Case Bewilder ' bewilder
         Calculate# = Int(Calculate# / 2) - UserRecord.Glamour
   End Select ' end selection of previous attack command
   If Calculate# <= False Then ' compare hits on player to miss
      Outpt = Prefix1 + Outpts + " missed!" ' make message
      Call IO.O ' send miss message
      Calculate# = False ' return value for live player
      Exit Sub ' exit routine
   End If ' end compare hits to miss
   If Monster.Number Then ' check monster is attacking
      If Magic.Spell = False Then ' check monster using spell
         Outpt = Prefix1 + Outpts + " attacks you" ' make attack message
         ' check monster using psionic
         If MonsterArray(Monster.Number).Psionic Then
            Outpt = Outpt + " with psionics" ' append psionic attack message
         End If ' end compare monster using psionics
         Outpt = Outpt + "!" ' append exclamation
         Call IO.O ' send attack message
      End If ' end compare monster using spell
   End If ' end compare monster attacking
   If UserRecord.Invisible Then ' check player is invisible
      Outpt = "Your invisibility protects you!" ' make message
      Call IO.O ' send invisible message
      Calculate# = False ' return value for live player
      Exit Sub ' exit routine
   End If ' end compare invisible player
   If Vehicle2 Then ' check if player is using a vehicle
      If Rnd > .75 Then ' random chance vehicle is hit
         Call Read.Record(TreasureFile, Vehicle3) ' get vehicle treasure record
         Inpt = TreasureRecord.ShortName ' store vehicle name
         Inpt = RTrim$(Inpt) ' trim name
         Inpt = LCase$(Inpt) ' lowercase name
         Outpt = Prefix1 + Outpts + " hits your " + Inpt + " for" + Str$(Calculate#) + " hits!"
         Call IO.O ' send message of hits on vehicle
         Vehicle2 = Vehicle2 - Calculate# ' decrement vehicle hits
         Call PutRoomRecordTreCharges(Vehicle1, Vehicle2) ' put vehicle in room
         If Vehicle2 <= False Then ' compare remaining vehicle hits
            Call PutRoomRecordTreasure(Vehicle1, Vehicle3) ' put vehicle in room
            Call PutRoomRecordTreCharges(Vehicle1, False) ' put vehicle in room
            Vehicle1 = False ' remove vehicle from player
            Vehicle2 = False ' remove vehicle from player
            Vehicle3 = False ' remove vehicle from player
            Outpt = Prefix1 + Outpts + " damaged your " + Inpt + "!" ' make message
            Call IO.O ' send vehicle message
         End If ' end compare vehicle hits
      End If ' end random chance to hit vehicle
      Calculate# = False ' return value for live player
      Call Share.Room.Record(Room) ' write room record
      Exit Sub ' exit routine
   End If ' end check player using vehicle
   Outpt = Prefix1 + Outpts + " hits you for"
   Call Hit.Player(Calculate#) ' routine to decrement hits on player statistics
   If UserRecord.Vitality = False Then ' check player vitality remaining
      Calculate# = True ' set player died flag
      Exit Sub ' exit routine
   End If ' end check player died
   Calculate# = False ' return value for live player
   If Weapon5 Then ' check player holding shield
      ' store shield strikes remaining
      Charges.Number = GetUserRecordCharges(Weapon5)
      If Charges.Number > False Then ' check shield strikes
         Charges.Number = Charges.Number - 1 ' decrement shield strikes
      End If ' end check shield strikes
      ' store shield strikes in user record
      Call PutUserRecordCharges(Weapon5, Charges.Number)
      If Charges.Number = False Then ' compare shield charges
         ' get shield treasure record
         Call Read.Record(TreasureFile, GetUserRecordInv(Weapon5))
         Inpt = TreasureRecord.ShortName ' store treasure name
         Inpt = RTrim$(Inpt) ' trim name
         Inpt = LCase$(Inpt) ' lowercase name
         Outpt = "Your " + Inpt + " breaks in half!" ' make message
         Call IO.O ' send weapon message
         Call Discard.Inventory(Weapon5, False) ' shield falls to ground
      End If ' end compare remaining shield charges
   End If ' end check player holding shield
   If Weapon4 Then ' check player is holding a armor
      Charges.Number = GetUserRecordCharges(Weapon4) ' store armor strikes remaining
      If Charges.Number > False Then ' compare armor strikes
         Charges.Number = Charges.Number - 1 ' decrement armor strikes
      End If ' end compare armor strikes
      ' store armor strikes in user record
      Call PutUserRecordCharges(Weapon4, Charges.Number)
      If Charges.Number = False Then ' check armor strikes remaining
         ' get treasure armor record
         Call Read.Record(TreasureFile, GetUserRecordInv(Weapon4))
         Inpt = TreasureRecord.ShortName ' store treasure name
         Inpt = RTrim$(Inpt) ' trim name
         Inpt = LCase$(Inpt) ' lowercase name
         Outpt = "Your " + Inpt + " crumbles!" ' make message
         Call IO.O ' send armor message
         Call Discard.Inventory(Weapon4, False) ' falls to ground
      End If ' end compare remaining armor charges
   End If ' end check player holding armor
End Sub ' end routine to get hits on player

Rem * routine to decrement hits on player from fatigue and vitality.
Rem * input variables:
Rem *   Hit.Points - hits to subtract, positive from fatigue,
Rem *   negative from vitality.

Sub Hit.Player (Hit.Points#)
   If Hit.Points# > False Then ' check decrement from fatigue
      If UserRecord.Fatigue - Hit.Points# >= False Then ' compare the decrement
         ' subtract from fatigue
         UserRecord.Fatigue = UserRecord.Fatigue - Hit.Points#
         Outpt = Outpt + Str$(Hit.Points#) + " fatigue points!" ' make message
         Call IO.O ' send hits message
         Exit Sub ' exit routine
      End If ' end compare the decrement
      ' get the difference from hits on fatigue
      Hit.Points# = Hit.Points# - UserRecord.Fatigue
      If UserRecord.Fatigue > False Then ' compare hits to fatigue remaining
         Outpt = Outpt + Str$(UserRecord.Fatigue) + " fatigue and" ' make message
      End If ' end compare fatigue message, remaining difference hits vitlaity
      UserRecord.Fatigue = False ' reset player fatigue to zero
   End If ' end decrement from fatigue
   Hit.Points# = Abs(Hit.Points#) ' convert hits on vitality to positive integer
   If Hit.Points# > False Then ' compare vitiality hits
      Outpt = Outpt + Str$(Hit.Points#) + " vitality points!" ' make message
      Call IO.O ' send hits message
      If UserRecord.Vitality - Hit.Points# <= False Then ' compare the decrement
         UserRecord.Vitality = False ' reset vitality
         Message1 = "You were just killed!" ' make died routine message
         Call Player.Died ' routine for dead player
         Exit Sub ' exit routine
      End If ' end compare vitality decrement
      ' decrement player vitality
      UserRecord.Vitality = UserRecord.Vitality - Hit.Points#
   End If ' end compare vitality hits
End Sub ' end routine to decrement hits on player

Rem * routine to fumble weapon and shield from player to ground.

Sub Fumble
   If Weapon5 Or Weapon6 Then ' check if holding weapon or shield
      Outpt = "You fumbled! " ' make first message
      Inpt = "You dropped your " ' make second part of message
      Graphics.Off = True ' reset color
      Carriage.Return = True ' disable cr/lf
      Call IO.O ' send first fumble message
      If Weapon5 Then ' check shield being held
         ' get shield treasure record
         Call Read.Record(TreasureFile, GetUserRecordInv(Weapon5))
         Treasure.Name$ = TreasureRecord.ShortName ' get treasure name
         Treasure.Name$ = RTrim$(Treasure.Name$) ' trim name
         Treasure.Name$ = LCase$(Treasure.Name$) ' lowercase name
         Call Discard.Inventory(Weapon5, False) ' falls to ground
         If Weapon6 Then ' check weapon also being held
            Outpt = " and " + Treasure.Name$ ' make next part of message
         Else ' check weapon
            Outpt = Inpt + Treasure.Name$ + "!" ' make message for shield only
         End If ' end check weapon being held
      End If ' end check shield held
      If Weapon6 Then ' check weapon being held
         ' get weapon treasure record
         Call Read.Record(TreasureFile, GetUserRecordInv(Weapon6))
         Treasure.Name$ = TreasureRecord.ShortName ' get treasure name
         Treasure.Name$ = RTrim$(Treasure.Name$) ' trim name
         Treasure.Name$ = LCase$(Treasure.Name$) ' lowercase name
         Outpt = Inpt + Treasure.Name$ + Outpt + "!" ' make last part of message
         Call Discard.Inventory(Weapon6, False) ' falls to ground
      End If ' end check weapon held
      Call IO.O ' send fumble message
      Graphics.Off = False ' reset color
   End If
End Sub ' end routine to fumble weapon and shield

Rem * routine to get permanent monster encounters in a room.
Rem * input variables:
Rem *   Room - number of room player is currently in.

Sub Encounter.Permanent
   ' loop through all nonplayer records
   For Monster.Index = 1 To LOF(NonPlayerFile) / Len(MonsterRecord)
      Call Read.Record(NonPlayerFile, Monster.Index) ' get nonplayer record
      ' store nonplayer room
      Monster.Rooms$ = "," + RTrim$(MonsterRecord.PluralName) + ","
      Player.Room$ = "," + Mid$(Str$(Room), 2) + "," ' store current room
      ' verify nonplayer in current room
      If InStr(Monster.Rooms$, Player.Room$) Then
         If Number.Monsters < 20 Then ' check maximum monsters in room
            Number.Monsters = Number.Monsters + 1 ' increment monsters in room
            MonsterIndex(Number.Monsters) = Monster.Index ' store nonplayer index
            MonsterArray(Number.Monsters) = MonsterRecord ' store nonplayer record
            MonsterArray(Number.Monsters).Magic = True ' set magical monster
            MonsterArray(Number.Monsters).Permanent = -2 ' set nonplayer flag
         End If ' end check maximum monsters
      End If ' end verify current room
   Next ' end loop through nonplayer records
   If RoomRecord.MonsterClass Then ' check room monster class number
      For Monsters = 1 To 10 ' loop through room monsterclass monsters
         ' get monster number
         Choice = Monster.Class(RoomRecord.MonsterClass, Monsters)
         If Choice > False And Choice <= LOF(MonsterFile) / Len(MonsterRecord) Then ' bounds
            Call Read.Record(MonsterFile, Choice) ' get monster record
            If MonsterRecord.Permanent = True Then ' check if monster permanent
               Number.Appearing = 1 ' set number of monsters to appear to one
               Call Get.Monster.Stats ' get the monster
            End If ' end check permanent monster
         End If ' end check file bounds
      Next ' end loop through room monsterclass
   End If ' end check room monster class number
End Sub ' end routine to get permanent monsters

Rem * routine to get one monster.
Rem * input variables:
Rem *   Parsed.Command1 - name or number of monster.
Rem * working variables:
Rem *   Choice - is the monster number.

Sub Summon.Monster
   If Room = 1! Then ' check player in safe room
      Outpt = "This is a safe haven!" ' display message
      Call IO.O ' send message
      Exit Sub ' exit routine for safe room
   End If ' end check player in safe room
   Choice = False ' reset monster number
   Inpt = Parsed.Command1 ' store command parameter
   Call Parse.Num(Inpt, Parse.Value) ' parse number
   Parse.Count = False ' reset monster found counter
   For Monster.Index = 1 To LOF(MonsterFile) / Len(MonsterRecord) 'loop through file
      Call Read.Record(MonsterFile, Monster.Index) ' get monster record
      ' compare monster record name to input monster name,
      ' truncated to input length
      If Left$(MonsterRecord.MonsterName, Len(Inpt)) = Inpt Then ' compare
         Parse.Count = Parse.Count + 1 ' increment monster found counter
         ' compare parsed number of monster to search for
         If Parse.Value = False Or Parse.Count = Parse.Value Then ' compare
            Exit For ' exit routine
         End If ' end compare counters
      End If ' end compare monster names
   Next ' end loop through monster file
   If Parse.Count = False Then ' check monster found
      Monster.Index = Int(Val(Inpt)) ' convert input to integer
      ' file bounds
      If Monster.Index > False And Monster.Index <= LOF(MonsterFile) / Len(MonsterRecord) Then
         Choice = Monster.Index ' store monster number
      End If ' end check file bounds
   Else ' check file bounds
      Choice = Monster.Index ' store monster number
   End If ' end check file bounds
   If Choice > False Then ' check monster number found
      Call Get.Monster ' get the monster
   End If ' end check monster number found
End Sub ' end routine to get a monster

Rem * routine to get a random monster.
Rem * processing variables:
Rem *   Choice - is the monster number.

Sub Call.Monster
   If Room = 1! Then ' check player in safe room
      Outpt = "This is a safe haven!" ' display message
      Call IO.O ' send message
      Exit Sub ' exit routine for safe room
   End If ' end check player in safe room
   Monster.Class = RoomRecord.MonsterClass ' store room monster class
   ' check monster class bounds
   If Monster.Class > False And Monster.Class <= Monclass.Max Then
      ' calculate a random monster from the room monsterclass
      Choice = Monster.Class(Monster.Class, Int(Rnd * 10 + 1)) ' random
      Call Get.Monster ' get the monster
   End If ' end check monsterclass bounds
End Sub ' end routine to get a random monster

Rem * routine to get a random monster from the room monster class, computed
Rem * from the monster encounter round rate counter, and the monster
Rem * percentage.
Rem * input variables:
Rem *   Room - number of the current room.
Rem *   Monster.Rate1 - counter of rounds from file.
Rem *   Monster.Rate2 - counter of rounds for monster rate.

Sub Encounter.Monster
   If Room = 1! Then ' check safe room
      Exit Sub ' exit routine
   End If ' end check safe room
   Monster.Class = RoomRecord.MonsterClass ' store room monster class
   ' check monster class bounds
   If Monster.Class > False And Monster.Class <= Monclass.Max Then
      If Monster.Rate1 = MaxInt Then ' check rate counter maximum integer
         Monster.Rate1 = 1 ' reset rate counter
      Else ' check rate
         Monster.Rate1 = Monster.Rate1 + 1 ' increment rate
      End If ' end check room monster counter rate
      For Monsters = 1 To 10 ' loop through monsters with specified percentages
         If Monster.Rate(Monster.Class, Monsters) Then ' check monster rate
            ' verify monster rate equals room rate
            If Monster.Rate1 / Monster.Rate(Monster.Class, Monsters) = Int(Monster.Rate1 / Monster.Rate(Monster.Class, Monsters)) Then
               ' verify random monster rate percentage
               If Rnd < (Monster.Percent(Monster.Class, Monsters) / 100) Then ' verify
                  ' store monster number
                  Choice = Monster.Class(Monster.Class, Monsters)
                  Call Get.Monster ' get the monster
               End If ' end verify random rate
            End If ' end verify monster rate percent
         End If ' end check monster rate
      Next ' end loop through monster percentages
      Monster.Rate2 = Monster.Rate2 + 1 ' increment monster rate for room
      If Monster.Rate2 >= Room.Monster.Rate Then ' check monster rate
         Monster.Rate2 = False ' reset monster rate
         Choice = Monster.Class(Monster.Class, Int(Rnd * 10 + 1)) ' get random monster
         If MonsterRecord.Rate = False Then ' verify monster has specific rate
            If Rnd < .5 Then ' random chance for encounter
               Call Get.Monster ' get the monster
            End If ' end random chance
         End If ' end verify monster specific rate
      End If ' end monster rate
   End If ' end check room has monster class
End Sub ' end routine to random monster from rates and percentages

Rem * routine to get monsters.
Rem * input variables:
Rem *   Choice - the number of the monster record.
Rem * processing variables:
Rem *   Number.Appearing - number of monsters to encounter.

Sub Get.Monster
   If Choice > False And Choice <= LOF(MonsterFile) / Len(MonsterRecord) Then ' file bounds
      Call Read.Record(MonsterFile, Choice) ' get the monster record
      If MonsterRecord.Permanent = False Then ' verify monster not permanent
         ' calculate random number of monsters to encounter
         Number.Appearing = Int(Rnd * MonsterRecord.NumberAppearing) + 1 ' calculate
         While Number.Appearing + Number.Monsters > 20 ' loop to compare maximum
            Number.Appearing = Number.Appearing - 1 ' monsters in room, decrementing
         Wend ' number appearing while room maximum is not exceeded
         Select Case Number.Appearing ' selection of number encountered
            Case 0 ' number appearing is greater than room can hold
               Exit Sub ' exit routine
            Case 1 ' format message for one monster
               ' compare monster name starting with a vowel
               If InStr("aeiou", Left$(MonsterRecord.MonsterName, 1)) Then
                  Outpt = "an " ' format prefix to monster name
               Else ' check vowel
                  Outpt = "a " ' format prefix to monster name
               End If ' end compare monster name
               Outpt = " " + Outpt + RTrim$(MonsterRecord.MonsterName)
            Case Else ' format message for many monsters
               Outpt = Str$(Number.Appearing) + " " + RTrim$(MonsterRecord.PluralName)
         End Select ' end selection of monsters appearing
         Outpt = "You encounter" + Outpt + "!" ' make message
         Call IO.O ' send encounter message
         Call Get.Monster.Stats ' add the monster to the arrays
      End If ' end verify not permanent
   End If ' end check file bounds
End Sub ' end routine to get monsters

Rem * routine to store monster statistics for encountered monster number.
Rem * input variables:
Rem *   Choice - is the monster number to the monster file.
Rem *   Number.Appearing - the number of monsters of type choice to encounter.

Sub Get.Monster.Stats
   If Number.Monsters >= 20 Then ' check room is full of monsters
      Exit Sub ' exit routine
   End If ' end check room full
   For Monsters = 1 To Number.Appearing ' loop through all number appearing
      If Number.Monsters < 20 Then ' check room full of monsters
         Number.Monsters = Number.Monsters + 1 ' increment the number of monsters
         MonsterIndex(Number.Monsters) = Choice ' store the monster record choice
         MonsterArray(Number.Monsters) = MonsterRecord ' store the monster record
         If MonsterArray(Number.Monsters).Psionic Then ' verify psionic monster
            ' get monster psionic spell number
            Spell.Number = MonsterArray(Number.Monsters).PsionicSpell
            MonsterArray(Number.Monsters).Psionic = False ' spell from array and
            If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then ' reset with
               ' psionic spell number from spell file
               Call Read.Record(SpellFile, Spell.Number)
               If SpellRecord.Psionic Then ' check spell record is psionic
                  If SpellRecord.PsionicMode = 1 Then ' check psionic is offense
                     ' store the psionic spell level in the monster array
                     MonsterArray(Number.Monsters).Psionic = SpellRecord.Level
                  End If ' end check spell psionic type
               End If ' end check spell is psionic
            End If ' end check spell file bounds
         End If ' end check monster is psionic
         If MonsterArray(Number.Monsters).Permanent = False Then ' check permanent
            For Treasure.Index = 1 To 5 ' loop through all monster inventory
               If Rnd > .75 Then ' random chance
                  ' clear monster treasure element
                  Call PutMonsterRecordTreasure(Treasure.Index, False)
               End If ' end random chance
            Next ' end loop through monster inventory
         End If ' end check nonpermanent monster
      End If ' end check maximum room monsters
   Next ' end loop through number appearing of monster type choice
   Action.Number = RoomRecord.Action ' store room action number
   ' check file bounds
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then
      Call Read.Record(ActionFile, Action.Number) ' read action record
      ' check room action to monster choice
      If ActionRecord.MonsterTrigger = Choice Then
         ' check room action not talk activated
         If ActionRecord.MonsterTalk = False Then
            Action1$ = "As you step forward," ' make action message one
            Action2$ = "The monster hits you for" ' make action message two
            ' routine to activate actions by specific trigger
            Call Actions(Action1$, Action2$)
         End If ' end check room action
      End If ' end check room has monster trigger
   End If ' end check file bounds
End Sub ' end routine to store encountered monster statistics

Rem * routine for dead player.

Sub Player.Died
   UserRecord.Poison = False ' reset player poisoned
   Vehicle2 = False ' reset vehicle
   Vehicle3 = False ' reset vehicle
   Weapon1 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon2 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon3 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon4 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon5 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon6 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon7 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon8 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon9 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon10 = False ' reset weapons, shield, armor, and rings being held/worn
   Weapon11 = False ' reset weapons, shield, armor, and rings being held/worn
   UserRecord.MonstersKilled = False ' reset user score
   If Normal.User = False Then ' check player is Asst. DM or DM or Sysop
      Outpt = Message1 ' copy death message
      Call IO.O ' send death message
      Outpt = "You were resurrected!" ' make message
      Call IO.O ' send message
      Call Read.Room.Record(1!) ' get room record #1
      Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
      Next.Room = CSng(Resurrection.Room) ' reset room number
      If Next.Room = 0! Then ' check room number
         Next.Room = 1! ' reset room
      End If ' end check room number
      Number.Monsters = False ' reset number of monsters in room
      Call Enter.Room ' send player to room one
      Exit Sub ' exit routine
   End If ' end compare normal user type
   ' rest of death routine for normal users
   Stat.Amount = GetUserRecordStats(5) ' store player constitution
   If Stat.Amount > 1 Then ' check constitution positive
      Call PutUserRecordStats(5, Stat.Amount - 1) ' store decremented player constitution
   End If ' end check constitution
   Stat.Amount = GetUserRecordStats(6) ' store player piety
   If Stat.Amount > 1 Then ' check piety positive
      Call PutUserRecordStats(6, Stat.Amount - 1) ' store player decremented piety
   End If ' end check piety
   If Rnd > .5 Then ' random chance for statistic decrement
      Stat.Number = Int(Rnd * 4 + 1) ' choose random statistic to decrement
      Stat.Amount = GetUserRecordStats(Stat.Number) ' store random statistic
      If Stat.Amount > 1 Then ' compare to positive value
         ' store player decremented statistic
         Call PutUserRecordStats(Stat.Number, Stat.Amount - 1)
      End If ' end compare value
   End If ' end random chance
   Number.Monsters = False ' reset room monsters
   UserRecord.Poison = False ' reset player poisoned flag
   Weight = False ' reset player inventory weight
   For Array.Index = 1 To 20 ' loop through player inventory
      Charges.Number = GetUserRecordCharges(Array.Index) ' store inventory charges
      Index.Number = GetUserRecordInv(Array.Index) ' store inventory number
      If Index.Number Then ' verify inventory
         ' falls to ground
         Call Add.Room.Treasure(Index.Number, Charges.Number, False, Item.Added)
         If Item.Added = False Then ' return variable for full room
            Exit For ' exit inventory loop
         End If ' end check return variable
      End If ' end check inventory
   Next ' end loop through player inventory
   For Array.Index = 1 To 20 ' loop through player inventory
      Call PutUserRecordInv(Array.Index, False) ' set inventory to zero
      Call PutUserRecordCharges(Array.Index, False) ' set inventory to zero
   Next ' end loop through inventory
   Call Read.Room.Record(1!) ' get room record #1
   Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
   Next.Room = CSng(Resurrection.Room) ' reset room number
   If Next.Room = 0! Then ' check room number
      Next.Room = 1! ' reset room
   End If ' end check room number
   Room = Next.Room ' reset user room
   Outpt = Message1 ' copy death message
   Call IO.O ' send death message
   ' calculate player constitution roll
   Level = UserRecord.Level ' store user level
   Calculate# = (Int(Rnd * 10 + 10) + GetUserRecordStats(5)) > 20 ' calculate
   If Calculate# = False Then ' compare constitution roll
      Outpt = "Constitution roll failed!" ' make message
      Call IO.O ' send fail message
      Outpt = "You were not resurrected!" ' make message
      Call IO.O ' send fail message
      Level = Int(Level / 2 - .5) ' player loses half levels
   Else ' end compare constitution roll
      ' player survives constitution roll
      Outpt = "You were resurrected!" ' make message
      Call IO.O ' send message
      If GetUserRecordStats(6) <= 1 Then ' compare piety
         Outpt = "Except your piety failed!" ' player loses roll anyway
         Call IO.O ' send fail message
         Level = Int(Level / 2 - .5) ' player loses half levels
      Else ' compare piety
         Level = Level - 1 ' decrement player level by one
      End If ' end compare piety
   End If ' end compare constitution roll
   If Level < False Then ' compare level
      Level = False ' reset level
   End If ' end compare level
   UserRecord.Level = Level ' store user level
   Call More.Prompt ' pause for next screen
   Call Enter.Room ' routine to move player to room number
   Call Zero.Stats ' routine to clear player statistics
End Sub ' end routine for dead player

Rem * routine to clear player statistics for death.

Sub Zero.Stats
   Call Experience(Calculate#) ' calculate new experience
   Calculate# = Calculate# - Int(Calculate# / 10) ' calculate decremented experience
   If Calculate# < UserRecord.Experience Then ' compare new experience
      UserRecord.Experience = Calculate# ' reset experience
   End If ' end compare new experience
   Call Gold(Calculate#) ' calculate new gold
   Calculate# = Calculate# - Int(Calculate# / 10) ' calculate decremented gold
   If Calculate# < UserRecord.Gold Then ' compare new gold
      UserRecord.Gold = Calculate# ' reset player gold
   End If ' end compare new gold
   Call New.Stats ' get new statistics for level
   UserRecord.Flags = UserRecord.Flags And Not Wished ' clear player flags
   UserRecord.Flags = UserRecord.Flags And Not Rerolled ' clear player flags
   UserRecord.Flags = UserRecord.Flags And Not Alignmented ' clear player flags
   For Stats = 1 To 7 ' loop through player statistics
      If GetUserRecordStats(Stats) <= False Then ' compare statistcis to zero
         Call PutUserRecordStats(Stats, 1) ' reset statistic
      End If ' end compare player statistics
   Next ' end loop through player statistics
   Call Put.User.Record ' store player record
End Sub ' end routine to clear dead player

Rem * routine for player to learn a spell chant.
Rem * processing variables:
Rem *   Learned.Spells - array of 0s and 1s of learned spells.
Rem * input variables:
Rem *   Parsed.Command1 - contains name of spell to learn.

Sub Learn.Spell
   Spell.Found = False ' flag for spell name found
   For Spell.Number = 1 To LOF(SpellFile) / Len(SpellRecord) ' loop through spells
      Call Read.Record(SpellFile, Spell.Number) ' get next spell record
      ' compare spell name with command parameter
      If RTrim$(SpellRecord.SpellName) = LCase$(Parsed.Command1) Then ' compare
         Spell.Found = True ' set spell flag found
         Exit For ' exit spell file loop
      End If ' end compare spell names
   Next ' end loop through spell file
   If Spell.Found = False Then ' compare spell flag
      Outpt = "You can't learn that spell!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare spell flag
   If Spell.Number > 1024 Then ' compare spell maximum
      Outpt = "You can't learn that spell!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare spell flag
   Outpt = "Enter chant to learn: " ' format input prompt
   Call IO.I ' get input spell chant
   Inpt = UCase$(Inpt) ' uppercase spell chant input
   If RTrim$(SpellRecord.Chant) <> Inpt Then ' compare spell chant entered
      Outpt = "Wrong spell chant!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare spell chants
   ' check length of learned spell flags
   If Spell.Number > Len(Learned.Spells) Then
      ' append 0s to learned spells
      Learned.Spells = Learned.Spells + String$(Spell.Number - Len(Learned.Spells), "0")
   End If ' end check length of learned spells flags
   Mid$(Learned.Spells, Spell.Number, 1) = "1" ' set flag for spell learned
   Outpt = "You now memorize the spell!" ' make message
   Call IO.O ' send spell message
End Sub ' end routine to learn spell

Rem * routine for preprocessing spell casting.
Rem * input variables:
Rem *   Require.Chant - set to true if entering the spell chant is required.
Rem * output variables:
Rem *   Spell.Cast.Type - set to 1=use command, 2=scroll used, 4=cast command.
Rem *   Spell.Number - number of the spell being used.
Rem *   Spell.Chant - true if spell being used w/o chant.

Sub Cast.Spell (Require.Chant)
   If Weapon6 Then ' check player is holding a weapon
      Outpt = "You must return your weapon first!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check weapon held
   Spell.Cast.Type = False ' reset casting variables
   Spell.Number = False ' reset casting variables
   Spell.Chant = False ' reset casting variables
   If Require.Chant Then ' check if chant required
      If UserRecord.Magic = False Then ' verify spell points remaining
         Outpt = "You have no more spell points!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end verify spell points
      Outpt = "Chant? " ' format input prompt
      Call IO.I ' get player input
      Inpt = UCase$(Inpt) ' uppercase input
      Chant.Found = False ' set spell chant found flag
      ' loop through all spell records
      For Spell.Number = 1 To LOF(SpellFile) / Len(SpellRecord)
         Call Read.Record(SpellFile, Spell.Number) ' get spell record
         Spell.Chant$ = SpellRecord.Chant ' store spell chant
         Spell.Chant$ = RTrim$(Spell.Chant$) ' trim chant
         Spell.Chant$ = UCase$(Spell.Chant$) ' uppercase chant
         If Spell.Chant$ = Inpt Then ' compare chants
            If SpellRecord.Psionic = False Then ' check for psionic spell
               Chant.Found = True ' set spell chant found flag
               Exit For ' exit spell file loop
            End If ' end check psionic spell
         End If ' end compare chants
         Spell.Chant$ = SpellRecord.SpellName ' store spell name
         Spell.Chant$ = RTrim$(Spell.Chant$) ' trim name
         Spell.Chant$ = UCase$(Spell.Chant$) ' uppercase name
         If Spell.Chant$ = Inpt Then ' compare spell names
            If SpellRecord.Psionic = False Then ' check for psionic spell
               ' check learned spells bounds
               If Spell.Number > 1024 Then ' compare spell maximum
                  Exit For ' exit spell file loop
               End If ' end compare spell maximum
               If Spell.Number <= Len(Learned.Spells) Then
                  ' check learned flag
                  If Mid$(Learned.Spells, Spell.Number, 1) = "1" Then
                     Chant.Found = True ' set spell chant found flag
                     Exit For ' exit spell file loop
                  End If ' end check learned flag
               End If ' end check bounds
            End If ' end check for psionic spell
         End If ' end compare spell names
      Next ' end loop through spell file
      If Chant.Found = False Then ' check spell chant found flag
         Outpt = "Wrong spell chant!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check spell chant flag
      Spell.Cast.Type = Cast.Spell.Type ' store spell type being used
   Else ' compare chant required
      Call Find.Inventory ' check magic device being used
      If Index.Number = False Then ' verify device used
         Outpt = "You can't use that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end verify device
      If TreasureRecord.Potion Then ' check device type
         Outpt = "You can't use that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check device type
      If TreasureRecord.Edible Then ' check device type
         Outpt = "You can't use that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check device
      If TreasureRecord.Scroll Then ' check device type
         Outpt = "The scroll disintegrated!" ' make message
         Call IO.O ' send message
         Call Discard.Inventory(Array.Number, True) ' remove item from inventory
         Spell.Cast.Type = Scroll.Spell.Type ' store spell type being used
      Else ' check device type
         If TreasureRecord.Spell = False Then ' check device type spell type
            Outpt = "That's not a magical device!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check device/spell type
         If GetUserRecordCharges(Array.Number) = False Then ' compare spell charges
            Outpt = "You can't, it's empty!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check spell charges
         Charges.Number = GetUserRecordCharges(Array.Number) ' store spell charges
         Charges.Number = Charges.Number - 1 ' decrement charges
         If Charges.Number < False Then ' compare zero
            Charges.Number = False ' set to zero
         End If ' end compare zero
         Call PutUserRecordCharges(Array.Number, Charges.Number) ' store spell charges
         Spell.Cast.Type = Use.Spell.Type ' store spell type being used
      End If ' end compare device type
      Spell.Number = TreasureRecord.Spell ' store spell type
      Spell.Chant = True ' no chant required flag
   End If ' end compare chant required
   Call Read.Record(SpellFile, Spell.Number) ' get the spell record being used
   Magic.Spell = SpellRecord.SpellType ' store the spell type being used
   Multiplier = SpellRecord.Level ' store the spell level
   Index.Number = False ' reset target variable
   If Parser Then ' check for second command parameter
      Parsed.Command1 = Parsed.Command2 ' store second parameter
      Call Numeric ' get pound sign number
      Call Examine.Treasure ' check for treasure target
      If Index.Number = False Then ' treasure target found
         Call Num ' decrement target number
         Call Examine.Objects ' check for object target
         If Index.Number > False Then ' check object target found
            Outpt = "Wrong spell target for chant" ' make message
            Call IO.O ' send message
            Exit Sub ' exit sub
         End If ' end check object target
      End If ' end check target
   End If ' end check second parameter
   Call Magic(Spell.Chant) ' routine to use the magic spell
End Sub ' end spell preprocessing routine

Rem * routine to cast a magic spell.
Rem * input variables:
Rem *   Magic.Spell - number of the spell Type
Rem *   Chant.Used - true if spell being used w/o chant
Rem *   Spell.Cast.Type - set to 1=use command, 2=scroll used, 4=cast command.
Rem *   Spell.Number - number of the spell being used.
Rem *   Index.Number - type of spell target.

Sub Magic (Chant.Used)
   Select Case Index.Number ' selection of the target type
      Case Is > False ' target is an item of treasure
         Select Case Magic.Spell ' selection of spell type for target of treasure
            Case Enchant ' enchant
               If TreasureRecord.Spell = False Then ' check treasure target spell type
                  Outpt = "Wrong spell target for chant!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end check treasure target spell type
            Case Vigor ' vigor
               If TreasureRecord.Vehicle = False Then ' check treasure target vehicle
                  Outpt = "Wrong spell target for chant!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end check treasure target vehicle type
            Case Else ' check target type
               If Magic.Spell <> Hiding Then ' check spell type on treasure target
                  Outpt = "Wrong spell target for chant!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end check treasure target type
         End Select ' end selection of target spell type
      Case False ' target is not an item of treasure
         If Parser Then ' check target parameter flag
            Select Case Magic.Spell ' selection of spells w/ targets
               Case Offense, Poison, LevelDrain, Befuddled, TurnUndead, Intoxicate
                  Index.Number = False
               Case Else ' spell w/o target
                  Outpt = "Wrong spell target for chant!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
            End Select ' end spell type selection
         End If ' end check target parameter
   End Select ' end target selection type
   If Chant.Used = False Then ' check spell usage type
      If Multiplier > UserRecord.Level Then ' compare spell level to player level
         Outpt = "You are not high enough level to cast the spell!" ' message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check spell/player levels
      If UserRecord.Magic = False Then ' check player magic points remaining
         Outpt = "You have no more spell points!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check player magic points
      ' verify spell can be cast by player class type
      If (SpellRecord.ClassType And 2 ^ UserRecord.ClassType) = False Then ' verify
         If Normal.User = True Then ' check player is Asst. DM/DM/Sysop
            Outpt = "Your class can't cast that spell!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check player type
      End If ' end verify spell type
      Charges.Number = UserRecord.Magic ' get player magic points
      ' decrement magic points by spell level
      Charges.Number = Charges.Number - Multiplier
      If Charges.Number < False Then ' check zero
         Charges.Number = False ' set to zero
      End If ' end compare zero
      UserRecord.Magic = Charges.Number ' store player magic points remaining
   End If ' end check spell usage type
   ' check spell ingredients.
   ' verify spell type uses ingredients
   If SpellRecord.SpellFlag And Spell.Cast.Type Then
      For Spell.Index = 1 To 5 ' loop through spell ingredients
         ' get treasure ingredient number
         Ingredient.Number = GetSpellRecordIngred(Spell.Index)
         If Ingredient.Number > False And Ingredient.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' bounds
            Player.Ingredient = False ' player has ingredient flag
            For Array.Index = 1 To 20 ' loop through player inventory
               ' check player inventory
               If GetUserRecordInv(Array.Index) = Ingredient.Number Then
                  Player.Ingredient = True ' set ingredient flag
                  Exit For ' exit player inventory loop
               End If ' end check player inventory is ingredient
            Next ' end loop through player inventory
            ' get treasure ingredient record
            Call Read.Record(TreasureFile, Ingredient.Number)
            If Player.Ingredient = False Then ' check player has ingredient flag
               Outpt = "You don't have the correct spell ingredients!" ' message
               Call IO.O ' send message
               Inpt = TreasureRecord.TreasureName ' get treasure name
               Inpt = RTrim$(Inpt) ' trim name
               Inpt = LCase$(Inpt) ' lowercase name
               Outpt = "You need " + Inpt + "!" ' make message
               Call IO.O ' send ingredient message
               Exit Sub ' exit routine
            End If ' end check player ingredient flag
            ' check ingredient charges
            If GetUserRecordCharges(Array.Index) = False Then
               Inpt = TreasureRecord.ShortName ' get treasure name
               Inpt = RTrim$(Inpt) ' trim name
               Inpt = LCase$(Inpt) ' lowercase name
               Outpt = "The " + Inpt + " are used up!" ' make message
               Call IO.O ' send ingredient message
               Exit Sub ' exit routine
            End If ' end check inrgedient charges
            ' decrement player ingredient charges
            Call PutUserRecordCharges(Array.Index, GetUserRecordCharges(Array.Index) - 1)
         End If ' end check file bounds
      Next ' end loop through spell ingredients
   End If ' end verify spell requires ingredients
   Spell.Name$ = SpellRecord.SpellName ' get spell name being cast
   Spell.Name$ = RTrim$(Spell.Name$) ' trim name
   Spell.Name$ = LCase$(Spell.Name$) ' lowercase name
   If InStr("aeiou", Left$(Spell.Name$, 1)) Then ' check spell vowel
      Spell.Name$ = " an " + Spell.Name$ ' append vowel
   Else ' check vowel
      Spell.Name$ = " a " + Spell.Name$ ' append vowel
   End If ' end check vowel spelling
   Outpt = SpellRecord.Desc ' get spell long cast description
   Outpt = RTrim$(Outpt) ' trim description
   If Outpt = Nul Then ' check description length
      Outpt = "You cast" + Spell.Name$ + " spell!" ' make default cast description
   End If ' end check description length
   Call IO.O ' send spell cast message
   Call Read.Room.Record(Room) ' get current room record
   Action.Number = RoomRecord.Action ' get room action number
   ' check file bounds
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then
      Call Read.Record(ActionFile, Action.Number) ' read action record
      If ActionRecord.SpellTrigger = Spell.Number Then ' check for spell trigger
         Action1$ = "As the spell is cast," ' action message
         Action2$ = "You are hit for" ' action message
         Call Actions(Action1$, Action2$) ' routine for activated action
      End If ' end check room for action triggered
   End If ' end check file bounds
   ' check for magic traps in room
   For Array.Index = 1 To 20 ' loop through room treasure
      If GetRoomRecordFlags(Array.Index) = Magic.Trap Then ' check room for trap
         If GetRoomRecordTreasure(Array.Index) = True Then ' check trap triggered
            If GetUserRecordStats(2) < Int(Rnd * 6 + 10) Then ' random chance for trap
               ' hits on player for magical trap
               Calculate# = CDbl(Int(Rnd * 10 + UserRecord.Level) * GetRoomRecordTreCharges(Array.Index))
               Call PutRoomRecordTreasure(Array.Index, False) ' reset room trap
               Call PutRoomRecordTreCharges(Array.Index, False) ' reset room trap
               Call PutRoomRecordFlags(Array.Index, False) ' reset room trap
               Call Share.Room.Record(Room) ' write new room record
               Outpt = "You triggered a magical trap!" ' message
               Call IO.O ' send trap message
               Outpt = "The trap hit you for" ' hits message
               Call Hit.Player(Calculate#) ' routine to hit player
               Exit For ' exit room trap loop
            End If ' end random chance
         End If ' end trap trigger
      End If ' end room trap
   Next ' end loop through room
   ' selections of the actual spell cast subroutines
   Select Case Magic.Spell ' select the spell type number
      Case Offense, Poison, LevelDrain, Befuddled, TurnUndead, Intoxicate
         Call Attack.Monster ' spell cast on monster target
      Case Enchant ' enchant (heal magic points)
         If Index.Number > False Then ' enchant on a target
            Call Read.Record(SpellFile, TreasureRecord.Spell) ' get target spell
            If SpellRecord.SpellType = 4 Then ' compare target to wish device
               Outpt = "You can't recharge that!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end compare wish item
            Calculate# = GetUserRecordCharges(Array.Number) ' get charges of item
            Calculate# = Calculate# + UserRecord.Level ' calculate new item charges
            If Calculate# > MaxInt Then ' compare maximum integer
               Calculate# = MaxInt ' reduce to maximum integer
            End If ' end compare maximum integer
            ' compare maximum treasure charges
            If Calculate# > TreasureRecord.Charges Then
               ' reduce to maximum treasure charges
               Calculate# = TreasureRecord.Charges
            End If ' end compare charges
            Call PutUserRecordCharges(Array.Number, CInt(Calculate#)) ' store new charges
            Outpt = "You recharge it!" ' make message
            Call IO.O ' send message
         Else ' compare enchant on a target
            ' calculate player enchant
            Calculate# = UserRecord.Magic + UserRecord.Level + Multiplier ' calculate
            If Calculate# > MaxInt Then ' compare maximum integer
               Calculate# = MaxInt ' set to maximum integer
            End If ' end compare maximum integer
            UserRecord.Magic = CInt(Calculate#) ' store player magic points
            Call New.Stats ' update player statistics
            Outpt = "You now have" + Str$(UserRecord.Magic) + " magic points!" ' message
            Call IO.O ' send message
         End If ' end compare enchant target
      Case Bless ' bless (heal piety)
         Outpt = "Nothing happened.." ' default bless message
         If GetUserRecordStats(6) < MaxStat Then ' compare player piety points
            Call PutUserRecordStats(6, GetUserRecordStats(6) + 1) ' increment player piety
            Outpt = "You feel a magical glow about you!" ' message
         End If ' end compare piety
         Call IO.O ' send piety message
      Case Wish ' wish
         ' compare spell cast type
         If Spell.Cast.Type = Use.Spell.Type Or Spell.Cast.Type = Scroll.Spell.Type Then ' compare
            ' reset player wish flag
            UserRecord.Flags = UserRecord.Flags And Not Wished ' reset
         End If ' end compare spell cast type
         Graphics.Off = True ' reset color
         Outpt = "The Ghods thunder.." ' message
         Call IO.O ' send ghod message
         Outpt = "   What do you wish for?" ' format input prompt
         Graphics.Off = False ' reset color
         Call IO.I ' get player input
         Stored.Parsed.Command2 = Inpt ' store item wish name
         Parsed.Command1 = Stored.Parsed.Command2 ' store item wish name
         Call Numeric ' parse out pound sign
         Inpt = Parsed.Command1 ' store parsed item wish name
         Inpt = LCase$(Inpt) ' lowercase item
         Call Drop(False) ' routine to get item from treasure file
      Case Vigor ' vigor (heal fatigue)
         If Index.Number > False Then ' check treasure target
            Call Read.Room.Record(Room) ' get current room
            Calculate# = GetRoomRecordTreCharges(Array.Number) ' get vehicle hits
            Calculate# = Calculate# + UserRecord.Level * 2 ' increment vehicle hits
            If Calculate# > MaxInt Then ' compare maximum integer
               Calculate# = MaxInt ' reset to maximum integer
            End If ' end compare maximum integer
            Vehicle.Hits = CInt(Calculate#) ' convert to integer
            ' compare maximum vehicle hits
            If Vehicle.Hits > TreasureRecord.VehicleHits Then
               ' reset to maximum vehicle hits
               Vehicle.Hits = TreasureRecord.VehicleHits
            End If ' end compare vehicle hits
            ' store new vehicle hits
            Call PutRoomRecordTreCharges(Array.Number, Vehicle.Hits)
            Call Share.Room.Record(Room) ' write room record
            Outpts = TreasureRecord.ShortName ' get vehicle name
            Outpts = RTrim$(Outpts) ' trim name
            Outpts = LCase$(Outpts) ' lowercase name
            Outpt = "The " + Outpts + " now has" + Str$(Vehicle.Hits) + " hits!" ' message
            Call IO.O ' send healed vehicle message
         Else ' compare target
            ' calculate player fatigue increase
            Calculate# = UserRecord.Fatigue + UserRecord.Level + Multiplier ' calculate
            If Calculate# > MaxInt Then ' compare to max integer
               Calculate# = MaxInt ' set to max integer
            End If ' end compare max integer
            UserRecord.Fatigue = CInt(Calculate#) ' set player fatigue to new value
            Call New.Stats ' routine to update player statistics
            Outpt = "You now have" + Str$(UserRecord.Fatigue) + " fatigue points!"
            Call IO.O ' send vigor message
         End If ' end compare target
      Case Heal ' (heal vitality)
         ' calculate player vitality increase
         Calculate# = UserRecord.Vitality + UserRecord.Level + Multiplier ' calculate
         If Calculate# > MaxInt Then ' compare to max integer
            Calculate# = MaxInt ' set to max integer
         End If ' end compare max integer
         UserRecord.Vitality = CInt(Calculate#) ' store new vitality
         Call New.Stats ' routine to update player statistics
         Outpt = "You now have" + Str$(UserRecord.Vitality) + " vitality points!"
         Call IO.O ' send new vitality message
      Case CurePoison ' cure poison
         Outpt = "Nothing happened.." ' default message
         If UserRecord.Poison Then ' check player is poisoned
            UserRecord.Poison = False ' clear player poisoned flag
            Outpt = "The poison disappeared!" ' message
         End If ' end check player poisoned
         Call IO.O ' send poison message
      Case Teleport ' teleport spell
         Next.Room = SpellRecord.Teleport ' copy teleportation room ro next room
         Teleported = True ' set teleporting flag
         Call Enter.Room ' routine to move player to new room
      Case PassDoor ' open nearby doors
         Pass.Door = True ' set passdoor flag
         Outpt = "You hear a click!" ' passdoor message
         Call IO.O ' send passdoor message
      Case Conjure ' call up random monster
         For Array.Index = 1 To Monclass.Max ' loop through all monster classes
            ' get a random monster choice
            Choice = Monster.Class(Array.Index, Int(Rnd * 10 + 1))
            If Choice Then ' verify choice exists
               Call Read.Record(MonsterFile, Choice) ' get monster record
               ' compare monster to player level
               If MonsterRecord.Level = UserRecord.Level Then ' compare
                  Call Get.Monster ' get the monster
                  Exit Sub ' exit routine
               End If ' end compare levels
            End If ' end verify choice
         Next ' end loop through monster classes
         Outpt = "Nothing conjures.." ' default message
         Call IO.O ' send message
      Case Psionic ' psionics
         Outpt = "You can't cast a psi spell!" ' error message
         Call IO.O ' send error message
      Case DetectLock ' detect hidden or invisible doors
         For Array.Index = 1 To 20 ' loop through room objects
            Object.Number = GetRoomRecordObject(Array.Index) ' get room object nummber
            If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then ' file bounds
               Call Read.Record(ObjectFile, Object.Number) ' get object record
               ' compare object invisible or hidden
               If ObjectRecord.Invisible Or ObjectRecord.Hidden Then ' compare
                  Outpt = "You detect hidden objects here!" ' display message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end compare object type
            End If ' end check file bounds
         Next ' end loop through room objects
         Outpt = "You detect nothing.." ' default message
         Call IO.O ' send message
      Case DetectEvil ' determine which direction the highest class monsters are
         High.Class = False ' highest monster class so far
         Monster.Direction = False ' direction of monster class
         For Direction.Number = 1 To 12 ' loop through all room directions
            Call Read.Room.Record(Room) ' get current room record
            ' get next room direction
            Next.Direction! = GetRoomRecordDirect(Direction.Number)
            If Next.Direction! Then ' compare room direction
               Call Read.Room.Record(Next.Direction!) ' get next room record
               ' compare monster class higher
               If RoomRecord.MonsterClass > High.Class Then ' compare
                  High.Class = RoomRecord.MonsterClass ' store higher monster class
                  Monster.Direction = Direction.Number ' store direction
               End If ' end compare monster class
            End If ' end compare room direction
         Next ' end loop through room directions
         Call Read.Room.Record(Room) ' get current room number
         If Monster.Direction Then ' verify room direction found w/ monsters
            ' make display message with room direction
            Outpt = "You detect evil going " + RTrim$(Direction(Monster.Direction)) + "!"
         Else ' verify direction
            Outpt = "You detect no evil presence.." ' make display message
         End If ' end verify room direction
         Call IO.O ' send display message
      Case DetectTrap ' find traps in room objects
         For Array.Index = 1 To 20 ' loop through room objects
            Object.Number = GetRoomRecordObject(Array.Index) ' get object number
            If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then ' file bounds
               Call Read.Record(ObjectFile, Object.Number) ' get object record
               If ObjectRecord.Trap Then ' verify object is a trap
                  Outpt = "You detect traps here!" ' make display message
                  Call IO.O ' send display message
                  Exit Sub ' exit routine
               End If ' end verify object trap
            End If ' end check file bounds
         Next ' end loop through room objects
         Outpt = "You detect nothing.." ' make default display message
         Call IO.O ' send default message
      Case SetTrap ' sets a magical trap in room
         Trap.Set = False ' room full flag
         For Treasure.Number = 1 To 20 ' loop through room treasure
            ' check empty treasure in room
            If GetRoomRecordTreasure(Treasure.Number) = False Then
               Trap.Set = True ' set room full flag
               Exit For ' exit loop
            End If ' end check empty treasure
         Next ' end loop room
         If Trap.Set = False Then ' check room full flag
            Outpt = "You didn't set the trap!" ' make display message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check room flag
         If GetUserRecordStats(2) < Int(Rnd * 5 + 5) Then ' calculate random percent
            Outpt = "You didn't set the trap!" ' make display message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end calculate percent
         ' calculate trap percent
         If GetUserRecordStats(2) + GetUserRecordStats(3) < Int(Rnd * 10 + 5) Then ' calculate
            Outpt = "The trap explodes in your face!" ' make display message
            Call IO.O ' send message
            Calculate# = CDbl(Int(Rnd * 10 + 2) * UserRecord.Level) ' get hits on player
            Outpt = "The trap hit you for" ' death message
            Call Hit.Player(Calculate#) ' routine to hit player
            Exit Sub ' exit routine
         End If ' end trap percent
         Call PutRoomRecordTreasure(Treasure.Number, True) ' set the trap activated
         Call PutRoomRecordTreCharges(Treasure.Number, UserRecord.Level) ' set trap level
         Call PutRoomRecordFlags(Treasure.Number, Magic.Trap) ' set trap flag
         Call Share.Room.Record(Room) ' write room record
         Outpt = "You set a magical trap!" ' make display message
         Call IO.O ' send message
      Case Hiding ' spell to hide an item in room
         Item.Hidden = False ' room full flag
         For Array.Index = 1 To 20 ' loop through room treasure
            ' found empty room treasure
            If GetRoomRecordTreasure(Array.Index) = False Then
               Item.Hidden = True ' set room full flag
               Exit For ' exit loop
            End If ' end check empty treasure
         Next ' end loop through room
         If Item.Hidden = False Then ' check room full flag
            Outpt = "You didn't hide the object!" ' make display message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end check room full flag
         If GetUserRecordStats(2) < Int(Rnd * 5 + 4) Then ' calculate random percent
            Outpt = "You didn't hide the object!" ' make display message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random percent
         ' remove item from player inventory
         Call Discard.Inventory(Array.Number, True)
         Call PutRoomRecordTreasure(Array.Index, Index.Number) ' add item to room
         ' add item charges to room
         Call PutRoomRecordTreCharges(Array.Index, Charges.Number)
         Call PutRoomRecordFlags(Array.Index, Magically.Hidden) ' set magically hidden flag
         Call Share.Room.Record(Room) ' write room record
         Outpt = "You hide the object magically!" ' make display message
         Call IO.O ' send message
      Case Search ' displays magically hidden objects in room
         Graphics.Off = True ' reset color
         Outpt = "You find " ' make first string part
         Carriage.Return = True ' disable cr/lf
         Call IO.O ' send output
         Items.Displayed = False ' items displayed counter
         Item.Displayed = False ' item displayed flag
         Outpt = Nul ' reset output string
         For Array.Index = 1 To 20 ' loop through room treasure
            Inpt = Nul ' next item string
            ' check hidden flag
            If GetRoomRecordFlags(Array.Index) = Magically.Hidden Then
               ' store treasure number
               Treasure.Number = GetRoomRecordTreasure(Array.Index)
               If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' bounds
                  Call Read.Record(TreasureFile, Treasure.Number) ' get treasure
                  Inpt = TreasureRecord.TreasureName ' store treasure name
                  Inpt = RTrim$(Inpt) + "[inv]" ' append treasure name
               End If ' end check file bounds
            End If ' end check treasure magically hidden flag
            If Outpt <> Nul And Inpt <> Nul Then ' check next output item
               Carriage.Return = True ' disable cr/lf
               Call IO.O ' send item to output
               ' increment items displayed counter
               Items.Displayed = Items.Displayed + 1
            End If ' end check output item
            If Inpt <> Nul Then ' check last item output
               Item.Displayed = True ' set item displayed flag
               Outpt = Inpt + ", " ' append comma
            End If ' end check items
         Next ' end loop through room
         If Item.Displayed = False Then ' check item displayed flag
            Outpt = "nothing special.." ' default display message
         End If ' end check item flag
         If Item.Displayed And Outpt <> Nul Then ' check item flag
            Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, add period
            If Items.Displayed > 1 Then ' check number of items displayed
               Outpt = "and " + Outpt ' append to string
            End If ' end check item number
         End If ' end check items
         Call IO.O ' send output
         Graphics.Off = False ' reset color
      Case Invisibility ' normal player invisibility spell
         Outpt = "Didn't work!" ' default display message
         Level = Int((UserRecord.Level + 4) / 2) ' calculate level
         If Level > False Then ' check level for invisibility
            Invisible = Level ' set invisibility counter
            UserRecord.Invisible = True ' set player record invisible flag
            Outpt = "You are now invisible!" ' make display message
         End If ' end check invisible percent
         Call IO.O ' send display message
      Case Identify ' spell to display monster stattistics
         Parsed.Command1 = Last.Monster ' get parameter in monster name
         Call Check.Monster ' check monster name
         If Monster.Number = False Then ' compare monster number found
            Outpt = "You are not fighting a monster now!" ' make error message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end compare monster number
         Graphics.Off = True ' reset color
         Call The.Or.An ' get prefix
         Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
         Outpts = RTrim$(Outpts) ' trim name
         Outpts = LCase$(Outpts) ' lowercase name
         Level = MonsterArray(Monster.Number).Level ' store monster level
         ' format message with monster to player level
         Outpt = "You are fighting " + Prefix1 + Outpts + "(level" + Str$((Level - 1) * 2 + 1) + " to" + Str$(Level * 2) + ")"
         Call IO.O ' send level message
         Outpt = "It has" + Str$(MonsterArray(Monster.Number).Hits) + " hits."
         Call IO.O ' display message of monster hit points
         Outpt = "It has" + Str$(MonsterArray(Monster.Number).Experience) + " experience."
         Call IO.O ' display message of monster experience
         Outpt = "It has" + Str$(MonsterArray(Monster.Number).Gold) + " gold."
         Call IO.O ' display message of monster gold
         Spell.Number = MonsterArray(Monster.Number).Spell ' get monster spell type
         ' file bounds
         If Spell.Number > False And Spell.Number <= LOF(SpellFile) / Len(SpellRecord) Then
            Call Read.Record(SpellFile, Spell.Number) ' get spell record
            ' format message with spell monster can cast
            Outpt = "It can cast " + RTrim$(SpellRecord.SpellName) + " spells."
            Call IO.O ' send spell message
         End If ' end check file bounds
         If MonsterArray(Monster.Number).Poison Then ' check poisonous monster
            Outpt = "It can poison." ' make message
            Call IO.O ' send message
         End If ' end check poisonous monster
         If MonsterArray(Monster.Number).LevelDrain Then ' check undead monster
            Outpt = "It can drain levels." ' make message
            Call IO.O ' send message
         End If ' end check undead monster
         If MonsterArray(Monster.Number).Psionic Then ' check psionic monster
            Outpt = "It can cast psi spells." ' make message
            Call IO.O ' send message
         End If ' end check psionic monster
         If MonsterArray(Monster.Number).Magic Then ' compare magical monster
            Outpt = "It's magical." ' make message
            Call IO.O ' send message
         End If ' end check magical monster
         If MonsterArray(Monster.Number).Block Then ' check monster blocks
            Outpt = "It blocks exits." ' make message
            Call IO.O ' send message
         End If ' end check monster blocks
         If MonsterArray(Monster.Number).Prevent Then ' check monster takes
            Outpt = "It prevents treasure take." ' make message
            Call IO.O ' send message
         End If ' end check monster takes
         If MonsterArray(Monster.Number).Follow Then ' check monster follows
            Outpt = "It follows users." ' make message
            Call IO.O ' send message
         End If ' end check monster follows
         If MonsterArray(Monster.Number).Jail Then ' check monster jails
            Outpt = "It jails attacker." ' make message
            Call IO.O ' send message
         End If ' end check monster jails
         If MonsterArray(Monster.Number).Teleport Then ' check monster teleports
            Outpt = "It can teleport." ' make message
            Call IO.O ' send message
         End If ' end check monster teleports
         Graphics.Off = False ' reset color
      Case Enlighten ' locates hidden objects in room
         Graphics.Off = True ' reset color
         Outpt = "You see " ' make first output string
         Carriage.Return = True ' disable cr/lf
         Call IO.O ' output first string
         Items.Displayed = False ' items displayed counter
         Item.Displayed = False ' item displayed flag
         Outpt = Nul ' reset output string
         For Array.Index = 1 To 20 ' loop through room treasure
            Inpt = Nul ' reset last item string
            ' compare item hidden
            If GetRoomRecordFlags(Array.Index) = Hidden.Object Then
               ' store treasure number
               Treasure.Number = GetRoomRecordTreasure(Array.Index)
               If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' bounds
                  Call Read.Record(TreasureFile, Treasure.Number) ' get treasure
                  Inpt = TreasureRecord.TreasureName ' get treasure name
                  Inpt = RTrim$(Inpt) + "[inv]" ' append to name
               End If ' end check file bounds
            End If ' end compare hidden item
            If Outpt <> Nul And Inpt <> Nul Then ' check next item display
               Carriage.Return = True ' disable cr/lf
               Call IO.O ' send item display
               ' increment items displayed counter
               Items.Displayed = Items.Displayed + 1
            End If ' end check items
            If Inpt <> Nul Then ' check last item displayed
               Item.Displayed = True ' set item displayed flag
               Outpt = Inpt + ", " ' append comma
            End If ' end check item
         Next ' end loop through room
         If Item.Displayed = False Then ' check item displayed flag
            Outpt = "nothing special.." ' make message
         End If ' end check item display flag
         If Item.Displayed And Outpt <> Nul Then ' check items displayed
            Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, add period
            If Items.Displayed > 1 Then ' check number of items
               Outpt = "and " + Outpt ' append to string
            End If ' end check number of items
         End If ' end check items
         Call IO.O ' send last output string
         Graphics.Off = False ' reset color
      Case Illuminate ' searches room for magic traps
         Graphics.Off = True ' reset color
         Outpt = "You see " ' set first output string
         Carriage.Return = True ' disable cr/lf
         Call IO.O ' send first string
         Items.Displayed = False ' set items displayed counter
         Item.Displayed = False ' set item displayed flag
         Outpt = Nul ' reset first item string
         For Array.Index = 1 To 20 ' loop through all room treasure
            Inpt = Nul ' set next item displayed
            ' compare treasure magic trap
            If GetRoomRecordFlags(Array.Index) = Magic.Trap Then
               If GetRoomRecordTreasure(Array.Index) = True Then ' check trap active
                  Inpt = "a magical trap " ' make display message
                  ' append magic trap hits to message
                  Inpt = Inpt + "[" + Mid$(Str$(GetRoomRecordTreCharges(Array.Index)), 2) + " hits]"
               End If ' end check active trap
            End If ' end compare magic trap
            If Outpt <> Nul And Inpt <> Nul Then ' check next item display
               Carriage.Return = True ' disable cr/lf
               Call IO.O ' send item output
               ' increment items displayed counter
               Items.Displayed = Items.Displayed + 1
            End If ' end check next item
            If Inpt <> Nul Then ' check items
               Item.Displayed = True ' set item displayed flag
               Outpt = Inpt + ", " ' append comma
            End If ' end check items
         Next ' end loop through room
         If Item.Displayed = False Then ' compare item displayed flag
            Outpt = "nothing special.." ' make display message
         End If ' end check item displayed flag
         If Outpt <> Nul Then ' check last item
            Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, add period
            If Items.Displayed > 1 Then ' check number of items displayed
               Outpt = "and " + Outpt ' append to string
            End If ' end check number of items
         End If ' end check items
         Call IO.O ' send last string
         Graphics.Off = False ' reset color
      Case Psyche ' heal psionic points
         ' calculate psionic points
         Calculate# = UserRecord.Psionic + UserRecord.Level + Multiplier ' calculate
         If Calculate# > MaxInt Then ' check max integer
            Calculate# = MaxInt ' set to max integer
         End If ' end check max integer
         UserRecord.Psionic = CInt(Calculate#) ' store players new psionic points
         Call New.Stats ' routine to update player statistics
         Outpt = "You now have" + Str$(UserRecord.Psionic) + " psionic points!" ' message
         Call IO.O ' send display message
      Case Else ' any other spell type
         Outpt = "You can't cast that spell!" ' make error message
         Call IO.O ' send message
   End Select ' end selection of spell type number
End Sub ' end routine to cast magic spell

Rem * routine to use psionic spell.
Rem * processing variables:
Rem *   Psi.Mode.Type - psi mode 1 or 2 (attack/defense).
Rem * output variables:
Rem *   Psi.Attack.Mode - level of psi spell.
Rem *   Psi.Defense.Mode - level of psi spell.

Sub Psi.Mode
   Select Case LCase$(Parsed.Command1) ' selection of command parameter
      Case "attack" ' attack command
         Psi.Mode.Type = PsiAttack ' store psi attack
      Case "defense" ' defense command
         Psi.Mode.Type = PsiDefense ' store psi defense
      Case Else ' other
         Outpt = "You can only use psi mode attack or defense!" ' make message
         Call IO.O ' send error message
         Exit Sub ' exit routine
   End Select ' end psi type selection
   Outpt = "Psi spell? " ' format input prompt
   Call IO.I ' get input
   Inpt = UCase$(Inpt) ' uppercase input
   Inpt = RTrim$(Inpt) ' trim input
   Spell.Found = False ' psi spell found flag
   ' loop through all spell records
   For Spell.Number = 1 To LOF(SpellFile) / Len(SpellRecord)
      Call Read.Record(SpellFile, Spell.Number) ' get spell record
      Outpt = SpellRecord.Chant ' store spell chant
      Outpt = RTrim$(Outpt) ' trim chant
      If Outpt = Inpt Then ' compare chant to input chant
         If SpellRecord.Psionic Then ' check spell is psi spell
            Spell.Found = True ' set psi spell flag found
            Exit For ' exit spell file loop
         End If ' end check psi spell
      End If ' end check chants
   Next ' end spell file loop
   If Spell.Found = False Then ' check psi spell found flag
      Outpt = "Unknown psi spell!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check psi spell flag
   Spell.Name$ = SpellRecord.SpellName ' store psi spell name
   Spell.Name$ = RTrim$(Spell.Name$) ' trim name
   Spell.Name$ = LCase$(Spell.Name$) ' lowercase name
   If Psi.Mode.Type <> SpellRecord.PsionicMode Then ' verify psi spell mode
      Outpt = "Incorrect spell for psi mode!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end verify psi spell mode
   Level = SpellRecord.Level ' store psi spell level
   If UserRecord.Psionic <= False Then ' check player psionic points remaining
      Outpt = "You have no more psi points!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check player psionic points
   Charges.Number = UserRecord.Psionic ' store player psionic points remaining
   ' decrement player psionic points by level of psi spell
   Charges.Number = Charges.Number - Level
   If Charges.Number < False Then ' compare to zero
      Charges.Number = False ' reset to zero
   End If ' end compare zero
   UserRecord.Psionic = Charges.Number ' store player psionic points remaining
   Outpt = "You cast a " + Spell.Name$ + " psi spell!" ' make message
   Call IO.O ' send psi spell cast message
   Select Case Psi.Mode.Type ' selection of psi spell type
      Case PsiAttack ' attack mode
         Psi.Attack.Mode = Level ' set psi attack mode to psi spell level
         Parsed.Command1 = Last.Monster ' store last monster attacked
         Call Attack.Monster ' attack monster
      Case PsiDefense ' defense mode
         Psi.Defense.Mode = Level ' set psi defense mode to psi spell level
         Outpt = "Your psi defense mode is now active!" ' make message
         Call IO.O ' send psi message
   End Select ' end selection of psi spell type
End Sub ' end routine to cast a psi spell

Rem * routine to get default monster name
Rem * output variables:
Rem *   Monster.Found - true for the first monster
Rem *   Monster.Number - index of monster
Rem *   Last.Monster - name of monster

Sub Get.Last.Monster (Monster.Found)
   Monster.Found = False ' reset return flag
   For Array.Number = 1 To Number.Monsters ' search through monsters in room
      If MonsterArray(Array.Number).Magic >= False Then ' check monster magical
         If MonsterArray(Array.Number).Permanent >= True Then ' check permanent
            Word.Parse1 = False ' reset parsing flag
            Word.Parse2 = False ' reset parsing flag
            Outpts = MonsterArray(Array.Number).MonsterName ' get monster name
            Outpts = RTrim$(Outpts) ' trim name
            Outpts = UCase$(Outpts) ' uppercase name
            Word.Parse1 = InStr(Word.Parse1 + 1, Outpts, " ") ' search for space
            While Word.Parse1 ' loop while spaces
               Word.Parse2 = Word.Parse1 ' store monster name parse
               Word.Parse1 = InStr(Word.Parse1 + 1, Outpts, " ") ' check spaces
            Wend ' end loop while spaces
            Monster.Found = True ' set return flag
            Monster.Number = Array.Number ' store monster array index
            Last.Monster = Mid$(Outpts, Word.Parse2 + 1) ' store monster name
            Exit Sub ' exit routine
         End If ' end check permanent monster
      End If ' end check magic monster
   Next ' end search through room
End Sub ' end routine to get first monster name

Rem * Filename: dnds4.bas Version: v5.0a r2.0a
Rem * This subprogram contains room edit routines, user list routines,
Rem * shopkeeper routines, and some main commands.

Rem * routine to toggle player sort flag.

Sub Sort.Inventory
   Select Case Sorting ' toggle sort method
      Case -1 ' now sort by charges
         Sorting = 0 ' change to no sort
         UserRecord.Sort = 0 ' store in user record
         Outpt = "Inventory sorting off."
      Case 0 ' now not sorting
         Sorting = 1 ' change to sorting by plus
         UserRecord.Sort = 1 ' store in user record
         Outpt = "Inventory plus sorting on."
      Case 1 ' now sort by plus
         Sorting = -1 ' change to sorting by charges
         UserRecord.Sort = -1 ' store in user record
         Outpt = "Inventory charges sorting on."
      Case Else ' not within range
         Sorting = 0 ' default sorting off
         UserRecord.Sort = 0 ' store in user record
         Outpt = "Inventory sorting off."
   End Select ' end toggle sort flag
   Call IO.O ' send display message
End Sub ' end routine to toggle sort flag

Rem * routine to sort player inventory.
Rem * input variables:
Rem *   Sorting - flag for sort toggle.

Sub Sorter
   If Sorting = False Then ' check sorting on
      Exit Sub ' exit routine
   End If ' end check sorting on
   ' perform a simple 'bubble sort' while swapping indexes of weapons variables
   For Sort1 = 1 To 20 ' loop through top bubble of player inventory
      For Sort2 = Sort1 + 1 To 20 ' loop through bottom bubble of player inventory
         If Sorting = -1 Then ' sort by charges
            '  make the comparison of charges left of two inventory items
            If GetUserRecordCharges(Sort1) < GetUserRecordCharges(Sort2) Then
               ' swap the two items if the second one is greater
               Var1 = GetUserRecordInv(Sort1): Var2 = GetUserRecordInv(Sort2)
               Call PutUserRecordInv(Sort1, Var2)
               Call PutUserRecordInv(Sort2, Var1)
               Var1 = GetUserRecordCharges(Sort1): Var2 = GetUserRecordCharges(Sort2)
               Call PutUserRecordCharges(Sort1, Var2)
               Call PutUserRecordCharges(Sort2, Var1)
               GoSub Swap.Weapons ' check weapons being held
            End If ' end compare value of two items
         Else
            If Sorting = 1 Then ' sort by plus
               Inventory1 = GetUserRecordInv(Sort1) ' get treasure index
               Inventory2 = GetUserRecordInv(Sort2) ' get treasure index
               If Inventory1 > 0 And Inventory2 > 0 Then ' compare index values
                  Call Read.Record(TreasureFile, Inventory1) ' get record
                  Inventory.Plus1 = TreasureRecord.Plus ' store treasure plus
                  Call Read.Record(TreasureFile, Inventory2) ' get record
                  Inventory.Plus2 = TreasureRecord.Plus ' store treasure plus
                  If Inventory.Plus1 < Inventory.Plus2 Then ' compare inventory pluses
                     ' swap the two items if the second one is greater
                     Var1 = GetUserRecordInv(Sort1): Var2 = GetUserRecordInv(Sort2)
                     Call PutUserRecordInv(Sort1, Var2)
                     Call PutUserRecordInv(Sort2, Var1)
                     Var1 = GetUserRecordCharges(Sort1): Var2 = GetUserRecordCharges(Sort2)
                     Call PutUserRecordCharges(Sort1, Var2)
                     Call PutUserRecordCharges(Sort2, Var1)
                     GoSub Swap.Weapons ' check weapons being held
                  End If
               End If
            End If
         End If
      Next ' end loop through bubble sort
   Next ' end loop through bubble sort
   Exit Sub ' exit sort routine
   Swap.Weapons:
   ' check if any of the weapons/armor/shield/rings held/worn are
   ' equal to one of the two items being swapped so their indexes
   ' still point to the correct player inventory array elements.
   Select Case Weapon4 ' select an index (armor)
      Case Sort1 ' swap 1
         Weapon4 = Sort2 ' switch index
      Case Sort2 ' swap 2
         Weapon4 = Sort1 ' switch index
   End Select ' end index selection
   Select Case Weapon5 ' select an index (shield)
      Case Sort1 ' swap 1
         Weapon5 = Sort2 ' switch index
      Case Sort2 ' swap 2
         Weapon5 = Sort1 ' switch index
   End Select ' end index selection
   Select Case Weapon6 ' select an index (weapon)
      Case Sort1 ' swap 1
         Weapon6 = Sort2 ' switch index
      Case Sort2 ' swap 2
         Weapon6 = Sort1 ' switch index
   End Select ' end index selection
   Select Case Weapon7 ' select an index (ring)
      Case Sort1 ' swap 1
         Weapon7 = Sort2 ' switch index
      Case Sort2 ' swap 2
         Weapon7 = Sort1 ' switch index
   End Select ' end index selection
   Return
End Sub ' end routine to sort player inventory

Rem * routine to page sysop at console by beeping.
Rem * processing variables:
Rem *   Chat - contains chat mode toggle.

Sub Page.Sysop
   If Normal.User Then ' check non DM
      If UserRecord.Level <= 1 Then ' check player level
         Outpt = "The Sysop is not answering pages now." ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check player level
   End If ' end check normal user
   Beep.Count = False ' reset beep counter
   Graphics.Off = True ' reset color
   Outpt = "Hit <control-k> to return to prompt." ' make message
   Call IO.O ' send message
   Outpt = "Sysop press <escape> to enter chat.." ' make message
   Call IO.O ' send message
   Outpt = "Paging Sysop:" ' make paging message
   Carriage.Return = True ' disable cr/lf
   Call IO.O ' send paging message
   Chat = True ' store chat flag toggle off
   Allow.Break = True ' enable control-k checking
   Break = False ' reset control-k flag
   Beep.Time! = Timer ' store current time
   Do While Chat ' loop until chat entered, 10 beeps, or control-k break entered
      If Break Then ' check control-k entered
         Exit Do ' exit chat loop
      End If ' end check control-k entered
      ' routine to compute time elapsed
      Call Second.Timer(Time.Elapsed, Beep.Time!, 2!)
      If Time.Elapsed Then ' check two seconds elapsed
         Outpt = Chr$(7) + Mask$ ' make remote beep plus character
         Carriage.Return = True ' disable cr/lf
         Call IO.O ' send page message
         Beep.Time! = Timer ' store current time
         Beep.Count = Beep.Count + 1 ' increment beep counter
         If Beep.Count = 10 Then ' check 10 beeps
            Exit Do ' exit chat loop
         End If ' end check beep exit
      End If ' end check time elapsed
   Loop ' end chat loop
   Allow.Break = False ' reset control-k checking off
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
   If Chat = False Then ' compare chat flag on
      Chat = True ' reset chat flag
      Call Enter.Chat ' routine to chat with player
   End If ' end compare chat flag
   Chat = False ' clear chat flag
   Call IO.O ' send empty cr/lf
End Sub

Rem * routine to chat with remote player.
Rem * input variables:
Rem *   Chat - is true, false to quit.
Rem * processing variables:
Rem *   Logged.On! - time player logged on (seconds from midnight).
Rem *   Chat.Start! - time chat started (seconds from midnight).
Rem *   Time.Remaining! - time player had remaining before chat in seconds.

Sub Enter.Chat
   Graphics.Off = True ' reset graphics
   Logged.On! = Timeon ' store timeon
   Chat.Start! = Timer ' store time now
   Time.Remaining! = Time.Left ' store time left
   Timeon = Timer ' reset time on to now
   Time.Left = 60! ' reset time left to 60 seconds
   Allow.Break = False ' disable control-k checking
   Break = False ' reset control-k flag
   Outpt = Nul ' format empty string
   Call IO.O ' send empty cr/lf
   Outpt = "Chat Mode.." ' make chat mode message
   Call IO.O ' send chat mode message
   User.Word.Wrap = User.Wordwrap ' store word wrap
   User.Wordwrap = False ' reset word wrap
   Word.Wrap = True ' enable 80 column word wrap
   Do While Chat ' chat in an input loop
      Timeon = Timer ' reset time on (disables timeout messages)
      Time.Left = 600! ' reset time left (disables timeout messages)
      Call IO.I ' continually process input (from keyboard and modem)
   Loop ' loop until chat toggla flag is reset
   ' end chat, restore time variables
   Timeon = Logged.On! + Fix(Timer - Chat.Start!) ' recalculate time on
   If Timeon > 86400! Then ' check chatted past midnight
      Timeon = Timeon - 86400! ' decrement midnight
   End If ' end check midnight
   Time.Left = Time.Remaining! ' restore time left
   Allow.Break = False ' disable control-k checking
   Break = False ' reset control-k flag
   Buffer = Nul ' clear buffer
   FunctionBuffer = Nul ' clear buffer
   Outpt = Nul ' clear buffer
   Word.Wrap = False ' disable word wrap
   User.Wordwrap = User.Word.Wrap ' restore word wrap
   If Len(Inpt) Then ' check last input
      Call IO.O ' output last input
   End If ' end check last input
   Inpt = Nul ' reset input buffer
End Sub ' end routine to chat with remote player

Rem * routine to display help text for DMs.
Rem * input variables:
Rem *   Stored.Parsed.Command1 - command to look up help text for.

Sub DM.Help
   Help.Command$ = Stored.Parsed.Command1 ' get command parameter
   ' compare first character to DM command prefix
   If Left$(Help.Command$, 1) <> "!" Then
      Outpt = "Enter DM command, form: !Help !<command>" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare DM command prefix
   ' store next part of parameter for command lookup
   Help.Command$ = Mid$(Help.Command$, 2)
   Stored.Parsed.Command1 = Help.Command$ ' store into parsed command variable
   Call Read.Help(True) ' routine to read help text
End Sub ' end routine for DM help text

Rem * routine displays help text for a command.
Rem * input variables:
Rem *   Help.Type is 1=!edit help, 0=normal command, -1=DM command lookup.

Sub Read.Help (Help.Type)
   Help.Command$ = Stored.Parsed.Command1 ' store parameter of command to lookup
   Help.Command$ = RTrim$(Help.Command$) ' trim command
   Help.Command$ = UCase$(Help.Command$) ' uppercase command
   Close #HelpFile ' close work file
   Select Case Help.Type ' selection of help file
      Case True ' check DM command
         FileName = "dmhelp.dat" ' store DM help filename
         Help.Command$ = Left$(Help.Command$, 8) ' truncate command
      Case False ' check normal command
         FileName = "help.dat" ' store normal command help filename
         Help.Command$ = Left$(Help.Command$, 8) ' truncate command
      Case 1 ' check !edit help command
         FileName = "edithelp.dat" ' store !edit help filename
   End Select ' end check command type
   Open FileName For Random Shared As #HelpFile Len = Len(HelpRecord) ' open file
   Do
      Allow.Break = True ' enable control-k checking
      Break = False ' reset control-k flag
      Continue = False ' reset continuous flag
      Graphics.Off = True ' reset color
      Help.Displayed = False ' reset help text displayed flag
      Page.Length = False ' reset page length counter
      For Record.Count = 2 To LOF(HelpFile) / Len(HelpRecord) ' loop through help records
         Call Read.Record(HelpFile, Record.Count) ' get next help record
         Command.Name$ = HelpRecord.CName ' store command name of help record
         Command.Name$ = RTrim$(Command.Name$) ' trim command
         Command.Name$ = UCase$(Command.Name$) ' uppercase command
         If Help.Type <= False Then ' check help file type
            Command.Name$ = Left$(Command.Name$, 8) ' truncate command
         End If ' end check command
         If Help.Displayed = True Then ' check help topic found already
            If Help.Command$ <> Command.Name$ Then ' compare against topic selected
               Exit For ' exit help file display loop
            End If ' end compare topic
         End If ' end check help topic displayed
         If Help.Command$ = Command.Name$ Then ' compare help record command
            If Help.Type = 1 Then ' check displaying !edit help
               If Help.Displayed = False Then ' verify topic already being displayed
                  If Help.Command$ <> "CONTENTS" Then ' compare help type
                     Count.Store = Record.Count ' save help file record counter
                     Topic.Name$ = RTrim$(HelpRecord.CName) ' get command being displayed
                     For Topic.Count = 2 To LOF(HelpFile) / Len(HelpRecord) ' loop through help records
                        Call Read.Record(HelpFile, Topic.Count) ' get next help record
                        Topic.Number$ = HelpRecord.Text ' store command name of help record
                        Topic.Number$ = RTrim$(Topic.Number$) ' trim command
                        Topic.Space = InStr(Topic.Number$, " ") ' store imbdded topic number
                        If Topic.Space Then ' remove topic sequence from topic
                           Topic.Number$ = Left$(Topic.Number$, Topic.Space - 1) ' remove topic
                        End If ' end check for topic sequence
                        If Topic.Number$ = Topic.Name$ Then ' compare to topic
                           Page.Length = Page.Length + 1 ' increment text displayed
                           Outpt = RTrim$(HelpRecord.Text) ' store topic number/name
                           Call IO.O ' send command name message
                           Exit For ' exit loop search
                        End If ' end compare topic
                     Next ' end loop search
                     Record.Count = Count.Store ' restore help file record counter
                  End If ' end compare help type
               End If ' end verify topic displayed
            End If ' end check displaying
            Call Read.Record(HelpFile, Record.Count) ' get help record
            Outpt = HelpRecord.Text ' get command help text
            Outpt = RTrim$(Outpt) ' trim help text
            Outpt = LTrim$(Outpt) ' trim help text
            Call IO.O ' display help text
            Help.Displayed = True ' set text displayed flag
            Page.Length = Page.Length + 1 ' increment page length counter
            If Page.Length = User.PageLength Then ' compare page length
               Page.Length = False ' reset page length counter
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause for more
                  If No Then ' check no more entered
                     Exit For
                  End If ' end check no  entered
               End If ' end check continuous flag
            End If ' end check page length
         End If ' end compare help commands
      Next ' end loop through all help file records
      If Help.Type = 1 Then ' check if !edit help topics are being displayed
         If Record.Count < LOF(HelpFile) / Len(HelpRecord) Then ' verify eof
            Graphics.Off = False ' reset color
            Outpt = "Continue search(y/n)? " ' prompt to continue to next topic
            No.Input.Out = "y" ' default input
            Call IO.I ' get user input
            If Yes Then ' verify to continue display
               Help.Command$ = Command.Name$ ' store last help topic
            Else
               Exit Do
            End If ' end verify to continue
         End If ' end verify file end
      Else
         Exit Do
      End If ' end check help display type
   Loop
   Graphics.Off = False ' reset color
   Allow.Break = False ' clear control-k flag
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
   If Page.Length Then ' check page length
      Call More.Prompt ' routine for more
   End If ' end check page length
   If Help.Displayed = False Then ' check help text displayed flag
      Outpt = "No help found on '" + LCase$(Help.Command$) + "'." ' make error message
      Call IO.O ' send error message
   End If ' end check help text flag
   Close #HelpFile ' close work file
End Sub ' end routine to display help text

Rem * routine trains player for next level.

Sub Train.Stats
   If UserRecord.Level < False Then ' check negative player level
      UserRecord.Level = False ' set player level
   End If ' end check negative level
   If UserRecord.Level >= MaxInt Then ' check player level to maximum integer
      Outpt = "Nothing happens.." ' make error message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check maximum integer
   If UserRecord.Level > False Then ' check player level again
      Call Gold(Required.Gold#) ' get gold required to train for next level
      UserRecord.Gold = UserRecord.Gold - Required.Gold# ' subtract gold from player
   End If ' end check player level
   Stat = Int(Rnd * 7 + 1) ' get random statistic to increment
   ' verify statistic below maximum statistic or player is DM type
   If GetUserRecordStats(Stat) < MaxStat Or Normal.User = False Then ' verify
      Call PutUserRecordStats(Stat, GetUserRecordStats(Stat) + 1) ' increment statistic
   End If ' end verify player type
   If UserRecord.Level <= 10 Then ' compare player level
      If GetUserRecordStats(6) < MaxStat Then ' check statistic below maximum stat
         Call PutUserRecordStats(6, GetUserRecordStats(6) + 1) ' increment piety
      End If ' end check statistic
   End If ' end compare level
   UserRecord.Level = UserRecord.Level + 1 ' increment the player level
   Call New.Stats ' routine to update statistics based on level
   Outpt = "After many hours of training and meditation..." ' make train message
   Call IO.O ' send train message
   Call Display.Health ' routine to display player statistics
   Call Display.Experience ' routine to display player requirements
End Sub ' end routine to train player for next level

Rem * routine recalculates player statistics based on player level.

Sub New.Stats
   If UserRecord.Level < False Then ' check player level
      UserRecord.Level = False ' reset player level
   End If ' end check player level
   ' compute player maximum fatigue points
   New.Stat# = CDbl(Training.Stats(UserRecord.ClassType, 1)) * CDbl(UserRecord.Level)
   If New.Stat# < 0# Then ' compare fatigue points
      New.Stat# = 0# ' reset fatigue points
   End If ' end compare points
   If New.Stat# > MaxInt Then ' compare fatigue points
      New.Stat# = MaxInt ' reset fatigue points
   End If ' end compare points
   ' store new maximum fatigue points in player record
   UserRecord.FatigueMax = CInt(New.Stat#) ' convert points to integer
   ' compute player maximum vitality points
   New.Stat# = CDbl(Training.Stats(UserRecord.ClassType, 2)) * CDbl(UserRecord.Level)
   If New.Stat# < 0# Then ' compare vitality points
      New.Stat# = 0# ' reset vitality points
   End If ' end compare points
   If New.Stat# > MaxInt Then ' compare vitality points
      New.Stat# = MaxInt ' reset vitality points
   End If ' end compare points
   ' store new maximum vitality points in player record
   UserRecord.VitalityMax = CInt(New.Stat#) ' convert points to integer
   ' compute player maximum magic points
   New.Stat# = CDbl(Training.Stats(UserRecord.ClassType, 3)) * CDbl(UserRecord.Level)
   If New.Stat# < 0# Then ' compare magic points
      New.Stat# = 0# ' reset magic points
   End If ' end compare points
   If New.Stat# > MaxInt Then ' compare magic points
      New.Stat# = MaxInt ' reset magic points
   End If ' end compare points
   ' store new maximum magic points in player record
   UserRecord.MagicMax = CInt(New.Stat#) ' convert points to integer
   ' compute player maximum psionic points
   New.Stat# = CDbl(Training.Stats(UserRecord.ClassType, 4)) * CDbl(UserRecord.Level)
   If New.Stat# < 0# Then ' compare psionic points
      New.Stat# = 0# ' reset psionic points
   End If ' end compare points
   If New.Stat# > MaxInt Then ' compare psionic points
      New.Stat# = MaxInt ' reset psionic points
   End If ' end compare points
   ' store new maximum psionic points in player record
   UserRecord.PsionicMax = CInt(New.Stat#) ' convert points to integer
   Stat = UserRecord.Fatigue ' store player fatigue points
   Stat.Max = UserRecord.FatigueMax ' store player maximum fatigue points
   If Stat < 0 Or Stat > Stat.Max Then ' compare fatigue points range
      Stat = Stat.Max ' reset fatigue to maximum fatigue points
   End If ' end compare points
   UserRecord.Fatigue = Stat ' store new fatigue points
   Stat = UserRecord.Vitality ' store player vitality points
   Stat.Max = UserRecord.VitalityMax ' store player maximum vitality points
   If Stat < 0 Or Stat > Stat.Max Then ' compare vitality points range
      Stat = Stat.Max ' reset vitality to maximum vitality points
   End If ' end compare points
   UserRecord.Vitality = Stat ' store new vitality points
   Stat = UserRecord.Magic ' store player magic points
   Stat.Max = UserRecord.MagicMax ' store player maximum magic points
   If Stat < 0 Or Stat > Stat.Max Then ' compare magic points range
      Stat = Stat.Max ' reset magic points to maximum magic points
   End If ' end compare points
   UserRecord.Magic = Stat ' store new magic points
   Stat = UserRecord.Psionic ' store player psionic points
   Stat.Max = UserRecord.PsionicMax ' store player maximum psionic points
   If Stat < 0 Or Stat > Stat.Max Then ' compare spionic points range
      Stat = Stat.Max ' reset psionic points to maximum psionic points
   End If ' end compare points
   UserRecord.Psionic = Stat ' store new psionic points
   ' routine for maximum statistic comparison
   If Normal.User Then ' check non DM
      For Stats = 1 To 7 ' loop through all player statistic points
         If GetUserRecordStats(Stats) > MaxStat Then ' compare statistic to maximum
            Call PutUserRecordStats(Stats, MaxStat) ' reset statistic to maximum
         End If ' end compare points
      Next ' end loop through statistics
   End If
   ' routine for zero stats
   If Normal.User Then
      Low.Stats = False ' reset low stats flag
      For Stats1 = 1 To 7 ' loop through statistics again
         If GetUserRecordStats(Stats1) <= False Then ' check low statistic
            Call PutUserRecordStats(Stats1, 1) ' reset low statistic
            ' loop through statistics again to prevent death loop
            For Stats2 = 1 To 7
               If GetUserRecordStats(Stats2) <= False Then ' check low statistic
                  Call PutUserRecordStats(Stats2, 1) ' reset statistic
               End If ' end check low stat
            Next ' end loop through stats
            Low.Stats = True ' set low stats flag
            Exit For ' exit loop so low stats don't death loop
         End If ' end check low stat
      Next ' end loop through stats
      If Low.Stats Then ' check low stats flag
         ' make death message
         Outpts = LCase$(RTrim$(Stat(Stats1)))
         Message1 = "Your " + Outpts + " is zero! You have died!"
         Call Player.Died ' routine for dead player
         Exit Sub ' exit low stats loop
      End If ' end check low stats flag
   End If ' end check normal player
   ' routine to assign upper class name to player
   If UserRecord.Level >= 10 Then ' check player level
      Class.Number = UserRecord.ClassType ' get player class type
      If Class.Number <= 0 Or Class.Number > 10 Then ' verify bounds of class type
         Class.Number = 1 ' reset class type to fighter
         UserRecord.ClassType = 1 ' reset class type to fighter
      End If ' end verify class type bounds
      Inpt = High.Class.Name(Class.Number) ' get player level 10 class name
      Call Valid(Inpt, 20) ' validate name
      If Len(Inpt) Then ' compare name length
         Call Encrypt(Inpt, True) ' encrypt name
         UserRecord.ClassName = Inpt ' assign class name to player record
      End If ' end compare name length
   End If ' end check player level
   Call Get.User.Stats ' routine to assign more player stats
End Sub ' end routine to recalculate player stats

Rem * routine adds and edits new room.
Rem * input variables:
Rem *   Next.Room - number of new room number to add.
Rem *   Last.Direction - last direction entered.
Rem * output variables:
Rem *   Room.Added - true for a new room added, false if not.

Sub Add.Room (Last.Direction, Room.Added)
   Graphics.Off = True ' reset color
   Outpt = "Add new room(y/n)? " ' make input prompt
   No.Input.Out = "N" ' default input
   Call IO.I ' get input
   Room.Added = False ' set return variable
   If No Then ' check response
      Exit Sub ' exit routine
   End If ' end check response
   Room.Added = True ' set return variable
   Next.Room = LOF(RoomFile) / Len(RoomRecord) + 1 ' store last room record
   Call Clear.Room(Next.Room) ' routine to clear room record
   ' routine to edit room description and monster class
   Call Change.Room(Next.Room)
   Outpt = "Add room link(y/n)? " ' input prompt
   No.Input.Out = "Y" ' default input
   Call IO.I ' get input
   If Yes Then ' check response
      If Last.Direction Then ' check direction entered
         Graphics.Off = True ' reset color
         Outpt = "Press <enter> for entry link:" ' make message
         Call IO.O ' send message
      End If ' end check direction entered
      ' routine to add link to room
      Call Add.Link(Room, Next.Room, Last.Direction)
   End If ' end check response
   Graphics.Off = False ' reset color
   Outpt = "New room" + Str$(Next.Room) + " added." ' make display message
   Call IO.O ' send display message
   Room = Next.Room ' update current room number to added room number
End Sub ' end routine to add new room

Rem * routine clears room record variables.
Rem * input variables:
Rem *   Room.Number! - room number.

Sub Clear.Room (Room.Number!)
   RoomRecord.ShortDesc = Nul ' set short description
   For Array.Index = 1 To 4 ' loop through long description
      Call PutRoomRecordLongDesc(Array.Index, Nul) ' set long description
   Next ' end loop through long description
   RoomRecord.Action = False ' clear variable
   RoomRecord.MonsterClass = False ' clear variable
   For Array.Index = 1 To 12 ' loop through room directions
      Call PutRoomRecordDirect(Array.Index, False) ' clear variable
   Next ' end loop through directions
   For Array.Index = 1 To 20 ' loop through room objects and treasure
      Call PutRoomRecordObject(Array.Index, False) ' clear variable
      Call PutRoomRecordHiddenObj(Array.Index, False) ' clear variable
      Call PutRoomRecordTreasure(Array.Index, False) ' clear variable
      Call PutRoomRecordTreCharges(Array.Index, False) ' clear variable
      Call PutRoomRecordFlags(Array.Index, False) ' clear variable
   Next ' end loop through room objects and treasure
   Call Clear.Container(0, True) ' routine to clear container record
   RoomRecord.Container = ContainerRec ' clear variable
   Call Share.Room.Record(Room.Number!) ' write room record number
End Sub ' end routine to clear room record

Rem * routine to add, delete, and list room links.

Sub Link.Room
   Do ' input entry loop
      Graphics.Off = False ' reset color
      Outpt = "Room link edit:" ' make option message
      Call IO.O ' send option message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make message
      Call IO.O ' send message
      Outpt = "[D]elete" ' make message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Enter room link option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get input
      Graphics.Off = True ' reset color
      Select Case UCase$(Inpt) ' make selection of input
         Case "A" ' add link
            Outpt = "Enter room number" ' make range prompt
            Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' make range number
            ' routine to get number from range
            Call Get.Room.Range2(0!, Max.Rooms!, Room.From!)
            If Room.From! Then ' check range
               Outpt = "Enter link room number" ' make range prompt
               ' routine to get number from range
               Call Get.Room.Range2(0!, Max.Rooms!, Room.To!)
               If Room.To! Then ' check range
                  ' routine to link two room numbers
                  Call Add.Link(Room.From!, Room.To!, False)
               End If ' end check range
            End If ' end check range
         Case "D" ' delete link
            Outpts = "Link not deleted." ' make default response
            Outpt = "Enter room number" ' make range prompt
            Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' make range number
            ' routine to get number from range
            Call Get.Room.Range2(0!, Max.Rooms!, Room.Delete!)
            If Room.Delete! Then ' check range
               Outpt = "Enter direction(N/E/S/W/NE/SE/SW/NW/U/D/I/O)? "
               Call IO.I ' get input
               ' routine to get room link number
               Call Find.Link(Inpt, Room.Link, False)
               If Room.Link >= 1 And Room.Link <= 12 Then ' check link number
                  Call Read.Room.Record(Room.Delete!) ' get room record
                  Call PutRoomRecordDirect(Room.Link, False) ' clear room link number
                  Call Share.Room.Record(Room.Delete!) ' write room record
                  Outpts = "Room" + Str$(Room.Delete!) + ", " + RTrim$(Direction(Room.Link)) + " link removed."
               End If ' end check link number
            End If ' end check range
            Outpt = Outpts ' store response
            Call IO.O ' send message
         Case "L" ' list links
            Graphics.Off = False ' reset color
            Outpt = "Enter range of room numbers:" ' make display message
            Call IO.O ' send message
            Graphics.Off = True ' reset color
            Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' store length of room file
            ' get range of rooms to list
            Call Get.Room.Range(Max.Rooms!, Room.List1!, Room.List2!)
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            Page.Length = False ' reset page length counter
            For Room.Number! = Room.List1! To Room.List2! ' loop through rooms to list
               If Page.Length + 4 >= User.PageLength Then ' compare page length
                  Page.Length = False ' clear page length
                  If Continue = False Then ' check continuous flag
                     Call More.Prompt ' routine to pause
                     If No Then ' check more promtp response
                        Exit For ' exit room link display loop
                     End If ' end check response
                  End If ' end check continuouu flag
               End If ' end compare page length
               Call Read.Room.Record(Room.Number!) ' get next room record
               Call Display.Room.Links(Room.Number!) ' display room links
               If Break Or No Then ' check break flag
                  Exit For ' exit display loop
               End If ' end check break flag
               Page.Length = Page.Length + 4 ' increment page length counter
            Next ' end loop through rooms
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
            If Page.Length Then ' check page length counter
               Call More.Prompt ' pause prompt
            End If ' end check page length
         Case "Q" ' quit
            Exit Do ' exit input loop
      End Select ' end selection of input
   Loop ' end input loop
   Call Read.Room.Record(Room) ' get current room record
End Sub ' end DM link routine

Rem * routine adds links between two room numbers.
Rem * input variables:
Rem *   Room.Number1! - room to link.
Rem *   Room.Number2! - room to link.
Rem *   Entry.Link - default entry link.

Sub Add.Link (Room.Number1!, Room.Number2!, Entry.Link)
   ' make direction link prompt
   Outpt = "Enter direction(N/E/S/W/NE/SE/SW/NW/U/D/I/O)? "
   Call IO.I ' get input
   ' routine converts direction to link number
   Call Find.Link(Inpt, Direction.Number, Entry.Link)
   If Direction.Number = False Or Direction.Number = 13 Then ' check link number
      Outpt = "Link not added." ' make error message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check link number
   Call Read.Room.Record(Room.Number1!) ' get room number one to link
   Call PutRoomRecordDirect(Direction.Number, Room.Number2!) ' add link to room
   Call Share.Room.Record(Room.Number1!) ' write room record
   ' make link message
   Outpt = "Room" + Str$(Room.Number1!) + " link added to room" + Str$(Room.Number2!) + "."
   Call IO.O ' send message
   ' make link back prompt
   Outpt = "Link room" + Str$(Room.Number2!) + " back to room" + Str$(Room.Number1!) + "(y/n)? "
   No.Input.Out = "Y" ' default input
   Call IO.I ' get input
   If Yes Then ' check response
      Call Find.Back.Link(Direction.Number, Return.Link) ' get return link
      If Return.Link Then ' check return link
         Call Read.Room.Record(Room.Number2!) ' get room record
         ' add return link to first room
         Call PutRoomRecordDirect(Return.Link, Room.Number1!)
         Call Share.Room.Record(Room.Number2!) ' write room record
         ' make return link message
         Outpt = "Room" + Str$(Room.Number2!) + " link added back to room" + Str$(Room.Number1!) + "."
         Call IO.O ' send message
      End If ' end check return link
   End If ' end check response
End Sub ' end routine to link two rooms

Rem * routine converts direction string to number.
Rem * input variables:
Rem *   Direction.Name$ - string of direction name.
Rem *   Entry.Link - default entry link number.
Rem * output variables:
Rem *   Direction.Number - direction number.

Sub Find.Link (Direction.Name$, Direction.Number, Entry.Link)
   Select Case UCase$(Direction.Name$) ' select direction string
      Case "N" ' north
         Direction.Number = 1 ' direction number
      Case "E" ' east
         Direction.Number = 2 ' direction number
      Case "S" ' south
         Direction.Number = 3 ' direction number
      Case "W" ' west
         Direction.Number = 4 ' direction number
      Case "NE" ' northeast
         Direction.Number = 5 ' direction number
      Case "SE" ' southeast
         Direction.Number = 6 ' direction number
      Case "SW" ' southwest
         Direction.Number = 7 ' direction number
      Case "NW" ' northwest
         Direction.Number = 8 ' direction number
      Case "U" ' up
         Direction.Number = 9 ' direction number
      Case "D" ' down
         Direction.Number = 10 ' direction number
      Case "I" ' in
         Direction.Number = 11 ' direction number
      Case "O" ' out
         Direction.Number = 12 ' direction number
      Case "G" ' go to portal
         Direction.Number = 13 ' go to direction number
      Case Else ' otherwise
         Direction.Number = Entry.Link ' no direction found
   End Select ' end selection of direction
End Sub ' end routine to convert direction

Rem * routine determines direction opposite to input direction.
Rem * input variables:
Rem *   Direction.Number - direction number.
Rem * output variables:
Rem *   Return.Direction - opposite direction number.

Sub Find.Back.Link (Direction.Number, Return.Direction)
   Select Case Direction.Number ' selection of direction number
      Case 1 ' north
         Return.Direction = 3 ' south
      Case 2 ' east
         Return.Direction = 4 ' west
      Case 3 ' south
         Return.Direction = 1 ' north
      Case 4 ' west
         Return.Direction = 2 ' east
      Case 5 ' northeast
         Return.Direction = 7 ' southwest
      Case 6 ' southeast
         Return.Direction = 8 ' northwest
      Case 7 ' southwest
         Return.Direction = 5 ' northeast
      Case 8 ' northwest
         Return.Direction = 6 ' southeast
      Case 9 ' up
         Return.Direction = 10 ' down
      Case 10 ' down
         Return.Direction = 9 ' up
      Case 11 ' in
         Return.Direction = 12 ' out
      Case 12 ' out
         Return.Direction = 11 ' in
      Case Else ' default
         Return.Direction = 0 ' no direction
   End Select ' end selection of direction
End Sub ' end routine to find return direction

Rem * routine displays room number links.
Rem * input variables:
Rem *   Room.Number! - room number to display.

Sub Display.Room.Links (Room.Number!)
   If Room.Number! <= 0! Then ' check number
      Exit Sub ' exit subroutine
   End If ' end check number
   Graphics.Off = False ' reset color
   Outpt = "Room number" + Str$(Room.Number!) + ":" ' make room number message
   Call IO.O ' send message
   Graphics.Off = True ' reset color
   Outpt = Nul ' clear output string
   For Link.Number = 1 To 4 ' loop through room directions
      ' make display line containing room directions
      Inpt = Mid$("NESW", Link.Number, 1) + "  " + Str$(GetRoomRecordDirect(Link.Number))
      Outpt = Outpt + Inpt + Space$(10 - Len(Inpt)) ' pad with blanks
   Next ' end loop through room directions
   Call IO.O ' send line one of room directions
   Outpt = Nul ' clear output string
   For Link.Number = 5 To 8 ' loop through room directions
      ' make display line containing room directions
      Inpt = Mid$("NESESWNW", (Link.Number - 4) * 2 - 1, 2) + " " + Str$(GetRoomRecordDirect(Link.Number))
      Outpt = Outpt + Inpt + Space$(10 - Len(Inpt)) ' pad with blanks
   Next ' end loop through room directions
   Call IO.O ' send line one of room directions
   Outpt = Nul ' clear output string
   For Link.Number = 9 To 12 ' loop through room directions
      ' make display line containing room directions
      Inpt = Mid$("UDIO", Link.Number - 8, 1) + "  " + Str$(GetRoomRecordDirect(Link.Number))
      Outpt = Outpt + Inpt + Space$(10 - Len(Inpt)) ' pad with blanks
   Next ' end loop through room directions
   Call IO.O ' send line two of room directions
   Outpt = Nul ' clear output string
End Sub ' end routine to display room links

Rem * routine moves player in a certain direction.
Rem * input variables:
Rem *   Direction.Number - direction to go.
Rem * processing variables:
Rem *   New.Room - true if room can be entered, false if not.

Sub Go.Direction (Direction.Number)
   ' change last command to direction
   Entry.Command = Last.Command.Number ' store room entry command
   Last.Command = UCase$(Direction(Direction.Number))
   Last.Command.Number = True ' reset last command number
   ' routine to check if next room can be entered
   Call Verify.Room(Direction.Number)
   If New.Room Then ' check next room variable
      Call Enter.Room ' routine to move the player
   End If ' end check next room
End Sub ' end routine to move player

Rem * routine compares player input to direction.
Rem * input variables:
Rem *   User.Command - original command input.
Rem * output variables:
Rem *   Direction.Number - contains direction number.

Sub Get.Direction (Direction.Number)
   For Direction.Number = 1 To 12 ' loop through direction names
      Outpts = Direction(Direction.Number) ' store direction name
      Outpts = RTrim$(Outpts) ' trim name
      Outpts = UCase$(Outpts) ' uppercase name
      If User.Command = Outpts Then ' compare to player direction
         Exit Sub ' exit routine
      End If ' end check directions
   Next ' end loop through direction names
   ' routine to get direction
   Call Find.Link(User.Command, Direction.Number, False)
   If Direction.Number = 13 Then ' check enter portal link
      Direction.Number = False ' reset direction
   End If ' end check direction
End Sub ' end routine to compare direction

Rem * routine to add item of treasure to player inventory.
Rem * input variables:
Rem *   Treasure.Number - treasure file index number.
Rem *   Treasure.Charges - treasure charges.
Rem * output variables:
Rem *   Item.Added - true if item added, false if not.

Sub Add.Inventory (Treasure.Number, Treasure.Charges, Item.Added)
   Item.Added = False ' clear return variable
   For Array.Index = 1 To 20 ' loop through all player inventory
      If GetUserRecordInv(Array.Index) = False Then ' check for empty inventory
         Call PutUserRecordInv(Array.Index, Treasure.Number) ' add treasure index
         Call PutUserRecordCharges(Array.Index, Treasure.Charges) ' add treasure charges
         Weight = Weight + TreasureRecord.Weight ' increment player weight
         Item.Added = True ' set return flag
         Exit Sub ' exit routine
      End If ' end check empty inventory
   Next ' end loop through player inventory
End Sub ' end routine to add item of treasure to player inventory

Rem * routine to add an object to player inventory.
Rem * input variables:
Rem *   Object.Number - object file index number.
Rem * output variables:
Rem *   Item.Added - true if item added, false if not.

Sub Add.Object (Object.Number, Item.Added)
   Item.Added = False ' clear return variable
   For Array.Index = 1 To 20 ' loop through all player inventory
      If GetUserRecordObject(Array.Index) = False Then ' check for empty inventory
         Call PutUserRecordObject(Array.Index, Object.Number) ' add object index
         Item.Added = True ' set return flag
         Exit Sub ' exit routine
      End If ' end check empty inventory
   Next ' end loop through player inventory
End Sub ' end routine to add item of treasure to player inventory

Rem * routine removes an item of treasure from player inventory.
Rem * input variables:
Rem *   Inventory.Number - number of inventory.
Rem *   Leave.Item - false to leave inventory in room, true to discard.

Sub Discard.Inventory (Inventory.Number, Leave.Item)
   ' store player treasure index
   Inventory.Index = GetUserRecordInv(Inventory.Number)
   ' store player treasure charges
   Inventory.Charges = GetUserRecordCharges(Inventory.Number)
   Call Read.Record(TreasureFile, Abs(Inventory.Index)) ' get treasure record of item
   Weight = Weight - TreasureRecord.Weight ' subtract weight
   If Weight < False Then ' compare weight
      Weight = False ' clear weight
   End If ' end check weight
   Call Discard.Inventory.Treasure(Inventory.Number) ' remove inventory item
   If GetUserRecordInv(1) = False Then ' check player inventory empty
      Weight = False ' clear weight
   End If ' end check player inventory
   Select Case Inventory.Number ' select weapon
      Case Weapon4 ' check armor being dropped
         Weapon1 = False ' clear item
         Weapon4 = False ' clear item
      Case Weapon5 ' check shield being dropped
         Weapon3 = False ' clear item
         Weapon5 = False ' clear item
      Case Weapon6 ' check weapon being dropped
         Weapon2 = False ' clear item
         Weapon6 = False ' clear item
         Weapon10 = False ' clear item
      Case Weapon7 ' check ring being dropped
         Weapon7 = False ' clear item
         Weapon8 = False ' clear item
         Weapon9 = False ' clear item
         Weapon11 = False ' clear item
   End Select ' end check item
   ' check inventory number
   If Inventory.Number < Weapon4 Then ' check armor index
      Weapon4 = Weapon4 - 1 ' shift item index
   End If ' end check inventory
   ' check inventory number
   If Inventory.Number < Weapon5 Then ' check shield index
      Weapon5 = Weapon5 - 1 ' shift item index
   End If ' end check inventory
   ' check inventory number
   If Inventory.Number < Weapon6 Then ' check weapon index
      Weapon6 = Weapon6 - 1 ' shift item index
   End If ' end check inventory
   ' check inventory number
   If Inventory.Number < Weapon7 Then ' check ring index
      Weapon7 = Weapon7 - 1 ' shift item index
   End If ' end check inventory
   If Leave.Item = False Then ' verify drop item in room
      ' routine to add item to room
      Call Add.Room.Treasure(Inventory.Index, Inventory.Charges, False, Item.Added)
   End If ' end check room
End Sub ' end routine to remove player item

Rem * routine adds item to room.
Rem * input variables:
Rem *   Treasure.Number - treasure file index.
Rem *   Treasure.Charges - treasure charges.
Rem *   Treasure.Flags - flags
Rem * output variables:
Rem *   Item.Added - return true if item added to room, false if not.

Sub Add.Room.Treasure (Treasure.Number, Treasure.Charges, Treasure.Flags, Item.Added)
   Item.Added = False ' clear return flag
   For Array.Index = 1 To 20 ' loop through room treasure inventory
      ' check empty room inventory
      If GetRoomRecordTreasure(Array.Index) = False Then
         Call PutRoomRecordTreasure(Array.Index, Treasure.Number) ' add treasure index
         Call PutRoomRecordTreCharges(Array.Index, Treasure.Charges) ' add treasure charges
         Call PutRoomRecordFlags(Array.Index, Treasure.Flags) ' add treasure flags
         Call Share.Room.Record(Room) ' write room record
         Item.Added = True ' set return flag
         Exit Sub ' exit routine
      End If ' end check empty inventory
   Next ' end loop through room treasure inventory
End Sub ' end routine to add item to room

Rem * routine removes item of treasure from room.
Rem * input variables:
Rem *   Inventory.Number - room inventory number to remove.

Sub Discard.Room.Treasure (Inventory.Number)
   Call PutRoomRecordTreasure(Inventory.Number, False) ' clear treasure items
   Call PutRoomRecordTreCharges(Inventory.Number, False) ' clear treasure items
   Call PutRoomRecordFlags(Inventory.Number, False) ' clear treasure items
   Call Share.Room.Record(Room) ' write room record
End Sub ' end routine to remove item from room

Rem * routine adds object to room.
Rem * input variables:
Rem *   Object.Number - object index.
Rem *   Hide.Item - true to hide object.
Rem * output variables:
Rem *   Item.Added - true if item added, false if not.

Sub Add.Room.Object (Object.Number, Hide.Item, Item.Added)
   Item.Added = False ' clear return flag
   For Array.Index = 1 To 20 ' loop through room object inventory
      ' check empty object inventory
      If GetRoomRecordObject(Array.Index) = False Then
         Call PutRoomRecordObject(Array.Index, Object.Number) ' add object index
         Call PutRoomRecordHiddenObj(Array.Index, Hide.Item) ' add hidden object
         Call Share.Room.Record(Room) ' write room record
         Item.Added = True ' set return flag
         Exit Sub ' exit routine
      End If ' end check empty object inventory
   Next ' end loop through room object inventory
End Sub ' end routine to add item to room

Rem * routine removes an object from room.
Rem * input variables:
Rem *   Inventory.Number - room inventory number to remove.

Sub Discard.Room.Object (Inventory.Number)
   Call PutRoomRecordObject(Inventory.Number, False) ' clear object item
   Call PutRoomRecordHiddenObj(Inventory.Number, False) ' clear object item
   Call Share.Room.Record(Room) ' write room record
End Sub ' end routine to remove item from room

Rem * routine removes an object from inventory.
Rem * input variables:
Rem *   Inventory.Number - player inventory number to remove.

Sub Discard.Inventory.Object (Inventory.Number)
   For Inventory.Count = Inventory.Number To 19 ' loop packer
      ' shift object items
      Call PutUserRecordObject(Inventory.Count, GetUserRecordObject(Inventory.Count + 1))
   Next ' end packing
   Call PutUserRecordObject(20, False) ' clear last object
End Sub ' end routine to remove item from room

Rem * routine removes a treasure from inventory,
Rem * and packs user inventory.
Rem * input variables:
Rem *   Inventory.Number - player inventory number to remove.

Sub Discard.Inventory.Treasure (Inventory.Number)
   For Inventory.Count = Inventory.Number To 19 ' loop packer
      ' shift treasure items
      Call PutUserRecordInv(Inventory.Count, GetUserRecordInv(Inventory.Count + 1))
      ' shift treasure items
      Call PutUserRecordCharges(Inventory.Count, GetUserRecordCharges(Inventory.Count + 1))
   Next ' end packing
   Call PutUserRecordInv(20, False) ' clear last treasure
   Call PutUserRecordCharges(20, False) ' clear last treasure
End Sub ' end routine to remove item from room

Rem * routine removes treasure from room after player leaves to new room.
Rem * input variables:
Rem *   Room - number of room to clean.

Sub Clean.Room
   If Room > False And Room <= LOF(RoomFile) / Len(RoomRecord) Then ' check file bounds
      Call Read.Room.Record(Room) ' get room record
      If RoomRecord.Container.Permanent = False Then ' check permanent container
         Call Clear.Container(0, True) ' routine to clear container record
         RoomRecord.Container = ContainerRec ' store container record in room
         Call Share.Room.Record(Room) ' write room record
      End If ' end check permanent container
      For Array.Index = 1 To 20 ' loop through room treasure
         Treasure.Number = GetRoomRecordTreasure(Array.Index) ' store treasure index
         If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
            Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
            If TreasureRecord.Permanent = False Then ' check permanent treasure
               If TreasureRecord.Invisible = False Then ' check invisible treasure
                  ' check treasure flags
                  If GetRoomRecordFlags(Array.Index) = False Then
                     ' remove item from room
                     Call Discard.Room.Treasure(Array.Index)
                  End If ' end check flags
               End If ' end check invisible
            End If ' end check permanent
         End If ' end check file bounds
      Next ' end loop through room treasure
   End If ' end check room number range
End Sub ' end routine to clean room

Rem * routine lists users.

Sub User.List
   Graphics.Off = True ' reset color
   GoSub Heading ' subroutine to display heading
   Allow.Break = True ' enable control-k checking
   Break = False ' reset control-k flag
   Continue = False ' reset continuous flag
   Page.Length = 3 ' set page length
   For User.Number = 1 To LOF(UserFile) / Len(UserRecord) ' loop through all users
      Call Read.Record(UserFile2, User.Number) ' get next user record
      Inpt = UserRecord2.CodeName ' store user codename
      Call Decrypt(Inpt) ' routine to decrypt codename
      Inpt = LCase$(Inpt) ' lowercase codename,
      List.User = True ' set list user flag
      If Left$(Inpt, 9) = Deleted$ Then ' check user record exists
         List.User = False ' reset list user flag
      Else ' check user
         If UserRecord2.Flags And Locked.User Then ' check locked user
            If Normal.User Then ' compare DM or Sysop
               List.User = False ' reset list user flag
            End If ' end compare normal user
         End If ' end check locked mailbox
      End If ' end check user
      If List.User Then ' check list user flag
         Outpt = Mid$(Str$(User.Number), 2) ' make output line
         Outpt = Outpt + Space$(9 - Len(Outpt)) ' with user number,
         Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' with codename,
         Outpt = Outpt + Inpt + " " ' make output line
         Inpt = UserRecord2.ClassName ' with user class name,
         Call Decrypt(Inpt) ' decrypt classname
         Outpt = Outpt + Inpt + " " ' make output line
         If UserRecord2.Race <= False Then ' verify race in bounds
            UserRecord2.Race = 1 ' reset race
         End If ' end verify race
         Inpt = Race(UserRecord2.Race) ' with user race name
         Inpt = RTrim$(Inpt) ' make output line
         Inpt = Inpt + Space$(8 - Len(Inpt)) ' append blanks
         Outpt = Outpt + Inpt ' with race name,
         If UserRecord2.Level <= False Then ' check player level
            Inpt = " " + Dead$ ' player is dead, append
         Else ' check level
            Inpt = Str$(UserRecord2.Level) ' add player level,
         End If ' end check level
         Inpt = Inpt + Space$(7 - Len(Inpt)) ' make output line
         If UserRecord2.ClassType >= AsstDM Then ' check special class type,
            Inpt = Inpt + "*" ' add an asterick for DMs
         End If ' end check class type
         If UserRecord2.Flags And Locked.User Then ' check locked user
            Inpt = Inpt + Mask$ ' add mask character for locked mailbox
         End If ' end check class type
         Outpt = Outpt + Inpt ' add to output line
         Call IO.O ' send output line
         If Break Then ' check control-k entered
            Exit For ' exit loop through user file
         End If ' end check control-k
         Page.Length = Page.Length + 1 ' increment page length
         If Page.Length = UserRecord.Pagelength Then ' compare page length
            Page.Length = 3 ' reset page length
            If Continue = False Then ' check continuous flag
               Call More.Prompt ' pause for more
               If No Then ' more response
                  Exit For ' exit loop through user file
               End If ' end check response
               GoSub Heading ' subroutine to display heading
            End If ' end check continuous flag
         End If ' end compare page length
      End If ' end check valid user
   Next ' end loop through user file
   Allow.Break = False ' disable control-k checking
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
   If Page.Length > 3 Then ' recheck page length
      Call More.Prompt ' display more prompt
   End If ' end check last page length
   Exit Sub ' exit routine

   Heading:
   ' make heading message
   Outpt = "The Adventure Door v" + Version$ + " User List For " + clock$ + "."
   Call IO.O ' send message
   Outpt = Nul ' empty cr/lf
   Call IO.O ' send empty cr/lf
   ' make heading message
   Outpt = "Number   User Name                      Class Name           Race     Level DM"
   Call IO.O ' send heading message
   Outpt = String$(78, "-") ' make heading
   Call IO.O ' send heading
   Return ' exit subroutine
End Sub ' end routine to display users

Rem * routine reads player userfile record, sets some variables.
Rem * input variables:
Rem *   User.Index - number of user file record.

Sub Get.User.Record
   Call Read.Record(UserFile, User.Index) ' read user file record
   Weight = False ' clear inventory weight
   For Inventory.Number = 1 To 20 ' loop through player inventory
      Treasure.Number = GetUserRecordInv(Inventory.Number) ' store treasure number
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasur record
         Weight = Weight + TreasureRecord.Weight ' add weight
      End If ' end check file bounds
   Next ' end loop through player userfile record
   User.Echo = UserRecord.Echo ' store user preference
   User.LineFeeds = UserRecord.Linefeeds ' store user preference
   User.LineLength = UserRecord.Linelength ' store user preference
   User.PageLength = UserRecord.Pagelength ' store user preference
   User.Wordwrap = UserRecord.Wordwrap ' store user preference
   Call Get.User.Stats ' routine to set any special player statistics
   Room = UserRecord.Room ' store player room number
   Hidden.Player = False ' reset hidden player flag
   If UserRecord.Brief Then ' check player brief mode
      Action.Prompt = "Next?" ' store command prompt
   Else ' check player
      Action.Prompt = "Command? " ' store command prompt
   End If ' end compare player brief mode
   If UserRecord.Beauty <= 1 Or UserRecord.Beauty > MaxStat Then ' check lady player
      UserRecord.Beauty = Int(Rnd * 15 + 5) ' recalculate lady stats
   End If ' end check lady stats
   ' check lady player
   If UserRecord.Glamour <= 1 Or UserRecord.Glamour > MaxStat Then
      UserRecord.Glamour = Int(Rnd * 15 + 5) ' recalculate lady stats
   End If ' end check lady stats
   Sorting = UserRecord.Sort ' get user sort preference
End Sub ' end routine to read user record

Rem * routine sets any special player statistics and attributes.

Sub Get.User.Stats
   If UserRecord.Flags And Special.Char1 Then ' check player is town mayor
      Town.Mayor = True ' set town mayor flag
   Else ' check player
      Town.Mayor = False ' set town mayor flag
   End If ' end check player special stats
   If UserRecord.Flags And Special.Char2 Then ' check player is governor
      Governor = True ' set governor flag
   Else ' check player
      Governor = False ' set governor flag
   End If ' end check player special stats
   If UserRecord.Flags And Special.Char3 Then ' check player is guild master
      Guild.Master = True ' set guild master flag
   Else ' check player
      Guild.Master = False ' set guild master flag
   End If ' end check player special stats
   If UserRecord.Flags And Special.Char4 Then ' check player is sysop
      Sysop = True ' set sysop flag
   Else ' check player
      Sysop = False ' set sysop flag
   End If ' end check player special stats
   If UserRecord.ClassType = AsstDM Then ' check player is assistant DM
      Dungeon.Master.Assistant = True ' set asst. dm flag
   Else ' check player
      Dungeon.Master.Assistant = False ' set asst. dm flag
   End If ' end check player
   If UserRecord.ClassType = DM Then ' check player is dungeon master
      Dungeon.Master = True ' set dm flag
   Else ' check player
      Dungeon.Master = False ' set dm flag
   End If ' end check player
   ' check special player types
   If Dungeon.Master Or Dungeon.Master.Assistant Or Sysop Then
      Normal.User = False ' set normal player mode off
   Else ' check player
      Normal.User = True ' set normal player mode on
   End If ' end check player
End Sub ' end routine to set player special statistics/attributes

Rem * routine writes user record.

Sub Put.User.Record
   UserRecord.Echo = User.Echo ' write user preference
   UserRecord.Linefeeds = User.LineFeeds ' write user preference
   UserRecord.Linelength = User.LineLength ' write user preference
   UserRecord.Pagelength = User.PageLength ' write user preference
   UserRecord.Wordwrap = User.Wordwrap ' write user preference
   UserRecord.Sort = Sorting ' user sort preference
   UserRecord.Room = Room ' store current room
   Call Share.Record(UserFile, User.Index) ' write user record
End Sub ' end routine to write user record

Rem * routine updates player health, room lights, drunkeness, and poison.
Rem * input variables:
Rem *   Room.Rate - stores rounds counter.
Rem *   Room.Health.Rate - stores health update rate.

Sub Health.Update
   Graphics.Off = False ' reset color
   Room.Rate = Room.Rate + 1 ' increment room health counter
   If Room.Rate < Room.Health.Rate Then ' compare health rate
      Exit Sub ' exit routine
   End If ' end check counter
   Room.Rate = False ' reset room health counter
   ' determine any lights in player inventory go out
   For Inventory.Number = 1 To 20 ' loop through player inventory
      ' get player inventory charges
      Charges.Number = GetUserRecordCharges(Inventory.Number)
      If Charges.Number < False Then ' check for light on
         Charges.Number = Charges.Number + 1 ' decrement negatively light charges
         Call PutUserRecordCharges(Inventory.Number, Charges.Number) ' store new charges
         If Charges.Number = False Then ' compare charges
            ' get treasure record
            Call Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            Outpts = TreasureRecord.ShortName ' get light name
            Outpts = RTrim$(Outpts) ' trim name
            Outpts = LCase$(Outpts) ' lowercase name
            Outpt = "The " + Outpts + " went out!" ' make light out message
            Call IO.O ' send message
         End If ' end compare charges remaining
      End If ' end check light on
   Next ' end loop through player inventory
   If Intoxicated > False Then ' verify player drunk
      Intoxicated = Intoxicated - 1 ' decrement drunkeness
      If Intoxicated <= False Then ' compare drunkeness counter
         Intoxicated = False ' reset counter
         Outpt = "Your drunk is over.." ' make message
      Else ' check drunk
         UserRecord.Fatigue = UserRecord.Fatigue - 2 ' decrement fatigue for drunk
         If UserRecord.Fatigue <= False Then ' compare fatigue
            UserRecord.Fatigue = False ' reset fatigue
            Intoxicated = False ' reset drunkeness
            Outpt = "Your drunk is over.." ' make message
         Else ' compare still drunk
            Outpt = "You feel drunk!" ' make message
         End If ' end compare drunk
      End If ' end check drunk
      Call IO.O ' send drunkeness message
   Else ' verify drunk player
      New.Stat! = UserRecord.Fatigue + 4 ' increment player fatigue
      If New.Stat! > MaxInt Then ' check maximum fatigue
         New.Stat! = MaxInt ' reduce to maximum integer
      End If ' end check maximum fatigue
      UserRecord.Fatigue = CInt(New.Stat!) ' store new fatigue
   End If ' end verify drunk player
   If UserRecord.Poison Then ' verify player poisoned
      UserRecord.Vitality = UserRecord.Vitality - 2 ' decrement player vitality
      Outpt = "You feel poison running through your veins!" ' make message
      Call IO.O ' send poisoned message
      If UserRecord.Vitality <= False Then ' check vitality
         UserRecord.Vitality = False ' reset vitality
         Message1 = "You finally died from your poisonous wounds!" ' message
         Call Player.Died ' routine for dead player
      End If ' end check vitality
   Else ' verify poisoned player
      New.Stat! = UserRecord.Vitality + 3 ' increment player vitality
      If New.Stat! > MaxInt Then ' check maximum integer
         New.Stat! = MaxInt ' reset to maximum integer
      End If ' end check maximum integer
      UserRecord.Vitality = CInt(New.Stat!) ' store new vitality
   End If ' end verify poisoned player
   New.Stat! = UserRecord.Magic + 2 ' increment player magic points
   If New.Stat! > MaxInt Then ' compare magic points to maximum integer
      New.Stat! = MaxInt ' reset to maximum integer
   End If ' end check maximum integer
   UserRecord.Magic = CInt(New.Stat!) ' store new magic points
   New.Stat! = UserRecord.Psionic + 1 ' increment psionic points
   If New.Stat! > MaxInt Then ' check maximum psionic points
      New.Stat! = MaxInt ' reset to maximum integer
   End If ' end check maximum integer
   UserRecord.Psionic = CInt(New.Stat!) ' store new psionic points
   Call New.Stats ' routine to update statistics
   If Invisible > False Then ' check invisible counter
      Invisible = Invisible - 1 ' decrement invisible counter
      If Invisible <= False Then ' compare counter
         UserRecord.Invisible = False ' reset invisible
         Invisible = False ' reset invisible
         Outpt = "You are no longer invisible!" ' make message
         Call IO.O ' send invisible message
      End If ' end compare counter
   End If ' end check counter
End Sub ' end health update routine

Rem * routine searches current room, displays hidden items.

Sub Search.Room
   Outpt = "You search the room.." ' make message
   Call IO.O ' send search message
   Graphics.Off = True ' reset color
   Outpt = "You find " ' make first display message
   Carriage.Return = True ' disable cr/lf
   Call IO.O ' send first message
   Items.Displayed = False ' items displayed counter
   For Room.Objects = 1 To 20 ' loop through room objects
      Object.Number = GetRoomRecordObject(Room.Objects) ' get room object index
      ' check object file bounds
      If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
         Call Read.Record(ObjectFile, Object.Number) ' get object record
         Display.Item = False ' set display flag
         Hidden.Flag = GetRoomRecordHiddenObj(Room.Objects) ' reset hidden flag
         If ObjectRecord.Invisible Or Hidden.Flag Then ' check object invisible
            If Rnd < .5 Then ' random chance
               Display.Item = True ' set display flag
            End If ' end random chance
         End If ' end check invisible object
         If ObjectRecord.Hidden Then ' check object hidden
            If Normal.User Then ' verify non DM
               Display.Item = False ' set display flag
            End If ' end verify normal player
         End If ' end check hidden object
         If Display.Item Then ' check display flag
            Carriage.Return = True ' disable cr/lf
            Call IO.O ' send previous string
            Outpt = RTrim$(ObjectRecord.ObjectName) + ", " ' store object name
            Items.Displayed = Items.Displayed + 1 ' increment item displayed counter
         End If ' end check display flag
      End If ' end check file bounds
   Next ' end loop through room objects
   For Room.Treasure = 1 To 20 ' loop through room treasure
      Treasure.Number = GetRoomRecordTreasure(Room.Treasure) ' get treasure index
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' file bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Display.Item = False ' set display flag
         ' check treasure hidden
         If GetRoomRecordFlags(Room.Treasure) = Hidden.Object Then
            Hidden.Flag = True ' reset flag
         Else ' check treasure
            Hidden.Flag = False ' reset flag
         End If ' end check treasure
         ' check treasure invisible
         If TreasureRecord.Invisible Or Hidden.Flag Then
            If Rnd < .5 Then ' random chance
               Display.Item = True ' set display flag
            End If ' end random chance
         End If ' end check hidden treasure
         If Display.Item Then ' check display flag
            Carriage.Return = True ' disable cr/lf
            Call IO.O ' send previous string
            Outpt = RTrim$(TreasureRecord.TreasureName) + ", " ' store treasure name
            Items.Displayed = Items.Displayed + 1 ' increment displayed counter
         End If ' end check display flag
      End If ' end check file bounds
   Next ' end loop through room treasure
   If Items.Displayed = False Then ' check items displayed counter
      Outpt = "nothing.." ' make message
   Else ' check item counter
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, add period
      If Items.Displayed > 1 Then ' check counter
         Outpt = "and " + Outpt ' append string
      End If ' end check counter
   End If ' end check counter
   Call IO.O ' send message
End Sub ' end routine to search room

Rem * routine processes actions, routine is entered when a trigger is
Rem * activated.
Rem * input variables:
Rem *   Activate.Action$ - the display string which activated the action.
Rem *   Trigger.Action$ - prefix string of what hit player.

Sub Actions (Activate.Action$, Trigger.Action$)
   If Room = 1 Then ' check resurrected room
      Exit Sub ' exit routine
   End If ' end check room
   Inpt = Nul ' reset output string
   Graphics.Off = False ' reset color
   Select Case ActionRecord.Inventory ' selection of room action
      Case 1 ' inventory action 1 breaks all weapons
         Item.Broke = False ' weapon broke flag
         Weapon2 = False ' reset weapon held
         Weapon6 = False ' reset weapon held
         Weapon10 = False ' reset weapon held
         For Inventory.Number = 1 To 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then 'file bounds
               Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
               If TreasureRecord.Plus Then ' check weapon plus
                  If TreasureRecord.Type = False Then ' check weapon
                     ' check weapon charges
                     If GetUserRecordCharges(Inventory.Number) Then
                        ' clear weapon charges
                        Call PutUserRecordCharges(Inventory.Number, False)
                        Item.Broke = True ' set weapon broke flag
                     End If ' end check charges
                  End If ' end check weapon
               End If ' end check weapon plus
            End If ' end check file bounds
         Next ' end loop through player inventory
         If Item.Broke Then ' compare weapon broke flag
            Outpt = Activate.Action$ ' copy routine string
            Call IO.O ' send string
            Outpt = "All your weapons break!" ' make message
            Call IO.O ' send weapon message
         End If ' end compare broke flag
      Case 2 ' inventory action 2 breaks all shields
         Item.Broke = False ' set shield broke flag
         Weapon3 = False ' clear shield held
         Weapon5 = False ' clear shield held
         For Inventory.Number = 1 To 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then 'file bounds
               Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
               If TreasureRecord.Type < False Then ' check shield type
                  If GetUserRecordCharges(Inventory.Number) Then ' check charges
                     ' clear shield charges
                     Call PutUserRecordCharges(Inventory.Number, False)
                     Item.Broke = True ' set shield broke flag
                  End If ' end check charges
               End If ' end check shield
            End If ' end check file bounds
         Next ' end loop through player inventory
         If Item.Broke Then ' compare shield broke flag
            Outpt = Activate.Action$ ' copy routine string
            Call IO.O ' send string
            Outpt = "All your shields break!" ' make message
            Call IO.O ' send shield message
         End If ' end compare broke flag
      Case 3 ' inventory action 3 breaks all armor
         Item.Broke = False ' set armor broke flag
         Weapon1 = False ' clear armor worn
         Weapon4 = False ' clear armor worn
         For Inventory.Number = 1 To 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then 'file bounds
               Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
               If TreasureRecord.Type > False Then ' check armor
                  If GetUserRecordCharges(Inventory.Number) Then ' check charges
                     ' clear armor charges
                     Call PutUserRecordCharges(Inventory.Number, False)
                     Item.Broke = True ' set armor broke flag
                  End If ' end check charges
               End If ' end check armor
            End If ' end check file bounds
         Next ' end loop through player inventory
         If Item.Broke Then ' compare armor broke flag
            Outpt = Activate.Action$ ' copy routine message
            Call IO.O ' send message
            Outpt = "All your armor breaks!" ' make message
            Call IO.O ' send armor message
         End If ' end compare broke flag
      Case 4 ' inventory action 4 breaks all magic items
         Item.Broke = False ' set magic item broke flag
         Weapon7 = False ' clear ring held
         Weapon8 = False ' clear ring held
         For Inventory.Number = 1 To 20 ' loop through all player inventory
            Treasure.Number = GetUserRecordInv(Inventory.Number) ' get treasure index
            If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then 'file bounds
               Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
               If TreasureRecord.Spell Then ' check magic item
                  If GetUserRecordCharges(Inventory.Number) Then ' check charges
                     ' clear magic item charges
                     Call PutUserRecordCharges(Inventory.Number, False)
                     Item.Broke = True ' set magic item broke flag
                  End If ' end check charges
               End If ' end check magic item
            End If ' end check file bounds
         Next ' end loop through player inventory
         If Item.Broke Then ' compare magic item broke flag
            Outpt = Activate.Action$ ' copy routine string
            Call IO.O ' send string
            Outpt = "All your magic items break!" ' make message
            Call IO.O ' send magic item message
         End If ' end compare broke flag
   End Select ' end selection of action
   If ActionRecord.Fumble Then ' verify room fumble action
      Call Fumble ' routine to fumble weapon/shield
   End If ' end verify fumble action
   Teleport.Number = ActionRecord.Teleport
   If Teleport.Number > False And Teleport.Number <> Room Then
      Outpt = Activate.Action$ ' copy routine string
      Call IO.O ' send string
      Outpt = "You are teleported elsewhere!" ' make message
      Call IO.O ' send teleport message
      Next.Room = Teleport.Number ' store new room number
      Teleported = True ' set teleporting flag
      Call Enter.Room ' routine to move player to new room
   End If ' end verify teleport action
   Room.Hits# = CDbl(Int(ActionRecord.HitPoints)) ' store room hit action
   If Room.Hits# > False Then ' verify hit points action
      Outpt = Activate.Action$ ' copy routine string
      Call IO.O ' send string
      Outpt = Trigger.Action$ ' copy second routine string
      Call Hit.Player(Room.Hits#) ' routine to hit player
   End If ' end verify hit points action
End Sub ' end routine to activate room actions

Rem * routine to verify monster blocking exits, then check valid direction.
Rem * input variables:
Rem *   Direction.Number - direction to go.
Rem * output variables:
Rem *   Next.Room - number of new room to enter.

Sub Verify.Room (Direction.Number)
   New.Room = False ' enter new room flag
   If UserRecord.ClassType < Lady Then ' check normal player class
      For Array.Index = 1 To Number.Monsters ' loop through monsters in room
         If MonsterArray(Array.Index).Block Then ' check monster blocks exits
            ' random percent
            If Rnd < (MonsterArray(Array.Index).BlockPercent / 100) Then
               Inpt = MonsterArray(Array.Index).MonsterName ' store monster name
               Inpt = RTrim$(Inpt) ' trim name
               Outpt = "The " + Inpt + " blocks your way!" ' make block message
               Call IO.O ' send block message
               Exit Sub ' exit routine
            End If ' end check random percentage
         End If ' end check monster blocks exits
      Next ' end loop through monsters
   End If ' end check normal user
   Next.Room = GetRoomRecordDirect(Direction.Number) ' get room number of direction
   If Next.Room = False Then ' check next room number
      If Normal.User Then ' check DM status
         Outpt = "You can't go in that direction!" ' make entry message
         Call IO.O ' send entry message
      Else ' check DM status
         Call Add.Room(Direction.Number, Room.Added) ' routine to add new room
         New.Room = Room.Added ' store new room flag
      End If ' end check DM
      Exit Sub ' exit routine
   Else ' check next room number
      ' routine to verify room direction restricted
      Call Restrict(Direction.Number, Restricted)
      If Restricted Then ' check room restricted flag
         Outpt = "Your level does not permit entrance to that room!" ' message
         Call IO.O ' send restricted message
         Exit Sub ' exit routine
      End If ' end check room restricted
      ' routine to verify room entry type restricted
      Call Restrict.Room.Type(Restricted)
      If Restricted Then ' check room restricted flag
         Outpt = "You can't walk to that room!" ' message
         Call IO.O ' send restricted message
         Exit Sub ' exit routine
      End If ' end check room restricted
   End If ' end check next room number
   New.Room = True ' store next room valid flag
End Sub ' end routine to verify room entry, direction

Rem * routine to exit a room with the Out command.

Sub Exit.Room
   Entry.Command = Last.Command.Number ' store room entry command
   User.Command = "O" ' store out command
   Last.Command = "OUT" ' store out command
   Last.Command.Number = True ' reset command type
   Call Verify.Room(5) ' routine to verify valid room
   If New.Room Then ' check valid room flag
      Call Enter.Room ' routine to move player to room
   End If ' end check valid room
End Sub ' end routine to go Out

Rem * routine to use the Up direction.

Sub Climb
   Entry.Command = Last.Command.Number ' store room entry command
   User.Command = "U" ' store up command
   Last.Command = "UP" ' store up command
   Last.Command.Number = True ' store command type
   Call Verify.Room(6) ' routine to check valid room
   If New.Room Then ' check valid room flag
      Call Enter.Room ' routine to move player to room
   End If ' end check valid room
End Sub ' end routine to go Up

Rem * routine processes room traps.

Sub Traps
   New.Room = False ' reset next room flag
   If Rnd < .5 Then ' random chance
      Outpt = "It's trapped! " ' make trap message
      Select Case ObjectRecord.Trap ' selection of room trap type
         Case 1 ' type 1
            Outpt = Outpt + "Poison needles!" ' make trap message
            Call IO.O ' send trap message
            UserRecord.Poison = True ' set player poison flag
         Case 2 ' type 2
            New.Room = ObjectRecord.Teleport ' store teleport room number
            ' file bounds
            If New.Room > False And New.Room <= LOF(RoomFile) / Len(RoomRecord) Then
               If New.Room <> Room Then ' check destination room for recursion
                  Outpt = Outpt + "Falling door!" ' make trap message
                  Call IO.O ' send trap message
                  Pass.Door = False ' clear pass door flag
                  Number.Monsters = False ' clear number of monsters
                  Next.Room = New.Room ' store room trap teleport number
                  Teleported = True ' set teleporting flag
                  Call Enter.Room ' routine to move player to a room
               End If ' end check destination room
            End If ' end check file bounds
         Case 3 ' type 3
            Outpt = Outpt + "Deadly spears!" ' make trap message
            Call IO.O ' send trap message
            Outpt = "You are hit for" ' make hits message
            ' get object trap hits on player
            Room.Hits# = CDbl(Int(ObjectRecord.Teleport))
            If Room.Hits# > False Then ' check hits
               Call Hit.Player(Room.Hits#) ' routine to hit player
            End If ' end check hits
      End Select ' end selection of trap type
   End If ' end check random chance
End Sub ' end routine for room traps

Rem * routine to hide player.

Sub Hide.User
   If Hidden.Player Then ' check player already hidden
      Outpt = "You hide in the shadows!" ' make hide message
      Call IO.O ' send hide message
      Exit Sub ' exit routine
   End If ' end check player hidden
   Hide.Flag = False ' set hide flag
   If Number.Monsters = False Then ' check number of monsters in room
      Hide.Flag = True ' set hide flag
   Else ' check monsters
      If Rnd > .66 Then ' random chance
         Hide.Flag = True ' set hide flag
      End If ' end check random chance
   End If ' end check monsters in room
   If Hide.Flag Then ' check hide flag
      Hidden.Player = True ' set player hide flag
      Outpt = "You hide in the shadows!" ' make hide message
   Else ' check hide flag
      Outpt = "Didn't work!" ' make hide message
   End If ' end check hide flag
   Call IO.O ' send hide message
End Sub ' end routine to hide player

Rem * routine for player to use a vehicle.

Sub Enter.Vehicle
   Call Check.Room.Treasure ' routine finds vehicle name in room
   If Index.Number = False Then ' check room vehicle found
      Outpt = "That's not a vehicle!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check vehicle found
   If TreasureRecord.Vehicle = False Then ' check item is vehicle
      Outpt = "That's not a vehicle!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check vehicle
   Outpts = TreasureRecord.ShortName ' store treasure name
   Outpts = RTrim$(Outpts) ' trim name
   Outpts = LCase$(Outpts) ' lowercase name
   If Charges.Number = False Then ' check vehicle hits
      Outpt = "The " + Outpts + " is damaged!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end chekc vehicle hits
   Vehicle1 = Array.Number ' store vehicle variable
   Vehicle2 = Charges.Number ' store vehicle variable
   Vehicle3 = Index.Number ' store vehicle variable
   Vehicle4 = TreasureRecord.VehicleType ' store vehicle variable
   Outpt = "You enter the " + Outpts + "." ' make vehicle message
   Call IO.O ' send vehicle message
End Sub ' end routine to use vehicle

Rem * routine to move player and vehicle in a direction or through a portal.

Sub Ride.Vehicle
   If Vehicle3 = False Then ' check player using a vehicle
      Outpt = "You're not riding a vehicle!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check player vehicle
   User.Command = Parsed.Command1 ' store direction parameter
   Call Get.Direction(Direction.Number) ' routine verifies direction
   If Direction.Number Then ' compare direction number
      Next.Room = GetRoomRecordDirect(Direction.Number) ' get room direction
      If Next.Room = False Then ' check room direction number
         Outpt = "You can't travel in that direction!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end check room direction number
      Call Vehicle.Type ' routine compares vehicle to room type
      If Next.Room = False Then ' check room type flag
         Outpt = "You can't travel in that direction!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end compare vehicle to room type
      Outpts = Direction(Direction.Number) ' store dirction name
      Outpts = RTrim$(Outpts) ' trim name
      Outpts = LCase$(Outpts) ' lowercase name
      Outpt = "You ride " + Outpts + "!" ' make vehicle message
      Call IO.O ' send vehicle message
      Call Enter.Room ' routine moves player to room
      Exit Sub ' exit routine
   End If ' end compare direction number
   Call Check.Room.Objects ' routine searches room for portal name
   If Index.Number = False Then ' check room portal number
      Outpt = "You can't travel in that direction!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check room portal
   If ObjectRecord.RoomLink = False Then ' check portal goes to room
      Outpt = "You can't travel there!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check portal to room number
   If ObjectRecord.JailTrap Then ' check room portal is a jail trap
      Outpt = "Trapped portal!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check room portal type
   If ObjectRecord.Closed Then ' check roomportal is locked
      If Pass.Door = False Then ' check pass door spell in effect
         Outpt = "You can't, it's closed!" ' make entry error message
         Call IO.O ' send entry error message
         Exit Sub ' exit routine
      End If ' end check pass door spell
   End If ' end check room portal locked
   If ObjectRecord.Relocks Then ' check room portal relocks
      ObjectRecord.DoorLock = 2 ' reset room portal lock
      ObjectRecord.Closed = True ' reset room portal lock
      Call Share.Record(ObjectFile, Index.Number) ' write object record
   End If ' end check room portal relock
   Outpt = ObjectRecord.ShortDesc ' store room entry display description
   If Outpt <> String$(80, 0) Then ' check description to nulls
      Outpt = RTrim$(Outpt) ' trim description
      If Outpt <> Nul Then ' compare length of description
         Call IO.O ' send room entry description message
      End If ' end compare description length
   End If ' end check description
   Pass.Door = False ' reset pass door spell
   Number.Monsters = False ' reset number of monsters in room
   ' store room number of object portal destination
   Next.Room = ObjectRecord.RoomLink ' store
   Call Vehicle.Type ' routine to verify vehicle to room type
   If Next.Room = False Then ' check verified room number
      Outpt = "You can't travel in that direction!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check vehicle to room type
   Outpts = ObjectRecord.ShortName ' store object name
   Outpts = RTrim$(Outpts) ' trim name
   Outpts = LCase$(Outpts) ' lowercase name
   Outpt = "You ride to the " + Outpts + "!" ' make vehicle message
   Call IO.O ' send message
   Call Enter.Room ' routine moves player to room
End Sub ' end routine to move player and vehicle

Rem * routine remove player from vehicle.

Sub Exit.Vehicle
   Call Check.Room.Treasure ' routine finds vehicle name in room
   If Array.Number = Vehicle1 Then ' compare treasure number to vehicle number
      Outpts = TreasureRecord.ShortName ' store treasure name
      Outpts = RTrim$(Outpts) ' trim name
      Outpts = LCase$(Outpts) ' lowercase name
      Vehicle1 = False ' reset vehicle variable
      Vehicle2 = False ' reset vehicle variable
      Vehicle3 = False ' reset vehicle variable
      Outpt = "You exit the " + Outpts + "!" ' make vehicle message
   Else ' compare vehicle numbers
      Outpt = "You can't exit that!" ' make error message
   End If ' end compare vehicle
   Call IO.O ' send message
End Sub ' end routine to exit vehicle

Rem * routine verifies vehicle type can enter room Type
Rem * output variables:
Rem *   Next.Room - false for invalid vehicle to room Type

Sub Vehicle.Type
   If Next.Room <= False Or Next.Room > LOF(RoomFile) / Len(RoomRecord) Then ' file bounds
      Next.Room = False ' return next room number
      Exit Sub ' exit routine
   End If ' end check file bounds
   Call Read.Room.Record(Next.Room) ' get destination room
   If Vehicle4 <> 3 Then ' check all terrain vehicle
      Action.Number = RoomRecord.Action ' store room action number
      If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then
         Call Read.Record(ActionFile, Action.Number) ' read action record
         If ActionRecord.Attribute2 <> Vehicle4 Then ' check vehicle terrain type
            Next.Room = False ' reset next room number
         End If ' end check vehicle terrain type
      End If ' end check action number range
   End If ' end check all terrain vehicle
   Call Read.Room.Record(Room) ' restore room record
End Sub ' end routine to compare vehicle to room type

Rem * routine determines if player can train for next level.

Sub Train
   If UserRecord.Level <= False Then ' verify player level
      Call Train.Stats ' train player
      Exit Sub ' exit routine
   End If ' end verify player level
   ' calculate experience needed to reach next level
   Call Experience(Exp.Required#)
   If UserRecord.Experience < Exp.Required# Then ' compare player experience
      Outpt = "You don't have enough experience to train!" ' train error message
      Call IO.O ' send train error message
      Exit Sub ' exit routine
   End If ' end compare player experience
   Call Gold(Gold.Required#) ' routine calculates gold needed for level
   If UserRecord.Gold < Gold.Required# Then ' compare to player gold
      Outpt = "You don't have enough Gold to train!" ' make train error message
      Call IO.O ' send train error message
      Exit Sub ' exit routine
   End If ' end compare gold
   Call Is.Training.Room(In.Room) ' check training room
   If In.Room = False Then ' check flag
      Outpt = "You are not in the training room for your class!" ' message
      Call IO.O ' output message
   Else ' check room number
      Call Train.Stats ' routine to train for next level
   End If ' end check room number
End Sub ' end train routine

Rem * routine determines player is in correct training room.
Rem * output variables:
Rem *   In.Room - true for in room

Sub Is.Training.Room (In.Room)
   In.Room = False ' reset flag
   Player.Class = UserRecord.ClassType ' store class number
   ' check for no specific training room
   If GetUserRecordTrainRoom(Player.Class) = SFalse Then
      In.Room = True ' reset flag
      Exit Sub ' exit routine
   End If ' end check room number
   ' compare room numbers
   If GetUserRecordTrainRoom(Player.Class) = Room Then
      In.Room = True ' reset flag
   End If ' end compare rooms
End Sub

Rem * routine teleports player to room.
Rem * input variables:
Rem *   Parsed.Command1 - parameter containing room number.

Sub Teleport.User
   Next.Room = Int(Val(Parsed.Command1)) ' convert parameter to room number
   Graphics.Off = True ' reset color
   Outpt = "A Dark Cloud Passes Overhead..." ' make ghod message
   Call IO.O ' send ghod message
   Outpt = "  A Bolt Of Lightning Strikes..." ' make ghod message
   Call IO.O ' send ghod message
   Outpt = "The Cloud Disappears..." ' make ghod message
   Call IO.O ' send ghod message
   Graphics.Off = False ' reset color
   Teleported = True ' set teleporting flag
   Call Enter.Room ' routine moves player to room
End Sub ' end routine to teleport player

Rem * routine moves player to room number in an object.
Rem * input variables:
Rem *   Parsed.Command1 - contains the object name.

Sub Enter.Object
   User.Command = Parsed.Command1 ' store command parameter
   Call Get.Direction(Direction.Number) ' compare name to direction to go to
   If Direction.Number Then ' check direction flag
      Entry.Command = Last.Command.Number ' store room entry command
      Call Verify.Room(Direction.Number) ' routine verifies room number
      If New.Room Then ' check room flag
         Call Enter.Room ' routine moves player
      End If ' end check room flag
      Exit Sub ' exit routine
   End If ' end check direction
   Call Check.Room.Treasure ' compare treasure name
   If Index.Number Then ' check treasure flag
      If TreasureRecord.Vehicle Then ' object to move to is vehicle
         Call Enter.Vehicle ' routine to enter vehicle
         Exit Sub ' exit routine
      End If ' end check object name
   End If ' end check treasure flag
   Call Check.Room.Objects ' compare room object
   If Index.Number = False Then ' check object flag
      Call Num ' count next object
      Call Check.Inventory.Objects ' compare user object
      If Index.Number = False Then ' check object flag
         Outpt = "You can't go there!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end check user object
   End If ' end check room object
   If ObjectRecord.RoomLink = False Then ' check object portal room number
      Outpt = "You can't go there!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check portal room number
   Call Restrict(12, Restricted) ' routine checks enter command restricted
   If Restricted Then ' compare restrict flag
      Outpt = "Your level does not permit entrance to that room!" ' message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare restricted room
   If ObjectRecord.JailTrap Then ' check object is jailed
      Outpt = "Trapped portal!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check jailed object
   If ObjectRecord.Closed Then ' check object lock
      If Pass.Door = False Then ' check pass door spell in effect
         Outpt = "You can't, it's closed!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end check pass door spell
   End If ' end check object lock
   If ObjectRecord.Relocks Then ' check object relocks after entry
      ObjectRecord.DoorLock = 2 ' relock object
      ObjectRecord.Closed = True ' relock object
      Call Share.Record(ObjectFile, Index.Number) ' write object record
   End If ' end check relocking object
   Outpt = ObjectRecord.ShortDesc ' store entry description
   If Outpt <> String$(80, 0) Then ' compare description to nulls
      Outpt = RTrim$(Outpt) ' trim description
      If Outpt <> Nul Then ' compare length of description
         Call IO.O ' send entry description message
      End If ' end compare description length
   End If ' end compare description
   If ObjectRecord.Trap Then ' verify object has trap
      Call Traps ' routine to activate object trap
      If New.Room Then ' check teleporting trap
         Exit Sub ' exit routine
      End If ' end check teleporting trap
   End If ' end verify object trap
   Pass.Door = False ' reset pass door spell
   Number.Monsters = False ' set monsters in room to zero
   Next.Room = ObjectRecord.RoomLink ' store object portal room number
   Call Enter.Room ' routine to move player to new room
End Sub ' end routine to move player through an object to room

Rem * routine moves player to new room number.
Rem * input variables:
Rem *   Next.Room - contains new room number to move player to.

Sub Enter.Room
   New.Room = True ' set room entry flag
   Call Clean.Room ' routine to remove treasure from old room
   If Next.Room > False And Next.Room <= LOF(RoomFile) / Len(RoomRecord) Then ' file bounds
      Swap Room, Next.Room ' store new room, saving old room number
   End If ' end check room file bounds
   Call Status.Line(False) ' routine updates status line
   Monster.Rate1 = False ' reset room monster encounter rate
   Call Read.Room.Record(Room) ' get the new room record
   If Vehicle1 > False Then ' verify vehicle variable used
      Move.Vehicle = False ' move vehicle flag
      Entry.Command = Last.Command.Number ' store command number to enter room
      ' vehicle entered room
      If Entry.Command = RideVehicle Or Entry.Command = DriveVehicle Then
         For Treasure.Number = 1 To 20 ' loop through room treasure
            ' check empty treasure
            If GetRoomRecordTreasure(Treasure.Number) = False Then
               ' store vehicle inventory
               Call PutRoomRecordTreCharges(Treasure.Number, Vehicle2)
               ' store vehicle inventory
               Call PutRoomRecordTreasure(Treasure.Number, Vehicle3)
               Call Share.Room.Record(Room) ' write new room record
               Call Read.Room.Record(Next.Room) ' get previous room
               Call PutRoomRecordTreasure(Vehicle1, False) ' reset vehicle inventory
               Call PutRoomRecordTreCharges(Vehicle1, False) ' reset vehicle inventory
               Call Share.Room.Record(Next.Room) ' write room record
               Call Read.Room.Record(Room) ' get current room record
               Move.Vehicle = True ' set move vehicle flag
               Vehicle1 = Treasure.Number ' store new vehicle treasure number
               Exit For ' exit loop through treasure in room
            End If ' end check for empty treasure in room
         Next ' end loop through room treasure inventory
      End If ' end check command used to enter room
      If Move.Vehicle = False Then ' verify vehicle moves to new room
         Call Read.Record(TreasureFile, Vehicle3) ' get treasure record
         Outpts = TreasureRecord.ShortName ' store treasure name
         Outpts = RTrim$(Outpts) ' trim name
         Outpts = LCase$(Outpts) ' lowercase name
         Outpt = "You exit the " + Outpts + "." ' make vehicle message
         Call IO.O ' send vehicle message
         Vehicle1 = False ' reset vehicle varible
         Vehicle2 = False ' reset vehicle varible
         Vehicle3 = False ' reset vehicle varible
         Vehicle4 = False ' reset vehicle varible
      End If ' end verify vehicle moves
   End If ' end verify vehicle used
   For Array.Index = 1 To Number.Monsters ' loop through room monsters
      ' compare permanent monster
      If MonsterArray(Array.Index).Permanent = True Then
         ' store permanent monster file index
         Monster.Number = MonsterIndex(Array.Index)
         Call Read.Record(MonsterFile, Monster.Number) ' get monster file record
         ' store permanent level
         MonsterRecord.Level = MonsterArray(Array.Index).Level
         MonsterRecord.Hits = MonsterArray(Array.Index).Hits ' stor permanent hits
         Call Share.Record(MonsterFile, Monster.Number) ' write monster record
      End If ' end compare permanent monster
   Next ' end loop through room monsters
   If Room = 1 Then ' check safe room
      Monster.Follow = False ' monsters followed flag
   Else ' end check safe room
      If UserRecord.ClassType < Lady Then ' check class number
         Monster.Follow = True ' monsters followed flag
      Else ' check class number
         Monster.Follow = False ' monsters followed flag
      End If ' end check class number
   End If ' end check safe room
   Teleported.Flag = Teleported ' store teleporting flag
   Teleported = False ' reset teleporting flag
   If Monster.Follow Then ' compare number of monsters followed
      If Teleported.Flag Then ' compare teleporting flag
         Outpts = " teleports with you!" ' make follow message
      Else ' check teleporting flag
         Outpts = " follows you!" ' make follow message
      End If ' end check teleporting flag
      Monsters.Followed = False ' number of monsters which followed counter
      For Array.Index = 1 To Number.Monsters ' loop through all monsters in room
         Monster.Followed = False ' set followed flag
         If MonsterArray(Array.Index).Follow Then ' check monster follows
            If Teleported.Flag Then ' check player teleported
               ' random chance monster teleports with player
               If Rnd < (MonsterArray(Array.Index).Teleport / 100) Then
                  Monster.Followed = True ' set followed flag
               End If ' end random chance
            Else ' check player teleported
               ' random chance monster follows player
               If Rnd < (MonsterArray(Array.Index).FollowPercent / 100) Then
                  Monster.Followed = True ' set followed flag
               End If ' end random chance
            End If ' end check player teleported
            If Monster.Followed Then ' verify followed flag
               ' permanent monster
               If MonsterArray(Array.Index).Permanent = False Then
                  ' increment number of monsters
                  Monsters.Followed = Monsters.Followed + 1
                  ' store monster
                  MonsterArray(Monsters.Followed) = MonsterArray(Array.Index)
                  ' store monster
                  MonsterIndex(Monsters.Followed) = MonsterIndex(Array.Index)
                  ' store monster name
                  Inpt = MonsterArray(Monsters.Followed).MonsterName
                  Inpt = RTrim$(Inpt) ' trim name
                  Inpt = LCase$(Inpt) ' lowercase name
                  Outpt = "The " + Inpt + Outpts ' make followed message
                  Call IO.O ' send message
               End If ' end check permanent monster
            End If ' end verify followed flag
         End If ' end check monster follows
      Next ' end loop through monsters in room
   End If ' end compare number of monsters following
   Monster.Rate1 = False ' reset room monster encounter rate
   Number.Monsters = Monsters.Followed ' store number of monsters followed
   Room.Rate = False ' reset room rate counter
   Rust.Rate = False ' reset room rate counter
   Steal.Rate = False ' reset room rate counter
   Teleported = False ' reset teleporting flag
   Call Check.Next.Room ' routine to get new room record
   Call Encounter.Permanent ' routine to get permanent monsters
   Call Display.Room ' routine displays room description
   Action.Number = RoomRecord.Action ' store room action number
   ' check file bounds
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then
      Call Read.Record(ActionFile, Action.Number) ' read action record
      If ActionRecord.SpellTrigger = False Then ' check room spell action
         If ActionRecord.MonsterTrigger = False Then ' check room monster action
            ' check room monster talk action
            If ActionRecord.MonsterTalk = False Then
               Action1$ = "As you enter the room," ' make action message
               Action2$ = "You are hit for" ' make action message
               Call Actions(Action1$, Action2$) ' routine for room actions
            End If ' end check room action
         End If ' end check room action
      End If ' end check room action
   End If ' end check file bounds
End Sub ' end routine to move player to new room

Rem * routine toggles invisible mode.

Sub Toggle.Invisible
   UserRecord.Invisible = Not UserRecord.Invisible ' negate player invisible mode
   If UserRecord.Invisible Then ' check invisible
      Outpt = "You are invisible!" ' make message
   Else ' check invisible
      Outpt = "You are no longer invisible!" ' make message
   End If ' end check invisible
   Call IO.O ' send message
End Sub ' end routine to toggle invisible mode

Rem * routine toggles linefeed mode.

Sub Toggle.Linefeeds
   User.LineFeeds = Not User.LineFeeds ' negate player linefeed mode
   If User.LineFeeds Then ' check linefeeds
      Outpt = "Linefeeds toggled off." ' make message
   Else ' check linefeeds
      Outpt = "Linefeeds toggled on." ' make message
   End If ' end check linefeeds
   Call IO.O ' send message
End Sub ' end routine to toggle linefeed mode

Rem * routine toggles echo mode.

Sub Toggle.Echo
   User.Echo = Not User.Echo ' negate player echo mode
   If User.Echo Then ' check echo
      Outpt = "Echo toggled off." ' make message
   Else ' check echo
      Outpt = "Echo toggled on." ' make message
   End If ' end check echo
   Call IO.O ' send message
End Sub ' end routine to toggle echo mode

Rem * routine toggles word wrap mode.

Sub Toggle.Wordwrap
   User.Wordwrap = Not User.Wordwrap ' negate player word wrap mode
   If User.Wordwrap Then ' check word wrap
      Outpt = "Word wrap toggled off." ' make message
   Else ' check word wrap
      Outpt = "Word wrap toggled on." ' make message
   End If ' end check word wrap
   Call IO.O ' send message
End Sub ' end routine to toggle word wrap mode

Rem * routine to toggle player Ansi mode.

Sub Toggle.ANSI
   Color.Graphics = Not Color.Graphics ' negate ansi color flag
   If Color.Graphics Then ' check ansi toggle
      Outpt = "ANSI codes enabled." ' make message
   Else ' check ansi
      Outpt = "ANSI codes disabled." ' make message
   End If ' end check ansi toggle
   Call IO.O ' send message
End Sub ' end routine to toggle ansi

Rem * routine to toggle brief mode.

Sub Brief.Mode
   If Normal.User Then ' verify non DM
      If UserRecord.Level <= 1 Then ' check player level
         Outpt = "Brief mode not allowed until level two!" ' make error message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check player level
   End If ' end check normal player
   UserRecord.Brief = Not UserRecord.Brief ' negate brief mode
   If UserRecord.Brief Then ' check brief mode
      Action.Prompt = "Next?" ' store new command prompt
      Outpt = "Brief mode on." ' make message
   Else ' check brief mode
      Action.Prompt = "Command? " ' store new command prompt
      Outpt = "Brief mode off." ' make message
   End If ' end check brief mode
   Call IO.O ' send message
End Sub ' end routine to toggle brief mode

Rem * routine to change pagelength.

Sub Change.Pagelength
   Outpt = "Enter pagelength(1-50)? " ' make input prompt
   No.Input.Out = "24" ' default input
   Call IO.I ' get user input
   Page.Length = Int(Val(Inpt)) ' convert input to integer
   If Page.Length >= 1 And Page.Length <= 50 Then ' check pagelength bounds
      User.PageLength = Page.Length ' store new pagelength
      Outpt = "Pagelength now" + Str$(Page.Length) + " lines." ' make output message
   Else ' check bounds
      Outpt = "Pagelength not changed." ' make output message
   End If ' end check bounds
   Call IO.O ' send output
End Sub ' end routine to change pagelength

Rem * routine to change linelength.

Sub Change.Linelength
   Outpt = "Enter linelength(1-132)? " ' make input prompt
   No.Input.Out = "80" ' default input
   Call IO.I ' get user input
   Line.Length = Int(Val(Inpt)) ' convert input to integer
   If Line.Length >= 1 And Line.Length <= 132 Then ' check linelength bounds
      User.LineLength = Line.Length ' store new linelength
      Outpt = "Linelength now" + Str$(Line.Length) + " characters." ' make output message
   Else ' check bounds
      Outpt = "Linelength not changed." ' make output message
   End If ' end check bounds
   Call IO.O ' send output
End Sub ' end routine to change linelength

Rem * routine to display preferences.

Sub Display.Prefs
   Graphics.Off = True ' reset color
   If User.LineFeeds Then ' check linefeeds
      Outpt = "Linefeeds toggled off." ' make message
   Else ' check linefeeds
      Outpt = "Linefeeds toggled on." ' make message
   End If ' end check linefeeds
   Call IO.O ' send message
   If User.Echo Then ' check echo
      Outpt = "Echo toggled off." ' make message
   Else ' check echo
      Outpt = "Echo toggled on." ' make message
   End If ' end check echo
   Call IO.O ' send message
   If User.Wordwrap Then ' check word wrap
      Outpt = "Word wrap toggled off." ' make message
   Else ' check word wrap
      Outpt = "Word wrap toggled on." ' make message
   End If ' end check word wrap
   Call IO.O ' send message
   If Color.Graphics Then ' check ansi toggle
      Outpt = "ANSI codes enabled." ' make message
   Else ' check ansi
      Outpt = "ANSI codes disabled." ' make message
   End If ' end check ansi toggle
   Call IO.O ' send message
   If UserRecord.Brief Then ' check brief mode
      Outpt = "Brief mode on." ' make message
   Else ' check brief mode
      Outpt = "Brief mode off." ' make message
   End If ' end check brief mode
   Call IO.O ' send message
   Outpt = "Pagelength now" + Str$(User.PageLength) + " lines." ' make message
   Call IO.O ' send output
   Outpt = "Linelength now" + Str$(User.LineLength) + " characters." ' make message
   Call IO.O ' send output
   Select Case UserRecord.Sort ' check sorting preference
      Case -1 ' check value
         Outpt = "Inventory charges sorting on." ' make display message
      Case 0 ' check value
         Outpt = "Inventory sorting off." ' make display message
      Case 1 ' check value
         Outpt = "Inventory plus sorting on." ' make display message
      Case Else ' check sorting
         Outpt = "Inventory sorting off." ' make display message
   End Select ' end check sorting
   Call IO.O ' send display message
   ' check player locked mailbox
   If UserRecord.Flags And Locked.User Then
      Outpt = "Mailbox locked." ' make locked message
   Else ' check player locked
      Outpt = "Mailbox unlocked." ' make locked message
   End If ' end check player locked flag
   Call IO.O ' send message
   Graphics.Off = False ' reset color
End Sub ' end display preferences routine

Rem * routine to have the blacksmith repair some item of treasure.

Sub Weapons.Shop
   Call Check.Inventory.Treasure ' routine to find treasure name
   If Index.Number = False Then ' check treasure index
      Outpt = "The Blacksmith says: You can't repair that!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check treasure index
   If TreasureRecord.Spell Then ' compare treasure is magic
      Outpt = "The Blacksmith says: Can't fix that here!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare magical treasure
   If TreasureRecord.Type = False Then ' check treasure is weapon, shield, armor
      If TreasureRecord.Plus = False Then ' check weapon plus
         Outpt = "The Blacksmith says: Can't fix that here!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end check weapon plus
   End If ' end check treasuer type
   If GetUserRecordCharges(Array.Number) <> False Then ' compare treasure charges
      Outpt = "The Blacksmith says: That isn't broken!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare treasure charges
   Item.Cost# = Int(TreasureRecord.Gold * .5) ' calculate price to repair item
   If Item.Cost# > UserRecord.Gold Then ' compare price to player gold
      Outpt = "The Blacksmith says: You don't have enough gold!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare price to gold
   ' make input prompt
   Outpt = "The Blacksmith asks: How about" + Str$(Item.Cost#) + " gold(y/n)? "
   No.Input.Out = "Y" ' default input
   Call IO.I ' get input
   If Yes Then ' compare response
      UserRecord.Gold = UserRecord.Gold - Item.Cost# ' decrement player gold
      ' store repaired charges
      Call PutUserRecordCharges(Array.Number, TreasureRecord.Charges)
      Outpt = "The Blacksmith says: There, it's repaired!" ' make message
   Else ' compare yresponse
      Outpt = "The Blacksmith says: It ain't repaired!" ' make message
   End If ' end compare response
   Call IO.O ' send message
End Sub ' end routine to repair item

Rem * routine to repair an item of magical treasure.

Sub Alchemist
   Call Check.Inventory.Treasure ' routine to find treasure name
   If Index.Number = False Then ' check treasure index found
      Outpt = "The Alchemist says: That can't be recharged here!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check treasure index
   Spell.Number = TreasureRecord.Spell ' store treasure spell number
   ' file bounds
   If Spell.Number <= False Or Spell.Number > LOF(SpellFile) / Len(SpellRecord) Then
      Outpt = "The Alchemist says: You can't recharge that here!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check file bounds
   Call Read.Record(SpellFile, Spell.Number) ' get spell record of magic item
   If SpellRecord.SpellType = Wish Then ' compare to wish item
      Outpt = "The Alchemist says: I won't recharge that item!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare wish item
   If GetUserRecordCharges(Array.Number) <> False Then ' check item charges
      Outpt = "The Alchemist says: That's not discharged!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check charges remaining
   Item.Cost# = Int(TreasureRecord.Gold * .5) ' calculate price to repair
   If Item.Cost# > UserRecord.Gold Then ' compare price to player gold
      Outpt = "The Alchemist says: You don't have enough gold!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare price to gold
   ' make input prompt
   Outpt = "The Alchemist says: How about" + Str$(Item.Cost#) + " gold(y/n)? "
   No.Input.Out = "Y" ' default input
   Call IO.I ' get input
   If Yes Then ' compare response
      UserRecord.Gold = UserRecord.Gold - Item.Cost# ' decrement player gold
      ' store item charges
      Call PutUserRecordCharges(Array.Number, TreasureRecord.Charges)
      Outpt = "The Alchemist chants an invocation!" ' make message
   Else ' compare response
      Outpt = "The Alchemist says: Didn't repair it!" ' make message
   End If ' end compare response
   Call IO.O ' send message
End Sub ' end routine to repair magic item

Rem * routine to sell treasure item from list.
Rem * input variables:
Rem *   Parsed.Command1 - number of item to purchase.

Sub Weapons.Shoppe
   Treasure.Number = Int(Val(Parsed.Command1)) ' convert parameter to integer
   If Treasure.Number <= False Or Treasure.Number > 20 Then ' compare integer bounds
      Outpt = "The Blacksmith says: You can't buy that!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare range
   If Treasure.Number > LOF(TreasureFile) / Len(TreasureRecord) Then ' check bounds
      Outpt = "The Blacksmith says: You can't buy that!" ' make error message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare file bounds
   Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
   If UserRecord.Gold - TreasureRecord.Gold < False Then ' compute price
      Outpt = "The Blacksmith says: You don't have enough gold!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare price to player gold
   If Weight + TreasureRecord.Weight > GetUserRecordStats(1) * 10 Then ' compute weight
      Outpt = "The Blacksmith says: You can't carry any more!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end compare weight of new item
   Call TreasureCharges(Charges.Amount) ' routine to get treasure charges
   ' routine to add item to player inventory
   Call Add.Inventory(Treasure.Number, Charges.Amount, Item.Added)
   If Item.Added Then ' check return variable for added inventory
      UserRecord.Gold = UserRecord.Gold - TreasureRecord.Gold ' decrement gold
      Outpt = "The Blacksmith says: There, sold!" ' make message
   Else ' check inventory added
      Outpt = "The Blacksmith says: You can't carry any more!" ' make message
   End If ' end check inventory added flag
   Call IO.O ' send output message
End Sub ' end routine to purchase item for sale

Rem * routine displays a sorted list of the top ten players, writes the
Rem * top ten ranking bulletin file.

Sub Top.Ten
   Max.Users = LOF(UserFile) / Len(UserRecord) ' store length of user file
   ReDim Temp.Array1(1 To Max.Users) As Integer
   ReDim Temp.ArrayZ(1 To Max.Users) As Double ' dimension working arrays
   Close #TempFile ' close work file
   Open "ranklist.dat" For Output As #TempFile ' open to work file
   Outpt = "The Adventure Door v" + Version$ + " Top Ten Player Rankings For " + clock$ + "."
   Print #TempFile, Outpt ' write to file
   Outpt = Nul ' make empty string
   Print #TempFile, Outpt ' write to file
   Player.Count = False ' reset player counter
   For User.Number = 1 To Max.Users ' loop through user file
      Call Read.Record(UserFile2, User.Number) ' get next user file record
      Outpt = UserRecord2.CodeName ' store player codename
      Call Decrypt(Outpt) ' decrypt codename
      If Left$(Outpt, 9) <> Deleted$ Then ' compare deleted user record
         If UserRecord2.Level > False Then ' check user level
            If (UserRecord2.Flags And Locked.User) = False Then ' check locked user
               Score# = UserRecord2.MonstersKilled * UserRecord2.Level ' compute score
               If Score# > False Then ' compare score
                  ' increment high score player counter
                  Player.Count = Player.Count + 1
                  Temp.Array1(Player.Count) = User.Number ' store record index
                  Temp.ArrayZ(Player.Count) = Int(Score#) ' store score
               End If ' end compare score
            End If ' end check locked user record
         End If ' end check user level
      End If ' end compare deleted user
   Next ' end loop through user file
   ' bubble sort
   For Sort1 = 1 To Player.Count ' loop through all items to sort
      For Sort2 = Sort1 + 1 To Player.Count ' loop through remaining items
         If Temp.ArrayZ(Sort1) < Temp.ArrayZ(Sort2) Then ' compare scores
            Swap Temp.Array1(Sort1), Temp.Array1(Sort2) ' swap lower array
            Swap Temp.ArrayZ(Sort1), Temp.ArrayZ(Sort2) ' swap lower array
         End If ' end compare scores
      Next ' end loop through array
   Next ' end loop through array
   If Player.Count > 10 Then ' check maximum number of users
      Player.Count = 10 ' reset to top ten
   End If ' end check maximum scoring players
   ' make header
   Outpt = "Username                       Level Classname            Ranking"
   Print #TempFile, Outpt ' write to file
   Outpt = String$(65, "-") ' make header line
   Print #TempFile, Outpt ' write to file
   For Array.Number = 1 To Player.Count ' loop through high scoring players
      User.Number = Temp.Array1(Array.Number) ' get user file record number
      Call Read.Record(UserFile2, User.Number) ' get user file record
      Outpt = UserRecord2.CodeName ' store player codename
      Call Decrypt(Outpt) ' decrypt codename
      Outpt = LCase$(Outpt) ' lowercase codename
      Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first word
      Outpt = Outpt + Str$(UserRecord2.Level) ' add player level
      Outpt = Outpt + Space$(7 - Len(Str$(UserRecord2.Level))) ' pad blanks
      Inpt = UserRecord2.ClassName ' store player class name
      Call Decrypt(Inpt) ' decrypt class name
      Outpt = Outpt + Inpt ' append class name
      Outpt = Outpt + Str$(Temp.ArrayZ(Array.Number)) ' add score
      Print #TempFile, Outpt ' write to file
   Next ' end loop through top ten players
   If Player.Count = False Then ' compare number of players
      Outpt = "No users have top scores." ' make score message
      Print #TempFile, Outpt ' write to file
   End If ' end compare number of players
   Close #TempFile ' close work file
   User.Line.Length = User.LineLength
   User.LineLength = 80
   Call Out.File("ranklist.dat")
   User.LineLength = User.Line.Length
   ReDim Temp.Array1(1) As Integer
   ReDim Temp.ArrayZ(1) As Double ' remove temporary arrays
End Sub ' end routine to display and write top ten list

Rem * routine deletes a user record.

Sub Delete.User
   Outpt = Deleted$ ' store deleted string
   Call Valid(Outpt, 30) ' validate string
   Call Encrypt(Outpt, True) ' encrypt string
   UserRecord.CodeName = Outpt ' store string in codename
   Outpt = Deleted$ ' store deleted string
   Call Valid(Outpt, 20) ' validate string
   Call Encrypt(Outpt, False) ' encrypt string
   UserRecord.PassWord = Outpt ' store string in password
   Outpt = Deleted$ ' store deleted string
   Call Valid(Outpt, 20) ' validate string
   Call Encrypt(Outpt, True) ' encrypt string
   UserRecord.ClassName = Outpt ' store string in class name
   UserRecord.ClassType = False ' reset class number
   UserRecord.Flags = False ' reset user flags
   UserRecord.FromHour = False ' reset time restriction
   UserRecord.FromMin = False ' reset time restriction
   UserRecord.Level = False ' reset player level
   UserRecord.MaxCalls = False ' reset maximum calls allowed
   UserRecord.MonstersKilled = False ' reset score counter
   UserRecord.ToHour = False ' reset time restriction
   UserRecord.ToMin = False ' reset time restriction
   For Array.Index = 1 To 20 ' loop through inventory
      Call PutUserRecordInv(Array.Index, False) ' reset inventory
      Call PutUserRecordCharges(Array.Index, False) ' reset inventory
   Next ' end loop through inventory
   For Array.Index = 1 To 20 ' loop through object inventory
      Call PutUserRecordObject(Array.Index, False) ' reset inventory
      Call PutUserRecordTrainRoom(Array.Index, False) ' reset empty array
   Next ' end loop through inventory
   Call Clear.Container(0, True) ' clear container record
   For Array.Index = 1 To 3 ' loop through all containers
      Call PutUserRecordContainer(Array.Index) ' store container record
   Next ' end loop through containers
End Sub ' end routine to delete a user record

Rem * Filename: dnds5.bas Version: v5.0a r2.0a
Rem * This subprogram contains all container, bank, and some display
Rem * and main routines.

Rem * routine to search player and room containers for parameter name
Rem * input variables:
Rem *   Parsed.Command1 - name of container.
Rem * output variables:
Rem *   Index.Number - true if container name found.
Rem *   TypeNumber - 0 for container in inventory, 1 for container in room.

Sub Examine.Container
   TypeNumber = False ' store container flag
   Call Check.Inventory.Container ' search player inventory containers
   If Index.Number = False Then ' check player container found
      Call Num ' decrement counters
      TypeNumber = 1 ' store container flag
      Call Check.Room.Container ' search room inventory containers
   End If ' end check player container found
End Sub ' end routine to search inventory for container name

Rem * routine to remove an item from player inventory and add to a container.

Sub Drop.Into.Container
   Call Parse ' get first parameter
   Call Numeric ' parse parameter number from # sign
   Call Check.Inventory.Treasure ' search for inventory to drop
   If Index.Number = False Then ' index to treasure file not found
      Outpt = "You can't drop that!" ' message
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check treasure index
   Item.Number = Array.Number ' store treasure index to array
   Item.Charges = Charges.Number ' store treasure charges
   Item.Index = Index.Number ' store treasure index to file
   Parsed.Command1 = Parsed.Command2 ' get second parameter
   Call Numeric ' parse parameter number from # sign
   Call Examine.Container ' check player inventory and room for container
   If Index.Number = False Then ' index to treasure file not found
      Outpt = "You can't drop that!" ' message
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check index
   If ContainerRec.Locked > False Then ' check container locked
      Outpt = "You can't, it's locked!" ' message
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check container
   If ContainerRec.Closed > False Then ' check container closed
      Outpt = "You can't, it's closed!" ' message
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check container
   Container.Number = Array.Number ' store index of inventory array of container
   For Array.Counter = 1 To 5 ' loop through container elements
      ' find empty container element
      If GetContainerRecInventory(Array.Counter) = False Then
         Container.Name$ = ContainerRec.ContainerName ' store container name
         Container.Name$ = RTrim$(Container.Name$) ' strip trailing blanks
         If Last.Command.Number = Hide.Command Then ' last command was hide
            ' set container record hide flag
            Call PutContainerRecInvisible(Array.Counter, True)
         Else ' not hide
            ' set container record hide
            Call PutContainerRecInvisible(Array.Counter, False)
         End If ' end check last command
         ' store container record index
         Call PutContainerRecInventory(Array.Counter, Item.Index)
         ' store container record charges
         Call PutContainerRecCharges(Array.Counter, Item.Charges)
         Select Case TypeNumber ' select room or inventory
            Case 1 ' drop item into container in room
               RoomRecord.Container = ContainerRec ' set room container record
               Call Share.Room.Record(Room) ' put room record
            Case 0 ' dropped item in container in player inventory
               ' update player container
               Call PutUserRecordContainer(Container.Number)
         End Select ' end select container drop
         Outpts = TreasureRecord.TreasureName ' store name of treasure dropped
         Outpts = RTrim$(Outpts) ' strip trailing blanks
         If Last.Command.Number = Hide.Command Then ' last command hide
            Inpt = " hide " ' set equal to hide
         Else ' not hide
            Inpt = " drop " ' set equal to drop
         End If ' end check last command
         Outpt = "You" + Inpt + Outpts + " in " + Container.Name$ + "!" ' format message
         Call IO.O ' send output
         Call Discard.Inventory(Item.Number, True) ' remove item from inventory
         Exit Sub ' return from routine
      End If ' end check empty container record
   Next ' loop through container records
   Outpt = "You can't, it's full!" ' message
   Call IO.O ' send output
End Sub ' end drop into container routine

Rem * routine to take an item of treasure from a container and add to
Rem * the player inventory.

Sub Take.From.Container
   Call ParseX ' parse second parameter
   Call Numeric ' parse parameter number from # sign
   Call Examine.Container ' check container mnemonic
   If Index.Number = False Then ' compare treasure index
      Outpt = "You can't get that!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare container index
   If ContainerRec.Locked > False Then ' check container locked
      Outpt = "You can't, it's locked!" ' message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check container locked
   If ContainerRec.Closed > False Then ' check container closed
      Outpt = "You can't, it's closed!" ' message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check container closed
   Container.Number = Array.Number ' store container index of player inventory
   Container.Name$ = ContainerRec.ContainerName ' store container name
   Container.Name$ = RTrim$(Container.Name$) ' edit container name
   Parsed.Command1 = Parsed.Command2 ' get first parameter
   Call Numeric ' parse parameter number from # sign
   Item.Counter = False ' parsed number counter
   For Array.Counter = 1 To 5 ' loop through all container treasure
      ' get treasure name from container
      Index.Number = GetContainerRecInventory(Array.Counter) ' store treasure index
      ' check treasure index range
      If Index.Number > False And Index.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
         Call Read.Record(TreasureFile, Index.Number) ' get treasure record
         Inpt = RTrim$(TreasureRecord.ShortName)
         If Len(Inpt) Then ' check treasure name length
            Inpt = Left$(Inpt, Len(Parsed.Command1)) ' truncate treasure name
            If Inpt = Parsed.Command1 Then ' compare to treasure to take
               Item.Counter = Item.Counter + 1 ' increment parsed counter
               ' compare to parameter number
               If Parse.Number = False Or Item.Counter = Parse.Number Then
                  Outpts = TreasureRecord.TreasureName ' format treasure name
                  Outpts = RTrim$(Outpts) ' trim name
                  ' compute weight player carrying
                  New.Weight# = CDbl(Int(Weight + TreasureRecord.Weight))
                  If New.Weight# > MaxInt Then ' check maximum integer
                     New.Weight# = MaxInt ' reduce to maximum integer
                  End If ' end check maximum integer
                  Total.Weight# = CDbl(GetUserRecordStats(1)) ' store player strength
                  Total.Weight# = Total.Weight# * 100 ' multiply weight player can carry
                  If New.Weight# > Total.Weight# Then ' compare wieght player can carry
                     Outpt = "You can't carry any more!" ' weight and make message
                     Call IO.O ' send message
                     Exit Sub ' exit routine
                  End If ' end compare weight
                  ' get treasure charges
                  Charges.Number = GetContainerRecCharges(Array.Counter)
                  ' add treasure to inventory
                  Call Add.Inventory(Index.Number, Charges.Number, Item.Added)
                  If Item.Added Then ' flag set to add
                     Call Clear.Container(Array.Counter, False) ' clear container item
                     Select Case TypeNumber ' select room or inventory
                        Case 1 ' flag indicating container in room or inventory
                           RoomRecord.Container = ContainerRec ' set room container record
                           Call Share.Room.Record(Room) ' write room record
                        Case 0 ' flag indicates player record
                           ' update player container
                           Call PutUserRecordContainer(Container.Number)
                     End Select ' end compare room or inventory
                     Outpt = "You take " + Outpts + " from " + Container.Name$ + "!" ' message
                     Call IO.O ' send message
                     Exit Sub ' exit routine
                  End If ' end compare add flag
               End If ' end check treasure index
            End If ' end compare parsed number
         End If ' end compare treasure name
      End If ' end check treaure name length
   Next ' end loop through player container inventory
   Outpt = "You can't get that!" ' make message
   Call IO.O ' send message
End Sub ' end take from container routine

Rem * sets container elements to zero.
Rem * input variables:
Rem *   Item.Number - container item to clear.
Rem *   Clear.Items - true to clear container, false to clear container item.

Sub Clear.Container (Item.Number, Clear.Items)
   If Clear.Items = False Then ' passed parameter to clear one container item
      Call PutContainerRecCharges(Item.Number, False) ' container element to zero
      Call PutContainerRecInventory(Item.Number, False) ' container element to zero
      Call PutContainerRecInvisible(Item.Number, False) ' container element to zero
   Else ' passed parameter to clear entire container
      ContainerRec.Closed = False ' container to zero
      ContainerRec.ContainerName = Nul ' container to zero
      ContainerRec.Keyed = False ' container to zero
      ContainerRec.Locked = False ' container to zero
      ContainerRec.ShortName = Nul ' container to zero
      ContainerRec.Permanent = False ' container to zero
      For Container.Item = 1 To 5 ' loop through all container elements
         Call PutContainerRecInventory(Container.Item, False) ' reset container element
         Call PutContainerRecCharges(Container.Item, False) ' reset container element
         Call PutContainerRecInvisible(Container.Item, False) ' reset container element
      Next ' end loop all container elements
   End If ' end passed parameter
End Sub ' end routine to clear container

Rem * routine to display container information.
Rem * input variables:
Rem *   ContainerRec - container record to display.
Rem *   TypeNumber - item is in room/inventory.

Sub Show.Container
   If TypeNumber Then ' container in room
      Prefix1 = "It's " ' format prefix
   Else ' container in inventory
      Prefix1 = "You are carrying " ' format prefix
   End If ' end container check
   Graphics.Off = True ' reset color
   Outpt = Prefix1 + RTrim$(ContainerRec.ContainerName) ' make container name
   If ContainerRec.Keyed Then ' check container has a lock number
      Outpt = Outpt + "(#" + Right$(Str$(ContainerRec.Keyed + 100000!), 5) + ")"
   End If ' end append container lock number
   Outpt = Outpt + "." ' append to message
   Call IO.O ' send message of container name
   If ContainerRec.Locked > False Then ' verify container is locked
      Outpt = "It's locked." ' make message
      Call IO.O ' send output
      Exit Sub ' exit routine
   End If ' end verify locked container
   If ContainerRec.Closed > False Then ' verify container is closed
      Outpt = "It's closed." ' make message
      Call IO.O ' send output
      Exit Sub ' exit routine
   End If ' end verify closed container
   Outpt = "It contains the following treasure:" ' message
   Call IO.O ' send prefix message
   Container.Count = False ' reset number of container items displayed
   Outpt = Nul ' reset message of container item
   For Array.Index = 1 To 5 ' loop through all container items
      ' store container invisibility flag
      Invisible.Container = GetContainerRecInvisible(Array.Index)
      ' store container inventory number
      Container.Number = GetContainerRecInventory(Array.Index)
      Display.Item = True ' reset flag to display container item
      If Container.Number Then ' check container item
         If Invisible.Container = True Then ' verify item is invisible
            If Normal.User Then ' determine non DM
               Display.Item = False ' set flag not to display item
            End If ' end normal user
         End If ' end invisible item
         If Display.Item = True Then ' check flag to display item
            Carriage.Return = True ' set flag to disable return/linefeed
            Call IO.O ' send message of previous item
            ' get treasure record of container item
            Call Read.Record(TreasureFile, Container.Number)
            Outpts = TreasureRecord.TreasureName ' store treasure name
            Outpts = RTrim$(Outpts) ' trim name
            If Invisible.Container = True Then ' compare invisible item
               Outpts = Outpts + "[inv]" ' sppend invisible message
            End If ' end compare invisible
            Outpt = Outpts + ", " ' append comma to item name
            ' increment number of items displayed
            Container.Count = Container.Count + 1
            If Container.Count = 1 Then ' check if first item displayed
               Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first item
            End If ' end compare to first item
         End If ' end check display flag
      End If ' end check container item
   Next ' end loop through all five items
   If Container.Count = False Then ' check if any container items displayed
      Outpt = "Nothing at all." ' make message
   Else ' check any container items displayed
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, append period
      If Container.Count > 1 Then ' more than one item displayed
         Outpt = "and " + Outpt ' append to message
      End If ' end check item number
   End If ' end check item number
   Call IO.O ' send message of last contaier item
   Graphics.Off = False ' reset color
End Sub ' end routine to display container information

Rem * routine to search player inventory for container name
Rem * input variables:
Rem *   Parsed.Command1 - name of container to search for.
Rem * output variables:
Rem *   Array.Number - container number.
Rem *   Index.Number - true if container found.
Rem *   Outpts - name of container.
Rem * processing variables:
Rem *   Parse.Count - increment counter.
Rem *   Container.Name$ - container name.

Sub Check.Inventory.Container
   Index.Number = False ' reset container found flag
   Parse.Count = False ' reset counter
   If Parsed.Command1 <> Nul Then ' check search string length
      For Array.Number = 1 To 3 ' loop through player inventory containers
         ' store container record
         X = GetUserRecordContainer(Array.Number)
         ' store container name
         Container.Name$ = RTrim$(ContainerRec.ShortName)
         Outpts = Container.Name$ ' store container name
         If Container.Name$ <> Nul Then ' check length of container name
            ' trim length of container name to length of search string
            Container.Name$ = Left$(Container.Name$, Len(Parsed.Command1)) ' trim
            If Container.Name$ = Parsed.Command1 Then ' compare container names
               Parse.Count = Parse.Count + 1 ' increment counter
               ' check increment counter not specified,
               ' or counter equals increment counter.
               If Parse.Number = False Or Parse.Count = Parse.Number Then
                  Index.Number = True ' set container found flag
                  Exit For ' exit loop through player containers
               End If ' end check counters
            End If ' end compare container names
         End If ' end check container name length
      Next ' end loop through player containers
   End If ' end check search string length
End Sub ' end routine to search player inventory for container name

Rem * routine to compare room container to search container name
Rem * input variables:
Rem *   Parsed.Command1 - name of container to ccompare.
Rem * output variables:
Rem *   Index.Number - room container matched.
Rem *   Outpts - name of container.
Rem * processing variables:
Rem *   Parse.Count - increment counter.
Rem *   Container.Name$ - container name.

Sub Check.Room.Container
   Index.Number = False ' reset container found flag
   Parse.Count = False ' reset counter
   If Parsed.Command1 <> Nul Then ' check search string length
      ' get room container name
      Container.Name$ = RTrim$(RoomRecord.Container.ShortName)
      Outpts = Container.Name$ ' store container name
      If Container.Name$ <> Nul Then ' check length of container name
         ' trim length of container name to length of search string
         Container.Name$ = Left$(Container.Name$, Len(Parsed.Command1)) ' trim
         If Container.Name$ = Parsed.Command1 Then ' compare container names
            Parse.Count = Parse.Count + 1
            ' check increment counter not specified,
            ' or counter equals increment counter.
            If Parse.Number = False Or Parse.Count = Parse.Number Then
               Index.Number = True ' set container match flag
               ContainerRec = RoomRecord.Container ' store container record
            End If ' end check counters
         End If ' end compare names
      End If ' end check container name
   End If ' end check search string length
End Sub

Rem * routine to remove an item of player inventory and add to room.

Sub Drop.Item
   Call Find.Object ' check inventory for object by mnemonic name
   If Index.Number Then ' returns index to objects file
      If Last.Command.Number = Hide.Command Then ' compare command to hide
         Outpt = "You hide " + Outpts + "!" ' format hide message
         Hide.Item = True ' set hide flag
      Else ' not hide command
         Outpt = "You drop " + Outpts + "!" ' format message
         Hide.Item = False ' set hide flag
      End If ' end compare command
      Call Add.Room.Object(Index.Number, Hide.Item, Item.Added) ' add item
      If Item.Added Then ' check object added flag
         Call Discard.Inventory.Object(Array.Number) ' remove object
      Else ' check item added to room
         Outpt = "You can't drop that!" ' message
      End If ' end check item added to room objects
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check inventory for object
   Call Num ' decrement parse temp
   Call Find.Inventory ' check inventory for treasure by mnemonic name
   If Index.Number Then ' returns index to treasure file
      If Last.Command.Number = Hide.Command Then ' compare command to hide
         Outpt = "You hide " + Outpts + "!" ' format hide message
         Hide.Item = Hidden.Object ' set hide flag
      Else ' not hide command
         Outpt = "You drop " + Outpts + "!" ' format message
         Hide.Item = False ' set hide flag
      End If ' end compare command
      ' add item to room
      Call Add.Room.Treasure(Index.Number, Charges.Number, Hide.Item, Item.Added)
      If Item.Added Then ' added to room flag
         Call Discard.Inventory(Array.Number, True) ' remove from player inventory
      Else ' not added
         Outpt = "You can't drop that!" ' message
      End If ' end check room flag
      Call IO.O ' send output
      Exit Sub ' return from routine
   End If ' end check inventory for treasure
   Call Num ' decrement parse temp
   Call Check.Inventory.Container ' check player inventory for container
   If Index.Number Then ' returns index of container to treasure
      If Last.Command.Number = Hide.Command Then ' compare to hide command
         Outpt = "You can't hide that!" ' message
         Call IO.O ' send output
         Exit Sub ' return from routine
      End If ' end compare to hide command
      If RTrim$(RoomRecord.Container.ShortName) <> Nul Then ' room container empty
         Outpt = "You can't put more than one container in a room!" ' message
         Call IO.O ' output message
         Exit Sub ' exit routine
      End If
      If RTrim$(RoomRecord.Container.ShortName) = Nul Then ' room container empty
         ' store room container
         X = GetUserRecordContainer(Array.Number)
         RoomRecord.Container = ContainerRec
         Call Share.Room.Record(Room) ' record structure, put room record,
         Outpts = RTrim$(ContainerRec.ContainerName) ' message
         Call Clear.Container(0, True) ' clear player container structure
         Call PutUserRecordContainer(Array.Number) ' clear container record
         Outpt = "You drop " + Outpts + "!" ' format message
         Call IO.O ' send output
         Exit Sub ' return from routine
      End If ' end compare room container
   End If ' end check container index
   Outpt = "You can't drop that!" ' message
   Call IO.O ' send output
End Sub ' end drop item routine

Rem * routine to take an object or item of treasure from the room and
Rem * add to the player inventory.

Sub Take.Object
   Call Check.Room.Objects ' find the mnemonic of object to take
   If Index.Number Then ' verify object file index
      Object.Flag = False ' reset flag
      If Normal.User = False Then ' check DM/Sysop
         Object.Flag = True ' reset flag
      Else ' end check user type
         If ObjectRecord.Permanent = False Then ' verify object permanent
            Object.Flag = True ' reset flag
         End If ' end check object
      End If ' end check player type
      If Object.Flag Then ' check take flag
         ' routine to add object to user
         Call Add.Object(Index.Number, Item.Added)
         If Item.Added Then ' an empty object element
            ' routine to remove item from room
            Call Discard.Room.Object(Array.Number)
            Outpt = "You take " + Outpts + "!" ' format message
            Call IO.O ' display message
         Else ' check empty object found
            Outpt = "You can't carry any more!" ' format message
            Call IO.O ' display message
         End If ' end compare empty object element
         Exit Sub ' exit routine
      End If ' end find object to take
   End If ' end normal player
   Call Num ' decrement parse temp
   Call Check.Room.Treasure ' find mnemonic of treasure to take
   If Index.Number Then ' treasure index found in room to take
      ' calculate weight player is carrying
      New.Weight# = CDbl(Int(Weight + TreasureRecord.Weight))
      If New.Weight# > MaxInt Then ' check maximum integer
         New.Weight# = MaxInt ' reduce to maximum integer
      End If ' end check maximum integer
      Total.Weight# = CDbl(GetUserRecordStats(1)) ' store player strength
      Total.Weight# = Total.Weight# * 100 ' multiply to weight player can carry
      If New.Weight# > Total.Weight# Then ' compare weight player can carry
         Outpt = "You can't carry any more!" ' weight ten times player strength
         Call IO.O ' format message
         Exit Sub ' exit routine
      End If ' end compare weight
      If UserRecord.ClassType < AsstDM Then ' normal class prevented to take item
         For Array.Counter = 1 To Number.Monsters ' loop through all monsters
            ' check monster can prevent
            If MonsterArray(Array.Counter).Prevent Then
               ' compute random percent
               If Rnd < (MonsterArray(Array.Counter).PreventPercent / 100) Then
                  Inpt = MonsterArray(Array.Counter).MonsterName ' store name
                  Inpt = RTrim$(Inpt) ' format message
                  Outpt = "The " + Inpt + " prevents you from getting it!"
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end compare random calculation
            End If ' end compare monster to prevent taking item
         Next ' end loop through all monsters in room
      End If ' end compare player class
      If TreasureRecord.Coin Then ' check item is coins
         Outpt = "You take " + Outpts + "!" ' format message
         Call IO.O ' send message
         UserRecord.Gold = UserRecord.Gold + TreasureRecord.Gold ' increment gold
         Outpt = "You now have" + FormatString$(UserRecord.Gold) + " Gold!" ' message
         Call IO.O ' send message
         Call Discard.Room.Treasure(Array.Number) ' remove item from room
         Exit Sub ' exit routine
      End If ' end compare item if coins
      ' add room item to inventory
      Call Add.Inventory(Index.Number, Charges.Number, Item.Added)
      If Item.Added Then ' return flag indicates successful addition to inventory
         Outpt = "You take " + Outpts + "!" ' format message
         Call IO.O ' send message
         Call Discard.Room.Treasure(Array.Number) ' remove item from room
      Else ' compare flag
         Outpt = "You can't carry any more!" ' make message
         Call IO.O ' send message
      End If ' end compare flag
      Exit Sub ' exit routine
   End If ' end item is treasure in room
   Call Num ' decrement parse temp
   Call Check.Room.Container ' verify item to take is a container
   If Index.Number Then ' found container index to treasure file
      ' loop through player inventory to find empty container
      For Array.Number = 1 To 3
         ' compare container name
         X = GetUserRecordContainer(Array.Number)
         If RTrim$(ContainerRec.ShortName) = Nul Then
            ' store container record
            ContainerRec = RoomRecord.Container
            Call PutUserRecordContainer(Array.Number)
            Outpts = RTrim$(ContainerRec.ContainerName) ' format message
            Outpt = "You get " + Outpts + "!" ' make message
            Call IO.O ' send message
            Call Clear.Container(0, True) ' clear entire container record
            RoomRecord.Container = ContainerRec ' clear room container
            Call Share.Room.Record(Room) ' write room record
            Exit Sub ' exit routine
         End If ' end compare empty container mnemonic name
      Next ' loop through inventory
      Outpt = "You can't carry more than three containers!" ' message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end verify taking container
   Outpt = "You can't get that!" ' message
   Call IO.O ' send message
End Sub ' end take item routine

Rem * routine to lock an object or container.
Rem * Note: Allows DM/Sysop to add locks.

Sub Lock.Object
   Call Examine.Container ' check container mnemonic
   If Index.Number Then ' container index to treasure file
      Container.Number = Array.Number ' store container number
      If ContainerRec.Locked > False Then ' container lock variable is already set
         Outpt = "It's already locked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container lock
      If ContainerRec.Locked = False Then ' container has no lock setting
         If Normal.User Then ' check DM/Sysop
            Outpt = "You can't lock that!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         Else ' check DM/Sysop
            Outpt = "Container has no lock. Add one(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            Call IO.I ' get input
            If No Then ' compare response
               Exit Sub ' exit routine
            End If ' end compare response
            Outpt = "Add a key to the lock(y/n)? " ' make message
            No.Input.Out = "N" ' default input
            Call IO.I ' get input
            If Yes Then ' check response
               Outpt = "Key number(1-32767)? " ' make input prompt
               No.Input.Out = "1" ' default input
               Call IO.I ' get user input
               ContainerRec.Keyed = Int(Val(Inpt)) ' set object key number
            End If ' end check response
         End If ' check DM/Sysop
      End If ' end compare container lock
      ContainerRec.Locked = 1 ' set container variable to locked
      ContainerRec.Closed = 1 ' set container variable to locked
      Select Case TypeNumber ' select room or inventory
         Case 1 ' container is locked in room
            RoomRecord.Container = ContainerRec
            Call Share.Room.Record(Room) ' write room record
         Case 0 ' container is locked in player inventory
            ' update player container
            Call PutUserRecordContainer(Container.Number)
      End Select ' end compare container locked
      Outpt = "You lock it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check container
   Call Num ' decrement parse temp
   Call Examine.Objects ' check object mnemonic
   If Index.Number Then ' found object index
      If ObjectRecord.RoomLink = False Then ' check object is a portal
         Outpt = "You can't lock that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare object portal
      If ObjectRecord.DoorLock = 2 Then ' object is already locked
         Outpt = "It's already locked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare object lock
      If ObjectRecord.DoorLock = False Then ' object has no lock setting
         If Normal.User Then ' check DM/Sysop
            Outpt = "You can't lock that!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         Else ' check DM/Sysop
            Outpt = "Object has no lock. Add one(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            Call IO.I ' get input
            If No Then ' compare response
               Exit Sub ' exit routine
            End If ' end compare response
            Outpt = "Add a key to the lock(y/n)? " ' make message
            No.Input.Out = "N" ' default input
            Call IO.I ' get input
            If Yes Then ' check response
               Outpt = "Key number(1-32767)? " ' make input prompt
               No.Input.Out = "1" ' default input
               Call IO.I ' get user input
               ObjectRecord.Keyed = Int(Val(Inpt)) ' set object key number
            End If ' end check response
         End If ' check DM/Sysop
      End If ' end compare object lock
      ObjectRecord.DoorLock = 2 ' set object lock variable
      ObjectRecord.Closed = True ' set object lock variable
      Call Share.Record(ObjectFile, Index.Number) ' write object record
      Outpt = "You lock it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check object
   Outpt = "You can't lock that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to lock an item

Rem * routine to unlock or picklock an object or container.

Sub Unlock.Object
   Call Examine.Container ' check container mnemonic
   If Index.Number Then ' found container treasure index
      Container.Number = Array.Number ' store container number
      If ContainerRec.Locked = False Then ' container has no lock setting
         Outpt = "You can't unlock that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container lock
      If ContainerRec.Locked < False Then ' container lock variable already set
         Outpt = "It's already unlocked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container lock
      If ContainerRec.Keyed = False Then ' container has no key number
         ContainerRec.Locked = True ' set container lock variable
         ContainerRec.Closed = 1 ' set container lock variable
         Select Case TypeNumber ' select room or inventory
            Case 1 ' container unlocked in room
               RoomRecord.Container = ContainerRec ' set room container record
               Call Share.Room.Record(Room) ' write room record
            Case 0 ' container is locked in player inventory
               ' update player container
               Call PutUserRecordContainer(Container.Number)
         End Select ' end store container record
         Outpt = "You unlock it!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container key number
      If Last.Command.Number = Picklock.Command Then ' picklock container
         If UserRecord.ClassType <> Thief Then ' compare player class to thief
            If Normal.User Then ' compare player type to normal
               Outpt = "You can't picklock doors!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end compare player type
         End If ' end compare class to thief
         If Int(Rnd * GetUserRecordStats(1)) + 3 < 7 Then ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random chance
         If Int(Rnd * GetUserRecordStats(2)) + 3 < 7 Then ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random chance
         If Rnd < .2 Then ' random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end chance
         ContainerRec.Locked = True ' set container record to unlocked
         ContainerRec.Closed = 1 ' set container record to unlocked
         Select Case TypeNumber ' select room or inventory
            Case 1 ' container unlocked in room
               RoomRecord.Container = ContainerRec ' set room container record
               Call Share.Room.Record(Room) ' write room record
            Case 0 ' container unlocked in player inventory
               ' update player container
               Call PutUserRecordContainer(Container.Number)
         End Select ' end set container record
         Outpt = "You picklock it!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check picklock container
      ' compare player inventory key numbers to container key number
      Key.Number = ContainerRec.Keyed ' store container key number to unlock
      Inpt = Right$(Str$(Key.Number + 100000!), 5) ' format key number to string
      For Inventory.Number = 1 To 20 ' loop through all player inventory
         ' player inventory treasure index
         If GetUserRecordInv(Inventory.Number) Then
            ' get treasure record
            Call Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            If TreasureRecord.Keyed Then ' compare inventory treasure to key
               Container.Locked = True ' container unlocked flag
               ' format key number in inventory to string
               Outpts = Right$(Str$(TreasureRecord.Keyed + 100000!), 5)
               For Digit.Counter = 1 To 5 ' loop through all key number digits
                  ' get value of key digit
                  Key.Digit = Val(Mid$(Outpts, Digit.Counter, 1))
                  ' compare player inventory key digit to container key digit
                  ' digits unequal
                  If Key.Digit <> Val(Mid$(Inpt, Digit.Counter, 1)) Then
                     ' player inventory key digit 0 matches
                     If Key.Digit > False Then
                        ' container is not unlocked flag
                        Container.Locked = False
                        Exit For ' exit loop, digit mismatch
                     End If ' end compare key digit 0 matches any key number
                  End If ' end compare key digit matches specific key number
               Next ' end loop through all key digits in player inventory key
               ' key in player inventory found to match container
               If Container.Locked Then
                  ContainerRec.Locked = True ' set container unlock
                  ContainerRec.Closed = 1 ' set container unlock
                  Select Case TypeNumber ' select room or inventory
                     Case 1 ' container unlocked with key in room
                        RoomRecord.Container = ContainerRec ' set room container
                        Call Share.Room.Record(Room) ' write room record
                     Case 0 ' container unlocked with key in player inventory
                        ' update player container
                        Call PutUserRecordContainer(Container.Number)
                  End Select ' end set container record
                  Outpt = "You unlock it!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end compare key match
            End If ' end compare player inventory to a key
         End If ' end compare player inventory
      Next ' end loop through all keys in player inventory
      Outpt = "You don't have the key!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check container
   Call Num ' decrement parse temp
   Call Examine.Objects ' check object mnemonic
   If Index.Number Then ' found object index
      If ObjectRecord.RoomLink = False Then ' check object is a portal
         Outpt = "You can't unlock that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object portal
      If ObjectRecord.DoorLock = False Then ' object does not lock
         Outpt = "You can't unlock that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object lock
      If ObjectRecord.DoorLock = 1 Then ' object is already unlocked
         Outpt = "It's already unlocked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object lock
      If ObjectRecord.Keyed = False Then ' object has no key number
         ObjectRecord.DoorLock = 1 ' set object unlocked
         ObjectRecord.Closed = True ' set object unlocked
         Call Share.Record(ObjectFile, Index.Number) ' write object record
         Outpt = "You unlock it!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object key number
      If Last.Command.Number = Picklock.Command Then ' picklock object
         If ObjectRecord.Hidden Then ' object is hidden
            Outpt = "You can't picklock hidden doors!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end object hidden
         If UserRecord.ClassType <> Thief Then ' check player class to thief
            If Normal.User Then ' check player is normal
               Outpt = "You can't picklock doors!" ' make message
               Call IO.O ' send message
               Exit Sub ' exit routine
            End If ' end compare normal player
         End If ' end compare class type
         If Int(Rnd * GetUserRecordStats(1)) + 3 < 7 Then ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random chance
         If Int(Rnd * GetUserRecordStats(2)) + 3 < 7 Then ' compute random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end random chance
         If Rnd < .2 Then ' random chance
            Outpt = "You didn't picklock it!" ' make message
            Call IO.O ' send message
            Exit Sub ' exit routine
         End If ' end chance
         ObjectRecord.DoorLock = 1 ' set object unlocked
         ObjectRecord.Closed = True ' set object unlocked
         Call Share.Record(ObjectFile, Index.Number) ' write object record
         Outpt = "You picklock it!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end picklock
      ' compare player inventory key numbers to object key number
      Inpt = Right$(Str$(ObjectRecord.Keyed + 100000!), 5) ' format key number
      For Inventory.Number = 1 To 20 ' loop through all player inventory
         ' player inventory treasure index
         If GetUserRecordInv(Inventory.Number) Then
            ' get treasure record
            Call Read.Record(TreasureFile, GetUserRecordInv(Inventory.Number))
            If TreasureRecord.Keyed Then ' compare treasure to key
               Container.Locked = True ' set object unlocked flag
               ' format player inventory key number to string
               Outpts = Right$(Str$(TreasureRecord.Keyed + 100000!), 5)
               For Digit.Counter = 1 To 5 ' loop through all key number digits
                  ' get inventory key digit
                  Key.Digit = Val(Mid$(Outpts, Digit.Counter, 1))
                  ' compare player inventory key digit to container key digit
                  ' compare to object
                  If Key.Digit <> Val(Mid$(Inpt, Digit.Counter, 1)) Then
                     If Key.Digit > False Then ' zero digit matches any key digit
                        Container.Locked = False ' set object is not unlocked flag
                        Exit For ' exit loop, found unmatching digit
                     End If ' end compare matching zero digit
                  End If ' end compare both digits
               Next ' end loop through all digits
               If Container.Locked Then ' object key number matched
                  ObjectRecord.DoorLock = 1 ' set object unlocked
                  ObjectRecord.Closed = True ' set object unlocked
                  Call Share.Record(ObjectFile, Index.Number) ' write object record
                  Outpt = "You unlock it!" ' make message
                  Call IO.O ' send message
                  Exit Sub ' exit routine
               End If ' end matching key number
            End If ' end locate player inventory key
         End If ' end find player inventory treasure
      Next ' end loop through all player inventory for keys
      Outpt = "You don't have the key!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end unlock object
   Outpt = "You can't unlock that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to unlock/picklock an item or container

Rem * routine to close an object.

Sub Close.Object
   Call Examine.Container ' check container mnemonic
   If Index.Number Then ' found container index to treasure file
      Container.Number = Array.Number ' store container number
      If ContainerRec.Locked = False Then ' container has no lock to set
         Outpt = "You can't close that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end container lock
      If ContainerRec.Locked > False Then ' check if container already locked
         Outpt = "It's already closed!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end container lock
      If ContainerRec.Closed = False Then ' container has no close setting
         Outpt = "You can't close that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare closed
      If ContainerRec.Closed > False Then ' container is already closed
         Outpt = "It's already closed!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare closed
      ContainerRec.Closed = 1 ' set container record closed variable
      Select Case TypeNumber ' select room or inventory
         Case 1 ' container closed in room
            RoomRecord.Container = ContainerRec ' set room container record
            Call Share.Room.Record(Room) ' write room record
         Case 0 ' container closed in player inventory
            ' update player container
            Call PutUserRecordContainer(Container.Number)
      End Select ' end reset container
      Outpt = "You close it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare container index
   Call Num ' decrement parse temp
   Call Examine.Objects ' check object mnemonic
   If Index.Number Then ' found object index to object file
      If ObjectRecord.DoorLock <> 1 Then ' compare object already locked
         Outpt = "You can't close that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare lock
      If ObjectRecord.Closed Then ' compare container already closed
         Outpt = "It's already closed!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare closed
      ObjectRecord.Closed = True ' set object record closed variable
      Call Share.Record(ObjectFile, Index.Number) ' write object record
      Outpt = "You close it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end object search
   Outpt = "You can't close that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to closed an item

Rem * routine to open an item.

Sub Open.Object
   Call Examine.Container ' check container mnemonic
   If Index.Number Then ' found container index
      Container.Number = Array.Number ' store container number
      If ContainerRec.Locked = False Then ' container has no lock to set
         Outpt = "You can't open that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container lock
      If ContainerRec.Locked > False Then ' compare container lock variable
         Outpt = "You can't, it's locked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare lock
      If ContainerRec.Closed = False Then ' container has no close variable to set
         Outpt = "You can't open that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare close
      If ContainerRec.Closed < False Then ' container is already open
         Outpt = "It's already open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare close
      ContainerRec.Closed = True ' set container open variable
      Select Case TypeNumber ' select room or inventory
         Case 1 ' container is opened in room
            RoomRecord.Container = ContainerRec ' reset room container
            Call Share.Room.Record(Room) ' write room record
         Case 0 ' container is open in inventory
            ' update player container
            Call PutUserRecordContainer(Container.Number)
      End Select ' end reset container
      Outpt = "You open it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare container
   Call Num ' reset parse temp
   Call Examine.Objects ' check object mnemonic
   If Index.Number Then ' found object index
      If ObjectRecord.DoorLock <> 1 Then ' compare object closed variable
         Outpt = "You can't open that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare closed
      If ObjectRecord.Closed = False Then ' compare object open variable
         Outpt = "It's already open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare opened
      ObjectRecord.Closed = False ' set object record open variable
      Call Share.Record(ObjectFile, Index.Number) ' write object record
      Outpt = "You open it!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end object index
   Outpt = "You can't open that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to open an item

Rem * routine to smash open a locked item.

Sub Smash.Object
   Call Examine.Container ' check container mnemonic
   If Index.Number Then ' found container index to treasure file
      Container.Number = Array.Number ' store container number
      If ContainerRec.Locked = False Then ' container has no available lock
         Outpt = "You can't smash that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container lock
      If ContainerRec.Locked < False Then ' lock variable is already open
         Outpt = "It's already unlocked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare lock
      If ContainerRec.Keyed > False Then ' container has a key number
         Outpt = "You can't smash that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare key number
      If Int(Rnd * GetUserRecordStats(2)) + 1 < 7 Then ' compute smash chance
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      If Int(Rnd * GetUserRecordStats(1)) + 1 < 7 Then ' compare smash chance
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      If Rnd < .2 Then ' random chance to smash lock
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      ContainerRec.Locked = True ' set container to unlocked
      ContainerRec.Closed = 1 ' set container to closed
      Select Case TypeNumber ' select room or inventory
         Case 1 ' container unlocked in room
            RoomRecord.Container = ContainerRec ' set new container record
            Call Share.Room.Record(Room) ' write room record
         Case 0 ' container unlocked in inventory
            ' update player container
            Call PutUserRecordContainer(Container.Number)
      End Select ' end reset container
      Outpt = "You smash it open!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare container index
   Call Num ' decrement parse temp
   Call Examine.Objects ' check object mnemonic
   If Index.Number Then ' found object index
      If ObjectRecord.Hidden Then ' object type is hidden
         Outpt = "You can't smash that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object type
      If ObjectRecord.RoomLink = False Then ' object is not a portal
         Outpt = "You can't smash that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end portal
      If ObjectRecord.Keyed Then ' object has a key number
         Outpt = "You can't smash that!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object key number
      If ObjectRecord.DoorLock = 1 Then ' object lock is already open
         Outpt = "It's already unlocked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end object lock
      If Int(Rnd * GetUserRecordStats(2)) + 1 < 7 Then ' compute random chance
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      If Int(Rnd * GetUserRecordStats(1)) + 1 < 7 Then ' compute random chance
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      If Rnd < .2 Then ' random chance
         Outpt = "You didn't smash it open!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end chance
      ObjectRecord.DoorLock = 1 ' set object record lock to open
      Call Share.Record(ObjectFile, Index.Number) ' write object record
      Outpt = "You smash it open!" ' make message
      Call IO.O ' send message
      If ObjectRecord.Trap Then ' check object is trapped
         Call Traps ' initiate any object traps
      End If ' end object trap
      Exit Sub ' exit routine
   End If ' end object index
   Outpt = "You can't smash that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to smash open lock

Rem * routine to search a container.

Sub Search.Object
   Call Examine.Container ' check player and room inventory for container name
   If Index.Number Then ' found container index, placed into container record
      If ContainerRec.Locked > False Then ' compare container lock
         Outpt = "You can't, it's locked!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container
      If ContainerRec.Closed > False Then ' compare container closed
         Outpt = "You can't, it's closed!" ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container
      Outpt = "You search the container.." ' make message
      Call IO.O ' send message
      Graphics.Off = True ' color white
      Outpt = "In it you find " ' preceding message
      Item.Counter = False ' container contents counter
      For Array.Number = 1 To 5 ' loop through all container contents
         If GetContainerRecInvisible(Array.Number) Then ' compare invisible item
            ' get container treasure index
            Inventory.Number = GetContainerRecInventory(Array.Number)
            If Inventory.Number > False And Inventory.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
               ' read index from treasure file
               Call Read.Record(TreasureFile, Inventory.Number)
               If Rnd > .5 Then ' compute random chance
                  Carriage.Return = True ' supress cr/lf
                  Call IO.O ' send message
                  Outpt = TreasureRecord.TreasureName ' get treasure name found
                  Outpt = RTrim$(Outpt) + ", " ' format treasure name
                  ' increment container contents counter
                  Item.Counter = Item.Counter + 1
               End If ' end random chance
            End If ' end check treasure file bounds
         End If ' end compare container contents invisible
      Next ' end loop through all container contents
      If Item.Counter = False Then ' compare no contents found
         Outpt = "In it you find nothing.." ' make message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare container content counter
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' format message, strip trailing comma
      If Item.Counter > 1 Then ' compare container content counter
         Outpt = "and " + Outpt ' format message with last contents found
      End If ' end compare content counter
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end find container to search
   Outpt = "You can't search that!" ' make message
   Call IO.O ' send message
End Sub ' end routine to search container

Rem * routine to display verbose text on an item entered on the game prompt,
Rem * including a room number, an object, an item of treasure, monster, and
Rem * contents of a container.

Sub Display.Information
   If Not Normal.User Then ' check non DM
      ' convert parameter to number (i.e. Look 12)
      Room.Number! = Int(Val(Parsed.Command1))
      ' check room bounds
      If Room.Number! > False And Room.Number! <= LOF(RoomFile) / Len(RoomRecord) Then
         Swap Room.Number!, Room ' store current room number
         Call Read.Room.Record(Room) ' get room to display
         Call Show.Room ' routine to display room
         Swap Room.Number!, Room ' restore current room number
         Call Read.Room.Record(Room) ' get current room
         Exit Sub ' exit routine
      End If ' end check room number
   End If ' end check normal user
   Outpt = UserRecord.CodeName ' get user codename
   Call Decrypt(Outpt) ' decrypt codename
   Outpt = Left$(Outpt, Len(Parsed.Command1)) ' store parameter
   If Outpt = Parsed.Command1 Then ' compare look command to user codename
      Call Display.Stats ' display player statistics
      Exit Sub ' exit routine
   End If ' end check command parameter
   Call Examine.Treasure ' routine to compare parameter to treasure name
   If Index.Number Then ' compare returned index to treasure file
      Call Show.Treasure ' routine to display information on an item of treasure
      Exit Sub ' exit routine
   End If ' end compare display treasure item
   Call Num ' reduce number of item
   Call Examine.Container ' compare parameter to container name
   If Index.Number Then ' check container to display
      Call Show.Container ' routine to display information on a container
      Exit Sub ' exit routine
   End If ' end display container
   Call Num ' reduce number of item
   Call Examine.Objects ' compare parameter to object
   If Index.Number Then ' check index to object record
      Call Show.Object ' routine to display information on an object
      Exit Sub ' exit routine
   End If ' end check object
   Call Num ' reduce number of item
   Call Check.Monster ' compare parameter to monster name in room
   If Monster.Number Then ' check index of monster array in room
      Call Show.Monster ' routine to display information on a monster
      Exit Sub ' exit routine
   End If ' end check monster name
   Outpt = "You can't examine that!" ' make error message
   Call IO.O ' send error message
End Sub ' end routine to display item

Rem * routine to display room description (short or long), objects in room,
Rem * monsters in room, containers in room, and directions/exits from room.
Rem * working variables:
Rem *   Lit.Room - returned true if room is unlit.

Sub Show.Room
   Call Check.Lit.Room(Lit.Room) ' routine to determine if room is unlit
   If Lit.Room Then ' check unlit room
      Outpt = "It's too dark to see!" ' make message
      Call IO.O ' send message output
      Exit Sub ' exit routine
   End If ' end check unlit room
   Graphics.Off = True ' reser color
   ' check if user is in brief mode, or room has no long description
   Outpt = GetRoomRecordLongDesc(1) ' store room short one line description
   Outpt = RTrim$(Outpt) ' trim brief description
   ' find any null characters from old structures
   If InStr(Outpt, Chr$(0)) Then
      Outpt = Left$(Outpt, InStr(Outpt, Chr$(0)) - 1) ' truncate off nulls
   End If ' end find old nulls
   If UserRecord.Brief Or Outpt = Nul Then
      If Normal.User = False Then ' compare non DM status
         Outpt = "(room:" + Str$(Room) + ", monclass:" ' display DM room statistics
         If Room = 1! Then ' check safe haven room
            Outpt = Outpt + " <safe haven>" ' append room message
         Else ' check safe haven
            Outpt = Outpt + Str$(RoomRecord.MonsterClass) ' display room number
         End If ' end check safe haven room number
         Outpt = Outpt + ", action:" + Str$(RoomRecord.Action) + ")"
         If Room = 1! Then ' recheck safe haven room
            Outpt = Outpt + " (resurrection room:" ' display resurrection room
            If RoomRecord.MonsterClass = False Then ' check room number
               Outpt = Outpt + " <none>)" ' append default
            Else ' check room number
               Outpt = Outpt + Str$(RoomRecord.MonsterClass) + ")" ' append room number
            End If ' end check room number
         End If ' end check safe haven room
         Call IO.O ' display room monster class
      End If ' end compare normal user in brief description
      Outpt = RoomRecord.ShortDesc ' store room short one line description
      Outpt = RTrim$(Outpt) ' trim brief description
      ' find any null characters from old structures
      If InStr(Outpt, Chr$(0)) Then
         Outpt = Left$(Outpt, InStr(Outpt, Chr$(0)) - 1) ' truncate off nulls
      End If ' end find old nulls
      Call IO.O ' display room short description
   Else ' display room four line long description
      For Desc.Line = 1 To 4 ' loop through all four long room description lines
         Outpt = GetRoomRecordLongDesc(Desc.Line) ' get next room description line
         Outpt = RTrim$(Outpt) ' trim room description line
         If InStr(Outpt, Chr$(0)) Then ' find any nul characters from old files
            Outpt = Left$(Outpt, InStr(Outpt, Chr$(0)) - 1) ' truncate off nulls
         End If ' end find old nulls
         If Outpt = Nul Then ' check if long description ends
            Exit For ' exit long description display loop
         End If ' end check description end
         Call IO.O ' display room description line
      Next ' end long room description loop
   End If ' end check room description to display
   Exit.Displayed = False ' set flag for an exit displayed
   If UserRecord.Brief Then ' check user is in brief mode
      Outpt = Nul ' assign empty display string
   Else ' user is not in brief mode
      Outpt = "The exits are " ' assign initial display string
   End If ' end check brief mode
   For Direction.Number = 1 To 12 ' loop through all room directions
      ' compare room structure for direction
      If GetRoomRecordDirect(Direction.Number) Then
         ' append exit to string
         Outpt = Outpt + RTrim$(Direction(Direction.Number)) + ", "
         Exit.Displayed = True ' set flag for an exit displayed
      End If ' end compare room direction exists
   Next ' end loop through room directions
   If Exit.Displayed Then ' compare flag for an exit displayed
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' truncate trailing comma, add period
      If UserRecord.Brief Then ' check user in brief mode
         Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' set first letter uppercase
      End If ' end check brief mode
      Call IO.O ' send room exit display
   End If ' end compare exit displayed flag
   ' following displays all objects, treasure, monsters, and container in room
   If UserRecord.Brief = False Then ' check user is in brief mode
      Outpt = "You see " ' assign intial string
      Carriage.Return = True ' set flag to disable return/linefeed
      Call IO.O ' send initial string
   End If ' end check brief mode
   Outpt = Nul ' assign empty display string
   Items.Displayed = False ' reset number of items displayed
   Object.Displayed = False ' reset flag indicating an item has been displayed
   For Array.Index = 1 To 20 ' loop through all 20 objects in room
      DisplayObject = False ' set flag to show object
      Object.Number = GetRoomRecordObject(Array.Index) ' store room object number
      ' compare range
      If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
         Call Read.Record(ObjectFile, Object.Number) ' get object record
         If ObjectRecord.Invisible Then ' check object is invisible
            If Normal.User Then ' verify non DM
               DisplayObject = True ' set flag to show object off
            End If ' end verify normal user
         End If ' end check invisible object
         If GetRoomRecordHiddenObj(Array.Index) Then ' check object is hidden
            If Normal.User Then ' verify non DM
               DisplayObject = True ' set flag to show object off
            End If ' end verify normal user
         End If ' end check invisible object
         If ObjectRecord.Hidden Then ' check object is hidden
            If Normal.User Then ' verify non DM
               DisplayObject = True ' set flag to show object off
            End If ' end verify normal user
         End If ' end check hidden object
         If DisplayObject = False Then ' check display flag
            Carriage.Return = True ' set flag to disable return/linefeed
            Call IO.O ' send display of previous item
            ' increment number of items displayed
            Items.Displayed = Items.Displayed + 1
            If Items.Displayed = 1 Then ' verify first item being displayed
               If UserRecord.Brief Then ' check user in brief mode
                  Outpt = "You see " ' send initial display string
                  Carriage.Return = True ' set flag to disable return/linefeed
                  Call IO.O ' send previous string
               End If ' end check brief mode
            End If ' end verify first item
            Outpt = ObjectRecord.ObjectName ' store object name
            Outpt = RTrim$(Outpt) ' trim object name
            If ObjectRecord.DoorLock > 1 Then ' check object is locked
               Outpt = Outpt + "[locked]" ' append message
            Else ' compare unlocked object
               If ObjectRecord.DoorLock Then ' check object has lock
                  If ObjectRecord.Closed Then ' check object is closed
                     Outpt = Outpt + "[closed]" ' append message
                  End If ' end check closed object
               End If ' end check object is lock
            End If ' end check object is locked
            Hidden.Flag = GetRoomRecordHiddenObj(Array.Index) ' store hidden flag
            If ObjectRecord.Hidden Then ' check object is hidden
               Outpt = Outpt + "[hidden]" ' append message
            Else ' check object not hidden
               ' check object is invisible/hidden
               If ObjectRecord.Invisible Or Hidden.Flag Then
                  Outpt = Outpt + "[inv]" ' append message
               End If ' end check invisible object
            End If ' end check object hidden
            Outpt = Outpt + ", " ' append comma
            Object.Displayed = True ' set flag indicating an item displayed
         End If ' end check display flag
      End If ' end compare object file record range
   Next ' end loop through objects in room
   For Array.Index = 1 To 20 ' loop through all room treasure
      DisplayTreasure = False ' set flag to display item
      Treasure.Number = GetRoomRecordTreasure(Array.Index) ' store treasure number
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then ' compare range
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         ' compare invisible treasure flag
         If GetRoomRecordFlags(Array.Index) > False Then
            If Normal.User Then ' check non DM
               DisplayTreasure = True ' set flag to display item
            End If ' end check normal user
         End If ' end compare invisible treasure
         If TreasureRecord.Invisible Then ' check treasure is invisible
            If Normal.User Then ' check non DM
               DisplayTreasure = True ' set display flag
            End If ' end check normal user
         End If ' end check invisible treasure
         If DisplayTreasure = False Then ' compare display flag
            Carriage.Return = True ' set flag to disable return/linefeed
            Call IO.O ' display previous item
            ' increment number of items displayed
            Items.Displayed = Items.Displayed + 1
            If Items.Displayed = 1 Then ' check first item
               If UserRecord.Brief Then ' check user is in brief mode
                  Outpt = "You see " ' make initial string
                  Carriage.Return = True ' set flag to disable return/linefeed
                  Call IO.O ' send initial string
               End If ' end check brief mode
            End If
            Outpt = TreasureRecord.TreasureName ' get treasure name
            Outpt = RTrim$(Outpt) ' trim treasure name
            If TreasureRecord.Keyed Then ' append key number to treasure name
               Outpt = Outpt + "(#" + Right$(Str$(TreasureRecord.Keyed + 100000!), 5) + ")"
            End If ' end compare treasure key number
            If TreasureRecord.Plus Then ' append plus number to treasure name
               Outpt = Outpt + "(+" + Mid$(Str$(Abs(TreasureRecord.Plus)), 2) + ")"
            End If ' end compare treasure plus
            If TreasureRecord.Spell Then ' append spell plus to treasure name
               Call Read.Record(SpellFile, TreasureRecord.Spell) 'get spell record
               Outpt = Outpt + "(+" + Mid$(Str$(SpellRecord.Level), 2) + ")"
            End If ' end compare treasure spell plus
            If TreasureRecord.Locked > False Then ' check treasure is locked
               Outpt = Outpt + "[locked]" ' append message
            Else ' check treasure locks
               If TreasureRecord.Locked < False Then ' check treasure lock
                  If TreasureRecord.Closed > False Then ' check treasure is closed
                     Outpt = Outpt + "[closed]" ' append message
                  End If ' end check closed treasure
               End If ' end check treasure lock
            End If ' end check locked treasure
            If TreasureRecord.Invisible Then ' check treasure is invisible
               Outpt = Outpt + "[inv]" ' append message
            Else ' treasure is not invisible
               ' check invisible
               If GetRoomRecordFlags(Array.Index) = Hidden.Object Then
                  Outpt = Outpt + "[inv]" ' treasure flag, append message
               End If ' end check invisible flag
            End If ' end check invisible treasure
            If GetRoomRecordFlags(Array.Index) = Magic.Trap Then ' check magic trap
               Outpt = Outpt + "[trap]" ' treasure flag, append message
            End If ' end check magic trap flag
            Outpt = Outpt + ", " ' append comma
            Object.Displayed = True ' set item displayed flag
         End If ' end compare display flag
      End If ' end compare treasure file record range
   Next ' end loop through treasure
   ContainerRec = RoomRecord.Container ' store room container into record
   If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
      Carriage.Return = True ' set flag to disable return/linefeed
      Call IO.O ' send previous string
      Items.Displayed = Items.Displayed + 1 ' increment number of items displayed
      If Items.Displayed = 1 Then ' check first item displayed
         If UserRecord.Brief Then ' check user is in brief mode
            Outpt = "You see " ' make initial string
            Carriage.Return = True ' set flag to disable return/linefeed
            Call IO.O ' send first string
         End If ' end check brief mode
      End If ' end check first display item
      Outpt = RTrim$(ContainerRec.ContainerName) ' store container name
      If ContainerRec.Locked > False Then ' check container is locked
         Outpt = Outpt + "[locked]" ' append message
      Else ' container is not locked
         If ContainerRec.Closed > False Then ' check container is closed
            Outpt = Outpt + "[closed]" ' append message
         End If ' end check closed container
      End If ' end check locked container
      If ContainerRec.Keyed Then ' check container has a key number
         Outpt = Outpt + "(#" + Right$(Str$(ContainerRec.Keyed + 100000!), 5) + ")"
      End If ' append key number to container name
      Outpt = Outpt + ", " ' append comma
      Object.Displayed = True ' set an item displayed flag
   End If ' end compare container name length
   For Monster.Number = 1 To Number.Monsters ' loop through all room monsters
      Carriage.Return = True ' set flag to disable return/linefeed
      Call IO.O ' send previous string
      Items.Displayed = Items.Displayed + 1 ' increment number of items displayed
      If Items.Displayed = 1 Then ' check first item
         If UserRecord.Brief Then ' check user is in brief mode
            Outpt = "You see " ' make initial string
            Carriage.Return = True ' set flag to disable return/linefeed
            Call IO.O ' send initial string
         End If ' end check brief mode
      End If ' end check first item
      Outpts = MonsterArray(Monster.Number).MonsterName ' store monster name
      Outpts = RTrim$(Outpts) ' trim monster name
      Call The.Or.An ' routine to get prefix (a, an, the)
      Outpt = Outpt + Prefix1 + Outpts + ", " ' make message of monster
      Object.Displayed = True ' set an item displayed flag
   Next ' end loop through monsters in room
   If Object.Displayed = False Then ' compare flag indicating an item is displayed
      If UserRecord.Brief = False Then ' check user is in brief mode
         Outpt = "nothing special." ' make last item string
      Else ' user is in brief mode
         Outpt = "You see nothing." ' make entire message
      End If ' end check brief mode
   Else ' an item was displayed
      If Outpt <> Nul Then ' compare string length
         Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, append period
         If Items.Displayed > 1 Then ' check more than one item was displayed
            Outpt = "and " + Outpt ' append to last item
         End If ' end check item numbers
      End If ' end check valid last item string
   End If ' end check flag of items displayed
   Call IO.O ' send last display item
End Sub ' end routine to show room description, etc.

Rem * routine to display player character inventory.

Sub Display.Inventory
   Treasure.Counter = False ' reset counter
   For Array.Index = 1 To 20 ' loop through all player treasure inventory
      Treasure.Number = GetUserRecordInv(Array.Index) ' get treasure number
      If Treasure.Number Then ' compare number
         Treasure.Counter = Treasure.Counter + 1 ' increment counter
      End If ' end check compare
   Next ' end loop counting
   For Array.Index = 1 To 20 ' loop through all player treasure inventory
      Treasure.Number = GetUserRecordObject(Array.Index) ' get treasure number
      If Treasure.Number Then ' compare number
         Treasure.Counter = Treasure.Counter + 1 ' increment counter
      End If ' end check compare
   Next ' end loop counting
   Container.Count = False ' reset counter
   For Array.Index = 1 To 3 ' loop through all player character containers
      ' store container into record
      X = GetUserRecordContainer(Array.Index)
      ' compare container name length
      If RTrim$(ContainerRec.ShortName) <> Nul Then
         Treasure.Counter = Treasure.Counter + 1 ' increment counter
         Container.Count = Container.Count + 1 ' increment counter
      End If ' end check compare
   Next ' end loop counting
   Graphics.Off = True ' reset color
   If Treasure.Counter = 1 Then ' check counter
      Outpt = "You are carrying" + Str$(Treasure.Counter) + " item" ' weight message
   Else ' check counter
      Outpt = "You are carrying" + Str$(Treasure.Counter) + " items" ' weight message
   End If ' end check counter
   Outpt = Outpt + " weighing" + Str$(Weight + Container.Count) + " pounds:" ' weight message
   Call IO.O ' send output message
   Items.Displayed = False ' number of items displayed
   For Array.Index = 1 To 20 ' loop through all player treasure inventory
      Treasure.Number = GetUserRecordInv(Array.Index) ' get treasure number
      If Treasure.Number Then ' compare number
         Carriage.Return = True ' disable return/linefeed
         Call IO.O ' send initial item
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         Outpts = TreasureRecord.TreasureName ' store treasure name
         Outpts = RTrim$(Outpts) ' trim name
         If TreasureRecord.Keyed Then ' append key number to treasure name
            Outpts = Outpts + "(#" + Right$(Str$(TreasureRecord.Keyed + 100000!), 5) + ")"
         End If ' end compare treasure key number
         If TreasureRecord.Plus Then ' append plus number to treasure name
            Outpts = Outpts + "(+" + Mid$(Str$(Abs(TreasureRecord.Plus)), 2) + ")"
         End If ' end compare treasure plus
         If TreasureRecord.Spell Then ' append spell plus to treasure name
            Call Read.Record(SpellFile, TreasureRecord.Spell) ' get spell record
            Outpts = Outpts + "(+" + Mid$(Str$(SpellRecord.Level), 2) + ")"
         End If ' end compare treasure spell plus
         If TreasureRecord.LightType Then ' check treasure is a light
            If GetUserRecordCharges(Array.Index) < False Then ' check light charges
               Outpts = Outpts + "[lit]" ' append message
            End If ' end check charges
         End If ' end check light
         If TreasureRecord.Locked > False Then ' check treasure is locked
            Outpts = Outpts + "[locked]" ' append message
         Else ' check treasure locks
            If TreasureRecord.Locked < False Then ' check treasure lock
               If TreasureRecord.Closed > False Then ' check treasure is closed
                  Outpts = Outpts + "[closed]" ' append message
               End If ' end check closed treasure
            End If ' end check treasure lock
         End If ' end check locked treasure
         If TreasureRecord.Invisible Then ' check treasure is invisible
            Outpts = Outpts + "[inv]" ' append message
         End If ' end check invisible
         Outpt = Outpts + ", " ' append comma
         Items.Displayed = Items.Displayed + 1 ' increment item number
         If Items.Displayed = 1 Then ' check first item
            Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
         End If ' end check first item
      End If ' end compare treasure number
   Next ' end loop through inventory treasure
   For Array.Index = 1 To 20 ' loop through player object inventory
      Object.Number = GetUserRecordObject(Array.Index) ' get object number
      If Object.Number Then ' compare object number
         Carriage.Return = True ' disable return/linefeed
         Call IO.O ' send previous item
         Call Read.Record(ObjectFile, Object.Number) ' get object record
         Outpts = ObjectRecord.ObjectName ' store object name
         Outpts = RTrim$(Outpts) ' trim object name
         If ObjectRecord.Invisible Then ' compare object is invisible
            Outpts = Outpts + "[inv]" ' append to object name
         End If ' end compare object invisible
         If ObjectRecord.Keyed Then ' compare object key, append number to name
            Outpts = Outpts + "(#" + Right$(Str$(ObjectRecord.Keyed + 100000!), 5) + ")"
         End If ' end compare object key number
         Outpt = Outpts + ", " ' append comma
         Items.Displayed = Items.Displayed + 1 ' increment item number
         If Items.Displayed = 1 Then ' check first item
            Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
         End If ' end check first item
      End If ' end check object number
   Next ' end loop through object inventory
   For Array.Index = 1 To 3 ' loop through all player character containers
      ' store container into record
      X = GetUserRecordContainer(Array.Index)
      ' compare container name length
      If RTrim$(ContainerRec.ShortName) <> Nul Then
         Carriage.Return = True ' disable retuen/linefeed
         Call IO.O ' send previous item
         Outpt = RTrim$(ContainerRec.ContainerName) ' store container name
         If ContainerRec.Locked > False Then ' check container locked
            Outpt = Outpt + "[locked]" ' append message
         Else ' container not locked
            If ContainerRec.Closed > False Then ' check container closed
               Outpt = Outpt + "[closed]" ' append message
            End If ' end check closed container
         End If ' end check locked container
         If ContainerRec.Keyed Then ' check container key number
            Outpt = Outpt + "(#" + Right$(Str$(ContainerRec.Keyed + 100000!), 5) + ")"
         End If ' append container key number to message
         Outpt = Outpt + ", " ' append comma
         Items.Displayed = Items.Displayed + 1 ' increment item number
         If Items.Displayed = 1 Then ' check first item
            Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
         End If ' end check first item
      End If ' end compare container name length
   Next ' end loop through player containers
   If Items.Displayed = False Then ' compare items displayed
      Outpt = "Nothing at all." ' display message
   Else ' some items displayed
      Outpt = Left$(Outpt, Len(Outpt) - 2) + "." ' trim comma, append period
      If Items.Displayed > 1 Then ' check more than one item displayed
         Outpt = "and " + Outpt ' add to message
      End If ' end check item numbers
   End If ' end compare items
   Call IO.O ' send last item
End Sub ' end routine to display player inventory

Rem * routine to sell item of treasure, object, or container.

Sub Pawn.Shop
   Sell.Container = False ' item to sell is container flag
   Call Check.Inventory.Treasure ' routine to find treasure name
   If Index.Number = False Then ' check treasure index
      Call Num ' reduce counter
      Call Check.Inventory.Container ' routine to find container name
      Sell.Container = True ' set container for sale flag
      Container.Number = Array.Number ' store container number
      If Index.Number = False Then ' check treasure index
         Outpt = "The Broker says: You can't sell that!" ' make error message
         Call IO.O ' send error message
         Exit Sub ' exit routine
      End If ' end check index
   End If ' end check index
   Outpts = TreasureRecord.ShortName ' store treasure name being sold
   If Sell.Container Then ' check container flag
      Item.Cost# = 0# ' reset price of container contents
      Outpts = ContainerRec.ShortName ' store container name
      If ContainerRec.Locked > False Then ' check container lock
         Outpt = "The Broker tries the lock and says: Arrghh!! Can't open it!"
         Call IO.O ' send lock message
         Exit Sub ' exit routine
      End If ' end check container lock
      For Array.Index = 1 To 5 ' loop through container contents
         ' store treasure number
         Treasure.Number = GetContainerRecInventory(Array.Index)
         ' verify file bounds
         If Treasure.Number > False And Treasure.Number < LOF(TreasureFile) / Len(TreasureRecord) Then
            Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
            ' verify item plus is not loadable or launchable object
            If TreasureRecord.Loadable = False And TreasureRecord.Launchable = False Then
               ' check item plus or spell
               If TreasureRecord.Plus Or TreasureRecord.Spell Then
                  ' compare charges
                  If GetContainerRecCharges(Array.Index) = False Then
                     Item.Cost# = Item.Cost# + 1# ' add 1 gold for discharged item
                  Else ' compare charges
                     ' add treasure gold
                     Item.Cost# = Item.Cost# + CDbl(Int(TreasureRecord.Gold * .95))
                  End If ' end compare charges
               Else ' compare item plus or spell
                  ' add treasure gold
                  Item.Cost# = Item.Cost# + CDbl(Int(TreasureRecord.Gold * .9))
               End If ' end compare item plus or spell
            Else ' compare item plus type
               ' add treasure gold
               Item.Cost# = Item.Cost# + CDbl(Int(TreasureRecord.Gold))
            End If ' end check item plus type
         End If ' end verify file bounds
      Next ' end loop through container contents
      If Item.Cost# = 0# Then ' compare container content price
         Item.Cost# = 1# ' reset price
         Sell.Container = 1 ' sell the container
      Else ' compare contents
         Outpts = "contents of the " + Outpts ' make purchase message
      End If ' end compare container contents
   Else ' check container
      Item.Cost# = CDbl(Int(TreasureRecord.Gold * .9)) ' calculate price of treasure item
      ' verify item plus is not loadable or launchable object
      If TreasureRecord.Loadable = False And TreasureRecord.Launchable = False Then
         If TreasureRecord.Plus Or TreasureRecord.Spell Then ' verify charges
            If GetUserRecordCharges(Array.Number) = False Then ' compare charges
               Item.Cost# = 1# ' set price to 1
               Outpts = "broken " + Outpts ' make broken item message
            End If ' end compare charges
         End If ' end compare charges left
      End If ' end verify charges
   End If ' end check container being sold
   Outpt = "The Broker says: Well, I'll give you" + Str$(Item.Cost#) + " gold for the " + RTrim$(LCase$(Outpts)) + "." ' message of price for items
   Call IO.O ' send purchase message
   Outpt = "The Broker asks: Is that a deal(y/n)? " ' make input prompt
   No.Input.Out = "Y" ' default input
   Call IO.I ' get input
   If Yes Then ' compare response
      UserRecord.Gold = UserRecord.Gold + Item.Cost# ' increment player gold
      Outpt = "The Broker says: Great!" ' make purchase message
      If Sell.Container = True Then ' check container
         ' sell contents of container
         For Container.Item = 1 To 5 ' loop through container contents
            ' routine resets container item
            Call Clear.Container(Container.Item, False)
         Next ' end loop through container contents
         ' store container record
         Call PutUserRecordContainer(Container.Number)
      Else ' check container
         ' sell container
         If Sell.Container = 1 Then ' check container
            Call Clear.Container(0, True) ' clear player container structure
            Call PutUserRecordContainer(Container.Number) ' clear container record
         Else ' check sell item
            ' routine removes item from player
            Call Discard.Inventory(Array.Number, True)
         End If ' end check sell item/container
      End If ' end check container sold
   Else ' compare repsonse
      Outpt = "The Broker says: Oh well!" ' make purchase message
   End If ' end compare response
   Call IO.O ' send purchase message
End Sub ' end routine to sell items

Rem * routine for monsters to talk.
Rem * input variables:
Rem *   Parsed.Command1 - command parameter of monster name.

Sub Talk.To.Monster
   Call Check.Monster ' get monster number
   If Monster.Number = False Then ' check monster found
      Outpt = "You can't talk to that!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check monster number
   ' get random number of monster variable of monster talk record file
   Talk.Number = GetMonsterRecordTalk(Int(Rnd * 5 + 1))
   Outpt = Nul ' reset talk response string
   ' check bounds
   If Talk.Number > False And Talk.Number <= LOF(MonTalkFile) / Len(MonsterTalkRecord) Then
      Call Read.Record(MonTalkFile, Talk.Number) ' get the random talk string
      Outpt = RTrim$(MonsterTalkRecord.TalkMessage) ' trim talk response
   End If ' end check monster talk file bounds
   If Outpt = Nul Then ' compare to length of string
      Outpt = "The monster doesn't reply!" ' set string
   End If ' end check string length
   Call IO.O ' send response
   Call Read.Room.Record(Room) ' get current room record
   ' compare monster number to room record talk action number
   Action.Number = RoomRecord.Action ' store room action number
   ' check file bounds
   If Action.Number > False And Action.Number <= LOF(ActionFile) / Len(ActionRecord) Then
      Call Read.Record(ActionFile, Action.Number) ' read action record
      If MonsterIndex(Monster.Number) = ActionRecord.MonsterTalk Then ' compare
         Action1$ = "As you begin to speak," ' format action string one
         Action2$ = "The monster hits you for" ' format action string two
         ' routine to activate actions by specific trigger
         Call Actions(Action1$, Action2$)
      End If ' end compare room monster talk action
   End If ' end check file bounds
End Sub ' end routine to talk to monster

Rem * routine to rust some item of player.
Rem * input variables:
Rem *   Room.Rust.Rate - number of rounds for rusting.
Rem *   Rust.Rate - counter of rounds for rusting.

Sub Rust.Weapon
   If Room.Rust.Rate = False Then ' check room has rust rate
      Exit Sub ' exit routine
   End If ' end check room rust rate
   Rust.Rate = Rust.Rate + 1 ' increment rust rate counter
   If Rust.Rate < Room.Rust.Rate Then ' check rust rate counter
      Exit Sub ' exit routine
   End If ' end check rust rate counter
   Rust.Rate = False ' reset rust rate counter
   Call Rust.Inventory(Weapon4) ' routine to rust armor
   Call Rust.Inventory(Weapon5) ' routine to rust shield
   Call Rust.Inventory(Weapon6) ' routine to rust weapon
   Call Rust.Inventory(Weapon7) ' routine to rust ring
End Sub ' end rust routine

Rem * routine to rust item.
Rem * input variables:
Rem *   Weapon.Number - array index to treasure file record.

Sub Rust.Inventory (Weapon.Number)
   If Weapon.Number > False Then ' check treasure index
      ' store index to treasure file
      Treasure.Number = GetUserRecordInv(Weapon.Number)
      If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then 'compare bounds
         Call Read.Record(TreasureFile, Treasure.Number) ' get treasure record
         If TreasureRecord.Rustable = True Then ' verify item can rust
            If Rnd < (TreasureRecord.RustPercent / 100) Then ' check rust percent
               Outpt = TreasureRecord.ShortName ' store treasure name
               Outpt = RTrim$(Outpt) ' trim name
               Outpt = LCase$(Outpt) ' lowercase name
               Outpt = "Your " + Outpt + " rusts!" ' make message
               Call IO.O ' send rust message
               ' remove item from inventory
               Call Discard.Inventory(Weapon.Number, True)
            End If ' end check random rust percent
         End If ' end verify item rusts
      End If ' end compare file bounds
   End If ' end check treasure index
End Sub ' end routine to rust items

Rem * routine for monster to steal item from player inventory.
Rem * input variables:
Rem *   Room.Steal.Rate - number of rounds for stealing.
Rem *   Steal.Rate - counter of rounds for stealing.

Sub Steal.Treasure
   If Room.Steal.Rate = False Then ' check room has steal rate
      Exit Sub ' exit routine
   End If ' end check room steal rate
   Steal.Rate = Steal.Rate + 1 ' increment steal rate counter
   If Steal.Rate < Room.Steal.Rate Then ' compare steal rates
      Exit Sub ' exit routine
   End If ' end compare steal rates
   Steal.Rate = False ' reset steal rate counter
   If Number.Monsters = False Then ' check monsters in room
      Exit Sub ' exit routine
   End If ' end compare monsters in room
   For Array.Index = 1 To 20 ' loop through player inventory
      If GetUserRecordInv(Array.Index) Then ' check player has inventory
         Call Read.Record(TreasureFile, GetUserRecordInv(Array.Index)) ' get record
         If TreasureRecord.Stealable = True Then ' verify item can be stolen
            ' check random percentage item can be stolen
            If Rnd < (TreasureRecord.StealPercent / 100) Then
               For Monster.Number = 1 To Number.Monsters ' loop through monsters
                  ' check permanent
                  If MonsterArray(Monster.Number).Permanent = False Then
                     For Array.Number = 1 To 5 ' loop through monsters inventory
                        ' locate empty inventory in monster
                        If GetMonsterRecordTreasure(Array.Number) = False Then
                           ' store player inventory in monster inventory
                           Call PutMonsterRecordTreasure(Array.Number, GetUserRecordInv(Array.Index)) ' store
                           ' monster name
                           Outpt = MonsterArray(Monster.Number).MonsterName
                           Outpt = RTrim$(Outpt) ' trim name
                           Inpt = TreasureRecord.ShortName ' store treasure name
                           Inpt = RTrim$(Inpt) ' trim name
                           Inpt = LCase$(Inpt) ' lowercase name
                           Outpt = "The " + Outpt + " steals your " + Inpt + "!" ' message
                           Call IO.O ' send stolen message
                           ' remove inventory from player
                           Call Discard.Inventory(Array.Index, True) ' remove
                           Exit Sub ' exit routine
                        End If ' end locate emtpt monster inventory
                     Next ' end loop through monster inventory array
                  End If ' end check permanent monster
               Next ' end loop through monsters in room
            End If ' end random percentage for stealing inventory
         End If ' end verify item can be stolen
      End If ' end check player inventory
   Next ' end loop through player inventory
End Sub ' end routine to steal item from player

Rem * routine to offer treasure or gold to a monster to leave the room.
Rem * processing variables:
Rem *   Monster.Number - number of monster.
Rem *   UserRecord.Gold - for offering gold.

Sub Offer
   Call ParseX ' get first command parameter
   If Parser = False Then ' check parameter
      Outpt = "Offer to whom?" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check parameter
   Call Numeric ' parse parameter number from # sign
   Call Check.Monster ' get monster number
   If Monster.Number = False Then ' check monster
      Outpt = "That's not here!" ' make message
      Call IO.O ' send error message
      Exit Sub ' exit routine
   End If ' end check monster number
   Parsed.Command1 = Parsed.Command2 ' get second parameter
   Call Numeric ' parse parameter number from # sign
   Accept.Offer = False ' flag for monster to accept offer
   Player.Gold# = False ' stores amount of gold offered
   If MonsterArray(Monster.Number).Magic = False Then ' check magical monster
      If MonsterArray(Monster.Number).Permanent = False Then ' permanent monster
         Call Check.Inventory.Treasure ' parameter is item of player inventory
         If Index.Number Then ' player offers item of inventory
            Call TreasureCharges(Charges.Amount) ' get treasure item charges
            If Charges.Amount > False Then ' compare remaining charges
               If TreasureRecord.Spell Then ' check item is magical
                  Call Read.Record(SpellFile, TreasureRecord.Spell) ' get record
                  If SpellRecord.Level >= 10 Then ' verify item spell level
                     Accept.Offer = True ' set accept flag
                  End If ' end compare spell level
               Else ' check magical item
                  If TreasureRecord.Plus >= 10 Then ' compare plus of item
                     ' set accept flag for plus greater than ten
                     Accept.Offer = True
                  End If ' end check item plus
               End If ' end check magical item
            End If ' end compare charges
         Else ' compare parameter to offer
            ' convert offer to integer
            Offer.Amount# = CDbl(Int(Val(Parsed.Command1)))
            If Offer.Amount# > False Then ' check value of offer
               ' compare value offered is greater than monster level
               Monster.Amount# = CDbl(Int(MonsterArray(Monster.Number).Level * 10))
               If Offer.Amount# >= Monster.Amount# Then ' compare
                  ' compute player gold
                  Player.Gold# = UserRecord.Gold - Offer.Amount#
                  If Player.Gold# >= False Then ' check player gold
                     Accept.Offer = True ' set accept flag
                  End If ' end compare player has gold
               End If ' end compare accept gold value offer
            End If ' end check offer value
         End If ' end item offered
      End If ' end check parmanent monster
   End If ' end check magical monster
   If Accept.Offer = False Then ' compare accept flag
      Outpt = "The monster ignores your offer!" ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare accept flag
   Call Remove.Monster ' routine to take monster out of array
   If Player.Gold# Then ' check gold offered
      UserRecord.Gold = Player.Gold# ' decrement player gold value
      Outpt = "The monster takes your offer and leaves!" ' make message
      Call IO.O ' send accept message
      Exit Sub ' exit routine
   End If ' end check gold offered
   Call Discard.Inventory(Array.Number, True) ' remove item from player inventory
   Outpt = "The monster trades with you and leaves!" ' make message
   Call IO.O ' send accept message
End Sub ' end routine to offer to monster

Rem * routine to determine room direction for player to move to in panic.

Sub Panic
   Directions = False ' room direction counter
   For Direction.Number = 1 To 12 ' loop through all room directions
      If GetRoomRecordDirect(Direction.Number) Then ' get next room direction
         Call Restrict(Direction.Number, Restricted) ' routine checks valid room
         If Restricted = False Then ' compare restriction flag
            Directions = Directions + 1 ' increment direction counter
         End If ' end compare restricted room
      End If ' end check next room direction
   Next ' end loop through room directions
   If Directions = False Then ' compare direction counter
      Outpt = "There is nowhere to run! Try Appeal.." ' make panic error message
      Call IO.O ' send panic error message
      Exit Sub ' exit routine
   End If ' end compare direction counter
   ' calculate random number of directions to search from direction counter
   New.Direction = Int(Rnd * Directions + 1) ' calculate
   Direction.Counter = False ' reset search counter
   For Direction.Number = 1 To 12 ' loop through room directions
      If GetRoomRecordDirect(Direction.Number) Then ' compare room direction
         Call Restrict(Direction.Number, Restricted) ' check room restriction
         If Restricted = False Then ' check restriction flag
            Direction.Counter = Direction.Counter + 1 ' increment search counter
            If Direction.Counter = New.Direction Then ' compare counters
               ' make panic message
               Outpt = "You run away like a screaming madman!"
               Call IO.O ' send panic message
               Call Fumble ' routine for inventory fumble
               ' store next room direction
               Next.Room = GetRoomRecordDirect(Direction.Number)
               Call Enter.Room ' routine to move player to room
               Exit Sub ' exit loop through room directions
            End If ' end compare counters
         End If ' end check restriction flag
      End If ' end compare room direction
   Next ' end loop through room directions
   Outpt = "Nothing happens.." ' default message
   Call IO.O ' send message
End Sub ' end routine for player panic

Rem * routine to move player with low statistics or in a room with no exits.

Sub Appeal
   If UserRecord.Fatigue < (UserRecord.FatigueMax * .25) Then ' compare stats
      Outpt = "Your fatigue is very low.." ' make message
      Call IO.O ' send message
      Outpt = "You are teleported elsewhere!" ' make teleport message
      Call IO.O ' send teleport message
      Call Read.Room.Record(1!) ' get room record #1
      Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
      Next.Room = CSng(Resurrection.Room) ' store resurrection room
      If Next.Room = 0! Then ' check room number
         Next.Room = 1! ' store new room number
      End If ' end check room
      Teleported = True ' set teleporting flag
      Call Enter.Room ' routine moves player to room
      Exit Sub ' exit routine
   End If ' end compare low statistics
   For Direction.Number = 1 To 12 ' loop through room directions
      If GetRoomRecordDirect(Direction.Number) Then ' compare room direction
         Call Restrict(Direction.Number, Restricted) ' routine checks valid room
         If Restricted = False Then ' compare restriction flag
            Outpt = "There are exits in the room!" ' make error message
            Call IO.O ' send error message
            Exit Sub ' exit routine
         End If ' end compare restriction flag
      End If ' end compare room directions
   Next ' end loop through room directions
   For Array.Index = 1 To 20 ' loop through room objects
      Object.Number = GetRoomRecordObject(Array.Index) ' get object record
      ' file bounds
      If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
         Call Read.Record(ObjectFile, Object.Number) ' get object record
         If ObjectRecord.RoomLink > False Then ' compare object room number
            If ObjectRecord.JailTrap = False Then ' check object jail trap
               Call Restrict(12, Restricted) ' routine checks valid room to go to
               If Restricted = False Then ' compare restriction flag
                  Outpt = "There are objects with exits in the room!" ' make error message
                  Call IO.O ' send error message
                  Exit Sub ' exit routine
               End If ' end check restriction flag
            End If ' end check object jail trap
         End If ' end compare object room link number
      End If ' end check file bounds
   Next ' end loop through objects in room
   Outpt = "You are teleported elsewhere!" ' make teleport message
   Call IO.O ' send teleport message
   Call Read.Room.Record(1!) ' get room record #1
   Resurrection.Room = RoomRecord.MonsterClass ' get resurrection room number
   Next.Room = CSng(Resurrection.Room) ' store resurrection room
   If Next.Room = 0! Then ' check room number
      Next.Room = 1! ' store new room number
   End If ' end check room
   Number.Monsters = False ' reset number of monsters
   Teleported = True ' set teleporting flag
   Call Enter.Room ' routine moves player to room
End Sub ' end routine to appeal player to new room

Rem * routine to calculate bank interest.

Sub Bank.Interest
   Outpt = UserRecord.DateOn ' get player last date online
   Call Decrypt(Outpt) ' decrypt date
   DATE1$ = Left$(Outpt, 10) ' store date
   If ValidDate(DATE1$) Then ' compare date validity
      If Date$ <> DATE1$ Then ' compare date to today
         ' calculate difference
         Interest.Days = Difference(DATE1$, Date$)
         If Interest.Days > False Then ' check difference of dates
            If UserRecord.Bank > False Then ' verify players bank balance
               Total.Interest# = False ' reset total interest
               ' loop through days since last online
               For Total.Days = 1 To Interest.Days
                  ' compute interest for each day
                  Interest# = Int(UserRecord.Bank * .065)
                  ' add interest to account
                  UserRecord.Bank = UserRecord.Bank + Interest#
                  Total.Interest# = Total.Interest# + Interest# ' add interest to total
               Next ' end loop through dates
               If Total.Interest# > False Then ' compare total interest
                  ' make new balance message
                  Outpt = "The bank reports interest posting of" + Str$(Total.Interest#) + " gold to your account!"
                  Call IO.O ' send display message
               End If ' end compare interest
            End If ' end verify bank balance
            If UserRecord.Borrow > False Then ' verify players bank loan
               Total.Interest# = False ' reset total interest
               ' loop through days since last online
               For Total.Days = 1 To Interest.Days
                  ' compaute interest for each day
                  Interest# = Int(UserRecord.Borrow * .075)
                  ' add interest to loan
                  UserRecord.Borrow = UserRecord.Borrow + Interest#
                  Total.Interest# = Total.Interest# + Interest# ' add interest to total
               Next ' end loop through dates
               If Total.Interest# > False Then ' compare total interest
                  ' make new balance message
                  Outpt = "The bank reports interest posting of" + Str$(Total.Interest#) + " gold to your loan!"
                  Call IO.O ' send display message
               End If ' end compare interest
            End If ' end verify loan balance
         End If ' end check difference of dates
      End If ' end compare todays date
   End If ' end compare validity
   Outpt = Date$ ' store todays date
   Call Valid(Outpt, 10) ' validate date string
   Call Encrypt(Outpt, True) ' encrypt date string
   UserRecord.DateOn = Outpt ' store new date in player record
   Call Share.Record(UserFile, User.Index) ' routine to write player record
End Sub ' end routine to calculate bank balances

Rem * get days difference between two dates.

Function Difference (startdate$, enddate$)

   Year1 = Val(Mid$(startdate$, 7, 4))
   Year2 = Val(Mid$(enddate$, 7, 4))

   dayselapsed1 = 0
   For x = 1980 To Year1 - 1
      dayselapsed1 = dayselapsed1 + 365 + IsLeapYear(x)
   Next

   Month1 = Val(Mid$(startdate$, 1, 2))
   Month2 = Val(Mid$(enddate$, 1, 2))

   For x = 1 To Month1 - 1
      Select Case x
         Case 1
            dayselapsed1 = dayselapsed1 + 31
         Case 2
            dayselapsed1 = dayselapsed1 + 28 + IsLeapYear(Year1)
         Case 3
            dayselapsed1 = dayselapsed1 + 31
         Case 4
            dayselapsed1 = dayselapsed1 + 30
         Case 5
            dayselapsed1 = dayselapsed1 + 31
         Case 6
            dayselapsed1 = dayselapsed1 + 30
         Case 7
            dayselapsed1 = dayselapsed1 + 31
         Case 8
            dayselapsed1 = dayselapsed1 + 31
         Case 9
            dayselapsed1 = dayselapsed1 + 30
         Case 10
            dayselapsed1 = dayselapsed1 + 31
         Case 11
            dayselapsed1 = dayselapsed1 + 30
         Case 12
            dayselapsed1 = dayselapsed1 + 31
      End Select
   Next

   Day1 = Val(Mid$(startdate$, 4, 2))
   Day2 = Val(Mid$(enddate$, 4, 2))

   dayselapsed1 = dayselapsed1 + Day1

   dayselapsed2 = 0
   For x = 1980 To Year2 - 1
      dayselapsed2 = dayselapsed2 + 365 + IsLeapYear(x)
   Next

   For x = 1 To Month2 - 1
      Select Case x
         Case 1
            dayselapsed2 = dayselapsed2 + 31
         Case 2
            dayselapsed2 = dayselapsed2 + 28 + IsLeapYear(Year2)
         Case 3
            dayselapsed2 = dayselapsed2 + 31
         Case 4
            dayselapsed2 = dayselapsed2 + 30
         Case 5
            dayselapsed2 = dayselapsed2 + 31
         Case 6
            dayselapsed2 = dayselapsed2 + 30
         Case 7
            dayselapsed2 = dayselapsed2 + 31
         Case 8
            dayselapsed2 = dayselapsed2 + 31
         Case 9
            dayselapsed2 = dayselapsed2 + 30
         Case 10
            dayselapsed2 = dayselapsed2 + 31
         Case 11
            dayselapsed2 = dayselapsed2 + 30
         Case 12
            dayselapsed2 = dayselapsed2 + 31
      End Select
   Next

   dayselapsed2 = dayselapsed2 + Day2
   Difference = dayselapsed2 - dayselapsed1
End Function

Rem * function returns 1 if year is leap year.

Function IsLeapYear (Var)
   leap = 0
   If (Var Mod 4) = 0 Then
      leap = -1
   End If
   If (Var Mod 100) = 0 Then
      leap = 0
   End If
   If (Var Mod 400) = 0 Then
      leap = -1
   End If
   If leap Then
      IsLeapYear = 1
   End If
End Function

' Def FNclock$=Format$(Now,"ddd dd mmm yy\, hh:mma/p")
Function clock$
   ' mm-dd-yyyy hh:mm:ss

   M = Val(Left$(Date$, 2))
   D = Val(Mid$(Date$, 4, 2))
   Y = Val(Right$(Date$, 2))
   If M < 3 Then M = M + 12: Y = Y - 1
   W = ((13 * M + 3) \ 5 + D + Y + Y \ 4 - Y \ 100 + Y \ 400 + 1) Mod 7
   Var$ = Mid$("SunMonTueWedThuFriSat", W * 3 + 1, 3)
   Var$ = Var$ + " " + Mid$(Date$, 4, 2) ' dd

   T = (Val(Mid$(Date$, 1, 2)) - 1) * 3 + 1
   Var$ = Var$ + " " + Mid$("JanFebMarAprMayJunJulAugSepOctNovDec", T, 3)

   Var$ = Var$ + " " + Mid$(Date$, 9, 2) ' yy
   T = Val(Left$(Time$, 2))
   If T > 11 Then T = T - 12
   If T = 0 Then T = 12
   Var$ = Var$ + ", " + Right$("00" + LTrim$(Str$(T)), 2) ' hh
   Var$ = Var$ + ":" + Mid$(Time$, 4, 2) ' mm

   T = Val(Left$(Time$, 2))
   If T < 12 Then
      Var$ = Var$ + "a"
   Else
      Var$ = Var$ + "p"
   End If
   clock$ = Var$
End Function

Rem * routine to process bank commands.

Sub Bank
   Call Put.User.Record ' routine to write user record
   Do ' loop through bank menu
      Graphics.Off = False ' reset color
      Outpt = "Bank menu options:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]ccount" ' make display message
      Call IO.O ' send message
      Outpt = "[B]orrow" ' make display message
      Call IO.O ' send message
      Outpt = "[D]eposit" ' make display message
      Call IO.O ' send message
      Outpt = "[P]ayback" ' make display message
      Call IO.O ' send message
      Outpt = "[T]ransfer" ' make display message
      Call IO.O ' send message
      Outpt = "[W]ithdraw" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "The broker asks: What can I do for you(q to quit)? " ' input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of bank option
         Case "A" ' option to display bank balances
            Outpt = "He fumbles with his ledgers and says:" ' make display message
            Call IO.O ' send message
            Graphics.Off = True ' reset color
            Outpt = "The current account interest rate is 6.5 percent." ' message
            Call IO.O ' send message
            Outpt = "The current lending interest rate is 7.5 percent." ' message
            Call IO.O ' send message
            ' make display message
            Outpt = "You have " + FormatString$(UserRecord.Gold) + " gold and " + FormatString$(UserRecord.Bank) + " in the bank."
            Call IO.O ' send message
            Outpt = "You have borrowed " + FormatString$(UserRecord.Borrow) + " gold from the bank." ' make display message
            Call IO.O ' send message
         Case "B" ' option to borrow gold from bank
            Outpt = "How much will you borrow(1-32767)? " ' make input prompt
            Call IO.I ' get user input
            Borrow.Amount# = CDbl(Int(Val(Inpt))) ' convert input to double
            ' compare input range
            If Borrow.Amount# <= False Or Borrow.Amount# > MaxInt Then
               Outpt = "The broker says: You can't borrow that much!" ' make message
               Call IO.O ' send message
            Else ' check input range
               Outpt = "The broker hands you the gold." ' make display message
               Call IO.O ' send message
               ' add gold to player gold
               UserRecord.Gold = UserRecord.Gold + Borrow.Amount#
               ' add gold to player loan
               UserRecord.Borrow = UserRecord.Borrow + Borrow.Amount#
            End If ' end check input range
         Case "D" ' option to deposit gold in bank
            If UserRecord.Gold <= False Then ' check player gold
               Outpt = "You have no gold to deposit!" ' make display message
               Call IO.O ' send message
            Else ' check player gold
               Outpt = "How much(1-" + Mid$(Str$(UserRecord.Gold), 2) + ")? " ' prompt
               Call IO.I ' get user input
               Deposit.Amount# = CDbl(Int(Val(Inpt))) ' convert input to double
               ' compare input range
               If Deposit.Amount# <= False Or Deposit.Amount# > UserRecord.Gold Then
                  Outpt = "You don't have that much gold!" ' make display message
                  Call IO.O ' send message
               Else ' check input range
                  ' add gold to bank account
                  UserRecord.Bank = UserRecord.Bank + Deposit.Amount#
                  ' subtract gold from player
                  UserRecord.Gold = UserRecord.Gold - Deposit.Amount#
                  Outpt = "You hand him the gold." ' make display message
                  Call IO.O ' send message
               End If ' end check input range
            End If ' end check player gold
         Case "P" ' option to pay back loan
            If UserRecord.Borrow <= False Then ' check player loan
               Outpt = "You don't have any loan with the bank!" ' make message
               Call IO.O ' send message
            Else ' check loan
               If UserRecord.Bank > False Then ' check player bank account
                  Outpt = "The broker asks: Pay back with your current bank account? "
                  No.Input.Out = "Y" ' default input
                  Call IO.I ' get user input
                  If Yes Then ' check input response
                     ' compute difference between player bank account and loan
                     If UserRecord.Borrow - UserRecord.Bank <= False Then ' compute
                        ' subtract loan from player bank account
                        UserRecord.Bank = UserRecord.Bank - UserRecord.Borrow ' subtract
                        UserRecord.Borrow = False ' reset loan amount
                        Outpt = "The broker says: Your loan is paid off!" ' message
                        Call IO.O ' send message
                     Else ' check computation
                        ' subtract player bank account from loan
                        UserRecord.Borrow = UserRecord.Borrow - UserRecord.Bank
                        UserRecord.Bank = False ' reset player bank account
                        Outpt = "The broker says: Your balance paid part of the loan!"
                        Call IO.O ' send display message
                     End If ' end check loan/bank account difference
                  End If ' end check user input
               End If ' end check player bank account
               If UserRecord.Borrow > False Then ' check player loan amount
                  If UserRecord.Gold > False Then ' check player gold
                     Outpt = "The broker asks: Pay back with your current gold? "
                     No.Input.Out = "Y" ' default input
                     Call IO.I ' get user input
                     If Yes Then ' check input response
                        ' compute difference between player gold and loan
                        If UserRecord.Borrow - UserRecord.Gold <= False Then ' compute
                           ' subtract loan from player gold
                           UserRecord.Gold = UserRecord.Gold - UserRecord.Borrow
                           UserRecord.Borrow = False ' reset loan
                           Outpt = "The broker says: Your loan is paid off!" ' message
                           Call IO.O ' send message
                        Else ' check computation
                           ' subtract player gold from loan
                           UserRecord.Borrow = UserRecord.Borrow - UserRecord.Gold
                           UserRecord.Gold = False ' reset player gold
                           Outpt = "The broker says: Your gold paid part of the loan!"
                           Call IO.O ' send display message
                        End If ' end loan/player gold difference
                     End If ' end check user input
                  End If ' end check player gold
               End If ' end check player loan
            End If ' end check player loan
         Case "T" ' option to tranfer bank funds to another player
            If UserRecord.Bank <= False Then ' check player bank account
               Outpt = "You have nothing in your account to transfer!" ' make message
               Call IO.O ' send message
            Else ' check player bank account
               Outpt = "How much(1-" + Mid$(Str$(UserRecord.Bank), 2) + ")? " ' prompt
               Call IO.I ' get user input
               Transfer.Amount# = CDbl(Int(Val(Inpt))) ' convert input to double
               ' check input range
               If Transfer.Amount# <= False Or Transfer.Amount# > UserRecord.Bank Then
                  Outpt = "You don't have that much gold in the bank!" ' make message
                  Call IO.O ' send message
               Else ' check input range
                  Call Share.Record(UserFile, User.Index) ' write current user record
                  Outpt = "Transfer to what player? " ' make input prompt
                  Call IO.I ' get user input
                  Inpt = UCase$(Inpt) ' uppercase input
                  Inpt = RTrim$(Inpt) ' trim input
                  Player.Found = False ' reset player found flag
                  For User.Number = 1 To LOF(UserFile) / Len(UserRecord) ' loop file
                     Call Read.Record(UserFile, User.Number) ' read user record
                     Outpt = UserRecord.CodeName ' store codename
                     Call Decrypt(Outpt) ' decrypt name
                     Outpt = RTrim$(Outpt) ' trim name
                     If Outpt <> Deleted$ Then ' compare deleted codename
                        If Outpt <> Dead$ Then ' compare dead codename
                           If Outpt = Inpt Then ' compare codename to transfer
                              If User.Index <> User.Number Then ' check indexes
                                 If (UserRecord.Flags And Locked.User) = False Then
                                    Player.Found = True ' set player found flag
                                    ' add amount to tranfer to player account
                                    UserRecord.Bank = UserRecord.Bank + Transfer.Amount#
                                    Call Share.Record(UserFile, User.Number) ' write
                                    Exit For ' exit loop through user file
                                 End If ' end check locked user record
                              End If ' end check indexes
                           End If ' end check codename
                        End If ' end check dead codename
                     End If ' end check deleted codename
                  Next ' end loop through user file records
                  If Player.Found Then ' check player found flag
                     Outpt = "The broker works with his ledgers for a while,"
                     Call IO.O ' send display message
                     Outpt = "And says: " + Mid$(Str$(Transfer.Amount#), 2) + " gold transferred to his account."
                     Call IO.O ' send display message
                  Else ' check player found flag
                     Transfer.Amount# = False ' reset gold tranfered
                     Outpt = "There's nobody in my ledgers with that name!"
                     Call IO.O ' send display message
                  End If ' end check player found flag
                  Call Read.Record(UserFile, User.Index) ' read current user record
                  ' subtract amount tranfered
                  UserRecord.Bank = UserRecord.Bank - Transfer.Amount#
               End If ' end check input range
            End If ' end check player bank account
         Case "W" ' option to withdraw gold from bank account
            If UserRecord.Bank <= False Then ' check bank account
               Outpt = "You have nothing to withdraw!" ' make display message
               Call IO.O ' send message
            Else ' check bank account
               If UserRecord.Borrow > False Then ' check bank loan
                  Outpt = "You must pay back your loan first!" ' make display message
                  Call IO.O ' send message
               Else ' check bank loan
                  Outpt = "How much(1-" + Mid$(Str$(UserRecord.Bank), 2) + ")? " ' prompt
                  Call IO.I ' get user input
                  Withdraw.Amount# = CDbl(Int(Val(Inpt))) ' convert input to double
                  If Withdraw.Amount# <= False Or Withdraw.Amount# > UserRecord.Bank Then ' check input range
                     Outpt = "You don't have that much gold!" ' make display message
                     Call IO.O ' send message
                  Else ' check input range
                     ' subtract from bank
                     UserRecord.Bank = UserRecord.Bank - Withdraw.Amount#
                     UserRecord.Gold = UserRecord.Gold + Withdraw.Amount# ' add to gold
                     Outpt = "The broker hands you the gold." ' make message
                     Call IO.O ' send message
                  End If ' end check input range
               End If ' end check bank loan
            End If '  end check bank account
         Case "Q" ' option to exit bank menu
            Exit Do ' exit bank menu
      End Select ' end selection of bank option
   Loop ' end loop through bank menu
End Sub ' end bank routine

Rem * Filename: dnds6.bas Version: v5.0a r2.0a
Rem * This subprogram contains mail and some find routines.

Rem * main routine to enter mail and process mail selections.

Sub Mail
   ReDim Array(1 To 19) As String ' dimension message array
   Do ' loop through mail commands
      Graphics.Off = True ' reset color
      Outpt = "[E]nter" ' make display message
      Call IO.O ' send message
      Outpt = "[K]ill" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ock" ' make display message
      Call IO.O ' send message
      Outpt = "[N]ew" ' make display message
      Call IO.O ' send message
      Outpt = "[R]ead" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Mail command(q to quit)? " ' input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get player input
      Select Case UCase$(Inpt) ' selection of input
         Case "E" ' option to enter message
            Call Enter.Message ' routine to enter message
         Case "K" ' option to kill message
            Call Kill.Message ' routine to kill message
         Case "L" ' option to toggle locked mailbox
            Call Lock.Mailbox ' routine to toggle locked mailbox
         Case "N" ' option to read new messages
            Call New.Messages ' routine to read new messages
         Case "R" ' option to read messages
            Call Read.Messages ' routine to read messages
         Case "Q" ' option to exit mail menu
            Exit Do ' exit mail menu
      End Select ' end mail option selection
   Loop ' end loop through mail menu
End Sub ' end mail routine

Rem * routine to enter a message.
Rem * output variables:
Rem *   Array - array containing message.
Rem *   Message.Length - number of message text lines.
Rem *   Message.To - codename or null of player message is to.
Rem *   Private.Message - true if message is private.
Rem *   Subject - contains subject of message.

Sub Enter.Message
   If LOF(TableFile) / Len(TableRecord) = MaxInt Then ' check table size
      Outpt = "The message table is full." ' make error output message
      Call IO.O ' send error output
      Exit Sub ' exit routine
   End If ' end check table size
   Call Share.Record(UserFile, User.Index) ' routine to store player record
   Do ' loop until message to stored
      Graphics.Off = True ' reset color
      Private.Message = False ' reset private message flag
      Outpt = "Message to(Press <enter> for ALL)? " ' input prompt
      Call IO.I ' get player input
      Message.To = UCase$(Inpt) ' uppercase and store input
      If Inpt = Nul Then ' compare null input
         Exit Do ' exit loop with message to all
      End If ' end compare input length
      For User.Number = 1 To LOF(UserFile) / Len(UserRecord) ' loop through user file
         Call Read.Record(UserFile, User.Number) ' read next player record
         Outpt = UserRecord.CodeName ' store codename
         Call Decrypt(Outpt) ' routine to decrypt codename
         Outpt = RTrim$(Outpt) ' trim codename
         Outpt = UCase$(Outpt) ' uppercase codename
         If Message.To = Outpt Then ' compare codename to input
            If User.Index <> User.Number Then ' check not self
               If (UserRecord.Flags And Locked.User) = False Then ' check locked
                  Outpt = "Private message(y/n)? " ' input prompt
                  No.Input.Out = "N" ' default input
                  Call IO.I ' get player input
                  Private.Message = Yes ' store private message flag
                  Exit Do ' exit loop until message to found
               End If ' end check locked player mailbox
            End If ' end check player indexes
         End If ' end check codename
      Next ' end loop through player file
      Outpt = "There is no such user." ' make display message
      Call IO.O ' send message
   Loop ' end loop until message to stored
   Call Read.Record(UserFile, User.Index) ' reread player record
   Do ' loop until subject entered
      Outpt = "Subject? " ' input prompt
      Call IO.I ' get player input
      Inpt = LTrim$(Inpt) ' trim input
      Inpt = RTrim$(Inpt) ' trim input
      Inpt = LCase$(Inpt) ' lowercase input
      If Inpt <> Nul Then ' check input length
         Subject = Inpt ' store input in subject string
         Exit Do ' exit subject loop
      End If ' end check input length
      Outpt = "Abort message(y/n)? " ' input prompt
      No.Input.Out = "N" ' default input
      Call IO.I ' get player input
      If Yes Then ' check input response
         Exit Sub ' exit message enter routine
      End If ' end check input response
   Loop ' end loop until subject entered
   Message.Reply = False
   Outpt = "Enter message." ' make message
   Call IO.O ' send message
   Call Write.Message ' routine to enter and store message
End Sub ' end routine to enter a message

Rem * routine to write message.

Sub Write.Message
   Message.Length = False ' reset message length entered
   User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
   UserRecord.Wordwrap = False ' reset word wrap
   Do ' loop until message entered
      Outpt = "Press <enter> on a blank line to edit." ' make message
      Call IO.O ' send message
      Do ' loop until blank line entered
         Graphics.Off = True ' reset color
         If Message.Length = 19 Then ' compare length of message entered
            Outpt = "Message buffer full." ' make buffer message
            Call IO.O ' send message
            Exit Do ' exit message entry loop
         End If ' end compare message length
         Word.Wrap = True ' set word wrap flag
         Outpt = "?" ' input prompt
         Call IO.I ' get player input
         If No.Input Then ' check blank line entered
            Exit Do ' exit message entry loop
         End If ' end check blank line
         Message.Length = Message.Length + 1 ' increment message length counter
         Inpt = RTrim$(Inpt) ' trim input
         Inpt = Left$(Inpt, 79) ' truncate input
         Array(Message.Length) = Inpt ' store next message line
      Loop ' end loop until blank line entered
      Word.Wrap = False ' reset word wrap flag
      UserRecord.Wordwrap = User.Word.Wrap ' restore user word wrap
      Call Edit.Message(Message.Edit) ' routine to edit message text
      ' selection of return variable from message editing
      Select Case Message.Edit
         Case True ' true returned
            Outpt = "Continue editing." ' make display message
            Call IO.O ' send message
         Case False ' false returned
            Exit Sub ' message aborted
         Case UnTrue ' other returned
            Call Store.Message ' message stored
            Exit Sub ' exit message entry routine
      End Select ' end selection of edit return variable
   Loop ' end loop until message entered
End Sub

Rem * routine to lock/unlock player mailbox.

Sub Lock.Mailbox
   ' check player locked mailbox
   If (UserRecord.Flags And Locked.User) = False Then
      Outpt = "Mailbox locked." ' make locked message
      UserRecord.Flags = UserRecord.Flags Or Locked.User ' set locked player flag
   Else ' check player locked
      Outpt = "Mailbox unlocked." ' make locked message
      UserRecord.Flags = UserRecord.Flags And Not Locked.User ' reset locked flag
   End If ' end check player locked flag
   Call IO.O ' send message
   Call Share.Record(UserFile, User.Index) ' routine to wite player record
End Sub

Rem * routine to delete message.

Sub Kill.Message
   Max.Messages = LOF(TableFile) / Len(TableRecord) ' store length of table file
   Outpt = "Enter message number to delete" ' make range input prompt
   Call Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
   If Message.Number > False Then ' check number bounds
      Call Read.Record(TableFile, Message.Number) ' read table record
      Outpt = UserRecord.CodeName ' store codename
      Call Decrypt(Outpt) ' decrypt codename
      Outpt = RTrim$(Outpt) ' trim codename
      Outpt = UCase$(Outpt) ' uppercase codename
      Inpt = TableRecord.ToUser ' store message from codename
      Inpt = RTrim$(Inpt) ' trim message codename
      Inpt = UCase$(Inpt) ' uppercase from codename
      If Outpt <> Inpt Then ' compare codenames
         Outpt = "Message" + Str$(Message.Number) + " is not a private message to you."
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare codenames
      If TableRecord.Private = False Then
         Outpt = "Message" + Str$(Message.Number) + " is not a private message to you."
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare codenames
      If TableRecord.Killed Then ' compare deleted message
         Outpt = "Message" + Str$(Message.Number) + " is already deleted." ' error message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end compare deleted message
      TableRecord.Killed = True ' set deleted message flag in table record
      Call Share.Record(TableFile, Message.Number) ' routine to write table record
      Outpt = "Message" + Str$(Message.Number) + " deleted." ' make deleted message
      Call IO.O ' send message
   End If ' end check number bounds
End Sub ' end routine to delete a message

Rem * routine to display message.
Rem * input variables:
Rem *   Message.Number - contains number of message to display.
Rem *   ListLines - true to list line numbers

Sub Read.Message (Message.Number, ListLines)
   Call Message.Header(Message.Number) ' routine to display message header
   Allow.Break = True ' set control-k checking flag
   Break = False ' reset control-k flag
   Continue = False ' reset continuous flag
   Page.Length = 5 ' reset page length counter
   ' store beginning of message record flag
   Message.Start! = TableRecord.Start
   ' store ending of message record flag
   Message.End! = Message.Start! + CSng(TableRecord.Length - 1)
   ' loop through message contents
   Line.Number = False ' reset line counter
   For Message.Record! = Message.Start! To Message.End!
      Call Read.Message.Record(MessageFile, Message.Record!) ' read message text line
      Outpt = MessageRecord.Message ' store message text
      Call Decrypt(Outpt) ' decrypt message text
      Outpt = RTrim$(Outpt) ' trim message
      Line.Number = Line.Number + 1 ' increment line counter
      If ListLines Then ' check line number list flag
         Outpt = Mid$(Str$(Line.Number), 2) + ":" + Outpt ' make line number
      End If ' end check line number flag
      Outpt = Left$(Outpt, 79) ' truncate message text
      Call IO.O ' send message text line
      If Break Then ' check control-k pressed flag
         Exit For ' exit message display loop
      End If ' end check control-k pressed
      Page.Length = Page.Length + 1 ' increment page length counter
      If Page.Length = UserRecord.Pagelength Then ' check page length
         Page.Length = False ' reset page length
         If Continue = False Then ' check continuous flag
            Call More.Prompt ' routine to pause
            If No Then ' check pause response
               Exit For ' exit message display loop
            End If ' end check pause response
         End If ' end check continuous flag
      End If ' end check page length
   Next ' end loop through message file
   Allow.Break = False ' reset control-k checking flag
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
End Sub ' end routine to display a message

Rem * routine to display message header.
Rem * input variables:
Rem *   Message.Number - contains number of message.

Sub Message.Header (Message.Number)
   ' make header
   Call Sub.Header(Message.Number) ' routine displays first part of header
   Outpt = "Rcvd:  " ' make header
   If TableRecord.Received Then ' check message
      Outpt = Outpt + "Yes" ' append header
   Else ' check message
      Outpt = Outpt + "No" ' append header
   End If ' end check message
   Outpt = Outpt + Space$(35 - Len(Outpt)) ' pad spaces
   Outpt = Outpt + "Read:" + Str$(TableRecord.TimesRead) + " times"
   Call IO.O ' send more header
   Outpt = "Reply: " ' make header
   If TableRecord.Reply Then ' check message type
      Outpt = Outpt + "Yes" ' append header
   Else ' check message
      Outpt = Outpt + "No" ' append header
   End If ' end check message type
   Outpt = Outpt + Space$(35 - Len(Outpt)) ' pad spaces
   Outpt = Outpt + "Thread:" ' append more header
   If TableRecord.Reply Then ' check message type again
      If TableRecord.Thread = False Then ' check message thread number
         Outpt = Outpt + "(forward)" ' append default header
      Else ' check thread number
         Outpt = Outpt + Str$(TableRecord.Thread) ' append thread number
      End If ' end check message type
   Else ' check message type
      Outpt = Outpt + " None" ' append more header
   End If ' end check message
   Call IO.O ' send header
   Outpt = "Stat:  " ' make more header
   Class.Number = TableRecord.ClassType ' store class type from
   Select Case Class.Number ' select class type
      Case 1 To 10 ' check class type range
         Outpt = Outpt + RTrim$(Class.Name(Class.Number)) ' store class name
   End Select ' end check class type range
   Outpt = Left$(Outpt, 35) ' trunacte more header
   Outpt = Outpt + Space$(35 - Len(Outpt)) ' pad spaces
   Outpt = Outpt + "Type: " ' append header
   Inpt = Nul ' reset message type from
   If TableRecord.Flags And Special.Char1 Then ' check message from
      Inpt = Inpt + "Town Mayor " ' append class type from
   End If ' end check class type message from
   If TableRecord.Flags And Special.Char2 Then ' check message from
      Inpt = Inpt + "Governor " ' append class type from
   End If ' end check class type message from
   If TableRecord.Flags And Special.Char3 Then ' check message from
      Inpt = Inpt + "Guild Master " ' append class type from
   End If ' end check class type message from
   If TableRecord.Flags And Special.Char4 Then ' check message from
      Inpt = Inpt + "Sysop " ' append class type from
   End If ' end check class type message from
   If Inpt = Nul Then ' check message from
      Inpt = "None" ' reset class type from
   End If ' end check class type message from
   Outpt = Outpt + Inpt ' append more header
   Outpt = Left$(Outpt, 79) ' truncate header
   Outpt = RTrim$(Outpt) ' trim header
   Call IO.O ' send more header
End Sub ' end routine to display message header

Rem * routine displays first part of message header
Rem * input variables:
Rem *   Message.Number - number of message being read.

Sub Sub.Header (Message.Number)
   Graphics.Off = False ' reset color
   Outpt = "Msg#:" + Str$(Message.Number) + " of" + Str$(LOF(TableFile) / Len(TableRecord))
   If TableRecord.Private Then ' check private message
      Outpt = Outpt + "(private)" ' append to header
   End If ' end check message
   If TableRecord.Killed Then ' check deleted message
      Outpt = Outpt + "(deleted)" ' append to header
   End If ' end check message
   Call IO.O ' send header
   Graphics.Off = True ' reset color
   Inpt = TableRecord.From ' make header
   Inpt = LCase$(Inpt) ' lowercase header
   Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' uppercase first character
   Outpt = "From:  " + Inpt ' make header
   Outpt = Left$(Outpt, 35) ' truncate hader
   Outpt = Outpt + Space$(35 - Len(Outpt)) ' pad spaces
   Inpt = RTrim$(TableRecord.Subject) ' make more header
   Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' uppercase more header
   Outpt = Outpt + "Subj: " + Inpt ' append header
   Call IO.O ' send more header
   Inpt = TableRecord.ToUser ' make more header
   Inpt = RTrim$(Inpt) ' trim header
   If Inpt = Nul Then ' check null message to
      Inpt = "ALL" ' default to all
   End If ' end check message to
   Inpt = LCase$(Inpt) ' lowercase header
   Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' uppercase header
   Outpt = "To:    " + Inpt ' make more header
   Outpt = Left$(Outpt, 35) ' truncate more header
   Outpt = Outpt + Space$(35 - Len(Outpt)) ' pad spaces
   Outpt = Outpt + "Time: " + TableRecord.Clock ' append header
   Call IO.O ' send more header
End Sub ' end routine to display subheader

Rem * routine to read range of messages.

Sub Read.Messages
   Code.Name$ = UserRecord.CodeName ' store codename
   Call Decrypt(Code.Name$) ' decrypt codename
   Code.Name$ = RTrim$(Code.Name$) ' trim codename
   Code.Name$ = UCase$(Code.Name$) ' uppercase codename
   Max.Messages = LOF(TableFile) / Len(TableRecord) ' store length of table file
   Outpt = "Starting message number" ' make range input prompt
   Call Get.Range2(1, Max.Messages, Message.Number) ' routine to get number
   Do ' message read loop
      If Message.Number > LOF(TableFile) / Len(TableRecord) Then ' compare table end
         Exit Do ' exit message read loop
      End If ' end compare table
      Call Read.Record(TableFile, Message.Number) ' read next table record
      Inpt = TableRecord.ToUser ' store message from
      Inpt = RTrim$(Inpt) ' trim message from
      Inpt = UCase$(Inpt) ' uppercase message from
      If TableRecord.Private And Inpt <> Code.Name$ Then ' compare private message
         Outpt = "Message #" + Mid$(Str$(Message.Number), 2) + " is private." ' message
         Call IO.O ' send message
      Else ' compare private message, codenames
         If TableRecord.Killed Then ' compare deleted message
            Outpt = "Message #" + Mid$(Str$(Message.Number), 2) + " was deleted."
            Call IO.O ' send message
         Else ' compare deleted message
            Call Read.Message(Message.Number, False) ' routine to display message
            TableRecord.Received = True ' set message recieved flag
            TableRecord.TimesRead = TableRecord.TimesRead + 1 ' increment times read
            Call Share.Record(TableFile, Message.Number) ' write table record
         End If ' end compare deleted message
      End If ' end compare private message
      Graphics.Off = False ' reset color
      If Message.Number > UserRecord.LastMessage Then ' check last message pointer
         UserRecord.LastMessage = Message.Number ' store new last message read
      End If ' end check last message read
      Outpt = "Read more (y)es/(n)o/(r)eply/(t)hread, msg number? " ' make prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get player input
      Select Case UCase$(Inpt) ' selection of prompt response
         Case "N", "Q" ' check input response
            Exit Do ' exit message read loop
         Case "R" ' reply to message
            Outpt = "Private message(y/n)? " ' prompt
            No.Input.Out = "N"
            Call IO.I ' get input
            Private.Message = Yes ' set message type
            Message.Reply = True ' set message type
            Message.Thread = Message.Number ' store message number
            Message.To = TableRecord.From ' store message header
            Subject = TableRecord.Subject ' store message header
            Outpt = "Enter message reply." ' make message
            Call IO.O ' send message
            Call Write.Message ' routine to enter and write message
            Call Read.Record(TableFile, Message.Number) ' read current table
            TableRecord.Reply = True ' set reply
            Call Share.Record(TableFile, Message.Number) ' write table record
            Message.Number = Message.Number - 1 ' adjust loop variable
         Case "T" ' thread selection
            Outpt = "Direction (b)ackward/(f)oward/(s)earch? " ' input prompt
            No.Input.Out = "F" ' default input
            Call IO.I ' get input
            Select Case UCase$(Inpt) ' selection of thread direction
               Case "B" ' backward thread
                  If TableRecord.Thread = False Then ' check thread
                     Outpt = "Message has no backward reply thread." ' make message
                     Call IO.O ' send message
                     Call More.Prompt ' pause prompt
                     Message.Number = Message.Number - 1 ' reset loop variable
                  Else ' check thread
                     ' change loop variable to thread number
                     Message.Number = TableRecord.Thread - 1
                  End If ' end check thread number
               Case "F" ' forward thread
                  ' loop through table
                  For Message.Search = Message.Number To LOF(TableFile) / Len(TableRecord)
                     Call Read.Record(TableFile, Message.Search) ' read table record
                     ' compare thread numbers
                     If TableRecord.Thread = Message.Number Then
                        Message.Number = Message.Search - 1 ' change loop variable
                        Exit For ' exit search loop
                     End If ' end compare thread numbers
                  Next ' end loop through table
                  If Message.Search > LOF(TableFile) / Len(TableRecord) Then ' check bounds
                     Outpt = "Message has no forward thread." ' make message
                     Call IO.O ' send message
                     Call More.Prompt ' pause prompt
                     Message.Number = Message.Number - 1 ' reset loop variable
                  End If ' end check loop variable
               Case "S" ' search forward thread
                  ' loop through table
                  For Message.Search = Message.Number To LOF(TableFile) / Len(TableRecord)
                     Call Read.Record(TableFile, Message.Search) ' read table record
                     ' compare thread numbers
                     If TableRecord.Thread = Message.Number Then
                        Call Sub.Header(Message.Search) ' routine displays subheader
                        Graphics.Off = False ' reset color
                        Outpt = "Message thread:(c)ontinue search/(r)ead? " ' prompt
                        No.Input.Out = "R" ' default input
                        Call IO.I ' get user input
                        If UCase$(Inpt) = "R" Then ' compare input
                           Message.Number = Message.Search - 1 ' change loop variable
                           Exit For ' exit search loop
                        End If ' end compare input
                     End If ' end compare thread numbers
                  Next ' end loop through table
                  If Message.Search > LOF(TableFile) / Len(TableRecord) Then ' check bounds
                     Outpt = "There are no more forward threads." ' make message
                     Call IO.O ' send message
                     Call More.Prompt ' pause prompt
                     Message.Number = Message.Number - 1 ' reset loop variable
                  End If ' end check loop variable
            End Select ' end thread selection
         Case Else ' other selection
            Next.Message = Int(Val(Inpt)) ' convert input to integer
            ' check input range
            If Next.Message > False And Next.Message <= LOF(TableFile) / Len(TableRecord) Then
               Message.Number = Next.Message - 1 ' reset message loop variable
            End If ' end check input range
      End Select ' end prompt selection
      Message.Number = Message.Number + 1 ' increment table number
   Loop ' end loop through message table
   Outpt = "End of messages." ' make message
   Call IO.O ' send message
End Sub ' end routine to read messages

Rem * routine to read range of new messages.

Sub New.Messages
   If UserRecord.LastMessage >= LOF(TableFile) / Len(TableRecord) Then ' compare last
      Outpt = "No new messages." ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end compare last message read variable to length of table
   Code.Name$ = UserRecord.CodeName ' store codename
   Call Decrypt(Code.Name$) ' decrypt codename
   Code.Name$ = RTrim$(Code.Name$) ' trim codename
   Code.Name$ = UCase$(Code.Name$) ' uppercase codename
   Message.Number = UserRecord.LastMessage + 1 ' store last message read variable
   Last.Message = LOF(TableFile) / Len(TableRecord) ' store length of table
   Do ' message read loop
      If Message.Number > Last.Message Then ' compare message range
         Exit Do ' exit message loop
      End If ' end compare message range
      Call Read.Record(TableFile, Message.Number) ' read next table record
      Inpt = TableRecord.ToUser ' store message to
      Inpt = RTrim$(Inpt) ' trim message to
      Inpt = UCase$(Inpt) ' uppercase message to
      Message.Read = True ' set read message flag
      If TableRecord.Private Then ' check private message flag
         If Inpt <> Code.Name$ Then ' compare codenames
            Message.Read = False ' reset read message flag
         End If ' end compare codenames
      End If ' end check private message flag
      If TableRecord.Killed Then ' check deleted message
         Message.Read = False ' reset read message flag
      End If ' end check deleted message
      If Message.Read Then ' verify read message flag
         Call Read.Message(Message.Number, False) ' routine to display message
      End If ' end verify read message flag
      Graphics.Off = False ' reset color
      ' update player last message variable
      UserRecord.LastMessage = Message.Number
      Outpt = "Read more (y)es/(n)o/(r)eply? " ' make more prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get player input
      If UCase$(Inpt) = "R" Then ' check input
         Outpt = "Private message(y/n)? " ' input prompt
         Call IO.I ' get player input
         Private.Message = Yes ' set message type
         Message.Reply = True ' set message type
         Message.Thread = Message.Number ' store message number
         Message.To = TableRecord.From ' store message header
         Subject = TableRecord.Subject ' store message header
         Outpt = "Enter message reply." ' make message
         Call IO.O ' send message
         Call Write.Message ' routine to enter and write message
         Call Read.Record(TableFile, Message.Number) ' read current table
         TableRecord.Reply = True ' set reply
         Call Share.Record(TableFile, Message.Number) ' write table record
         Message.Number = Message.Number - 1 ' adjust read loop variable
      End If ' end check input response
      If No Or Quit Then ' check input response
         Exit Do ' exit new message read loop
      End If ' end check response
      Message.Number = Message.Number + 1 ' increment message variable
   Loop ' end loop through new messages
End Sub ' end routine to read new messages

Rem * routine containing status line commands. status line toggle,
Rem * initializing, clearing, and updating.
Rem * input variables:
Rem *   Status.Display -
Rem *       -2 clear both status lines (rows 24, 25),
Rem *       -1 toggle status line displaying remote user statistics, or
Rem *          interactive console function key list,
Rem *        0 update remote user status line statistics,
Rem *        1 initialize both status lines.
Rem * processing variables:
Rem *   CursorX - temporary variable containing the cursor row position.
Rem *   CursorY - temporary variable containing the cursor column position.
Rem *   Statusline.Mode - static variable saved between calls containing the
Rem *      toggle mode of the status line.
Rem * notes on routine:
Rem *   since the status line can be toggled while an online user is
Rem *   entering input, the color of the screen is restored upon exit
Rem *   from the routine. i.e. the status line toggles during any
Rem *   character of i/o.

Sub Status.Line (Status.Display)
   Static Statusline.Mode ' status line toggle (variable saved between calls)
   If Status.Display = -1 Then ' compare to toggle command
      If Local.Mode Then ' compare local console logged in
         Exit Sub ' exit routine w/o toggling status line
      End If ' end compare local mode
      Statusline.Mode = Not Statusline.Mode ' toggle status line
   End If ' end compare toggle command
   If Status.Display = -2 Then ' compare to clear status line command
      For FunctionKeys = 1 To 10 ' loop through all ten function keys
         KEY FunctionKeys, Nul ' reset/disable function key
      Next ' end loop through function keys
      CursorX = CsrLin ' store current cursor row
      CursorY = Pos(0) ' store current cursor column
      Color 7, 0 ' set color white on black non-intensity
      Locate 25, 1 ' locate at bottom status line
      Print Space$(79); ' clear status line w/ blanks
      Locate 24, 1 ' locate at second to bottom status line
      Print Space$(79); ' clear status line w/ blanks
      Locate CursorX, CursorY, 1 ' relocate at stored cursor row, column
      Call Restore.Color ' subroutine to restore screen color
      Exit Sub ' exit routine
   End If ' end compare clear status line command
   If Status.Display > False Then ' compare to initialize status line command
      Statusline.Mode = False ' set the default status line mode
      If Local.Mode Then ' check console logged in
         If Normal.User = False Then ' verify user logged in is not DM/Sysop
            Statusline.Mode = True ' reset the default status line mode
         End If ' end verify DM/Sysop
      End If ' end check local mode
      'Call Door.Status.Line ' initialize the door information status line
   End If ' end compare initialize status line command
   If Statusline.Mode = False Then ' compare status line mode
      Call Make.Status.Line ' routine to update status line w/ player stats
   Else ' status line mode
      If Status.Display <> False Then ' any status line command than update
         Call Sysop.Status.Line ' routine for sysop function key status line
      End If ' end compare status line command
   End If ' end compare status line command
End Sub ' end status line commands routine

Rem * routine to make status line containing player character statistics.
Rem * processing variables:
Rem *   CursorX - contains the current cursor row.
Rem *   CursorY - contains the current cursor column.

Sub Make.Status.Line
   Status$ = UserRecord.CodeName ' store player codename
   Call Decrypt(Status$) ' decrypt codename
   Status$ = LCase$(Status$) ' set codename lowercase
   If Left$(Status$, 9) = Deleted$ Then ' check codename is invalid/deleted
      Exit Sub ' exit routine
   End If ' end check codename
   Mid$(Status$, 1, 1) = UCase$(Mid$(Status$, 1, 1)) ' uppercase first codename letter
   If UserRecord.ClassType = MagicUser Then ' compare player class type to MU
      Status2$ = "MU" ' set status line player class name to MU abbreviation
   Else ' compare to non MU
      ' set status line player class name to left part of entire class name
      Status2$ = Left$(Class.Name(UserRecord.ClassType), 8)
   End If ' end compare MU
   If Dungeon.Master.Assistant Then ' compare player class name to Asst. DM
      Status2$ = "ADM" ' set status line to Asst. DM abbreviation
   End If ' end compare ADM
   If Dungeon.Master Then ' compare player class name to DM
      Status2$ = "DM" ' set status line to DM abbreviation
   End If ' end compare DM
   If Sysop Then ' compare player to sysop
      Status2$ = "SYS" ' set status line to sysop abbreviation
   End If ' end compare sysop
   Status$ = Status$ + " " + Status2$ ' combine player codename with class name
   Status$ = Left$(Status$, 39) ' truncate to left half
   Status$ = Status$ + Space$(39 - Len(Status$)) ' append trailing blanks
   Status.Value = UserRecord.Fatigue ' store current player fatigue
   If Status.Value < False Then ' check validity of fatigue
      Status.Value = False ' reset to zero
   End If ' end check validity
   Status$ = Status$ + " Fat:" + Mid$(Str$(Status.Value), 2) ' append fatigue message
   Status.Value = UserRecord.Vitality ' store current player vitality
   If Status.Value < False Then ' check validity of vitality
      Status.Value = False ' reset to zero
   End If ' end check validity
   Status$ = Status$ + " Vit:" + Mid$(Str$(Status.Value), 2) ' append vitality message
   Status.Value = UserRecord.Magic ' store current player magic points
   If Status.Value < False Then ' check magic points validity
      Status.Value = False ' reset to zero
   End If ' end check validity
   ' append magic points message
   Status$ = Status$ + " Mag:" + Mid$(Str$(Status.Value), 2)
   Status.Value = UserRecord.Psionic ' store current player psionic points
   If Status.Value < False Then ' check psionic points validity
      Status.Value = False ' reset to zero
   End If ' end check validity
   ' append psionic points message
   Status$ = Status$ + " Psi:" + Mid$(Str$(Status.Value), 2)
   Status.Value = UserRecord.Level ' store player level
   If Status.Value < False Then ' compare level validity
      Status.Value = False ' reset to zero
   End If ' end compare validity
   Status2$ = " Lvl:" + Mid$(Str$(Status.Value), 2) ' append level message
   ' verify string appended to status line
   If Len(Status$) + Len(Status2$) <= 79 Then
      ' string is less than screen line length, and append
      Status$ = Status$ + Status2$
   End If ' end verify status line length
   Status.Value = Room ' store current room number
   If Status.Value < False Then ' check validity of room number
      Status.Value = False ' reset to zero
   End If ' end check validity
   Status2$ = " Rm:" + Mid$(Str$(Status.Value), 2) ' append room message
   ' verify string appended to status line
   If Len(Status$) + Len(Status2$) <= 79 Then
      ' string is less than screen line length, and append
      Status$ = Status$ + Status2$
   End If ' end verify status line length
   Status$ = Left$(Status$, 79) ' truncate status string length
   Status$ = Status$ + Space$(79 - Len(Status$)) ' append blanks to status string
   CursorX = CsrLin ' store current cursor row
   CursorY = Pos(0) ' store current cursor column
   Locate 25, 1 ' position cursor at row 25
   Color 14, 1 ' color hi-intensity yellow on blue
   Print Status$; ' display the combined status string
   Locate CursorX, CursorY, 1 ' restore cursor position
   Call Restore.Color ' routine to restore screen color
End Sub ' end routine to make and display status line

Rem * routine to initialize the console function keys, initialize the
Rem * DM/sysop status line with the function key names.
Rem * processing variables:
Rem *   CursorX, CursorY - contain the current cursor position.

Sub Sysop.Status.Line
   For FunctionKeys = 1 To 10 ' loop through all ten function keys
      KEY FunctionKeys, Nul ' clear the function key
   Next ' loop through keys
   If Local.Mode Then ' check console logged in
      If Normal.User = False Then ' check user is DM/sysop
         KEY 1, "!EDIT" + Chr$(13) ' assign key 1
         KEY 2, "!STA" + Chr$(13) ' assign key 2
         KEY 3, "!DIS " ' assign key 3
         KEY 4, "!REDU " ' assign key 4
         KEY 5, "!CALL" ' assign key 5
         KEY 6, "!KILL " ' assign key 6
         KEY 7, "!TELE " ' assign key 7
         KEY 8, "!INV" + Chr$(13) ' assign key 8
         KEY 9, "!GET " ' assign key 9
         KEY 10, "!LINK" + Chr$(13) ' assign key 10
      End If ' end check DM
   End If ' end check local mode
   CursorX = CsrLin ' store cursor row
   CursorY = Pos(0) ' store cursor column
   Color 14, 1 ' set color to hi-intensity yellow on blue
   For FunctionKeys = 1 To 10 ' loop through the ten function key numbers
      ' position the cursor at the function key column
      Locate 25, FunctionKeys * 8 - 7
      ' display the function key and number
      Print "F" + Right$(Str$(FunctionKeys + 10), 1);
   Next ' end loop through function key numbers
   Color 15, 1 ' set color to hi-intensity white on blue
   ' store function key names
   FunctionKeys$ = "EDITSTA DIS REDUCALLKILLTELEINV GET LINK"
   For FunctionKeys = 1 To 10 ' loop through ten function key names
      ' position the cursor at the status function key name
      Locate 25, FunctionKeys * 8 - 5
      ' make the function key name with DM prefix from the function string
      Status$ = "!" + Mid$(FunctionKeys$, (FunctionKeys - 1) * 4 + 1, 4)
      If FunctionKeys < 10 Then ' check for last key
         Status$ = Status$ + " " ' append space
      End If ' end check last key
      Print Status$; ' display the function key name
   Next ' end loop through function key names
   Locate CursorX, CursorY, 1 ' restore cursor position
   Call Restore.Color ' routine to restore screen color
End Sub ' end routine to initialize function keys and DM status line

Rem * routine to restore current ansi color after status line is displayed.
Rem * input variables:
Rem *   Color.Code - contains current color in cycling.

Sub Restore.Color
   If Graphics.Off Then ' check color cycling
      Call Convert.Color(37) ' restore color to white
   Else ' color check
      Call Convert.Color(Color.Code) ' restore color
   End If ' end check color cycling on
End Sub ' end routine to restore color

Rem * routine displays spell types.

Sub Spell.Types
   Graphics.Off = True ' reset color
   Outpt = "[A]Enchant      [O]Psionic"
   Call IO.O ' send output
   Outpt = "[B]Offense      [P]Detect Lock"
   Call IO.O ' send output
   Outpt = "[C]Bless        [R]Detect Evil"
   Call IO.O ' send output
   Outpt = "[D]Wish         [S]Detect Trap"
   Call IO.O ' send output
   Outpt = "[E]Poison       [T]Intoxicate"
   Call IO.O ' send output
   Outpt = "[F]Vigor        [U]Set Trap"
   Call IO.O ' send output
   Outpt = "[G]Heal         [V]Hide"
   Call IO.O ' send output
   Outpt = "[H]Curepoison   [W]Search"
   Call IO.O ' send output
   Outpt = "[I]Level Drain  [X]Invisibility"
   Call IO.O ' send output
   Outpt = "[J]Teleport     [Y]Identify"
   Call IO.O ' send output
   Outpt = "[K]Befuddle     [Z]Enlighten"
   Call IO.O ' send output
   Outpt = "[L]Turn Undead  [1]Illuminate"
   Call IO.O ' send output
   Outpt = "[M]Pass Door    [2]Psyche"
   Call IO.O ' send output
   Outpt = "[N]Conjure      [3]Telepathy"
   Call IO.O ' send output
End Sub

Rem * routine changes message.

Sub Change.Message
   Do ' loop through message changing
      Graphics.Off = False ' reset color
      Max.Messages = LOF(TableFile) / Len(TableRecord) 'store length of message table
      Outpt = "Message number to edit" ' make range input prompt
      Call Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
      If Message.Number > False Then ' check number bounds
         Call Read.Record(TableFile, Message.Number) ' read table record
         Graphics.Off = False ' reset color
         Outpt = "Edit message header(y/n)? " ' make input prompt
         No.Input.Out = "N" ' default input
         Call IO.I ' get user input
         If Yes Then ' check nput response
            Call Message.Header(Message.Number) ' routine to display header
            Graphics.Off = False ' reset color
            Outpt = "Enter new message header:" ' make display message
            Call IO.O ' send message
            Graphics.Off = True ' reset color
            Outpt = "From(press <enter> for Sysop)? " ' make input prompt
            No.Input.Out = "Sysop" ' default input
            Call IO.I ' get user input
            TableRecord.From = Inpt ' store message from
            Outpt = "To(press <enter> for ALL)? " ' make input prompt
            Call IO.I ' get user input
            TableRecord.ToUser = UCase$(Inpt) ' store message to
            Outpt = "Subject? " ' make input prompt
            Call IO.I ' get user input
            TableRecord.Subject = LCase$(Inpt) ' store message subject
            Call Share.Record(TableFile, Message.Number) ' routine to write table record
         End If ' end check response
         Do ' loop through message text editing
            Graphics.Off = False ' reset color
            Outpt = "Edit more message text(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            Call IO.I ' get user input
            If No Then ' check input response
               Exit Do ' exit message editing loop
            End If ' end check response
            Call Read.Message(Message.Number, True) ' routine to display message
            Graphics.Off = False ' rset color
            MessageRec.Length = TableRecord.Length ' store message length
            Outpt = "Line number to edit" ' make range input prompt
            ' routine to get number
            Call Get.Range2(0, MessageRec.Length, Message.Line)
            If Message.Line > False Then ' check range
               Outpt = "New message text:" ' make display message
               Call IO.O ' send message
               Graphics.Off = True ' reset color
               Outpt = "?" ' make input prompt
               Call IO.I ' get user input
               Call Valid(Inpt, 80) ' validate input
               Call Encrypt(Inpt, True) ' encrypt input
               MessageRecord.Message = Inpt ' store new message text
               MessageRec.Number! = CSng(TableRecord.Start + Message.Line - 1)
               Call Share.Message(MessageFile, MessageRec.Number!) ' write message record
            End If ' end check range
         Loop ' end loop through editing
      End If ' end check number bounds
      Graphics.Off = False ' reset color
      Outpt = "Edit another message(y/n)? " ' make input prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get user input
      If No Then ' check input response
         Exit Sub ' exit routine
      End If ' end check response
   Loop ' end loop through editing
End Sub ' end message editing routine

Rem * routine to delete message.

Sub Delete.Message
   Do ' loop through delete routine
      Max.Messages = LOF(TableFile) / Len(TableRecord) ' store length of table file
      Outpt = "Message number to delete" ' make range input prompt
      Call Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
      If Message.Number > False Then ' check number bounds
         Call Read.Record(TableFile, Message.Number) ' read table record
         If TableRecord.Killed Then ' check deleted message flag
            Outpt = "Message" + Str$(Message.Number) + " is already deleted." ' message
         Else ' check deleted message
            TableRecord.Killed = True ' set deleted message flag
            Call Share.Record(TableFile, Message.Number) ' routine to write table record
            Outpt = "Message" + Str$(Message.Number) + " deleted." ' make deleted message
         End If ' end check deleted message flag
         Call IO.O ' send message
      End If ' end check number bounds
      Graphics.Off = False ' reset color
      Outpt = "Delete more messages(y/n)? " ' make input prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get user input
      If No Then ' check input response
         Exit Sub ' exit routine
      End If ' end check response
   Loop ' end loop through deleting
End Sub ' end delete message routine

Rem * routine to list range of messages.

Sub List.Messages
   Messages.Max = LOF(TableFile) / Len(TableRecord) ' store length of table
   ' routine to get range of numbers
   Call Get.Range(Messages.Max, Messages1, Messages2)
   For Message.List = Messages1 To Messages2 ' loop through range of messages
      Call Read.Record(TableFile, Message.List) ' read table record
      Call Read.Message(Message.List, False) ' routine to display message
      Graphics.Off = False ' reset color
      Outpt = "Read more(y/n)? " ' make prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get player input
      If No Then ' check input response
         Exit Sub ' exit routine
      End If ' end check response
   Next ' end message display loop
End Sub ' end message list routine

Rem * routine to undelete messages.

Sub Undelete.Message
   Do ' loop through undeleting
      Max.Messages = LOF(TableFile) / Len(TableRecord) ' store table length
      Outpt = "Message number to undelete" ' make range input prompt
      Call Get.Range2(0, Max.Messages, Message.Number) ' routine to get number
      If Message.Number > False Then ' check number bounds
         Call Read.Record(TableFile, Message.Number) ' read table record
         If TableRecord.Killed = False Then ' check deleted message
            Outpt = "Message" + Str$(Message.Number) + " is not deleted." ' make message
         Else ' check deleted message
            TableRecord.Killed = False ' set deleted message flag
            Call Share.Record(TableFile, Message.Number) ' routine to write table record
            Outpt = "Message" + Str$(Message.Number) + " undeleted." ' make message
         End If ' end check undeleted message
         Call IO.O ' send message
      End If ' end check number bounds
      Graphics.Off = False ' reset color
      Outpt = "Undelete more messages(y/n)? " ' make input prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get user input
      If No Then ' check input response
         Exit Sub ' exit routine
      End If ' end check input
   Loop ' end undeleting loop
End Sub ' end undelete routine

Rem * routine to pack messages files.

Sub Pack.Messages
   Outpt = "Pack messages(y/n)? " ' make input prompt
   No.Input.Out = "N" ' default input
   Call IO.I ' get user input
   If Yes Then ' check response
      Outpt = "Packing messages.." ' make message
      Call IO.O ' send message
      Call Share.Record(UserFile, User.Index) ' store user record
      Max.Users = LOF(UserFile) / Len(UserRecord) ' store length of user file
      ' dimension working array
      ReDim ArrayX(1 To Max.Users) As Integer
      ' packing pass 1
      For User.Number = 1 To Max.Users ' loop through user file
         Call Read.Record(UserFile, User.Number) ' get next user file record
         ArrayX(User.Number) = UserRecord.LastMessage ' store last message number
      Next ' end user file loop
      For Message.Number = 1 To LOF(TableFile) / Len(TableRecord) 'loop through table
         Call Read.Record(TableFile, Message.Number) ' read table record
         If TableRecord.Killed Then ' check for deleted message
            For User.Number = 1 To Max.Users ' loop through user array
               Last.Message = ArrayX(User.Number) ' store last message
               If Last.Message <= Message.Number Then ' check last message
                  Last.Message = Last.Message - 1 ' decrement last message read
                  ArrayX(User.Number) = Last.Message ' store last message read
               End If ' end check last message
            Next ' end user array loop
         End If ' end check deleted message
      Next ' end message pack loop
      For User.Number = 1 To Max.Users ' loop through user file
         Call Read.Record(UserFile, User.Number) ' get next user file record
         UserRecord.LastMessage = ArrayX(User.Number) ' store last message number
         Call Share.Record(UserFile, User.Number) ' write user record
      Next ' end user file loop
      Call Read.Record(UserFile, User.Index) ' reread current player record
      ' packing pass 2
      Max.Messages = LOF(TableFile) / Len(TableRecord) ' store maximum table records
      ' dimension working array
      ReDim ArrayX(1 To Max.Messages) As Integer
      For Message.Number = 1 To Max.Messages ' loop through table file
         Call Read.Record(TableFile, Message.Number) ' read table record
         ArrayX(Message.Number) = TableRecord.Thread ' store thread number
      Next ' end table file rad loop
      For Message.Number = 1 To Max.Messages ' loop through table file
         Call Read.Record(TableFile, Message.Number) ' read table record
         If TableRecord.Killed Then ' check for deleted message
            Thread.Number = ArrayX(Message.Number) ' store thread number
            ' search for forward thread
            For Message.Search = Message.Number + 1 To Max.Messages
               ' verify forward message thread
               If ArrayX(Message.Search) = Message.Number Then
                  ArrayX(Message.Search) = Thread.Number ' store thread number
                  If Thread.Number = False Then ' check for thread start
                     ' search for end of thread
                     For Thread.Search = Message.Search + 1 To Max.Messages
                        ' compare thread has another ending thread
                        If ArrayX(Thread.Search) = Message.Search Then
                           Exit For ' exit thread end loop
                        End If ' end compare for end thread
                     Next ' end end thread search loop
                     If Thread.Search > Max.Messages Then ' compare end thread
                        Call Read.Record(TableFile, Message.Search) ' read table
                        TableRecord.Reply = False ' reset thread reply start
                        Call Share.Record(TableFile, Message.Search) 'write table
                     End If ' end compare end thread
                  End If ' end check thread start
               End If ' end verify forward message thread
            Next ' end forward message thread search loop
            ' search through all forward thread numbers
            For Message.Search = Message.Number + 1 To Max.Messages
               Thread.Number = ArrayX(Message.Search) ' store thread number
               If Thread.Number >= Message.Number Then ' verify forward thread
                  Thread.Number = Thread.Number - 1 ' decrement thread number
                  ArrayX(Message.Search) = Thread.Number ' store thread number
               End If ' end verify forward message thread
            Next ' end forward message thread search loop
         End If ' end check deleted message
      Next ' end message pack loop
      For Message.Number = 1 To Max.Messages ' loop through table file
         Call Read.Record(TableFile, Message.Number) ' read table record
         TableRecord.Thread = ArrayX(Message.Number) ' store thread number
         Call Share.Record(TableFile, Message.Number) ' write table record
      Next ' end table file read loop
      ' packing pass 3
      Close #TempFile ' close temporary file
      FileName = "msgtable.bak" ' get table filename
      Kill FileName ' remove temporary file
      ' open temporary backup file
      Open FileName For Random Shared As #TempFile Len = Len(TableRecord)
      For Message.Number = 1 To LOF(TableFile) / Len(TableRecord) ' loop through table
         Call Read.Record(TableFile, Message.Number) ' read table record
         Call Share.Record(TempFile, Message.Number) ' store table record
      Next ' end message pack loop
      Close #TempFile ' close temporary file
      FileName = "messages.bak" ' get table filename
      Kill FileName ' remove temporary file
      ' open temporary backup file
      Open FileName For Random Shared As #TempFile Len = Len(MessageRecord)
      For Message.Number! = 1 To LOF(MessageFile) / Len(MessageRecord) ' loop message
         Call Read.Message.Record(MessageFile, Message.Number!) ' read record
         Call Share.Message(TempFile, Message.Number!) ' store message record
      Next ' end message pack loop
      Close #TableFile, #MessageFile, #TempFile, #TempFile2 ' close temp, mail files
      FileName = "messages.dat" ' get mail filename
      Kill FileName ' remove mail file
      FileName = "msgtable.dat" ' get mail filename
      Kill FileName ' remove mail file
      Open "msgtable.dat" For Random Shared As #TableFile Len = Len(TableRecord)
      Open "messages.dat" For Random Shared As #MessageFile Len = Len(MessageRecord)
      Open "msgtable.bak" For Random Shared As #TempFile Len = Len(TableRecord)
      Open "messages.bak" For Random Shared As #TempFile2 Len = Len(MessageRecord)
      Table.Number = False ' reset table record number
      New.Message.Record! = False ' reset message file record counter
      For Message.Number = 1 To LOF(TempFile) / Len(TableRecord) 'loop through table
         Call Read.Record(TempFile, Message.Number) ' read table record
         If TableRecord.Killed = False Then ' check for deleted message
            Message.Start! = TableRecord.Start ' store beginning
            Message.End! = Message.Start! + CSng(TableRecord.Length - 1) ' store end
            Table.Number = Table.Number + 1 ' increment next table record
            TableRecord.Start = New.Message.Record! + 1 ' store message beginning
            Call Share.Record(TableFile, Table.Number) ' write new table file record
            For Message.Record! = Message.Start! To Message.End! ' message loop
               Call Read.Message.Record(TempFile2, Message.Record!) 'read message
               New.Message.Record! = New.Message.Record! + 1! ' store next message
               Call Share.Message(MessageFile, New.Message.Record!) ' write message
            Next ' end loop through message text array
         End If ' end check deleted message
      Next ' end table file loop
      Close #TempFile, #TempFile2 ' close temporary files
   End If ' end check response
End Sub ' end routine to pack messages

Rem * routine to store message.

Sub Store.Message
   Call Read.Record(UserFile, User.Index) ' read user file
   Outpt = UserRecord.CodeName ' store codename
   Call Decrypt(Outpt) ' decrypt codename
   Inpt = RTrim$(Outpt) ' store codename
   Inpt = UCase$(Inpt) ' uppercase ccodename
   Outpts = Message.To ' store message header
   Outpts = RTrim$(Outpts) ' trim header
   Outpts = UCase$(Outpts) ' uppercase header
   If Inpt = Outpts Then ' compare codenames
      Message.To = Nul ' reset message header
   End If ' end compare codenames
   TableRecord.ClassType = UserRecord.ClassType ' store user class type
   TableRecord.Clock = clock$ ' store message creation time
   TableRecord.Date = Date$ ' store system date
   TableRecord.Flags = UserRecord.Flags ' store user flags
   TableRecord.From = Outpt ' store user codename
   TableRecord.Killed = False ' reset deleted message flag
   TableRecord.Length = Message.Length ' store length of message
   TableRecord.Private = Private.Message ' store private message flag
   TableRecord.Received = False ' store flag for message received
   TableRecord.Reply = Message.Reply ' store message reply flag
   Message.Record.Number! = CSng(LOF(MessageFile) / Len(MessageRecord))
   TableRecord.Start = Message.Record.Number! + 1! ' store message beginning
   TableRecord.Subject = Subject ' store message subject
   TableRecord.Time = Time$ ' store system time
   TableRecord.Timer = Timer ' store system time in seconds
   TableRecord.TimesRead = False ' reset number of times message read
   TableRecord.ToUser = Message.To ' store message to
   If Message.Reply Then ' check message type
      TableRecord.Thread = Message.Thread ' store message thread number
   Else ' check message
      TableRecord.Thread = False ' store thread number
   End If ' end check message
   Table.Record.Number = LOF(TableFile) / Len(TableRecord) + 1 'store next table record
   Call Share.Record(TableFile, Table.Record.Number) ' routine to write table record
   For Message.Number = 1 To Message.Length ' loop through message text array
      Outpt = Array(Message.Number) ' store message text line
      Call Valid(Outpt, 80) ' routine to validate text
      Call Encrypt(Outpt, True) ' encrypt text
      MessageRecord.Message = Outpt ' store text line
      Message.Record.Number! = Message.Record.Number! + 1! ' store next message
      Call Share.Message(MessageFile, Message.Record.Number!) ' write message text
   Next ' end loop through message text array
   Outpt = "Message stored." ' make message
   Call IO.O ' send message
End Sub ' end routine to store message

Rem * routine to check for new messages.

Sub Check.Mail
   New.Message = False ' reset number of new messages
   Outpt = UserRecord.CodeName ' store codename
   Call Decrypt(Outpt) ' decrypt codename
   Outpt = RTrim$(Outpt) ' trim codename
   Outpt = LCase$(Outpt) ' lowercase codename
   Last.Message = UserRecord.LastMessage ' store last message number read
   Table.Number = LOF(TableFile) / Len(TableRecord) ' store length of message table
   ' compare range of messages
   If Last.Message > False And Last.Message <= Table.Number Then
      For Table.Index = Last.Message To Table.Number ' loop through new messages
         Call Read.Record(TableFile, Table.Index) ' read table record
         If TableRecord.Killed = False Then ' check deleted message
            Inpt = TableRecord.ToUser ' store message to
            Inpt = RTrim$(Inpt) ' trim message to
            Inpt = LCase$(Inpt) ' lowercase message to
            If Inpt <> Nul Then ' check public message to
               If Inpt = Outpt Then ' compare codenames
                  New.Message = New.Message + 1 ' increment number of new message
               End If ' end compare codenames
            End If ' end check public message
         End If ' end check deleted message
      Next ' end loop through new messages
   End If ' end compare new message range
   Graphics.Off = False ' reset color
   If New.Message > False Then ' check new messages
      Outpt = "You have" + Str$(New.Message) + " new messages." ' make message
   Else ' check new message variable
      Outpt = "You have no new messages." ' make output message
   End If ' end check new message variable
   Call IO.O ' send message
   Graphics.Off = False ' reset color
End Sub ' end routine to check for new mail

Rem * routine to replace line of message text.

Sub Replace.Line
   Messages.Max = Message.Length ' store message length
   Outpt = "Line number" ' store input range prompt
   Call Get.Range2(0, Messages.Max, Message.Number) ' routine to get number
   If Message.Number > False Then ' check number bounds
      Outpt = "Replacement line:" ' input prompt
      Call IO.O ' send prompt
      Outpt = "?" ' input prompt
      Call IO.I ' get user input
      Inpt = RTrim$(Inpt) ' trim input
      Inpt = Left$(Inpt, 79) ' truncate input
      Array(Message.Number) = Inpt ' store new message line
      Outpt = "Line number" + Str$(Message.Number) + " replaced." ' make display message
      Call IO.O ' send message
   End If ' end check number bounds
End Sub ' end routine to replace line of message text

Rem * routine to edit a line of message text.

Sub Edit.Line
   Messages.Max = Message.Length ' store length of message
   Outpt = "Line number" ' make input range prompt
   Call Get.Range2(0, Messages.Max, Message.Number) ' routine to get number
   If Message.Number > False Then ' check number bounds
      Outpt = "Replace what word? " ' make input prompt
      Call IO.I ' get user input
      If Inpt = Nul Then ' check input length
         Outpt = "No replacements made." ' make display message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check input length
      Message.Number$ = Inpt ' store input
      Outpt = "Replace with what word? " ' make input prompt
      Call IO.I ' get user input
      If Inpt = Nul Or Inpt = Message.Number$ Then ' check input length, compare inputs
         Outpt = "No replacements made." ' make display message
         Call IO.O ' send message
         Exit Sub ' exit routine
      End If ' end check inputs
      Edit.Replace$ = Inpt ' store input
      Outpt = "Replace all occurences? " ' make input prompt
      No.Input.Out = "Y" ' default input
      Call IO.I ' get user input
      Replace.Word = 1 ' reset string search index
      Replace.All = Yes ' store input response
      Replacements = False ' reset replacement flag
      EditLine$ = RTrim$(Array(Message.Number)) ' store message line to edit
      ' routine loop replaces Message.Number$ in EditLine$ with Edit.Replace$
      Do ' loop until replacements finished
         If EditLine$ = Nul Then ' check length of message line
            Exit Do ' exit loop if line length reduced to null
         End If ' end check length of message line
         ' get position of search string
         Replace = InStr(Replace.Word, EditLine$, Message.Number$)
         If Replace = False Then ' check search string exists
            Exit Do ' exit loop if search string not found
         End If ' end check search string exists
         ' replace search string with replacement string
         EditLine$ = Left$(EditLine$, Replace - 1) + Edit.Replace$ + Mid$(EditLine$, Replace + Len(Message.Number$))
         EditLine$ = Left$(EditLine$, 79) ' truncate message line
         ' recalculate next position index for search
         Replace.Word = Replace + Len(Edit.Replace$)
         Replacements = Replacements + 1 ' increment number of replacements made
         If Replacements = 1 Then ' check first replacement
            If Replace.All = False Then ' check flag to replace all searches
               Exit Do ' exit loop after only one replacement made
            End If ' end check replacement flag
         End If ' end check replacement number
      Loop ' end replacement loop
      Array(Message.Number) = EditLine$ ' store edited message line
      Select Case Replacements ' selectionn of number of replacements made
         Case 0 ' no replacements
            Outpt = "No replacements made." ' make display message
         Case 1 ' one replacement
            Outpt = "One replacement made." ' make display message
         Case Else ' more than one replacement
            Outpt = Mid$(Str$(Replacements), 2) + " replacements made." ' make message
      End Select ' end selection of number of replacements
      Call IO.O ' send display message
   End If ' end check number bounds
End Sub ' end routine to edit message line

Rem * routine to delete range of lines from message text.

Sub Delete.Line
   Outpt = "Enter line numbers:" ' make display message
   Call IO.O ' send message
   Messages.Max = Message.Length ' store length of message
   ' routine to get range of numbers
   Call Get.Range(Messages.Max, Message.Line1, Message.Line2)
   ' input prompt
   Outpt = "Delete lines" + Str$(Message.Line1) + " to" + Str$(Message.Line2) + "(y/n)? "
   No.Input.Out = "Y" ' default input
   Call IO.I ' get user input
   If No Then ' check input response
      Exit Sub ' exit routine
   End If ' end check input response
   ' loop through the total number of lines to delete
   For Message.Index = 1 To Message.Line2 - Message.Line1 + 1
      Message.Length = Message.Length - 1 ' decrement the length of message
      ' loop from the first line to delete
      For Array.Index = Message.Line1 To Message.Length
         Array(Array.Index) = Array(Array.Index + 1) ' packing the remaining message
      Next ' end loop through deleted lines
   Next ' end loop through number of lines to delete
   If Message.Length = False Then ' check length of message
      Outpt = "No message left." ' make error message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check message length
   Outpt = "Line numbers" + Str$(Message.Line1) + " to" + Str$(Message.Line2) + " deleted." ' make message
   Call IO.O ' send message
End Sub ' end routine to delete range of lines

Rem * routine to insert lines into message text.

Sub Insert.Lines
   If Message.Length = 64 Then ' check length of message
      Outpt = "Message buffer full." ' make message
      Call IO.O ' send message
      Exit Sub ' exit routine
   End If ' end check message length
   Messages.Max = Message.Length ' store message length
   Outpt = "Before line number" ' make input range prompt
   Call Get.Range2(0, Messages.Max, Message.Line) ' routine to get number
   If Message.Line > False Then ' check number bounds
      Graphics.Off = True ' reset color
      User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
      UserRecord.Wordwrap = False ' reset word wrap
      Do While Message.Length < 64 ' loop while message length range
         Word.Wrap = True ' set word wrap flag
         Outpt = "?" ' make input prompt
         Call IO.I ' get user input
         Word.Wrap = False ' reset word wrap flag
         If No.Input Then ' check length of input flag
            Exit Do ' exit message netry loop
         End If ' end check input length
         ' loop backwards through message
         For Array.Index = Message.Length To Message.Line Step -1
            ' pack lines before inserted line
            Array(Array.Index + 1) = Array(Array.Index)
         Next ' end backward loop
         Inpt = RTrim$(Inpt) ' trim input
         Inpt = Left$(Inpt, 79) ' truncate input
         Array(Message.Line) = Inpt ' store new inserted message line
         Message.Line = Message.Line + 1 ' increment line number to insert before
         Message.Length = Message.Length + 1 ' increment length of message
      Loop ' end message insert loop
      UserRecord.Wordwrap = User.Word.Wrap ' restore word wrap
      Graphics.Off = False ' reset color
      If Message.Length = 64 Then ' compare length of message
         Outpt = "Message buffer full." ' make buffer message
         Call IO.O ' send message
      End If ' end check length of message
   End If ' end check number bounds
End Sub ' end insert routine

Rem * routine to list range of message text lines.

Sub List.Lines
   Outpt = "Enter line numbers:" ' make display message
   Messages.Max = Message.Length ' store length of message
   ' routine to get range of numbers
   Call Get.Range(Messages.Max, Message.Line1, Message.Line2)
   Outpt = "Display line numbers(y/n)? " ' make input prompt
   No.Input.Out = "Y" ' default input
   Call IO.I ' get user input
   ListLines = Yes ' store response
   Allow.Break = True ' turn on allow break flag
   Break = False ' reset control-k flag
   Continue = False ' reset continuous flag
   Page.Length = False ' reset page counter
   Graphics.Off = True ' reset color
   ' loop through range of message lines
   User.Line.Length = UserRecord.Linelength ' store user line length
   UserRecord.Linelength = False ' store line length
   For Message.Line = Message.Line1 To Message.Line2
      If ListLines Then ' check line number list flag
         Outpt = Mid$(Str$(Message.Line), 2) + ":" ' make line number
      Else ' check line number flag
         Outpt = Nul ' reset line number string
      End If ' end check line number flag
      Outpt = Outpt + Array(Message.Line) ' make line display
      Outpt = Left$(Outpt, 79) ' truncate line
      Call IO.O ' send line display
      If Break Then ' check control-k pressed
         Exit For ' exit text input loop
      End If ' end check control-k
      Page.Length = Page.Length + 1 ' increment page counter
      If Page.Length = UserRecord.Pagelength Then ' check page counter
         Page.Length = False ' reset page counter
         If Continue = False Then ' check continuous flag
            Call More.Prompt ' routine to pause
            If No Then ' check pause response
               Exit For ' exit routine
            End If ' end check response
         End If ' end check continuous flag
      End If ' end check page counter
   Next ' end loop through range of lines
   UserRecord.Linelength = User.Line.Length ' restore line length
   Allow.Break = False ' reset allow break flag
   If Break Then ' check control-k flag
      Break = False ' reset control-k flag
      Outpt = Nul ' set output to null
      Call IO.O ' send empty return
   End If ' end check control-k flag
   If Page.Length Then ' recheck page counter
      Call More.Prompt ' pause routine
   End If ' end recheck counter
End Sub ' end list lines routine

Rem * routine to edit rooms.

Sub Edit.Room
   Call Share.Room.Record(Room) ' store current room
   Do ' loop through room edit menu
      Graphics.Off = False ' reset color
      Outpt = "Room edit:" ' make output display
      Call IO.O ' send output
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make output display
      Call IO.O ' send output
      Outpt = "[C]hange" ' make output display
      Call IO.O ' send output
      Outpt = "[L]ist" ' make output display
      Call IO.O ' send output
      Graphics.Off = False ' reset color
      Outpt = "Enter room edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I
      Select Case UCase$(Inpt) ' selection of input
         Case "A" ' option to add new room
            Call Add.Room(False, Room.Added) ' routine to add room
         Case "C" ' option to select another room number to edit
            Outpt = "Enter room number" ' prompt
            Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' store length of room records
            ' routine to get number from range
            Call Get.Room.Range2(1!, Max.Rooms!, Room.Number!)
            Call Change.Room(Room.Number!) ' routine to edit room descriptions
         Case "L" ' display room description
            Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' store length of room record
            ' routine to get range to display
            Call Get.Room.Range(Max.Rooms!, Start.Room!, End.Room!)
            ' loop through range of room numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Room.Number! = Start.Room! To End.Room!
               Call Read.Room.Record(Room.Number!) ' get room record
               Call Display.Room.Desc(Room.Number!) ' routine to display room
               If Break Then ' check break flag
                  Exit For ' exit display loop
               End If
               Graphics.Off = False ' reset color
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' pause prompt
                  If No Then ' compare continue
                     Exit For ' exit loop through rooms
                  End If ' end compare
               End If ' end check continuous flag
            Next ' end loop through rooms
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "Q" ' option to exit menu
            Call Read.Room.Record(Room) ' get current room
            Exit Do ' exit edit menu
      End Select ' end input selection
   Loop ' end room edit menu
End Sub ' end room edit routine

Rem * routine to read !edit help.

Sub Edit.Help
   Do ' help menu loop
      Graphics.Off = False ' reset color
      Outpt = "Edit help:" ' make output message
      Call IO.O ' send output message
      Graphics.Off = True ' reset color
      Outpt = "[C]ontents" ' make output message
      Call IO.O ' send output message
      Outpt = "[T]opic" ' make output message
      Call IO.O ' send output message
      Graphics.Off = False ' reset color
      No.Input.Out = "Q" ' default input
      Outpt = "Enter help option(q to quit)? " ' make input prompt
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of input
         Case "C" ' display !edit help contents
            Stored.Parsed.Command1 = "contents" ' store help topic
            Call Read.Help(1) ' routine to read !edit help
         Case "T" ' select help topic number
            Outpt = "Enter help topic number sequence? " ' make input prompt
            Call IO.I ' get user input
            Stored.Parsed.Command1 = Inpt ' store help topic
            Call Read.Help(1) ' routine to read !edit help
         Case "Q" ' exit menu loop
            Exit Do ' exit loop
      End Select ' end input selection
   Loop ' end menu loop
End Sub ' end routine to read !edit help

Rem * routine to edit container.
Rem * input variables:
Rem *   Container.Type     -1 for room/0 for user
Rem *   ContainerRec    structure of container to edit

Sub Change.Container (Container.Type)
   Do ' loop through room treasure edit menu
      Graphics.Off = False ' reset color
      If Container.Type Then ' check call type
         Outpt = "Room container edit options:" ' make display message
      Else ' check type
         Outpt = "User container edit options:" ' make display message
      End If ' end check type
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]container name" ' make display message
      Call IO.O ' send message
      Outpt = "[B]container lock" ' make display message
      Call IO.O ' send message
      Outpt = "[C]permanent" ' make display message
      If ContainerRec.Permanent Then ' check container value
         Outpt = Outpt + ": yes" ' append to message
      Else ' check value
         Outpt = Outpt + ": no" ' append to message
      End If ' end check value
      Call IO.O ' send message
      Outpt = "[D]inventory" ' make display message
      Number.Items = False ' reset counter
      For Treasure.Number = 1 To 5 ' loop through items
         Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
         ' compare item range
         If Treasure.Item > False And Treasure.Item <= LOF(TreasureFile) / Len(TreasureRecord) Then
            Number.Items = Number.Items + 1 ' increment counter
         End If ' end check range
      Next ' end item loop
      Outpt = Outpt + ":" + Str$(Number.Items)
      If Number.Items = 1 Then ' check counter
         Outpt = Outpt + " item." ' append message
      Else ' check counter
         Outpt = Outpt + " items." ' append message
      End If ' end check counter
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      If Container.Type Then ' check call type
         Outpt = "Room container edit option(q to quit)? " ' make input prompt
      Else ' check type
         Outpt = "User container edit option(q to quit)? " ' make input prompt
      End If ' end check type
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of room container option
         Case "A" ' option to change container name
            Outpt = "Container name?" ' container input
            Call IO.I ' get response
            Inpt = RTrim$(Inpt) ' trim value
            If Inpt <> Nul Then ' compare value
               ContainerRec.ContainerName = LCase$(Inpt) ' container name from treasure.dat file
            End If ' end compare value
            Outpt = "Container mnemonic name?" ' container input
            Call IO.I ' get response
            Inpt = RTrim$(Inpt) ' trim value
            If Inpt <> Nul Then ' compare value
               ContainerRec.ShortName = UCase$(Inpt) ' mnemonic of container name from treasure.dat file
            End If ' end compare value
         Case "B" ' option to change container lock/key
            Outpt = "Container locked(y/n)?" ' container input
            Call IO.I ' get input
            If Yes Then ' check value
               ContainerRec.Locked = 1 ' locked flag 1=locked, 0=n/a, -1=unlocked
               ContainerRec.Closed = 1 ' closed flag 1=closed, 0=n/a, -1=open
            Else ' check value
               ContainerRec.Locked = 0 ' locked flag 1=locked, 0=n/a, -1=unlocked
               ContainerRec.Closed = 0 ' closed flag 1=closed, 0=n/a, -1=open
            End If ' end check value
            Outpt = "Container has key(y/n)?" ' container input
            Call IO.I ' get input
            If Yes Then ' check value
               Outpt = "Container key number(1-32767)?" ' input value
               Call IO.I ' get input
               Key.Number = Int(Val(Inpt)) ' store input
               If Key.Number > False And Key.Number <= MaxInt Then ' check value
                  ContainerRec.Keyed = Key.Number ' container key number 1-32767
               End If ' end check value
            Else ' check value
               ContainerRec.Keyed = False ' container key number 1-32767
            End If ' end check value
         Case "C" ' option to change permanent container
            Outpt = "Container is permanent(y/n)?" ' container input
            Call IO.I ' get input
            ContainerRec.Permanent = Yes ' flag for permanent container (added 05/25/01)
         Case "D" ' option to edit container contents
            Do ' loop through container treasure edit menu
               Graphics.Off = False ' reset color
               If Container.Type Then ' check call type
                  Outpt = "Room container treasure edit options:" ' make display message
               Else ' check type
                  Outpt = "User container treasure edit options:" ' make display message
               End If ' end check type
               Call IO.O ' send message
               Graphics.Off = True ' reset color
               Outpt = "[A]dd" ' make display message
               Call IO.O ' send message
               Outpt = "[D]elete" ' make display message
               Call IO.O ' send message
               Outpt = "[L]ist" ' make display message
               Call IO.O ' send message
               Graphics.Off = False ' reset color
               If Container.Type Then ' check call type
                  Outpt = "Room container treasure edit option(q to quit)? " ' make input prompt
               Else ' check type
                  Outpt = "User container treasure edit option(q to quit)? " ' make input prompt
               End If ' end check type
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of room container option
                  Case "A" ' option to add container treasure
                     Outpt = "Enter item number(1-5)?" ' get item number
                     Call IO.I ' get input
                     Container.Slot = Int(Val(Inpt)) ' store input
                     If Container.Slot > False And Container.Slot <= 5 Then ' check range
                        Treasure.Item = GetContainerRecInventory(Container.Slot) ' store item index
                        If Treasure.Item = False Then ' compare value
                           Call Find.Treasure(Treasure.Number)
                           If Treasure.Number > False Then ' check number
                              Call TreasureCharges(Treasure.Charges) ' add treasure
                              Call PutContainerRecInventory(Container.Slot, Treasure.Number) ' index to treasure.dat of container contents
                              Call PutContainerRecCharges(Container.Slot, Treasure.Charges) ' treasure charges of container contents
                              Outpt = "Item is invisible(y/n)?" ' container input
                              Call IO.I ' get input
                              Call PutContainerRecInvisible(Container.Slot, Yes) ' invisible flag -1/0 of container contents
                           End If ' end check number
                        End If ' end check treasure
                     End If ' end check range
                  Case "D" ' option to delete container treasure
                     Outpt = "Enter item number(1-5)?" ' get item number
                     Call IO.I ' get input
                     Treasure.Number = Int(Val(Inpt)) ' store input
                     If Treasure.Number > False And Treasure.Number <= 5 Then ' check range
                        Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                        If Treasure.Item Then ' compare value
                           Call PutContainerRecInventory(Treasure.Number, False) ' reset container
                           Call PutContainerRecCharges(Treasure.Number, False) ' reset container
                           Call PutContainerRecInvisible(Treasure.Number, False) ' reset container
                           Outpt = "Treasure item deleted." ' format output
                           Call IO.O ' send output
                        End If ' end check value
                     End If ' end check range
                  Case "L" ' option to list container treasure
                     Graphics.Off = True ' reset color
                     For Treasure.Number = 1 To 5 ' loop through items
                        Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                        ' compare item range
                        If Treasure.Item > False And Treasure.Item <= LOF(TreasureFile) / Len(TreasureRecord) Then
                           ' display treasure name and number
                           Call Read.Record(TreasureFile, Treasure.Item) ' get treasure name
                           Outpt = RTrim$(TreasureRecord.TreasureName) ' store treasure name
                           Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
                           Outpt = "#" + Mid$(Str$(Treasure.Number), 2) + ":" + Outpt ' prepend container number
                           Call IO.O ' send output
                        End If ' end check range
                     Next ' end item loop
                  Case "Q" ' option to exit room container edit menu
                     Exit Do ' exit room container treasure edit menu
               End Select ' end selection of room container treasure edit menu
            Loop ' end loop through room container treasure edit option menu
         Case "Q" ' option to exit room container treasure edit menu
            Exit Do ' exit room treasure edit menu
      End Select ' end selection of room treasure menu
   Loop ' end loop through room container edit option menu
End Sub ' end routine to edit container

Rem * routine to edit room description and monster class.
Rem * input variables:
Rem *   Room.Number! - room number to edit.

Sub Change.Room (Room.Number!)
   Call Read.Room.Record(Room.Number!) ' get room record to edit
   Do ' loop while edit
      Call Display.Room.Desc(Room.Number!) ' routine to display room
      Graphics.Off = False ' reset color
      Outpt = "Room edit options:" ' make display output
      Call IO.O ' send output
      Graphics.Off = True ' reset color
      Outpt = "[A]ction" ' make display message
      Call IO.O ' send message
      Outpt = "[C]ontainer" ' make display message
      ContainerRec = RoomRecord.Container ' store room container into record
      If RTrim$(ContainerRec.ShortName) = Nul Then ' compare container name length
         Outpt = Outpt + ": no" ' append message
      Else ' check container name
         Outpt = Outpt + ": yes" ' append message
      End If ' end check container name
      Call IO.O ' send message
      Outpt = "[D]escription" ' make display message
      Call IO.O ' send message
      If Room.Number! > 1! Then ' check safe haven room
         Outpt = "[M]onster class" ' make display message
         Call IO.O ' send message
      End If ' end check room number
      If Room.Number! = 1! Then ' check safe haven room
         Outpt = "[R]esurrection room" ' make message
         Call IO.O ' send message
      End If ' end check room number
      Outpt = "[O]bjects:" ' make display message
      Number.Items = False ' reset counter
      For Array.Index = 1 To 20 ' loop through room objects
         Object.Number = GetRoomRecordObject(Array.Index) ' store object index
         If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then ' bounds
            Number.Items = Number.Items + 1 ' increment counter
         End If ' end check object bounds
      Next ' end count objects
      If Number.Items = 1 Then ' check value
         Outpt = Outpt + Str$(Number.Items) + " item." ' make display message
      Else ' check value
         Outpt = Outpt + Str$(Number.Items) + " items." ' make display message
      End If ' end check value
      Call IO.O ' send message
      Outpt = "[T]reasure:" ' make display message
      Number.Items = False ' reset counter
      For Array.Index = 1 To 20 ' loop through room treasure
         ' store room treasure number
         Treasure.Number = GetRoomRecordTreasure(Array.Index)
         If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
            Number.Items = Number.Items + 1 ' increment counter
         End If ' end check treasure bounds
      Next ' end treasure objects
      If Number.Items = 1 Then ' check value
         Outpt = Outpt + Str$(Number.Items) + " item." ' make display message
      Else ' check value
         Outpt = Outpt + Str$(Number.Items) + " items." ' make display message
      End If ' end check value
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Room edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of room edit option
         Case "A" ' option to change room action number
            Outpt = "Enter action number" ' prompt
            Max.Action = LOF(ActionFile) / Len(ActionRecord) ' store length of room
            ' routine to get number from range
            Call Get.Range2(0, Max.Action, Action.Number)
            RoomRecord.Action = Action.Number ' store action number
         Case "C" ' option to change room container
            Do ' loop through room container edit menu
               Graphics.Off = False ' reset color
               Outpt = "Room container edit options:" ' make display message
               Call IO.O ' send message
               Graphics.Off = True ' reset color
               Outpt = "[A]dd container" ' make option message
               Call IO.O ' send option message
               Outpt = "[C]hange container" ' make option message
               Call IO.O ' send option message
               Outpt = "[D]elete container" ' make option message
               Call IO.O ' send option message
               Outpt = "[L]ist container" ' make option message
               Call IO.O ' send option message
               Graphics.Off = False ' reset color
               Outpt = "Enter room container edit option(q to quit)? " ' make option prompt
               No.Input.Out = "Q" ' store default input
               Call IO.I ' get option input
               Select Case UCase$(Inpt) ' make selection of input option
                  Case "A" ' add container
                     Treasure.Added = False ' set container added flag
                     ContainerRec = RoomRecord.Container ' store room container into record
                     If RTrim$(ContainerRec.ShortName) = Nul Then ' compare container name length
                        ' routine to get treasure number
                        Call Find.Treasure(Treasure.Number)
                        If Treasure.Number > False Then ' check treasure number
                           If TreasureRecord.Container Then ' check treasure type
                              ' store container variables
                              ContainerRec.Closed = TreasureRecord.Closed
                              ContainerRec.ContainerName = TreasureRecord.TreasureName
                              ContainerRec.Locked = TreasureRecord.Locked
                              ContainerRec.Keyed = TreasureRecord.Keyed
                              ContainerRec.ShortName = TreasureRecord.ShortName
                              ContainerRec.Permanent = TreasureRecord.Permanent
                              For Container.Count = 1 To 5 ' loop through container contents
                                 ' clear container contents
                                 Call Clear.Container(Container.Count, False)
                              Next ' end loop through container
                              RoomRecord.Container = ContainerRec ' add container record to room
                              Treasure.Added = True ' set container added flag
                           End If ' end check treasure type
                        End If ' end check treasure number
                     End If ' end check container name length
                     If Treasure.Added Then ' check added flag
                        Outpt = "Container added to room." ' make message
                     Else ' check added flag
                        Outpt = "Container not added to room." ' make message
                     End If ' end check added flag
                     Call IO.O ' display message
                  Case "C" ' change container
                     ContainerRec = RoomRecord.Container ' store room container into record
                     Call Change.Container(-1) ' call container edit menu
                     RoomRecord.Container = ContainerRec ' store container into room record
                  Case "D" ' delete container
                     Treasure.Deleted = False ' set container deleted flag
                     ContainerRec = RoomRecord.Container ' store room container into record
                     If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
                        Call Clear.Container(0, True) ' clear entire container record
                        RoomRecord.Container = ContainerRec ' clear room container
                        Treasure.Deleted = True ' reset container deleted flag
                     End If ' end compare container in room
                     If Treasure.Deleted Then ' check deleted flag
                        Outpt = "Container deleted from room." ' make message
                     Else ' check added flag
                        Outpt = "Container not deleted from room." ' make message
                     End If ' end check added flag
                     Call IO.O ' display message
                  Case "L" ' list container
                     Graphics.Off = True ' reset color
                     ContainerRec = RoomRecord.Container ' store room container into record
                     If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
                        Outpt = RTrim$(ContainerRec.ContainerName) ' store container name
                        Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
                        If ContainerRec.Locked > False Then ' check container is locked
                           Outpt = Outpt + "[locked]" ' append message
                        Else ' container is not locked
                           If ContainerRec.Closed > False Then ' check container is closed
                              Outpt = Outpt + "[closed]" ' append message
                           End If ' end check closed container
                        End If ' end check locked container
                        If ContainerRec.Keyed Then ' check container has a key number
                           Outpt = Outpt + "(#" + Right$(Str$(ContainerRec.Keyed + 100000!), 5) + ")"
                        End If ' append key number to container name
                        Number.Items = False ' reset counter
                        For Treasure.Number = 1 To 5 ' loop through items
                           Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                           ' compare item range
                           If Treasure.Item > False And Treasure.Item <= LOF(TreasureFile) / Len(TreasureRecord) Then
                              Number.Items = Number.Items + 1 ' increment counter
                           End If ' end check range
                        Next ' end item loop
                        Outpt = Outpt + ":" + Str$(Number.Items)
                        If Number.Items = 1 Then ' check counter
                           Outpt = Outpt + " item." ' append message
                        Else ' check counter
                           Outpt = Outpt + " items." ' append message
                        End If ' end check counter
                     Else ' check for container
                        Outpt = "<no container>" ' make message
                     End If ' end compare container name length
                     Call IO.O ' display message
                  Case "Q" ' quit container menu
                     Exit Do
               End Select
            Loop ' end room container loop
         Case "D" ' option to edit room descriptions
            Do ' loop through room description edit menu
               Graphics.Off = True ' reset color
               Outpt = "[L]ong description" ' make option message
               Call IO.O ' send option message
               Outpt = "[S]hort description" ' make option message
               Call IO.O ' send option message
               Graphics.Off = False ' reset color
               Outpt = "Enter room edit option(q to quit)? " ' make option prompt
               No.Input.Out = "Q" ' store default input
               Call IO.I ' get option input
               Select Case UCase$(Inpt) ' make selection of input option
                  Case "L" ' edit long description
                     Graphics.Off = False ' reset color
                     Outpt = "Edit room long description(y/n)? " ' input prompt
                     No.Input.Out = "N" ' default input
                     Call IO.I ' get input
                     If Yes Then ' compare input
                        Graphics.Off = False ' reset color
                        Outpt = "Enter four lines for long description:" ' make message
                        Call IO.O ' send edit message
                        Outpt = "Press <enter> when done." ' make edit message
                        Call IO.O ' send edit message
                        Graphics.Off = True ' reset color
                        For Room.Desc = 1 To 4 ' loop through room long description
                           ' clear room long description
                           Call PutRoomRecordLongDesc(Room.Desc, Nul)
                        Next ' end loop through room
                        User.Word.Wrap = UserRecord.Wordwrap ' store user word wrap
                        UserRecord.Wordwrap = False ' reset word wrap
                        Word.Wrap = True ' enable word wrap
                        For Room.Desc = 1 To 4 ' loop through input for long description
                           Outpt = "?" ' make input prompt
                           If Room.Desc = 4 Then ' check last long description line
                              Word.Wrap = False ' disable word wrap
                           End If ' end check last input
                           Call IO.I ' get input
                           If No.Input Then ' check empty cr/lf entered
                              Exit For ' exit description edit loop
                           End If ' end check empty input
                           ' store long description
                           Call PutRoomRecordLongDesc(Room.Desc, Inpt)
                        Next ' end loop through input
                        Word.Wrap = False ' disable word wrap
                        UserRecord.Wordwrap = User.Word.Wrap ' restore word wrap
                     End If ' end compare input
                  Case "S" ' edit short description
                     Graphics.Off = False ' reset color
                     Outpt = "Edit room short decription(y/n)? " ' input prompt
                     No.Input.Out = "N" ' default input
                     Call IO.I ' get input
                     If Yes Then ' compare input
                        Graphics.Off = False ' reset color
                        Outpt = "Enter short description(78 characters):" ' make message
                        Call IO.O ' send message
                        Outpt = "Press <enter> to leave unchanged." ' make message
                        Call IO.O ' send message
                        Graphics.Off = True ' reset color
                        Line.Length = 78 ' set length of input
                        Outpt = "?" ' set input prompt
                        Call IO.I ' get input
                        If No.Input = False Then ' check length of input
                           RoomRecord.ShortDesc = Inpt ' store room short description
                        End If ' end check input length
                     End If ' end compare input
                  Case "Q" ' quit
                     Exit Do ' exit prompt loop
               End Select ' end selection of input
            Loop ' end loop through selection input
         Case "R" ' check resurrection room number
            If Room.Number! = 1! Then ' check safe haven room
               Outpt = "Enter resurrection room number(1-32767)?" ' make message
               Call IO.I ' get input
               RoomRecord.MonsterClass = Int(Val(Inpt)) ' reset resurrection room
               Resurrection.Room = Int(Val(Inpt)) ' store resurrection room
            End If ' end check room number
         Case "M" ' edit monster class
            If Room.Number! > 1! Then ' check safe haven room
               Outpt = "Enter monster class" ' make input prompt
               Call Get.Range2(0, Monclass.Max, Monclass.Number) ' get number from range
               RoomRecord.MonsterClass = Monclass.Number ' store new monster class
               Outpt = "Monster class" + Str$(Monclass.Number) + " added to room" + Str$(Room.Number!) + "."
               Call IO.O ' send display message
            End If ' end check room number
         Case "O" ' option to edit room objects
            Do ' loop through room object editing
               Graphics.Off = False ' reset color
               Outpt = "Room object options:" ' make display message
               Call IO.O ' send message
               Graphics.Off = True ' reset color
               Outpt = "[A]dd" ' make display message
               Call IO.O ' send message
               Outpt = "[D]elete" ' make display message
               Call IO.O ' send message
               Outpt = "[L]ist" ' make display message
               Call IO.O ' send message
               Graphics.Off = False ' reset color
               Outpt = "Room object edit option(q to quit)? " ' make input prompt
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of room object edit option
                  Case "A" ' option to add room object
                     Object.Added = False ' object added flag
                     Call Find.Objects(Object.Number) ' routine to get object number
                     If Object.Number > False Then ' check object number
                        For Array.Index = 1 To 20 ' loop through room object inventory
                           ' check empty object inventory
                           If GetRoomRecordObject(Array.Index) = False Then
                              Call PutRoomRecordObject(Array.Index, Object.Number) ' add object index
                              Object.Added = True ' set return flag
                              Exit For ' exit room object loop
                           End If ' end check empty object inventory
                        Next ' end loop through room object inventory
                     End If ' end check object number
                     If Object.Added Then ' check object added flag
                        Outpt = "Object added to room." ' make message
                     Else ' check object added flag
                        Outpt = "Object not added to room." ' make message
                     End If ' end check object added flag
                     Call IO.O ' send message
                  Case "D" ' option to delete room object
                     Outpt = "Object number to delete" ' make range prompt
                     Call Get.Range2(0, 10, Object.Number) ' get number from range
                     If Object.Number > False Then ' check number bounds
                        Call PutRoomRecordObject(Object.Number, False) ' clear treasure items
                        Outpt = "Object " + Str$(Object.Number) + " deleted from room." ' make message
                        Call IO.O ' send message
                     End If ' end check number bounds
                  Case "L" ' option to list objects in room
                     Objects.Counted = False ' reset counter
                     Graphics.Off = True ' reset color
                     For Array.Index = 1 To 20 ' loop through room objects
                        Object.Number = GetRoomRecordObject(Array.Index) ' store object index
                        If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then ' bounds
                           Objects.Counted = True ' set counter
                           Call Read.Record(ObjectFile, Object.Number) ' read object record
                           Outpt = "[" + Mid$(Str$(Array.Index), 2) + "]" + RTrim$(ObjectRecord.ObjectName) ' make object name display
                           Call IO.O ' send object message
                        End If ' end check object file bounds
                     Next ' end loop through room objects
                     If Objects.Counted Then ' check counter
                        Call More.Prompt ' display more prompt
                     Else ' check counter
                        Outpt = "<no objects>" ' make display message
                        Call IO.O ' send message
                     End If ' end check counter
                  Case "Q" ' option to exit room object edit menu
                     Exit Do ' exit room object edit menu
               End Select ' end select room object edit options
            Loop ' end loop through room object option menu
         Case "T" ' option to edit room treasure
            Do ' loop through room treasure edit menu
               Graphics.Off = False ' reset color
               Outpt = "Room treasure options:" ' make display message
               Call IO.O ' send message
               Graphics.Off = True ' reset color
               Outpt = "[A]dd" ' make display message
               Call IO.O ' send message
               Outpt = "[D]elete" ' make display message
               Call IO.O ' send message
               Outpt = "[L]ist" ' make display message
               Call IO.O ' send message
               Graphics.Off = False ' reset color
               Outpt = "Room treasure edit option(q to quit)? " ' make input prompt
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of room treasure option
                  Case "A" ' option to add room treasure
                     Treasure.Added = False ' set treasure added flag
                     ' routine to get treasure number
                     Call Find.Treasure(Treasure.Number)
                     If Treasure.Number > False Then ' check treasure number
                        If TreasureRecord.Container = False Then ' check treasure type
                           For Array.Index = 1 To 20 ' loop through room treasure inventory
                              ' check empty room inventory
                              If GetRoomRecordTreasure(Array.Index) = False Then
                                 Call PutRoomRecordTreasure(Array.Index, Treasure.Number) ' add treasure index
                                 ' add treasure charges
                                 Call TreasureCharges(Charges.Number)
                                 Call PutRoomRecordTreCharges(Array.Index, Charges.Number)
                                 Call PutRoomRecordFlags(Array.Index, False) ' add treasure flags
                                 Treasure.Added = True ' set return flag
                                 Exit For ' exit room inventory loop
                              End If ' end check empty inventory
                           Next ' end loop through room treasure inventory
                        End If ' end check treasure type
                     End If ' end check treasure number
                     If Treasure.Added Then ' check treasure added flag
                        Outpt = "Treasure added to room." ' make message
                     Else ' check flag
                        Outpt = "Treasure not added to room." ' make message
                     End If ' end check treasure added flag
                     Call IO.O ' send message
                  Case "D" ' option to delete treasure number
                     Outpt = "Treasure number to delete" ' make range prompt
                     Call Get.Range2(0, 10, Treasure.Number) ' get number from range
                     If Treasure.Number > False Then ' check number bounds
                        Call PutRoomRecordTreasure(Treasure.Number, False) ' clear treasure items
                        Call PutRoomRecordTreCharges(Treasure.Number, False) ' clear treasure items
                        Call PutRoomRecordFlags(Treasure.Number, False) ' clear treasure items
                        Outpt = "Treasure " + Str$(Treasure.Number) + " deleted from room." ' make message
                        Call IO.O ' send message
                     End If ' end check number bounds
                  Case "L" ' option to list treasure in room
                     Treasure.Counted = False ' reset counter
                     Graphics.Off = True ' reset color
                     For Array.Index = 1 To 20 ' loop through room treasure
                        ' store room treasure number
                        Treasure.Number = GetRoomRecordTreasure(Array.Index)
                        If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
                           Treasure.Counted = True ' increment counter
                           Call Read.Record(TreasureFile, Treasure.Number) ' read treasure
                           Outpt = "[" + Mid$(Str$(Array.Index), 2) + "]" + RTrim$(TreasureRecord.TreasureName) ' make treasure name
                           Call IO.O ' send treasure name message
                        End If ' end check file bounds
                     Next ' end loop through room treasure
                     If Treasure.Counted Then ' check counter
                        Call More.Prompt ' display more prmpt
                     Else ' check counter
                        Outpt = "<no treasure>" ' make display message
                        Call IO.O ' send message
                     End If ' end check counter
                  Case "Q" ' option to exit room treasure edit menu
                     Exit Do ' exit room treasure edit menu
               End Select ' end selection of room treasure menu
            Loop ' end loop through room treasure edit option menu
         Case "Q" ' option to exit room edit menu
            Call Share.Room.Record(Room.Number!) ' write current room number
            Exit Do ' exit room edit menu
      End Select ' end selection of room edit menu
   Loop ' end loop through room edit option menu
End Sub ' end routine to edit room number

Rem * routine to edit message text.
Rem * output variables:
Rem *   Message.Edit - true to continue, false to abort, 1 to store message.

Sub Edit.Message (Message.Edit)
   Do ' loop through message edit menu
      Graphics.Off = False ' reset color
      Outpt = "Editing options:" ' make dislay message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]bort" ' make dislay message
      Call IO.O ' send message
      Outpt = "[C]ontinue" ' make dislay message
      Call IO.O ' send message
      If Message.Length > False Then ' check length of message
         Outpt = "[D]elete" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      If Message.Length > False Then ' check length of message
         Outpt = "[E]dit" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      If Message.Length > False Then ' check length of message
         Outpt = "[I]nsert" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      If Message.Length > False Then ' check length of message
         Outpt = "[L]ist" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      If Message.Length > False Then ' check length of message
         Outpt = "[R]eplace" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      If Message.Length > False Then ' check length of message
         Outpt = "[S]tore" ' make dislay message
         Call IO.O ' send message
      End If ' end check length
      Graphics.Off = False ' reset color
      Outpt = "Edit command? " ' make input prompt
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of edit option
         Case "C" ' option to continue message
            Message.Edit = True ' set return variable
            Exit Sub ' exit routine
         Case "R" ' option to replace line
            If Message.Length > False Then ' check length of message
               Call Replace.Line ' replace routine
            End If ' end check length
         Case "E" ' option to edit line
            If Message.Length > False Then ' check length of message
               Call Edit.Line ' line edit routine
            End If ' end check length
         Case "D" ' option to delete line
            If Message.Length > False Then ' check length of message
               Call Delete.Line ' delete routine
            End If ' end check length
         Case "I" ' option to insert lines
            If Message.Length > False Then ' check length of message
               Call Insert.Lines ' insert routine
            End If ' end check length
         Case "A" ' option to abort message
            Outpt = "Are you sure you want to abort(y/n)? " ' make input prompt
            No.Input.Out = "N" ' default input
            Call IO.I ' get user input
            If Yes Then ' check input response
               Message.Edit = False ' set return variable
               Exit Sub ' exit routine
            End If ' end check response
         Case "L" ' option to list lines
            If Message.Length > False Then ' check length of message
               Call List.Lines ' line list routine
            End If ' end check length
         Case "S" ' option to store message
            If Message.Length > False Then ' check length of message
               Message.Edit = UnTrue ' set return variable
               Exit Sub ' exit routine
            End If ' end check length
      End Select ' end selection of input
   Loop ' end loop through edit menu
End Sub ' end edit menu routine

Rem * routine to edit messages files.

Sub Edit.Mail
   Do ' loop through mail edit menu
      Graphics.Off = False ' reset color
      Outpt = "Mail edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[D]elete" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Outpt = "[P]ack" ' make display message
      Call IO.O ' send message
      Outpt = "[U]ndelete" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Message edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' select option
         Case "C" ' change option
            Call Change.Message ' routine to change message
         Case "D" ' delete option
            Call Delete.Message ' routine to delete message
         Case "L" ' list option
            Call List.Messages ' routine to list messages
         Case "P" ' pack option
            Call Pack.Messages ' routine to pack messages
         Case "U" ' undelete option
            Call Undelete.Message ' routine to undelete message
         Case "Q" ' exit mail edit menu option
            Exit Do ' exit edit mail menu
      End Select ' end mail edit option selection
   Loop ' end mail edit menu loop
End Sub ' end mail edit menu routine

Rem * routine to parse out number after pound sign in a string
Rem * input variables:
Rem *   Parsed.Input$ - string to check.
Rem * output variables:
Rem *   Parsed.Input$ - string truncated before pound sign, lowercased.
Rem *   Parsed.Value - number parsed after pound sign.
Rem * work variables:
Rem *   Parsed.Token - contains position of # sign.

Sub Parse.Num (Parsed.Input$, Parsed.Value)
   Parsed.Input$ = LCase$(Parsed.Input$) ' lowercase string
   Parsed.Value = False ' set return variable
   Parsed.Token = InStr(Parsed.Input$, "#") ' search string for pound sign
   If Parsed.Token > False Then ' check string search
      ' set return variable
      Parsed.Value = Int(Val(Mid$(Parsed.Input$, Parsed.Token + 1)))
      Parsed.Input$ = Left$(Parsed.Input$, Parsed.Token - 1) ' truncate string
   End If ' end check string
End Sub ' end routine

Rem * routine to search player and room treasure for parameter name
Rem * input variables:
Rem *   Parsed.Command1 - name of treasure.
Rem * output variables:
Rem *   Charges.Number - charges of treasure.
Rem *   Index.Number - index of treasure to file.
Rem *   TypeNumber - 0 for treasure in inventory, 1 for treasure in room.

Sub Examine.Treasure
   TypeNumber = False ' store treasure flag
   Call Check.Inventory.Treasure ' routine to search player inventory treasure
   If Index.Number = False Then ' check player treasure found
      Call Num ' decrement counters
      TypeNumber = 1 ' store treasure flag
      Call Check.Room.Treasure ' routine to search room inventory treasure
   End If ' end check player treasure found
End Sub ' end routine to search inventory treasure for treasure name

Rem * routine to search player and room objects for parameter name
Rem * input variables:
Rem *   Parsed.Command1 - name of object.
Rem * output variables:
Rem *   Charges.Number - charges of object.
Rem *   Index.Number - index of object to file.
Rem *   TypeNumber - 0 for object in inventory, 1 for object in room.

Sub Examine.Objects
   TypeNumber = False ' store object flag
   Call Check.Inventory.Objects ' routine to search player inventory objects
   If Index.Number = False Then ' check player inventory object found
      Call Num ' decrement counters
      TypeNumber = 1 ' store object flag
      Call Check.Room.Objects ' routine to search room inventory objects
   End If ' end check player inventory object found
End Sub ' end routine to search inventory objects for object name

Rem * routine to search player inventory for treasure name
Rem * input variables:
Rem *   Parsed.Command2 - contains command parameter.
Rem * output variables:
Rem *   Charges.Number - treasure charges.
Rem *   Index.Number - treasure index to file.

Sub Find.Inventory
   Call Parse ' parse command parameter
   If Parser Then ' check for parsed command
      Call Numeric ' store # sign counter
   End If ' end check parsed command
   Call Check.Inventory.Treasure ' routine to search player treasure inventory
End Sub ' end routine to search player treasure inventory

Rem * routine to search player inventory for object name
Rem * input variables:
Rem *   Parsed.Command2 - contains command parameter.
Rem * output variables:
Rem *   Charges.Number - object charges.
Rem *   Index.Number - object index to file.

Sub Find.Object
   Call Parse ' parse command parameter
   If Parser Then ' check for parsed command
      Call Numeric ' store # sign counter
   End If ' end check parsed command
   Call Check.Inventory.Objects ' routine to search player object inventory
End Sub ' end routine to search player object inventory

Rem * routine searches for treasure name in room treasure inventory
Rem * input variables:
Rem *   Parsed.Command1 - name of treasure to search for.
Rem *   Parse.Number - number increment of treasure.
Rem * output variables:
Rem *   Charges.Number - treasure charges.
Rem *   Index.Number - treasure file index.
Rem *   Outpts - treasure name.
Rem * processing variables:
Rem *   Parse.Count - counter of treasure found in search.

Sub Check.Room.Treasure
   Charges.Number = False ' reset treasure charges
   Index.Number = False ' reset treasure index
   Parse.Count = False ' reset treasure counter
   If Parsed.Command1 <> Nul Then ' compare search string length
      For Array.Number = 1 To 20 ' loop through room inventory
         If GetRoomRecordTreasure(Array.Number) Then ' check room inventory index
            ' get treasure record
            Call Read.Record(TreasureFile, GetRoomRecordTreasure(Array.Number))
            Outpts = TreasureRecord.ShortName ' store treasure name
            ' trim length of treasure name to length of search string
            Outpts = Left$(Outpts, Len(Parsed.Command1)) ' trim
            ' compare treasure name to search name
            If Outpts = Parsed.Command1 Then
               Parse.Count = Parse.Count + 1 ' increment counter
               ' check increment counter not specified,
               ' or counter equals increment counter.
               If Parse.Number = False Or Parse.Count = Parse.Number Then
                  Outpts = TreasureRecord.TreasureName ' get treasure name
                  Outpts = RTrim$(Outpts) ' trim name
                  ' store treasure index
                  Index.Number = GetRoomRecordTreasure(Array.Number)
                  ' store treasure charges
                  Charges.Number = GetRoomRecordTreCharges(Array.Number)
                  Exit For ' exit loop through room treasure inventory
               End If ' end check counters equal
            End If ' end compare treasure names
         End If ' end check room inventory index
      Next ' end loop through room treasure inventory
   End If ' end compare search string length
End Sub ' end room treasure inventory search

Rem * routine searches for object name in room object inventory
Rem * input variables:
Rem *   Parsed.Command1 - name of object to search for.
Rem *   Parse.Number - number increment of object.
Rem * output variables:
Rem *   Charges.Number - object charges.
Rem *   Index.Number - object file index.
Rem *   Outpts - object name.
Rem * processing variables:
Rem *   Parse.Count - counter of object found in search.

Sub Check.Room.Objects
   Charges.Number = False ' reset object charges
   Index.Number = False ' reset object index
   Parse.Count = False ' reset object counter
   If Parsed.Command1 <> Nul Then ' check search string length
      For Array.Number = 1 To 20 ' loop through room object inventory
         If GetRoomRecordObject(Array.Number) Then ' check room object index
            ' get room object record
            Call Read.Record(ObjectFile, GetRoomRecordObject(Array.Number))
            Outpts = ObjectRecord.ShortName ' store room object name
            ' trim length of object name to length of search string
            Outpts = Left$(Outpts, Len(Parsed.Command1)) ' trim
            ' compare object name to search string
            If Outpts = Parsed.Command1 Then
               Parse.Count = Parse.Count + 1 ' increment counter
               ' check increment counter not specified,
               ' or counter equals increment counter.
               If Parse.Number = False Or Parse.Count = Parse.Number Then
                  Outpts = ObjectRecord.ObjectName ' store object name
                  Outpts = RTrim$(Outpts) ' trim name
                  ' store object index
                  Index.Number = GetRoomRecordObject(Array.Number)
                  Exit For ' exit loop through room object inventory
               End If ' end check counters
            End If ' end compare object names
         End If ' end check room object index
      Next ' end loop through room object inventory
   End If ' end check search string length
End Sub ' end routine to search room object inventory

Rem * routine searches for treasure name in player treasure inventory
Rem * input variables:
Rem *   Parsed.Command1 - name of treasure to search for.
Rem *   Parse.Number - number increment of treasure.
Rem * output variables:
Rem *   Charges.Number - treasure charges.
Rem *   Index.Number - treasure file index.
Rem *   Outpts - treasure name.
Rem * processing variables:
Rem *   Parse.Count - counter of treasure found in search.

Sub Check.Inventory.Treasure
   Charges.Number = False ' reset treasure charges
   Index.Number = False ' reset treasure index
   Parse.Count = False ' reset treasure counter
   If Parsed.Command1 <> Nul Then ' compare search string length
      For Array.Number = 1 To 20 ' loop through player treasure inventory
         If GetUserRecordInv(Array.Number) Then ' check player inventory index
            ' get treasure record
            Call Read.Record(TreasureFile, GetUserRecordInv(Array.Number))
            Outpts = TreasureRecord.ShortName ' store treasure name
            ' trim length of treasure name to length of search string
            Outpts = Left$(Outpts, Len(Parsed.Command1)) ' trim
            ' compare treasure name to search string
            If Outpts = Parsed.Command1 Then
               Parse.Count = Parse.Count + 1 ' increment counter
               ' check increment counter not specified,
               ' or counter equals increment counter.
               If Parse.Number = False Or Parse.Count = Parse.Number Then
                  Outpts = TreasureRecord.TreasureName ' store treasure name
                  Outpts = RTrim$(Outpts) ' trim name
                  ' store treasure index
                  Index.Number = GetUserRecordInv(Array.Number)
                  ' store treasure charges
                  Charges.Number = GetUserRecordCharges(Array.Number)
                  Exit For ' exit loop through player treasure inventory
               End If ' end check counters
            End If ' end compare treasure names
         End If ' end check player treasure inventory index
      Next ' end loop through player treasure inventory
   End If ' end check search string length
End Sub ' end routine to search player treasure inventory for treasure name

Rem * routine searches for object name in player object inventory
Rem * input variables:
Rem *   Parsed.Command1 - name of object to search for.
Rem *   Parse.Number - number increment of object.
Rem * output variables:
Rem *   Charges.Number - object charges.
Rem *   Index.Number - object file index.
Rem *   Outpts - object name.
Rem * processing variables:
Rem *   Parse.Count - counter of object found in search.

Sub Check.Inventory.Objects
   Charges.Number = False ' reset object charges
   Index.Number = False ' reset object index
   Parse.Count = False ' reset counter
   If Parsed.Command1 <> Nul Then ' check search strig length
      For Array.Number = 1 To 20 ' loop through player object inventory
         If GetUserRecordObject(Array.Number) Then ' check player object index
            ' get object record
            Call Read.Record(ObjectFile, GetUserRecordObject(Array.Number))
            Outpts = ObjectRecord.ShortName ' store object name
            ' trim length of object name to length of search string
            Outpts = Left$(Outpts, Len(Parsed.Command1)) ' trim
            ' compare object name to search string
            If Outpts = Parsed.Command1 Then
               Parse.Count = Parse.Count + 1 ' increment counter
               ' check increment counter not specified,
               ' or counter equals increment counter.
               If Parse.Number = False Or Parse.Count = Parse.Number Then
                  Outpts = ObjectRecord.ObjectName ' store objecct name
                  Outpts = RTrim$(Outpts) ' trim name
                  ' store object index
                  Index.Number = GetUserRecordObject(Array.Number)
                  Exit For ' exit loop through player object inventory
               End If ' end check counters
            End If ' end compare object names
         End If ' end check player object index
      Next ' end loop through player object inventory
   End If ' end check search string length
End Sub ' end routine to search player object inventory

Rem * routine to search for monster name in room monsters
Rem * input variables:
Rem *   Parsed.Command1 - name of monster to search for.
Rem * output variables:
Rem *   Monster.Number - number of monster array.
Rem *   Last.Monster - name of monster found.
Rem * work variables:
Rem *   Word.Parse1, Word.Parse2.

Sub Check.Monster
   Array.Number = False ' reset monster number loop counter
   Parse.Count = False ' reset increment counter
   Monster.Number = False ' reset monster number
   If Parsed.Command1 <> Nul Then ' check search string length
      For Array.Number = 1 To Number.Monsters ' loop through all room monsters
         Word.Parse1 = False ' reset monster name parse variable
         Word.Parse2 = False ' reset monster name parse variable
         Outpts = MonsterArray(Array.Number).MonsterName ' store monster name
         Outpts = RTrim$(Outpts) ' trim name
         Outpts = UCase$(Outpts) ' uppercase name
         ' locate first imbedded space in name
         Word.Parse1 = InStr(Word.Parse1 + 1, Outpts, " ")
         While Word.Parse1 ' loop until last space in monster name found
            Word.Parse2 = Word.Parse1 ' store position of last space
            Word.Parse1 = InStr(Word.Parse1 + 1, Outpts, " ") ' locate next space
         Wend ' end loop
         ' monster name is last word of full name
         Inpt = Mid$(Outpts, Word.Parse2 + 1)
         ' trim length of monster name to length of search string
         Inpt = Left$(Inpt, Len(Parsed.Command1)) ' trim
         If Inpt = Parsed.Command1 Then ' compare monster name to search string
            Parse.Count = Parse.Count + 1 ' increment counter
            ' check increment counter not specified,
            ' or counter equals increment counter.
            If Parse.Number = False Or Parse.Count = Parse.Number Then
               ' store monster name
               Outpts = MonsterArray(Array.Number).MonsterName
               Outpts = RTrim$(Outpts) ' trim name
               Monster.Number = Array.Number ' store monster number
               Last.Monster = Parsed.Command1 ' store monster name
               Exit For ' exit loop through room monsters
            End If ' end check counters
         End If ' end compare search strings
      Next ' end loop through room monsters
   End If ' end check search string length
End Sub ' end routine to search room monsters for monster name

Rem * Filename: dnds7.bas Version: v5.0a r2.0a
Rem * This subprogram contains all the main editor routines.

Rem * main routine to enter the editor and process edit selections.

Sub Editor
   Call Put.User.Record ' store current user settings
   Graphics.Off = False ' reset ansi color
   Outpt = "Edit utility v" + Version$ ' editor intro prompt
   Call IO.O ' send output
   Do ' loop through edit selections
      Graphics.Off = True ' reset color
      Outpt = "[A]ction" ' edit selection
      Call IO.O ' send output
      Outpt = "[E]mail" ' edit selection
      Call IO.O ' send output
      Outpt = "[H]elp" ' edit selection
      Call IO.O ' send output
      Outpt = "[L]inks" ' edit selection
      Call IO.O ' send output
      Outpt = "[M]onster" ' edit selection
      Call IO.O ' send output
      Outpt = "[N]onplayer" ' edit selection
      Call IO.O ' send output
      Outpt = "[O]bject" ' edit selection
      Call IO.O ' send output
      Outpt = "[R]oom" ' edit selection
      Call IO.O ' send output
      Outpt = "[S]pell" ' edit selection
      Call IO.O ' send output
      Outpt = "[T]reasure" ' edit selection
      Call IO.O ' send output
      Outpt = "[U]ser" ' edit selection
      Call IO.O ' send output
      Graphics.Off = False ' reset ansi color
      Outpt = "Edit option(q to quit)? " ' display edit prompt
      No.Input.Out = "Q" ' default entry
      Call IO.I ' get input
      Select Case UCase$(Inpt) ' selection of input choice
         Case "A" ' action selection
            Call Edit.Action ' action editor
         Case "E" ' email selection
            Call Edit.Mail ' email editor
         Case "H" ' help selection
            Call Edit.Help ' !edit help
         Case "L" ' option to edit room links
            Call Link.Room ' room link edit
         Case "M" ' monster selection
            Call Edit.Monster(Rate.Changed) ' monster editor
            If Rate.Changed Then ' check to reread monster class
               Call Read.Monclass ' reread monster class arrays
            End If ' end check monster class
         Case "N" ' nonplayer selection
            Call Edit.Nonplayer ' nonplayer editor
         Case "O" ' object selection
            Call Edit.Object ' object editor
         Case "R" ' rooms selection
            Call Edit.Room ' room editor
         Case "S" ' spell selection
            Call Edit.Spell ' spell editor
         Case "T" ' treasure selection
            Call Edit.Treasure ' treasure editor
         Case "U" ' user selection
            Call Share.Record(UserFile, User.Index) ' store player record
            Call Edit.User ' user editor
            Call Read.Record(UserFile, User.Index) ' read player record
         Case "Q" ' quit selection
            Exit Do ' exit editor
      End Select ' end choice selection
   Loop ' end edit loop
   Call Get.User.Record ' restore user stats
End Sub ' end main routine to select edit option

Rem * routine to edit actions.

Sub Edit.Action
   Do ' loop through action edit menu
      Graphics.Off = False ' reset color
      Outpt = "Action edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Action edit option(q to quit)? " ' option input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of option input
         Case "A" ' add action option
            New.Record = LOF(ActionFile) / Len(ActionRecord) + 1
            ActionRecord.SpellTrigger = False
            ActionRecord.MonsterTrigger = False
            ActionRecord.HealthRate = False
            ActionRecord.EncounterRate = False
            ActionRecord.HitPoints = False
            ActionRecord.Inventory = False
            ActionRecord.MonsterTalk = False
            ActionRecord.Fumble = False
            ActionRecord.Teleport = False
            ActionRecord.Level = False
            ActionRecord.Restrictions = False
            ActionRecord.RustRate = False
            ActionRecord.StealRate = False
            ActionRecord.Attribute1 = False
            ActionRecord.Attribute2 = False
            Call Share.Record(ActionFile, New.Record)
            Outpt = "Action" + Str$(New.Record) + " added." ' make message
            Call IO.O ' send message
         Case "L" ' option to list actions
            Max.Actions = LOF(ActionFile) / Len(ActionRecord) ' store action records
            ' routine to get range values
            Call Get.Range(Max.Actions, Action.List1, Action.List2)
            ' loop through range values
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Action.Count = Action.List1 To Action.List2
               Call Read.Record(ActionFile, Action.Count) ' read action record
               Call Display.Action(Action.Count) ' routine to display actions
               If Break Then ' check control-k flag
                  Exit For ' exit display loop
               End If ' end check control-k
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check pause input
                     Exit For ' exit action list loop
                  End If ' end check input
               End If ' end check continuous flag
            Next ' end loop through actions
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "C"
            Outpt = "Enter action number" ' make range prompt
            Max.Actions = LOF(ActionFile) / Len(ActionRecord) ' store action records
            ' routine to get range of numbers
            Call Get.Range2(0, Max.Actions, Action.Number)
            If Action.Number > False Then ' check number range
               Call Read.Record(ActionFile, Action.Number) ' get action record
               Do ' loop through action edit menu
                  Call Display.Action(Action.Number)
                  Graphics.Off = False ' reset color
                  Outpt = "Action edit option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of action edit option
                     Case "A" ' option for monster encounter trigger
                        ' routine to get monster number
                        Call Find.Monster(Monster.Number)
                        ' store monster number trigger
                        ActionRecord.MonsterTrigger = Monster.Number
                     Case "B" ' option for spell trigger
                        ' routine to get spell number
                        Call Find.Spell(Spell.Number)
                        ' store spell number trigger
                        ActionRecord.SpellTrigger = Spell.Number
                     Case "C" ' option monster talk number
                        ' routine to get monster number
                        Call Find.Monster(Monster.Number)
                        ' store monster talk number trigger
                        ActionRecord.MonsterTalk = Monster.Number
                     Case "D" ' option for player level entry trigger
                        Graphics.Off = True ' reset color
                        ' make message
                        Outpt = "[H]ighest level (at least) for entry to room"
                        Call IO.O ' send level trigger display message
                        ' make message
                        Outpt = "[L]owest level (at most) for entry to room"
                        Call IO.O ' send level trigger display message
                        Graphics.Off = False ' reset color
                        ' make input prompt
                        Outpt = "Action level entry option(q to quit)? "
                        No.Input.Out = "Q" ' default prompt
                        Call IO.I ' get user input
                        Select Case UCase$(Inpt) ' selection of level entry option
                           Case "H" ' high level entry option
                              Outpt = "Highest level player must be (at least) to enter room? "
                              No.Input.Out = "1" ' default input
                              Call IO.I ' get user input
                              ActionRecord.Level = Int(Val(Inpt)) ' store level entry trigger
                           Case "L" ' low level entry option
                              Outpt = "Lowest level player must be (at most) to enter room? "
                              No.Input.Out = "1" ' default input
                              Call IO.I ' get user input
                              ActionRecord.Level = -Int(Val(Inpt)) ' store level entry trigger
                        End Select
                        Do
                           Graphics.Off = False
                           Outpt = "Direction options:"
                           Call IO.O
                           Graphics.Off = True ' reset color
                           Outpt = "[N]orth" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[E]ast" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[S]outh" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[W]est" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[NE]northeast" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[SE]southeast" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[SW]southwest" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[NW]northwest" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[U]p" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[D]own" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[I]n" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[O]ut" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[G]o to portal" ' make direction display message
                           Call IO.O ' send direction display message
                           Outpt = "[X]clear directions" ' make direction display message
                           Call IO.O ' send direction message
                           Graphics.Off = False ' reset color
                           ' input prompt
                           Outpt = "Action entry direction option(q to quit)? "
                           No.Input.Out = "Q" ' default prompt
                           Call IO.I ' get user input
                           If UCase$(Inpt) = "Q" Then ' compare input to quit option
                              Exit Do ' exit direction restriction edit loop
                           End If ' end compare input option
                           If UCase$(Inpt) = "X" Then ' compare clear directions
                              ActionRecord.Restrictions = False ' clear directions
                           Else ' compare to direction
                              ' routine to get direction number
                              Call Find.Link(Inpt, Direction.Number, False)
                              If Direction.Number > False Then ' compare direction number
                                 ' set the direction bit in the action record trigger flag
                                 ActionRecord.Restrictions = ActionRecord.Restrictions Or 2 ^ Direction.Number
                              End If ' end compare clear direction
                           End If ' end compare direction number
                        Loop ' loop through level entry trigger edit loop
                     Case "E" ' option for health hits result
                        Graphics.Off = True ' reset color
                        Outpt = "[F]atigue" ' make health message
                        Call IO.O ' send health message
                        Outpt = "[V]itality" ' make health message
                        Call IO.O ' send health message
                        Graphics.Off = False ' reset color
                        ' make input prompt
                        Outpt = "Action health rate option(q to quit)? "
                        No.Input.Out = "Q" ' default input
                        Call IO.I ' get user input
                        Select Case UCase$(Inpt) ' selection of health option
                           Case "F" ' option for action to hit for fatigue
                              ' make input prompt
                              Outpt = "Number of hits of fatigue(1-32767)? "
                              No.Input.Out = "10" ' default input
                              Call IO.I ' get user input
                              ' store health hit result
                              ActionRecord.HitPoints = Int(Val(Inpt))
                           Case "V" ' option for action to hit for vitality
                              ' make input prompt
                              Outpt = "Number of hits of vitality(1-32767)? "
                              No.Input.Out = "10" ' default input
                              Call IO.I ' get user input
                              ' store health hit result
                              ActionRecord.HitPoints = -Int(Val(Inpt))
                        End Select ' end selection health edit option
                     Case "F" ' option to edit action rates
                        Graphics.Off = True ' reset color
                        Outpt = "[E]ncounter" ' make rate display message
                        Call IO.O ' send rate display message
                        Outpt = "[H]ealth" ' make rate display message
                        Call IO.O ' send rate display message
                        Graphics.Off = False ' reset color
                        Outpt = "Action rate option(q to quit)? " ' make input prompt
                        No.Input.Out = "Q" ' default input
                        Call IO.I ' get user input
                        Select Case UCase$(Inpt) ' selection of rate option
                           Case "E" ' option to change action monster encounter rate
                              Outpt = "Encounter rate(1-32767)? " ' make input prompt
                              No.Input.Out = "6" ' default input
                              Call IO.I ' get user input
                              ' store encounter rate
                              ActionRecord.EncounterRate = Int(Val(Inpt))
                           Case "H" ' option to change action health update rate
                              Outpt = "Health rate(1-32767)? " ' make input prompt
                              No.Input.Out = "6" ' default input
                              Call IO.I ' get user input
                              ' store health update rate
                              ActionRecord.HealthRate = Int(Val(Inpt))
                        End Select ' end selection of rate option
                     Case "G" ' option to edit inventory result options
                        Graphics.Off = True ' reset color
                        Outpt = "[W]eapons" ' make inventory display message
                        Call IO.O ' send inventory display message
                        Outpt = "[S]hields" ' make inventory display message
                        Call IO.O ' send inventory display messagec
                        Outpt = "[A]rmor" ' make inventory display message
                        Call IO.O ' send inventory display message
                        Outpt = "[M]agic items" ' make inventory display message
                        Call IO.O ' send inventory display message
                        Graphics.Off = False ' reset color
                        Outpt = "Action inventory option(q to quit)? " ' make input prompt
                        No.Input.Out = "Q" ' default input
                        Call IO.I ' get user input
                        Select Case UCase$(Inpt) ' selection of inventory option
                           Case "W" ' option for inventory result
                              ActionRecord.Inventory = 1 ' store inventory option number
                           Case "S" ' option for inventory result
                              ActionRecord.Inventory = 2 ' store inventory option number
                           Case "A" ' option for inventory result
                              ActionRecord.Inventory = 3 ' store inventory option number
                           Case "M" ' option for inventory result
                              ActionRecord.Inventory = 4 ' store inventory option number
                        End Select ' end selection of inventory option
                     Case "H" ' option for fumble result
                        ActionRecord.Fumble = Not ActionRecord.Fumble ' set fumble flag
                     Case "I" ' option for teleport result
                        Outpt = "Room number" ' make range prompt
                        Max.Rooms! = LOF(RoomFile) / Len(RoomRecord) ' store room file record
                        ' routine to get range of numbers
                        Call Get.Room.Range2(0!, Max.Rooms!, Room.Number!)
                        ' store room teleport result number
                        ActionRecord.Teleport = Room.Number!
                     Case "J" ' option to change rust rate
                        ' input prompt
                        Outpt = "Rate for weapons to rust(# of action prompts)? "
                        No.Input.Out = "6" ' default input
                        Call IO.I ' get user input
                        RustRate = Int(Val(Inpt)) ' convert input to integer
                        ActionRecord.RustRate = RustRate ' store rust rate
                     Case "K" ' option to change steal rate
                        Outpt = "Rate for monsters to steal treasure(# of action prompts)? "
                        No.Input.Out = "6" ' default input
                        Call IO.I ' get user input
                        StealRate = Int(Val(Inpt)) ' convert input to integer
                        ActionRecord.StealRate = StealRate ' store steal rate
                     Case "L"
                        Do ' loop through attribute editing
                           Graphics.Off = True ' reset color
                           Outpt = "[A]ir" ' make option message
                           Call IO.O ' send message
                           Outpt = "[L]and" ' make option message
                           Call IO.O ' send message
                           Outpt = "[W]ater" ' make option message
                           Call IO.O ' send message
                           Outpt = "[U]nlit" ' make option message
                           Call IO.O ' send message
                           Outpt = "[X]Clear" ' make option message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           ' make input prompt
                           Outpt = "Action attribute option(q to quit)? "
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of attribute option
                              Case "A" ' option for room attribute
                                 ActionRecord.Attribute2 = Air ' set flag
                              Case "L" ' option for room attribute
                                 ActionRecord.Attribute2 = Land ' set flag
                              Case "W" ' option for room attribute
                                 ActionRecord.Attribute2 = Water ' set flag
                              Case "U" ' option for room attribute
                                 ActionRecord.Attribute1 = UnlitRoom ' reset flag
                              Case "X" ' option to clear room attributes
                                 ActionRecord.Attribute1 = False ' reset room attribute
                                 ActionRecord.Attribute2 = False ' reset room attribute
                              Case "Q" ' option to exit room attribute editing
                                 Exit Do ' exit room attribute editing
                           End Select ' end selection of room attribute editing
                        Loop ' end loop through room attribute editing menu options
                     Case "X" ' option to clear triggers and results
                        ActionRecord.SpellTrigger = False
                        ActionRecord.MonsterTrigger = False
                        ActionRecord.HealthRate = False
                        ActionRecord.EncounterRate = False
                        ActionRecord.HitPoints = False
                        ActionRecord.Inventory = False
                        ActionRecord.MonsterTalk = False
                        ActionRecord.Fumble = False
                        ActionRecord.Teleport = False
                        ActionRecord.Level = False
                        ActionRecord.Restrictions = False
                        ActionRecord.RustRate = False
                        ActionRecord.StealRate = False
                        ActionRecord.Attribute1 = False
                        ActionRecord.Attribute2 = False
                        Outpt = "Action cleared." ' make clear message
                        Call IO.O ' send clear message
                     Case "Q" ' option to exit room action edit menu
                        Call Share.Record(ActionFile, Action.Number) ' write action file
                        Outpt = "Action" + Str$(Action.Number) + " changed."
                        Call IO.O ' send display message
                        Exit Do ' exit room action edit menu
                  End Select ' end selection of action edit options
               Loop ' end loop through action edit menu
            End If ' end check range
         Case "Q" ' selection to quit action editing
            Exit Do ' exit action editing
      End Select ' end action editing selection
   Loop ' end loop through room action menu
End Sub ' end routine to edit room actions

Rem * routine to edit monsters file.
Rem * output variables:
Rem *   Rate.Changed  -  true if any monster rate edited

Sub Edit.Monster (Rate.Changed)
   Rate.Changed = False ' reset flag
   Do ' loop through monster editing menu
      Graphics.Off = False ' reset color
      Outpt = "Monster edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make monster edit display message
      Call IO.O ' send monster edit display message
      Outpt = "[C]hange" ' make monster edit display message
      Call IO.O ' send monster edit display message
      Outpt = "[L]ist" ' make monster edit display message
      Call IO.O ' send monster edit display message
      Outpt = "[M]onster class" ' make monster edit display message
      Call IO.O ' send monster edit display message
      Outpt = "[T]alk responses" ' make monster edit display message
      Call IO.O ' send monster edit display message
      Graphics.Off = False ' reset color
      Outpt = "Monster edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of monster edit option
         Case "A" ' option to add an empty monster record
            ' store the last monster record number
            New.Monster = LOF(MonsterFile) / Len(MonsterRecord) + 1
            MonsterRecord.MonsterName = Nul ' reset monster record variable
            MonsterRecord.PluralName = Nul ' reset monster record variable
            For Array.Index = 1 To 5 ' loop through monster treasure inventory
               ' reset monster record variable
               Call PutMonsterRecordTreasure(Array.Index, False)
            Next ' end loop through monster treasure inventory
            MonsterRecord.Experience = False ' reset monster record variable
            MonsterRecord.NumberAppearing = False ' reset monster record variable
            MonsterRecord.Level = False ' reset monster record variable
            MonsterRecord.Hits = False ' reset monster record variable
            MonsterRecord.Poison = False ' reset monster record variable
            MonsterRecord.LevelDrain = False ' reset monster record variable
            MonsterRecord.Spell = False ' reset monster record variable
            MonsterRecord.Block = False ' reset monster record variable
            MonsterRecord.Prevent = False ' reset monster record variable
            MonsterRecord.Follow = False ' reset monster record variable
            MonsterRecord.Magic = False ' reset monster record variable
            MonsterRecord.Jail = False ' reset monster record variable
            MonsterRecord.Teleport = False ' reset monster record variable
            MonsterRecord.FollowPercent = False ' reset monster record variable
            MonsterRecord.BlockPercent = False ' reset monster record variable
            MonsterRecord.PreventPercent = False ' reset monster record variable
            MonsterRecord.SpellPercent = False ' reset monster record variable
            MonsterRecord.PoisonPercent = False ' reset monster record variable
            MonsterRecord.DrainPercent = False ' reset monster record variable
            MonsterRecord.Rate = False ' reset monster record variable
            MonsterRecord.RatePercent = False ' reset monster record variable
            MonsterRecord.Permanent = False ' reset monster record variable
            MonsterRecord.Gold = False ' reset monster record variable
            For Array.Index = 1 To 5 ' loop through monster talk responses
               ' reset monster record variable
               Call PutMonsterRecordTalk(Array.Index, False)
            Next ' end loop through monster talk responses
            MonsterRecord.Psionic = False ' reset monster record variable
            MonsterRecord.PsionicSpell = False ' reset monster record variable
            Call Share.Record(MonsterFile, New.Monster) ' routine to write a monster record
         Case "C" ' option to change monster values
            Call Find.Monster(Mon.Number) ' get monster number to edit
            If Mon.Number > False Then ' compare monster number
               Do ' loop through monster change menu
                  Call Display.Monster(Mon.Number) ' routine to display monster
                  Graphics.Off = False ' reset color
                  Outpt = "Monster change option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of monster change option
                     Case "A" ' option to change monster name
                        Outpt = "Monster name(30 characters)? " ' input prompt
                        Call IO.I ' get user input
                        MonsterRecord.MonsterName = LCase$(Inpt) ' store new name
                     Case "B" ' option to change plural of monster name
                        Outpt = "Plural of name(30 characters)? " ' input prompt
                        Call IO.I ' get user input
                        ' store new monster plural
                        MonsterRecord.PluralName = LCase$(Inpt)
                     Case "C" ' option to change monster level
                        Outpt = "Monster level(1-32767)? " ' input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Level = Int(Val(Inpt)) ' store new monster level
                     Case "D" ' option to change monster magical type
                        Outpt = "Magical monster(y/n)? " ' input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Magic = Yes ' store monster type
                     Case "E" ' option to change monster hit points
                        Outpt = "Hit points? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Hits = CDbl(Int(Val(Inpt))) ' store monster hits
                     Case "F" ' option to change monster experience
                        Outpt = "Experience points? " ' input prompt
                        No.Input.Out = "100" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Experience = CDbl(Int(Val(Inpt))) ' store exp
                     Case "G" ' option to change monster gold hoard
                        Outpt = "Gold points? " ' make input prompt
                        No.Input.Out = "100" ' default prompt
                        Call IO.I ' get user input
                        MonsterRecord.Gold = CDbl(Int(Val(Inpt))) ' store monster gold
                     Case "H" ' option to change number encountered
                        Outpt = "Number appearing(1-32767)? " ' input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.NumberAppearing = Int(Val(Inpt)) ' store number
                     Case "I" ' option to change monster poison type
                        Outpt = "Poisonous monster(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default prompt
                        Call IO.I ' get user input
                        MonsterRecord.Poison = False ' clear monster poison
                        MonsterRecord.PoisonPercent = False ' clear monster percent
                        If Yes Then ' check yes repsonse
                           Outpt = "Percent chance monster " ' make percent display
                           Outpt = Outpt + "will poison(1-100)? " ' input message
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              MonsterRecord.Poison = True ' make monster poisonous
                              MonsterRecord.PoisonPercent = Percent ' store percent
                           End If ' end check percent
                        End If ' end check response
                     Case "J" ' option to change monster level drain type
                        Outpt = "Level draining monster(y/n)? " ' input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.LevelDrain = False ' clear monster level drain
                        MonsterRecord.DrainPercent = False ' clear monster percent
                        If Yes Then ' check yes response
                           Outpt = "Percent chance monster " ' make percent display
                           Outpt = Outpt + "will level drain(1-100)? " ' input message
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              ' make monster level drainer
                              MonsterRecord.LevelDrain = True
                              MonsterRecord.DrainPercent = Percent ' store percent
                           End If ' end check percent
                        End If ' end check response
                     Case "K" ' option to change monster blocks exits type
                        Outpt = "Monster blocks exits(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Block = False ' clear monster block type
                        MonsterRecord.BlockPercent = False ' clear monster percent
                        If Yes Then ' check yes response
                           Outpt = "Percent chance monster " ' make percent display
                           Outpt = Outpt + "will block exits(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              MonsterRecord.Block = True ' make monster blocker
                              MonsterRecord.BlockPercent = Percent ' store percent
                           End If ' end check percent
                        End If ' end check response
                     Case "L" ' option to change monster preventing type
                        Outpt = "Monster prevents player from taking treasure(y/n)? "
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Prevent = False ' clear monster prevent type
                        MonsterRecord.PreventPercent = False ' clear monster percent
                        If Yes Then ' check yes response
                           Outpt = "Percent chance monster " ' make input prompt
                           Outpt = Outpt + "prevents player from taking treasure(1-100)? "
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              MonsterRecord.Prevent = True ' make monster preventer
                              MonsterRecord.PreventPercent = Percent ' store percent
                           End If ' end check percent
                        End If ' end check response
                     Case "M" ' option to change monster following type
                        Outpt = "Monster follows player(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default prompt
                        Call IO.I ' get user input
                        MonsterRecord.Follow = False ' clear monster follow type
                        MonsterRecord.FollowPercent = False ' clear percent
                        MonsterRecord.Teleport = False ' clear percent
                        If Yes Then ' check yes response
                           MonsterRecord.Follow = True ' make monster follows
                           Outpt = "Percent chance monster " ' make percent
                           Outpt = Outpt + "will follow player(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              MonsterRecord.FollowPercent = Percent ' store percent
                           End If ' end check percent
                           Outpt = "Percent chance monster " ' make percent
                           Outpt = Outpt + "will teleport with player(1-100)? " ' prompt
                           No.Input.Out = "50" ' default prompt
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= False And Percent <= 100 Then ' check percent
                              MonsterRecord.Teleport = Percent ' store percent
                           End If ' end check percent
                        End If ' end check response
                     Case "N" ' option to change monster spell casting
                        Outpt = "Monster casts spells(y/n)? " ' input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Spell = False ' clear monster spell flag
                        MonsterRecord.SpellPercent = False ' clear monster spell number
                        If Yes Then ' check yes response
                           Call Find.Spell(Spell.Number) ' routine to get spell number
                           If Spell.Number > False Then ' check spell number
                              Outpt = "Percent chance monster " ' percent display
                              Outpt = Outpt + "casts spell(1-100)? " ' input prompt
                              No.Input.Out = "50" ' default input
                              Call IO.I ' get user input
                              Percent = Int(Val(Inpt)) ' convert input to integer
                              If Percent >= False And Percent <= 100 Then ' check percent
                                 ' store monster spell number
                                 MonsterRecord.Spell = Spell.Number
                                 MonsterRecord.SpellPercent = Percent ' store percent
                              End If ' end check percent
                           End If ' end check spell number range
                        End If ' end check response
                     Case "O" ' option to change monster jail type
                        Outpt = "Monster jails attacker(y/n)? " ' input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Jail = Yes ' store yes response
                        If Yes Then ' check response
                           MonsterRecord.Follow = False ' reset flag for teleport
                           Outpt = "Jail room number?" ' prompt for room number
                           Call IO.I ' get input
                           MonsterRecord.Teleport = Int(Val(Inpt)) ' store room number
                        End If ' end check response
                     Case "P" ' option to change monster rates
                        Rate.Changed = True ' reset flag
                        Outpt = "Monster encounter rate(# of action prompts)? " ' prompt
                        No.Input.Out = "6" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Rate = Int(Val(Inpt)) ' store encounter rate
                        Outpt = "Monster encounter percent(1-100)? " ' input prompt
                        No.Input.Out = "50" ' default input
                        Call IO.I ' get user input
                        Percent = Int(Val(Inpt)) ' convert input to integer
                        If Percent >= False And Percent <= 100 Then ' check percent
                           MonsterRecord.RatePercent = Percent ' store percent
                        End If ' end check percent
                     Case "R" ' option to change monster permanent type
                        Outpt = "Monster is a permanent monster(y/n)? " ' input prompt
                        No.Input.Out = "N" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Permanent = Yes ' store yes response
                     Case "S" ' option to change monster psionic spell casting
                        Outpt = "Monster uses psionics(y/n)? " ' input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Psionic = False ' clear monster psionics
                        MonsterRecord.PsionicSpell = False ' clear monster psionics
                        If Yes Then ' check yes response
                           Outpt = "Enter psionic spell:" ' find spell prompt
                           Call Find.Spell(Spell.Number) ' routine to get spell number
                           If Spell.Number > False Then ' check spell number
                              If SpellRecord.Psionic = False Then ' check psionic type
                                 Outpt = "That spell is not a psionic spell!" ' message
                                 Call IO.O ' send error message
                              Else ' check psionic spell type
                                 MonsterRecord.Psionic = True ' make monster psionic
                                 ' store psi spell
                                 MonsterRecord.PsionicSpell = Spell.Number
                              End If ' end check psionic spell
                           End If ' end check spell number
                        End If ' end response
                     Case "T" ' option changes monster treasure inventory
                        Graphics.Off = True ' reset color
                        For Array.Number = 1 To 5 ' loop through monster treasure
                           ' get monster treasure number
                           Treasure.Number = GetMonsterRecordTreasure(Array.Number)
                           ' check file bounds
                           If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
                              ' get treasure record
                              Call Read.Record(TreasureFile, Treasure.Number)
                              Outpt = "[" + Mid$(Str$(Array.Number), 2) + "]" + TreasureRecord.TreasureName ' store treasure name
                              Call IO.O ' send output message
                           End If ' end check file bounds
                        Next ' end loop through monster treasure
                        Graphics.Off = False ' reset color
                        Outpt = "Enter five treasure names:" ' make display message
                        Call IO.O ' send display message
                        Outpt = "Press <enter> for none." ' make display message
                        Call IO.O ' send display message
                        Graphics.Off = True ' reset color
                        For Array.Index = 1 To 5 ' loop through monster inventory
                           ' routine to get treasure number
                           Call Find.Treasure(Treasure.Number)
                           ' store monster treasure
                           Call PutMonsterRecordTreasure(Array.Index, Treasure.Number)
                        Next ' end loop through monster inventory
                     Case "Q" ' option to exit monster change menu
                        Exit Do ' exit monster change menu
                  End Select ' end selection of monster change option
               Loop ' end loop through monster change menu
               Call Share.Record(MonsterFile, Mon.Number) ' write monster record
               Outpt = "Monster '" + RTrim$(MonsterRecord.MonsterName) + "' changed."
               Call IO.O ' send monster changed message
            End If ' end compare monster number
         Case "L" ' option to list range of monsters
            Max.Monsters = LOF(MonsterFile) / Len(MonsterRecord) ' store number records
            ' routine to get range of numbers
            Call Get.Range(Max.Monsters, Monster.List1, Monster.List2)
            ' loop through range of numbers
            Allow.Break = False ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Monster.Count = Monster.List1 To Monster.List2
               Call Read.Record(MonsterFile, Monster.Count) ' read monster record
               Call Display.Monster(Monster.Count) ' routine to display monster
               If Break Then ' check control-k flag
                  Exit For ' exit display loop
               End If ' end check control-k
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check pause response
                     Exit For ' exit list loop
                  End If ' end check response
               End If ' end check continuous flag
            Next ' end loop through range
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "M" ' option to edit monster class
            Do ' loop through monster class edit menu
               Graphics.Off = True ' reset color
               Outpt = "[A]dd class" ' make monster class display message
               Call IO.O ' send monster class display message
               Outpt = "[C]hange class" ' make monster class display message
               Call IO.O ' send monster class display message
               Outpt = "[L]ist class" ' make monster class display message
               Call IO.O ' send monster class display message
               Graphics.Off = False ' reset color
               Outpt = "Monster class edit option(q to quit)? " ' make input prompt
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of monster class edit option
                  Case "A" ' option to add empty monster class
                     New.Monster = LOF(MonClassFile) / Len(MonclassRecord) + 1 ' last record
                     For Array.Index = 1 To 10 ' loop through monster class record
                        ' clear monster class
                        Call PutMonclassRecordMonsters(Array.Index, False)
                     Next ' end loop through record
                     Call Share.Record(MonClassFile, New.Monster) ' write new monster class record
                     Rate.Changed = True ' reset flag
                  Case "C" ' option to change a monster class
                     Outpt = "Monster class" ' make range prompt
                     ' get number of records in file
                     Max.Monclasses = LOF(MonClassFile) / Len(MonclassRecord)
                     ' routine to get range of numbers
                     Call Get.Range2(0, Max.Monclasses, Monclass.Number)
                     If Monclass.Number > False Then ' check number range
                        Outpt = "Enter ten monster names:" ' make display message
                        Call IO.O ' send display message
                        Outpt = "Press <enter> for none." ' make display message
                        Call IO.O ' send display message
                        Graphics.Off = True ' reset color
                        For Monster.Count = 1 To 10 ' loop through monster class array
                           ' routine to get monster number
                           Call Find.Monster(Monster.Number)
                           ' store monster number
                           Call PutMonclassRecordMonsters(Monster.Count, Monster.Number)
                        Next ' end loop through monster class array
                        Call Share.Record(MonClassFile, Monclass.Number) ' write monster class record
                        ' make message
                        Outpt = "Monclass number" + Str$(Monclass.Number) + " changed."
                        Call IO.O ' send display message
                        Rate.Changed = True ' reset flag
                     End If ' end check number range
                  Case "L" ' option to list range of monster classes
                     Max.Monclasses = LOF(MonClassFile) / Len(MonclassRecord) ' get records
                     ' routine to get range of numbers
                     Call Get.Range(Max.Monclasses, Monclass.List1, Monclass.List2)
                     ' loop through range of monster classes
                     Allow.Break = True ' set allow break flag
                     Break = False ' reset control-k flag
                     Continue = False ' set continuous flag
                     For Monclass.Number = Monclass.List1 To Monclass.List2
                        ' get monster class record
                        Call Read.Record(MonClassFile, Monclass.Number)
                        ' routine to display monclass
                        Call Display.Monster.Class(Monclass.Number)
                        If Break Then ' check control-k flag
                           Exit For ' exit display loop
                        End If ' end check control-k
                        If Continue = False Then ' check continuous flag
                           Call More.Prompt ' routine to pause
                           If No Then ' check pause response
                              Exit For ' exit monster class list loop
                           End If ' end check response
                        End If ' end check continuous flag
                     Next ' end loop through range
                     Allow.Break = False ' reset allow break flag
                     If Break Then ' check control-k flag
                        Break = False ' reset control-k flag
                        Outpt = Nul ' set output to null
                        Call IO.O ' send empty return
                     End If ' end check control-k flag
                  Case "Q" ' option to exit monster class edit menu
                     Exit Do ' exit monster class edit menu
               End Select ' end selection of monster class option
            Loop ' end loop through monster class menu
         Case "T" ' option to edit monster talk responses
            Do ' loop through monster talk response edit menu
               Graphics.Off = True ' reset color
               Outpt = "[A]dd response" ' make monster talk response display message
               Call IO.O ' send monster talk response display message
               Outpt = "[C]hange response" ' make monster talk response message
               Call IO.O ' send monster talk response display message
               Outpt = "[L]ist response" ' make monster talk response display message
               Call IO.O ' send monster talk response display message
               Outpt = "[M]onster response" ' make monster talk response message
               Call IO.O ' send monster talk response display message
               Graphics.Off = False ' reset color
               Outpt = "Monster talk edit option(q to quit)? " ' make input prompt
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of monster talk response option
                  Case "A" ' option to add talk response string
                     Outpt = "Enter response(80 characters):" ' make display message
                     Call IO.O ' send display message
                     Outpt = "Press <enter> for none." ' make display message
                     Call IO.O ' send display message
                     Outpt = "? " ' make input prompt
                     Call IO.I ' get user input
                     ' get last response record
                     New.Talk = LOF(MonTalkFile) / Len(MonsterTalkRecord) + 1
                     MonsterTalkRecord.TalkMessage = Inpt ' store talk response
                     Call Share.Record(MonTalkFile, New.Talk) ' write talk response record
                  Case "C" ' option to change a talk response
                     Outpt = "Response number to change" ' make range prompt
                     ' get number of records in file
                     Max.Responses = LOF(MonTalkFile) / Len(MonsterTalkRecord)
                     ' routine to get range of numbers
                     Call Get.Range2(0, Max.Responses, Response.Number)
                     If Response.Number > False Then ' check number range
                        ' display message
                        Outpt = "Enter response number" + Str$(Response.Number) + ":"
                        Call IO.O ' send message
                        Outpt = "? " ' make input prompt
                        Call IO.I ' get user input
                        MonsterTalkRecord.TalkMessage = Inpt ' store talk response string
                        Call Share.Record(MonTalkFile, Response.Number) ' write talk response record
                        ' display message
                        Outpt = "Response number" + Str$(Response.Number) + " changed."
                        Call IO.O ' send message
                     End If ' end check number range
                  Case "L" ' option to list monster talk responses
                     Allow.Break = True ' set allow break flag
                     Break = False ' reset control-k flag
                     Continue = False ' reset continuous flag
                     Page.Length = False ' clear paginate counter
                     ' loop through file
                     For Montalk.Number = 1 To LOF(MonTalkFile) / Len(MonsterTalkRecord)
                        ' read monster talk record
                        Call Read.Record(MonTalkFile, Montalk.Number)
                        Graphics.Off = False ' reset color
                        ' make message
                        Outpt = "Response number" + Str$(Montalk.Number) + ":"
                        Call IO.O ' send message
                        Graphics.Off = True ' reset color
                        Outpt = RTrim$(MonsterTalkRecord.TalkMessage) ' store response
                        If Outpt = Nul Then ' compare length of talk response
                           Outpt = "<none>" ' reset to none
                        End If ' end check length
                        Call IO.O ' send response string
                        If Break Then ' check control-k flag
                           Exit For ' exit display loop
                        End If ' end check control-k
                        Page.Length = Page.Length + 2 ' increment page counter
                        If Page.Length >= User.PageLength Then ' check page counter
                           Page.Length = False ' reset counter
                           If Continue = False Then ' check continuous flag
                              Call More.Prompt ' routine to pause
                              If No Then ' check pause response
                                 Exit For ' exit file loop
                              End If ' end check yes
                           End If ' end check continuous flag
                        End If ' end check page
                     Next ' end loop through file
                     Allow.Break = False ' reset allow break flag
                     If Break Then ' check control-k flag
                        Break = False ' reset control-k flag
                        Outpt = Nul ' set output to null
                        Call IO.O ' send empty return
                     End If ' end check control-k flag
                     If Page.Length Then ' recheck page counter
                        Call More.Prompt ' routine to pause
                     End If ' end recheck counter
                  Case "M" ' option to add talk response numbers to a monster
                     Call Find.Monster(Monster.Number) ' get monster number to change
                     If Monster.Number > False Then ' check monster number
                        Call Read.Record(MonsterFile, Monster.Number) ' get monster
                        Outpt = "Enter five response numbers:" ' make display message
                        Call IO.O ' send display message
                        Outpt = "Press <enter> for none." ' make display message
                        Call IO.O ' send display message
                        For Talk.Count = 1 To 5 ' loop through five responses
                           Outpt = "Enter response number" ' make range prompt
                           Max.Talk = LOF(MonTalkFile) / Len(MonsterTalkRecord)
                           ' routine to get range of numbers
                           Call Get.Range2(0, Max.Talk, Talk.Number)
                           ' add talk response number
                           Call PutMonsterRecordTalk(Talk.Count, Talk.Number)
                           ' message
                           Outpt = "Response number" + Str$(Talk.Count) + " added."
                           Call IO.O ' send display message
                        Next ' end loop through five responses
                        Call Share.Record(MonsterFile, Monster.Number) ' write monster record
                     End If ' end check monster number
                  Case "Q" ' option to exit monster talk response edit menu
                     Exit Do ' exit monster talk response edit menu
               End Select ' end selection of monster talk response option
            Loop ' end loop through monster talk response edit menu
         Case "Q" ' option to exit monster edit menu
            Exit Do ' exit monster edit menu
      End Select ' end selection of monster edit options
   Loop ' end loop through monster edit menu
End Sub ' end routine to edit monsters

Rem * routine to edit nonplayers file.

Sub Edit.Nonplayer
   Do ' loop through nonplayer edit menu
      Graphics.Off = False ' reset color
      Outpt = "Nonplayer edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Nonplayer edit option(q to quit)? " ' option input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of option input
         Case "A" ' add nonplayer option
            ' store last nonplayer record number
            New.Nonplayer = LOF(NonPlayerFile) / Len(MonsterRecord) + 1
            MonsterRecord.MonsterName = Nul ' set nonplayer record items to zero
            MonsterRecord.PluralName = Nul ' reset nonplayer record item
            For Array.Number = 1 To 5 ' loop through nonplayer treasure
               ' set treasure items to zero
               Call PutMonsterRecordTreasure(Array.Number, False)
            Next ' end loop through nonplayer treasure
            MonsterRecord.Experience = False ' set nonplayer record items to zero
            MonsterRecord.NumberAppearing = 1 ' reset nonplayer record items
            MonsterRecord.Level = False ' reset nonplayer record items
            MonsterRecord.Hits = False ' reset nonplayer record items
            MonsterRecord.Gold = False ' reset nonplayer record item
            MonsterRecord.Poison = False ' reset nonplayer record items
            MonsterRecord.LevelDrain = False ' reset nonplayer record items
            MonsterRecord.Spell = False ' reset nonplayer record items
            MonsterRecord.Block = False ' reset nonplayer record items
            MonsterRecord.Prevent = False ' reset nonplayer record items
            MonsterRecord.Follow = False ' reset nonplayer record items
            MonsterRecord.Magic = True ' reset nonplayer record items
            MonsterRecord.Jail = False ' reset nonplayer record items
            MonsterRecord.Teleport = False ' reset nonplayer items
            MonsterRecord.FollowPercent = False ' reset nonplayer items
            MonsterRecord.BlockPercent = False ' reset nonplayer record items
            MonsterRecord.PreventPercent = False ' reset nonplayer record items
            MonsterRecord.SpellPercent = False ' reset nonplayer record items
            MonsterRecord.PoisonPercent = False ' reset nonplayer record items
            MonsterRecord.DrainPercent = False ' reset nonplayer record items
            MonsterRecord.Rate = False ' reset nonplayer record items
            MonsterRecord.RatePercent = False ' reset nonplayer record items
            MonsterRecord.Permanent = -2 ' reset nonplayer record items
            For Array.Number = 1 To 5 ' loop through monster talk responses
               Call PutMonsterRecordTalk(Array.Number, False) ' reset nonplayer responses
            Next ' end loop through monster talk responses
            MonsterRecord.Psionic = False ' reset nonplayer record items
            MonsterRecord.PsionicSpell = False ' reset nonplayer record items
            ' routine to write next nonplayer record
            Call Share.Record(NonPlayerFile, New.Nonplayer)
         Case "C" ' option to change nonplayer file record
            ' routine to find nonplayer to edit
            Call Find.Nonplayer(Nonplayer.Number)
            If Nonplayer.Number > False Then ' check nonplayer number
               Do ' loop through nonplayer number edit option menu
                  ' routine to display nonplayer
                  Call Display.Nonplayer(Nonplayer.Number)
                  Graphics.Off = False ' reset color
                  Outpt = "Nonplayer change option(q to quit)? " ' input prompt
                  No.Input.Out = "Q" ' default prompt
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of nonplayer change option
                     Case "A" ' option to change nonplayer name
                        Outpt = "Nonplayer name(30 characters)? " ' make input prompt
                        Call IO.I ' get user input
                        MonsterRecord.MonsterName = LCase$(Inpt) ' store name
                     Case "B" ' option to change nonplayer rooms
                        Outpt = "Rooms(separated by commas)? " ' make input prompt
                        Call IO.I ' get user input
                        MonsterRecord.PluralName = Inpt ' store rooms
                     Case "C" ' option to change nonplayer level
                        Outpt = "Nonplayer level(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Level = Int(Val(Inpt)) ' store nonplayer level
                     Case "D" ' option to change nonplayer hit points
                        Outpt = "Hit points? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Hits = CDbl(Int(Val(Inpt))) ' store nonplayer hits
                     Case "E" ' option to change nonplayer experience points
                        Outpt = "Experience points? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Experience = CDbl(Int(Val(Inpt))) ' store exp
                     Case "F" ' option to change nonplayer gold points
                        Outpt = "Gold points? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Gold = CDbl(Int(Val(Inpt))) ' store experience
                     Case "G" ' option to change poisonous nonplayer
                        Outpt = "Poisonous nonplayer(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Poison = False ' reset poisonous nonplayer
                        MonsterRecord.PoisonPercent = False ' reset posionous percent
                        If Yes Then ' check user input response
                           MonsterRecord.Poison = True ' set poisonous nonplayer
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "will poison(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.PoisonPercent = Int(Val(Inpt)) ' store percent
                        End If ' end check user input
                     Case "H" ' option to change level draining nonplayer
                        Outpt = "Level draining nonplayer(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.LevelDrain = False ' reset level drain nonplayer
                        MonsterRecord.DrainPercent = False ' reset level drain nonplayer
                        If Yes Then ' check user input response
                           ' set level draining nonplayer
                           MonsterRecord.LevelDrain = True
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "will level drain(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.DrainPercent = Int(Val(Inpt)) ' store percent
                        End If ' end check user input
                     Case "I" ' option to change exit blocking nonplayer
                        Outpt = "Nonplayer blocks exits(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Block = False ' reset blocking nonplayer
                        MonsterRecord.BlockPercent = False ' reset blocking nonplayer
                        If Yes Then ' check user input response
                           MonsterRecord.Block = True ' set blocking nonplayer
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "will block exits(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.BlockPercent = Int(Val(Inpt)) ' store percent
                        End If ' end check user input
                     Case "J" ' option to change preventing nonplayer
                        ' input prompt
                        Outpt = "Nonplayer prevents taking treasure(y/n)? "
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Prevent = False ' reset preventing nonplayer
                        ' reset preventing nonplayer
                        MonsterRecord.PreventPercent = False
                        If Yes Then ' check user input response
                           MonsterRecord.Prevent = True ' set preventing nonplayer
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "prevents player from taking treasure(1-100)? "
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.PreventPercent = Int(Val(Inpt)) ' store percent
                        End If ' end check user input
                     Case "K" ' option to change following nonplayer
                        Outpt = "Nonplayer follows player(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Follow = False ' reset following nonplayer
                        MonsterRecord.FollowPercent = False ' reset following nonplayer
                        MonsterRecord.Teleport = False ' reset following nonplayer
                        If Yes Then ' check user input response
                           MonsterRecord.Follow = True ' set following nonplayer
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "will follow player(1-100)? " ' input prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.FollowPercent = Int(Val(Inpt)) ' store percent
                           Outpt = "Percent chance nonplayer " ' make input prompt
                           Outpt = Outpt + "will teleport with player(1-100)? " ' prompt
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           MonsterRecord.Teleport = Int(Val(Inpt)) ' store percent
                        End If ' end check user input
                     Case "L" ' option to change spell casting nonplayer
                        Outpt = "Nonplayer casts spells(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Spell = False ' reset spell casting nonplayer
                        MonsterRecord.SpellPercent = False ' reset casting nonplayer
                        If Yes Then ' check user input prompt
                           ' routine to find spell number
                           Call Find.Spell(Spell.Number)
                           If Spell.Number > False Then ' check spell number range
                              MonsterRecord.Spell = Spell.Number ' store spell number
                              Outpt = "Percent chance nonplayer " ' make input prompt
                              Outpt = Outpt + "casts spell(1-100)? " ' input prompt
                              No.Input.Out = "50" ' default input
                              Call IO.I ' get user input
                              ' store percent
                              MonsterRecord.SpellPercent = Int(Val(Inpt))
                           End If ' end check spell number
                        End If ' end check user input
                     Case "M" ' option to change jailing nonplayer
                        Outpt = "Nonplayer jails attacker(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Jail = Yes ' set jailing nonplayer
                        If Yes Then ' check response
                           MonsterRecord.Follow = False ' reset flag for teleport
                           Outpt = "Jail room number?" ' prompt for room number
                           Call IO.I ' get input
                           MonsterRecord.Teleport = Int(Val(Inpt)) ' store room number
                        End If ' end check response
                     Case "N" ' option to change nonplayer encounter rate
                        ' prompt
                        Outpt = "Nonplayer encounter rate(# of action prompts)? "
                        No.Input.Out = "6" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Rate = Int(Val(Inpt)) ' store encounter rate
                        ' make input prompt
                        Outpt = "Nonplayer encounter percent(1-100)? "
                        No.Input.Out = "50" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.RatePercent = Int(Val(Inpt)) ' store percent
                     Case "O" ' option to change nonplayer psionics spell
                        Outpt = "Nonplayer uses psionics(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        MonsterRecord.Psionic = False ' reset nonplayer psionics spell
                        MonsterRecord.PsionicSpell = False ' reset nonplayer psionics
                        If Yes Then ' check user input
                           Outpt = "Psionic spell?" ' make input prompt
                           ' routine to find spell number
                           Call Find.Spell(Spell.Number)
                           If Spell.Number > False Then ' check spell number range
                              If SpellRecord.Psionic = False Then ' check psi spell
                                 Outpt = "That spell is not a psionic spell!" ' message
                                 Call IO.O ' send error message
                                 ' reset nonplayer psionics
                                 MonsterRecord.Psionic = False
                                 MonsterRecord.PsionicSpell = False ' reset psionics
                              Else ' check psionic spell type
                                 MonsterRecord.Psionic = True ' set nonplayer psionics
                                 ' store psi number
                                 MonsterRecord.PsionicSpell = Spell.Number
                              End If ' end check psionic spell type
                           End If ' end check spell number
                        End If ' end check user input
                     Case "P" ' option to change nonplayer treasure
                        Outpt = "Enter five treasure names:" ' make display message
                        Call IO.O ' send message
                        Outpt = "Press <enter> for none." ' make display message
                        Call IO.O ' send message
                        Graphics.Off = True ' reset color
                        For Treasure.Count = 1 To 5 ' loop through nonplayer treasures
                           ' routine to get treasure number
                           Call Find.Treasure(Treasure.Number)
                           ' store nonplayer treasure
                           Call PutMonsterRecordTreasure(Treasure.Count, Treasure.Number)
                        Next ' end loop through nonplayer treasures
                     Case "Q" ' option to exit nonplayer change menu
                        Exit Do ' exit nonplayer change menu
                  End Select ' end selections of nonplayer change options
               Loop ' end loop through nonplayer change option menu
               Call Share.Record(NonPlayerFile, Nonplayer.Number) ' write nonplayer file
               Outpt = "Nonplayer '" + RTrim$(MonsterRecord.MonsterName) + "' changed."
               Call IO.O ' send display message
            End If ' end check nonplayer edit number
         Case "L" ' option to list range of nonplayers
            ' store length of nonplayer file
            Max.Nonplayers = LOF(NonPlayerFile) / Len(MonsterRecord)
            ' routine to get range of numbers
            Call Get.Range(Max.Nonplayers, Nonplayer.List1, Nonplayer.List2)
            ' loop through range of nonplayers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Nonplayers.Count = Nonplayer.List1 To Nonplayer.List2
               Call Read.Record(NonPlayerFile, Nonplayers.Count) ' read nonplayer
               ' routine to display nonplayer
               Call Display.Nonplayer(Nonplayers.Count)
               If Break Then ' check control-k flag
                  Exit For ' exit display loop
               End If ' end check control-k
               Graphics.Off = False ' reset color
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check pause response
                     Exit For ' exit nonplayer list
                  End If ' end check response
               End If ' end check continuous flag
            Next ' end loop through range of nonplayers
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "Q" ' option to exit nonplayer edit options menu
            Exit Do ' exit nonplayer edit options menu
      End Select ' end selection of nonplayers edit menu
   Loop ' end loop through nonplater edit menu
End Sub ' end routine to edit nonplayers

Rem * routine to edit objects file.

Sub Edit.Object
   Do ' loop through objects edit menu
      Graphics.Off = False ' reset color
      Outpt = "Object edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Object edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of object edit option
         Case "A" ' option to add an object record
            New.Object = LOF(ObjectFile) / Len(ObjectRecord) + 1 'store last object record
            ObjectRecord.ObjectName = Nul ' reset object record item
            ObjectRecord.ShortName = Nul ' reset object record item
            ObjectRecord.LongDesc = Nul ' reset object record item
            ObjectRecord.ShortDesc = Nul ' reset object record item
            ObjectRecord.RoomLink = False ' reset object record item
            ObjectRecord.Invisible = False ' reset object record item
            ObjectRecord.JailTrap = False ' reset object record item
            ObjectRecord.DoorLock = False ' reset object record item
            ObjectRecord.Relocks = False ' reset object record item
            ObjectRecord.Hidden = False ' reset object record item
            ObjectRecord.Closed = False ' reset object record item
            ObjectRecord.Keyed = False ' reset object record item
            ObjectRecord.Trap = False ' reset object record item
            ObjectRecord.Teleport = False ' reset object record item
            ObjectRecord.LightRoom = False ' reset object record item
            ObjectRecord.LightTime = False ' reset object record item
            ObjectRecord.FromHour = False ' reset object record item
            ObjectRecord.FromMin = False ' reset object record item
            ObjectRecord.ToHour = False ' reset object record item
            ObjectRecord.ToMin = False ' reset object record item
            ObjectRecord.Permanent = True ' reset object record item
            Call Share.Record(ObjectFile, New.Object) ' routine to write object record
         Case "C" ' option to change an object
            Call Find.Objects(Object.Number) ' routine to get object number
            If Object.Number > False Then ' check object number range
               Do ' loop through object change menu
                  Call Display.Object(Object.Number) ' routine to display object
                  Graphics.Off = False ' reset color
                  Outpt = "Object change option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of object change option
                     Case "A" ' option to change object name
                        Outpt = "Object name(30 characters)? " ' make input prompt
                        Call IO.I ' get user input
                        ObjectRecord.ObjectName = LCase$(Inpt) ' store object name
                     Case "B" ' option to change object mnemonic name
                        Outpt = "Object identifier(30 characters)? " ' make input prompt
                        Call IO.I ' get user input
                        ObjectRecord.ShortName = UCase$(Inpt) ' store object mnemonic
                     Case "C" ' option to change object room number link
                        Outpt = "Room number link? " ' make input prompt
                        Call IO.I ' get user input
                        ObjectRecord.RoomLink = Int(Val(Inpt)) ' store object room link
                     Case "D" ' option to change object trap
                        Outpt = "Trapped portal(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Teleport = False ' reset object trap
                        ObjectRecord.Trap = False ' reset object trap
                        If Yes Then ' check user input response
                           Graphics.Off = True ' reset color
                           Outpt = "[P]oison needles" ' make display message
                           Call IO.O ' send message
                           Outpt = "[T]eleport to room number" ' make display message
                           Call IO.O ' send message
                           Outpt = "[H]its for fatigue or vitality" ' make message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Object trap option? " ' make input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of object trap type
                              Case "P" ' poison object trap type
                                 ObjectRecord.Trap = 1 ' set object poison trap type
                              Case "T" ' teleport object trap type
                                 ObjectRecord.Trap = 2 ' set object teleport trap type
                                 Outpt = "Room number? " ' make input prompt
                                 No.Input.Out = "1" ' default input
                                 Call IO.I ' get user input
                                 ' store telepot room
                                 ObjectRecord.Teleport = Int(Val(Inpt))
                              Case "H" ' objects hits trap type
                                 Hits.Sign = False ' reset hits sign multiplier
                                 Graphics.Off = True ' reset color
                                 Outpt = "[F]atigue" ' make display message
                                 Call IO.O ' send message
                                 Outpt = "[V]itality" ' make display message
                                 Call IO.O ' send message
                                 Outpt = "[N]one" ' make display message
                                 Call IO.O ' send message
                                 Graphics.Off = False ' reset color
                                 Outpt = "Object trap hits option? " ' make input prompt
                                 No.Input.Out = "Q" ' default input
                                 Call IO.I ' get user input
                                 Select Case UCase$(Inpt) ' selection of hits option
                                    Case "F" ' fatigue option
                                       Hits.Sign = 1 ' set hits sign multiplier
                                    Case "V" ' vitality option
                                       Hits.Sign = True ' set hits sign multiplier
                                    Case "N" ' option for none
                                       Hits.Sign = False ' reset sign multiplier
                                 End Select ' end hits option selection
                                 Hits.Amount = False ' reset hits number
                                 If Hits.Sign Then ' check hits type
                                    ObjectRecord.Trap = 3 ' set object trap type
                                    Outpt = "Number of hits(1-32767)? " ' make input prompt
                                    No.Input.Out = "10" ' default input
                                    Call IO.I ' get user input
                                    Hits.Amount = Int(Val(Inpt)) ' store number of hits
                                 End If ' end check hits type
                                 ' calculate hits
                                 ObjectRecord.Teleport = Hits.Sign * Hits.Amount
                           End Select ' end selection of object trap option
                        End If ' end check user input
                     Case "E" ' option to change object description
                        ' make display message
                        Outpt = "Long description(80 characters):"
                        Call IO.O ' send message
                        Outpt = "? " ' make input prompt
                        Call IO.I ' get user input
                        ObjectRecord.LongDesc = Inpt ' store object description
                     Case "F" ' option to change object entry description
                        Outpt = "Entry description(80 characters):" ' make message
                        Call IO.O ' send message
                        Outpt = "? " ' make input prompt
                        Call IO.I ' get user input
                        ObjectRecord.ShortDesc = Inpt ' store object entry description
                     Case "G" ' option to change hidden object
                        Outpt = "Hidden object(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Hidden = Yes ' store hidden object type
                     Case "H" ' option to change invisible object
                        Outpt = "Invisible object(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Invisible = Yes ' store invisible object type
                     Case "I" ' option to change object jail type
                        Outpt = "Jail attacker trap(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.JailTrap = Yes ' store object jail type
                     Case "J" ' option to change object locked type
                        Outpt = "Locked portal(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.DoorLock = False ' reset object lock type
                        ObjectRecord.Closed = False ' reset object lock type
                        If Yes Then ' check user input response
                           ObjectRecord.DoorLock = 2 ' set object lock
                           ObjectRecord.Closed = True ' set object lock
                        End If ' end check user input
                     Case "K" ' option to change object portal type
                        Outpt = "Relocking portal(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Relocks = Yes ' set object portal type
                     Case "L" ' option to change object key number
                        Outpt = "Key number(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Keyed = Int(Val(Inpt)) ' set object key number
                     Case "M" ' option to change permanent object
                        Outpt = "Permanent Object(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.Permanent = Yes ' set permanent object
                     Case "N" ' option to change object light type
                        Outpt = "Object is a light(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ObjectRecord.LightRoom = False ' reset object light type
                        ObjectRecord.LightTime = False ' reset object light type
                        If Yes Then ' check user input response
                           ObjectRecord.LightRoom = True ' set object light type
                           Outpt = "Object lights at a time(y/n)? " ' make input prompt
                           No.Input.Out = "Y" ' default input
                           Call IO.I ' get user input
                           If Yes Then ' check user input response
                              ObjectRecord.LightTime = True ' set object light time type
                              Outpt = "From hour(HH)? " ' make input prompt
                              Call IO.I ' get user input
                              ' store object hour light time
                              Time.Amount = Int(Val(Inpt))
                              ' check hour time range
                              If Time.Amount >= 0 And Time.Amount <= 23 Then
                                 ' store hour light time
                                 ObjectRecord.FromHour = Time.Amount
                              End If ' end check hour time range
                              Outpt = "From minute(MM)? " ' make input prompt
                              Call IO.I ' get user input
                              ' store object minute light time
                              Time.Amount = Int(Val(Inpt))
                              ' check minute time range
                              If Time.Amount >= 0 And Time.Amount <= 59 Then
                                 ' store minute light time
                                 ObjectRecord.FromMin = Time.Amount
                              End If ' end check minute time range
                              Outpt = "To hour(HH)? " ' make input prompt
                              Call IO.I ' get user input
                              ' store object hour light time
                              Time.Amount = Int(Val(Inpt))
                              ' check hour time range
                              If Time.Amount >= 0 And Time.Amount <= 23 Then
                                 ' store hour light time
                                 ObjectRecord.ToHour = Time.Amount
                              End If ' end check hour time range
                              Outpt = "To minute(MM)? " ' make input prompt
                              Call IO.I ' get user input
                              ' store object minute light time
                              Time.Amount = Int(Val(Inpt))
                              ' check minute time range
                              If Time.Amount >= 0 And Time.Amount <= 59 Then
                                 ' store minute light time
                                 ObjectRecord.ToMin = Time.Amount
                              End If ' end check minute time range
                           End If ' end check user input
                        End If ' end check user input
                     Case "Q" ' option to exit object change menu
                        Exit Do ' exit object change menu
                  End Select ' end selection of object change menu options
               Loop ' end loop through object change options
               Call Share.Record(ObjectFile, Object.Number) ' write object file
               Outpt = "Object '" + RTrim$(ObjectRecord.ObjectName) + "' changed."
               Call IO.O ' send display message
            End If ' end check object number range
         Case "L" ' option to list range of objects
            Max.Objects = LOF(ObjectFile) / Len(ObjectRecord) ' store length of object
            ' routine to get range of objects
            Call Get.Range(Max.Objects, Objects.List1, Objects.List2)
            ' loop through range of object numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Object.Count = Objects.List1 To Objects.List2
               Call Read.Record(ObjectFile, Object.Count) ' read object file
               Call Display.Object(Object.Count) ' routine to display object
               If Break Then ' check control-k flag
                  Exit For ' exit display flag
               End If ' end check control-k
               Graphics.Off = False ' reset color
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check user input
                     Exit For ' exit list loop
                  End If ' end check input
               End If ' end check continuous flag
            Next ' end loop through object list
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "Q" ' option to exit object edit menu
            Exit Do ' exit object edit menu
      End Select ' end selection of object edit menu option
   Loop ' end loop through object edit options
End Sub ' end routine to edit objects

Rem * routine to edit spells file.

Sub Edit.Spell
   Do ' loop through spell edit menu
      Graphics.Off = False ' reset color
      Outpt = "Spell edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' make message
      Graphics.Off = False ' reset color
      Outpt = "Spell edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of spell edit option
         Case "A" ' option to add a spell
            New.Spell = LOF(SpellFile) / Len(SpellRecord) + 1 ' store last spell record
            SpellRecord.SpellName = Nul ' reset spell record item
            SpellRecord.Chant = Nul ' reset spell record item
            SpellRecord.Desc = Nul ' reset spell record item
            SpellRecord.Level = False ' reset spell record item
            SpellRecord.SpellType = False ' reset spell record item
            SpellRecord.Teleport = False ' reset spell record item
            SpellRecord.ClassType = False ' reset spell record item
            SpellRecord.Psionic = False ' reset spell record item
            SpellRecord.PsionicMode = False ' reset spell record item
            For Array.Index = 1 To 5 ' loop through spell ingredients
               Call PutSpellRecordIngred(Array.Index, False) ' reset spell record item
            Next ' end loop through spell ingredients
            SpellRecord.SpellFlag = False ' reset spell record item
            Call Share.Record(SpellFile, New.Spell) ' write next spell record
         Case "C" ' option to edit a spell
            Call Find.Spell(Spell.Number) ' routine to get spell number
            If Spell.Number > False Then ' check spell number
               Do ' loop through spell edit options menu
                  Call Display.Spell(Spell.Number) ' routine to display spell
                  Graphics.Off = False ' reset color
                  Outpt = "Spell change option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of spell edit option
                     Case "A" ' option to change spell name
                        Outpt = "Spell name(30 characters)? " ' make input prompt
                        Call IO.I ' get user input
                        SpellRecord.SpellName = LCase$(Inpt) ' store spell name
                     Case "B" ' option to change spell chant
                        Outpt = "Spell chant(80 characters):" ' make display message
                        Call IO.O ' send message
                        Outpt = "? " ' make input prompt
                        Call IO.I ' get user input
                        SpellRecord.Chant = UCase$(Inpt) ' store spell chant
                     Case "C" ' option to change spell description
                        Outpt = "Spell cast description(80 characters):" ' make message
                        Call IO.O ' send message
                        Outpt = "? " ' make input prompt
                        Call IO.I ' get user input
                        SpellRecord.Desc = Inpt ' store spell description
                     Case "D" ' option to change spell level
                        Outpt = "Spell level(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        SpellRecord.Level = Int(Val(Inpt)) ' store spell level
                     Case "E" ' option to change spell type
                        Call Spell.Types ' routine displays spell types
                        Graphics.Off = False ' reset color
                        Outpt = "Spell type option(q to quit)? " ' make input prompt
                        No.Input.Out = "Q" ' default input
                        Call IO.I ' get user input
                        Inpt = UCase$(Inpt) ' uppercase input
                        ' compare input
                        Spell.Type = InStr("ABCDEFGHIJKLMNOPRSTUVWXYZ123", Inpt)
                        ' check input value
                        If Spell.Type > False And Spell.Type <= 28 Then
                           SpellRecord.SpellType = Spell.Type ' store spell type
                           If Spell.Type = 10 Then ' check teleport spell
                              Outpt = "Room it teleports to(1-" + Mid$(Str$(LOF(RoomFile) / Len(RoomRecord)), 2) + ")? " 'prompt
                              No.Input.Out = "1" ' default input
                              Call IO.I ' get user input
                              SpellRecord.Teleport = Int(Val(Inpt)) ' store spell room
                           End If ' end check teleport spell
                        End If ' end check input value
                     Case "F" ' option to change psionic spell
                        Outpt = "Spell is psionic spell(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        SpellRecord.Psionic = False ' reset psionic spell
                        SpellRecord.PsionicMode = False ' reset psionic spell
                        If Yes Then ' check user input response
                           SpellRecord.Psionic = True ' set psionic spell
                           Graphics.Off = True ' reset color
                           Outpt = "[A]ttack mode" ' make display message
                           Call IO.O ' send message
                           Outpt = "[D]efense mode" ' make display message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Spell psionic option? " ' make input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           SpellRecord.PsionicMode = False ' reset spell psionic type
                           Select Case UCase$(Inpt) ' selection of psionic type
                              Case "A" ' attack psionic type
                                 SpellRecord.PsionicMode = 1 ' store psionic type
                              Case "D" ' defense psionic type
                                 SpellRecord.PsionicMode = 2 ' store psionic type
                           End Select ' end selection of psionic type
                        End If ' end check user input
                     Case "G" ' option to change spell class restrictions
                        Do ' loop through spell cast options menu
                           Graphics.Off = True ' reset color
                           For Class.Number = 1 To 8 ' loop through class names
                              Outpt = "[" + Chr$(64 + Class.Number) + "]" + RTrim$(Class.Name(Class.Number))
                              Call IO.O ' send class name message
                           Next ' end loop through class names
                           Outpt = "[N]one" ' make message
                           Call IO.O ' send message
                           Outpt = "[X]All" ' make message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Spell class option(q to quit)? " ' make input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of class name
                              Case "A" To "H" ' range of class names
                                 ' store number of class
                                 Class.Number = InStr("ABCDEFGH", Inpt)
                                 ' set spell cast restriction flag
                                 SpellRecord.ClassType = SpellRecord.ClassType Or 2 ^ Class.Number
                              Case "N" ' option for no class restrictions
                                 ' reset class restriction flag
                                 SpellRecord.ClassType = False
                              Case "X" ' option to set all class restrictions
                                 For Class.Number = 1 To 8 ' loop through class types
                                    ' set spell cast restriction flag
                                    SpellRecord.ClassType = SpellRecord.ClassType Or 2 ^ Class.Number
                                 Next ' end loop through class types
                              Case "Q" ' option to exit spell cast restriction menu
                                 Exit Do ' exit spell cast restriction menu
                           End Select ' end selection of spell cast option
                        Loop ' end loop through spell cast restriction option menu
                     Case "H" ' option to change spell ingredients
                        Outpt = "Enter five treasure items for spell ingredients:"
                        Call IO.O ' send display message
                        Outpt = "Press <enter> for none." ' make display message
                        Call IO.O ' send message
                        Graphics.Off = True ' reset color
                        For Array.Count = 1 To 5 ' loop through spell ingredients
                           ' routine to get treasure number
                           Call Find.Treasure(Treasure.Number)
                           ' store spell ingredient
                           Call PutSpellRecordIngred(Array.Count, Treasure.Number)
                        Next ' end loop through spell ingredients
                        Graphics.Off = False ' reset color
                     Case "I" ' option to change spell ingredient cast type
                        Do ' loop through spell cast type option menu
                           Graphics.Off = True ' reset color
                           Outpt = "[A]Use command" ' make display message
                           Call IO.O ' send message
                           Outpt = "[B]Read scroll" ' make display message
                           Call IO.O ' send message
                           Outpt = "[C]Cast spell" ' make display message
                           Call IO.O ' send message
                           Outpt = "[N]one" ' make display message
                           Call IO.O ' send message
                           Outpt = "[X]All" ' make display message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Spell ingredient option(q to quit)? " ' make prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of spell cast type
                              Case "A" ' option to change spell cast type
                                 ' store spell cast type flag
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Use.Spell.Type
                              Case "B" ' option to change spell cast type
                                 ' store spell cast type flag
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Scroll.Spell.Type
                              Case "C" ' option to change spell cast type
                                 ' store spell cast type flag
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Cast.Spell.Type
                              Case "N" ' option to clear spell cast types
                                 ' clear spell cast type flag
                                 SpellRecord.SpellFlag = False
                              Case "X" ' option to add all spell cast type
                                 ' store all spell cast type flag
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Use.Spell.Type
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Scroll.Spell.Type
                                 SpellRecord.SpellFlag = SpellRecord.SpellFlag Or Cast.Spell.Type
                              Case "Q" ' option to exit spell ingredient cast types menu
                                 Exit Do ' exit spell ingredient cast types menu
                           End Select ' end selection of spell cast type
                        Loop ' end loop through spell cast types options menu
                     Case "Q" ' option to exit spell edit menu
                        Exit Do ' exit spell edit menu
                  End Select ' end selection of spell edit menu
               Loop ' end loop through spell editing
               Call Share.Record(SpellFile, Spell.Number) ' write spell record
               Outpt = "Spell '" + RTrim$(SpellRecord.SpellName) + "' changed." ' message
               Call IO.O ' send display message
            End If ' end check spell number
         Case "L" ' option to list range of spells
            Max.Spells = LOF(SpellFile) / Len(SpellRecord) ' store length of spell file
            ' routine to get range of numbers
            Call Get.Range(Max.Spells, Spell.List1, Spell.List2)
            ' loop through range of spell numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Spell.Count = Spell.List1 To Spell.List2
               Call Read.Record(SpellFile, Spell.Count) ' read spell record
               Call Display.Spell(Spell.Count) ' routine to display spell
               If Break Then ' check control-k flag
                  Exit For ' exit display loop
               End If ' end check control-k
               Graphics.Off = False ' reset color
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check pause response
                     Exit For ' exit spell list loop
                  End If ' end check input
               End If ' end check continuous flag
            Next ' end loop through spell list
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "Q" ' option to exit spell menu
            Exit Do ' exit spell options menu
      End Select ' end selection of spell options
   Loop ' end loop through spell options menu
End Sub ' end routine to edit spell

Rem * routine to edit treasure file.

Sub Edit.Treasure
   Do ' loop through treasure edit menu
      Graphics.Off = False ' reset color
      Outpt = "Treasure edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "Treasure edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of treasure edit option
         Case "A" ' option to add treasure
            New.Treasure = LOF(TreasureFile) / Len(TreasureRecord) + 1 ' store treasure
            TreasureRecord.Keyed = False ' reset treasure record item
            TreasureRecord.Weight = False ' reset treasure record item
            TreasureRecord.Gold = False ' reset treasure record item
            TreasureRecord.Plus = False ' reset treasure record item
            TreasureRecord.Charges = False ' reset treasure record item
            TreasureRecord.Type = False ' reset treasure record item
            TreasureRecord.Loadable = False ' reset treasure record item
            TreasureRecord.Ammunition = False ' reset treasure record item
            TreasureRecord.AmmoLoads = False ' reset treasure record item
            TreasureRecord.Container = False ' reset treasure record item
            TreasureRecord.Potion = False ' reset treasure record item
            TreasureRecord.Locked = False ' reset treasure record item
            TreasureRecord.Closed = False ' reset treasure record item
            TreasureRecord.Invisible = False ' reset treasure record item
            TreasureRecord.Proficiency = False ' reset treasure record item
            TreasureRecord.TreasureName = Nul ' reset treasure record item
            TreasureRecord.ShortName = Nul ' reset treasure record item
            TreasureRecord.Spell = False ' reset treasure record item
            TreasureRecord.Scroll = False ' reset treasure record item
            TreasureRecord.RingType = False ' reset treasure record item
            TreasureRecord.RingSpell = False ' reset treasure record item
            TreasureRecord.Edible = False ' reset treasure record item
            TreasureRecord.Rustable = False ' reset treasure record item
            TreasureRecord.RustPercent = False ' reset treasure record item
            TreasureRecord.Stealable = False ' reset treasure record item
            TreasureRecord.StealPercent = False ' reset treasure record item
            TreasureRecord.Launchable = False ' reset treasure record item
            TreasureRecord.LaunchLoads = False ' reset treasure record item
            TreasureRecord.LaunchAmmo = False ' reset treasure record item
            TreasureRecord.Movable = False ' reset treasure record item
            TreasureRecord.Vehicle = False ' reset treasure record item
            TreasureRecord.VehicleHits = False ' reset treasure record item
            TreasureRecord.VehicleType = False ' reset treasure record item
            TreasureRecord.LightType = False ' reset treasure record item
            TreasureRecord.LightCharges = False ' reset treasure record item
            TreasureRecord.FuelType = False ' reset treasure record item
            TreasureRecord.FuelCharges = False ' reset treasure record item
            TreasureRecord.Permanent = False ' reset treasure record item
            TreasureRecord.Invisible = False ' reset treasure record item
            Call Share.Record(TreasureFile, New.Treasure) ' write next treasure record
         Case "C" ' option to change treasure
            Call Find.Treasure(Treasure.Number) ' routine to get treasure number
            If Treasure.Number > False Then ' check treasure number
               Do ' loop through treasure edit options
                  ' routine to display treasure
                  Call Display.Treasure(Treasure.Number)
                  Graphics.Off = False ' reset color
                  Outpt = "Treasure change option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of treasure option
                     Case "A" ' option to change treasure name
                        Outpt = "Treasure name(30 characters)? " ' make input prompt
                        Call IO.I ' get user input
                        TreasureRecord.TreasureName = LCase$(Inpt) ' store treasure name
                     Case "B" ' option to change treasure mnemonic
                        ' make input prompt
                        Outpt = "Treasure identifier(30 characters)? "
                        Call IO.I ' get user input
                        ' store treasure mnemonic
                        TreasureRecord.ShortName = UCase$(Inpt)
                     Case "C" ' option to change treasure weight
                        Outpt = "Weight(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Weight = Int(Val(Inpt)) ' store treasure weight
                     Case "D" ' option to change treasure gold value
                        Outpt = "Gold value? " ' make input prompt
                        No.Input.Out = "10" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Gold = Int(Val(Inpt)) ' store treasure gold value
                     Case "E" ' option to change treasure type
                        Graphics.Off = True ' reset color
                        Outpt = "[W]eapon" ' make display message
                        Call IO.O ' send message
                        Outpt = "[S]hield" ' make display message
                        Call IO.O ' send message
                        Outpt = "[A]rmor" ' make display message
                        Call IO.O ' send message
                        Outpt = "[T]reasure" ' make display message
                        Call IO.O ' send message
                        Graphics.Off = False ' reset color
                        Outpt = "Treasure type option? " ' make input prompt
                        No.Input.Out = "Q" ' default input
                        Call IO.I
                        Select Case UCase$(Inpt) ' selecction of treasure type
                           Case "W" ' option for weapon type
                              TreasureRecord.Type = False ' set treasure type
                              TreasureRecord.Permanent = False ' set treasure type
                              Outpt = "Hit plus(1-32767)? " ' make input prompt
                              No.Input.Out = "10" ' default input
                              Call IO.I ' get user input
                              TreasureRecord.Plus = Int(Val(Inpt)) ' store treasure plus
                           Case "S" ' option for shield type
                              TreasureRecord.Type = True ' set treasure type
                              TreasureRecord.Permanent = False ' set treasure type
                           Case "A" ' option for armor type
                              TreasureRecord.Type = 1 ' set treasure type
                              TreasureRecord.Permanent = False ' set treasure type
                           Case "T" ' option for treasure type
                              TreasureRecord.Plus = False ' set treasure type
                              TreasureRecord.Charges = False ' set treasure type
                              TreasureRecord.Type = False ' set treasure type
                              TreasureRecord.Permanent = False ' set treasure type
                        End Select ' end selection of treasure type
                     Case "F" ' option to change treasure hits plus
                        Outpt = "Hit plus(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Plus = Int(Val(Inpt)) ' store treasure hits plus
                     Case "G" ' option to change treasure charges
                        Outpt = "Charges(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Charges = Int(Val(Inpt)) ' store treasure charges
                     Case "H" ' option to change treasure weapon type
                        Graphics.Off = True ' reset color
                        Outpt = "[B]lunt" ' make display message
                        Call IO.O ' send message
                        Outpt = "[P]ole" ' make display message
                        Call IO.O ' send message
                        Outpt = "[S]harp" ' make display message
                        Call IO.O ' send message
                        Outpt = "[T]hrusting" ' make display message
                        Call IO.O ' send message
                        Graphics.Off = False ' reset color
                        Outpt = "Treasure weapon class option? " ' make input prompt
                        No.Input.Out = "Q" ' default input
                        Call IO.I ' get user input
                        Select Case UCase$(Inpt) ' selection of treasure weapon type
                           Case "B" ' blunt weapon type
                              TreasureRecord.Proficiency = 1 ' set weapon type
                           Case "P" ' pole weapon type
                              TreasureRecord.Proficiency = 2 ' set weapon type
                           Case "S" ' sharp weapon type
                              TreasureRecord.Proficiency = 3 ' set weapon type
                           Case "T" ' thrusting weapon type
                              TreasureRecord.Proficiency = 4 ' set weapon type
                        End Select ' end selection of treasure weapon type
                     Case "I" ' option to change permanent treasure
                        Outpt = "Treasure is permanent(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Permanent = Yes ' store permanent type
                     Case "J" ' option to change edible treasure
                        Outpt = "Treasure is edible(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Edible = Yes ' store edible type
                     Case "K" ' option to change treasure spell number
                        Outpt = "Treasure is magical(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Spell = False ' reset treasure spell number
                        If Yes Then ' check user input response
                           Call Find.Spell(Spell.Number) ' routine to get spell number
                           If Spell.Number > False Then ' check spell number range
                              ' store treasure spell number
                              TreasureRecord.Spell = Spell.Number
                           End If ' end check spell range
                        End If ' end check input
                     Case "L" ' option to change ring treasure
                        Outpt = "Treasure is ring(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.RingType = False ' reset treasure ring
                        TreasureRecord.RingSpell = False ' reset treasure ring
                        If Yes Then ' check user input response
                           Graphics.Off = True ' reset color
                           Outpt = "[P]oison" ' make display message
                           Call IO.O ' send message
                           Outpt = "[L]evel Drain" ' make display message
                           Call IO.O ' send message
                           Outpt = "[S]pell" ' make display message
                           Call IO.O ' send message
                           Outpt = "[G]eneric spell" ' make display message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Treasure ring option? " ' make input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of ring type
                              Case "P" ' option for anti-poison ring
                                 TreasureRecord.RingSpell = False ' store ring spell type
                                 TreasureRecord.RingType = 1 ' store ring type
                              Case "L" ' option for anti-leveldrain ring
                                 TreasureRecord.RingSpell = False ' store ring spell type
                                 TreasureRecord.RingType = 2 ' store ring type
                              Case "S" ' option for number of anti-spell ring
                                 ' routine to get spell number
                                 Call Find.Spell(Spell.Number)
                                 If Spell.Number > False Then ' check spell number
                                    ' store ring spell number
                                    TreasureRecord.RingSpell = Spell.Number
                                    TreasureRecord.RingType = 3 ' store ring type
                                 End If ' end check spell number
                              Case "G" ' option for generic anti-spell ring
                                 TreasureRecord.RingSpell = True ' store ring spell type
                                 TreasureRecord.RingType = 3 ' store ring type
                           End Select ' end selection of ring type
                        End If ' end check user input
                     Case "M" ' option to change treasure light
                        Outpt = "Treasure is a light(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.LightType = False ' reset treasure light type
                        TreasureRecord.LightCharges = False ' reset treasure light type
                        If Yes Then ' check user input response
                           Outpt = "Light charges(1-32767)? " ' make input prompt
                           Call IO.I ' get user input
                           Light.Charges = Int(Val(Inpt)) ' convert input to integer
                           TreasureRecord.LightType = True ' set treasure light type
                           ' store treasure charges
                           TreasureRecord.LightCharges = Light.Charges
                        End If ' end check user input
                     Case "N" ' option to change treasure vehicle
                        Outpt = "Treasure is a vehicle(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Vehicle = False ' reset treasure vehicle
                        TreasureRecord.VehicleHits = False ' reset treasure vehicle
                        TreasureRecord.VehicleType = False ' reset treasure vehicle
                        If Yes Then ' check user input response
                           Outpt = "Vehicle hits(1-32767)? " ' make input prompt
                           Call IO.I ' get user input
                           Vehicle.Hits = Int(Val(Inpt)) ' convert input to integer
                           If Vehicle.Hits > False Then ' check input range
                              TreasureRecord.Vehicle = True ' set treasure vehicle
                              ' store vehicle hits
                              TreasureRecord.VehicleHits = Vehicle.Hits
                              Graphics.Off = True ' reset color
                              Outpt = "[A]ir" ' make display message
                              Call IO.O ' send message
                              Outpt = "[L]and" ' make display message
                              Call IO.O ' send message
                              Outpt = "[W]ater" ' make display message
                              Call IO.O ' send message
                              Outpt = "[X]All terrain" ' make display message
                              Call IO.O ' send message
                              Graphics.Off = False ' reset color
                              Outpt = "Vehicle type edit option? " ' make input prompt
                              No.Input.Out = "Q" ' default input
                              Call IO.I ' get user input
                              Select Case UCase$(Inpt) ' selection of vehicle type
                                 Case "A" ' air vehicle type
                                    TreasureRecord.VehicleType = Air ' set vehicle type
                                 Case "L" ' land vehicle type
                                    TreasureRecord.VehicleType = Land ' set vehicle type
                                 Case "W" ' underwaater vehicle type
                                    TreasureRecord.VehicleType = Water ' set vehicle type
                                 Case "X" ' all terrain vehicle type
                                    TreasureRecord.VehicleType = AllTerrain ' vehicle type
                              End Select ' end selection of vehicle type
                           End If ' end check input range
                        End If ' end check user input
                     Case "O" ' option to change treasure ammunition type
                        Outpt = "Treasure loads from devices(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Ammunition = Yes ' set treasure ammunition type
                     Case "P" ' option to change treasure loadable type
                        Outpt = "Treasure is loadable device(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        ' reset treasure loadable type
                        TreasureRecord.Loadable = False
                        TreasureRecord.AmmoLoads = False ' reset treasure loadable type
                        If Yes Then ' check user input response
                           ' write current treasure record
                           Call Share.Record(TreasureFile, Treasure.Number)
                           Outpt = "Treasure which loads this device:" ' make message
                           Call IO.O ' send message
                           ' routine to get treasure number
                           Call Find.Treasure(Treasure.Device)
                           If Treasure.Device = False Then ' check number range
                              ' read current treasure record
                              Call Read.Record(TreasureFile, Treasure.Number)
                              TreasureRecord.Loadable = False ' reset loadable type
                              TreasureRecord.AmmoLoads = False ' reset loadable type
                           Else ' check input range
                              ' read loadable treasure record
                              Call Read.Record(TreasureFile, Treasure.Device)
                              If TreasureRecord.Ammunition = False Then ' check type
                                 ' read current record
                                 Call Read.Record(TreasureFile, Treasure.Number)
                                 Outpt = "That treasure is not ammunition." ' message
                                 Call IO.O ' send message
                                 TreasureRecord.Loadable = False ' reset loadable type
                                 TreasureRecord.AmmoLoads = False ' reset loadable type
                              Else ' check ammunition type
                                 ' read current record
                                 Call Read.Record(TreasureFile, Treasure.Number)
                                 ' set ammunition loadable
                                 TreasureRecord.Loadable = True
                                 ' store ammunition number
                                 TreasureRecord.AmmoLoads = Treasure.Device
                              End If ' end check ammunition type
                           End If ' end check user input range
                           ' write current record number
                           Call Share.Record(TreasureFile, Treasure.Number)
                        End If ' end check user input
                     Case "R" ' option to change treasure launchs
                        ' make input prompt
                        Outpt = "Treasure launchs from devices(y/n)? "
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.LaunchAmmo = Yes ' store launchs type
                     Case "S" ' option to change treasure launchable type
                        Outpt = "Treasure is launchable device(y/n)? " ' make input
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Launchable = False ' reset launchable type
                        TreasureRecord.LaunchLoads = False ' reset launchable type
                        If Yes Then ' check user input response
                           ' write current treasure record
                           Call Share.Record(TreasurFile, Treasure.Number)
                           Outpt = "Treasure which launchs from this device:" ' message
                           Call IO.O ' send message
                           ' routine to get treasure number
                           Call Find.Treasure(Treasure.Device)
                           ' check treasure number range
                           If Treasure.Device = False Then
                              ' read current treasure record
                              Call Read.Record(TreasureFile, Treasure.Number)
                              TreasureRecord.Launchable = False ' reset launchable type
                              TreasureRecord.LaunchLoads = False ' reset launchable type
                           Else ' check number range
                              ' read lannchable treasure
                              Call Read.Record(TreasureFile, Treasure.Device)
                              If TreasureRecord.LaunchAmmo = False Then ' check launch
                                 ' read current record
                                 Call Read.Record(TreasureFile, Treasure.Number)
                                 Outpt = "That treasure does not launch from devices."
                                 Call IO.O ' send error message
                                 ' reset launchable type
                                 TreasureRecord.Launchable = False
                                 TreasureRecord.LaunchLoads = False ' reset launchable
                              Else ' check launchable type
                                 ' read current record
                                 Call Read.Record(TreasureFile, Treasure.Number)
                                 TreasureRecord.Launchable = True ' set launchable
                                 ' store launch number
                                 TreasureRecord.LaunchLoads = Treasure.Device
                              End If ' end check launchable number type
                           End If ' end check launchable treasure number
                           Call Share.Record(TreasureFile, Treasure.Number) ' write current record
                        End If ' end check user input
                     Case "T" ' option to change movable type
                        Outpt = "Launchable device can be moved(y/n)? " ' make input
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Movable = Yes ' store movable type
                     Case "U" ' option  to change coin type
                        Outpt = "Treasure is coins(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Coin = Yes ' store coin type
                     Case "V" ' option to change potion type
                        Outpt = "Treasure is potion(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Potion = Yes ' store potion type
                     Case "W" ' option to change scroll type
                        Outpt = "Treasure is scroll(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Scroll = Yes ' store scroll type
                     Case "X" ' option to change invisible treasure
                        Outpt = "Treasure is invisible(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Invisible = Yes ' store invisible type
                     Case "Y" ' option to change container type
                        Outpt = "Treasure is a container(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Container = Yes ' store container type
                     Case "Z" ' option to change container lock
                        Outpt = "Container is locked(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Locked = False ' reset container lock
                        TreasureRecord.Closed = False ' reset container lock
                        If Yes Then ' check user input response
                           TreasureRecord.Locked = 1 ' set container lock
                           TreasureRecord.Closed = 1 ' set container lock
                        End If ' end check user input
                     Case "1" ' option to change key number
                        Outpt = "Treasure key number(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Keyed = Int(Val(Inpt)) ' store key number
                     Case "2" ' option to change treasure fuel
                        Outpt = "Treasure is fuel(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.FuelType = False ' reset fuel type
                        TreasureRecord.FuelCharges = False ' reset fuel type
                        If Yes Then ' check user input response
                           Outpt = "Fuel charges(1-32767)? " ' make input prompt
                           Call IO.I ' get user input
                           Fuel.Charges = Int(Val(Inpt)) ' convert input to integer
                           TreasureRecord.FuelType = True ' set fuel type
                           ' store fuel charges
                           TreasureRecord.FuelCharges = Fuel.Charges
                        End If ' end check user input
                     Case "3" ' option to change rusting treasure
                        Outpt = "Treasure can rust(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Rustable = False ' reset rusting treasure
                        TreasureRecord.RustPercent = False ' reset rusting treasure
                        If Yes Then ' check user input response
                           Outpt = "Percent chance weapon " ' make input prompt
                           Outpt = Outpt + "will rust during room rust rate(1-100)? "
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to integer
                           If Percent >= 1 And Percent <= 100 Then ' check input range
                              TreasureRecord.Rustable = True ' set rusting treasure
                              ' store rusting percent
                              TreasureRecord.RustPercent = Percent
                           Else ' check range
                              TreasureRecord.Rustable = False ' reset rusting treasure
                              ' reset rusting treasure
                              TreasureRecord.RustPercent = False
                           End If ' end check range
                        End If ' end check user input
                     Case "4" ' option to change stealable treasure
                        Outpt = "Treasure can be stolen(y/n)? " ' make input prompt
                        No.Input.Out = "Y" ' default input
                        Call IO.I ' get user input
                        TreasureRecord.Stealable = False ' reset steable treasure
                        TreasureRecord.StealPercent = False ' reset steable treasure
                        If Yes Then ' check user input response
                           Outpt = "Percent chance treasure " ' make input prompt
                           Outpt = Outpt + "stolen during room steal rate(1-100)? "
                           No.Input.Out = "50" ' default input
                           Call IO.I ' get user input
                           Percent = Int(Val(Inpt)) ' convert input to inetger
                           If Percent >= 1 And Percent <= 100 Then ' check input range
                              TreasureRecord.Stealable = True ' set steable treasure
                              ' set steable percent
                              TreasureRecord.StealPercent = Percent
                           Else ' check range
                              TreasureRecord.Stealable = False ' reset steable treasure
                              ' reset steable treasure
                              TreasureRecord.StealPercent = False
                           End If ' end check range
                        End If ' end check user input
                     Case "Q" ' option to exit treasure change menu
                        Exit Do ' exit treasure change menu
                  End Select ' end selection of treasure menu option
               Loop ' end loop through treasure change menu
               Call Share.Record(TreasureFile, Treasure.Number) ' write treasure record
               Outpt = "Treasure '" + RTrim$(TreasureRecord.TreasureName) + "' changed."
               Call IO.O ' send message
            End If ' end check number range
         Case "L" ' option to list range of treasure numbers
            Max.Treasure = LOF(TreasureFile) / Len(TreasureRecord) ' store length of treasure file
            ' get range of treasure numbers
            Call Get.Range(Max.Treasure, Treasure.List1, Treasure.List2)
            ' loop through treasure numbers
            Allow.Break = True ' set allow break flag
            Break = False ' reset control-k flag
            Continue = False ' set continuous flag
            For Treasure.Count = Treasure.List1 To Treasure.List2
               Call Read.Record(TreasureFile, Treasure.Count) ' read treasure record
               Call Display.Treasure(Treasure.Count) ' routine to display treasure
               If Break Then ' check control-k flag
                  Exit For ' exit display loop
               End If ' end check control-k
               Graphics.Off = False ' reset color
               If Continue = False Then ' check continuous flag
                  Call More.Prompt ' routine to pause
                  If No Then ' check pause input response
                     Exit For ' exit loop through treasure numbers
                  End If ' end check user input
               End If ' end check continuous flag
            Next ' end loop through treasure numbers
            Allow.Break = False ' reset allow break flag
            If Break Then ' check control-k flag
               Break = False ' reset control-k flag
               Outpt = Nul ' set output to null
               Call IO.O ' send empty return
            End If ' end check control-k flag
         Case "Q" ' option to exit treasure edit menu
            Exit Do ' exit treasure edit menu
      End Select ' end selection of treasure edit option
   Loop ' end loop through treasure edit menu
End Sub ' end routine to edit treasure

Rem * routine to edit user file.

Sub Edit.User
   Do ' loop through user edit option menu
      Graphics.Off = False ' reset color
      Outpt = "User edit:" ' make display message
      Call IO.O ' send message
      Graphics.Off = True ' reset color
      Outpt = "[A]dd" ' make display message
      Call IO.O ' send message
      Outpt = "[C]hange" ' make display message
      Call IO.O ' send message
      Outpt = "[D]elete" ' make display message
      Call IO.O ' send message
      Outpt = "[L]ist" ' make display message
      Call IO.O ' send message
      Outpt = "[U]pdate" ' make display message
      Call IO.O ' send message
      Graphics.Off = False ' reset color
      Outpt = "User edit option(q to quit)? " ' make input prompt
      No.Input.Out = "Q" ' default input
      Call IO.I ' get user input
      Select Case UCase$(Inpt) ' selection of user edit option
         Case "A" ' option to add user record
            If Local.Mode = False Then ' check local sysop online
               Outpt = "Remote DMs cannot add users." ' make message
               Call IO.O ' send error message
            Else ' check local sysop
               New.User = LOF(UserFile) / Len(UserRecord) + 1 ' store last user number
               Outpt = Dead$ ' store empty codename
               Call Valid(Outpt, 30) ' validate codename
               Call Encrypt(Outpt, True) ' encrypt codename
               UserRecord.CodeName = Outpt ' store empty codename
               Outpt = Deleted$ ' store empty password
               Call Valid(Outpt, 20) ' validate password
               Call Encrypt(Outpt, False) ' encrypt password
               UserRecord.PassWord = Outpt ' store empty password
               Outpt = Class.Name(1) ' store defualt class name
               Call Valid(Outpt, 20) ' validate class name
               Call Encrypt(Outpt, True) ' encrypt class name
               UserRecord.ClassName = Outpt ' store default class name
               UserRecord.ClassType = Fighter ' store default class type
               For Array.Index = 1 To 7 ' loop through user statistics
                  Call PutUserRecordStats(Array.Index, 12) ' store default statistic
               Next ' end loop through statistics
               For Array.Index = 1 To 4 ' loop through user weapons
                  Call PutUserRecordWeapons(Array.Index, False) ' store default weapon
               Next ' end loop through weapons
               Call PutUserRecordWeapons(3, 10) ' store default weapon
               UserRecord.Room = 1! ' reset user record item
               UserRecord.Level = 1 ' reset user record item
               UserRecord.Experience = 64 ' reset user record item
               UserRecord.Gold = 2048 ' reset user record item
               UserRecord.Borrow = False ' reset user record item
               UserRecord.NumCalls = False ' reset user record item
               UserRecord.Poison = False ' reset user record item
               UserRecord.Invisible = False ' reset user record item
               UserRecord.Race = 1 ' reset user record item
               UserRecord.Proficiency = 3 ' reset user record item
               UserRecord.Bank = False ' reset user record item
               UserRecord.MonstersKilled = False ' reset user record item
               UserRecord.Brief = False ' reset user record item
               UserRecord.Echo = False ' reset user echo mode
               UserRecord.Linefeeds = False ' reset user linefeed mode
               UserRecord.Linelength = 80 ' reset user linelength
               UserRecord.Pagelength = 24 ' reset pagelength
               UserRecord.Wordwrap = False ' reset user word wrap
               Outpt = Date$ ' store current date
               Call Valid(Outpt, 10) ' validate date
               Call Encrypt(Outpt, True) ' encrypt date
               UserRecord.DateOn = Outpt ' reset user record item
               UserRecord.Align1 = False ' reset user record item
               UserRecord.Align2 = False ' reset user record item
               UserRecord.Flags = False ' reset user record item
               UserRecord.MaxCalls = False ' reset user record item
               UserRecord.FromHour = False ' reset user record item
               UserRecord.FromMin = False ' reset user record item
               UserRecord.ToHour = False ' reset user record item
               UserRecord.ToMin = False ' reset user record item
               ContainerRec.Closed = False ' reset user record item
               ContainerRec.ContainerName = Nul ' reset user record item
               ContainerRec.Keyed = False ' reset user record item
               ContainerRec.Locked = False ' reset user record item
               ContainerRec.ShortName = Nul ' reset user record item
               ContainerRec.Permanent = False ' reset user record item
               Call Clear.Container(0, True) ' routine to clear the container structure
               For Array.Count = 1 To 3 ' loop through user containers
                  ' store empty container
                  Call PutUserRecordContainer(Array.Count)
               Next ' end loop through containers

               Call PutUserRecordTrainRoom(1, 36)
               Call PutUserRecordTrainRoom(2, 4)
               Call PutUserRecordTrainRoom(3, 123)
               Call PutUserRecordTrainRoom(4, 163)
               Call PutUserRecordTrainRoom(5, 162)
               Call PutUserRecordTrainRoom(6, 426)
               Call PutUserRecordTrainRoom(7, 421)
               Call PutUserRecordTrainRoom(8, 417)
               Call PutUserRecordTrainRoom(9, 1)
               Call PutUserRecordTrainRoom(10, 1)

               Call Share.Record(UserFile, New.User) ' write user record
               Outpt = "New user record added." ' make display message
               Call IO.O ' send message
               Call Read.Record(UserFile, User.Index)
            End If ' end check local sysop
         Case "D" ' option to delete user
            If Local.Mode = False Then ' check local sysop online
               Outpt = "Remote DMs cannot delete users." ' make message
               Call IO.O ' send error message
            Else ' check local sysop
               Call Find.User(User.Found, User.Number) ' get user number
               If User.Found Then ' check user found flag
                  If User.Number = User.Index Then ' check user number
                     Outpt = "Can't delete you. Try Suicide.." ' make output
                     Call IO.O ' send output
                  Else ' check user number
                     Inpt = UserRecord.CodeName ' store player codename
                     Call Decrypt(Inpt) ' decrypt codename
                     Inpt = RTrim$(Inpt) ' trim codename
                     Inpt = LCase$(Inpt) ' lowercase codename
                     Mid$(Inpt, 1, 1) = UCase$(Mid$(Inpt, 1, 1)) ' uppercase first word
                     Outpt = "Delete '" + Inpt + "'. Are you sure(y/n)? " ' make input prompt
                     No.Input.Out = "N" ' default input
                     Call IO.I ' get user input
                     If Yes Then ' check input response
                        Outpt = "User deleted." ' make deleted message
                        Call IO.O ' send message
                        Call Delete.User ' routine to clear user record
                        Call Share.Record(UserFile, User.Number) ' write user record
                     End If ' end check input response
                  End If ' end check user number
               End If ' end check user found flag
            End If ' end check local sysop online
            Call Read.Record(UserFile, User.Index)
         Case "C" ' option to change user
            Call Find.User(User.Found, User.Number) ' get user number
            If User.Found Then ' compare found flag
               Do ' loop through codename edit option menu
                  Call Display.User(User.Number) ' routine to display player
                  Graphics.Off = False ' reset color
                  Outpt = "User change option(q to quit)? " ' make input prompt
                  No.Input.Out = "Q" ' default input
                  Call IO.I ' get user input
                  Select Case UCase$(Inpt) ' selection of player edit option
                     Case "A" ' option to change player codename
                        If Local.Mode = False Then ' check local sysop online
                           Outpt = "Remote DMs cannot edit codename." ' make message
                           Call IO.O ' send error message
                        Else ' check local sysop
                           Outpt = "Codename? " ' make input prompt
                           No.Input.Out = Deleted$ ' default input
                           Call IO.I ' get user input
                           Inpt = UCase$(Inpt) ' uppercase input
                           Call Valid(Inpt, 30) ' validate input
                           If Inpt = Nul Then ' check validation
                              Outpt = "Illegal characters in codename." ' message
                              Call IO.O ' send error message
                           Else ' check validation
                              Call Encrypt(Inpt, True) ' encrypt codename
                              UserRecord.CodeName = Inpt ' store codename
                           End If ' end check validate
                        End If ' end check response
                     Case "B" ' option to change player password
                        If Local.Mode = False Then ' check local sysop online
                           Outpt = "Remote DMs cannot edit password." ' make message
                           Call IO.O ' send error message
                        Else ' check local sysop
                           Inpt = UserRecord.PassWord ' store password
                           Call Decrypt(Inpt) ' decrypt password
                           If Inpt = Nul Then ' check decryption byte checksum
                              Outpt = "This password has a checksum error." ' message
                              Call IO.O ' send error message
                           End If ' end check byte checksum
                           Outpt = "Password? " ' make input prompt
                           No.Input.Out = Deleted$ ' default input
                           Call IO.I ' get user input
                           Inpt = UCase$(Inpt) ' uppercase input
                           Call Valid(Inpt, 20) ' validate input
                           If Inpt = Nul Then ' check validation
                              Outpt = "Illegal characters in password." ' message
                              Call IO.O ' send error message
                           Else ' check validation
                              Call Encrypt(Inpt, False) ' encrypt password
                              UserRecord.PassWord = Inpt ' store password
                           End If ' end check validation
                        End If ' end check local sysop online
                     Case "C" ' option to change player level
                        Outpt = "Level(1-32767)? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        Level# = Int(Val(Inpt)) ' convert input to double
                        If Level# > False And Level# <= MaxInt Then ' compare input range
                           New.Level = CInt(Level#) ' convert input to integer
                           UserRecord.Level = New.Level ' store player level
                        End If ' end compare input range
                     Case "D" ' option to change player class number
                        Graphics.Off = True ' reset color
                        If Local.Mode = False Then ' check local sysop online
                           Max.Class = 8 ' store maximum class number
                        Else ' check local sysop
                           Max.Class = 10 ' store maximum class number
                        End If ' end check local sysop online
                        ' loop through class number range
                        For Class.Count = 1 To Max.Class
                           ' make class display message
                           Outpt = "[" + Mid$(Str$(Class.Count), 2) + "]" + RTrim$(Class.Name(Class.Count))
                           Call IO.O ' send message
                        Next ' end loop through class numbers
                        Outpt = "Class option" ' make range prompt
                        ' return number from range
                        Call Get.Range2(0, Max.Class, Class.Number)
                        If Class.Number > False Then ' check number bounds
                           ' store player class number
                           UserRecord.ClassType = Class.Number
                        End If ' end check number bounds
                     Case "E" ' option to change player weapon type
                        Graphics.Off = True ' reset color
                        For Weapon.COunt = 1 To 4 ' loop through weapon type names
                           ' make weapon type message
                           Outpt = "[" + Mid$(Str$(Weapon.COunt), 2) + "]" + RTrim$(WeaponTypeName(Weapon.COunt))
                           Call IO.O ' send message
                        Next ' end loop through weapon type names
                        Graphics.Off = False ' reset color
                        Outpt = "Weapon proficiency" ' make range prompt
                        Call Get.Range2(0, 4, Weapon.Value) ' get number from range
                        If Weapon.Value > False Then ' check number bounds
                           ' store weapon type number
                           UserRecord.Proficiency = Weapon.Value
                        End If ' end check number bounds
                     Case "F" ' option to change weapon proficiency number
                        Outpt = "Blunt weapon proficiency%(1-100)? " ' input prompt
                        No.Input.Out = "10" '  default input
                        Call IO.I ' get user input
                        Call PutUserRecordWeapons(1, Int(Val(Inpt))) ' store weapon value
                     Case "G" ' option to change weapon proficiency number
                        Outpt = "Pole weapon proficiency%(1-100)? " ' input prompt
                        No.Input.Out = "10" '  default input
                        Call IO.I ' get user input
                        Call PutUserRecordWeapons(2, Int(Val(Inpt))) ' store weapon value
                     Case "H" ' option to change weapon proficiency number
                        Outpt = "Sharp weapon proficiency%(1-100)? " ' input prompt
                        No.Input.Out = "10" '  default input
                        Call IO.I ' get user input
                        Call PutUserRecordWeapons(3, Int(Val(Inpt))) ' store weapon value
                     Case "I" ' option to change weapon proficiency number
                        ' input prompt
                        Outpt = "Thrusting weapon proficiency%(1-100)? "
                        No.Input.Out = "10" '  default input
                        Call IO.I ' get user input
                        Call PutUserRecordWeapons(4, Int(Val(Inpt))) ' store weapon value
                     Case "J" ' option to change player class name
                        Outpt = "Classname? " ' make input prompt
                        No.Input.Out = Deleted$ ' default input
                        Call IO.I ' get user input
                        Call Valid(Inpt, 20) ' validate input
                        If Inpt = Nul Then ' check validation
                           Outpt = "Illegal characters in classname." ' make message
                           Call IO.O ' send error message
                        Else ' end check validation
                           Call Encrypt(Inpt, True) ' encrypt clsss name
                           UserRecord.ClassName = Inpt ' store class name
                        End If ' end check validation
                     Case "K" ' option to change player statistic
                        Outpt = "Strength? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(1, Int(Val(Inpt))) ' store player staistic
                     Case "L" ' option to change player statistic
                        Outpt = "Intelligence? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(2, Int(Val(Inpt))) ' store player staistic
                     Case "M" ' option to change player statistic
                        Outpt = "Wisdom? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(3, Int(Val(Inpt))) ' store player staistic
                     Case "N" ' option to change player statistic
                        Outpt = "Dexterity? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(4, Int(Val(Inpt))) ' store player staistic
                     Case "O" ' option to change player statistic
                        Outpt = "Constitution? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(5, Int(Val(Inpt))) ' store player staistic
                     Case "P" ' option to change player statistic
                        Outpt = "Piety? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(6, Int(Val(Inpt))) ' store player staistic
                     Case "R" ' option to change player statistic
                        Outpt = "Charisma? " ' make input prompt
                        No.Input.Out = "12" ' default input
                        Call IO.I ' get user input
                        Call PutUserRecordStats(7, Int(Val(Inpt))) ' store player staistic
                     Case "S" ' option to change player experience
                        Outpt = "Experience? " ' make input prompt
                        No.Input.Out = "128" ' default input
                        Call IO.I ' get user input
                        UserRecord.Experience = CDbl(Int(Val(Inpt)))
                     Case "T" ' option to change player gold hoard
                        Outpt = "Gold? " ' make input prompt
                        No.Input.Out = "256" ' default input
                        Call IO.I ' get user input
                        UserRecord.Gold = CDbl(Int(Val(Inpt))) ' store player gold
                     Case "U" ' option to change player room number
                        Outpt = "Room number? " ' make input prompt
                        No.Input.Out = "1" ' default input
                        Call IO.I ' get user input
                        UserRecord.Room = Int(Val(Inpt)) ' store room number
                     Case "V" ' option to change player calls
                        Graphics.Off = True ' reset color
                        Outpt = "Calls made today? " ' make input prompt
                        No.Input.Out = "0" ' default input
                        Call IO.I ' get user input
                        UserRecord.NumCalls = Int(Val(Inpt)) ' store player calls
                        Outpt = "Maximum calls per day? " ' make input prompt
                        No.Input.Out = "5" ' default input
                        Call IO.I ' get user input
                        UserRecord.MaxCalls = Int(Val(Inpt)) ' store plater calls
                        Outpt = "Timeon restriction(from hour: form HH)? " ' prompt
                        No.Input.Out = "00" ' default input
                        Call IO.I ' get user input
                        UserRecord.FromHour = Int(Val(Inpt)) ' store player time
                        Outpt = "Timeon restriction(from minute: form MM)? " ' prompt
                        No.Input.Out = "00" ' default input
                        Call IO.I ' get user input
                        UserRecord.FromMin = Int(Val(Inpt)) ' store player time
                        Outpt = "Timeon restriction(to hour: form HH)? " ' prompt
                        No.Input.Out = "00" ' default input
                        Call IO.I ' get user input
                        UserRecord.ToHour = Int(Val(Inpt)) ' store player time
                        Outpt = "Timeon restriction(to minute: form MM)? " ' prompt
                        No.Input.Out = "00" ' default input
                        Call IO.I ' get user input
                        UserRecord.ToMin = Int(Val(Inpt)) ' store player time
                        Graphics.Off = False ' reset color
                     Case "W" ' option to edit player inventory
                        Do ' loop through inventory edit menu
                           Graphics.Off = True ' reset color
                           Outpt = "[C]ontainers" ' make display message
                           Call IO.O ' send message
                           Outpt = "[O]bjects" ' make display message
                           Call IO.O ' send message
                           Outpt = "[T]reasure" ' make display message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Inventory edit option(q to quit)? " ' prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of inventory option
                              Case "C" ' option to edit player containers
                                 Do ' loop through user container edit menu
                                    Graphics.Off = False ' reset color
                                    Outpt = "User container edit options:" ' make display message
                                    Call IO.O ' send message
                                    Graphics.Off = True ' reset color
                                    Outpt = "[A]dd" ' make option message
                                    Call IO.O ' send option message
                                    Outpt = "[C]hange" ' make option message
                                    Call IO.O ' send option message
                                    Outpt = "[D]elete" ' make option message
                                    Call IO.O ' send option message
                                    Outpt = "[L]ist" ' make option message
                                    Call IO.O ' send option message
                                    Graphics.Off = False ' reset color
                                    Outpt = "Enter user container edit option(q to quit)? " ' make option prompt
                                    No.Input.Out = "Q" ' store default input
                                    Call IO.I ' get option input
                                    Select Case UCase$(Inpt) ' make selection of input option
                                       Case "A" ' add container
                                          Treasure.Added = False ' set container added flag
                                          For Number.Containers = 1 To 3 ' loop through user containers
                                             X = GetUserRecord.Container(Number.Containers) ' store user container into record
                                             If RTrim$(ContainerRec.ShortName) = Nul Then ' compare container name length
                                                Outpt = "Add container name to slot #" + Mid$(Str$(Number.Containers), 2) + "." ' make add message
                                                Call IO.O ' send message
                                                ' routine to get treasure number
                                                Call Find.Treasure(Treasure.Number)
                                                If Treasure.Number > False Then ' check treasure number
                                                   If TreasureRecord.Container Then ' check treasure type
                                                      ' store container variables
                                                      ContainerRec.Closed = TreasureRecord.Closed
                                                      ContainerRec.ContainerName = TreasureRecord.TreasureName
                                                      ContainerRec.Locked = TreasureRecord.Locked
                                                      ContainerRec.Keyed = TreasureRecord.Keyed
                                                      ContainerRec.ShortName = TreasureRecord.ShortName
                                                      ContainerRec.Permanent = TreasureRecord.Permanent
                                                      For Container.Count = 1 To 5 ' loop through container contents
                                                         ' clear container contents
                                                         Call Clear.Container(Container.Count, False)
                                                      Next ' end loop through container
                                                      Call PutUserRecordContainer(Number.Containers) ' add container record to room
                                                      Treasure.Added = True ' set container added flag
                                                   End If ' end check treasure type
                                                End If ' end check treasure number
                                             End If ' end check container name length
                                          Next ' end loop through user containers
                                          If Treasure.Added Then ' check added flag
                                             Outpt = "Container(s) added to user." ' make message
                                          Else ' check added flag
                                             Outpt = "Container not added to user." ' make message
                                          End If ' end check added flag
                                          Call IO.O ' display message
                                       Case "C" ' change container
                                          Outpt = "Enter container number(1-3)?" ' get container number
                                          Call IO.I ' get input
                                          Container.Number = Int(Val(Inpt)) ' store input
                                          If Container.Number >= 1 And Container.Number <= 3 Then ' check container number
                                             X = GetUserRecordContainer(Container.Number) ' store user container into record
                                             If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
                                                Call Change.Container(0) ' routine to edit container
                                                Call PutUserRecordContainer(Container.Number) ' store container
                                                Outpt = "Container changed." ' make message
                                                Call IO.O ' send message
                                             End If ' end check container name
                                          End If ' end check container number
                                       Case "D" ' delete container
                                          Treasure.Deleted = False ' set container deleted flag
                                          Outpt = "Enter container number(1-3)?" ' get container number
                                          Call IO.I ' get input
                                          Container.Number = Int(Val(Inpt)) ' store input
                                          If Container.Number >= 1 And Container.Number <= 3 Then ' check container number
                                             X = GetUserRecordContainer(Container.Number) ' store user container into record
                                             If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
                                                Call Clear.Container(0, True) ' clear entire container record
                                                Call PutUserRecordContainer(Container.Number) ' clear user container
                                                Treasure.Deleted = True ' reset container deleted flag
                                             End If ' end compare container in user
                                          End If ' end check container number
                                          If Treasure.Deleted Then ' check deleted flag
                                             Outpt = "Container deleted from user." ' make message
                                          Else ' check added flag
                                             Outpt = "Container not deleted from user." ' make message
                                          End If ' end check added flag
                                          Call IO.O ' display message
                                       Case "L" ' list containers
                                          Graphics.Off = True ' reset color
                                          Container.Count = False ' reset container count
                                          For Number.Containers = 1 To 3 ' loop through all user containers
                                             X = GetUserRecordContainer(Number.Containers) ' store user container into record
                                             If RTrim$(ContainerRec.ShortName) <> Nul Then ' compare container name length
                                                Container.Count = True ' reset count flag
                                                Outpt = RTrim$(ContainerRec.ContainerName) ' store container name
                                                Mid$(Outpt, 1, 1) = UCase$(Mid$(Outpt, 1, 1)) ' uppercase first letter
                                                Outpt = "#" + Mid$(Str$(Number.Containers), 2) + ":" + Outpt ' store container number
                                                If ContainerRec.Locked > False Then ' check container is locked
                                                   Outpt = Outpt + "[locked]" ' append message
                                                Else ' container is not locked
                                                   If ContainerRec.Closed > False Then ' check container is closed
                                                      Outpt = Outpt + "[closed]" ' append message
                                                   End If ' end check closed container
                                                End If ' end check locked container
                                                If ContainerRec.Keyed Then ' check container has a key number
                                                   Outpt = Outpt + "(#" + Right$(Str$(ContainerRec.Keyed + 100000!), 5) + ")"
                                                End If ' append key number to container name
                                                Number.Items = False ' reset counter
                                                For Treasure.Number = 1 To 5 ' loop through items
                                                   Treasure.Item = GetContainerRecInventory(Treasure.Number) ' store item index
                                                   ' compare item range
                                                   If Treasure.Item > False And Treasure.Item <= LOF(TreasureFile) / Len(TreasureRecord) Then
                                                      Number.Items = Number.Items + 1 ' increment counter
                                                   End If ' end check range
                                                Next ' end item loop
                                                Outpt = Outpt + ":" + Str$(Number.Items)
                                                If Number.Items = 1 Then ' check counter
                                                   Outpt = Outpt + " item." ' append message
                                                Else ' check counter
                                                   Outpt = Outpt + " items." ' append message
                                                End If ' end check counter
                                                Call IO.O ' display message
                                             End If ' end compare container name length
                                          Next ' end container count loop
                                          If Container.Count = False Then
                                             Outpt = "<no containers>" ' make message
                                             Call IO.O ' display message
                                          End If
                                       Case "Q" ' quit container menu
                                          Exit Do
                                    End Select
                                 Loop ' end room container loop
                              Case "O" ' option to edit player object inventory
                                 Do ' loop through object inventory edit menu
                                    Graphics.Off = False ' reset color
                                    Outpt = "Object inventory options:" ' make message
                                    Call IO.O ' send message
                                    Graphics.Off = True ' reset color
                                    Outpt = "[A]dd" ' make display message
                                    Call IO.O ' send message
                                    Outpt = "[D]elete" ' make display message
                                    Call IO.O ' send message
                                    Outpt = "[L]ist" ' make display message
                                    Call IO.O ' send message
                                    Graphics.Off = False ' reset color
                                    Outpt = "Object inventory edit option(q to quit)? "
                                    No.Input.Out = "Q" ' default input
                                    Call IO.I ' get user input
                                    ' selection of object edit
                                    Select Case UCase$(Inpt)
                                       Case "A" ' option to add object
                                          Object.Added = False ' object added flag
                                          ' routine to get object
                                          Call Find.Objects(Object.Number)
                                          ' check object number
                                          If Object.Number > False Then
                                             Call Add.Object(Object.Number, Object.Added)
                                          End If ' end check object number range
                                          If Object.Added Then ' check object added flag
                                             Outpt = "Object added to inventory." ' message
                                          Else ' check added flag
                                             Outpt = "Object not added to inventory."
                                          End If ' end check object added flag
                                          Call IO.O ' send message
                                       Case "D" ' option to delete object from inventory
                                          Outpt = "Number to delete" ' range prompt
                                          Call Get.Range2(0, 20, Object.Number) ' get number
                                          If Object.Number > False Then ' check number bounds
                                             ' check object
                                             If GetUserRecordObject(Object.Number) > False Then
                                                Call Discard.Inventory.Object(Object.Number)
                                                Outpt = "Object deleted from inventory."
                                                Call IO.O ' send message
                                             Else ' check object
                                                Outpt = "Object not deleted from inventory."
                                                Call IO.O ' send message
                                             End If ' end check object number range
                                          End If ' end check number bounds
                                       Case "L"
                                          Objects.Count = False ' reset count flag
                                          Graphics.Off = True ' reset color
                                          ' loop through player objects
                                          For Object.Index = 1 To 20
                                             ' store object
                                             Object.Number = GetUserRecordObject(Object.Index)
                                             ' check object number
                                             If Object.Number > False And Object.Number <= LOF(ObjectFile) / Len(ObjectRecord) Then
                                                ' increment counter
                                                Objects.Count = True
                                                ' read object
                                                Call Read.Record(ObjectFile, Object.Number)
                                                ' store
                                                Object.Name$ = ObjectRecord.ObjectName
                                                ' lowercase name
                                                Object.Name$ = LCase$(Object.Name$)
                                                Outpt = "[" + Mid$(Str$(Object.Index), 2) + "]" + Object.Name$
                                                Call IO.O ' send object  name message
                                             End If ' end check object number range
                                          Next ' end loop through player object inventory
                                          If Objects.Count Then ' check objects counted
                                             Call More.Prompt ' display more prmpt
                                          Else ' check objects counted
                                             Outpt = "<no objects>" ' make display message
                                             Call IO.O ' display message
                                          End If ' end objects counted
                                       Case "Q" ' option to exit inventory object menu
                                          Exit Do ' exit inventory object edit menu
                                    End Select ' end selection of object edit option
                                 Loop ' end loop through player object inventory menu
                              Case "T" ' option to edit player treasure inventory
                                 Do ' loop through treasure inventory edit menu
                                    Graphics.Off = False ' reset color
                                    Outpt = "Treasure inventory options:" ' message
                                    Call IO.O ' send message
                                    Graphics.Off = True ' reset color
                                    Outpt = "[A]dd" ' make message
                                    Call IO.O ' send message
                                    Outpt = "[D]elete" ' make message
                                    Call IO.O ' send message
                                    Outpt = "[L]ist" ' make message
                                    Call IO.O ' send message
                                    Graphics.Off = False ' reset color
                                    Outpt = "Treasure inventory edit option(q to quit)? "
                                    No.Input.Out = "Q" ' default input
                                    Call IO.I ' get user input
                                    Select Case UCase$(Inpt) ' selection of option
                                       Case "A" ' option to add inventory treasure
                                          Treasure.Added = False ' treasure added flag
                                          ' routine to get number
                                          Call Find.Treasure(Treasure.Number)
                                          If Treasure.Number > False Then ' check number
                                             ' check treasure type
                                             If TreasureRecord.Container = False Then
                                                Call TreasureCharges(Treasure.Charges)
                                                Call Add.Inventory(Treasure.Number, Treasure.Charges, Treasure.Added)
                                             End If ' end check treasure type
                                          End If ' end check treasure number
                                          ' check treasure added flag
                                          If Treasure.Added Then
                                             Outpt = "Treasure added to inventory."
                                          Else ' check flag
                                             Outpt = "Treasure not added to inventory."
                                          End If ' end check added flag
                                          Call IO.O ' send message
                                       Case "D" ' option to delete treasure number
                                          Outpt = "Number to delete" ' range prompt
                                          Call Get.Range2(0, 20, Treasure.Number) ' get number
                                          If Treasure.Number > False Then ' check number bounds
                                             ' check treasure index
                                             If GetUserRecordInv(Treasure.Number) > False Then
                                                If User.Number = User.Index Then ' check user
                                                   ' discard online user treasure
                                                   Call Discard.Inventory(Treasure.Number, True)
                                                Else ' check user
                                                   ' discard offline user treasure
                                                   Call Discard.Inventory.Treasure(Treasure.Number)
                                                End If ' end check user
                                                Outpt = "Treasure deleted from inventory."
                                                Call IO.O ' send message
                                             Else ' end check number range
                                                Outpt = "Treasure not deleted from inventory."
                                                Call IO.O ' send message
                                             End If ' end check deleted flag
                                          End If ' end check number bounds
                                       Case "L"
                                          Treasure.Counted = False ' reset counter
                                          Graphics.Off = True ' reset color
                                          ' loop through inventory
                                          For Treasure.Index = 1 To 20
                                             ' store number
                                             Treasure.Number = GetUserRecordInv(Treasure.Index)
                                             ' check number
                                             If Treasure.Number > False And Treasure.Number <= LOF(TreasureFile) / Len(TreasureRecord) Then
                                                ' increment counter
                                                Treasure.Counted = True
                                                ' get name
                                                Call Read.Record(TreasureFile, Treasure.Number)
                                                Treasure.Name$ = TreasureRecord.TreasureName
                                                ' lowercase name
                                                Treasure.Name$ = LCase$(Treasure.Name$)
                                                Outpt = "[" + Mid$(Str$(Treasure.Index), 2) + "]" + Treasure.Name$
                                                Call IO.O ' send message
                                             End If ' end check treasure number range
                                          Next ' end loop through inventory treasure
                                          If Treasure.Counted Then ' check counter
                                             Call More.Prompt ' display more prompt
                                          Else ' check counter
                                             Outpt = "<no treasure>" ' make message
                                             Call IO.O ' display message
                                          End If ' end check treasure counted
                                       Case "Q" ' option to exit inventory treasure edit
                                          Exit Do ' exit inventory treasure edit menu
                                    End Select ' end selection of treasure edit option
                                 Loop ' end loop through treasure edit menu
                              Case "Q" ' option to exit player inventory edit menu
                                 Exit Do ' exit player inventory edit menu
                           End Select ' end selection of inventory menu option
                        Loop ' end loop through player inventory edit menu
                     Case "X" ' option to change player special class
                        Do ' loop through special class edit menu
                           Graphics.Off = True ' reset color
                           Outpt = "[A]Town Mayor" ' make display message
                           Call IO.O ' send message
                           Outpt = "[B]Governor" ' make display message
                           Call IO.O ' send message
                           Outpt = "[C]Guild Master" ' make display message
                           Call IO.O ' send message
                           Outpt = "[D]Sysop" ' make display message
                           Call IO.O ' send message
                           Outpt = "[E]None" ' make display message
                           Call IO.O ' send message
                           Graphics.Off = False ' reset color
                           Outpt = "Special class option(q to quit)? " ' input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Select Case UCase$(Inpt) ' selection of special class
                              Case "A" ' option for special class
                                 UserRecord.Flags = UserRecord.Flags Or Special.Char1
                                 Outpt = "Player is now town mayor." ' make message
                                 Call IO.O ' send message
                              Case "B" ' option for special class
                                 UserRecord.Flags = UserRecord.Flags Or Special.Char2
                                 Outpt = "Player is now governor." ' make message
                                 Call IO.O ' send message
                              Case "C" ' option for special class
                                 UserRecord.Flags = UserRecord.Flags Or Special.Char3
                                 Outpt = "Player is now guild master." ' make message
                                 Call IO.O ' send message
                              Case "D" ' option for special class
                                 UserRecord.Flags = UserRecord.Flags Or Special.Char4
                                 Outpt = "Player is now sysop." ' make message
                                 Call IO.O ' send message
                              Case "E" ' option to clear special class
                                 UserRecord.Flags = UserRecord.Flags And Not Special.Char1
                                 UserRecord.Flags = UserRecord.Flags And Not Special.Char2
                                 UserRecord.Flags = UserRecord.Flags And Not Special.Char3
                                 UserRecord.Flags = UserRecord.Flags And Not Special.Char4
                                 Outpt = "Player is cleared of special character class."
                                 Call IO.O ' send message
                              Case "Q" ' option to exit special class menu
                                 Exit Do ' exit special class menu
                           End Select ' end selection of special class option
                        Loop ' end loop through special class option menu
                     Case "Y" ' option to customize player's training rooms
                        Do ' loop through special class edit menu
                           Graphics.Off = False ' reset color
                           Outpt = "Custom training room menu:" ' output message
                           Call IO.O ' send output
                           Graphics.Off = True ' reset color
                           For Train.Count = 1 To 10 ' display all classnames
                              Inpt = Chr$(Train.Count + 64) ' store letter output
                              ' make class display message
                              Outpt = "[" + Inpt + "]" + RTrim$(Class.Name(Train.Count)) + " room number:"
                              ' store training room value
                              Train.Temp! = GetUserRecordTrainRoom(Train.Count)
                              If Train.Temp! = SFalse Then ' compare value
                                 Outpt = Outpt + " <none>" ' append message
                              Else ' compare value
                                 Outpt = Outpt + Str$(Train.Temp!) + "." ' append message
                              End If ' end compare value
                              Call IO.O ' send message
                           Next ' end loop through training room array
                           Outpt = "[R]eset rooms" ' make output
                           Call IO.O ' send output
                           Graphics.Off = False ' reset color
                           Outpt = "Training room option(q to quit)? " ' input prompt
                           No.Input.Out = "Q" ' default input
                           Call IO.I ' get user input
                           Inpt = UCase$(Inpt) ' uppercase input
                           Select Case Inpt ' selection of special class
                              Case "R" ' option for rooms reset
                                 Outpt = "Reset training rooms(y/n)? " ' message
                                 Call IO.I ' get response
                                 If Yes Then ' check response
                                    ' reset training rooms
                                    Call PutUserRecordTrainRoom(1, 36)
                                    Call PutUserRecordTrainRoom(2, 4)
                                    Call PutUserRecordTrainRoom(3, 123)
                                    Call PutUserRecordTrainRoom(4, 163)
                                    Call PutUserRecordTrainRoom(5, 162)
                                    Call PutUserRecordTrainRoom(6, 426)
                                    Call PutUserRecordTrainRoom(7, 421)
                                    Call PutUserRecordTrainRoom(8, 417)
                                    Call PutUserRecordTrainRoom(9, 1)
                                    Call PutUserRecordTrainRoom(10, 1)
                                    Outpt = "Training rooms reset." ' message
                                    Call IO.O ' send message
                                 End If ' end check response
                              Case "Q" ' exit training room loop
                                 Exit Do ' exit special class menu
                              Case Else ' check response
                                 If Inpt >= "A" And Inpt <= "J" Then ' check response range
                                    Train.Count = Asc(Inpt) - 64 ' convert to number
                                    Outpt = "Enter 0 for auto-train." ' make message
                                    Call IO.O ' send output
                                    Graphics.Off = True ' reset color
                                    ' make class range prompt
                                    Outpt = "[" + Mid$(Str$(Train.Count), 2) + "]" + RTrim$(Class.Name(Train.Count)) + " room number"
                                    ' return number from range
                                    Max.Rooms! = LOF(RoomFile) / Len(RoomRecord)
                                    Call Get.Room.Range2(0!, Max.Rooms!, Train.Number!)
                                    Call PutUserRecordTrainRoom(Train.Count, Train.Number!)
                                 End If ' end check input
                           End Select ' end selection of special class option
                        Loop ' end loop for training rooms
                     Case "Q" ' option to exit player edit menu
                        Exit Do ' exit player edit menu
                  End Select ' end selection of player edit option
               Loop ' end loop through player edit menu
               Call Share.Record(UserFile, User.Number) ' write user record
            End If ' end check user found flag
            Call Read.Record(UserFile, User.Index)
         Case "L" ' option to list user file
            Do ' loop through user file list menu
               Graphics.Off = True ' reset color
               Outpt = "[L]ist" ' make display message
               Call IO.O ' send message
               Outpt = "[T]op ten" ' make display message
               Call IO.O ' send message
               Outpt = "[U]sers" ' make display message
               Call IO.O ' send message
               Graphics.Off = False ' reset color
               Outpt = "User list option(q to quit)? " ' make input prompt
               No.Input.Out = "Q" ' default input
               Call IO.I ' get user input
               Select Case UCase$(Inpt) ' selection of user list option
                  Case "L" ' option to list range of user numbers
                     Outpt = "Enter range:" ' make display message
                     Call IO.O ' send message
                     Max.Users = LOF(UserFile) / Len(UserRecord) ' store length of file
                     ' routine to get range of numbers
                     Call Get.Range(Max.Users, User.List1, User.List2)
                     ' loop through range of user numbers
                     Allow.Break = True ' set allow break flag
                     Break = False ' reset control-k flag
                     Continue = False ' set continuous flag
                     For User.Count = User.List1 To User.List2
                        Call Read.Record(UserFile, User.Count) ' read user file
                        Code.Name$ = UserRecord.CodeName ' store user codename
                        Call Decrypt(Code.Name$) ' decrypt codename
                        If Left$(Code.Name$, 9) <> Deleted$ Then ' check deleted codename
                           Call Display.User(User.Count) ' routine to display user
                           If Break Then ' check control-k flag
                              Exit For ' exit display loop
                           End If ' end check control-k
                           If Continue = False Then ' check continuous flag
                              Call More.Prompt ' routine to pause
                              If No Then ' check input pause response
                                 Exit For ' exit user list loop
                              End If ' end check user input
                           End If ' end check continuous flag
                        End If '  end check deleted codename
                     Next ' end loop through user numbers
                     Allow.Break = False ' reset allow break flag
                     If Break Then ' check control-k flag
                        Break = False ' reset control-k flag
                        Outpt = Nul ' set output to null
                        Call IO.O ' send empty return
                     End If ' end check control-k flag
                  Case "T" ' option to list top ten report
                     Call Top.Ten ' routine for top ten
                  Case "U" ' option to list user report
                     Call User.List ' routine for user report
                  Case "Q" ' option to exit user list menu
                     Exit Do ' exit user list menu
               End Select ' end selection of user list option
            Loop ' end loop through user list menu
            Call Read.Record(UserFile, User.Index)
         Case "U" ' option to update user file
            If Local.Mode = False Then ' check local sysop online
               Outpt = "Remote DMs cannot update users." ' make message
               Call IO.O ' send error message
            Else ' check local sysop
               Do
                  Outpt = "Delete " + Dead$ + " users(y/n)? " ' make input prompt
                  No.Input.Out = "Y" ' default input
                  Call IO.I ' get user input
                  Delete.Dead.Users = Yes ' store input response
                  Outpt = "Delete users before date(y/n)? " ' make input prompt
                  No.Input.Out = "N" ' default input
                  Call IO.I ' get user input
                  Delete.By.Date = Yes ' store user input response
                  If Delete.By.Date Then ' check date update response
                     Outpt = "Enter date(form: MM-DD-YYYY)? " ' make input prompt
                     No.Input.Out = "01-01-1980" ' default input
                     Call IO.I ' get user input
                     If ValidDate(Inpt) = 0 Then
                        Outpt = "Invalid date."
                        Call IO.O
                     Else
                        Delete.Date$ = Inpt
                        Exit Do
                     End If
                  Else
                     Exit Do
                  End If ' end check date update response
               Loop
               Graphics.Off = False ' reset color
               Outpt = "Updating userfile.." ' make display message
               Call IO.O ' send message
               Users.Deleted = False ' reset counter
               For User.Count = 1 To LOF(UserFile) / Len(UserRecord) ' loop through users
                  Call Read.Record(UserFile, User.Count) ' read next user file record
                  Code.Name$ = UserRecord.CodeName ' store user codename
                  Call Decrypt(Code.Name$) ' decrypt codename
                  Delete.User.Flag = False ' delete user flag
                  If Left$(Code.Name$, Len(Deleted$)) <> Deleted$ Then ' check already deleted
                     If Delete.Dead.Users Then ' check update flag
                        If UserRecord.Level = False Then ' compare player level
                           Delete.User.Flag = True ' set delete user flag
                        End If ' end compare player level
                        If Left$(Code.Name$, Len(Dead$)) = Dead$ Then ' compare dead player
                           Delete.User.Flag = True ' set delete user flag
                        End If ' end compare dead player
                     End If ' end check update flag
                     If Delete.By.Date Then ' check date update flag
                        Inpt = UserRecord.DateOn ' store player last date online
                        Call Decrypt(Inpt) ' decrypt date
                        Var1$ = Mid$(Inpt, 7, 4) + Mid$(Inpt, 4, 2) + Mid$(Inpt, 1, 2)
                        Var2$ = Mid$(Delete.Date$, 7, 4) + Mid$(Delete.Date$, 4, 2) + Mid$(Delete.Date$, 1, 2)
                        If Var1$ < Var2$ Then
                           Delete.User.Flag = True ' set delete user flag
                        End If ' end calculate dates
                     End If ' end check date update flag
                  End If ' end check deleted user
                  If Delete.User.Flag Then ' check delete user flag
                     If User.Count <> User.Index Then ' check to current user
                        Users.Deleted = Users.Deleted + 1 ' increment counter
                        Call Delete.User ' routine to clear user record
                        Call Share.Record(UserFile, User.Count) ' write user record
                     End If ' end check current user
                  End If ' end check delete user flag
               Next ' end loop through user file
               If Users.Deleted = False Then ' check counter
                  Outpt = "No users deleted." ' make message
               Else ' check counter
                  If Users.Deleted = 1 Then ' compare counter
                     Outpt = "1 user deleted." ' make message
                  Else ' check counter
                     Outpt = Mid$(Str$(Users.Deleted), 2) + " users deleted." ' message
                  End If ' check counter
               End If ' end check counter
               Call IO.O ' display message
               Call Read.Record(UserFile, User.Index) ' restore current user
            End If ' end check local sysop
         Case "Q" ' option to exit user edit menu
            Exit Do ' exit user edit menu
      End Select ' end selection of user edit option
   Loop ' end loop through user edit menu options
End Sub ' end routine to edit user file

Rem * routine to find user.
Rem * output variables:
Rem *   User.Found  -  true if user found
Rem *   User.Number  -  user index

Sub Find.User (User.Found, User.Number)
   Graphics.Off = False ' reset color
   Outpt = "Codename, or number" ' make input prompt
   Outpt = Outpt + "(1-" + Mid$(Str$(LOF(UserFile) / Len(UserRecord)), 2) + ")? "
   No.Input.Out = "1" ' default input
   Call IO.I ' get user input
   Call Parse.Num(Inpt, Parse.Value) ' routine to parse pound sign
   Inpt = UCase$(Inpt) ' uppercase input
   Parse.Number = False ' reset counter
   User.Found = False ' reset user found flag
   Input.Length = Len(Inpt) ' store length of input
   For User.Number = 1 To LOF(UserFile) / Len(UserRecord) ' loop through user file
      Call Read.Record(UserFile, User.Number) ' read user file
      Code.Name$ = UserRecord.CodeName ' store user codename
      Call Decrypt(Code.Name$) ' decrypt codename
      ' compare codename to input
      If Left$(Code.Name$, Input.Length) = Inpt Then
         Parse.Number = Parse.Number + 1 ' increment counter
         ' compare counters
         If Parse.Value = False Or Parse.Number = Parse.Value Then
            User.Found = True ' set user found flag
            Exit For ' exit user loop
         End If ' end compare counters
      End If ' end compare codename
   Next ' end loop through users
   If User.Found = False Then ' check user found flag
      User.Number = Int(Val(Inpt)) ' convert input to integer
      ' check bounds
      If User.Number > False And User.Number <= LOF(UserFile) / Len(UserRecord) Then
         User.Found = True ' set user found flag
      End If ' end check file bounds
   End If ' end check user flag
   If User.Found = False Then ' check user found flag
      Outpt = "Codename not found." ' make not found message
      Call IO.O ' send message
   End If ' end check user found flag
   If User.Found Then ' check user found flag
      Call Read.Record(UserFile, User.Number) ' read user record
      Code.Name$ = UserRecord.CodeName ' store codename
      Call Decrypt(Code.Name$) ' decrypt codename
      If Left$(Code.Name$, 9) = Deleted$ Then ' compare deleted codename
         Outpt = "Deleted user record." ' make deleted user message
         Call IO.O ' send message
         User.Found = False ' reset flag
      End If ' end check delete codename
   End If ' end check user found
End Sub

' formats a double numeric string
Function FormatString$ (s#)
   x$ = Nul
   s$ = Str$(s#)
   If InStr(s$, "D") Then ' return string
      FormatString$ = s$
      Exit Function
   End If
   If Left$(s$, 1) = "-" Then ' store sign
      e$ = "-"
      s$ = Mid$(s$, 2)
   End If
   s$ = LTrim$(s$) ' format string
   If InStr(s$, ".") Then
      q$ = Mid$(s$, InStr(s$, "."))
      s$ = Left$(s$, InStr(s$, ".") - 1)
   End If
   For l = Len(s$) To 3 Step -3
      x$ = Mid$(s$, l - 2, 3) + "," + x$
   Next
   If l > 0 Then
      x$ = Mid$(s$, 1, l) + "," + x$
   End If
   If Len(s$) < 3 Then
      x$ = s$
   End If
   If Right$(x$, 1) = "," Then
      x$ = Left$(x$, Len(x$) - 1)
   End If
   x$ = e$ + x$ + q$ ' construct string
   FormatString$ = x$
End Function

Function ValidDate (Var$)
   ' mm-dd-yyyy
   ValidDate = 0
   For V = 1 To 10
      Select Case V
         Case 3, 6
            If Mid$(Var$, V, 1) = "-" Then
               Eat$ = ""
            Else
               Exit Function
            End If
         Case Else
            If Mid$(Var$, V, 1) >= "0" And Mid$(Var$, V, 1) <= "9" Then
               Eat$ = ""
            Else
               Exit Function
            End If
      End Select
   Next
   ' mm-dd-yyyy
   V = Int(Val(Mid$(Var$, 1, 2)))
   If V >= 1 And V <= 12 Then
      Eat$ = ""
   Else
      Exit Function
   End If
   V = Int(Val(Mid$(Var$, 4, 2)))
   If V >= 1 And V <= 31 Then
      Eat$ = ""
   Else
      Exit Function
   End If
   V = Int(Val(Mid$(Var$, 7, 4)))
   If V >= 1980 And V <= 2079 Then
      Eat$ = ""
   Else
      Exit Function
   End If
   ValidDate = True
End Function

Function GetContainerRecCharges (Array.Counter)
   GetContainerRecCharges = CVI(Mid$(ContainerRec.Charges, (Array.Counter - 1) * 2 + 1, 2))
End Function

Sub PutContainerRecCharges (Array.Counter, Item.Charges)
   Mid$(ContainerRec.Charges, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
End Sub

Function GetContainerRecInventory (Array.Counter)
   GetContainerRecInventory = CVI(Mid$(ContainerRec.Inventory, (Array.Counter - 1) * 2 + 1, 2))
End Function

Sub PutContainerRecInventory (Array.Counter, Item.Charges)
   Mid$(ContainerRec.Inventory, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
End Sub

Function GetContainerRecInvisible (Array.Counter)
   GetContainerRecInvisible = CVI(Mid$(ContainerRec.Invisible, (Array.Counter - 1) * 2 + 1, 2))
End Function

Sub PutContainerRecInvisible (Array.Counter, Item.Charges)
   Mid$(ContainerRec.Invisible, (Array.Counter - 1) * 2 + 1, 2) = MKI$(Item.Charges)
End Sub

Sub PutUserRecordContainer (Var)
   Select Case Var
      Case 1
         UserRecord.Container1 = ContainerRec ' store user container into record
      Case 2
         UserRecord.Container2 = ContainerRec ' store user container into record
      Case 3
         UserRecord.Container3 = ContainerRec ' store user container into record
   End Select
End Sub

Function GetUserRecordContainer (Var)
   Select Case Var
      Case 1
         ContainerRec = UserRecord.Container1 ' get user container into record
      Case 2
         ContainerRec = UserRecord.Container2 ' get user container into record
      Case 3
         ContainerRec = UserRecord.Container3 ' get user container into record
   End Select
   GetUserRecordContainer = -1
End Function

Function GetRoomRecordDirect (Direction.Number) ' get room number direction
   GetRoomRecordDirect = CVS(Mid$(RoomRecord.Direct, (Direction.Number - 1) * 4 + 1, 4))
End Function

Sub PutRoomRecordDirect (Array.Index, Var!)
   Mid$(RoomRecord.Direct, (Array.Index - 1) * 4 + 1, 4) = MKS$(Var!)
End Sub

Function GetRoomRecordFlags (Array.Number)
   GetRoomRecordFlags = CVI(Mid$(RoomRecord.Flags, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutRoomRecordFlags (Array.Index, Var)
   Mid$(RoomRecord.Flags, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetRoomRecordLongDesc$ (Array.Number) ' get room description line
   GetRoomRecordLongDesc$ = Mid$(RoomRecord.LongDesc, (Array.Number - 1) * 80 + 1, 80)
End Function

Sub PutRoomRecordLongDesc (Array.Index, Var$) ' set long description
   Mid$(RoomRecord.LongDesc, (Array.Index - 1) * 80 + 1, 80) = Var$
End Sub

Function GetRoomRecordObject (Array.Number) ' get object index to file
   GetRoomRecordObject = CVI(Mid$(RoomRecord.Object, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutRoomRecordObject (Array.Index, Var)
   Mid$(RoomRecord.Object, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetRoomRecordHiddenObj (Array.Number)
   GetRoomRecordHiddenObj = CVI(Mid$(RoomRecord.HiddenObj, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutRoomRecordHiddenObj (Array.Index, Var)
   Mid$(RoomRecord.HiddenObj, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetRoomRecordTreasure (Inventory.Number)
   GetRoomRecordTreasure = CVI(Mid$(RoomRecord.Treasure, (Inventory.Number - 1) * 2 + 1, 2))
End Function

Sub PutRoomRecordTreasure (Array.Index, Var)
   Mid$(RoomRecord.Treasure, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetRoomRecordTreCharges (Inventory.Number)
   GetRoomRecordTreCharges = CVI(Mid$(RoomRecord.TreCharges, (Inventory.Number - 1) * 2 + 1, 2))
End Function

Sub PutRoomRecordTreCharges (Array.Index, Var)
   Mid$(RoomRecord.TreCharges, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetUserRecordInv (Treasure.Index)
   GetUserRecordInv = CVI(Mid$(UserRecord.Inv, (Treasure.Index - 1) * 2 + 1, 2))
End Function

Sub PutUserRecordInv (Array.Index, Var)
   Mid$(UserRecord.Inv, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetUserRecordCharges (Array.Number)
   GetUserRecordCharges = CVI(Mid$(UserRecord.Charges, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutUserRecordCharges (Array.Number, Var)
   Mid$(UserRecord.Charges, (Array.Number - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetUserRecordObject (Object.Index)
   GetUserRecordObject = CVI(Mid$(UserRecord.Object, (Object.Index - 1) * 2 + 1, 2))
End Function

Sub PutUserRecordObject (Array.Index, Var)
   Mid$(UserRecord.Object, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetUserRecordTrainRoom (Player.Class)
   GetUserRecordTrainRoom = CVS(Mid$(UserRecord.TrainRoom, (Player.Class - 1) * 4 + 1, 4))
End Function

Sub PutUserRecordTrainRoom (Train.Count, Var!)
   Mid$(UserRecord.TrainRoom, (Train.Count - 1) * 4 + 1, 4) = MKS$(Var!)
End Sub

Function GetUserRecordStats (Var)
   GetUserRecordStats = CVI(Mid$(UserRecord.Stats, (Var - 1) * 2 + 1, 2))
End Function

Sub PutUserRecordStats (Var, Var2)
   Mid$(UserRecord.Stats, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
End Sub

Function GetUserRecordWeapons (Var)
   GetUserRecordWeapons = CVI(Mid$(UserRecord.Weapons, (Var - 1) * 2 + 1, 2))
End Function

Sub PutUserRecordWeapons (Var, Var2)
   Mid$(UserRecord.Weapons, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
End Sub

Function GetSpellRecordIngred (Var)
   GetSpellRecordIngred = CVI(Mid$(SpellRecord.Ingred, (Var - 1) * 2 + 1, 2))
End Function

Sub PutSpellRecordIngred (Var, Var2)
   Mid$(SpellRecord.Ingred, (Var - 1) * 2 + 1, 2) = MKI$(Var2)
End Sub

Function GetMonclassRecordMonsters (Monster.Class.Number)
   GetMonclassRecordMonsters = CVI(Mid$(MonclassRecord.Monsters, (Monster.Class.Number - 1) * 2 + 1, 2))
End Function

Sub PutMonclassRecordMonsters (Monster.Classes, Var)
   Mid$(MonclassRecord.Monsters, (Monster.Classes - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetMonsterRecordTreasure (Array.Number)
   GetMonsterRecordTreasure = CVI(Mid$(MonsterRecord.Treasure, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutMonsterRecordTreasure (Array.Index, Var)
   Mid$(MonsterRecord.Treasure, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

Function GetMonsterRecordTalk (Array.Number)
   GetMonsterRecordTalk = CVI(Mid$(MonsterRecord.Talk, (Array.Number - 1) * 2 + 1, 2))
End Function

Sub PutMonsterRecordTalk (Array.Index, Var)
   Mid$(MonsterRecord.Talk, (Array.Index - 1) * 2 + 1, 2) = MKI$(Var)
End Sub

